*DECK AGAUSS
      FUNCTION AGAUSS(A,B,AUX,EPS,IFLAGG)
      COMMON/QSTATS/ERR,KOUNT,INTS
*CALL ERRORF
C
C  GAUSS, A ROUTINE FOR EVALUATING DEFINITE INTEGRALS USING AN ADAPTIVE
C  GAUSS-LAGENDRE ALGORITHM.
C
C  AGAUSS=INT(A TO B(AUX(X).DX) )  TO A TOLERANCE OF EPS
C  THE INTEGRAL IS EVALUATED OVER THE ENTIRE INTERVAL (A,B) USING A
C  5-POINT AND A 7-POINT FORMULA.  IF THE DIFFERENCE BETWEEN THE TWO
C  ESTIMATES @ RELATIVE ERROR REQUIREMENT THE INTEGRAL IS ACCEPTED.
C  OTHERWISE THE SAME PROCEEDURE IS CARRIED OUT ON EACH HALF OF THE
C  INTERVAL.
C
C  A, B - LIMITS OF INTEGRATION
C  AUX - EXTERNAL FUNCTION TO EVALUATE THE INTEGRAND F(X)
C  EPS - REQUIRED RELATIVE ERROR TOLERANCE
C  IFLAGG ERROR WARNING FLAG.  IF RETURNED ZERO NO PROBLEMS ENCOUNTERED
C         OTHERWISE RETURNED AS MINUS NUMBER OF     PROBLEMS ENCOUNTERED
C        IF ENTERED  NON ZERO A MESSAGE IS GIVEN FOR EACH PROBLEM.
C
C  ERR - ESTIMATE OF THE MAXIMUM ERROR IN THE INTEGRAL
C  INTS - NUMBER OF INTERVALS USED
C
C  X5 - REAL ARRAY USED TO CALCULATE POINTS X FOR THE 5-POINT FORMULA
C  X7 - REAL ARRAY USED TO CALCULATE POINTS X FOR THE 7-POINT FORMULA
C  W5 - REAL ARRAY WHICH STORES THE WEIGHTS FOR THE 5-POINT FORMULA
C  W7 - REAL ARRAY WHICH STORES THE WEIGHTS FOR THE 7-POINT FORMULA
C  I - COUNTER WHICH LIMITS THE NUMBER OF INTERVALS CONSIDERED
C  SUM1, SUM2 - TWO ESTIMATES FOR A GIVEN SUBINTERVAL
C  GAUSS, TEMP1 - CUMULATIVE ESTIMATES OF INTEGRAL OVER ACCEPTED
C                 SUBINTERVALS USING 7-POINT AND 5-POINT RULES
C                 RESPECTIVELY
C
C  THE FUNCTION GAUSS RETURNS THE BEST ESTIMATE OF THE INTEGRAL.
C  IF THE ERROR REQUIREMENT CAN NOT BE MET IN THE ALLOWED NUMBER OF
C  INTERVALS, IFLAGG IS DECREMENTED AND INTEGRATION CONTINUES
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C   SOURCE KAHANER DK COMPARISON OF NUMERICAL QUADRATURE FORMULAS,
C                    MATHEMATICAL SOFTWARE JR RICE ED, ACADEMIC 1971
C       PROGRAMMED BY    DK KAHANER, J MELENDEZ LOS ALAMOS     1971
C       REVISED    BY    VALERIE JONES                     AUG 1976
C
      DIMENSION X5(2), W5(3), X7(3), W7(4)
      DATA(X5(I), I=1,2) / 4.6910077030668E-2, .23076534494715/
      DATA(W5(I),I=1,3)/.11846344252809 , .23931433524968, .2844444444
     ,444/
      DATA(X7(I),I=1,3)/2.5446043828621E-2, .1292344072003, .297077424
     ,3113/
      DATA(W7(I),I=1,4)/6.4742483084434E-2, .13985269574468,
     ,.19091502525255, .208979591836734/
      DATA MES/2250/
*CALL STATF
      KOUNT=0
      INTS = 0
      I=0
      IPRINT=IFLAGG
      IFLAGG=0
      TEMP1=0.
      IFLAG=0
      GAUSS = 0.0
      ERR = 0.
      SIGN=1.
      IF(A-B) 10,70,5
    5 A0=A
      A=B
      B=A0
      SIGN=-1.
   10 XM=B
      A1=A
   15 I=I+1
C
C     ERROR CHECK CRITERION
C     IF THE MAXIMUM NUMBER OF SUBDIVISIONS HAS BEEN REACHED WITHOUT
C     THE ERROR REQUIREMENT BEING MET, THE SMALLEST SUBINTERVAL IS
C     ACCEPTED REGARDLESS OF ITS ACCURACY.
C
      IF(I.LE.50) GOTO 40
C
C       NORMALLY TROUBLE AREA IS SMALL BUT IF ERROR IS TOO STRINGENT
C             IT MAY BE SPREAD OVER MANY INTERVALS . IN CASE COMBINE
C                  ADJACENT PROBLEM INTERVALS AND ISSUE ONLY ONE MESSAGE
C
      IF(IFLAG.EQ.1) GOTO 20
         IFLAG=1
         SUBSUM=SUM2*C
         SUBER=ER*C
         PA1=A1
         PB1=B1
         GOTO 60
   20 SUBSUM=SUBSUM+SUM2*C
      SUBER=ER*C+SUBER
      PB1=B1
      GOTO 60
   25 IFLAG=0
      IFLAGG=IFLAGG-1
      IF(IPRINT.EQ.0) GOTO 60
      ENCODE(11,30,IERWD(3))PA1
*IF -DEF,AELIB5
      IERWD(4)=IERWD(4).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(4)=IERWD(4).AND.MASK(6)
*ENDIF
      ENCODE(11,30,IERWD(5))PB1
*IF -DEF,AELIB5
      IERWD(6)=IERWD(6).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(6)=IERWD(6).AND.MASK(6)
*ENDIF
      ENCODE(20,35,IERWD(7))SUBSUM
*IF -DEF,AELIB5
      IERWD(9)=IERWD(9).A.MASK(0)
*ENDIF
*IF DEF,AELIB5
      IERWD(9)=IERWD(9).AND.MASK(0)
*ENDIF
      ENCODE(11,30,IERWD(10))SUBER
*IF -DEF,AELIB5
      IERWD(11)=IERWD(11).A.MASK(6)
      CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(11)=IERWD(11).AND.MASK(6)
      CALL AELERR(MES,'NF')
*ENDIF
      GOTO 60
C
   30 FORMAT(E11.5)
   35 FORMAT(E20.14)
C
C     IF THE ERROR REQUIREMENT IS NOT SATISFIED AND THE MAXIMUM NUMBER
C     OF SUBDIVISIONS HAS NOT BEEN REACHED, THE INTERVAL IS DIVIDED
C     IN HALF,
C
   40 B1 = XM
   45 C=B1-A1
      XM=.5*(A1+B1)
      FM=AUX(XM)
C
C     TWO ESTIMATES OF THE INTERVAL ARE MADE AND THE ERROR ESTIMATE IS
C  COMPARED TO THE RELATIVE ERROR TOLERANCE.
C
      SUM1=W5(3)*FM
      DO 50 J=1,2
         AC=C*X5(J)
         X1=A1+AC
         X2=B1-AC
   50 SUM1=SUM1+W5(J)*(AUX(X1)+AUX(X2))
      SUM2=W7(4)*FM
      DO 55 J=1,3
         AC=C*X7(J)
         X1=A1+AC
         X2=B1-AC
   55 SUM2=SUM2+W7(J)*(AUX(X1)+AUX(X2))
      KOUNT=KOUNT+11
      ER=ABS(SUM2-SUM1)
      IF(ER.GT.EPS*ABS(SUM2)) GOTO 15
C
C     IF THE ERROR REQUIREMENT IS MET THE INTERVAL IS ACCEPTED.  THE
C     ESTIMATES ARE ADDED ON TO THEIR RESPECTIVE SUMS, THE COUNT OF THE
C     NUMBER OF INTERVALS USED (INTS) IS INCREASED BY ONE, AND THE NEXT
C     INTERVAL ON THE RIGHT IS CONSIDERED.
C
      IF(IFLAG.EQ.1) GOTO 25
   60 GAUSS = GAUSS + C*SUM2
      TEMP1=TEMP1+(C*SUM1)
      INTS = INTS + 1
      IF (B1.EQ.B) GOTO 65
         A1=B1
         I=I-1
         B1=A1+2.*C
         IF (B1.LE.B) GOTO 45
         B1=B
         GOTO 45
C
C     WHEN THE ENTIRE INTERVAL HAS BEEN CONSIDERED THE ANSWER IS
C     RETURNED ALONG WITH AN ESTIMATE OF THE ERROR.
C
   65 ERR = ABS(GAUSS-TEMP1)
      IF(SIGN.EQ.1) GOTO 70
         B=A
         A=A0
         GAUSS=-GAUSS
   70 AGAUSS=GAUSS
      END
