*DECK AIKINT
      FUNCTION AIKINT(Z,X,Y,N,EPS)
C
C     AIKINT   1-14-02
C
C  ...REVISION HISTORY
C        80-AUG,  BY E.G. LONG
C           -CHECK THAT X-VALUES ARE MONOTONICALLY INCREASING,
C           -CONVERT ERROR MESSAGE HANDLING TO NEW SCHEME.
C
      DIMENSION X(N),Y(N),W(10),F(10)
C
C  ...LOCAL VARIABLES
C
      INTEGER IROUT
C                 ...DEFINES AELIB ROUTINE NUMBER
C
C  ...EXTERNAL REFERENCES
C        ALERPR,  SETBAD
C
      LOGICAL ALERPR
C
C  ...ABNORMAL TERMINATION
C        1. PARAMETER CHECKS FAIL-
C           A. N .LE. 2
C           B. X NOT MONOTONICALLY INCREASING
C        2. REQUESTED ACCURACY NOT OBTAINED
C
      DATA IROUT / 11402 /
*CALL  STATF
C
C     CHECK PARAMETERS
C
      IF (N .LT. 2) GOTO 130
         DO 120 I = 2, N
            IF (X(I) .GE. X(I-1)) GOTO 120
               IER = 0
               IF (.NOT. ALERPR (IROUT, IER)) GOTO 110
                  PRINT 2010
                  CALL TRACEB (0)
110            CONTINUE
               CALL SETBAD (AIKINT, 1, 1, 1)
               GOTO 500
120      CONTINUE
130   CONTINUE
      IF (N .GT. 2) GOTO 150
         IER = 0
         IF (.NOT. ALERPR (IROUT, IER)) GOTO 140
            PRINT 2020, N
            CALL TRACEB (0)
140      CONTINUE
         IF (N .LE. 0) AIKINT = 0.0
         IF (N .EQ. 1) AIKINT = Y(1)
         IF (N .EQ. 2) AIKINT = ((Z - X(2)) * Y(1) - (Z - X(1)) * Y(2))/
     ,                          (X(1) - X(2))
         GOTO 500
150   CONTINUE
      IX = 1
      IF(X(IX)-Z) 22,21,20
   21 K=1
      F(K)=Y(IX)
      GO TO 503
C     EXTRAPOLATION BEFORE TABLE BEGINS
   20 IU=1
      JA=1
      GO TO 31
   22 IL=IX
      IX=N
      IF(X(IX)-Z)25,21,9
C     EXTRAPOLATION BEYOND TABLE
   25 IL=N
      JA=2
      GO TO 32
C     FINDING WHERE Z LIES IN THE TABLE
    9 IU=IX
      JA=3
      A=N
      M=ALOG(A)/ALOG(2.)+2
      DO 11 I=1,M
      IR=IU-IL
      IF(IR.EQ.1) GO TO 30
   13 IX=IL+IR/2
      IF(X(IX)-Z) 17,21,15
   15 IU=IX
      GO TO 11
   17 IL=IX
   11 CONTINUE
   38 IU=IX
      IL=IX-1
      GO TO 30
C     EXTRAPOLATION BEFORE TABLE BEGINS
   31 K=2
      NA=MIN0(N,10)
      DO 40 I=1,NA
      W(I)=X(I)
   40 F(I)=Y(I)
      GO TO 600
C     EXTRAPOLATION BEYOND TABLE
   32 K=2
      NA=MIN0(N,10)
      DO 41 I=1,NA
      J=N-I+1
      W(I)=X(J)
   41 F(I)=Y(J)
      GO TO 600
   30 K=0
      MC=3
C     FINDING NEAREST ARGUMENTS TO Z
C     IF POSSIBLE, THE ARGUMENTS ARE CHOSEN IN PAIRS, SO THAT THEY ARE O
C     SIDE OF Z.  THE FIRST LINEAR CROSS MEANS IS CALCULATED USING THE C
C     PAIR.  SUBSEQUENT LINEAR CROSS MEANS ARE CALCULATED USING FIRST TH
C     CLOSEST ARGUMENT TO Z OF THE NEXT PAIR AND THEN THE OTHER ARGUMENT
C     OTHERWISE THE ARGUMENTS ARE CHOSEN IN ORDER OF CLOSENESS.
 402   NA=MIN0(N,10)
  601 IF(K.EQ.NA) GO TO 501
      IF(IU.EQ.(N+1)) GO TO 401
      IF(IL.EQ.0) GO TO 400
      IF(MC-2)401,400,450
  450 MC=0
      D1=ABS(X(IU)-Z)
      D2=ABS(X(IL)-Z)
      IF (D1-D2) 400,400,401
  400 MC=MC+1
      K=K+1
      F(K)=Y(IU)
      W(K)=X(IU)
      IU=IU+1
      IF(K.GE.2)GO TO 600
      GO TO 601
  401 MC=MC+2
      K=K+1
      F(K)=Y(IL)
      W(K)=X(IL)
      IL=IL-1
      IF(K.GE.2)GO TO 600
      GO TO 601
C     EVALUATION OF POSSIBLE ANSWERS
  600 KA=K-1
      DO 200 I=1,KA
      F(K)=((Z-W(K))*F(I)-(Z-W(I))*F(K))/(W(I)-W(K))
  200 CONTINUE
      IF(ABS(F(K)-F(KA)).LT.EPS) GO TO 503
*IF -DEF,AELIB5
      GO TO (301,301,601) JA
*ENDIF
*IF DEF,AELIB5
      GO TO (301,301,601) JA
      CALL GOTOER
*ENDIF
 301     IF(K.EQ.NA)GO TO 501
      K=K+1
      GO TO 600
C
C     REQUESTED ACCURACY WAS NOT OBTAINED
C
501   CONTINUE
      IER = 0
      IF (.NOT. ALERPR (IROUT, IER)) GOTO 502
         PRINT 2030, Z
         CALL TRACEB (0)
502   CONTINUE
      NAA=NA-2
      DO 650 I=1,NAA
      RATIO=ABS(F(I+2)-F(I+1))/ABS(F(I+1)-F(I))
      IF(RATIO.LT.1.0) GO TO 650
      AIKINT=F(I+1)
      GO TO 500
  650 CONTINUE
  503 AIKINT=F(K)
  500 RETURN
C
*IF -DEF,AELIB5
2010  FORMAT("***INPUT PARAMETER VIOLATION.  X-VALUES ARE NOT "
     ,       " MONOTONICALLY INCREASING.")
2020  FORMAT ("***INPUT PARAMETER VIOLATION, N=" I5 ", N MUST BE LARGER"
     ,        " THAN 2.")
2030  FORMAT("***THE REQUESTED ACCURACY WAS NOT OBTAINED FOR THE "
     ,       " ARGUMENT " E12.5)
*ENDIF
*IF DEF,AELIB5
2010  FORMAT('***INPUT PARAMETER VIOLATION.  X-VALUES ARE NOT '
     ,       ,' MONOTONICALLY INCREASING.')
2020  FORMAT ('***INPUT PARAMETER VIOLATION, N=' ,I5 ,', N MUST BE LARGE
     ,R'        ,' THAN 2.')
2030  FORMAT('***THE REQUESTED ACCURACY WAS NOT OBTAINED FOR THE '
     ,       ,' ARGUMENT ' ,E12.5)
*ENDIF
      END
