*DECK ALEROF
      SUBROUTINE ALEROF(ROUTNE,ERROR)
C
C     ******************************************************************
C     *                                                                *
C     *   ALEROF IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL       *
C     * ROUTINES:                                                      *
C     *                      ALRINI                                    *
C     *                      ALRDEL                                    *
C     *                      ALRFND                                    *
C     *                      ALRTST                                    *
C     *                      TRACEB                                    *
C     * ALRINI THROUGH ALRTST ACCOMPANY ALEROF ON THE AELIB SOURCE     *
C     * TAPE UNDER THE DECK NAME ALEROF.  TRACEB IS STORED ON THE      *
C     * AELIB SOURCE TAPE UNDER THE DECK NAME TRACEB.                  *
C     *                                                                *
C     ******************************************************************
C     ROUTINE ALEROF: DISABLE THE PRINTING OF THE ERROR MESSAGE
C                     "ERROR" (I.E. THE MESSAGE ASSOCIATED WITH
C                     ERROR NUMBER "ERROR") IN THE AELIB ROUTINE
C                     WITH ROUTINE NUMBER "ROUTNE".  IF "ERROR" = 0
C                     THEN DISABLE ALL MESSAGES FOR ROUTINE "ROUTNE".
C
C     ERROR EXITS
C     -----------
C     "ALEROF" CONTAINS TWO ERROR EXITS:
C     (1)  IF "ALEROF" IS CALLED WITH "ROUTNE = 0", THEN
C          THE CALL IS IGNORED, A MESSAGE IS PRINTED AND
C          CONTROL IS RETURNED TO THE USER'S PROGRAM.
C          THIS WAS INCLUDED BECAUSE THE OPTION OF TURNING
C          OFF ALL THE MESSAGES AT ONCE WAS NOT PUT INTO
C          USE.
C     (2)  IF THE REQUEST TO TURN OFF A MESSAGE CANNOT BE
C          ACCOMODATED BECAUSE OF INSUFFICIENT TABLE SIZE,
C          A MESSAGE IS PRINTED AND CONTROL IS RETURNED TO
C          THE CALLING PROGRAM WITHOUT DISABLING THE REQUESTED ERROR
C          MESSAGE OR MESSAGES.
C
C     AUTHOR & REVISION HISTORY
C     -------------------------
C     MAY 1979     - WRITTEN, P. BUMBULIS
C
*CALL ALRCOM
C
C       THE LABELED COMMON BLOCKS "ALRCM1" AND "ALRCM2" CONTAIN
C     THE WORKING TABLE FOR USER CONTROL OF THE PRINTING OF ERROR
C     MESSAGES, AND THE POINTERS "NLAST" AND "N" FOR THIS TABLE.
C
C       THE TABLE (AND POINTERS) ARE AS FOLLOWS:
C                            !------!------!
C                            !ROUTAB!ERRTAB!
C                            !------!------!
C                       (1)  !      !      !
C                            !------!------!
C                            !  .   !  .   !
C                               :      :
C                            !------!------!
C                "NLAST"---->!      !      !
C     "NLAST" POINTS TO THE  !------!------!
C     LAST USED ENTRY           :      :
C                            !  .   !  .   !
C                            !------!------!
C                            !      !      !
C                            !------!------!
C                       (N)  !      !      !
C                            !------!------!
C
C     "N" POINTS TO THE PHYSICAL END OF THE TABLE.
C
C       EACH ENTRY IN THE TABLE CONTAINS 2 PIECES OF INFORMATION:
C     1)  "ROUTAB" IS THE NUMBER OF AN AELIB ROUTINE (IF IT IS 0,
C         THEN THE ENTRY IS CONSIDERED EMPTY)
C AND 2)  "ERRTAB" IS THE NUMBER OF THE ERROR MESSAGE THAT IS TO
C         BE CONTROLLED (IF IT IS 0, AND THE CORRESPONDING ENTRY
C         IN "ROUTAB" IS NOT 0, THEN THE DEFAULT FOR ALL MESSAGES
C         FOR THAT ROUTINE IS NOT TO PRINT)
C
C       IF A MESSAGE IDENTIFIED BY A FLAG VALUE "IER" IN ROUTINE
C     "ROUTNE" IS TO BE PRINTED THEN THE TABLES WILL CONTAIN EITHER
C     NO ENTRY FOR THAT ROUTINE, OR BOTH OF THE ENTRIES (ROUTNE,0)
C     AND (ROUTNE,IER).  IF, HOWEVER, ONLY ONE OF (ROUTNE,0) OR
C     (ROUTNE,IER) APPEARS, THEN THE MESSAGE WILL NOT BE PRINTED.
C
C       BY CONVENTION, (1) AELIB ROUTINES ARE ASSIGNED A 5 DIGIT
C     INTEGER NUMBER FROM THEIR CLASSIFICATION NUMBER AS FOLLOWS:
C               A-B-C  BECOMES  A0B0C
C              A-BB-C     "     ABB0C
C             A-BB-CC     "     ABBCC
C     AND (2) THE ERROR FLAG PARAMETERS, USUALLY CALLED "IER" ARE USED
C     TO IDENTIFY PARTICULAR MESSAGES WITHIN A ROUTINE.
C
C       SO, TO TURN OFF THE MESSAGE CORRESPONDING TO "IER" VALUE 5 IN
C     ROUTINE 1-11-3,
C                    CALL ALEROF(11103,5)
C
C     HOW TO EXPAND THE TABLE
C     -----------------------
C       THE TABLE HAS BEEN INITIALLY SET UP WITH 20 ENTRIES;  THIS
C     SHOULD BE ENOUGH FOR MOST CASES.  IF, HOWEVER, THERE IS A
C     NEED FOR MORE SPACE IN THE TABLE THE USER CAN EXTEND IT HIMSELF.
C     TO ALLOW FOR EASY EXPANSION OF THE TABLE, THE TABLE HAS
C     BEEN SPLIT INTO TWO COMMON BLOCKS.  AS A RESULT OF THIS
C     SPLITTING OF THE TABLE, THE ONLY THINGS THAT HAVE TO BE
C     DONE TO EXTEND THE TABLE ARE:
C        1) DEFINE THE COMMON BLOCKS IN A (USER) ROUTINE WITH
C           THE ARRAYS "ROUTAB" AND "ERRTAB" DIMENSIONED TO THE
C           REQUIRED SIZE
C      & 2) BEFORE ANY CALL TO "ALEROF", OR "ALERON" SET "N" TO
C           THE NEW SIZE, AND "NLAST" TO 0.  THIS MAY BE DONE IN
C           A DATA STATEMENT.  IF THERE HAVE BEEN SOME CALLS TO
C           "ALEROF", OR "ALERON" DO NOT SET "NLAST".
C
C     A WARNING ON USING OVERLAYS
C     ---------------------------
C       IF YOU ARE USEING OVERLAYS, YOU WILL HAVE TO MAKE SURE THAT
C     THE COMMON BLOCKS ARE DEFINED IN THE (0,0) OVERLAY.  OTHERWISE
C     UNPREDICTABLE THINGS MAY HAPPEN.
C
C     VARIABLES USED/FUNCTIONS CALLED:
C     -------------------------------
C (A) ROUTNE,ERROR - DESCRIBES THE ERROR MESSAGE(S) TO TURN OFF
C     ERRFLG - SET TO 1 (NORMALLY 0) IF INTERNAL TABLES OVEFLOW
C     INDEX  - INDEX INTO THE TABLES
C     ALRTST - TESTS TO SEE IF WE WOULD PRINT THIS MESSAGE
C     PRINT  - .TRUE. IF WE ARE TO PRINT
C     ALRINI - INITIALIZATION ROUTINE
C *** (A) DENOTES THAT THE VARIABLE IS A FORMAL PARAMETER (ARGUMENT)
C
      INTEGER ROUTNE,ERROR,ERRFLG,INDEX
      LOGICAL PRINT,ALRTST
C
*IF -DEF,AELIB5
    2 FORMAT(" ***TOO MANY ERROR CONTROL REQUESTS - ",
     *       "EXPAND TABLE IF NECCESSARY")
*ENDIF
*IF DEF,AELIB5
    2 FORMAT(' ***TOO MANY ERROR CONTROL REQUESTS - ',
     *       'EXPAND TABLE IF NECCESSARY')
*ENDIF
C
*CALL STATF
      CALL ALRINI
C
      IF (ROUTNE .NE. 0) GO TO 4
C
C        OPTION OF TURNING OFF ALL ERROR MESSAGES IS NOT SUPPORTED
C
         PRINT 3
*IF -DEF,AELIB5
    3    FORMAT(" ***OPTION OF TURNING OFF ALL ERROR MESSAGES ",
     *          "NOT SUPPORTED")
*ENDIF
*IF DEF,AELIB5
    3    FORMAT(' ***OPTION OF TURNING OFF ALL ERROR MESSAGES ',
     *          'NOT SUPPORTED')
*ENDIF
         CALL TRACEB(0)
      RETURN
    4    IF (ERROR .NE. 0) GO TO 10
C
C           TURN OFF ALL MESSAGES FOR THIS ROUTINE
C
            CALL ALRDEL(ROUTNE)
            CALL ALRFND(INDEX,ERRFLG)
            IF (ERRFLG .EQ. 0) GO TO 5
               PRINT 2
               CALL TRACEB(0)
            RETURN
    5          ROUTAB(INDEX) = ROUTNE
               ERRTAB(INDEX) = 0
         RETURN
C
C           TURN OFF A SPECIFIC MESSAGE
C
   10       PRINT = ALRTST(ROUTNE,ERROR,INDEX)
            IF (.NOT. PRINT) RETURN
               IF (INDEX .NE. 0) GO TO 20
                  CALL ALRFND(INDEX,ERRFLG)
                  IF (ERRFLG .NE. 0) GO TO 15
                     ROUTAB(INDEX) = ROUTNE
                     ERRTAB(INDEX) = ERROR
                  RETURN
   15                PRINT 2
                     CALL TRACEB(0)
                  RETURN
   20          CONTINUE
                  ROUTAB(INDEX) = 0
               RETURN
      END
      SUBROUTINE ALRINI
C
C     ROUTINE ALRINI: INITIALIZE COMMON AT FIRST CALL TO ANY OF THE
C                     USER INTERFACE ROUTINES (IF USER HAS NOT SET IT
C                     HIMSELF)
C
C     AUTHOR & REVISION HISTORY
C     -------------------------
C     MAY 1979     - WRITTEN, P. BUMBULIS
C
*CALL ALRCOM
C     FOR A COMPLETE DESCRIPTION OF THE COMMON BLOCKS, PLEASE SEE THE
C     WRITE UP IN "ALEROF".
C
      IF (1 .LE. N .AND. N .LE. 100) RETURN
         N      = 20
         NLAST  = 0
      RETURN
      END
      SUBROUTINE ALRFND(INDEX,ERRFLG)
C
C     ROUTINE ALRFND: FIND A FREE ENTRY INTO THE TABLES AND RETURN
C                     ITS POSITION IN "INDEX".  "ERRFLG" IS SET TO
C                     1 IF THERE IS NO MORE ROOM IN THE TABLES.
C                     [AS WITH ALRDEL, AND ALRTST THIS ROUTINE IS
C                     SEPARATE BECAUSE THE COMPLEXITY OF ITS CODE
C                     WOULD CONFUSE THE ISSUES OF THE CODE IN WHICH
C                     IT WOULD BE IMBEDDED.]
C
C     AUTHOR & REVISION HISTORY
C     -------------------------
C     MAY 1979     - WRITTEN, P. BUMBULIS
C
*CALL ALRCOM
C     FOR A COMPLETE DESCRIPTION OF THE COMMON BLOCKS, PLEASE SEE THE
C     WRITE UP IN "ALEROF".
C
C     VARIABLES USED/FUNCTIONS CALLED:
C     -------------------------------
C (A) INDEX  - POSITION OF FREE ENTRY (N+1 IF NOT FOUND)
C (A) ERRFLG - RETURNS 1 IF TABLE HAS OVERFLOWED (0 OTHERWISE)
C *** (A) DENOTES THAT THE VARIABLE IS A FORMAL PARAMETER (ARGUMENT)
C
      INTEGER INDEX,ERRFLG
C
      ERRFLG = 0
      INDEX  = 1
   10 IF (INDEX .GT. NLAST) GO TO 20
      IF(ROUTAB(INDEX) .EQ. 0) GO TO 20
         INDEX = INDEX + 1
      GO TO 10
   20 CONTINUE
      IF (INDEX .LE. N) GO TO 30
         ERRFLG = 1
      RETURN
   30 IF (INDEX .LE. NLAST) RETURN
         NLAST = INDEX
      RETURN
      END
      SUBROUTINE ALRDEL(ROUTNE)
C
C     ROUTINE ALRDEL: DELETE ALL INSTANCES OF ANY REFERENCE TO ROUTINE
C                     "ROUTNE" IN THE TABLES.
C
C     AUTHOR & REVISION HISTORY
C     -------------------------
C     MAY 1979     - WRITTEN, P. BUMBULIS
C
*CALL ALRCOM
C     FOR A COMPLETE DESCRIPTION OF THE COMMON BLOCKS, PLEASE SEE THE
C     WRITE UP IN "ALEROF".
C
C     VARIABLES USED/FUNCTIONS CALLED:
C     -------------------------------
C (A) ROUTNE - THE ROUTINE WHOSE ENTRIES ARE TO BE DELETED FROM TABLES
C     I      - INDEX INTO THE TABLES
C     ZFLAG  - TRUE IFF ONLY ZERO (NULL) ELEMENTS AT THE END OF TABLES
C *** (A) DENOTES THAT THE VARIABLE IS A FORMAL PARAMETER (ARGUMENT)
C
      INTEGER ROUTNE,I
      LOGICAL ZFLAG
C
      ZFLAG = .TRUE.
      I = NLAST
    5 IF (I .LT. 1) GO TO 40
         IF (ROUTAB(I) .EQ. ROUTNE)
     *      ROUTAB(I) = 0
         IF (.NOT. ZFLAG) GO TO 30
            IF (ROUTAB(I) .NE. 0) GO TO 10
               NLAST = NLAST - 1
            GO TO 20
   10          ZFLAG = .FALSE.
   20       CONTINUE
   30    CONTINUE
         I = I - 1
      GO TO 5
   40 CONTINUE
      RETURN
      END
      LOGICAL FUNCTION ALRTST(ROUTNE,ERROR,INDEX)
C
C     ROUTINE ALRTST: SEE WHETHER TO PRINT ERROR MESSAGE "ERROR" FOR
C                     ROUTINE "ROUTNE", OR NOT.  IF THERE IS A
C                     SPECIFIC REFERENCE TO (ROUTNE,ERROR) IN THE
C                     TABLES, RETURN ITS POSITION IN "INDEX";
C                     OTHERWISE SET "INDEX" TO 0.
C
C     AUTHOR & REVISION HISTORY
C     -------------------------
C     MAY 1979     - WRITTEN, P. BUMBULIS
C
*CALL ALRCOM
C     FOR A COMPLETE DESCRIPTION OF THE COMMON BLOCKS, PLEASE SEE THE
C     WRITE UP IN "ALEROF".
C
C     VARIABLES USED/FUNCTIONS CALLED:
C     -------------------------------
C (A) ROUTNE,ERROR - DESCRIBES UNIQUELY AN ENTRY INTO THE TABLES
C (A) INDEX  - IF EXPLICITLY REFERENCED IN TABLES, ITS INDEX
C     ALRTST - RETURNS .TRUE. IFF TO PRINT THE MESSAGE
C     I      - AN INDEX INTO THE TABLES
C *** (A) DENOTES THAT THE VARIABLE IS A FORMAL PARAMETER (ARGUMENT)
C
      INTEGER ROUTNE,ERROR,INDEX,I
C
      ALRTST = .TRUE.
      INDEX  = 0
C
C     IF WE SEE ONE OF (ROUTNE,0), (ROUTNE,ERROR) THEN WE DON'T WANT
C     TO PRINT.  IF NEITHER OR BOTH SEEN, THEN WE DO WANT TO PRINT.
C
      I = 1
   10 IF (I .GT. NLAST) RETURN
         IF (ROUTAB(I) .NE. ROUTNE) GO TO 50
            IF (ERRTAB(I) .NE. 0) GO TO 20
               ALRTST = .NOT. ALRTST
            GO TO 40
   20       IF (ERRTAB(I) .NE. ERROR) GO TO 30
               ALRTST = .NOT. ALRTST
               INDEX  = I
   30       CONTINUE
   40       CONTINUE
   50    CONTINUE
         I = I + 1
      GO TO 10
      END
