*DECK AXIS
      SUBROUTINE AXIS(XPAGE,YPAGE,IBCD,NCHAR,AXLEN,ANGLE,FIRSTV,DELTAV)
************************************************************************
*     THIS SUBROUTINE DRAWS ANY LENGTH LINE AT ANY ANGLE,DIVIDES IT
*     INTO ONE- INCH SEGMENTS, ANNOTATES THE DIVISIONS WITH APPROPRIATE
*     SCALE VALUES, AND LABELS THE AXIS WITH A CENTERED TITLE
*     AXIS MUST BE CALLED SEPARATELY FOR EACH AXIS REQUIRED.
*
************************************************************************
* ARGUMENTS
*     XPAGE - X COORDINATE OF AXIS STARTING POINT
*     YPAGE - Y COORDINATE OF AXIS STARTING POINT
*     IBCD  - TITLE, WHICH WILL BE CENTERED AND PLACED PARALLEL TO AXIS
*     NCHAR - MAGNITUDE GIVES NUMBER OF CHARACTERS IN TITLE
*           - IF POSITIVE, LABELLING ON COUNTERCLOCKWISE SIDE OF AXIS
*           - IF NEGATIVE, LABELLING ON CLOCKWISE SIDE OF AXIS
*     AXLEN - LENGTH OF AXIS LINE, IN INCHES
*     ANGLE - ANGLE, IN DEGREES AT WHICH AXIS IS TO BE DRAWN .
*            (BECAUSE ONLY 8 ANGLES ARE AVAILABLE FOR PLOTTING TITLES,
*             LABELLING WILL BE DONE AT 0 DEGREES IF ANGLE SPECIFIED
*             IS NOT ONE OF AVAILABLE 8)
*     FIRSTV - STARTING VALUE FOR FIRST TIC MARK ON AXIS. MAY BE
*             COMPUTED BY USER  OR BY SCALE ROUTINE
*     DELTAV - NUMBER OF DATA UNITS PER INCH OF AXIS. MAY BE
*             COMPUTED BY USER OR BY SCALE ROUTINE
C ROUTINES CALLED -
C        PLT, NUMBER, SYMBOL FROM CALCOMP PACKAGE
C
C AUTHOR - L EVANS            SEPTEMBER 1973
C REVISED - TO OPTIMIZE PLOTTING BY L EVANS  OCTOBER 1975
C REVISED - TO FIX AXIS LABELLING WHEN TIC LABELS LE 0
C                                     L EVANS NOVEMBER 1975
C
************************************************************************
*
      DIMENSION IBCD(1)
*CALL BLOCKS
*CALL STATF
*
* MODIFY SCALE FACTOR TO SATISFY  1<=SCALE<=99
*
*     DELTAV = SCALE*(10**EXP)
*
      SCALE = ABS(DELTAV)
      EXP=0.
*     DO  WHILE  SCALE>99 OR SCALE <1
*
*IF -DEF,AELIB5
 50   IF ((SCALE.LE.99.).A.(SCALE.GE.1.)) GOTO 150
*ENDIF
*IF DEF,AELIB5
 50   IF ((SCALE.LE.99.).AND.(SCALE.GE.1.)) GOTO 150
*ENDIF
*
*         IF SCALE > 99.
*
          IF(SCALE.LE.99.)GOTO 100
*         THEN...
*
               SCALE=SCALE/10.0
               EXP=EXP+1.0
               GOTO 50
*
*
*
 100     IF(SCALE.GE..1) GOTO 150
*
*         THEN...
               SCALE=SCALE*10.0
               EXP=EXP-1.0
               GOTO 100
*     END
*
*  INITIALIZE ANGES AND TRIGONOMETRIC FUNCTIONS
*   RANGLE - ANGLE AT WHICH TITLES AND LABELS ARE TO BE PRINTED
*   ANGLE  - ANGLE AT WHICH AXIS IS TO BE PLOTTED
C   IF ANNOTATION ON COUNTER CLOCKWISE SIDE OF AXIS,
C        RANGLE = ANGLE. SYMBOL ROUNDS TO NEXT HIGHEST MULTIPLE OF 45
C
C   IF ANNOTATION ON CLOCKWISE SIDE OF AXIS,
C        RANGLE = NEXT LOWEST MULTIPLE OF 45 DEG.
*
  150 RANGLE=0.
      IF(NCHAR.GE.0)RANGLE = ANGLE
      IF(NCHAR.LT.0)RANGLE = 45.*INT(ANGLE/45.)
      SINA=SIN(ANGLE/57.256)
      SINRA=SIN(RANGLE/57.256)
      COSA=COS(ANGLE/57.256)
      COSRA=COS(RANGLE/57.256)
*
*  DIRECTION OF ANNOTATION  - DIR = 1- FOR  COUNTER CLOCKWISE
*                             DIR =-1. FOR  CLOCKWISE
*
      DIR=ISIGN(1,NCHAR)
*
*  PLOT AXIS WITH TIC MARKS
*
      CALL PLT(XPAGE,YPAGE,3)
      XS=XPAGE
      YS=YPAGE
      NTIC=AXLEN+1.0
*
*  FOR EACH TIC,.DO THREE PLOT CALLS. GO TO NEXT POINT, DO TIC, AND RETU
*
      DO  20 I=1,NTIC
         CALL PLT(XS,YS,2)
          XINC=XS-.08*DIR*SINA
          YINC=YS+.08*DIR*COSA
         CALL PLT(XINC,YINC,2)
      CALL PLT(XS,YS,2)
          XS=XS+COSA
          YS=YS+SINA
 20   CONTINUE
*
*  CALCULATE STARTING POINT FOR FIRST LABEL, AND VALUE FOR THAT LABEL
*
      XINC=-.12/FACT
      YINC=0.135*DIR - .025
      XLABEL=10.0**(-EXP)*FIRSTV
      XLINC=10.0**(-EXP)*DELTAV
C
C     POSITION FIRST LABEL SO NOT TO INTERFERE WITH LABEL FROM OTHER
C     DIRECTION
C
      IF(XLABEL.GE.0.)XINC=-.20/FACT
      XS=XINC*COSRA-YINC*SINRA+XPAGE
      YS=XINC*SINRA+YINC*COSRA+YPAGE
*
*  PLOT LABELS AND TITLE
*
      DO 200 I=1,NTIC
          CALL NUMBER(XS,YS,0.08,XLABEL,RANGLE,2)
          XLABEL=XLABEL+XLINC
C
C     CALCULATE POSITION FOR NEXT LABEL
C
            IFEXP=2
C
C           CALCULATE LENGTH BY LOGS UNLESS XLABEL APPROX ZERO
C
            IF(ABS(XLABEL).LT..01) GOTO 250
            IFEXP=ALOG10(ABS(XLABEL))+2
            IF(ABS(XLABEL).LT.1.)IFEXP=IFEXP+1
  250       CONTINUE
            XINC=I-IFEXP*.08-.04
            YINC=0.135*DIR-.025
            XS=XINC*COSRA-YINC*SINRA + XPAGE
            YS=XINC*SINRA+YINC*COSRA+YPAGE
*         IF CURRENT POSITION IS (ROUGHLY) IN MIDDLE OF AXIS
          IF(I.NE.NTIC/2)GOTO 200
*         THEN   PLOT TITLE
               LEN=IABS(NCHAR)
               IF(EXP.NE.0) LEN=LEN+6
               XINC=-.08*LEN+AXLEN*.5
C
            YINC=.370*DIR
               XL=XINC*COSRA -YINC*SINRA +XPAGE
               YL=XINC*SINRA +YINC*COSRA +YPAGE
               CALL SYMBOL(XL,YL,.16,IBCD(1),RANGLE,IABS(NCHAR))
*
*              IF SCALING FACTOR NECESSARY IN TITLE
               IF(EXP.EQ.0.)GOTO 200
*              THEN...
               LEN=IABS(NCHAR)+1
                    XL=XL+LEN*COSRA*.16
                    YL=YL+LEN*SINRA*.16
*IF -DEF,AELIB5
                    CALL SYMBOL(XL,YL,.16,3H*10,RANGLE,3)
*ENDIF
*IF DEF,AELIB5
                    CALL SYMBOL(XL,YL,.16,'*10',RANGLE,3)
*ENDIF
                    XL=XL+(3*COSRA-.8*SINRA)*.16
                    YL=YL+(3*SINRA+.8*COSRA)*.16
                    ZEXP=-EXP
                    CALL NUMBER(XL,YL,.08,ZEXP,RANGLE,-1)
 200  CONTINUE
      RETURN
      END
