*DECK AXLGX
      SUBROUTINE AXLGX(FORMX,NCX,FORMY,NCY,ITICXS,FORMYS,NCYS)
************************************************************************
**                                                                    **
**    NOTE THAT AXLGX IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL **
**    ROUTINES:                                                       **
**       LOGPR9                                                       **
**       LOGPR10                                                      **
**       LOGPR11                                                      **
**    LOGPR9 THROUGH LOGPR11 ACCOMPANY AXLGX ON AELIB SOURCE TAPE     **
**    UNDER THE DECK NAME AXLGX.                                      **
**                                                                    **
************************************************************************
C
C
C AXLGX IS A SUBROUTINE FOR PLOTTING AXIS LABELS ON SEMI-LOGARITHMIC
C GRAPHS (X-AXIS LOGARITHMIC, Y-AXIS LINEAR).  IT PLOTS THE TIC LABELS
C AND CENTRED AXIS TITLES.
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A) AXLGX  - USED WHEN ENGLISH DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED
C    B) AXLGXM - USED WHEN METRIC DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED.
C A SWITCH INDICATING IF METRIC DIMENSIONS HAVE BEEN USED IS PASSED IN
C A COMMON BLOCK TO THE ROUTINES CALLED BY THIS ROUTINE.  THE VALUE OF
C THIS SWITCH DETERMINES WHETHER CONVERSION OF DIMENSIONS IS REQUIRED
C AND WHETHER PLOT OR PLOTM IS TO BE CALLED.
C
C
C ARGUMENT LIST:
C  FORMX...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TITLE.
C  NCX.....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TITLE.
C  FORMY...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TITLE.
C  NCY.....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TITLE.
C  ITICXS..INTEGER, X-AXIS LOGARITHMIC TIC LABEL SWITCH.
C            =0  NO X-AXIS TIC LABELS WILL BE PRINTED.
C            =-1 ONLY LABEL THE POWERS OF 10.
C            OTHERWISE: LABEL ALL TICS.
C  FORMYS..EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TIC LABELS.
C  NCYS....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TIC LABELS.
C
C ERROR CONDITIONS:
C  AXLGX ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT TWO DIFFERENT ERROR
C  CONDITIONS:
C   1) ***AXLGX CALL NOT PRECEEDED BY PLOGX MODE 1 CALL
C        SINCE THE SCALING FACTORS AND TIC VALUES MUST BE RETRIEVED
C        THROUGH A MODE 4 CALL OF PLOGX, A PLOGX MODE 1 CALL MUST
C        PRECEED A CALL TO AXLGX IN THE CALLING PROGRAM.  THIS IS A
C        FATAL ERROR.  THE MESSAGE IS PRINTED AND EXECUTION TERMINATED.
C   2) ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT
C         IS 0
C        IF ANY OF THE PARAMETERS NCX, NCY OR NCYS HAVE NEGATIVE VALUES,
C        IT IS NOT CLEAR WHAT THE USER WOULD LIKE TO HAVE DONE.  THE
C        VALUE IS THUS ASSUMED TO BE ZERO, WHICH MEANS THAT THE PARTIC-
C        ULAR TITLE OR TIC LABELLING REFERRED TO WILL NOT BE PLOTTED.
C        THE MESSAGE IS PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF BOTH OF THESE ERROR MESSAGES IS HANDLED BY THE STAN-
C  DARD AELIB ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE AXIS LABELLING ROUTINES (AXLGX, AXLGY, AXLGXY, AXLIN) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE AXIS LABELLING PACKAGE.
C  MODULES IN THE PACKAGE INCLUDE LOGPR3, LOGPR9, LOGPR10, LOGPR11, AND
C  LOGPR12.  AXLGX FIRST CALLS LOGPR9.  THIS ROUTINE CHECKS IF PLOGX
C  MODE 1 HAS BEEN CALLED PREVIOUSLY (IT CALLS ENTRY POINT LOGPR5 IN
C  MODULE LOGPR3 IN ORDER TO DO THIS).  IF PLOGX HAD NOT BEEN CALLED,
C  A FATAL ERROR MESSAGE IS ISSUED.  OTHERWISE LOGPR9 GOES ON TO
C  RETRIEVE THE MODE 1 PARAMETERS BY CALLING ENTRY POINT LOGPR4 IN
C  MODULE LOGPR3.  FROM THESE PARAMETERS THE SCALING FACTORS ARE CALC-
C  ULATED.  LOGPR9 THEN HANDLES THE PLOTTING OF THE AXIS TITLES BEFORE
C  RETURNING CONTROL BACK TO AXLGX.  NEXT, USING THE PARAMETERS RETURNED
C  BY LOGPR9, AXLGX CALLS LOGPR10.  LOGPR10 HANDLES THE PLOTTING OF THE
C  LINEAR Y-AXIS TIC LABELS.  FINALLY, AXLGX CALLS LOGPR11, AGAIN USING
C  THE PARAMETERS RETURNED BY LOGPR9.  LOGPR11 HANDLES THE PLOTTING OF
C  THE LOGARITHMIC X-AXIS TIC LABELS (WITH THE HELP OF LOGPR12).
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      LOGICAL METRIC
      REAL MTOE
*CALL STATF
C
      METRIC=.FALSE.
      GO TO 100
C
C
C     ENTRY AXLGXM ALLOWS METRIC DIMENSIONS TO BE USED
C
*IF -DEF,AELIB5
      ENTRY AXLGXM
*ENDIF
*IF DEF,AELIB5
      ENTRY AXLGXM(FORMX,NCX,FORMY,NCY,ITICXS,FORMYS,NCYS)
*ENDIF
*CALL STATF
C
      METRIC=.TRUE.
      ETOM=25.4
      MTOE=1/ETOM
C
C     CHECK THAT PLOGX MODE 1 HAS BEEN CALLED PREVIOUSLY AND RETRIEVE
C     THE PARAMETERS.  CALCULATE THE SCALING FACTORS AND PLOT THE AXIS
C     TITLES (IF APPROPRIATE).
C
  100 CALL LOGPR9(1,FORMX,NCX,FORMY,NCY,XMINR,XMAXR,YMINR,YMAXR,TICXR,
     $            TICYR,XSC,YSC,NCYS)
C
C
C     PLOT THE LINEAR Y-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR10(2,1,FORMYS,NCYS,YMINR,YMAXR,TICYR,YSC,XSC,XMINR)
C
C
C     PLOT THE LOGARITHMIC X-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR11(XMINR,XMAXR,TICXR,XSC,YSC,YMINR,1,2,ITICXS)
C
      RETURN
      END
      SUBROUTINE LOGPR9(MODESW,FORMX,NCX,FORMY,NCY,XMINR,XMAXR,YMINR,
     $                  YMAXR,TICXR,TICYR,XSC,YSC,NCYS)
C
C
C LOGPR9 IS ONE OF THE MODULES OF THE AXIS LABELLING SYSTEM.  IT
C RETRIEVES THE PARAMETERS OF THE PREVIOUS MODE 1 CALL OF PLOGZZ OR PLOT
C AND RETURNS THESE VALUES TO THE CALLING PROGRAM.  IN ADDITION, IT
C CALCULATES AND RETURNS THE SCALING FACTORS AND HANDLES THE PLOTTING OF
C THE AXIS TITLES.
C
C IF THE ALTERNATE ENTRY IN THE AXIS LABELLING ROUTINE HAS BEEN USED
C (METRIC DIMENSIONS HAVE BEEN USED), THE RETRIEVED METRIC DIMENSIONAL
C QUANTITIES FROM THE LAST MODE 1 CALL OF PLOTM MUST BE CONVERTED TO
C ENGLISH DIMENSIONS TO CALCULATE THE SCALING FACTORS.  THEN IF THE MAIN
C ENTRY HAS BEEN USED IN THE CALLING PROGRAM, PLOT IS CALLED, AND IF THE
C ALTERNATE ENTRY HAS BEEN USED, CORRESPONDING PLOTM CALLS ARE MADE.
C
C
C ARGUMENT LIST:
C  MODESW..INTEGER, INDICATES WHICH AXIS LABELLING ROUTINE IS CALLING.
C          1 - AXLGX,  2 - AXLGY,  3 - AXLGXY,  4 - AXLIN.
C  FORMX...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TITLE.
C  NCX.....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TITLE.
C  FORMY...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TITLE.
C  NCY.....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TITLE.
C  XMINR...REAL, RETURNS THE MINIMUM VALUE ON THE X-AXIS.
C  XMAXR...REAL, RETURNS THE MAXIMUM VALUE ON THE X-AXIS.
C  YMINR...REAL, RETURNS THE MINIMUM VALUE ON THE Y-AXIS.
C  YMAXR...REAL, RETURNS THE MAXIMUM VALUE ON THE Y-AXIS.
C  TICXR...EITHER REAL OR INTEGER, RETURNS THE X-AXIS TIC VALUE.
C  TICYR...EITHER REAL OR INTEGER, RETURNS THE Y-AXIS TIC VALUE.
C  XSC.....REAL, RETURNS THE X-AXIS SCALING FACTOR FOR CONVERTING INCHES
C          TO USER UNITS.
C  YSC.....REAL, RETURNS THE Y-AXIS SCALING FACTOR.
C  NCYS....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TIC LABELS.  IF
C          Y-AXIS IS LOGARITHMIC NCYS EQUALS ZERO.
C
C ERROR CONDITIONS:
C  LOGPR9 CAN DETECT TWO DIFFERENT ERROR CONDITIONS.
C   1) ***AXLGZZ CALL NOT PRECEEDED BY PLOGZZ MODE 1 CALL
C        THE AXIS LABELLING ROUTINES REQUIRE THE INFORMATION CONTAINED
C        IN THE PREVIOUS MODE 1 CALL OF PLOGZZ OR PLOT.  THIS IS OBTAIN-
C        ED THROUGH A MODE 4 CALL TO THE APPROPRIATE ROUTINE.  THUS THE
C        AXIS LABELLING REQUIRES THE PLOGZZ OR PLOT MODE 1 CALL TO HAVE
C        BEEN MADE PREVIOUSLY.  THIS MESSAGE IS GIVEN FOR THE SEMI-LOG
C        AND LOG-LOG ROUTINES.  AXLIN CANNOT CHECK THE CONDITION DIRECT-
C        LY.  IT MAKES A MODE 4 CALL OF PLOT AND, IF PLOT MODE 1 HAD NOT
C        BEEN CALLED PREVIOUSLY, PLOT ISSUES THE MESSAGE ***NO MODE 1
C        CALL BEFORE CALL OF MODE 2,3,4 OR 5.  IN ALL CASES THE ERROR IS
C        FATAL.
C   2) ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT
C         IS 0
C        IF EITHER NCX OR NCY IS NEGATIVE, IT IS NOT CLEAR WHAT THE USER
C        WOULD LIKE DONE.  THUS THE VALUE IS ASSUMED TO BE ZERO, WHICH
C        MEANS THAT THE CORRESPONDING TITLE WILL NOT BE PLOTTED.  THE
C        MESSAGE IS PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF ALL ERROR MESSAGES IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  LOGPR9 IS CALLED BY ALL FOUR AXIS LABELLING ROUTINES.  THE CALLING
C  ROUTINE IDENTIFIES ITSELF THROUGH THE MODESW PARAMETER.  IF AXLIN IS
C  CALLING, THE PLOT MODE 1 PARAMETERS ARE RETRIEVED THROUGH A MODE 4
C  CALL OF PLOT.  OTHERWISE, ERROR CONDITION (1) ABOVE IS CHECKED FOR BY
C  CALLING ENTRY POINT LOGPR5 IN MODULE LOGPR3.  IF THE CONDITION IS NOT
C  PRESENT THE PLOGZZ MODE 1 PARAMETERS ARE RETRIEVED BY CALLING ENTRY
C  POINT LOGPR4 IN MODULE LOGPR3 (LOGPR4 IS REALLY JUST A GENERALIZED
C  PLOGZZ MODE 4 CALL).  ONCE THE MODE 1 PARAMETERS HAVE BEEN RETRIEVED
C  THE X AND Y-AXIS SCALING FACTORS ARE CALCULATED.  NCX IS CHECKED FOR
C  LEGALITY AND IF >0 THE X-AXIS TITLE IS PLOTTED PARALLEL TO AND HALF
C  AN INCH BELOW THE X-AXIS, CENTRED ACCORDING TO NCX.  THEN NCY IS
C  CHECKED FOR LEGALITY AND IF >0 THE Y-AXIS TITLE IS PLOTTED PARALLEL
C  TO AND TO THE LEFT OF THE Y-AXIS, CENTRED ACCORDING TO NCY.  THE
C  DISPLACEMENT OF THE Y-AXIS TITLE FROM THE Y-AXIS DEPENDS UPON THE
C  NCYS PARAMETER.  IF NCYS > 0 THE X LOCATION IS VARIED SO AS TO ALLOW
C  TIC LABELS OF LENGTH NCYS CHARACTERS TO BE PLOTTED BETWEEN THE TITLE
C  AND AXIS.  OTHERWISE, IF NCYS @0 (IF Y-AXIS IS LOGARITHMIC NCYS WILL
C  HAVE BEEN SET TO ZERO), THE Y-AXIS TITLE IS PLOTTED 0.65 INCHES TO
C  THE LEFT OF THE Y-AXIS.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      INTEGER PLOGER(3), AXLGER(3)
*CALL ERRORF
      COMMON/MET/METRIC,ETOM,MTOE
      LOGICAL METRIC
      REAL MTOE
*IF -DEF,AELIB5
      DATA AXLGER/5LAXLGX, 5LAXLGY, 6LAXLGXY/
*ENDIF
*IF DEF,AELIB5
      DATA AXLGER/L"AXLGX", L"AXLGY", L"AXLGXY"/
*ENDIF
*IF -DEF,AELIB5
      DATA PLOGER/5LPLOGX, 5LPLOGY, 6LPLOGXY/
*ENDIF
*IF DEF,AELIB5
      DATA PLOGER/L"PLOGX", L"PLOGY", L"PLOGXY"/
*ENDIF
C
C
C     INITIALIZE ERROR NUMBERS FOR AELIB ERROR ROUTINE.
C        3530 - ***AXLGZZ CALL NOT PRECEEDED BY PLOGZZ MODE 1 CALL.
C        3531 - ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS -
C                       DEFAULT IS 0.
C
      NERR1=3530
      NERR2=3531
C
C
C     IF AXLGZZ IS MAKING REFERENCE, CHECK IF CORRESPONDING MODE 1 CALL
C     OF PLOGZZ HAS BEEN MADE PREVIOUSLY.  RETRIEVE THE MODE ONE
C     PARAMETERS.
C
      IF(MODESW.EQ.4) GO TO 20
      IERWD(3) = AXLGER(MODESW)
      IERWD(4) = PLOGER(MODESW)
      CALL LOGPR5(MAINSW)
*IF -DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR1,1HF)
*ENDIF
*IF DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR1,'F')
*ENDIF
      CALL LOGPR4(MODESW,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      GO TO 50
C
C
C     IF AXLIN IS MAKING REFERENCE, RETRIEVE THE PLOT MODE 1 PARAMETERS.
C
   20 IF (METRIC) GO TO 30
      CALL PLOT(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      GO TO 50
   30 CALL PLOTM(4,XLRM,YLRM,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
C
C     CONVERT METRIC DIMENSIONS TO ENGLISH TO CALCULATE SCALING FACTORS
C
      XLR=XLRM*MTOE
      YLR=YLRM*MTOE
C
C
C     CALCULATE SCALING FACTORS (INCHES TO USER UNITS).
C
   50 XSC=(XMAXR-XMINR)/XLR
      YSC=(YMAXR-YMINR)/YLR
C
C
C     CHECK NCX FOR LEGALITY AND PLOT X-AXIS TITLE IF NCX .NE. ZERO.
C
      IF(NCX) 60,80,70
C
*IF -DEF,AELIB5
   60 CALL AELERR(NERR2,2HNF)
*ENDIF
*IF DEF,AELIB5
   60 CALL AELERR(NERR2,'NF')
*ENDIF
      GO TO 80
C
   70 XLOC=0.5*(XMINR+XMAXR)-0.08*XSC*NCX
      YLOC=YMINR-0.5*YSC
      IF (METRIC) GO TO 75
      CALL PLOT(3,FORMX,2,0,XLOC,YLOC)
      GO TO 80
   75 CALL PLOTM(3,FORMX,2,0,XLOC,YLOC)
C
C
C     CHECK NCY FOR LEGALITY AND PLOT Y-AXIS TITLE IF NCY .NE. ZERO.
C
   80 IF(NCY) 90,110,100
C
*IF -DEF,AELIB5
   90 CALL AELERR(NERR2,2HNF)
*ENDIF
*IF DEF,AELIB5
   90 CALL AELERR(NERR2,'NF')
*ENDIF
      GO TO 110
C
  100 OFFSET=(NCYS+2.5)*0.08
      IF(NCYS.LE.0) OFFSET=0.65
      XLOC=XMINR-OFFSET*XSC
      YLOC=0.5*(YMINR+YMAXR)-0.08*YSC*NCY
      IF (METRIC) GO TO 105
      CALL PLOT(3,FORMY,2,6,XLOC,YLOC)
      RETURN
  105 CALL PLOTM(3,FORMY,2,6,XLOC,YLOC)
C
  110 RETURN
      END
      SUBROUTINE LOGPR10(I,J,FORMLS,NCLS,RMINR,RMAXR,TICR,X1SC,X2SC,
     $                   OPMIN)
C
C
C LOGPR10 IS ONE OF THE MODULES OF THE AXIS LABELLING SYSTEM.  IT CAN
C HANDLE THE PLOTTING OF LINEAR SCALE TIC LABELS FOR EITHER THE X OR
C Y-AXIS.
C
C
C IF THE MAIN ENTRY IN THE AXIS LABELLING ROUTINE HAS BEEN USED (ENGLISH
C DIMENSIONS HAVE BEEN USED TO PLOT THE GRAPH) PLOT IS CALLED, AND IF
C THE ALTERNATE ENTRY HAS BEEN USED (METRIC DIMENSIONS HAVE BEEN USED
C TO PLOT THE GRAPH) PLOTM IS CALLED.
C
C
C ARGUMENT LIST:
C  I,J.....INTEGERS, TOGETHER THEY SPECIFY WHICH AXIS IS TO BE LABELLED:
C          1,2 - X-AXIS,  2,1 - Y-AXIS.
C  FORMLS..EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE LINEAR SCALE TIC LABELS.
C  NCLS....INTEGER, NUMBER OF CHARACTERS IN THE TIC LABELS.
C  RMINR...REAL, MINIMUM VALUE ON THE AXIS TO BE LABELLED.
C  RMAXR...REAL, MAXIMUM VALUE ON THE AXIS TO BE LABELLED.
C  TICR....EITHER REAL OR INTEGER, LINEAR TIC VALUE FOR THE AXIS TO BE
C          LABELLED.
C  X1SC....REAL, SCALING FACTOR (INCHES TO USER UNITS) FOR THE AXIS TO
C          BE LABELLED.
C  X2SC....REAL, SCALING FACTOR FOR THE OPPOSITE AXIS.
C  OPMIN...REAL, MINIMUM VALUE ON THE OPPOSITE AXIS.
C
C ERROR CONDITIONS:
C  LOGPR10 CHECKS FOR ONLY ONE ERROR CONDITION:
C   ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT IS 0
C     IF NCLS IS NEGATIVE, IT IS NOT CLEAR WHAT THE USER WOULD LIKE TO
C     HAVE DONE.  THUS THE VALUE IS ASSUMED TO BE ZERO, WHICH MEANS THAT
C     THE TIC LABELS FOR THIS AXIS WILL NOT BE PLOTTED.  THE MESSAGE IS
C     PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF THIS ERROR MESSAGE IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  LOGPR10 FIRST CHECKS NCLS FOR LEGALITY.  IF IT EQUALS ZERO, RETURN.
C  OTHERWISE, IF NCLS>0 TICR IS FORCED INTO TYPE REAL.  IF TICR IS THEN
C  FOUND TO BE @0, MEANING THAT NO TICS WERE PLOTTED ON THIS AXIS BY THE
C  MODE 1 CALL OF PLOGZZ OR PLOT, LOGPR10 RETURNS.  OTHERWISE IT GOES ON
C  TO PLOT THE TIC LABELS.  IT DOES THIS BY FIRST DETERMINING THE TYPE
C  OF FORMAT SPECIFIED.  THE FORMAT, FORMLS, IS SEARCHED FOR THE LETTER
C  I.  IF ONE IS FOUND THE FORMAT IS ASSUMED TO BE INTEGER.  IF NOT IT
C  IS ASSUMED REAL.  THE LOCATION OF THE FIRST TIC AS WELL AS THE OPPOS-
C  ITE AXIS LOCATION (SINCE IT IS THE SAME FOR ALL THE LABELS) ARE DET-
C  ERMINED.  IF THE X-AXIS TICS ARE TO BE LABELLED, AND IF NCLS IS LARGE
C  ENOUGH SO THAT THE LABELS WILL OVERLAP, THE TIC VALUE IS DOUBLED,
C  MEANING ONLY EVERY SECOND TIC WILL BE LABELLED.  THE TIC LABELS ARE
C  THEN PLOTTED BEGINNING WITH THE FIRST TIC LOCATION AND PROCEEDING IN
C  INCREMENTS OF THE TIC VALUE.  THE LABEL VALUES (WHICH ARE ALSO, IN
C  FACT, THE TIC LOCATIONS) ARE CALCULATED AS REALS AND THEN CONVERTED
C  TO INTEGERS, IF NEED BE, DEPENDING UPON THE TYPE OF THE FORMAT.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL X(2),OFFSET(2),MTOE
      INTEGER FORMLS(1), TEMP(10)
      LOGICAL IFORMAT,METRIC
      EQUIVALENCE (ITIC,RTIC)
      DATA OFFSET/0.2,0.0/
    1 FORMAT(10A1)
C
C
C     INITIALIZE ERROR NUMBER FOR AELIB ERROR ROUTINE.
C        3531 - ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS -
C                       DEFAULT IS 0.
C
      NERR=3531
C
C
C     CHECK NCLS FOR LEGALITY.  RETURN IF NCLS=0.
C
      IF(NCLS) 10,100,20
C
*IF -DEF,AELIB5
   10 CALL AELERR(NERR,2HNF)
*ENDIF
*IF DEF,AELIB5
   10 CALL AELERR(NERR,'NF')
*ENDIF
      GO TO 100
C
C
C     FORCE TICR INTO TYPE REAL.  RETURN IF NO TICS WERE PLOTTED BY
C     MODE 1.
C
   20 TIC=RTIC=TICR
      IF(SHIFT(ITIC,-48).EQ.0) TIC=ITIC
      IF(TIC.LE.0.0) GO TO 100
C
C
C     SCAN FORMAT FOR THE LETTER I.  IF PRESENT, ASSUME THE FORMAT IS
C     INTEGER.  OTHERWISE, ASSUME IT IS REAL.
C
      ILEFT=0
      IRIGHT=0
      IFORMAT=.TRUE.
      DO 25 K=1,3
         DECODE(10,1,FORMLS(K)) TEMP
         DO 23 K2=1,10
            IF(TEMP(K2).EQ.1HI) GO TO 27
            IF(TEMP(K2).EQ.1H() ILEFT=ILEFT+1
            IF(TEMP(K2).EQ.1H)) IRIGHT=IRIGHT+1
*IF -DEF,AELIB5
            IF((ILEFT.EQ.IRIGHT).A.(ILEFT.NE.0)) GO TO 26
*ENDIF
*IF DEF,AELIB5
            IF((ILEFT.EQ.IRIGHT).AND.(ILEFT.NE.0)) GO TO 26
*ENDIF
   23    CONTINUE
C
   25 CONTINUE
C
   26 IFORMAT=.FALSE.
C
C
C     ASSIGN RMIN THE ACTUAL MINIMUM OF RMINR AND RMAXR.
C     ASSIGN RMAX THE ACTUAL MAXIMUM OF RMINR AND RMAXR.
C
   27 RMIN=RMINR
      RMAX=RMAXR
      IF(RMIN.LE.RMAX) GO TO 30
      RMIN=RMAX
      RMAX=RMINR
C
C
C     MODIFY RMIN SO THAT IT REPRESENTS THE LOCATION OF THE FIRST TIC.
C
   30 N=RMIN/TIC
      IF(N*TIC.EQ.RMIN) GO TO 40
      IF(RMIN.GT.0.0) N=N+1
      RMIN=N*TIC
C
C
C     DETERMINE CONSTANT OPPOSITE AXIS LOCATION.
C
   40 OFFSET(2)=(NCLS+0.5)*0.08
      X(J)=OPMIN-OFFSET(I)*X2SC
C
C
C     IF X-AXIS AND IF SCALE NUMBERS OVERLAP, ONLY LABEL EVERY SECOND
C     TIC.
C
      NINC=1
      IF(I.EQ.2) GO TO 50
      IF(TIC/X1SC.LT.(NCLS+1)*0.08) NINC=2
C
C
C     PLOT THE SCALE NUMBERS, CONVERTING THEIR TYPE WHERE NECESSARY.
C
   50 X(I)=RMIN-(J-1)*0.04*NCLS*X1SC
      IF(IFORMAT) GO TO 60
      IF (METRIC) GO TO 55
      CALL PLOT(3,FORMLS,1,0,X(1),X(2),RMIN,1)
      GO TO 70
   55 CALL PLOTM(3,FORMLS,1,0,X(1),X(2),RMIN,1)
      GO TO 70
C
   60 IMIN=RMIN
      IF (METRIC) GO TO 65
      CALL PLOT(3,FORMLS,1,0,X(1),X(2),IMIN,1)
      GO TO 70
   65 CALL PLOTM(3,FORMLS,1,0,X(1),X(2),IMIN,1)
C
   70 N=N+NINC
      RMIN=N*TIC
      IF(RMIN.LE.RMAX) GO TO 50
C
C
  100 RETURN
      END
      SUBROUTINE LOGPR11(RMINR,RMAXR,ITIC,X1SC,X2SC,OPMIN,I,J,ITICS)
C
C
C LOGPR11 IS ONE OF THE MODULES OF THE AXIS LABELLING SYSTEM.  IT CAN
C HANDLE THE PLOTTING OF LOGARITHMIC SCALE TIC LABELS FOR EITHER THE X
C OR Y-AXIS.
C
C
C IF THE MAIN ENTRY IN THE AXIS LABELLING ROUTINE HAS BEEN USED (ENGLISH
C DIMENSIONS HAVE BEEN USED TO PLOT THE GRAPH) PLOT IS CALLED, AND IF
C THE ALTERNATE ENTRY HAS BEEN USED (METRIC DIMENSIONS HAVE BEEN USED
C TO PLOT THE GRAPH) PLOTM IS CALLED.
C
C
C ARGUMENT LIST:
C  RMINR...REAL, ALOG10(MINIMUM VALUE ON THE AXIS TO BE LABELLED).
C  RMAXR...REAL, ALOG10(MAXIMUM VALUE ON THE AXIS TO BE LABELLED).
C  ITIC....INTERGER, LOGARITHMIC TIC VALUE FOR THE AXIS TO BE LABELLED.
C  X1SC....REAL, SCALING FACTOR (INCHES TO LOGARITHMS OF USER UNITS) FOR
C          THE AXIS TO BE LABELLED.
C  X2SC....REAL, SCALING FACTOR FOR THE OPPOSITE AXIS.
C  OPMIN...REAL, MINIMUM VALUE ON THE OPPOSITE AXIS.
C  I,J.....INTEGERS, TOGETHER THEY SPECIFY WHICH AXIS IS TO BE LABELLED:
C          1,2 - X-AXIS,  2,1 - Y-AXIS.
C  ITICS...INTEGER, LOGARITHMIC TIC LABEL SWITCH:
C           =0  NO LOGARITHMIC TIC LABELS WILL BE PRINTED.
C           =-1 ONLY LABEL THE POWERS OF TEN.
C           OTHERWISE: LABEL ALL TICS.
C
C ERROR CONDITIONS:
C  LOGPR11 DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C
C METHOD:
C  IF THE LOGARITHMIC TICKING VALUE IS @0, MEANING THAT NO TICS WERE
C  PLOTTED ON THIS AXIS BY THE MODE 1 CALL OF PLOGZZ, LOGPR11 RETURNS.
C  OTHERWISE IT CHECKS THE LOGARITHMIC TIC LABEL SWITCH, ITICS.  IF
C  ITICS=0, LOGPR11 RETURNS.  IF ITICS=-1, LOGPR11 ARRANGES TO LABEL
C  ONLY THE POWERS OF TEN BY SETTING THE LOGARITHMIC TICKING VALUE,
C  ITIC, TO 1.  LOGPR12 IS THEN CALLED TO DETERMINE THE LOOP VALUES
C  THAT ARE TO BE USED WHEN LABELLING THE AXIS.  THESE LOOP VALUES
C  DEPEND UPON ITIC AND ENSURE THAT THE LABELLING WILL CORRESPOND WITH
C  THE TICKING THAT WAS PERFORMED BY PLOGZZ MODE 1.  ENCORPORATING THESE
C  VALUES WITHIN A DO LOOP, LOGPR11 THEN PLOTS THE LOGARITHMIC TIC
C  LABELS ACCORDING TO A STANDARD FORMAT.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL X(2),TABLOG(11),OFFSET(2),OFFST1(2),OFFST2(2),SC(2),MTOE
      LOGICAL METRIC
      DATA TABLOG/0.0,0.0,0.0,0.30103,0.47712,0.60206,0.69897,
     $            0.77815,0.845098,0.90309,0.954243/
      DATA OFFSET/0.30,0.56/, OFFST1/0.08,0.23/, OFFST2/0.12,0.10/
      DATA TITLE1/4H(I3)/, TITLE2/6H(2H10)/, TITLE3/4H(I1)/
C
C
C     RETURN IF ITIC@0 OR IF ITICS=0.  IF ITICS=-1, ARRANGE TO PLOT ONLY
C     POWERS OF TEN.
C
      IF(ITIC.LE.0) GO TO 40
      IF(ITICS.EQ.0) GO TO 40
      IF(ITICS.EQ.-1) ITIC=1
C
C
C     RMIN IS ASSIGNED ACTUAL MINIMUM OF RMINR AND RMAXR.
C     RMAX IS ASSIGNED ACTUAL MAXIMUM OF RMINR AND RMAXR.
C
      RMIN=RMINR
      RMAX=RMAXR
      IF(RMIN.LE.RMAX) GO TO 10
      RMIN=RMAX
      RMAX=RMINR
   10 SC(1)=X1SC
      SC(2)=X2SC
C
C
C     DETERMINE LOOP VALUES TO BE USED WHEN PLOTTING LOGARITHMIC TIC
C     LABELS.
C
      CALL LOGPR12(RMIN,RMAX,ITIC,MIN,MAX,I1,I2,ISTEP)
C
C
C     PLOT THE LOGARITHMIC TIC LABELS ACCORDING TO A STANDARD FORMAT.
C
   20 DO 30 K=I1,I2,ISTEP
         X(I)=MIN+TABLOG(K)
         IF(X(I).LT.RMIN) GO TO 30
         IF(X(I).GT.RMAX) GO TO 40
C
         IF(TABLOG(K).NE.0.0) GO TO 25
         X(I)=X(I)-OFFST1(J)*X1SC
         X(J)=OPMIN-OFFSET(I)*X2SC
         IF (METRIC) GO TO 21
         CALL PLOT(3,TITLE2,2,0,X(1),X(2))
         GO TO 23
   21    CALL PLOTM(3,TITLE2,2,0,X(1),X(2))
   23    X(1)=X(1)+0.26*SC(I)
         IF(MIN.GE.0) X(1)=X(1)-0.08*SC(I)
         X(2)=X(2)+0.125*SC(J)
         IF (METRIC) GO TO 24
         CALL PLOT(3,TITLE1,1,0,X(1),X(2),MIN,1)
         GO TO 30
   24    CALL PLOTM(3,TITLE1,1,0,X(1),X(2),MIN,1)
         GO TO 30
C
   25    X(J)=OPMIN-OFFST2(I)*X2SC
         L=K-2
         IF (METRIC) GO TO 27
         CALL PLOT(3,TITLE3,1,0,X(1),X(2),L,1)
         GO TO 30
   27    CALL PLOTM(3,TITLE3,1,0,X(1),X(2),L,1)
C
   30 CONTINUE
C
      MIN=MIN+1
      IF(MIN.LE.MAX) GO TO 20
C
C
   40 RETURN
      END
