*DECK AXLGXY
      SUBROUTINE AXLGXY(FORMX,NCX,FORMY,NCY,ITICXS,ITICYS)
************************************************************************
**                                                                    **
**    NOTE THAT AXLGXY IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL *
**    ROUTINES:                                                       **
**       LOGPR9                                                       **
**       LOGPR11                                                      **
**    LOGPR9 THROUGH LOGPR11 ARE STORED ON AELIB SOURCE TAPE WITH     **
**    ROUTINE AXLGX UNDER THE DECK NAME AXLGX.                        **
**                                                                    **
************************************************************************
C
C
C AXLGXY IS A SUBROUTINE FOR PLOTTING AXIS LABELS ON LOG - LOG GRAPHS.
C IT PLOTS THE TIC LABELS AND CENTRED AXIS TITLES.
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A)AXLGXY  - USED WHEN ENGLISH DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED
C    B)AXLGXYM - USED WHEN METRIC DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED.
C A SWITCH INDICATING IF METRIC DIMENSIONS HAVE BEEN USED IS PASSED IN
C A COMMON BLOCK TO THE ROUTINES CALLED BY THIS ROUTINE.  THE VALUE OF
C THIS SWITCH DETERMINES WHETHER CONVERSION OF DIMENSIONS IS REQUIRED
C AND WHETHER PLOT OR PLOTM IS TO BE CALLED.
C
C
C ARGUMENT LIST:
C  FORMX...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TITLE.
C  NCX.....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TITLE.
C  FORMY...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TITLE.
C  NCY.....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TITLE.
C  ITICXS..INTEGER, X-AXIS LOGARITHMIC TIC LABEL SWITCH.
C            =0  NO X-AXIS TIC LABELS WILL BE PRINTED.
C            =-1 ONLY LABEL THE POWERS OF 10.
C            OTHERWISE: LABEL ALL TICS.
C  ITICYS..INTEGER, Y-AXIS LOGARITHMIC TIC LABEL SWITCH.
C            =0  NO Y-AXIS TIC LABELS WILL BE PRINTED.
C            =-1 ONLY LABEL THE POWERS OF 10.
C            OTHERWISE: LABEL ALL TICS.
C
C ERROR CONDITIONS:
C  AXLGXY ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT TWO DIFFERENT ERROR
C  CONDITIONS:
C   1) ***AXLGXY CALL NOT PRECEEDED BY PLOGXY MODE 1 CALL
C        SINCE THE SCALING FACTORS AND TIC VALUES MUST BE RETRIEVED
C        THROUGH A MODE 4 CALL OF PLOGXY, A PLOGXY MODE 1 CALL MUST
C        PRECEED A CALL TO AXLGXY IN THE CALLING PROGRAM.  THIS IS A
C        FATAL ERROR.  THE MESSAGE IS PRINTED AND EXECUTION TERMINATED.
C   2) ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT
C         IS 0
C        IF EITHER OF THE PARAMETERS NCX OR NCY HAS A NEGATIVE VALUE
C        IT IS NOT CLEAR WHAT THE USER WOULD LIKE TO HAVE DONE.  THE
C        VALUE IS THUS ASSUMED TO BE ZERO, WHICH MEANS THAT THE PARTIC-
C        ULAR TITLE REFERRED TO WILL NOT BE PLOTTED.
C        THE MESSAGE IS PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF BOTH OF THESE ERROR MESSAGES IS HANDLED BY THE STAN-
C  DARD AELIB ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE AXIS LABELLING ROUTINES (AXLGX, AXLGY, AXLGXY, AXLIN) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE AXIS LABELLING PACKAGE.
C  MODULES IN THE PACKAGE INCLUDE LOGPR3, LOGPR9, LOGPR10, LOGPR11, AND
C  LOGPR12.  AXLGXY DOES NOT USE LOGPR10.  AXLGXY FIRST CALLS LOGPR9.
C  THIS ROUTINE CHECKS IF PLOGXY MODE 1 HAS BEEN CALLED PREVIOUSLY (IT
C  CALLS ENTRY POINT LOGPR5 IN MODULE LOGPR3 IN ORDER TO DO THIS).  IF
C  PLOGXY HAD NOT BEEN CALLED, A FATAL ERROR MESSAGE IS ISSUED.  OTHER-
C  WISE LOGPR9 GOES ON TO RETRIEVE THE MODE 1 PARAMETERS BY CALLING
C  ENTRY POINT LOGPR4 IN MODULE LOGPR3.  FROM THESE PARAMETERS THE
C  SCALING FACTORS ARE CALCULATED.  LOGPR9 THEN HANDLES THE PLOTTING
C  OF THE AXIS TITLES BEFORE RETURNING CONTROL BACK TO AXLGXY.  NEXT,
C  USING THE PARAMETERS RETURNED BY LOGPR9, AXLGXY MAKES TWO CALLS OF
C  LOGPR11.  THE FIRST CALL HANDLES THE PLOTTING OF THE LOGARITHMIC
C  X-AXIS TIC LABELS.  THE SECOND HANDLES THE LOGARITHMIC Y-AXIS TIC
C  LABELS.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
C
*CALL STATF
C
C
      METRIC=.FALSE.
      GO TO 100
C
C
C     ENTRY AXLGXYM ALLOWS METRIC DIMENSIONS TO BE USED
C
*IF -DEF,AELIB5
      ENTRY AXLGXYM
*ENDIF
*IF DEF,AELIB5
      ENTRY AXLGXYM(FORMX,NCX,FORMY,NCY,ITICXS,ITICYS)
*ENDIF
*CALL STATF
C
      ETOM=25.4
      MTOE=1./ETOM
      METRIC=.TRUE.
C
C     CHECK THAT PLOGXY MODE 1 HAS BEEN CALLED PREVIOUSLY AND RETIEVE
C     THE PARAMETERS.  CALCULATE THE SCALING FACTORS AND PLOT THE AXIS
C     TITLES (IF APPROPRIATE).
C
  100 CALL LOGPR9(3,FORMX,NCX,FORMY,NCY,XMINR,XMAXR,YMINR,YMAXR,TICXR,
     $            TICYR,XSC,YSC,0)
C
C
C     PLOT THE LOGARITHMIC X-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR11(XMINR,XMAXR,TICXR,XSC,YSC,YMINR,1,2,ITICXS)
C
C
C     PLOT THE LOGARITHMIC Y-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR11(YMINR,YMAXR,TICYR,YSC,XSC,XMINR,2,1,ITICYS)
C
      RETURN
      END
