*DECK AXLGY
      SUBROUTINE AXLGY(FORMX,NCX,FORMY,NCY,FORMXS,NCXS,ITICYS)
************************************************************************
**                                                                    **
**    NOTE THAT AXLGY IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL **
**    ROUTINES:                                                       **
**       LOGPR9                                                       **
**       LOGPR10                                                      **
**       LOGPR11                                                      **
**    LOGPR9 THROUGH LOGPR11 ARE STORED ON AELIB SOURCE TAPE WITH     **
**    ROUTINE AXLGX UNDER THE DECK NAME AXLGX.                        **
**                                                                    **
************************************************************************
C
C
C AXLGY IS A SUBROUTINE FOR PLOTTING AXIS LABELS ON SEMI-LOGARITHMIC
C GRAPHS (X-AXIS LINEAR, Y-AXIS LOGARITHMIC).  IT PLOTS THE TIC LABELS
C AND CENTRED AXIS TITLES.
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A) AXLGY  - USED WHEN ENGLISH DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED
C    B) AXLGYM - USED WHEN METRIC DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED.
C A SWITCH INDICATING IF METRIC DIMENSIONS HAVE BEEN USED IS PASSED IN
C A COMMON BLOCK TO THE ROUTINES CALLED BY THIS ROUTINE.  THE VALUE OF
C THIS SWITCH DETERMINES WHETHER CONVERSION OF DIMENSIONS IS REQUIRED
C AND WHETHER PLOT OR PLOTM IS TO BE CALLED.
C
C
C
C ARGUMENT LIST:
C  FORMX...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TITLE.
C  NCX.....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TITLE.
C  FORMY...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TITLE.
C  NCY.....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TITLE.
C  FORMXS..EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TIC LABELS.
C  NCXS....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TIC LABELS.
C  ITICYS..INTEGER, Y-AXIS LOGARITHMIC TIC LABEL SWITCH.
C            =0  NO Y-AXIS TIC LABELS WILL BE PRINTED.
C            =-1 ONLY LABEL THE POWERS OF 10.
C            OTHERWISE: LABEL ALL TICS.
C
C ERROR CONDITIONS:
C  AXLGY ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT TWO DIFFERENT ERROR
C  CONDITIONS:
C   1) ***AXLGY CALL NOT PRECEEDED BY PLOGY MODE 1 CALL
C        SINCE THE SCALING FACTORS AND TIC VALUES MUST BE RETRIEVED
C        THROUGH A MODE 4 CALL OF PLOGY, A PLOGY MODE 1 CALL MUST
C        PRECEED A CALL TO AXLGY IN THE CALLING PROGRAM.  THIS IS A
C        FATAL ERROR.  THE MESSAGE IS PRINTED AND EXECUTION TERMINATED.
C   2) ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT
C         IS 0
C        IF ANY OF THE PARAMETERS NCX, NCY OR NCXS HAVE NEGATIVE VALUES,
C        IT IS NOT CLEAR WHAT THE USER WOULD LIKE TO HAVE DONE.  THE
C        VALUE IS THUS ASSUMED TO BE ZERO, WHICH MEANS THAT THE PARTIC-
C        ULAR TITLE OR TIC LABELLING REFERRED TO WILL NOT BE PLOTTED.
C        THE MESSAGE IS PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF BOTH OF THESE ERROR MESSAGES IS HANDLED BY THE STAN-
C  DARD AELIB ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE AXIS LABELLING ROUTINES (AXLGX, AXLGY, AXLGXY, AXLIN) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE AXIS LABELLING PACKAGE.
C  MODULES IN THE PACKAGE INCLUDE LOGPR3, LOGPR9, LOGPR10, LOGPR11, AND
C  LOGPR12.  AXLGY FIRST CALLS LOGPR9.  THIS ROUTINE CHECKS IF PLOGY
C  MODE 1 HAS BEEN CALLED PREVIOUSLY (IT CALLS ENTRY POINT LOGPR5 IN
C  MODULE LOGPR3 IN ORDER TO DO THIS).  IF PLOGY HAD NOT BEEN CALLED,
C  A FATAL ERROR MESSAGE IS ISSUED.  OTHERWISE LOGPR9 GOES ON TO
C  RETRIEVE THE MODE 1 PARAMETERS BY CALLING ENTRY POINT LOGPR4 IN
C  MODULE LOGPR3.  FROM THESE PARAMETERS THE SCALING FACTORS ARE CALC-
C  ULATED.  LOGPR9 THEN HANDLES THE PLOTTING OF THE AXIS TITLES BEFORE
C  RETURNING CONTROL BACK TO AXLGY.  NEXT, USING THE PARAMETERS RETURNED
C  BY LOGPR9, AXLGY CALLS LOGPR10.  LOGPR10 HANDLES THE PLOTTING OF THE
C  LINEAR X-AXIS TIC LABELS.  FINALLY, AXLGY CALLS LOGPR11, AGAIN USING
C  THE PARAMETERS RETURNED BY LOGPR9.  LOGPR11 HANDLES THE PLOTTING OF
C  THE LOGARITHMIC Y-AXIS TIC LABELS (WITH THE HELP OF LOGPR12).
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
C
*CALL STATF
C
C
      METRIC=.FALSE.
      GO TO 100
C
C
C     ENTRY AXLGYM ALLOWS METRIC DIMENSIONS TO BE USED
C
*IF -DEF,AELIB5
      ENTRY AXLGYM
*ENDIF
*IF DEF,AELIB5
      ENTRY AXLGYM(FORMX,NCX,FORMY,NCY,FORMXS,NCXS,ITICYS)
*ENDIF
*CALL STATF
C
      ETOM=25.4
      MTOE=1./ETOM
      METRIC=.TRUE.
C
C     CHECK THAT PLOGY MODE 1 HAS BEEN CALLED PREVIOUSLY AND RETRIEVE
C     THE PARAMETERS.  CALCULATE THE SCALING FACTORS AND PLOT THE AXIS
C     TITLES (IF APPROPRIATE).
C
  100 CALL LOGPR9(2,FORMX,NCX,FORMY,NCY,XMINR,XMAXR,YMINR,YMAXR,TICXR,
     $            TICYR,XSC,YSC,0)
C
C
C     PLOT THE LINEAR X-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR10(1,2,FORMXS,NCXS,XMINR,XMAXR,TICXR,XSC,YSC,YMINR)
C
C
C     PLOT THE LOGARITHMIC Y-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR11(YMINR,YMAXR,TICYR,YSC,XSC,XMINR,2,1,ITICYS)
C
      RETURN
      END
