*DECK AXLIN
      SUBROUTINE AXLIN(FORMX,NCX,FORMY,NCY,FORMXS,NCXS,FORMYS,NCYS)
************************************************************************
**                                                                    **
**    NOTE THAT AXLIN IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL **
**    ROUTINES:                                                       **
**       LOGPR9                                                       **
**       LOGPR10                                                      **
**    LOGPR9 THROUGH LOGPR10 ARE STORED ON AELIB SOURCE TAPE WITH     **
**    ROUTINE AXLGX UNDER THE DECK NAME AXLGX.                        **
**                                                                    **
************************************************************************
C
C
C AXLIN IS A SUBROUTINE FOR PLOTTING AXIS LABELS ON LINEAR - LINEAR
C GRAPHS.  IT PLOTS THE TIC LABELS AND CENTRED AXIS TITLES.
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A) AXLIN  - USED WHEN ENGLISH DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED
C    B) AXLINM - USED WHEN METRIC DIMENSIONS HAVE BEEN USED IN THE
C CALLING PROGRAM TO PLOT THE GRAPH WHICH IS TO BE LABELLED.
C A SWITCH INDICATING IF METRIC DIMENSIONS HAVE BEEN USED IS PASSED IN
C A COMMON BLOCK TO THE ROUTINES CALLED BY THIS ROUTINE.  THE VALUE OF
C THIS SWITCH DETERMINES WHETHER CONVERSION OF DIMENSIONS IS REQUIRED
C AND WHETHER PLOT OR PLOTM IS TO BE CALLED.
C
C
C ARGUMENT LIST:
C  FORMX...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TITLE.
C  NCX.....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TITLE.
C  FORMY...EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TITLE.
C  NCY.....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TITLE.
C  FORMXS..EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE X-AXIS TIC LABELS.
C  NCXS....INTEGER, NUMBER OF CHARACTERS IN THE X-AXIS TIC LABELS.
C  FORMYS..EITHER REAL OR INTEGER ARRAY CONTAINING A VARIABLE FORMAT FOR
C          THE Y-AXIS TIC LABELS.
C  NCYS....INTEGER, NUMBER OF CHARACTERS IN THE Y-AXIS TIC LABELS.
C
C ERROR CONDITIONS:
C  AXLIN ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT TWO DIFFERENT ERROR
C  CONDITIONS:
C   1) ***NO MODE 1 CALL BEFORE CALL OF MODE 2,3,4 OR 5 (ISSUED BY PLOT)
C        SINCE THE SCALING FACTORS AND TIC VALUES MUST BE RETRIEVED
C        THROUGH A MODE 4 CALL OF PLOT, A PLOT MODE 1 CALL MUST
C        PRECEED A CALL TO AXLIN IN THE CALLING PROGRAM.  THIS IS A
C        FATAL ERROR.  THE MESSAGE IS PRINTED AND EXECUTION TERMINATED.
C   2) ***NEGATIVE VALUE SPECIFIED FOR NUMBER OF CHARACTERS - DEFAULT
C         IS 0
C        IF ANY OF THE PARAMETERS NCX, NCY, NCXS OR NCYS ARE NEGATIVE
C        IT IS NOT CLEAR WHAT THE USER WOULD LIKE TO HAVE DONE.  THE
C        VALUE IS THUS ASSUMED TO BE ZERO, WHICH MEANS THAT THE PARTIC-
C        ULAR TITLE OR TIC LABELLING REFERRED TO WILL NOT BE PLOTTED.
C        THE MESSAGE IS PRINTED AND EXECUTION CONTINUES.
C  THE PRINTING OF BOTH OF THESE ERROR MESSAGES IS HANDLED BY THE STAN-
C  DARD AELIB ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE AXIS LABELLING ROUTINES (AXLGX, AXLGY, AXLGXY, AXLIN) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE AXIS LABELLING PACKAGE.
C  MODULES IN THE PACKAGE INCLUDE LOGPR3, LOGPR9, LOGPR10, LOGPR11, AND
C  LOGPR12.  AXLIN DOES NOT USE LOGPR11 AND LOGPR12.  AXLIN FIRST CALLS
C  LOGPR9.  THIS ROUTINE RIETRIEVES THE PLOT MODE 1 PARAMETERS BY CALL-
C  ING PLOT MODE 4.  FROM THESE PARAMETERS THE SCALING FACTORS ARE CALC-
C  ULATED.  LOGPR9 THEN HANDLES THE PLOTTING OF THE AXIS TITLES BEFORE
C  RETURNING CONTROL BACK TO AXLIN.  NEXT, USING THE PARAMETERS RETURNED
C  BY LOGPR9, AXLIN MAKES TWO CALLS OF LOGPR10.  THE FIRST CALL HANDLES
C  THE PLOTTING OF THE LINEAR X-AXIS TIC LABELS.  THE SECOND HANDLES THE
C  LINEAR Y-AXIS TIC LABELS.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      LOGICAL METRIC
      REAL MTOE
C
*CALL STATF
      METRIC=.FALSE.
      GO TO 100
C
C
C     ENTRY AXLINM ALLOWS METRIC DIMENSIONS TO BE USED
C
*IF -DEF,AELIB5
      ENTRY AXLINM
*ENDIF
*IF DEF,AELIB5
      ENTRY AXLINM(FORMX,NCX,FORMY,NCY,FORMXS,NCXS,FORMYS,NCYS)
*ENDIF
*CALL STATF
C
      ETOM=25.4
      MTOE=1/ETOM
      METRIC=.TRUE.
C
C     RETRIEVE THE PARAMETERS OF THE PREVIOUS PLOT MODE 1 CALL.  CALC-
C     ULATE THE SCALING FACTORS AND PLOT THE AXIS TITLES (IF APPROP-
C     RIATE).
C
  100 CALL LOGPR9(4,FORMX,NCX,FORMY,NCY,XMINR,XMAXR,YMINR,YMAXR,TICXR,
     1            TICYR,XSC,YSC,NCYS)
C
C
C     PLOT THE LINEAR X-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR10(1,2,FORMXS,NCXS,XMINR,XMAXR,TICXR,XSC,YSC,YMINR)
C
C
C     PLOT THE LINEAR Y-AXIS TIC LABELS (IF APPROPRIATE).
C
      CALL LOGPR10(2,1,FORMYS,NCYS,YMINR,YMAXR,TICYR,YSC,XSC,XMINR)
C
      RETURN
      END
