*DECK CADRE
      FUNCTION CADRE(A,B,F,EPS,IFLAG)
      COMMON/STATS/ERR, KOUNT, INTS
C  THIS FUNCTION RETURNS AN ESTIMATE, CADRE, FOR THE NUMBER
C       INT=INTEGRAL OF F(X) FROM A TO B
C  WHICH SHOULD SATISFY
C       ABS(INT-CADRE) .LE. EPS*ABS(INT)
C  THE PROGRAM USES CAUTIOUS ADAPTIVE ROMBERG EXTRAPOLATION.  IN THIS
C  SCHEME, THE INTEGRAL IS CALCULATED AS THE SUM OF INTEGRALS OVER
C  SUITABLY SMALL SUBINTERVALS.  ON EACH SUBINTERVAL, AN ESTIMATE, VINT,
C  WITH ESTIMATED ABSOLUTE ERROR, ERRER, IS FOUND BY CATIOUS ROMBERG
C  EXTRAPOLATION.  IF ERRER IS SMALL ENOUGH, VINT IS ACCEPTED AND ADDED
C  TO CADRE, AND ERRER IS ADDED TO ERR.  OTHERWISE THE SUBINTERVAL IS
C  HALVED, AND EACH HALF IS CONSIDERED SEPARATELY, INFORMATION ABOUT
C  THE OTHER HALF BEING TEMPORARILY STACKED.
C
C                       INPUT
C
C  F    - THE NAME OF A SINGLE ARGUMENT REAL FUNCTION SUBPROGRAM THIS
C         NAME MUST APPEAR IN THE CALLING PROGRAM IN AN EXTERNAL
C         STATEMENT.
C  A,B  - THE UPPER AND LOWER LIMITS OF INTEGRATION.
C  EPS  - THE REQUIRED ERROR TOLERANCE.
C  IFLAG INTEGER ON INPUT TO CONTROL DIAGNOSTIC PRINTING
C                ON RETURN TO FLAG ERRORS IF ANY
C        IF =0 OR 1  DO NOT PRINT DIAGNOSTICS
C           =  2 - SUCCESS OR FAILURE MESSAGE AND LIST OF SINGULARITIES
C                  ENCOUNTERED (IF ANY).
C           =  3 - IN ADDITION, ALL SUBINTERVALS CONSIDERED ARE LISTED
C                  TOGETHER WITH THE KIND OF REGULAR BEHAVIOUR FOUND
C                  (IF ANY).
C           =  4 - IN ADDITION , ALL RATIOS CONSIDERED ARE LISTED AS IS
C                  INFO ON WHICH DECISION PROCEDURE IS BASED.
C           =  5 - IN ADDITION, ALL T-TABLES ARE LISTED.
C
C                       OUTPUT
C
C  CADRE - ESTIMATE OF THE INTEGRAL, RETURNED VIA THE FUNCTION CALL.
C  ERR   - ESTIMATE OF THE ERROR, RETURNED THROUGH THE COMMON BLOCK.
C  KOUNT - NUMBER OF FUNCTION EVALUATIONS USED, RETURNED THROUGH THE
C          COMMON BLOCK, STATS.
C  INTS  - NUMBER OF INTERVALS USED TO MEET THE RELATIVE ERROR
C          REQUIREMENT, RETURNED THROUGH THE COMMON BLOCK, STATS.
C  IFLAG - SET TO INDICATE WHETHER ANY DIFFICULTIES WERE ENCOUNTERED
C       IF GE 0  NO DIFFICULTIES
C            -2  ONE OR MORE SINGULARITIES WERE HANDLED SUCCESSFULLY
C            -3  IN SOME SUBINTERVAL(S) THE ESTIMATE WAS ACCEPTED
C               MERELY BECAUSE ERRER WAS SMALL, EVEN THOUGH NO REGULAR
C               BEHAVIOUR WAS RECOGNIZED.
C            -4  CONVERGENCE FAILURE.
C                TOO MANY INTERVALS REQUIRED , STORAGE LIMIT REACHED.
C            -5  CONVERGENCE FAILURE.
C                TOO SMALL INTERVALS REQUIRED FOR MACHINE ACCURACY.
C               MAY BE DUE TO TOO MUCH NOISE IN THE FUNCTION (RELATIVE
C               TO THE GIVEN ERROR REQUIREMENTS) OR DUE TO A PLAIN
C               ORNERY INTEGRAND.
C
C                       LIST OF MAJOR VARIABLES
C
C  CUREST BEST ESTIMATE SO FAR FOR
C              INT - (INTEGRAL OVER CURRENTLY CONSIDERED SUBINTERVAL)
C  FNSIZE MAXIMUM AVERAGE FUNCTION SIZE SO FAR ENCOUNTERED
C  ERRR   RELATIVE ERROR REQUIREMENT USED. (DERIVED FROM INPUT EPS)
C  STAGE  EQUAL TO 2**(-(ISTAGE)
C  THESE FOUR QUANTITIES ARE USED IN THE DETERMINATION OF THE LOCAL
C  ERROR REQUIREMENT.
C
C  STEPMN MINIMUM SUBINTERVAL LENGTH PERMITTED.
C  TS     STACK OF VALUES OF F(X) SO FAR COMPLETED BUT NOT YET
C         SUCCESSFULLY USED.
C  ISTAGE AN INTEGER INDICATING THE HEIGHT OF THE STACK OF INTERVALS
C         YET TO BE PROCESSED.
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C       SOURCE C DEBOOR  CADRE AN ALGORITHM FOR NUMERICAL QUADRATURE.
C                       MATHEMATICAL SOFTWARE J R RICE ED. ACADEMIC 1971
C       PROGRAMMED BY    CARL DEBOOR
C       REVISED    BY    VALERIE JONES                     AUG 1976
C
C REVISION HISTORY :
C
C       REVISED    BY    LEVANS TO FIX ERROR MESSAGE  FEB 1979
C
C     OCT 3, 1980  J.M. BLAIR - REVISION CHANGES EPS FROM ABSOLUTE ERROR
C                               TOLERANCE TO RELATIVE ERROR TOLERANCE.
C                               SEE NEWSLETTER VOL.7, NO.10.
      COMMON/PARAM/RATIO
*CALL ERRORF
      DIMENSION MES(2)
      DIMENSION T(10,10),R(10),AIT(10),DIF(10),RN(4),TS(2049),IBEGS(30),
     ,      BEGIN(30),FINIS(30),EST(30)
      LOGICAL H2CONV,AITKEN,RIGHT,REGLAR,REGLSV(30)
      REAL LENGTH, JUMPTL
      DATA TOLMCH,AITLOW,H2TOL,AITTOL,JUMPTL,MAXTS,MAXTBL,MXSTGE
     ,      /2.E-13, 1.1, .15, .1, .01, 2049, 10, 30/
      DATA RN/.71420053, .34662815, .843751, .12633046/
      DATA ALG402/.301029995664/
      DATA MES/2260,2261/
C
*CALL STATF
      CADRE=0
      ERR=0.
      LEVEL=IFLAG=IABS(IFLAG)
      LENGTH=B-A
      IF (LENGTH.EQ.0.) RETURN
      ERRR=AMIN1(.1,AMAX1(ABS(EPS), 10.*TOLMCH))
      STEPMN=AMAX1(LENGTH/FLOAT(2**MXSTGE),
     ,      AMAX1(LENGTH,ABS(A),ABS(B))*TOLMCH)
      STAGE=.5
      ISTAGE=1
      AIT(1)=0.
      CUREST=0.
      FNSIZE=0.
      PREVER=0.
      REGLAR=.FALSE.
C
C     THE GIVEN INTERVAL IS THE FIRST INTERVAL CONSIDERED.
C
      BEG=A
      FBEG=F(BEG)/2.
      TS(1)=FBEG
      IBEG=1
      END=B
      FEND=F(END)/2.
      TS(2)=FEND
      IEND=2
      KOUNT = 2
      KOUNTU = 0
      INTS = 0
C
    5 RIGHT=.FALSE.
C
C     INVESTIGATION OF A PARTICULAR SUBINTERVAL BEGINS AT THIS POINT.
C
C                       MAJOR VARIABLES
C     BEG
C     END    ENDPOINTS OF THE CURRENT INTERVAL
C     FBEG
C     FEND   ONE HALF THE VALUE OF F(X) AT THE ENDPOINTS
C     STEP   SIGNED LENGTH OF CURRENT SUBINTERVAL
C     ISTAGE HEIGHT OF THE CURRENT SUBINTERVAL IN STACK OF SUBINTERVALS
C            YET TO BE DONE
C     RIGHT  A LOGICAL VARIABLE INDICATING WHETHER CURRENT SUBINTERVAL
C            IS RIGHT HALF OF PREVIOUS INTERVAL. NEEDED IN 80FF AND
C            90FF TO DECIDE WHAT INTERVAL TO LOOK AT NEXT.
C     TS(I), I=IBEG, IEND, CONTAINS THE FUNCTION VALUES FOR THIS
C            SUBINTERVAL SO FAR COMPUTED. SPECIFICALLY
C               TS(I) = F(BEG + (I-IBEG)/(IEND-IBEG)*STEP), ALL I
C            EXCEPT THAT TS(IBEG) = FBEG, TS(IEND) = FEND
C     REGLAR A LOGICAL VARIABLE INDICATING WHETHER OR NOT THE CURRENT
C            SUBINTERVAL IS REGULAR
C     H2CONV A LOGICAL VARIABLE INDICATING WHETHER H**2 CONVERGENCE OF
C            THE TRAPEZOID SUMS FOR THIS INTERVAL IS RECOGNIZED
C     AITKEN A LOGICAL VARIABLE INDICATING WHETHER CONVERGENCE OF RATIOS
C            FOR THIS SUBINTERVAL IS RECOGNIZED
C     T      CONTAINS THE FIRST L ROWS OF THE ROMBERG T-TABLE FOR THIS
C            SUBINTERVAL IN ITS LOWER TRIANGULAR PART. SPECIFICALLY
C               T(I,1) = TRAPAZOID SUM (WITHOUT THE FACTOR STEP)
C                        ON 2**(I-1) + 1 EQUISPACED POINTS, I=1,L
C               T(I,J+1) = T(I,J) + (T(I,J)-T(I-1,J))/(4**J - 1)
C                          J=2,I-1, I=2,L
C            FURTHER, THE STRICTLY UPPER TRIANGULAR PART OF T CONTAINS
C            THE RATIOS FOR THE VARIOUS COLUMNS OF THE T-TABLES.
C            SPECIFICALLY
C               T(J,I) = (T(I,J)-T(I-1,J))/(T(I+1,J)-T(I,J)),
C               I=J+1,L-1,  J=1,L-1
C            FINALLY, THE LAST OR L-TH COLUMN CONTAINS
C               T(J,L) = T(L,J) - T(L-1,J), J=1,L-1
C
   10 STEP=END-BEG
      ASTEP=ABS(STEP)
      IF (ASTEP.LT.STEPMN) GOTO 335
      IF (LEVEL.GE.3) PRINT 15,BEG,STEP,ISTAGE
   15 FORMAT(10H BEG, STEP , 2E16.8, I5)
      T(1,1)=FBEG+FEND
      TABS=ABS(FBEG)+ABS(FEND)
      L=1
      N=1
      H2CONV=.FALSE.
      AITKEN=.FALSE.
      GOTO 25
   20 IF (LEVEL.GE.4) PRINT 295,L,T(1,LM1)
   25 LM1=L
      L=L+1
C
C     CALCULATE THE NEXT TRAPEZOID SUM, T(L,1), WHICH IS BASED ON
C     N2 + 1 EQUISPACED POINTS. HERE, N2 = N*2 = 2**(L-1).
C
      KOUNT1 = 0
      N2=N*2
      FN=N2
      ISTEP=(IEND-IBEG)/N
      IF (ISTEP.GT.1) GOTO 35
         II=IEND
         IEND=IEND+N
         IF (IEND.GT.MAXTS) GOTO 325
         HOVN=STEP/FN
         III=IEND
         DO 30 I=1,N2,2
            TS(III)=TS(II)
            TS(III-1)=F(END-FLOAT(I)*HOVN)
            KOUNT = KOUNT + 1
            KOUNT1 = KOUNT1 + 1
            III=III-2
   30    II=II-1
         ISTEP=2
   35 ISTEP2=IBEG+ISTEP/2
      SUM=0.
      SUMABS=0.
      DO 40 I=ISTEP2,IEND,ISTEP
         SUM=SUM+TS(I)
   40 SUMABS=SUMABS+ABS(TS(I))
      T(L,1) =T(L-1,1)/2.+SUM/FN
      TABS=TABS/2.+SUMABS/FN
      ABSI=ASTEP*TABS
      N=N2
C
C     GET PRELIMINARY VALUE FOR VINT FROM LAST TRAPEZOID SUM AND UPDATE
C     THE ERROR REQUIREMENT ERGOAL FOR THIS SUBINTERVAL.  THE ERROR
C     REQUIREMENT IS NOT PRORATED ACCORDING TO THE LENGTH OF THE CURRENT
C     SUBINTERVAL RELATIVE TO THE INTERVAL OF INTEGRATION, BUT ACCORDING
C     TO THE HEIGHT, ISTAGE, OF THE CURRENT SUBINTERVAL IN THE STACK OF
C     SUBINTERVALS YET TO BE DONE.
C     THIS PROCEDURE IS NOT BACKED BY ANY RIGOROUS ARGUMENT, BUT
C     SEEMS TO WORK.
C
      IT=1
      VINT=STEP*T(L,1)
      TABTLM=TABS*TOLMCH
      FNSIZE=AMAX1(FNSIZE,ABS(T(L,1)))
      ERGOAL = AMAX1( ASTEP * TOLMCH * FNSIZE, STAGE * ERRR * ABS(
     ,                     CUREST + VINT ) )
C
C     COMPLETE ROW L AND COLUMN L OF THE T ARRAY.
C
      FEXTRP=1.
      DO 45 I=1,LM1
         FEXTRP=FEXTRP*4.
         T(I,L)=T(L,I)-T(L-1,I)
   45 T(L,I+1)=T(L,I)+T(I,L)/(FEXTRP-1.)
      ERRER=ASTEP*ABS(T(1,L))
C
C                  PRELIMINARY DECISION PROCEDURE
C
C     IF L = 2 AND T(2,1) = T(1,1), GO TO 60 TO FOLLOW UP THE IMPRESSION
C     THAT INTEGRAND IS STRAIGHT LINE.
C
      IF(L.GT.2) GOTO 50
         IF(ABS(T(1,2)).LE.TABTLM) GOTO 205
         GOTO 25
C
C     CALCULATE NEW RATIOS FOR COLUMNS 1,...,L-2 OF T-TABLE
C     RATIO IS SET TO ZERO IF DIFFERENCE IN LAST TWO ENTRIES OF
C     COLUMN IS ABOUT ZERO.
C
   50 DO 55 I=2,LM1
         DIFF=0.
         IF (ABS(T(I-1,L)).GT.TABTLM) DIFF=T(I-1,LM1)/T(I-1,L)
   55 T(I-1,LM1)=DIFF
C
C     T(1,LM1) IS THE RATIO DERIVED FROM LAST THREE TRAPEZOID SUMS, IE.,
C        T(1,LM1) = (T(L-1,1)-T(L-2,1))/(T(L,1)-T(L-1,1)).
C     IF THIS RATIO IS ABOUT 4, GO TO 20 FOR ROMBERG EXTRAPOLATION.
C     IF THIS RATIO IS ZERO, IE., IF LAST TWO TRAPEZOID SUMS ARE ABOUT
C        EQUAL, GO TO 18 FOR POSSIBLE NOISE CHECK.
C     IF THIS RATIO IS ABOUT 2 IN ABSOLUTE VALUE, GO TO 50 WITH THE
C        BELIEF THAT INTEGRAND HAS JUMP DISCONTINUITY.
C     IF THIS RATIO IS, AT LEAST ABOUT EQUAL TO THE PREVIOUS RATIO, THEN
C        THE INTEGRAND MAY WELL HAVE A NICE INTEGRABLE SINGULARITY.
C        GO TO 30 TO FOLLOW UP THIS HUNCH.
C
      IF(ABS(4.-T(1,LM1)).LE.H2TOL) GOTO 70
      IF(T(1,LM1).EQ.0.) GOTO 65
      IF (ABS(2.-ABS(T(1,LM1))).LT.JUMPTL) GOTO 190
      IF (L.EQ.3) GOTO 20
      H2CONV=.FALSE.
      IF (ABS((T(1,LM1)-T(1,L-2))/T(1,LM1)).LE.AITTOL) GOTO 95
C
C     AT THIS POINT, NO REGULAR BEHAVIOUR WAS DETECTED.
C     IF CURRENT SUBINTERVAL IS NOT REGULAR AND ONLY 4 TRAPAZOID SUMS
C     WERE COMPUTED SO FAR, TRY ONE MORE TRAPAZOID SUM.
C     IF, AT LEAST, LAST TWO TRAPAZOID SUMS ARE ABOUT EQUAL, THEN
C     FAILURE TO RECOGNIZE REGULAR BEHAVIOUR MAY WELL BE DUE TO NOISE.
C     GO TO 70 TO CHECK THIS OUT.
C     OTHERWISE GO TO 90 FOR FURTHER SUBDIVISIONS.
C
   60 IF (REGLAR) GOTO 65
         IF (L.EQ.4) GOTO 20
   65 IF (ERRER.LE.ERGOAL) GOTO 225
      IF (LEVEL.GE.4) PRINT 295,L,T(1,LM1)
      GOTO 285
C
C                  CAUTIOUS ROMBERG EXTRAPOLATION
C
C     THE CURRENT, OR L-TH, ROW OF THE ROMBERG T-TABLE HAS L ENTRIES.
C     FOR J=1,...,L-2, THE ESTIMATE
C             STEP*T(L,J+1)
C     IS BELIEVED TO HAVE ITS ERROR BOUNDED BY
C        ABS(STEP*T(L,J)-T(L-1,J))/(4**J-1))
C     IF THE LAST RATIO
C        T(J,LM1) = (T(L-1,J)-T(L-2,J))/(T(L,J)-T(L-1,J))
C     FOR COLUMN J OF THE T-TABLE IS ABOUT 4**J.
C     THE FOLLOWING IS A SLIGHTLY RELAXED EXPRESSION OF THIS BELIEF.
C
   70 IF (LEVEL.GE.4) PRINT 75,L,T(1,LM1)
   75 FORMAT(I5, E16.8, 5X, 6HH2CONV)
      IF (H2CONV) GOTO 85
         AITKEN=.FALSE.
         H2CONV=.TRUE.
         IF (LEVEL.GE.3) PRINT 80,L
*IF -DEF,AELIB5
   80    FORMAT(22H H2CONVERGENCE AT ROW, I3)
*ENDIF
*IF DEF,AELIB5
   80    FORMAT(22H H2CONVERGENCE AT ROW, ,I3)
*ENDIF
   85 FEXTRP=4.
   90 IT=IT+1
      VINT=STEP*T(L,IT)
      ERRER=ABS(STEP/(FEXTRP-1.)*T(IT-1,L))
      IF (ERRER.LE.ERGOAL) GOTO 250
      IF (IT.EQ.LM1) GOTO 180
      IF (T(IT,LM1).EQ.0.) GOTO 90
      IF (T(IT,LM1).LE.FEXTRP) GOTO 180
      IF (ABS(T(IT,LM1)/4.-FEXTRP)/FEXTRP.LT.AITTOL) FEXTRP=FEXTRP*4.
      GOTO 90
C
C     INTEGRAND MAY HAVE X**ALPHA TYPE SINGULARITY RESULTING IN A RATIO
C     OF SING = 2**(ALPHA + 1)
C
   95 IF (LEVEL.GE.4) PRINT 100,L,T(1,LM1)
  100 FORMAT(I5, E16.8, 5X, 6HAITKEN)
      IF (T(1,LM1).LT.AITLOW) GOTO 285
      IF (AITKEN) GOTO 110
         H2CONV=.FALSE.
         AITKEN=.TRUE.
         IF (LEVEL.GE.3) PRINT 105,L
  105    FORMAT(14H AITKEN AT ROW, I3)
  110 FEXTRP=T(L-2,LM1)
      IF (FEXTRP.GT.4.5) GOTO 85
      IF (FEXTRP.LT.AITLOW) GOTO 285
      IF (ABS(FEXTRP-T(L-3,LM1))/T(1,LM1).GT.H2TOL) GOTO 285
      IF (LEVEL.GE.3) PRINT 115,FEXTRP
  115 FORMAT(6H RATIO,F12.8)
      SING=FEXTRP
      FEXTM1=FEXTRP-1.
      DO 120 I=2,L
         AIT(I)=T(I,1)+(T(I,1)-T(I-1,1))/FEXTM1
         R(I)=T(1,I-1)
  120 DIF(I)=AIT(I)-AIT(I-1)
      IT=2
  125 VINT=STEP*AIT(L)
      IF (LEVEL.LT.5) GOTO 135
         PRINT 130,(R(I+1),I=IT,LM1)
         PRINT 130,(AIT(I),I=IT,L)
         PRINT 130,(DIF(I+1),I=IT,LM1)
  130    FORMAT(1X, 8E15.8)
  135 ERRER=ERRER/FEXTM1
      IF (ERRER.GT.ERGOAL) GOTO 145
         ALPHA=ALOG10(SING)/ALG402-1.
         IF (LEVEL.GE.2) PRINT 140,ALPHA,BEG,END
*IF -DEF,AELIB5
  140    FORMAT(11X, 42HINTEGRAND SHOWS SINGULAR BEHAVIOR OF TYPE ,
     ,            "X**("F4.2")"/"  BETWEEN "E15.8" AND "E15.8)
*ENDIF
*IF DEF,AELIB5
  140    FORMAT(11X, 42HINTEGRAND SHOWS SINGULAR BEHAVIOR OF TYPE ,
     ,            'X**(',F4.2,')'/'  BETWEEN ',E15.8,' AND ',E15.8)
*ENDIF
         IFLAG=MIN0(IFLAG,-2)
         GOTO 250
  145 IT=IT+1
      IF(IT.EQ.LM1) GOTO 180
      IF (IT.GT.3) GOTO 150
         H2NEXT=4.
         SINGNX=2.*SING
  150 IF (H2NEXT.LT.SINGNX) GOTO 155
         FEXTRP=SINGNX
         SINGNX=2.*SINGNX
         GOTO 160
  155 FEXTRP=H2NEXT
      H2NEXT=4.*H2NEXT
  160 DO 165 I=IT,LM1
         R(I+1)=0.
  165 IF (ABS(DIF(I+1)).GT.TABTLM) R(I+1)=DIF(I)/DIF(I+1)
      IF (LEVEL.GE. 4) PRINT 170,FEXTRP,R(L-1),R(L)
  170 FORMAT(16H FEXTRP + RATIOS, 3F15.8)
      H2TFEX=-H2TOL*FEXTRP
      IF (R(L)-FEXTRP.LT.H2TFEX) GOTO 180
      IF(R(L-1)-FEXTRP.LT.H2TFEX) GOTO 180
      ERRER=ASTEP*ABS(DIF(L))
      FEXTM1=FEXTRP-1.
      DO 175 I=IT,L
         AIT(I)=AIT(I)+DIF(I)/FEXTM1
  175 DIF(I)=AIT(I)-AIT(I-1)
      GOTO 125
C
C     CURRENT TRAPEZOID SUM AND RESULTING EXTRAPOLATED VALUES DID NOT
C     GIVE A SMALL ENOUGH ERRER.
C     IF LESS THAN FIVE TRAPEZOID SUMS WERE COMPLETED SO FAR, TRY NEXT
C     TRAPEZOID SUM.
C     OTHERWISE, DECIDE WHETHER TO GO ON OR TO SUBDIVIDE AS FOLLOWS.
C     WITH T(L,IT) GIVING THE CURRENT BEST ESTIMATE, GIVE UP ON
C     DEVELOPING THE T-TABLE FURTHER IF L .GT. IT+2, IE., IF
C     EXTRAPOLATION DID NOT GO VERY FAR INTO THE T-TABLE.
C     FURTHER, GIVE UP IF REDUCTION IN ERRER AT CURRENT RATE DOES NOT
C     PREDICT AN ERRER LESS THAN ERGOAL BY THE TIME MAXTPL
C     TRAPEZOID SUMS HAVE BEEN COMPLETED.
C     NOTE
C     HAVING PREVER .LT. ERRER IS AN ALMOST CERTAIN SIGN OF BEGINNING
C     TROUBLE WITH NOISE IN THE FUNCTION VALUES.  HENCE, A WATCH FOR,
C     AND CONTROL OF, NOISE SHOULD BEGIN HERE.
C
  180 FEXTRP=AMAX1(PREVER/ERRER, AITLOW)
      PREVER=ERRER
      IF (L.LT.5) GOTO 25
      IF (LEVEL.GE.3) PRINT 185,ERRER,ERGOAL,FEXTRP,IT
  185 FORMAT(23H ERRER,ERGOAL,FEXTRP,IT, 2E15.8, E14.5, I3)
      IF (L-IT.GT.2 .AND. ISTAGE.LT.MXSTGE) GOTO 280
      IF (ERRER/FEXTRP**(MAXTBL-L).LT.ERGOAL) GOTO 25
      GOTO 280
C
C     INTEGRAND HAS JUMP.
C
  190 IF (LEVEL.GE.4) PRINT 195,L,T(1,LM1)
  195 FORMAT(I5, E16.8, 5X, 4HJUMP)
      IF (ERRER.GT.ERGOAL) GOTO 280
      DIFF=ABS(T(1,L))*2.*FN
      IF (LEVEL.GE.2) PRINT 200,DIFF,BEG,END
*IF -DEF,AELIB5
  200 FORMAT(13X, 36HINTEGRAND SEEMS TO HAVE JUMP OF SIZE, E13.6,
     ,      /"  BETWEEN "E15.8" AND "E15.8)
*ENDIF
*IF DEF,AELIB5
  200 FORMAT(13X, 36HINTEGRAND SEEMS TO HAVE JUMP OF SIZE, E13.6,
     ,      /'  BETWEEN ',E15.8,' AND ',E15.8)
*ENDIF
      GOTO 250
C
C     INTEGRAND IS STRAIGHT LINE
C     TEST THIS ASSUMPTION BY COMPARING THE VALUE OF THE INTEGRAND AT
C     FOUR RANDOMLY CHOSEN POINTS WITH THE VALUE OF THE STRAIGHT LINE
C     INTERPOLATING THE INTEGRAND AT THE TWO ENDPOINTS OF THE
C     SUBINTERVAL.  IF TEST IS PASSED, ACCEPT VINT.
C
  205 IF (LEVEL.GE.4) PRINT 210,L
  210 FORMAT(I5, 21X, 13HSTRAIGHT LINE )
      SLOPE=(FEND-FBEG)*2.
      FBEG2=FBEG*2.
      DO 215 I=1,4
         DIFF=ABS(F(BEG+RN(I)*STEP)-FBEG2-RN(I)*SLOPE)
         KOUNT = KOUNT + 1
         IF (DIFF.GT.TABTLM) GOTO 240
  215 CONTINUE
      IF (LEVEL.GE.3) PRINT 220,BEG,END
  220 FORMAT(27X, 43HINTEGRAND SEEMS TO BE STRAIGHT LINE BETWEEN ,
     ,      E15.8, 4H AND, E15.8)
      GOTO 250
C
C     NOISE MAY BE DOMINANT FEATURE.
C     ESTIMATE NOISE LEVEL BY COMPARING THE VALUE OF THE INTEGRAND AT
C     FOUR RANDOMLY CHOSEN POINTS WITH THE VALUE OF THE STRAIGHT LINE
C     INTERPOLATING THE INTEGRAND AT THE ENDPOINTS.  IF SMALL ENOUGH,
C     ACCEPT VINT.
C
  225 IF (LEVEL.GE.4) PRINT 230,L,T(1,LM1)
  230 FORMAT(I5, E16.8, 5X, 5HNOISE)
      SLOPE=(FEND-FBEG)*2.
      FBEG2=FBEG*2.
      I=1
  235 DIFF=ABS(F(BEG+RN(I)*STEP)-FBEG2-RN(I)*SLOPE)
      KOUNT = KOUNT + 1
  240 ERRER=AMAX1(ERRER,ASTEP*DIFF)
      IF(ERRER.GT.ERGOAL) GOTO 285
      I=I+1
      IF (I.LE.4) GOTO 235
      IF (LEVEL.GE.3) PRINT 245,BEG,END
  245 FORMAT(15H NOISE BETWEEN , E15.8, 4H AND, E15.8)
      IFLAG=-3
C
C     INTEGRATION OVER CURRENT SUBINTERVAL SUCCESSFUL.
C       ADD VINT TO CADRE AND ERRER TO ERR, THEN SET UP NEXT
C     SUBINTERVAL, IF ANY.
C
  250 CADRE=CADRE+VINT
      ERR = ERR + ERRER
      INTS = INTS + 1
      KOUNTU = KOUNTU + KOUNT1
      IF (LEVEL.LT.3) GOTO 270
         IF (LEVEL.LT.5) GOTO 260
            DO 255 I=1,L
  255       PRINT 295,I,(T(I,J),J=1,L)
  260    PRINT 265,VINT,ERRER,L,IT
*IF -DEF,AELIB5
  265    FORMAT(12H INTEGRAL IS, E16.8, 7H, ERROR, E15.8, 9H FROM T(,
     ,            I1, 1H,,I1, 1H))
*ENDIF
*IF DEF,AELIB5
  265    FORMAT(12H INTEGRAL IS, E16.8, 7H, ERROR, E15.8, 9H FROM T(,
     ,            ,I1, 1H,,I1, 1H))
*ENDIF
C
  270 IF(RIGHT) GOTO 275
         ISTAGE=ISTAGE-1
         IF(ISTAGE.EQ.0) GOTO 360
C
         REGLAR=REGLSV(ISTAGE)
         BEG=BEGIN(ISTAGE)
         END=FINIS(ISTAGE)
         CUREST=CUREST-EST(ISTAGE+1) +VINT
         IEND=IBEG-1
         FEND=TS(IEND)
         IBEG=IBEGS(ISTAGE)
         GOTO 305
  275 CUREST=CUREST+VINT
      STAGE=STAGE*2.
      IEND=IBEG
      IBEG=IBEGS(ISTAGE)
      END=BEG
      BEG=BEGIN(ISTAGE)
      FEND=FBEG
      FBEG=TS(IBEG)
      GOTO 5
C
C     INTEGRATION OVER CURRENT SUBINTERVAL IS UNSUCCESSFUL.
C     MARK SUBINTERVAL FOR FURTHER SUBDIVISION.  SET UP NEXT INTERVAL.
C
  280 REGLAR=.TRUE.
  285 IF (ISTAGE.EQ.MXSTGE) GOTO 335
      IF (LEVEL.LT.5) GOTO 300
         DO 290 I=1,L
  290    PRINT 295,I,(T(I,J),J=1,L)
  295    FORMAT(I5, 7E16.8/3E16.8)
  300 IF (RIGHT) GOTO 310
         REGLSV(ISTAGE+1) =REGLAR
         BEGIN(ISTAGE)=BEG
         IBEGS(ISTAGE)=IBEG
         STAGE=STAGE/2.
  305    RIGHT=.TRUE.
         BEG=(BEG+END)/2.
         IBEG=(IBEG+IEND)/2.
         TS(IBEG)=TS(IBEG)/2.
         FBEG=TS(IBEG)
         GOTO 10
  310 NNLEFT=IBEG-IBEGS(ISTAGE)
      IF (IEND+NNLEFT.GE.MAXTS) GOTO 325
      III=IBEGS(ISTAGE)
      II=IEND
      DO 315 I=III,IBEG
         II=II+1
  315 TS(II)=TS(I)
      DO 320 I=IBEG,II
         TS(III)=TS(I)
  320 III=III+1
      IEND=IEND+1
      IBEG=IEND-NNLEFT
      FEND=FBEG
      FBEG=TS(IBEG)
      FINIS(ISTAGE)=END
      END=BEG
      BEG=BEGIN(ISTAGE)
      BEGIN(ISTAGE)=END
      REGLSV(ISTAGE)=REGLAR
      ISTAGE=ISTAGE+1
      REGLAR=REGLSV(ISTAGE)
      EST(ISTAGE)=VINT
      CUREST=CUREST+EST(ISTAGE)
      GOTO 5
C
C     FAILURE TO HANDLE GIVEN INTEGRATION PROBLEM.
C
  325 IF(LEVEL.LT.2) GOTO 330
         ENCODE(15,350,IERWD(3))BEG
*IF -DEF,AELIB5
         IERWD(4)=IERWD(4).A.MASK(30)
*ENDIF
*IF DEF,AELIB5
         IERWD(4)=IERWD(4).AND.MASK(30)
*ENDIF
         ENCODE(15,350,IERWD(5))END
*IF -DEF,AELIB5
         IERWD(6)=IERWD(6).A.MASK(30)
         CALL AELERR(MES(1),2HNF)
*ENDIF
*IF DEF,AELIB5
         IERWD(6)=IERWD(6).AND.MASK(30)
         CALL AELERR(MES(1),'NF')
*ENDIF
  330 IFLAG=-4
      GOTO 355
  335 IFLAG=-5
      IF (LEVEL.LT.2) GOTO 355
         IF (LEVEL.LT.5) GOTO 345
            DO 340 I=1,L
  340       PRINT 295,I,(T(I,J),J=1,L)
 345     ENCODE(15,350,IERWD(3))BEG
*IF -DEF,AELIB5
         IERWD(4) = IERWD(4).A.MASK(30)
*ENDIF
*IF DEF,AELIB5
         IERWD(4) = IERWD(4).AND.MASK(30)
*ENDIF
         ENCODE(15,350,IERWD(5))END
*IF -DEF,AELIB5
         IERWD(6) = IERWD(6).A.MASK(30)
*ENDIF
*IF DEF,AELIB5
         IERWD(6) = IERWD(6).AND.MASK(30)
*ENDIF
*IF -DEF,AELIB5
         CALL AELERR(MES(2),2HNF)
*ENDIF
*IF DEF,AELIB5
         CALL AELERR(MES(2),'NF')
*ENDIF
  350    FORMAT(E15.8)
  355 CADRE=CUREST+VINT
  360 RETURN
      END
