*DECK CBESY0
      COMPLEX FUNCTION CBESY0(Z)
C
C   THE FUNCTION EVALUATES THE BESSEL FUNCTIONS OF THE SECOND KIND
C    AND ORDER ZERO,Y0(Z), WHERE THE ARGUMENT
C    Z IS COMPLEX.DIFFERENT FORMULAE ARE USED TO EVALUATE THE BESSEL
C    FUNCTIONS DEPENDING ON THE VALUE OF Z..LARGE Z IS ANY Z WHERE
C    CABS(Z).GE.12.
C    Y(K),V(K),W(K) ARE STATEMENT FUNCTIONS.
C    XR IS THE NUMERATOR OF THE NEW TERM TO BE CALCULATED IN EACH
C    RUN OF THE LOOP.THIS IS ONLY DONE FOR SMALL Z.
C    AKNEW IS THE NEW TERM CALCULATED WHEN EVALUATING J0.
C    IN OTHER WORDS AKNEW=A(K+1), WHERE A(K+1)=Y(K)*A(K).
C    YR=THE ADDITION TO THE NUMERATOR IN EACH RUN OF THE LOOP FOR
C    LARGE Z.
C    TERM IS JUST THE NEW TERM TO BE ADDED TO THE SERIES,ST.
C    SST IS THE VALUE GIVEN TO THE TOTAL SUM OF THE TERMS IN
C    THE SERIES.(ST-SST) TESTS THE NEW SUM OF THE SERIES AGAINST THE
C    PREVIOUS SUM. IF THE DIFFERENCE IS INSIGNIFICANT (I.E. LESS THAN
C    1.E-11),THE CALCULATION OF THE SERIES TERMINATES.
C    NON-FATAL ERROR MESSAGES ARE GIVEN FOR ILLEGAL ARGUMENTS FOR THE
C    BESSEL FUNCTION. THESE ARGUMENTS ARE INFINITES,INDEFINITES, AND
C    ZERO.
C    PROGRAMMED BY BARBARA PURCELL          SEPTEMBER,1975
C
      COMPLEX Z,Y,V,TERM,ST,SST,P0,Q0,Y0,AKNEW,J0, ADD,W,I2
      COMPLEX CINDEF
      INTEGER INDEF(2)
      EQUIVALENCE(CINDEF,INDEF(1))
*IF -DEF,AELIB5
      DATA INDEF/2*17770000000000000000B/
*ENDIF
*IF DEF,AELIB5
      DATA INDEF/2*O"17770000000000000000"/
*ENDIF
      DATA GAMMA/.57721566490153/
      DATA PIU4/.785398163397448/
      DATA X2UPI/.6366197723676/
*IF -DEF,AELIB5
      Y(K)=-((0.5*Z)/K)**2
      V(K)=-1./((2*K-1)*2*K*(8*Z)**2)
      W(K)=-1./((2*K-1)*(2*K-2)*(8*Z)**2)
*ENDIF
*IF DEF,AELIB5
      Y(K)=-((0.5*Z)/K)**2
      V(K)=-1./((2*K-1)*2*K*(8*Z)**2)
      W(K)=-1./((2*K-1)*(2*K-2)*(8*Z)**2)
*ENDIF
*CALL STATF
      MESNO1=200
      I2=(0.,2.)
      IQUAD=1.
C
C    TEST FOR VALID ARGUMENT.
C
      IF(LEGVAR(REAL(Z)).NE.0.OR.LEGVAR(AIMAG(Z)).NE.0) GOTO 55
C
C    TEST FOR ARGUMENT EQUAL TO ZERO.
C
      IF(Z.EQ.(0.,0.)) GOTO 55
C
C    TEST FOR Z IN SECOND OR THIRD QUADRANTS.
C
      IF(REAL(Z).GE.0.0) GOTO 5
         Z=-Z
         IQUAD=-1.
         X=REAL(Z)
         C=AIMAG(Z)
    5 IF(CABS(Z).GE.12.) GOTO 20
C
C    THIS PART OF THE PROGRAM IS FOR CABS(Z).LT.12.
C
      AKNEW=1.
      J0=AKNEW
      TERM=-1.
      ST=0
      XR=0.
      DO 10 I=1,1000
         K=I
         AKNEW=Y(K)*AKNEW
         J0=J0+AKNEW
         XR=1./K+XR
         TERM=Y(K)*TERM
         ADD=TERM*XR
         ST=ST+ADD
         IF(CABS(  ADD ).LT.1.E-11) GOTO 15
   10 CONTINUE
   15 CBESY0=X2UPI*(J0*(GAMMA+CLOG(0.5*Z))+ST)
      GOTO 45
C
C    THIS PART OF THE PROGRAM IS FOR CABS(Z).GE.12.
C
   20 TERM=1.
      ST=0.
      DO 25 I=1,100
         K=I
         YR=((4*K-3)*(4*K-1))**2
         TERM=TERM*(YR*V(K))
         ST=ST+TERM
         P0=1.+ST
         IF(CABS( TERM ).LT.1.E-11) GOTO 30
   25 CONTINUE
   30 TERM=-1./(8.*Z)
      ST=TERM
      DO 35 I=2,100
         K=I
         YR=((4*K-5)*(4*K-3))**2
         TERM=TERM*(YR*W(K))
         ST=ST+TERM
         Q0=ST
         IF(CABS( TERM ).LT.1.E-11) GOTO 40
   35 CONTINUE
   40 CBESY0=(CSQRT(X2UPI/Z))*(P0*CSIN(Z-PIU4)+Q0*CCOS(Z-PIU4))
   45 IF(IQUAD.EQ.1.) GOTO 50
         CALL JBESCPX(0.,X,C,R,S)
         J0=CMPLX(R,S)
         Z=-Z
         CBESY0=CBESY0-I2*SIGN(1.,C)*J0
   50 RETURN
C
C    ERROR SECTION FOR INDEFINITES,INFINITES,OR ZERO.
C
*IF -DEF,AELIB5
   55 CALL AELERR(MESNO1,2HNF)
*ENDIF
*IF DEF,AELIB5
   55 CALL AELERR(MESNO1,'NF')
*ENDIF
      CBESY0=CINDEF
      RETURN
      END
