*DECK CHNGEC
       LOGICAL FUNCTION CHNGEC(NECS,IAVL,ISP)
C
C **********************************************************************
C ******                                                          ******
C ******             DYNAMIC ALLOCATION OF ECS                    ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * SPECIAL NOTE                                                       *
C *                                                                    *
C *           CHNGEC IS WRITTEN IN MODULAR FORM USING THE MODULES      *
C *           GIVEN BELOW. SOURCE CODE IS INCLUDED IN THE DECK CHNGEC  *
C *           UNLESS OTHERWISE INDICATED.                              *
C *                                                                    *
C *                     _ ICMMGEC                                      *
C *                     _ CHNGECS                                      *
C *                     _ MEMECS                                       *
C *                                                                    *
C * PURPOSE  TO ALLOCATE ECS IN VARIOUS AMOUNTS AS REQUIRED BY THE USER*
C *                                                                    *
C *                                                                    *
C * ENTRY     L = CHNGEC(NECS,IAVL,ISP)                                *
C *                                                                    *
C *           NECS - AMOUNT OF ECS REQUIRED                            *
C *                  IF NECS LT 1000B AND GT 0 1000B ECS ALLOCATED     *
C *                  INTEGER INPUT VARIABLE                            *
C *           NECS - RETURNS THE AMOUNT OF ECS ALLOCATED               *
C *                  INTEGER OUTPUT VARIABLE                           *
C *           IAVL - AMOUNT OF AVAILABLE ECS DEFINED ON INITIAL CALL   *
C *                  INTEGER OUTPUT VARIABLE                           *
C *           ISP  - THE REMAINING SPACE AVAILABLE IN ECS              *
C *                  INTEGER OUTPUT VARIABLE                           *
C *           L    - LOGICAL VARIABLE ASSIGNED BY USER TO DETERMINE    *
C *                  SUCCESS OR FAILURE OF THE OPERATION               *
C *                                                                    *
C * ADDITIONAL VARIABLES                                               *
C *                                                                    *
C *           IAV1  - ZERO ON FIRST CALL THEREAFTER SET TO 1.          *
C *           IROUT - INTEGER VARIABLE DEFINING THE ROUTINE NUMBER FOR *
C *                   CHNGEC TO BE USED IN ERROR MESSAGE CONTROL.      *
C *                                                                    *
C * EXTERNAL REFERENCE                                                 *
C *                                                                    *
C *           ICMMGEC  - RETURNS THE AMOUNT OF ECS AVAILABLE FROM      *
C *                      THE JOB CARD. I.E. MAXIMUM ECS FOR THIS JOB   *
C *           MEMECS   - RETURNS THE AMOUNT OF ECS PRESENTLY AVAILABLE *
C *                      FROM RFL(EC=  ) OR THE LAST CALL TO CHNGECS   *
C *           CHNGECS  - GETS NECS WORDS OF ECS TO THE NEAREST 1000B   *
C *                                                                    *
C * ABNORMAL TERMINATION                                               *
C *           IF THE VALUE OF NECS IS NEGATIVE                         *
C *           RESULTS IN CHNGEC RETURNING AN APPROPRIATE ERROR MESSAGE *
C *                            AND IF                                  *
C *           THE NECS IS GREATER THAN THE AMOUNT OF ECS AVAILABLE     *
C *           A LOGICAL VALUE OF .FALSE. FOR L AND THE ALLOCATED ECS   *
C *           IN NECS AND THE REMAINING ECS AVIALABLE IN ISP           *
C * AUTHOR  D. G. STEWART   WRITTEN DECEMBER 1979                      *
C *                                                                    *
C **********************************************************************
C
C
       LOGICAL ALERPR
C
      DATA IAV1,IROUT/0,20536/
C
C      LIBRARY STATS CALL FROM FTN.
C
*CALL STATF
C
       CHNGEC=.FALSE.
      IF(IAV1.NE.0) GO TO 10
C
C     IAVL = AMOUNT OF AVAILABLE ECS ON THE JOB CARD
C           DEFINED ON FIRST CALL ONLY.
      IAVL=ICMMGEC(DUM)
      IAV1=1
C
 10   CONTINUE
C
C     INPUT PARAMETER NECS IS NEGATIVE
C
      IF(NECS.LT.0) GO TO 25
C
C     IF NOT ENOUGH ECS AVAILABLE RETURN FALSE
C
*IF -DEF,AELIB5
      IF(NECS.GT.IAVL) 20,30
*ENDIF
*IF DEF,AELIB5
      IF(NECS.GT.IAVL) GOTO 20
      GOTO 30
*ENDIF
C
C    INPUT PARAMETER ERROR MESSAGE.
C
 25    IF(.NOT.ALERPR(IROUT,0)) GO TO 20
       PRINT 100,NECS
 100  FORMAT(54H ***INPUT PARAMETER VIOLATION. NECS NEGATIVE.  NECS = ,
     1  O20,1HB)
       CALL TRACEB(0)
C
      GO TO 20
C
C
 30    CONTINUE
       CHNGEC=.TRUE.
C
C     CHNGECS GETS NECS WORDS OF ECS TO THE NEAREST 1000B.
C
       CALL CHNGECS(NECS)
C
 20    CONTINUE
C
C      NECS FROM MEMECS GETS THE ACTUAL AMOUNT OF AVAILABLE ECS.
C
       NECS=MEMECS(DUM)
       ISP=IAVL-NECS
       RETURN
       END
          IDENT     ICMMGEC
*
*   THIS ROUTINE CALLS MEMORY AND RETURNS
*   THE AMOUNT OF ECS DECLARED ON THE JOB CARD.
*
          SST
          ENTRY ICMMGEC
 ICMMGEC  DATA 0
          MX6  29
          SA6  ECSCMWD
          MEMORY ECS,ECSCMWD,RCL
          SA1 ECSCMWD
          AX1 30
          BX6 X1
          EQ ICMMGEC
 ECSCMWD  DATA 0
          END
          IDENT  MEMECS
          ENTRY  MEMECS
*
*   THIS ROUTINE CALLS MEMORY AND RETURNS
*   THE AMOUNT OF ECS CURRENTLY AVAILABLE.
*
STA0      BSS 1
STATUS    BSS 1
MEMECS    PS
          SX6 A0
          SA6 STA0
          SX7 0
          SA7 STATUS
          MEMORY ECS,STATUS,RECALL
          SA1 STATUS
          MX0 30
          BX6 X0*X1
          LX6 30
          SA1 STA0
          SA0 X1
          EQ     MEMECS
          END
          IDENT CHNGECS
          ENTRY CHNGECS
          EXT    AELERR      AELIB ERROR ROUTINE
*
*  COMMON BLOCK FOR ERROR MESSAGE INSERTIONS FROM COMPASS
*
          USE    /AELERCM/
IERWD     BSS    15
          USE    *
**
*         CALL CHNGECS (ECS)
*
*  THIS ROUTINE IS DESIGNED FOR USE BY PROGRAMS WRITTEN IN FTN.
*   IT CHANGES THE ECS OF THE JOB
*
*  ENTRY- ECS = REQUESTED ECS.
*
*  EXIT-   THE ECS FIELD LENGTH HAS BEEN SET TO THE REQUESTED AMOUNT.
*
*  ERROR EXIT-    IF ECS IS REAL, NEG., TOO LARGE (>377777B)
          SST
          LIST   F
*
*  CALLS-  AELERR, LIBSTAT
*          AELERR, LIBSTAT
*
*
*
*
*    INTERACTION WITH FORSYS= HAS BEEN TEMP DELETED.
          USE     *
TRACE     VFD    42/0LCHNGECS,18/CHNGECS
          BSS    1
*
+         SA5      TRACE+1
          SA0      X5
CHNGECS   PS
          SX6    A0
          SA6    TRACE+1     SAVE A0
*
          SA1 X1        X1=ECS
*                            CHECK ECS PARAM., ERROR IF:
*                         1. REAL, NEG OR TOO LARGE (GT 377777B)
          MX6    41
          BX5    X1*X6       MASK OFF UPPER 43 BITS
          NZ     X5,PERR     IF NOT ZERO...ERROR
*                            (NEG, REAL OR TOO LARGE)
*
*  CALL MEM TO RESET ECS LENGTH
          BX6    X1
          LX6    30
          SA6    MEMSTAT
          MEMORY ECS,MEMSTAT,RECALL
          EQ     CHNGECS-1
PERR      MX3    -3
          BX3    -X3         FORM A 3 BIT MASK IN LOW ORDER BITS
          SX4    33B         BASE FOR DISPLAY CODE CONVERSION
          SB1    1
          SB2    2           COMMON BLCK BIAS
          SB3    2           OUTER LOOP (STRT1) COUNTER
STRT1     SB4    10          INNER LOOP (STRT2) COUNTER
          MX6    0           CLEAR X6
STRT2     LX1    3           SHIFT LEFT 1 OCTAL CHAR.
          BX5    X1*X3       MASK 3 BITS FROM FL
          IX5    X5+X4       CONVERT TO DISPLAY CODE
          LX6    6           SHIFT LEFT 1 DISPLAY CODE CHAR.
          BX6    X6+X5       .OR. THIS CHAR. INTO X6
          SB4    B4-B1       DECREMENT LOOP CNTR.
END2      NZ     B4,STRT2
          SA6    IERWD+B2    X6 FILLED, TRANSFER TO COMMON BLCK
          SB2    B2+B1       INCREMENT COMMON BLCK BIAS
          SB3    B3-B1       DECREMENT LOOP CNTR.
END1      NZ     B3,STRT1
          MX6    0           THE ERROR RTN. REQ. THAT THIS STRING BE
          SA6    IERWD+B2    TERMINATED BY A ZERO BYTE
*
          SA1    LIST
+         RJ     AELERR
-         VFD    12/0,18/TRACE
*
*                            AELERR PARAMETERS
ERR       DATA   3930                  ERROR NUMBER
FLAG      VFD    60/10HF               FATAL ERROR FLAG
*                            AELERR PARAMETERS LIST
LIST      VFD    42/0,18/ERR
          VFD    42/0,18/FLAG
          VFD    60/0
*
MEMSTAT   DATA   0           FOR MEMORY REQUEST
ENDLBL    END
