*DECK CHNGEX
      LOGICAL FUNCTION CHNGEX(Z,INC,IFW,ISP,IDO)
C
C **********************************************************************
C ******                                                          ******
C ******              DYNAMIC ALLOCATION OF CORE                  ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C *                                                                    *
C * PURPOSE : TO ALLOCATE, RELEASE, SHRINK AND/OR TERMINATE THE        *
C *           VARIABLE STATUS OF AN EXPANDABLE BLOCK (BLOCK WHICH MUST *
C *           EXIST ABOVE HHA) VIA COMMON MEMORY MANAGEMENT (CMM)      *
C *           ROUTINES.                                                *
C *                                                                    *
C * ENTRY :   L = CHNGEX(Z,INC,IFW,ISP,IDO)                            *
C *                                                                    *
C *           Z   - A WORD IN BLANK COMMON USED AS THE BASE POSITION   *
C *                 FOR CALCULATING IFW.                               *
C *               - REAL, INPUT, BLANK-COMMON ARRAY ELEMENT.           *
C *           INC - THE AMOUNT OF NEW WORKING STORAGE REQUIRED.        *
C *               - INTEGER, INPUT VARIABLE.                           *
C *           IFW - THE FIRST WORD OF THE NEW WORKING STORAGE.         *
C *               - TO BEGIN DYNAMIC ALLOCATION OF CORE WITH CHNGEX    *
C *                 THE USER SETS IFW=0, IDO=0, AND INC>0.  IFW IS     *
C *                 THEN SET BY CHNGEX FOR FUTURE USE.  THE VALUE OF   *
C *                 IFW IS RELATIVE TO Z HENCE THE USER MUST USE FTN   *
C *                 ROUTINE LOCF TO OBTAIN THE ADDRESS OF Z(IFW).      *
C *               - INTEGER VARIABLE.                                  *
C *           ISP - THE REMAINING SPACE AVAILABLE FOR WORKING STORAGE. *
C *               - INTEGER, OUTPUT VARIABLE.                          *
C *           IDO - PARAMETER INDICATING THE OPERATIONS CHNGEX IS TO   *
C *                 PERFORM.                                           *
C *                   IDO < 0  FREE EXPANDABLE BLOCK Z(IFW).           *
C *                   IDO = 0  OBTAIN NEW IFW FOR EXPANDABLE BLOCK     *
C *                            Z(IFW).                                 *
C *                   IDO > 0 - INC > 0 EXPAND BLOCK Z(IFW)            *
C *                           - INC = 0 TERMINATE THE VARIABLE STATUS  *
C *                                     OF BLOCK Z(IFW)                *
C *                           - INC < 0 SHRINK BLOCK Z(IFW) BY THE AB- *
C *                                     SOLUTE VALUE OF INC            *
C *                           - N.B. : FOR INC>0, OR INC<0 IF IDO>1 THE*
C *                                    VARIABLE STATUS OF BLOCK Z(IFW) *
C *                                    IS TERMINATED.                  *
C *               - INTEGER, INPUT VARIABLE.                           *
C *           L   - LOGICAL VARIABLE REPORTING TO THE USER THE SUCCESS *
C *                 OR FAILURE OF CHNGEX.                              *
C *                                                                    *
C * ADDITIONAL VARIABLE :                                              *
C *                                                                    *
C *           IROUT - INTEGER VARIABLE DEFINING THE ROUTINE NUMBER FOR *
C *                   CHNGEX TO BE USED IN ERROR MESSAGE CONTROL.      *
C *                                                                    *
C * EXTERNAL REFERENCES : CMM ROUTINES                                 *
C *                                                                    *
C *           CMMFRF  : FREES A BLOCK RELEASING IT TO CMM.             *
C *           CMMGFS  : RETURNS THE LENGTH OF THE LARGEST EXPANDABLE   *
C *                     BLOCK THAT FITS THE GIVEN CONDITIONS.  IN THIS *
C *                     ROUTINE THE CONDITIONS ARE AS FOLLOWS :        *
C *                          1. THE FIELD LENGTH CAN INCREASE TO THE   *
C *                             MAXFL.                                 *
C *                          2. THE BLOCK MUST BE ABOVE HHA.           *
C *           CMMALF  : ALLOCATES AN EXPANDABLE BLOCK OF LENGTH        *
C *                     INC (CONTENTS UNDEFINED) AND RETURNS THE FIRST *
C *                     WORD ADDRESS OF THE ASSIGNED BLOCK.            *
C *                     NOTE : ASSIGNING A BLOCK EXPANDABLE STATUS     *
C *                            IS ACCOMPLISHED BY ASSIGNING THE        *
C *                            PARAMETER ICODE, BLOCK-SIZE-CODE, TO 1  *
C *                            INDICATING THAT THE LAST WORD ADDRESS   *
C *                            (LWA) END CAN GROW. THE LAST PARAMETER  *
C *                            IN THE CALLING LIST, THE GROUP-ID, IS   *
C *                            SET TO 0 INDICATING THAT THE ACQUIRED   *
C *                            BLOCK DOES NOT BELONG TO A GROUP.       *
C *           CMMCSF  : CHANGES THE SPECIFICATIONS OF THE EXPANDABLE   *
C *                     BLOCK Z(IFW).  AFTER CMMALF HAS GIVEN A        *
C *                     BLOCK EXPANDABLE STATUS, BEFORE THE OPERATIONS *
C *                     TO SHRINK OR TERMINATE THE VARIABLE STATUS     *
C *                     OF AN EXPANDABLE BLOCK CAN BE PERFORMED,       *
C *                     THE EXPANDABLE BLOCK"S SPECIFICATIONS MUST     *
C *                     BE ALTERED.   THE SECOND PARAMETER, ICODE,     *
C *                     IN THE CALLING LIST FOR CMMCSF CHANGES THE     *
C *                     SPECIFICATIONS OF BLOCK Z(IFW). IF ICODE = 2   *
C *                     LWA CAN SHRINK. IF ICODE = 0 THE EXPANDABLE    *
C *                     STATUS OF THE BLOCK TERMINATES.                *
C *           CMMSLF  : SHRINKS THE EXPANDABLE BLOCK BY INC.  IFW > 0. *
C *                     SINCE INC MUST BE  \ 0 CHNGEX INSURES THIS BY  *
C *                     ASSIGNING INC THE ABSOLUTE VALUE OF INC. SAID  *
C *                     ASSIGNMENT ALLOWS THE USER TO ENTER A NEGATIVE *
C *                     INC WHICH INDICATES TO THE USER A DECREASE IN  *
C *                     SIZE OF THE EXPANDABLE BLOCK WITHOUT INCURING  *
C *                     A CMM ERROR.                                   *
C *           CMMGLF  : INCREASE THE LENGTH OF THE EXPANDABLE BLOCK    *
C *                     Z(IFW) BY INC.                                 *
C *                                                                    *
C * ABNORMAL TERMINATION :                                             *
C *                                                                    *
C *           1. AN ATTEMPT TO SHRINK AN EXPANDABLE BLOCK BELOW IT"S   "
C *              FWA (FIRST WORD ADDRESS) RESULTS IN A FTN - FATAL     *
C *              ERROR 52 AND A CMM ERROR MESSAGE. CHNGEX ABORTS.      *
C *           2. AN ATTEMPT TO FREE A BLOCK Z(IFW) WITH IFW < 0, IDO < *
C *              0 OR IDO > 0 RESULTS IN CHNGEX RETURNING AN APPROPRI- *
C *              ATE ERROR MESSAGE, A LOGICAL VALUE OF .FALSE. FOR L   *
C *              AND THE CURRENT VALUE OF ISP.                         *
C *           3. AN ATTEMPT TO OBTAIN A NEW IFW FOR EXPANDABLE BLOCK   *
C *              Z(IFW) WITH INC @ 0 RESULTS IN CHNGEX RETURNING AN    *
C *              APPROPRIATE ERROR MESSAGE, A LOGICAL VALUE OF .FALSE. *
C *              FOR L AND THE CURRENT VALUE OF ISP.                   *
C *                                                                    *
C * REFERENCES :                                                       *
C *                                                                    *
C *           DYNAMIC ALLOCATION OF CORE ON THE CRNL NOS/BE 1.3 SYSTEM *
C *           M.B. CARVER, C.J. TANNER, G.L. KLAWITTER, D.G. STEWART   *
C *                                                                    *
C * REVISION HISTORY :                                                 *
C *                                                                    *
C *           REVISED NOV 1981 BY G.L. KLAWITTER AND C.J. TANNER       *
C *           REVISION ALLOWS USE OF CDC CMM INTERFACE ROUTINES        *
C *                                                                    *
C * AUTHORS :                                                          *
C *           M.B. CARVER                                              *
C *           G.L. KLAWITTER                                           *
C *           WRITTEN DECEMBER 4, 1979.                                *
C *                                                                    *
C *                                                                    *
C **********************************************************************
C
C
      REAL Z(1)
      INTEGER INC,IFW,ISP,IDO
      LOGICAL ALERPR
      LOGICAL CMMFAKE
      DATA CMMFAKE/.FALSE./
C
C       EXTERNAL REFERENCE TO *CMMSLIB* INCLUDED TO FORCE
C       LDSET LIB=SYMLIB TO OBTAIN CMM INTERFACE ROUTINES
C
C
      IF(CMMFAKE)CALL CMMSLIB
C
C
C
C     LIBRARY STATS CALL FROM FTN.
C
*CALL STATF
C
C     IROUT DEFINED. IROUT IS THE ROUTINE NUMBER OF CHNGEX IN AECL-6076.
C
      IROUT = 20533
C
C     INITIALIZATION OF CHNGEX AND ISP.
C
      CHNGEX = .FALSE.
      CALL CMMGFS(1,0,ISP)
C
C     CHECK IDO AND TRANSFER TO APPROPRIATE OPERATIONS.
C
      IF(IDO) 100,200,300
C
C        ***************************************************************
C        ******  FREE EXPANDABLE BLOCK Z(IFW) AFTER CHECKING      ******
C        ******  INPUT PARAMETER IFW.  IFW @ 0 IS INVALID.        ******
C        ***************************************************************
C
  100    IF(IFW.LE.0) GO TO 500
         CALL CMMFRF(LOCF(Z(IFW)))
C
C        TRANSFER TO ALLOW REASSIGNMENT OF CHNGEX AND ISP.
C
         GO TO 400
C
C        ***************************************************************
C        ******  CHECK IFW FOR POSSIBLE RELEASE OF BLOCK Z(IFW).  ******
C        ******  OBTAIN NEW IFW FOR EXPANDABLE BLOCK Z(IFW).      ******
C        ***************************************************************
C
C        IFW > 0  FREE EXPANDABLE BLOCK Z(IFW),  OBTAIN NEW ISP
C                 AND THEN TEST TO INSURE SUFFICIENT SPACE TO OBTAIN
C                 EXPANDABLE BLOCK Z(IFW).
C        IFW @ 0  TRANSFER TO TEST TO INSURE SUFFICIENT SPACE TO OBTAIN
C                 EXPANDABLE BLOCK Z(IFW).
C
C        INPUT PARAMETER CHECK. INC @ 0 IS INVALID.
C
  200    IF(INC.LE.0) GO TO 600
         IF(IFW.LE.0) GO TO 210
            CALL CMMFRF(LOCF(Z(IFW)))
            CALL CMMGFS(1,0,ISP)
C
C           TEST TO INSURE SUFFICIENT SPACE TO OBTAIN BLOCK Z(IFW).
C
  210       IF(INC.GT.ISP) RETURN
C
C           OBTAIN NEW IFW FOR EXPANDABLE BLOCK Z(IFW).
C           NOTE : ICMMALF RETURNS THE FIRST WORD ADDRESS OF THE
C                  ASSIGNED EXPANDABLE BLOCK. SINCE IFW IS THE FIRST
C                  WORD OF THE EXPANDABLE BLOCK RELATIVE TO Z, IFW IS
C                  OBTAINED BY SUBTRACTING THE ADDRESS OF Z FROM THE
C                  FIRST WORD ADDRESS OF THE ASSIGNED EXPANDABLE BLOCK
C                  AND ADDING 1.  LOCF IS THE FTN ROUTINE USED TO DETER-
C                  MINE THE ADDRESS OF Z.
C
            ICODE = 1
            CALL CMMALF(INC,ICODE,0,IADDR)
*IF -DEF,AELIB5
            IFW = IADDR - LOCF(Z) + 1
*ENDIF
*IF DEF,AELIB5
            IFW = IADDR - LOCF(Z(1)) + 1
*ENDIF
C
C           TRANSFER TO ALLOW REASSIGNMENT OF CHNGEX AND ISP.
C
            GO TO 400
C
C        ***************************************************************
C        ******  OPERATE ON EXPANDABLE BLOCK Z(IFW) AS DICTATED   ******
C        ******  BY THE VALUE OF INC.                             ******
C        ***************************************************************
C
  300    IF(INC) 310,320,330
C
C           ******  INC < 0.
C           ******  CHECK INPUT PARAMETER IFW. IFW @ 0 IS INVALID.
C           ******  CHANGE SPECIFICATIONS OF EXPANDABLE BLOCK Z(IFW)
C                   TO ALLOW LWA END TO SHRINK.
C           ******  SHRINK BLOCK Z(IFW) BY THE ABSOLUTE VALUE OF INC.
C
  310       IF(IFW.LE.0) GO TO 500
            ICODE = 2
            CALL CMMCSF(LOCF(Z(IFW)),ICODE)
            INC = IABS(INC)
            CALL CMMSLF(LOCF(Z(IFW)),INC)
C
C
C           CHECK IDO. IF IDO = 1 TRANSFER TO ALLOW REASSIGNMENT OF
C           CHNGEX AND ISP. EXPANDABLE BLOCK RETAINS ITS EXPANDABLE
C           STATUS.
C
            IF(IDO.EQ.1) GO TO 400
C
C           ******  INC = 0.
C           ******  CHECK INPUT PARAMETER IFW. IFW @ 0 IS INVALID.
C           ******  CHANGE SPECIFICATIONS OF EXPANDABLE BLOCK Z(IFW)
C                   TO TERMINATE IT"S VARIABLE STATUS.
C           ******  TERMINATE THE VARIABLE STATUS OF BLOCK Z(IFW).
C
  320       IF(IFW.LE.0) GO TO 500
            ICODE = 0
            CALL CMMCSF(LOCF(Z(IFW)),ICODE)
C
C           TRANSFER TO ALLOW REASSIGNMENT OF CHNGEX AND ISP.
C
            GO TO 400
C
C           ******  INC > 0.
C           ******  ASSIGN ISP A NEW VALUE.
C           ******  CHECK INPUT PARAMETER IFW. IFW @ 0 IS INVALID.
C           ******  TEST TO INSURE SUFFICIENT SPACE TO EXPAND BLOCK
C                   Z(IFW) AND IF SO EXPAND IT.
C
  330       CALL CMMGFS(1,0,ISP)
            IF(IFW.LE.0) GO TO 500
            IF(INC.GT.ISP) RETURN
            CALL CMMGLF(LOCF(Z(IFW)),INC)
C
C           CHECK IDO. IF IDO > 1 TRANSFER TO TERMINATE THE EXPANDABLE
C           STATUS OF BLOCK Z(IFW).
C
            IF(IDO.NE.1) GO TO 320
C
C        ***************************************************************
C        ******  THE APPROPRIATE OPERATION HAS BEEN SUCCESSFULLY  ******
C        ******  COMPLETED ON EXPANDABLE BLOCK Z(IFW).            ******
C        ******  RESET ISP AND CHNGEX. RETURN.                    ******
C        ***************************************************************
C
  400    CALL CMMGFS(1,0,ISP)
         CHNGEX = .TRUE.
         RETURN
C
C           INPUT PARAMETER ERROR MESSAGE. IFW @ 0 IS INVALID.
C
  500       IF(.NOT.ALERPR(IROUT,0)) GO TO 520
               PRINT 510,IFW
               CALL TRACEB(0)
  520          RETURN
C
C           INPUT PARAMETER ERROR MESSAGE. INC @ 0 IS INVALID.
C
  600       IF(.NOT.ALERPR(IROUT,0)) GO TO 620
               PRINT 610,INC
               CALL TRACEB(0)
  620          RETURN
C
C
*IF -DEF,AELIB5
  510 FORMAT(" ***INPUT PARAMETER VIOLATION.  IFW @ 0 IS INVALID.   IFW
     ,= ",I10)
*ENDIF
*IF DEF,AELIB5
  510 FORMAT(' ***INPUT PARAMETER VIOLATION.  IFW @ 0 IS INVALID.   IFW
     ,= ',I10)
*ENDIF
*IF -DEF,AELIB5
  610 FORMAT(" ***INPUT PARAMETER VIOLATION.  INC @ 0 IS INVALID.   INC
     ,= ",I10)
*ENDIF
*IF DEF,AELIB5
  610 FORMAT(' ***INPUT PARAMETER VIOLATION.  INC @ 0 IS INVALID.   INC
     ,= ',I10)
*ENDIF
               END
          IDENT  CMMFRE
          B1=1
          TITLE    CMMFRE - FREE A BLOCK
          COMMENT       FTN-CMM - FREE A BLOCK
          SPACE 2
**        CMMFRE
*
*         TITLE
*           FREE A BLOCK
*
*         ENTRY
*           CALL CMMFRE (BASE, IOFFSET)
*
*           BASE    - USERS ARRAY (SAME AS IN CALL TO CMMALC)
*           IOFFSET - ADDRESS OF BLOCK RELATIVE TO BASE.
*
*         EXIT
*           THE BLOCK HAS BEEN RELEASED TO CMM.
*
*         CALLS
*           CMM.FRF (IN CMM)
*
*         METHOD
*           THE ADDRESS OF THE BLOCK IS CALCULATED AND -CMM.FRF- IS
*         CALLED TO FREE THIS BLOCK.
*
TRACE.    VFD    42/0LCMMFRE,18/CMMFRE
          ENTRY  CMMFRE
CMMFRE    DATA   0
          SB1    1
          SA3    A1+B1        ADDRESS OF OFFSET
          SA2    X3           OFFSET
          IX1    X1+X2        ADDRESS OF BLOCK =
          SX1    X1-1        IADDR = IOFFSET + LOCF(BASE) - 1
+         RJ     =XCMM.FRF    CALL CMM
-         VFD    30/TRACE.      FOR FORTRAN TRACE BACK.
          EQ     CMMFRE       RETURN
          END
          IDENT  ICMMGFS
          TITLE    CMMGFS - GET FIXED BLOCK SIZE
          COMMENT       FTN-CMM - GET MAX FIXED BLOCK SIZE
          SPACE  2
**        ICMMGFS
*
*         TITLE
*           GET SIZE OF LARGEST FIXED BLOCK SATISFYING GIVEN CONDITIONS.
*
*         ENTRY
*           ISIZE = ICMMGFS (IFL, IHHA)
*
*           IFL   - FIELD LENGTH FLAG
*                   = 0 ASSUME NO FIELD LENGTH INCREASE
*                   = 1 ASSUME FIELD LENGTH CAN INCREASE TO MAXFL
*           IHHA  - HIGHEST HIGH ADDRESS FLAG
*                   = 0 BLOCK IS ABOVE HHA (PERMANENT)
*                   = 1 BLOCK CAN BE BELOW HHA (TEMPORARY)
*           ISIZE - SIZE IN CM WORDS OF LARGEST FIXED BLOCK
*
*         EXIT
*           FUNCTION RETURNS THE LENGTH OF THE LARGEST FIXED POSITION
*         BLOCK THAT FITS THE GIVEN CONDITIONS.
*
*         CALLS
*           CMM.GFS (IN CMM)
*
*         METHOD
*
*           THE PARAMETERS -IFL- AND -IHHA- ARE LOADED INTO X1 AND X2
*         RESPECTIVELY AND -CMM.GFS- IS CALLED. THIS SUBROUTINE LEAVES
*         THE RESULTS IN X6, SO NO FURTHER ACTION IS NEEDED.
*
TRACE.    VFD    42/0LICMMGFS,18/ICMMGFS
          ENTRY  ICMMGFS
ICMMGFS   DATA   0
          SA2    A1+1       ADDRESS OF IHHA
          SA1    X1         IFL
          SA2    X2         IHHA
+         RJ     =XCMM.GFS      CALL CMM
-         VFD    30/TRACE.      FOR FORTRAN TRACE BACK.
          EQ     ICMMGFS        RETURN
          END
          IDENT  ICMMALF
          TITLE    CMMALF - ALLOCATE FIXED POSITION BLOCK
          COMMENT       FTN-CMM - ALLOCATE FIXED BLOCK
          SPACE  2
**        ICMMALF
*
*         TITLE
*           ALLOCATE A FIXED POSITION BLOCK
*
*         ENTRY
*           IADDR = ICMMALF (ISIZE, ICODE, ID)
*
*           IADDR - ADDRESS OF ASSIGNED BLOCK
*           ISIZE - LENGTH OF BLOCK IN WORDS.
*           ICODE - SIZE CODE
*                   = 0 - NO SIZE VARIATIONS WILL TAKE PLACE
*                   = 1XX - FWA END CAN SHRINK
*                   = X1X - LWA END CAN SHRINK
*                   = XX1 - LWA END CAN GROW
*           ID    - GROUP ID. SET TO ZERO IF BLOCK DOES NOT BELONG
*                   TO A GROUP.
*
*         EXIT
*           ADDRESS OF ASSIGNED BLOCK IS RETURNED IN X6.
*
*         CALLS
*           CMM.ALF (IN CMM)
*
*         METHOD
*           THE PARAMETERS ARE LOADED INTO X2 AND X3, -CMM.ALF- IS
*         CALLED, AND THE BLOCK ADDRESS IS MOVED TO X6 FOR THE
*         FUNCTION RETURN.
*
TRACE.    VFD    42/0LICMMALF,18/ICMMALF
          ENTRY  ICMMALF
ICMMALF   DATA   0
          SA2    X1     (X2) = BLOCK-SIZE
          SA3    A1+1   GET SIZE CODE ADDRESS
          MX6    -6
          SA4    X3     GET SIZE CODE
          BX3    -X6*X4
          LX3    6      (X3) BITS 11-6 = SIZE CODE
          SA4    A3+1   GET GROUP-ID
          SA4    X4
          MX5    -17
          BX5    -X5*X4
          LX5    12
          BX3    X5+X3  (X3) BITS 28-12 = GROUP-ID
          MX4    1R4    CAUSE SEARCH FROM BOTTOM UP FOR SPACE
+         RJ     =XCMM.ALF      CALL *CMM*
-         VFD    30/TRACE.
          BX6    X1     RETURN, (X6) = BLOCK-FWA
          EQ     ICMMALF    RETURN
          END
          IDENT    CMMSLIB
*
*   DUMMY ROUTINE REFERENCED BY CHNGEX AND CHNGFX.
*   CONTAINS LDSET FOR SYSTEM LIBRARY SYMLIB TO
*   GET CMM INTERFACE ROUTINES
*
          ENTRY    CMMSLIB
CMMSLIB   BSS      0
          LDSET    LIB=SYMLIB
          END
