*DECK CHNGFX
      LOGICAL FUNCTION CHNGFX(Z,INC,IFW,ISP,IDO)
C
C **********************************************************************
C ******                                                          ******
C ******              DYNAMIC ALLOCATION OF CORE                  ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C *                                                                    *
C * PURPOSE : TO ALLOCATE AND RELEASE FIXED BLOCKS VIA COMMON MEMORY   *
C *           MANAGEMENT (CMM) ROUTINES.                               *
C *                                                                    *
C * ENTRY :   L = CHNGFX(Z,INC,IFW,ISP,IDO)                            *
C *                                                                    *
C *           Z   - A WORD IN BLANK COMMON USED AS THE BASE POSITION   *
C *                 FOR CALCULATING IFW. USUALLY THE FIRST WORD.       *
C *               - REAL, INPUT, BLANK-COMMON ARRAY ELEMENT.           *
C *           INC - THE AMOUNT OF NEW WORKING STORAGE REQUIRED.        *
C *               - INTEGER, INPUT VARIABLE.                           *
C *           IFW - THE FIRST WORD OF THE NEW WORKING STORAGE.         *
C *               - TO BEGIN DYNAMIC ALLOCATION OF CORE WITH CHNGFX    *
C *                 THE USER SETS IFW=0, IDO\0 AND INC>0.  IFW IS      *
C *                 THEN SET BY CHNGFX FOR RUTURE USE.  THE VALUE OF   *
C *                 IFW IS RELATIVE TO Z HENCE THE USER MUST USE FTN   *
C *                 ROUTINE LOCF TO DETERMINE THE ADDRESS OF Z(IFW).   *
C *               - INTEGER VARIABLE.                                  *
C *           ISP - THE REMAINING SPACE AVAILABLE FOR WORKING STORAGE. *
C *               - INTEGER, OUTPUT VARIABLE.                          *
C *           IDO - PARAMETER INDICATING THE OPERATIONS CHNGFX IS TO   *
C *                 PERFORM.                                           *
C *                   IDO < 0 - IFW " 0  FREE BLOCK Z(IFW)             *
C *                           - IFW = 0  FREE ALL TEMPORARY BLOCKS     *
C *                   IDO \ 0 - IFW " 0  FREE BLOCK Z(IFW)             *
C *                           - IFW = 0  ALLOCATE BLOCK Z(IFW).        *
C *                   N.B. : IF IDO=0, A FIXED BLOCK IS ALLOCATED.     *
C *                          SUCH A BLOCK EXISTS ABOVE HHA AND HAS AN  *
C *                          UNRESTRICTED LIFETIME.                    *
C *                          IF IDO>0, A TEMPORARY BLOCK IS ALLOCATED. *
C *                          SUCH A BLOCK PREFERENTIALLY EXISTS BELOW  *
C *                          HHA (ALTHOUGH IT MAY NOT) AND MUST BE     *
C *                          RELEASED BEFORE THE NEXT OVERLAY CHANGE.  *
C *               - INTEGER, INPUT VARIABLE.                           *
C *           L   - LOGICAL VARIABLE REPORTING TO THE USER THE SUCCESS *
C *                 OR FAILURE OF CHNGFX.                              *
C *                                                                    *
C * ADDITIONAL VARIABLE :                                              *
C *                                                                    *
C *           IROUT - INTEGER VARIABLE DEFINING THE ROUTINE NUMBER FOR *
C *                   CHNGFX TO BE USED IN ERROR MESSAGE CONTROL.      *
C *                                                                    *
C *            IBLOCK - INTEGER VARIABLE DEFINING THE BLOCK ID OF      *
C *                     THE TEMPORARY-FIXED BLOCK OBTAINED BY THE      *
C *                     USER.                                          *
C *                                                                    *
C * EXTERNAL REFERENCES : CMM ROUTINES                                 *
C *                                                                    *
C *           CMMALF  : ALLOCATES A BLOCK AND RETURNS THE FIRST WORD   *
C *                     OF THE BLOCK IN IFW.                           *
C *           CMMFRF  : FREES A BLOCK RELEASING IT TO CMM.             *
C *           CMMFGR  : FREES ALL TEMPORARY BLOCKS RELEASING THEM TO   *
C *                     CMM.                                           *
C *           CMMGFS  : RETURNS THE LENGTH OF THE LARGEST FIXED-       *
C *                     TEMPORARY BLOCK THAT FITS THE GIVEN CONDITIONS.*
C *                     IN THIS ROUTINE THE CONDITIONS ARE AS FOLLOWS: *
C *                          1. THE FIELD LENGTH CAN INCREASE TO THE   *
C *                             MAXFL.                                 *
C *                          2. THE BLOCK CAN BE BELOW HHA.            *
C *                                                                    *
C * ABNORMAL TERMINATION :                                             *
C *           1. AN ATTEMPT TO COMPLETE ANY OF THE OPERATIONS IN       *
C *              CHNGFX WITH INC @ 0 RESULTS IN CHNGFX RETURNING AN    *
C *              APPROPRIATE ERROR MESSAGE , A LOGICAL VALUE OF .FALSE *
C *              FOR L AND THE CURRENT VALUE OF ISP.                   *
C *           2. AN ATTEMPT TO EXECUTE AN OVERLAY AFTER ONE OR SEVERAL *
C *              TEMPORARY BLOCKS HAVE BEEN ACQUIRED WITHOUT  FREEING  *
C *              ALL TEMPORARY BLOCKS BY CALLING CHNGFX WITH IDO<0     *
C *              AND IFW=0 RESULTS IN A FTN - FATAL ERROR 52 AND       *
C *              AN APPROPRIATE CMM ERROR MESSAGE.  CHNGFX ABORTS.     *
C *                                                                    *
C * REFERENCES :                                                       *
C *                                                                    *
C *           DYNAMIC ALLOCATION OF CORE ON THE CRNL NOS/BE 1.3 SYSTEM *
C *           M.B. CARVER, C.J. TANNER, G.L. KLAWITTER, D.G. STEWART   *
C *                                                                    *
C * REVISION HISTORY :                                                 *
C *                                                                    *
C *           REVISED NOV 1981 BY G.L. KLAWITTER AND C.J. TANNER       *
C *           REVISION ALLOWS USE OF CDC CMM INTERFACE ROUTINES        *
C *                                                                    *
C * AUTHORS :                                                          *
C *                                                                    *
C *           M.B. CARVER                                              *
C *           G.L. KLAWITTER                                           *
C *           WRITTEN DECEMBER 4, 1979.                                *
C *                                                                    *
C *                                                                    *
C **********************************************************************
C
C
      REAL Z(1)
      INTEGER INC,IFW,ISP,IDO
      LOGICAL ALERPR
      LOGICAL CMMFAKE
      DATA IBLOCK/0/
      DATA CMMFAKE/.FALSE./
C
C    EXTERNAL REFERENCE TO CMMSLIB FORCES LDSET LIB=SYMLIB TO
C    OBTAIN CMM INTERFACE ROUTINES
C
      IF(CMMFAKE)CALL CMMSLIB
C
C
C
C     LIBRARY STATS CALL FROM FTN.
C
*CALL STATF
C
C     IROUT DEFINED. IROUT IS THE ROUTINE NUMBER OF CHNGFX IN AECL-6076.
C
      IROUT = 20532
C
C     INITIALIZATION OF CHNGFX AND ISP.
C
      CHNGFX = .FALSE.
      CALL CMMGFS(1,1,ISP)
C
C     INPUT PARAMETER CHECK. INC @ 0 IS INVALID.
C
      IF(INC.LE.0) GO TO 300
C
C     CHECK IDO AND TRANSFER TO APPROPRIATE OPERATIONS.
C
      IF(IDO.GE.0) GO TO 100
C
C        ***************************************************************
C        ******  FREE BLOCKS AS DICTATED BY IFW.                  ******
C        ***************************************************************
C
C        IFW = 0  FREE ALL TEMPORARY BLOCKS.
C           IFW > 0  FREE FIXED BLOCK Z(IFW).
C
         IF(IFW.NE.0) CALL CMMFRF(LOCF(Z(IFW)))
         IF(IFW.EQ.0) CALL CMMFGR(IBLOCK)
         IF(IFW.EQ.0) IBLOCK = 0
C
C        TRANSFER TO ALLOW REASSIGNMENT OF CHNGFX AND ISP.
C
         GO TO 200
C
C        ***************************************************************
C        ******  CHECK IFW FOR POSSIBLE RELEASE OF BLOCK Z(IFW).  ******
C        ******  OBTAIN BLOCK LENGTH INC AND SUBSEQUENT NEW IFW.  ******
C        ***************************************************************
C
C           IFW " 0  FREE FIXED BLOCK Z(IFW).  OBTAIN NEW ISP
C                 AND THEN TEST TO INSURE SUFFICIENT SPACE TO ACQUIRE
C                 Z(IFW).
C        IFW = 0  TRANSFER TO TEST TO INSURE SUFFICIENT SPACE TO
C                 OBTAIN BLOCK Z(IFW).
C
  100    IF(IFW.EQ.0) GO TO 110
            CALL CMMFRF(LOCF(Z(IFW)))
            CALL CMMGFS(1,1,ISP)
C
C           TEST TO INSURE SUFFICIENT SPACE TO OBTAIN BLOCK Z(IFW).
C
  110       IF(INC.GT.ISP) RETURN
C
C           SUFFICIENT SPACE AVAILABLE. ASSIGN BLOCK Z(IFW).
C
            IF(IDO.GT.0) GO TO 120
C
C              ASSIGN A PERMANENT BLOCK.
C
               CALL CMMALF(INC,0,0,IFW)
               GO TO 200
C
C              ASSIGN A TEMPORARY BLOCK AFTER OBTAINING
C              A BLOCK ID IF NONE HAS BEEN ASSIGNED
C              BY A PREVIOUS CALL.
C
  120          IF(IBLOCK.EQ.0) CALL CMMAGR(1,IBLOCK)
               CALL CMMALF(INC,0,IBLOCK,IFW)
C
C        ***************************************************************
C        ******  BLOCK Z(IFW) HAS BEEN OBTAINED.                  ******
C        ******  RESET ISP AND CHNGFX.  RETURN.                   ******
C        ***************************************************************
C
*IF -DEF,AELIB5
  200    IFW = IFW - LOCF(Z) + 1
*ENDIF
*IF DEF,AELIB5
  200    IFW = IFW - LOCF(Z(1)) + 1
*ENDIF
         CALL CMMGFS(1,1,ISP)
         CHNGFX = .TRUE.
         RETURN
C
C           INPUT PARAMETER ERROR MESSAGE. INC @ 0 IS INVALID.
C
  300       IF(.NOT.ALERPR(IROUT,0)) GO TO 320
               PRINT 310,INC
               CALL TRACEB(0)
  320          RETURN
C
C
*IF -DEF,AELIB5
  310 FORMAT(" ***INPUT PARAMETER VIOLATION.  INC @ 0 IS INVALID.   INC
     ,= ",I10)
*ENDIF
*IF DEF,AELIB5
  310 FORMAT(' ***INPUT PARAMETER VIOLATION.  INC @ 0 IS INVALID.   INC
     ,= ',I10)
*ENDIF
               END
      SUBROUTINE CMMALC (BASE, IOFFSET, ISIZE, ITEMP)
      COMMON /FTNCMM/ TEMPID
      INTEGER BLOCKID, TEMPID
      DATA TEMPID/0/
C*        CMMALC
C
C         TITLE
C           ALLOCATE A BLOCK
C
C         ENTRY
C
C           CALL CMMALC (BASE, IOFFSET, ISIZE, ITEMP)
C
C           BASE    - ADDRESS OF USERS ARRAY, USUALLY IN BLANK COMMON.
C           IOFFSET - ADDRESS OF ASSIGNED BLOCK, RELATIVE TO START OF
C                     BASE
C           ISIZE   - LENGTH OF BLOCK IN WORDS.
C           ITEMP   - SET LIFETIME OF BLOCK:
C                      = 0 - LIFETIME IS UNRESTRICTED.
C                      = 1 - BLOCK MUST BE RELEASED BEFORE NEXT
C                            OVERLAY CHANGE.
C
C         EXIT
C           IOFFSET CONTAINS THE ADDRESS OF THE ASSIGNED BLOCK RELATIVE
C         TO THE STARTING ADDRESS OF -BASE-.
C
C         CALLS
C           ICMMAGR
C           ICMMALF
C
C         USES
C           FTNCMM  (INTERNAL COMMON BLOCK)
C
C         METHOD
C           IF THE FIELD -ITEMP- IS PRESENT AND NON ZERO, A TEMPORARY
C         BLOCK IS BEING ASSIGNED. IF A GROUP ID FOR THESE BLOCKS HAS
C         NOT BEEN OBTAINED, THEN -ICMMAGR- IS CALLED TO OBTAIN ONE.
C           NOW -ICMMALF- IS CALLED TO DO THE ACTUAL BLOCK ASSIGNMENT.
C         UPON RETURN, THE ADDRESS OF THE ASSIGNED BLOCK RELATIVE TO THE
C         BASE ADDRESS IC CALCULATED AND STORED IN -IOFFSET-.
C
      IF (LOCF(ITEMP).EQ.0 .OR. ITEMP.EQ.0) GO TO 100
C
C   ASSIGN TEMPORARY BLOCK
      IF (TEMPID .EQ. 0) TEMPID = ICMMAGR (1)
      BLOCKID = TEMPID
      GO TO 150
C  BLOCK IS PERMANENT
100   BLOCKID = 0
C  ASSIGN BLOCK
150   IADDR = ICMMALF(ISIZE, 0, BLOCKID)
      IOFFSET = IADDR - LOCF(BASE) + 1
      RETURN
      END
      SUBROUTINE CMMFTB
      COMMON /FTNCMM/ TEMPID
      INTEGER TEMPID
C*        CMMFTB
C
C         TITLE
C           FREE ALL TEMPORARY BLOCKS
C
C         ENTRY
C           CALL CMMFTB
C
C         EXIT
C           ALL TEMPORARY BLOCKS HAVE BEEN RETURNED TO CMM.
C
C         CALLS
C           CMMFGR
C
C         USES
C           FTNCMM      (INTERNAL COMMON BLOCK)
C
C         METHOD
C           -CMMFGR- IS CALLED WITH THE BLOCK ID OF THE TEMPORARY
C         BLOCKS AS PARAMETER. IF NO TEMPORARY BLOCKS HAVE BEEN ISSUED
C         A WARNING MESSAGE IS ISSUED.
*
      IF (TEMPID .NE. 0) GO TO 100
*IF -DEF,AELIB5
      CALL REMARK (" CMMFTB CALLED WHEN NO TEMPORARY BLOCKS EXIST")
*ENDIF
*IF DEF,AELIB5
      CALL REMARK (' CMMFTB CALLED WHEN NO TEMPORARY BLOCKS EXIST')
*ENDIF
      RETURN
C  RETURN ALL TEMPORARY BLOCKS
100   CALL CMMFGR (TEMPID)
      TEMPID = 0
      RETURN
      END
