*DECK CIPLOT
      SUBROUTINE CIPLOT(X,Y,W,YFIT,N,B,KTRM,IH,NT,WS,PROB,
     ,WSPLOT,LOPT,IER)
C
C **********************************************************************
C ******                                                          ******
C ******               CALCULATE CONFIDENCE INTERVAL              ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE:     FOR A REGRESSION EQUATION IN ONE INDEPENDENT          *
C *              VARIABLE FOUND BY RLSTP, PLOT EITHER THE              *
C *              CONFIDENCE LIMITS FOR THE PREDICTED MEAN              *
C *              VALUES OR FOR PREDICTED INDIVIDUAL VALUES.            *
C *                                                                    *
C * NOTE:                                                              *
C *              CIPLOT IS WRITTEN IN MODULAR FORM USING THE           *
C *              ADDITIONAL ROUTINE SUBPLOT.  SUBPLOT ACCOMPANIES      *
C *              CIPLOT ON THE AELIB SOURCE TAPE UNDER THE DECK        *
C *              NAME CIPLOT.                                          *
C *                                                                    *
C * ENTRY:                                                             *
C *    CALL CIPLOT(X,Y,W,YFIT,N,B,KTRM,IH,NT,WS,PROB,WSPLOT,LOPT)      *
C *                                                                    *
C *    X(N)......REAL ARRAY OF DATA POINTS.                            *
C *    Y(N)......REAL ARRAY OF DATA VALUES OF THE DEPENDENT            *
C *              VARIABLE.                                             *
C *    W(N)......REAL INPUT ARRAY OF WEIGHTS FOR THE DEPENDENT         *
C *              VARIABLE.  FOR A WEIGHTED FIT USE                     *
C *              W(I) = 1 / VARIANCE(Y(I)). FOR AN UNWEIGHTED          *
C *              FIT USE W(I) = 1.0.                                   *
C *    YFIT(N)...REAL ARRAY SET UP BY A PRIOR CALL TO PRFIT OR         *
C *              POLREG CONTAINING THE FITTED VALUES OF THE DEPENDENT  *
C *              VARIABLE.                                             *
C *    N.........INTEGER INPUT VARIABLE, NUMBER OF DATA POINTS (N>NT)  *
C *    B(NT).....REAL INPUT ARRAY CONTAINING THE COEFFICIENTS          *
C *              OF THE FITTED POLYNOMIAL DEFINED BY PRFIT OR POLREG.  *
C *    KTRM(1,NCK)                                                     *
C *              INTEGER MATRIX DEFINING THE NON-CONSTANT TERMS, IN    *
C *              ORDER, OF THE POLYNOMIAL FITTED BY PRFIT              *
C *              OR POLREG.  NCK \ NT - 1.                             *
C *    IH(NT)....INTEGER INPUT ARRAY CONTAINING A VECTOR OF PLUS       *
C *              OR MINUS ONES AS DEFINED IN PRFIT OR POLREG.          *
C *              IH(I) = -1 IMPLIES THE I-TH TERM IS NOT IN            *
C *              THE FITTED LINE.  IH(I) = 1 IMPLIES THAT THE          *
C *              I-TH TERM IS IN THE FITTED LINE.                      *
C *    NT........INTEGER INPUT VARIABLE, NUMBER OF TERMS IN THE        *
C *              POLYNOMIAL BEING FITTED AND PLOTTED.                  *
C *    WS(NRW)...WORKING STORAGE ARRAY SET UP BY A PRIOR CALL TO       *
C *              PRFIT OR POLREG CONTAINING ERROR INFORMATION          *
C *              NEEDED IN PSTATF TO CALCULATE THE VARIANCE-           *
C *              COVARIANCE MATRIX.                                    *
C *              NRW \ (NT * (2 * N + NT + 3)) / 2  + 1 + M            *
C *    PROB......REAL INPUT VALUE, 100 * ( 1 - PROB ) % CONFIDENCE     *
C *              LIMITS WILL BE CALCULATED.                            *
C *    WSPLOT(NRW2)                                                    *
C *              REAL WORKING STORAGE ARRAY FOR INTERNAL USE           *
C *              BY CIPLOT.  (NRW2 \ MAX (NT * NT, 18))                *
C *    LOPT......= 1     CALCULATES CONFIDENCE INTERVAL ON MEAN VALUES *
C *              NE 1    CALCULATES CONFIDENCE INTERVAL ON PREDICTED   *
C *              VALUES                                                *
C *    IER.......ERROR FLAG SET IN CIPLOT                              *
C *              =0,  NO ERRORS                                        *
C *              =1,  PROB DOESN'T LIE BETWEEN ZERO AND ONE            *
C *              =2,  (N-NT) IS LESS THAN OR EQUAL TO ONE              *
C *              =3,  INTERNAL ERROR IN IMSL ROUTINE, MDSTI            *
C *                                                                    *
C *                                                                    *
C * ERROR EXITS                                                        *
C *              WHEN ENCOUNTERING AN ERROR IN CIPLOT, IER IS SET      *
C *              TO 1,2, OR 3 AND CONTROL IS RETURNED TO THE CALLING   *
C *              ROUTINE.  THE CONFIDENCE INTERVAL WILL NOT HAVE       *
C *              BEEN CALCULATED, THEREFORE NO PLOT WILL BE PRODUCED.  *
C *                                                                    *
C *                                                                    *
C * AUTHORS :                                                          *
C *                                                                    *
C *           C. A. WILLS                                              *
C *           M. T. BOULANGER                                          *
C *           WRITTEN NOVEMBER 1981.                                   *
C *                                                                    *
C *                                                                    *
C **********************************************************************
      INTEGER KTRM(1,1), IH(1)
      LOGICAL ALERPR
      REAL X(N), Y(N), YF(100), B(1), WS(1), XX(100)
      REAL SDYF(100), WSPLOT(1), W(N), YFIT(N)
      REAL CINT1(100), CINT2(100)
      COMMON /SCALES/ XMIN,XMAX,TICX,FXSCAL,YMIN,YMAX,TICY,FYSCAL
C
C
C     IROUT IS AN INTEGER VARIABLE DEFINING THE ROUTINE NUMBER
C     FOR CIPLOT TO BE USED IN ERROR MESSAGE CONTROL.
C
      IROUT = 11116
C
C     FORMAT STATEMENTS
C
*IF -DEF,AELIB5
 6000 FORMAT(" ***PROB DOESN'T LIE BETWEEN ZERO AND ONE-NO PLOT PRODUCED
     ,")
 6100 FORMAT(" ***DEGREES OF FREEDOM (N-NT) ARE LESS THAN OR EQUAL TO ON
     ,E-NO PLOT PRODUCED")
 6200 FORMAT(" ***INTERNAL ERROR IN IMSL ROUTINE MDSTI-NO PLOT PRODUCED"
     ,)
*ENDIF
*IF DEF,AELIB5
 6000 FORMAT(' ***PROB DOESN''T LIE BETWEEN ZERO AND ONE-NO PLOT PRODUCE
     ,D')
 6100 FORMAT(' ***DEGREES OF FREEDOM (N-NT) ARE LESS THAN OR EQUAL TO ON
     ,E-NO PLOT PRODUCED')
 6200 FORMAT(' ***INTERNAL ERROR IN IMSL ROUTINE MDSTI-NO PLOT PRODUCED'
     ,)
*ENDIF
*CALL STATF
C
C   CALCULATE THE MINIMUM AND MAXIMUM VALUE OF X(THE INDEPENDENT
C   VARIABLE.
C
      IER = 0
      XMIN = X(1)
      XMAX = X(1)
      DO 30 I=2,N
         IF(X(I) .LT. XMIN) XMIN = X(I)
         IF(X(I) .GT. XMAX) XMAX = X(I)
   30 CONTINUE
      XINT = (XMAX - XMIN) / 80.0
      XMAX = XMAX + XINT
      XMIN = XMIN - XINT
      XINT = (XMAX - XMIN) / 99.0
C
C   CHOOSE 100 EQUALLY SPACED POINT AT WHICH TO CALCULATE
C   THE FUNCTION AND THEN, CALCULATE THE FUNCTION.
C
      M = 100
      NT1 = NT - 1
      DO 40 I=1,M
         XX(I) = XMIN + ( I - 1 ) * XINT
         YF(I) = B(1)
         DO 50 J=1,NT1
            IF(IH(J) .EQ. -1) GO TO 50
            YF(I) = YF(I) + ( XX(I) ** KTRM(1,J) ) * B(J + 1)
   50      CONTINUE
   40 CONTINUE
C
C     VERIFY THAT THE PROBABILITY LIES IN THE RANGE(0,1)
C
      Q = PROB
      IF((Q .LE. 0.0) .OR. (Q .GE. 1.0)) GO TO 500
C
C     CHECK IF THE DEGREES OF FREEDOM(N-NT) FOR T DISTRIBUTION
C     ARE GREATER THAN ONE
C
      F = N - NT
      IF(F  .LE. 1) GO TO 600
C
C     CALL IMSL ROUTIONE MDSTI TO FIND APPROPRIATE VALUE FOR
C     STUDENT'S T DISTRIBUTION.
C
      CALL MDSTI(Q,F,T,IER)
      IF(IER .GT. 0) GO TO 700
C
C     CALL TO PSTATF(AELIB) TO PRODUCE THE VARIANCE-COVARIANCE
C     MATRIX.
C
      ICON = 1
      CALL PSTATF(ICON,IH,NT,N,WSPLOT,NT,SDYF,SS,WS)
C
C     SETTING UP BOUNDS TO STORE INFORMATION IN WORKING STORAGE
C     ARRAY(WSPLOT)
C
      NK = ( NT * ( NT + 1)) / 2
      IBB = NK
      ICC = NK + NT
C
C     VARIANCE-COVARIANCE MATRIX TO SYMMETRIC STORAGE MODE
C
      K = 1
      DO 60 J=1,NT
         DO 70 I=1,J
            WSPLOT(K) = WSPLOT((J - 1) * NT + I)
            IF(I .EQ. J) WSPLOT(K) = (WSPLOT(K)) ** 2
            K = K + 1
   70    CONTINUE
   60 CONTINUE
C
C     CALCULATE CONFIDENCE INTERVAL DEPENDING ON VALUE OF LOPT
C
      IF(LOPT .EQ. 1) GO TO 90
C
C     CHECK TO SEE IF FIT IS WEIGHTED OR UNWEIGHTED
C
      SAME = W(1)
      DO 80 K=1,N
        IF(W(K) .NE. SAME) GO TO 200
   80 CONTINUE
C
C     CALCULATE THE CONFIDENCE INTERVAL ON THE MEAN VALUES
C     AND PREDICTED IF WEIGHTS ARE ALL THE SAME(UNWEIGHTED FIT)
C
   90 CONTINUE
C     CALCULATE STANDARD DEVIATION FOR EACH OF THE 100 PREDICTED
C     VALUES
C               S.D.(Y(I)) = X(I) * VAR - COVAR * X(I) (+ SS IF
C                              PREDICTED AND UNWEIGHTED)
C
      NOPT = 0
      DO 100 I=1,M
         WSPLOT(IBB + 1) = 1.0
         DO 110 J=2,NT
            WSPLOT(IBB + J) = XX(I) ** (J-1)
  110    CONTINUE
C
C     MATRIX MULTIPLICATION, FULL BY SYMMETRIC
C
         CALL VMULFS(WSPLOT(IBB+1),WSPLOT,1,NT,1,WSPLOT(ICC+1),1)
         SUM = 0.0
         DO 120 J=1,NT
            SUM = WSPLOT(ICC+J) * WSPLOT(IBB + J) + SUM
  120    CONTINUE
         IF(LOPT .EQ. 1) GO TO 130
         SDYF(I) =SQRT(SUM +SS)
         GO TO 140
  130    SDYF(I) =SQRT(SUM)
  140    CONTINUE
C
C     USE STANDARD DEVIATION TO CALCULATE CONFIDENCE INTERVAL
C
         CINT1(I) = YF(I) + T * SDYF(I)
         CINT2(I) = YF(I) - T * SDYF(I)
  100 CONTINUE
      K = 100
      GO TO 300
C
C     CALCULATE THE CONFIDENCE INTERVAL ON PREDICTED VALUES
C     IF FIT IS WEIGHTED(WEIGHTS ARE DIFFERENT)
C
  200 NOPT = 1
      DO 210 I=1,N
        SDYF(I) = SQRT(((SDYF(I))**2) + (1.0 / W(I)))
        CINT1(I) = YFIT(I) + T * SDYF(I)
        CINT2(I) = YFIT(I) - T * SDYF(I)
  210 CONTINUE
      K = N
C
C     FIND THE MINIMUM AND MAXIMUM VALUE FOR THE Y AXIS
C
  300 YMIN = CINT2(1)
      YMAX = CINT1(1)
C
C
      DO 310 I=2,K
         IF(CINT2(I) .LT. YMIN) YMIN = CINT2(I)
         IF(CINT1(I) .GT. YMAX) YMAX = CINT1(I)
  310 CONTINUE
C
C
      YINT = (YMAX - YMIN) / 80.0
      YMAX = YMAX + YINT
      YMIN = YMIN - YINT
C
C
C     CALL TO SUBPLOT TO PLOT INFORMATION CALCULATED BY CIPLOT
C     USING SIMPLT AND PLODA(AELIB ROUTINES)
C
      CALL SUBPLOT(X,Y,N,XX,YF,M,CINT1,CINT2,PROB,LOPT,NOPT)
      GO TO 9000
C
C
  500 CONTINUE
C
C     PROB DOES NOT LIE BETWEEN ZERO AND ONE.
C
      IER = 1
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 510
         PRINT 6000
         CALL TRACEB(0)
  510 CONTINUE
      GO TO 9000
C
C     DEGREES OF FREEDOM(F) ARE LESS THAN OR EQUAL TO ONE
C
  600 CONTINUE
      IER = 2
      IF(.NOT.ALERPR(IROUT,IER))GO TO 610
         PRINT 6100
         CALL TRACEB(0)
  610 CONTINUE
      GO TO 9000
C
C     INTERNAL ERROR IN IMSL ROUTINE MDSTI
C
  700 CONTINUE
      IER = 3
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 710
         PRINT 6200
         CALL TRACEB(0)
  710 CONTINUE
C
C
 9000 CONTINUE
      RETURN
      END
      SUBROUTINE SUBPLOT(X,Y,N,XX,YF,M,CINT1,CINT2,PROB,LOPT,NOPT)
C
C **********************************************************************
C *****                                                            *****
C *****                  PLOT CONFIDENCE INTERVAL                  *****
C *****                                                            *****
C **********************************************************************
C *                                                                    *
C * PURPOSE:     SUBROUTINE TO PLOT THE DATA POINTS AS SUPPLIED BY     *
C *              THE USER, THE FITTED FUNCTION CALCULATED BY CIPLOT    *
C *              USING 100 EQUALLY SPACED POINTS (CALCULATED BY        *
C *              CIPLOT) AND THE CONFIDENCE INTERVAL ON MEAN VALUES    *
C *              OR PREDICTED VALUES AS REQUESTED BY USER.  THE        *
C *              CONFIDENCE INTERVAL IS PLOTTED USING DASHED LINES.    *
C *                                                                    *
C *                                                                    *
C * ENTRY:                                                             *
C *    CALL SUBPLOT(X,Y,N,XX,YF,M,CINT1,CINT2,PROB,LOPT,NOPT)          *
C *                                                                    *
C *    X(N)......REAL ARRAY OF DATA POINTS                             *
C *    Y(N)......REAL ARRAY OF DATA VALUES OF THE DEPENDENT VARIABLE   *
C *    N.........INTEGER VARIABLE, NUMBER OF DATA POINTS               *
C *    XX(M).....REAL ARRAY SET UP BY CIPLOT CONTAINING 100            *
C *              EQUALLY SPACED DATA POINTS                            *
C *    YF(M).....REAL ARRAY SET UP BY CIPLOT CONTAINING THE            *
C *              FITTED VALUES FOR THE 100 EQUALLY SPACED POINTS       *
C *    M.........INTEGER VARIABLE, SET TO 100 IN CIPLOT                *
C *    CINT1(M)..REAL ARRAY SET UP BY CIPLOT CONTAINING THE UPPER      *
C *              BOUNDARIES OF THE CONFIDENCE INTERVAL.                *
C *    CINT2(M)..REAL ARRAY SET UP BY CIPLOT CONTAINING THE LOWER      *
C *              BOUNDARIES OF THE CONFIDENCE INTERVAL.                *
C *    PROB......REAL VARIABLE, 100 * ( 1 - PROB ) % CONFIDENCE        *
C *              INTERVAL                                              *
C *    LOPT......=1  CALCULATES CONFIDENCE INTERVAL ON MEAN VALUES     *
C *              NE 1 CALCULATES CONFIDENCE INTERVAL ON PREDICTED      *
C *              VALUES                                                *
C *    NOPT......=0,  PLOTS INFORMATION REQUESTED FOR 100 EQUALLY      *
C *              SPACED POINTS                                         *
C *              =1,  PLOTS INFORMATION REQUESTED FOR N DATA POINTS    *
C *                                                                    *
C *                                                                    *
C *                                                                    *
C * AUTHORS :                                                          *
C *                                                                    *
C *           C. A. WILLS                                              *
C *           M. T. BOULANGER                                          *
C *           WRITTEN FEBRUARY 1982.                                   *
C *                                                                    *
C *                                                                    *
C **********************************************************************
      INTEGER STVARS
      REAL X(N), Y(N),  XX(M),  YF(M), CINT1(M), CINT2(M)
      REAL FRAC(4), WS1(396)
      COMMON/SCALES/XMIN,XMAX,TICX,FXSCAL,YMIN,YMAX,TICY,FYSCAL
      COMMON/FORMAT/XL,YL,FTITLE(8),FSUBTIT(8),FXAXTIT(8),FYAXTIT(8),
     ,NSTVARS,STVARS(10)
      DATA FRAC,DLENG/0.4,0.2,0.2,0.2,1.0/
*CALL STATF
C
C     SET UP PARAMETERS FOR PLOT SUBTITLE
C
      STVARS(1) = ( 1 - PROB ) * 100
      NSTVARS = 1
      IF(LOPT .NE. 1) GO TO 10
C
C     SET UP FORMAT FOR PLOT SUBTITLE WHEN LOPT=1
C
*IF -DEF,AELIB5
      FSUBTIT(1) = 10H(1X,I3,*%
*ENDIF
*IF DEF,AELIB5
       FSUBTIT(1) = 10H(1X,I3,'%
*ENDIF
      FSUBTIT(2) = 10HCONFIDENCE
      FSUBTIT(3) = 10H INTERVAL
      FSUBTIT(4) = 10HON MEAN VA
*IF -DEF,AELIB5
      FSUBTIT(5) = 10HLUES*)
*ENDIF
*IF DEF,AELIB5
       FSUBTIT(5) = 10HLUES')
*ENDIF
      GO TO 20
C
C     SET UP FORMAT FOR PLOT SUBTITLES WHEN LOPT IS NOT EQUAL TO 1
C
*IF -DEF,AELIB5
   10 FSUBTIT(1) = 10H(1X,I3,*%
*ENDIF
*IF DEF,AELIB5
  10   FSUBTIT(1) = 10H(1X,I3,'%
*ENDIF
      FSUBTIT(2) = 10HCONFIDENCE
      FSUBTIT(3) = 10H INTERVAL
      FSUBTIT(4) = 10HON PREDICT
      FSUBTIT(5) = 10HED VALUES
*IF -DEF,AELIB5
      FSUBTIT(6) = 10H*)
*ENDIF
*IF DEF,AELIB5
       FSUBTIT(6)=10H')
*ENDIF
   20 CONTINUE
C
C     PLOT THE DATA POINTS
C
      JOBOPT = 1
      PEN = 0.0
      SYMBOL = 8.0
      CALL SIMPLT(JOBOPT,X,Y,N,PEN,SYMBOL)
C
C     PLOT THE FITTED FUNCTION(USING THE 100 EQUALLY SPACED POINTS)
C
      JOBOPT = 3
      PEN = 1.0
      SYMBOL = 0.0
      CALL SIMPLT(JOBOPT,XX,YF,M,PEN,SYMBOL)
C
C     PLOT THE CONFIDENCE INTERVAL ON MEAN VALUES AND WHEN
C     WEIGHTS ARE ALL THE SAME.
C
      IF(NOPT .EQ. 1) GO TO 40
      IOPT = 1
      LWS = 396
      CALL PLODA(IOPT,XX,CINT1,M,FRAC,DLENG,WS1,LWS)
      CALL PLODA(IOPT,XX,CINT2,M,FRAC,DLENG,WS1,LWS)
      RETURN
C
C     PLOT THE CONFIDENCE INTERVAL ON PREDICTED VALUES WHEN WEIGHTS
C     ARE DIFFERENT
C
   40 CONTINUE
      IOPT = 1
      LWS = 4 * ( N - 1 )
      CALL PLODA(IOPT,X,CINT1,N,FRAC,DLENG,WS1,LWS)
      CALL PLODA(IOPT,X,CINT2,N,FRAC,DLENG,WS1,LWS)
      RETURN
      END
