*DECK CLEBSCH
      FUNCTION CLEBSCH (VJ1,VJ2,VM1,VM2,VJ3)
*  THIS PROGRAM, WRITTEN BY R.Y. CUSSON AND J.H. SCHMIDT, USES THE
*  METHODS OF SCHWINGER AND REGGE (QUANTUM THEORY OF ANGULAR MOMENTUM
*  BY BIEDERHARN AND VAN DAM, PP. 246 AND 296).
*
*  SINCE MANY PROGRAMS WHICH USE CLEBSCH-GORDAN COEFFICIENTS ALSO USE
*  RACAH COEFFICIENTS, COMMON BLOCK CGRACAH CONTAINS TABLES OF LOGS
*  AND SQUARE ROOTS SO THEY DO NOT HAVE TO BE CALCULATED EACH TIME THE
*  FUNCTIONS ARE CALLED. NOTE THAT ALOGF(0) IS OFTEN REQUIRED IN THE
*  CALCULATIONS AND SO ALOGF0=ALOGF(0).
*
*  COMMON BLOCK KLEBSYS IS INCLUDED TO ENSURE THAT ALL ITS ELEMENTS ARE
*  STORED SEQUENTIALLY IN CORE. THIS IS NECESSARY FOR EQUIVALENCING.
      COMMON/CGRACAH/INITIAL,ALOGF0,ALOGF(80),SQRF(80)
      COMMON/KLEBSYS/N11,N21,N31,N12,N22,N32,N13,N23,N33
      COMMON/TRIANG/IPRINT
      DIMENSION NN(3,3),VN(10)
*IF -DEF,AELIB5
      EQUIVALENCE(RJ1,N11,NN),(RJ2,N22),(RJ3,N23),(RJ4,N32),(RJ5,N33)
*ENDIF
*IF DEF,AELIB5
      EQUIVALENCE(RJ1,N11,NN(1,1)),(RJ2,N22),(RJ3,N23),(RJ4,N32),(RJ5,N3
     +3)
*ENDIF
      DATA INITIAL/0/,ALOGF0/0./
      DATA IPRINT/0/
*IF -DEF,AELIB5
 400  FORMAT (*  ILLEGAL PARAMETERS IN CLEBSCH(*,5F5.1,*)*)
*ENDIF
*IF DEF,AELIB5
 400  FORMAT ('  ILLEGAL PARAMETERS IN CLEBSCH(',5F5.1,')')
*ENDIF
*CALL  STATF
      CLEBSCH=0.
      IF ((VJ1+VJ2+VJ3) .GT. 78.) GO TO 900
*  ENSURES THAT ALOGF AND SQRF ARE SET UP ONLY ONCE IN EACH PROGRAM
      IF (INITIAL .EQ. 1) GO TO 20
      ALOGF(1)=0.
*IF -DEF,AELIB5
      SQRF(1)=1.   $    VM3=1.
*ENDIF
*IF DEF,AELIB5
      SQRF(1)=1.
      VM3=1.
*ENDIF
      DO 15 J=2,80
      VM3=VM3+1.
      SQRF(J)=SQRT(VM3)
 15   ALOGF(J)=ALOGF(J-1)+ALOG(VM3)
      INITIAL=1
*  SET UP THE REGGE SYMBOL
 20   VM3=VM1+VM2
      VN(1)=VJ1+VJ2-VJ3
      VN(2)=VJ2+VJ3-VJ1
      VN(3)=VJ1-VJ2+VJ3
      VN(4)=VJ3+VM3
      VN(5)=VJ1-VM1
      VN(6)=VJ2-VM2
      VN(7)=VJ3-VM3
      VN(8)=VJ1+VM1
      VN(9)=VJ2+VM2
      VN(10)=VJ1+VJ2+VJ3
*  CHECK THAT THE REGGE SYMBOL CONTAINS ONLY NON-NEGATIVE INTEGERS
      IEXIT=0
      DO 40 J=1,10
      NT=ABS(VN(J))
      IF(FLOAT(NT).EQ.VN(J)) GOTO 40
      IF(J.GT.3.OR.IPRINT.EQ.1) GOTO 900
      IEXIT=1
*IF -DEF,AELIB5
 40   IF(J.NE.10)NN(J)=NT
*ENDIF
*IF DEF,AELIB5
 40   IF(J.NE.10)NN(J,1)=NT
*ENDIF
      IF(IEXIT.EQ.1) RETURN
      GO TO 201
*IF -DEF,AELIB5
      ENTRY FCLEBSH
*ENDIF
*IF DEF,AELIB5
      ENTRY FCLEBSH(VJ1,VJ2,VM1,VM2,VJ3)
*ENDIF
*  NOTE THAT CLEBSCH MUST BE CALLED BEFORE FCLEBSH BECAUSE FCLEBSH
*  BYPASSES THE INITIALIZATION OF ALOGF AND SQRF
*  THE CALLING SEQUENCE IS   F=FCLEBSH(N1,N2,N3,N4,N5)  WHERE
*         N1=J1+J2-J3         N4=J1+M1
*         N2=J1-M1            N5=J2+M2
*         N3=J2-M2
*  NOTE THAT FCLEBSH DOES NO ARGUMENT CHECKING
*  THE NEXT LINE COPIES THE (INTEGER) FORMAL ARGUMENTS INTO N(I,J)
*IF -DEF,AELIB5
      RJ1=VJ1  $  RJ2=VJ2  $  RJ3=VM1  $  RJ4=VM2  $  RJ5=VJ3
*ENDIF
*IF DEF,AELIB5
      RJ1=VJ1
      RJ2=VJ2
      RJ3=VM1
      RJ4=VM2
      RJ5=VJ3
*ENDIF
*  SET UP THE REGGE SYMBOL FOR THE INTEGER CASE
*IF -DEF,AELIB5
      NU=N22-N11  $  NV=N33-N11  $  N12=N23+NV
      N21=N32+NV  $  N13=N32+NU  $  N31=N23+NU
*ENDIF
*IF DEF,AELIB5
      NU=N22-N11
      NV=N33-N11
      N12=N23+NV
      N21=N32+NV
      N13=N32+NU
      N31=N23+NU
*ENDIF
 201  NW = MIN0(N11,N12,N13,N21,N22,N23,N31,N32,N33)
      JT= N21+N22+N23
      JJP=JT-N11+1
      IFSE=0
*  FIND THE SMALLEST ELEMENT OF THE REGGE SYMBOL,BRING IT TO THE N11
*  POSITION AND PERMUTE THE SYMBOL FOR FURTHER CALCULATIONS
      IF (NW .EQ. N11) GO TO  7
      IF (NW .EQ. N31) GO TO 43
      IF (NW .EQ. N12) GO TO 44
      IF (NW .EQ. N22) GO TO 45
      IF (NW .EQ. N32) GO TO 46
      IF (NW .EQ. N13) GO TO 47
      IF (NW .EQ. N23) GO TO 48
      IF (NW .EQ. N33) GO TO 49
      IFSE=N33
*IF -DEF,AELIB5
      NW= N21$ N21= N11$ N11= NW$ NU= N12$ N12= N22$ N22= NU
      NV= N13$ N13= N23$ N23= NV
*ENDIF
*IF DEF,AELIB5
      NW= N21
      N21= N11
      N11= NW
      NU= N12
      N12= N22
      N22= NU
      NV= N13
      N13= N23
      N23= NV
*ENDIF
      GO TO 7
 43   IFSE=N22
*IF -DEF,AELIB5
      NW= N31$ N31= N11$ N11= NW$ NU= N12$ N12= N32$ N32= NU
      NV= N13$ N13= N33$ N33= NV
*ENDIF
*IF DEF,AELIB5
      NW= N31
      N31= N11
      N11= NW
      NU= N12
      N12= N32
      N32= NU
      NV= N13
      N13= N33
      N33= NV
*ENDIF
      GO TO 7
 44   IFSE=N33
*IF -DEF,AELIB5
      NW= N12$ N12= N11$ N11= NW$ NU= N21$ N21= N22$ N22= NU
      NV= N31$ N31= N32$ N32= NV
*ENDIF
*IF DEF,AELIB5
      NW= N12
      N12= N11
      N11= NW
      NU= N21
      N21= N22
      N22= NU
      NV= N31
      N31= N32
      N32= NV
*ENDIF
      GO TO 7
*IF -DEF,AELIB5
 45   NW= N22$ N22= N11$ N11= NW$ NT= N21$ N21= N12$ N12= NT
      NU= N13$ N13= N23$ N23= NU$ NV= N31$ N31= N32$ N32= NV
*ENDIF
*IF DEF,AELIB5
 45   NW= N22
      N22= N11
      N11= NW
      NT= N21
      N21= N12
      N12= NT
      NU= N13
      N13= N23
      N23= NU
      NV= N31
      N31= N32
      N32= NV
*ENDIF
      GO TO 7
 46   IFSE=N11+N32
*IF -DEF,AELIB5
      NW= N32$ N32= N11$ N11= NW$NT= N31$N31=N12$ N12= NT
      NU= N13$ N13= N33$ N33= NU$ NV= N21$ N21= N22$ N22= NV
*ENDIF
*IF DEF,AELIB5
      NW= N32
      N32= N11
      N11= NW
      NT= N31
      N31=N12
      N12= NT
      NU= N13
      N13= N33
      N33= NU
      NV= N21
      N21= N22
      N22= NV
*ENDIF
      GO TO 7
 47   IFSE=N22
*IF -DEF,AELIB5
      NW= N13$ N13= N11$ N11= NW$ NT= N21$ N21= N23$ N23= NT
      NU= N31$ N31= N33$ N33= NU
*ENDIF
*IF DEF,AELIB5
      NW= N13
      N13= N11
      N11= NW
      NT= N21
      N21= N23
      N23= NT
      NU= N31
      N31= N33
      N33= NU
*ENDIF
      GO TO 7
 48   IFSE=N11+N23
*IF -DEF,AELIB5
      NW= N23$ N23= N11$ N11= NW$NU=N31$N31= N33$ N33= NU
      NT= N21$ N21= N13$ N13= NT$ NV= N12$ N12= N22$ N22= NV
*ENDIF
*IF DEF,AELIB5
      NW= N23
      N23= N11
      N11= NW
      NU=N31
      N31= N33
      N33= NU
      NT= N21
      N21= N13
      N13= NT
      NV= N12
      N12= N22
      N22= NV
*ENDIF
      GO TO 7
*IF -DEF,AELIB5
 49   NW= N33$ N33= N11$ N11= NW$ NT= N31$ N31= N13$ N13= NT
      NU= N21$ N21= N23$ N23= NU$ NV= N12$ N12= N32$ N32= NV
*ENDIF
*IF DEF,AELIB5
 49   NW= N33
      N33= N11
      N11= NW
      NT= N31
      N31= N13
      N13= NT
      NU= N21
      N21= N23
      N23= NU
      NV= N12
      N12= N32
      N32= NV
*ENDIF
*  NOW CONSTRUCT   DELTA*NORMALIZATION  AND DO THE SUMMATION
*  THE EXPRESSION  (IFSE .AND. 1) SIMPLY CHECKS IF IFSE IS ODD OR EVEN
 7    IFSE=IFSE .AND. 1
*  IN WELL OVER HALF THE CASES, N11=0 OR 1. THEREFORE THESE CASES ARE
*  TREATED SEPARATELY TO SIMPLIFY AND SPEED UP THE CALCULATIONS
      RJ=(ALOGF(N12)+ALOGF(N13)+ALOGF(N21)+ALOGF(N31))
     1  -(ALOGF(N22)+ALOGF(N33)+ALOGF(JT+1))
      IF (N11 - 1) 12,125,149
 12   RJ=.5*(RJ-(ALOGF(N23)+ALOGF(N32)))
      CLEBSCH=EXP(RJ)*SQRF(JJP)*(1-IFSE-IFSE)
      RETURN
 125  RJ=.5*(RJ+ALOGF(N23)+ALOGF(N32))-(ALOGF(N32-1)+ALOGF(N23-1))
*IF -DEF,AELIB5
      ZA=N22*N33    $    ZB=N23*N32
*ENDIF
*IF DEF,AELIB5
      ZA=N22*N33
      ZB=N23*N32
*ENDIF
      SUP=1.-ZA/ZB
      GO TO 13
 149  RJ=.5*(RJ+ALOGF(N23)+ALOGF(N32)-ALOGF(N11))
     1         -(ALOGF(N32-N11)+ALOGF(N23-N11))
*IF -DEF,AELIB5
      ZA=N22-N11    $    ZB=N33-N11
      ZC=N11+1      $    ZD=N23+1     $    ZE=N32+1
*ENDIF
*IF DEF,AELIB5
      ZA=N22-N11
      ZB=N33-N11
      ZC=N11+1
      ZD=N23+1
      ZE=N32+1
*ENDIF
      R=2.
      SUP=1.-(ZA+1.)*(ZB+1.)/(N11*N23*N32)
 150  SUP=1.-SUP*R*(ZA+R)*(ZB+R)/((ZC-R)*(ZD-R)*(ZE-R))
      R=R+1.
      IF (R .LT. ZC) GO TO 150
 13   CLEBSCH=SUP*EXP(RJ)*SQRF(JJP)*(1-IFSE-IFSE)
*  NOTE   SUP*EXP(RJ) = 3J SYMBOL
 999  RETURN
 900  PRINT 400,VJ1,VJ2,VM1,VM2,VJ3
      GO TO 999
      END
