*DECK CMMULT
      SUBROUTINE CMMULT(P,Q,R,NP,NQ,NR,K,L,M,N)
C
C **********************************************************************
C ******                                                          ******
C ******                CMMULT(CMADD,CMSUB)      1-7-02           ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE : TO MULTIPLY, ADD OR SUBTRACT COMPLEX MATRICES.           *
C *                                                                    *
C * ENTRY :   CALL CMMULT(P,Q,R,NP,NQ,NR,K,L,M,N)                      *
C *           CALL CMADD(P,Q,R,NP,NQ,NR,K,L,M,N)                       *
C *           CALL CMSUB(P,Q,R,NP,NQ,NR,K,L,M,N)                       *
C *                                                                    *
C *           P(K,L) - COMPLEX INPUT MATRIX.                           *
C *           Q(M,N) - COMPLEX INPUT MATRIX.                           *
C *           R      - COMPLEX OUTPUT MATRIX WHOSE DIMENSIONS DEPEND   *
C *                    ON THOSE OF P AND Q.                            *
C *           NP     - INTEGER INPUT VARIABLE.                         *
C *                  - NUMBER OF ROWS IN P AS DIMENSIONED IN THE       *
C *                    CALLING PROGRAM.                                *
C *           NQ     - INTEGER INPUT VARIABLE.                         *
C *                  - NUMBER OF ROWS IN Q AS DIMENSIONED IN THE       *
C *                    CALLING PROGRAM.                                *
C *           NR     - INTEGER INPUT VARIABLE.                         *
C *                  - NUMBER OF ROWS IN R AS DIMENSIONED IN THE       *
C *                    CALLING PROGRAM.                                *
C *                                                                    *
C * ADDITIONAL VARIABLES :                                             *
C *                                                                    *
C *           IROUT  - INTEGER VARIABLE DEFINING THE ROUTINE NUMBER    *
C *                    FOR CMMULT TO BE USED IN ERROR MESSAGE CONTROL. *
C *           OPTION - INTEGER VARIABLE USED TO EXECUTE THE REQUESTED  *
C *                    OPERATION.                                      *
C *           ROWR   - INTEGER VARIABLE DEFINING THE NUMBER OF ROWS    *
C *                    IN OUTPUT ARRAY R.                              *
C *                  - IF R=P*Q, ROWR=K.                               *
C *                    IF R=P+Q OR R=P-Q, ROWR=MAX(K,M).               *
C *           COLR   - INTEGER VARIABLE DEFINING THE NUMBER OF CLOUMNS *
C *                    IN OUTPUT ARRAY R.                              *
C *                  - IF R=P*Q, COLR=N.                               *
C *                    IF R=P+Q OR R=P-Q, COLR=MAX(L,N).               *
C *           NUROWR - INTEGER VARIABLE.                               *
C *                  - SINCE SETBAD OPERATES ON REAL MATRICES AND      *
C *                    CMMULT ON COMPLEX MATRICES, NUROWR IS SET TO    *
C *                    2*NR FOR MATRIX TRANSFER COMPATIBILITY.         *
C *                                                                    *
C * ABNORMAL TERMINATION :                                             *
C *                                                                    *
C *           AN ATTEMPT TO EXECUTE CMMULT, CMADD OR CMSUB WITH ONE OF *
C *           THE CONDITIONS GIVEN BELOW RESULTS IN A RETURN TO THE    *
C *           CALLING ROUTINE WITH AN APPROPRIATE ERROR MESSAGE.  THE  *
C *           ENTRIES IN ARRAY R ARE RETURNED SET TO INDEFINITE.       *
C *                1. NP,NQ,NR,K,L,M OR N < 1.                         *
C *                2. K>NP OR M>NQ.                                    *
C *                3. K>NR IF CMMULT IS CALLED OR MAX(K,M)>NR IF CMADD *
C *                   OR CMSUB IS CALLED.                              *
C *                4. L.NE.M IF CMMULT IS CALLED.
C *                5. THE SMALLER MATRIX NOT COMPLETELY CONTAINED IN   *
C *                   THE LARGER MATRIX IF CMADD OR CMSUB ARE CALLED.  *
C *                                                                    *
C * AUTHOR :  G.L. KLAWITTER                                           *
C *           WRITTEN FEBRUARY, 1980 TO REPLACE PREVIOUS VERSION BY    *
C *           P.H. GREEN SEPTEMBER 1970.                               *
C *                                                                    *
C **********************************************************************
C
C
      INTEGER NP,NQ,NR,K,L,M,N,ROWR,COLR
      COMPLEX P(NP,1),Q(NQ,1),R(NR,1)
      LOGICAL ALERPR
C
C     ESTABLISH PROPER ENTRY POINT AND INITIALIZE OPTION.  TRANSFER TO
C     DETERMINE SIZE OF OUTPUT MATRIX R.
C
C     CMMULT ENTRY POINT.
C
      OPTION = 6HCMMULT
      GO TO 50
C
C     CMADD ENTRY POINT.
C
*IF -DEF,AELIB5
      ENTRY CMADD
*ENDIF
*IF DEF,AELIB5
      ENTRY CMADD(P,Q,R,NP,NQ,NR,K,L,M,N)
*ENDIF
      OPTION = 5HCMADD
      GO TO 50
C
C     CMSUB ENTRY POINT.
C
*IF -DEF,AELIB5
      ENTRY CMSUB
*ENDIF
*IF DEF,AELIB5
      ENTRY CMSUB(P,Q,R,NP,NQ,NR,K,L,M,N)
*ENDIF
      OPTION = 5HCMSUB
C
C     ESTABLISH OUTPUT MATRIX SIZE.
C
   50 ROWR = K
      COLR = N
      IF(OPTION.NE.6HCMMULT) ROWR = MAX0(K,M)
      IF(OPTION.NE.6HCMMULT) COLR = MAX0(L,N)
C
C
C     CALL LIBRARY STATISTICS.
C
*CALL STATF
C
C     DEFINE IROUT, THE ROUTINE NUMBER OF CMMULT/CMADD/CMSUB IN
C     AECL-6076.
C
      IROUT = 10702
C
C     CHECK PARAMETERS FOR ABNORMAL TERMINATION.
C
      IF(NP.LT.1.OR.NQ.LT.1.OR.NR.LT.1.OR.K.LT.1.OR.L.LT.1.OR.M.LT.1.OR.
     ,N.LT.1) GO TO 400
      IF(K.GT.NP.OR.M.GT.NQ) GO TO 500
      IF(ROWR.GT.NR) GO TO 600
      IF(OPTION.EQ.6HCMMULT.AND.(L.NE.M)) GO TO 700
      IF(OPTION.NE.6HCMMULT.AND.(.NOT.((K.LE.M.AND.L.LE.N).OR.(M.LE.K.AN
     ,D.N.LE.L)))) GO TO 800
C
C     ZERO THE OUTPUT MATRIX.
C
      DO 100 I=1,ROWR
         DO 100 J=1,COLR
            R(I,J) = 0.0
  100 CONTINUE
C
C     TEST OPTION AND TRANSFER TO APPROPRIATE OPERATION AS DICTATED BY
C     OPTION.
C
      IF(OPTION.NE.6HCMMULT) GO TO 200
C
C        ***************************************************************
C        ******  MULTIPLY GIVEN MATRICES AND RETURN.              ******
C        ***************************************************************
C
         DO 150 I=1,ROWR
            DO 150 J=1,COLR
               DO 150 KOL=1,L
                  R(I,J) = R(I,J) + P(I,KOL) * Q(KOL,J)
  150    CONTINUE
         RETURN
C
C        ***************************************************************
C        ******  ADD OR SUBTRACT GIVEN MATRICES AS REQUESTED.     ******
C        ***************************************************************
C
C        THE TASK IS ACCOMPLISHED BY INITIALIZING R WITH CORRESPONDING
C        ENTRIES FROM P AND THEN ADDING TO R OR SUBTRACTING FROM R THE
C        CORRESPONDING ENTRIES IN Q.
C
  200    DO 250 I=1,K
            DO 250 J=1,L
               R(I,J) = P(I,J)
  250    CONTINUE
C
C        TEST OPTION AND PERFORM REQUESTED OPERATION.
C
         IF(OPTION.EQ.5HCMSUB) GO TO 300
C
C           ******  ADD GIVEN MATRICES AND RETURN.
C
            DO 275 I=1,M
               DO 275 J=1,N
                  R(I,J) = R(I,J) + Q(I,J)
  275       CONTINUE
            RETURN
C
C           ******  SUBTRACT GIVEN MATRICES AND RETURN.
C
  300       DO 350 I=1,M
               DO 350 J=1,N
                  R(I,J) = R(I,J) - Q(I,J)
  350       CONTINUE
            RETURN
C
C
C        ***************************************************************
C        ******  OUTPUT APPROPRIATE ERROR MESSAGE.                ******
C        ******  SET ENTRIES IN RETURNING MATRIX R TO INDEFINITE. ******
C        ***************************************************************
C
C        INPUT PARAMETER ERROR.  PARAMETERS SPECIFIED INCORRECTLY.
C
  400    IF(.NOT.ALERPR(IROUT,0)) GO TO 950
            PRINT 2000,NP,NQ,NR,K,L,M,N
            GO TO 900
C
C        INPUT PARAMETER ERROR. K>NP OR M>NQ IS INVALID.
C
  500       IF(.NOT.ALERPR(IROUT,0)) GO TO 950
            PRINT 2010,K,NP,M,NQ
            GO TO 900
C
C        INPUT PARAMETER ERROR RESULTS IN MATRIX R BEING TOO SMALL TO
C        HOLD RESULTS.
C
  600    IF(.NOT.ALERPR(IROUT,0)) GO TO 950
            PRINT 2020,NR,ROWR
            GO TO 900
C
C        MATRIX MULTIPLICATION ERROR.  NUMBER OF COLUMNS IN ARRAY P
C        .NE. NUMBER OF ROWS IN ARRAY Q.
C
  700    IF(.NOT.ALERPR(IROUT,0)) GO TO 950
            PRINT 2030,L,M
            GO TO 900
C
C        MATRIX ADDITION OR SUBTRACTION ERROR.  SMALLER MATRIX NOT
C        CONTAINED IN LARGER.
C
  800    IF(.NOT.ALERPR(IROUT,0)) GO TO 950
            PRINT 2040,K,L,M,N
  900       CALL TRACEB(0)
C
C           SET ENTRIES IN RETURNING MATRIX R TO INDEFINITE VIA SETBAD.
C           RETURN.
C
  950       NUROWR = 2 * NR
            CALL SETBAD(R,NUROWR,NUROWR,COLR)
            RETURN
C
C
*IF -DEF,AELIB5
 2000 FORMAT(" ***INPUT PARAMETER VIOLATION.  ALL INTEGER INPUT PARAMETE
     ,RS MUST BE > 0.", /4X,"NP=",I5," NQ=",I5," NR=",I5," K=",I5," L=",
     ,I5," M=",I5," N=",I5)
 2010 FORMAT(" ***ARRAY BOUNDS EXCEEDED.      K>NP OR M>NQ IS INVALID.",
     ,/4X,"K=",I5," NP=",I5,"    M=",I5," NQ=",I5)
 2020 FORMAT(" ***ARRAY BOUNDS EXCEEDED.",/4X,"FOR CMMULT K>NR IS INVALI
     ,D.  FOR CMADD/CMSUB MAX(K,M)>NR IS INVALID.",/4X,"NR=",I5,"     K
     ,OR MAX(K,M) =",I5)
 2030 FORMAT(" ***MATRIX MULTIPLICATION ERROR.  COLUMNS IN P .NE. ROWS I
     ,N Q.",/4X,"NUMBER OF COLUMNS IN P=",I5,"   NUMBER OF ROWS IN Q=",I
     ,5)
 2040 FORMAT(" ***MATRIX SIZES IN ERROR.",/4X,"SMALLER MATRIX NOT COMPLE
     ,TELY CONTAINED IN LARGER.",/4X,"P(",I5," BY",I5,")    Q(",I5," BY"
     ,,I5,")")
*ENDIF
*IF DEF,AELIB5
 2000 FORMAT(' ***INPUT PARAMETER VIOLATION.  ALL INTEGER INPUT PARAMETE
     ,RS MUST BE > 0.', /4X,'NP=',I5,' NQ=',I5,' NR=',I5,' K=',I5,' L=',
     ,I5,' M=',I5,' N=',I5)
 2010 FORMAT(' ***ARRAY BOUNDS EXCEEDED.      K>NP OR M>NQ IS INVALID.',
     ,/4X,'K=',I5,' NP=',I5,'    M=',I5,' NQ=',I5)
 2020 FORMAT(' ***ARRAY BOUNDS EXCEEDED.',/4X,'FOR CMMULT K>NR IS INVALI
     ,D.  FOR CMADD/CMSUB MAX(K,M)>NR IS INVALID.',/4X,'NR=',I5,'     K
     ,OR MAX(K,M) =',I5)
 2030 FORMAT(' ***MATRIX MULTIPLICATION ERROR.  COLUMNS IN P .NE. ROWS I
     ,N Q.',/4X,'NUMBER OF COLUMNS IN P=',I5,'   NUMBER OF ROWS IN Q=',I
     ,5)
 2040 FORMAT(' ***MATRIX SIZES IN ERROR.',/4X,'SMALLER MATRIX NOT COMPLE
     ,TELY CONTAINED IN LARGER.',/4X,'P(',I5,' BY',I5,')    Q(',I5,' BY'
     ,,I5,')')
*ENDIF
      END
