*DECK COSIMP
      FUNCTION COSIMP(A, B, AUX, EPS, ISUB)
      COMMON/QSTATS/ERR,KOUNT,INTS
C
C  COSIMP, A ROUTINE FOR EVALUATING DEFINITE INTEGRALS USING AN
C  ADAPTIVE SIMPSONS ALGORITHM.
C  IF ISUB = 0 -
C     OVER THE INTERVAL (A,B) THE INTEGRAL IS ESTIMATED AS A SINGLE
C     STRIP AND AS A PAIR OF STRIPS.
C       EST1 = F(A) + 4*F(A+B/2) + F(B)
C       EST2 = F(A) + 4*F(3A+B/4) + 2*F(A+B/2) + 4*F(A+3B/4) + F(B)
C     IF THESE ESTIMATES DO NOT AGREE TO WITHIN EPS THE INTEGRAL IS
C     CALCULATED BY THE SAME METHOD TO WITHIN EPS/2 ON EACH HALF OF THE
C     RANGE SEPARATELY.
C  IF ISUB " 0 -
C     THE INTERVAL (A,B) IS SUBDIVIDED INTO A MINIMUM OF 2**ISUB SMALLER
C     STRIPS.  EACH OF THESE SMALLER STRIPS IS EVALUATED AS ABOVE UNTIL
C     THE ERROR REQUIREMENT IS SATISFIED.
C
C  A AND B - LIMITS OF INTEGRATION
C  AUX - EXTERNAL FUNCTION TO EVALUATE THE INTEGRAND, F(X)
C  EPS - REQUIRED RELATIVE ERROR TOLERANCE
C  ISUB - SPECIFICATION OF THE MINIMUM NUMBER OF DIVISIONS REQUIRED
C       RETURNS THE NUMBER OF DIVISIONS USED ( NEGATIVE IF NO CONVERGED)
C         IN EACH CASE THE NUMBER OF DIVISIONS IS 2**ISUB.
C        ISUB MAY BE ENTERED NEGATIVE TO ACTIVATE ERROR MESSAGES
C
C  ERR - ESTIMATE OF THE MAXIMUM ERROR IN THE INTEGRAL
C  KOUNT - NUMBER OF FUNCTION EVALUATIONS
C  INTS - NUMBER OF INTERVALS USED
C
C  Y - REAL ARRAY STORES VALUES OF F(X) TO AVOID RECALCULATION
C  S, SS - TWO ESTIMATES FOR A GIVEN SUBINTERVAL
C  SUMM - CUMULATIVE ESTIMATE OF THE INTEGRAL OVER THE ACCEPTED
C  SUBINTERVALS (UNSCALED)
C  PA1, PB1 - LIMITS OF INTERVAL NOT SATISFYING ERROR REQUIREMENT
C  SUBSUM - ESTIMATE OF INTEGRAL OVER THE ABOVE INTERVAL
C
C  IF ISUB IS OUTSIDE THE ACCETABLE RANGE, THIS MESSAGE IS PRINTED
C     ERROR IN ISUB, 0 @ ISUB @ 18. ISUB SET TO 0
C  COSIMP IS EVALUATED WITH ISUB = 0.
C  THE FUNCTION COSIMP RETURNS THE BEST ESTIMATE OF THE INTEGRAL.
C  THE FINEST SUBDIVISION ALLOWED IS (B-A)/2**20.  IF THE ERROR
C  REQUIREMENT IS STILL NOT SATISFIED, AN ERROR MESSAGE IS PRINTED
C  GIVING THE BOUNDS OF THE UNACCEPTABLE REGION, THE ESTIMATED INTEGRAL
C  OVER THIS INTERVAL, AND AN ESTIMATE OF THE ERROR THERE. THE VALUE
C  OF COSIMP RETURNED IS STILL THE BEST ESTIMATE OVER THE ENTIRE RANGE.
C       ISUB IS RETURNED NEGATIVE WHEN CONVERGENCE IS NOT COMPLETE.
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C       PROGRAMMED BY    D HOWDEN  IN APEX LANGUAGE        JUL 1965
C       TRANSLATED BY    C J JOHNSON                       NOV 1969
C       REVISED    BY    VALERIE JONES                     AUG 1976
C
      DIMENSION P(20),NR(20),Y(41)
*CALL ERRORF
      DATA MES/2250/
C
*CALL STATF
      COSIMP = ERR = 0.
      KOUNT = INTS = 0
      IFLAG=0
      IFLOG=0
      IF(A.EQ.B)RETURN
      IPRINT=0
      IF(ISUB.LT.0)IPRINT=1
      ISUB=MIN0(18,MAX0(0,ISUB))
      NSUB=ISUB
C
C
      LL=ISUB+2
      INTS = 0
      ERR = 0.
      ND=20
      P(1)=1.
      DO 5 N=2,ND
    5 P(N)=P(N-1)*2.
      C=B-A
      X=.5
      L=1
      N=2
      Y(1)=AUX(A)
      Y(2)=AUX(B)
      Y(3)=AUX(C*X+A)
      KOUNT=3
      SUMM=0
      M=3
      NR(N)=2
      AK=.5
   10 AJ=C*AK/2.
      NSUB=MAX0(NSUB,N)
      AJJ=C*X+A
      Y(N+N)=AUX(AJJ-AJ)
      Y(N+N+1)=AUX(AJJ+AJ)
      KOUNT=KOUNT+2
C
C     IF THE MINIMUM NUMBER OF SUBDIVISIONS (ISUB) HAS NOT BEEN MET,
C     THE INTERVAL IS DIVIDED IN HALF.
C
      IF(N.LT.LL) GOTO 25
      MM=NR(N)
C
C     TWO ESTIMATES OF THE INTERVAL ARE MADE AND THE ERROR ESTIMATE IS
C     COMPARED TO THE REQUIRED ERROR TOLERANCE.
C
      S=Y(L)+4.*Y(M)+Y(MM)
      SS=S+2.*(2.*(Y(N+N)+Y(N+N+1))-Y(M))
      ER = (AJ*(SS-2.*S)) / 45.
      IF(ABS(ER).GT.(AJ*EPS*(ABS(SS)+ABS(S))/4.5)) GO TO 20
      IF(IFLAG.EQ.1) GOTO 40
C
C     IF THE ERROR REQUIREMENT IS MET THE INTERVAL IS ACCEPTED.  THE
C     ESTIMATES ARE ADDED ON TO THEIR RESPECTIVE SUMS, THE COUNT OF THE
C     NUMBER OF INTERVALS USED (INTS) IS INCREASED BY ONE, AND THE NEXT
C     INTERVAL ON THE RIGHT IS CONSIDERED.
C
   15 SUMM=SUMM+(SS*AK)
      ERR = ERR + ER
      INTS=INTS+1
      IF (NR(N).LE.2) GOTO 55
      L=NR(N)
      NE=L/2
      AK=1./P(NE+2)
      X=X+AK+(1./P(N))
      N=NE+2
      NR(N)=NR(N-1)
      M=2*NE+3
      GOTO 10
C
C       ERROR RECOGNITION CRITERION
C
   20 IF (N.GE.ND) GOTO 30
C
C     IF THE ERROR REQUIREMENT IS NOT SATISFIED AND THE MAXIMUM NUMBER
C     OF SUBDIVISIONS HAS NOT BEEN REACHED, THE INTERVAL IS DIVIDED
C     IN HALF.
C
   25 NR(N+1)=M
      M=2*N
      AK=1./P(N+1)
      X=X-AK
      N=N+1
      GOTO 10
C
C     IF THE MAXIMUM NUMBER OF SUBDIVISIONS HAS BEEN REACHED WITHOUT
C     THE ERROR REQUIREMENT BEING MET, THE BOUNDS OF THE OFFENDING
C     INTERVAL ARE STORED WITH THE ESTIMATE OF THE INTEGRAL AND THE
C     ERROR FLAG IS SET.
C
   30 PA=AJJ-2.*AJ
      PB=AJJ+2.*AJ
      IF (IFLAG.EQ.1) GOTO 35
         PA1=PA
         PB1=PB
         SUBSUM=SS*AK
         SUBER = ER
         IFLAG=1
         IFLOG=1
         GOTO 15
C
C     CONSECUTIVE INTERVALS WHERE THE ERROR REQUIREMENT IS NOT MET ARE
C     COMBINED FOR READABILITY.
C
   35 PB1=PB
      SUBSUM=SUBSUM+(SS*AK)
      SUBER = SUBER + ER
      GOTO 15
C
C     WHEN THE NON CONVERGENT INTERVAL HAS BEEN COMPLETELY DETERMINED
C     A MESSAGE IS PRINTED GIVING THE BOUNDS, INTEGRAL, AND ERROR.
C
C
   40 IFLOG=1
      IFLAG=0
      IF(IPRINT.EQ.0) GOTO 15
      SSUBSUM=SUBSUM*(C/6.)
      SSUBER=ABS(SUBER)
      ENCODE(11,45,IERWD(3))PA1
*IF -DEF,AELIB5
      IERWD(4)=IERWD(4).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(4)=IERWD(4).AND.MASK(6)
*ENDIF
      ENCODE(11,45,IERWD(5))PB1
*IF -DEF,AELIB5
      IERWD(6)=IERWD(6).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(6)=IERWD(6).AND.MASK(6)
*ENDIF
      ENCODE(20,50,IERWD(7))SSUBSUM
*IF -DEF,AELIB5
      IERWD(9)=IERWD(9).A.MASK(0)
*ENDIF
*IF DEF,AELIB5
      IERWD(9)=IERWD(9).AND.MASK(0)
*ENDIF
      ENCODE(11,45,IERWD(10))SSUBER
*IF -DEF,AELIB5
      IERWD(11)=IERWD(11).A.MASK(6)
      CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(11)=IERWD(11).AND.MASK(6)
      CALL AELERR(MES,'NF')
*ENDIF
   45 FORMAT(E11.5)
   50 FORMAT(E20.14)
      GOTO 15
C
C     WHEN THE ENTIRE INTERVAL HAS BEEN CONSIDERED THE ANSWER IS
C     RETURNED ALONG WITH AN ESTIMATE OF THE ERROR.
C
   55 COSIMP=SUMM*(C/6.)
      ERR = ABS(ERR)
      ISUB=NSUB
      IF(IFLOG.EQ.1)ISUB=-NSUB
      RETURN
      END
