*DECK EIGCHER
      SUBROUTINE EIGCHER( N, IORDER, AR, AI, NVECS, WORKSTG,
     +                    NUM, EIGVAL, EIGVECR, EIGVECI )
C
************************************************************************
**                                                                    **
**    NOTE THAT EIGCHER IS WRITTEN IN MODULAR FORM USING THE          **
**     ADDITIONAL ROUTINES:                                           **
**     HTRIDI                                                         **
**     HTRIBK                                                         **
**     IMTQL1                                                         **
**     IMTQL2                                                         **
**     SORT                                                           **
**     HTRIDI AND HTRIBK ACCOMPANY EIGCHER ON THE AELIB SOURCE TAPE   **
**     UNDER THE DECK NAME EIGCHER.                                   **
**     IMTQL1, IMTQL2, AND SORT ARE STORED WITH THE EIGENANALYSIS     **
**     ROUTINE EIGRSYM UNDER THE DECK NAME EIGRSYM.                   **
**                                                                    **
************************************************************************
C
C-----------------------------------------------------------------------
C
C                            *** EIGCHER ***
C        AUTHOR C.H. KERR                                  SEPT. 1974
C
C        THIS SUBROUTINE IS AN INTERFACE TO EISPACK ROUTINES TO
C        COMPUTE EIGENVALUES( AND OPTIONALLY, EIGENVECTORS ) OF ANY
C        COMPLEX HERMITIAN MATRIX
C
C        I.E. A MATRIX A OF COMPLEX ELEMENTS SUCH THAT
C             A = CONJUGATE ( A TRANSPOSE )
C
C        THE SEQUENCE OF OPERATIONS IS AS SUGGESTED IN THE EISPACK
C        FLOWCHARTS, P. 1.2
C
C        ON INPUT -
C
C             N         INTEGER; ROW DIMENSION OF ARRAYS AR, AI,
C                       EIGVECR, EIGVECI, AS DECLARED IN THE CALLING
C                       PROGRAM DIMENSION STATEMENT
C
C             IORDER    INTEGER; ORDER OF LEADING SUBMATRIX OF INPUT MAT
C                       FOR WHICH THE EIGENSYSTEM IS DESIRED.
C                       IORDER MUST BE NO GREATER THAN N
C
C             AR, AI    REAL 2-DIMENSIONAL ARRAYS WITH ROW DIMENSION
C                       N AND COLUMN DIMENSION AT LEAST IORDER. THEY
C                       CONTAIN THE REAL AND IMAGINARY PARTS,
C                       RESPECTIVELY, OF THE COMPLEX HERMITIAN MATRIX
C                       WHOSE EIGENSYSTEM IS DESIRED. ONLY THE MAIN
C                       DIAGONAL AND LOWER TRIANGLE NEED BE USER
C                       DEFINED. EISPACK ROUTINES DESTROY THE CONTENTS
C                       OF THESE ARRAYS.
C
C             NVECS     INTEGER FLAG TO INDICATE WHETHER EIGENVECTORS
C                       ARE ALSO DESIRED.
C
C                       = 0  IF ONLY EIGENVALUES ARE TO BE FOUND
C                       " 0  IF EIGENVECTORS CORRESPONDING TO ALL
C                            EIGENVALUES ARE ALSO TO BE CALCULATED.
C
C             WORKSTG   REAL 1-DIMENSIONAL ARRAY OF SIZE 3 * N.
C                       NEEDED FOR WORKING STORAGE BY EISPACK
C                       CALCULATION ROUTINES.
C
C        ON OUTPUT -
C
C             NUM       INTEGER; NUMBER OF EIGENVALUES (AND EIGENVECTORS
C                       THAT WERE CALCULATED.
C
C                       IF 0 < NUM < IORDER THE (NUM + 1)ST EIGENVALUE W
C                                           OBTAINED AFTER 30 ITERATIONS
C                                           THOSE EIGENVALUES THAT WERE
C                                           CALCULATED, WILL BE IN ASCEN
C                                           ORDER IN 1ST NUM POSITIONS O
C                                           EIGVAL.
C
C                                      NOTE:THEY ARE NOT NECESSARILY THE
C                                           SMALLEST NUM EIGENVALUES.
C
C                                           SIMILARLY, IF REQUESTED, THE
C                                           EIGENVECTORS WILL BE FOUND I
C                                           1ST NUM COLUMNS OF EIGVECR,E
C
C                       IF NUM = IORDER     EIGENVALUES OF (AR, AI) WILL
C                                           BE FOUND IN ASCENDING ORDER
C                                           IN EIGVAL.
C
C             EIGVAL    REAL 1-DIMENSIONAL ARRAY OF AT LEAST IORDER ELEM
C                       CONTAINING THE EIGENVALUES OF THE COMPLEX
C                       HERMITIAN MATRIX STORED IN AR AND AI.
C
C             EIGVECR,  REAL 2-DIMENSIONAL ARRAYS WITH ROW DIMENSION N
C             EIGVECI   AND COLUMN DIMENSION AT LEAST IORDER. THEY
C                       CONTAIN THE REAL AND IMAGINARY PARTS,
C                       RESPECTIVELY, OF THE ORTHONORMAL EIGENVECTORS
C                       OF THE SYSTEM.
C                       I.E. THE JTH COLUMNS OF (EIGVECR, EIGVECI)
C                            CONTAIN THE EIGENVECTOR OF LENGTH 1
C                            CORRESPONDING TO THE EIGENVALUE IN THE JTH
C                            POSITION OF EIGVAL.
C
C-----------------------------------------------------------------------
C
C
      DIMENSION AR(N, IORDER), AI(N, IORDER), EIGVAL(IORDER),
     +          EIGVECR(N, IORDER), EIGVECI(N, IORDER), WORKSTG(1)
C
C
C
C-----   FIND ALL EIGENVALUES, BUT NO EIGENVECTORS. 1ST REDUCE
C-----   COMPLEX HERMITIAN MATRIX TO A REAL SYMMETRIC TRIDIAGONAL ONE
C-----   ( HTRIDI ), THEN FIND THE EIGENVALUES (IMTQL1 )
C
*CALL  STATF
      NUM = IORDER
      CALL HTRIDI( N, IORDER, AR, AI, EIGVAL, WORKSTG(1), WORKSTG(1),
     +             WORKSTG(N + 1) )
      IF( NVECS .NE. 0 ) GO TO 100
      CALL IMTQL1( IORDER, EIGVAL, WORKSTG(1), IERR )
      IF( IERR .EQ. 0 ) RETURN
      NUM = IERR - 1
*IF -DEF,AELIB5
      WRITE 250, NUM
*ENDIF
*IF DEF,AELIB5
      PRINT 250, NUM
*ENDIF
      RETURN
C
C-----   FIND ALL EIGENVALUES AND EIGENVECTORS.
C-----   INITIALIZE EIGVEC TO IDENTITY MATRIX. THEN CONVERT COMPLEX
C-----   HERMITIAN MATRIX TO A REAL SYMMETRIC TRIDIAGONAL ONE ( HTRIDI )
C-----   FIND EIGENVALUES AND VECTORS ( IMTQL2 ), AND BACK-TRANSFORM
C-----   THE EIGENVECTORS FROM IMTQL2 INTO THOSE OF THE ORIGINAL
C-----   COMPLEX HERMITIAN MATRIX
C
 100  CALL PRESET( EIGVECR(1, 1), N* IORDER, 0.0 )
      DO 110 I = 1, IORDER
         EIGVECR(I, I) = 1.0
 110  CONTINUE
      CALL IMTQL2( N, IORDER, EIGVAL, WORKSTG(1), EIGVECR, IERR )
      IF( IERR .EQ. 0 ) GO TO 150
      NUM = IERR - 1
*IF -DEF,AELIB5
      WRITE 275, NUM
*ENDIF
*IF DEF,AELIB5
      PRINT 275, NUM
*ENDIF
 150  CALL HTRIBK( N, IORDER, AR, AI, WORKSTG(N + 1), NUM,
     +             EIGVECR, EIGVECI )
      IF( NUM .LT. IORDER ) CALL SORT( N, IORDER, NUM, .TRUE., .FALSE.,
     +                                 EIGVAL, WORKSTG, WORKSTG,
     +                                 EIGVECR, EIGVECI )
      RETURN
C
C
 250  FORMAT( 35H *** WARNING FROM EIGCHER *** ONLY , I5,
     +        23H EIGENVALUES WERE FOUND  )
C
 275  FORMAT( 35H *** WARNING FROM EIGCHER *** ONLY , I5,
     +        35H EIGENVALUES AND VECTORS WERE FOUND  )
C
      END
      SUBROUTINE HTRIDI(NM,N,AR,AI,D,E,E2,TAU)
      REAL AR(NM,N),AI(NM,N),D(N),E(N),E2(N),TAU(2,N)
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE TRED1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A COMPLEX HERMITIAN MATRIX
C     TO A REAL SYMMETRIC TRIDIAGONAL MATRIX USING
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX HERMITIAN INPUT MATRIX.
C          ONLY THE LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.
C
C     ON OUTPUT-
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION IN THEIR FULL LOWER
C          TRIANGLES.  THEIR STRICT UPPER TRIANGLES AND THE
C          DIAGONAL OF AR ARE UNALTERED,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE THE TRIDIAGONAL MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO,
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED,
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C     ARITHMETIC IS REAL EXCEPT FOR THE USE OF THE SUBROUTINES
C     CABS AND CMPLX IN COMPUTING COMPLEX ABSOLUTE VALUES.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
      TAU(1,N) = 1.0
      TAU(2,N) = 0.0
      DO 100 I = 1, N
  100 D(I) = AR(I,I)
C     ********** FOR I=N STEP -1 UNTIL 1 DO -- **********
      DO  300 II = 1, N
         I = N + 1 - II
         L = I - 1
         H = 0.0
         SCALE = 0.0
         IF (L .LT. 1) GO TO 130
C     ********** SCALE ROW (ALGOL TOL THEN NOT NEEDED) **********
         DO 120 K = 1, L
  120    SCALE = SCALE + ABS(AR(I,K)) + ABS(AI(I,K))
C
         IF (SCALE .NE. 0.0) GO TO 140
         TAU(1,L) = 1.0
         TAU(2,L) = 0.0
  130    E(I) = 0.0
         E2(I) = 0.0
         GO TO 290
C
  140    DO 150 K = 1, L
            AR(I,K) = AR(I,K) / SCALE
            AI(I,K) = AI(I,K) / SCALE
            H = H + AR(I,K) * AR(I,K) + AI(I,K) * AI(I,K)
  150    CONTINUE
C
         E2(I) = SCALE * SCALE * H
         G = SQRT(H)
         E(I) = SCALE * G
         F = CABS(CMPLX(AR(I,L),AI(I,L)))
C     ********** FORM NEXT DIAGONAL ELEMENT OF MATRIX T **********
         IF (F .EQ. 0.0) GO TO 160
         TAU(1,L) = (AI(I,L) * TAU(2,I) - AR(I,L) * TAU(1,I)) / F
         SI = (AR(I,L) * TAU(2,I) + AI(I,L) * TAU(1,I)) / F
         H = H + F * G
         G = 1.0 + G / F
         AR(I,L) = G * AR(I,L)
         AI(I,L) = G * AI(I,L)
         IF (L .EQ. 1) GO TO 270
         GO TO 170
  160    TAU(1,L) = -TAU(1,I)
         SI = TAU(2,I)
         AR(I,L) = G
  170    F = 0.0
C
         DO 240 J = 1, L
            G = 0.0
            GI = 0.0
C     ********** FORM ELEMENT OF A*U **********
            DO 180 K = 1, J
               G = G + AR(J,K) * AR(I,K) + AI(J,K) * AI(I,K)
               GI = GI - AR(J,K) * AI(I,K) + AI(J,K) * AR(I,K)
  180       CONTINUE
C
            JP1 = J + 1
            IF (L .LT. JP1) GO TO 220
            DO 200 K = JP1, L
               G = G + AR(K,J) * AR(I,K) - AI(K,J) * AI(I,K)
               GI = GI - AR(K,J) * AI(I,K) - AI(K,J) * AR(I,K)
  200       CONTINUE
C     ********** FORM ELEMENT OF P **********
  220       E(J) = G / H
            TAU(2,J) = GI / H
            F = F + E(J) * AR(I,J) - TAU(2,J) * AI(I,J)
  240    CONTINUE
C
         HH = F / (H + H)
C     ********** FORM REDUCED A **********
         DO 260 J = 1, L
            F = AR(I,J)
            G = E(J) - HH * F
            E(J) = G
            FI = -AI(I,J)
            GI = TAU(2,J) - HH * FI
            TAU(2,J) = -GI
            DO 260 K = 1, J
               AR(J,K) = AR(J,K) - F * E(K) - G * AR(I,K)
     X                           + FI * TAU(2,K) + GI * AI(I,K)
               AI(J,K) = AI(J,K) - F * TAU(2,K) - G * AI(I,K)
     X                           - FI * E(K) - GI * AR(I,K)
  260    CONTINUE
  270    DO 280 K = 1, L
            AR(I,K) = SCALE * AR(I,K)
            AI(I,K) = SCALE * AI(I,K)
  280    CONTINUE
         TAU(2,L) = -SI
  290    HH = D(I)
         D(I) = AR(I,I)
         AR(I,I) = HH
         AI(I,I) = SCALE * SCALE * H
  300 CONTINUE
      RETURN
      END
      SUBROUTINE HTRIBK(NM,N,AR,AI,TAU,M,ZR,ZI)
      REAL AR(NM,N),AI(NM,N),TAU(2,N),ZR(NM,M),ZI(NM,M)
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE TRBAK1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A COMPLEX HERMITIAN
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     REAL SYMMETRIC TRIDIAGONAL MATRIX DETERMINED BY  HTRIDI.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION BY  HTRIDI  IN THEIR
C          FULL LOWER TRIANGLES EXCEPT FOR THE DIAGONAL OF AR.
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS,
C
C        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED,
C
C        ZR CONTAINS THE EIGENVECTORS TO BE BACK TRANSFORMED
C          IN ITS FIRST M COLUMNS.
C
C     ON OUTPUT-
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE TRANSFORMED EIGENVECTORS
C          IN THEIR FIRST M COLUMNS.
C
C     NOTE THAT THE LAST COMPONENT OF EACH RETURNED VECTOR
C     IS REAL AND THAT VECTOR EUCLIDEAN NORMS ARE PRESERVED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C
C     ********** TRANSFORM THE EIGENVECTORS OF THE REAL SYMMETRIC
C                TRIDIAGONAL MATRIX TO THOSE OF THE HERMITIAN
C                TRIDIAGONAL MATRIX. **********
      DO 50 K = 1, N
         DO 50 J = 1, M
            ZI(K,J) = - ZR(K,J) * TAU(2,K)
            ZR(K,J) = ZR(K,J) * TAU(1,K)
   50 CONTINUE
      IF (N .EQ. 1) GO TO 200
C     ********** RECOVER AND APPLY THE HOUSEHOLDER MATRICES **********
      DO 140 I = 2, N
         L = I - 1
         H = AI(I,I)
         IF (H .EQ. 0.0) GO TO 140
         DO 130 J = 1, M
            S = 0.0
            SI = 0.0
            DO 110 K = 1, L
               S = S + AR(I,K) * ZR(K,J) - AI(I,K) * ZI(K,J)
               SI = SI + AR(I,K) * ZI(K,J) + AI(I,K) * ZR(K,J)
  110       CONTINUE
            S = S / H
            SI = SI / H
            DO 120 K = 1, L
               ZR(K,J) = ZR(K,J) - S * AR(I,K) - SI * AI(I,K)
               ZI(K,J) = ZI(K,J) - SI * AR(I,K) + S * AI(I,K)
  120       CONTINUE
  130    CONTINUE
  140 CONTINUE
  200 RETURN
      END
