*DECK EIGRSYM
      SUBROUTINE EIGRSYM( N, IORDER, AR, NVECS, WORKSTG, NUM,
     +                    EIGVAL, EIGVEC )
C
************************************************************************
**                                                                    **
**    NOTE THAT EIGRSYM IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL*
**    ROUTINES:                                                       **
**       IMTQL1                                                       **
**       IMTQL2                                                       **
**       SORT                                                         **
**       TRED1                                                        **
**       TRED2                                                        **
**       TRIDIAG                                                      **
**    IMTQL1 THROUGH TRIDIAG ACCOMPANY EIGRSYM ON AELIB SOURCE TAPE   **
**    UNDER THE DECK NAME EIGRSYM.                                    **
**                                                                    **
************************************************************************
C
C-----------------------------------------------------------------------
C
C                            *** EIGRSYM ***
C
C        AUTHOR C.H. KERR                                  SEPT. 1974
C
C        THIS SUBROUTINE IS AN INTERFACE TO EISPACK ROUTINES TO
C        COMPUTE EIGENVALUES ( AND OPTIONALLY, EIGENVECTORS ) OF
C        ANY REAL SYMMETRIC MATRIX.  THE SEQUENCE OF OPERATIONS IS
C        AS SUGGESTED IN THE EISPACK FLOW CHARTS, PP. 3.1 AND 3.2
C
C        ON INPUT-
C
C             N         INTEGER; ROW DIMENSION OF ARRAYS AR AND
C                       EIGVEC, AS DECLARED IN THE CALLING PROGRAM
C                       DIMENSION STATEMENT.
C
C             IORDER    INTEGER; ORDER OF THE LEADING SUBARRAY OF AR FOR
C                       EIGENSYSTEM IS DESIRED. IORDER MUST BE NO
C                       GREATER THAN N.
C
C             AR        A REAL ARRAY WITH EXACTLY N ROWS, AND AT
C                       LEAST IORDER COLUMNS. THE LEADING IORDER
C                       BY IORDER SUBARRAY MUST CONTAIN THE REAL
C                       SYMMETRIC MATRIX WHOSE EIGENSYSTEM IS TO BE
C                       COMPUTED. ONLY THE MAIN DIAGONAL AND LOWER
C                       TRIANGLE NEED BE DEFINED. CONTENTS OF AR ARE
C                       DESTROYED BY EISPACK ROUTINES.
C
C             NVECS     INTEGER FLAG TO INDICATE WHETHER OR NOT
C                       EIGENVECTORS ARE ALSO DESIRED.
C
C                       = 0  IF ONLY EIGENVALUES ARE TO BE FOUND
C                       " 0  IF EIGENVECTORS CORRESPONDING TO ALL
C                            EIGENVALUES SHOULD ALSO BE CALCULATED.
C
C             WORKSTG   REAL 1 - DIMENSIONAL ARRAY OF SIZE N NEEDED
C                       FOR WORKING STORAGE BY EISPACK CALCULATION
C                       ROUTINES
C
C        ON OUTPUT -
C
C             NUM       INTEGER; NUMBER OF EIGENVALUES (AND EIGENVECTORS
C                       THAT WERE CALCULATED.
C
C                       IF 0 < NUM < IORDER THE (NUM + 1)ST EIGENVALUE
C                                           WAS NOT OBTAINED AFTER 30
C                                           ITERATIONS.
C                                           THOSE EIGENVALUES THAT
C                                           WERE CALCULATED WILL BE
C                                           FOUND IN ASCENDING ORDER IN
C                                           1ST NUM POSITIONS OF EIGVAL.
C                                           SIMILARLY, IF REQUESTED, THE
C                                           1ST NUM EIGENVECTORS WILL
C                                           BE FOUND IN THE 1ST NUM
C                                           COLUMNS OF EIGVEC.
C
C                       IF NUM = IORDER     THE EIGENVALUES OF AR
C                                           WILL BE FOUND IN ASCENDING
C                                           ORDER IN EIGVAL.
C
C             EIGVAL    REAL 1-DIMENSIONAL ARRAY OF AT LEAST IORDER
C                       ELEMENTS, CONTAINING THE EIGENVALUES OF AR.
C
C             EIGVEC    REAL 2-DIMENSIONAL ARRAY WITH EXACTLY N ROWS AND
C                       AT LEAST IORDER COLUMNS. IT CONTAINS AN ORTHONOR
C                       SET OF EIGENVECTORS OF AR, STORED BY COLUMN IN 1
C                       IORDER X IORDER LEADING SUBARRAY OF EIGVEC.
C                       I.E. THE JTH COLUMN OF EIGVEC CONTAINS AN
C                            EIGENVECTOR OF LENGTH ONE CORRESPONDING
C                            TO THE EIGENVALUE IN THE JTH ELEMENT OF EIG
C
C-----------------------------------------------------------------------
C
C
      DIMENSION AR(N, N), EIGVAL( IORDER ), EIGVEC(N, IORDER ) ,
     +          WORKSTG( N )
      LOGICAL TRIDIAG
C
C
*CALL  STATF
      NUM = IORDER
*IF -DEF,AELIB5
      IF( TRIDIAG( N, IORDER, AR, .TRUE. ) ) GO TO 500
*ENDIF
*IF DEF,AELIB5
      IF( TRIDIAG( N, IORDER, AR(1,1), .TRUE. ) ) GO TO 500
*ENDIF
      IF( NVECS .NE. 0 ) GO TO 200
C
C-----   FIND ALL EIGENVALUES AND NO EIGENVECTORS
C-----        TRED1 CONVERTS SUBMATRIX OF AR TO A SYMMETRIC
C-----              TRIDIAGONAL ONE.
C-----        IMTQL1 FINDS EIGENVALUES OF TRIDIAGONAL MATRIX
C
      CALL TRED1( N, IORDER, AR, EIGVAL, WORKSTG(1), WORKSTG(1) )
      CALL IMTQL1( IORDER, EIGVAL, WORKSTG(1), IERR )
 105  IF( IERR .EQ. 0 ) RETURN
      NUM = IERR - 1
*IF -DEF,AELIB5
      WRITE 910, NUM
*ENDIF
*IF DEF,AELIB5
      PRINT 910, NUM
*ENDIF
      RETURN
C
C-----   FIND ALL EIGENVALUES AND EIGENVECTORS
C-----        TRED2 WILL CONVERT AR TO A TRIDIAGONAL MATRIX
C-----        IMTQL2 WILL FIND ALL EIGENVALUES AND CORRESPONDING
C-----               EIGENVECTORS
C
 200  CALL TRED2( N, IORDER, AR, EIGVAL, WORKSTG(1), EIGVEC )
      CALL IMTQL2( N, IORDER, EIGVAL, WORKSTG(1), EIGVEC, IERR )
 205  IF( IERR .EQ. 0 ) RETURN
      NUM = IERR - 1
*IF -DEF,AELIB5
      WRITE 920, NUM
*ENDIF
*IF DEF,AELIB5
      PRINT 920, NUM
*ENDIF
      CALL SORT( N, IORDER, NUM, .TRUE., .FALSE., EIGVAL, WORKSTG,
     +           WORKSTG, EIGVEC, WORKSTG )
      RETURN
C
C
C
C-----   INPUT MATRIX IS ALREADY TRIDIAGONAL.
C-----   SAVE DIAGONAL ELEMENTS OF AR IN ITS 1ST COLUMN , AND
C-----   SUBDIAGONAL ELEMENTS IN WORKSTG FOR EISPACK ROUTINES.
C
  500 DO 525 I = 2, IORDER
         WORKSTG(I) = AR(I, I - 1 )
         AR(I, 1) = AR(I, I)
  525 CONTINUE
      IF( NVECS .NE. 0 ) GO TO 600
C
C-----   FIND ONLY THE EIGENVALUES
C-----        NOTE: ITH EIGENVALUE IS RETURNED BY EISPACK IN AR(I, 1)
C
      CALL IMTQL1( IORDER, AR(1, 1), WORKSTG(1), IERR )
      DO 550 I = 1, IORDER
         EIGVAL(I) = AR(I, 1)
 550  CONTINUE
      GO TO 105
C
C-----   FIND ALL EIGENVALUES AND EIGENVECTORS
C-----      INITIALIZE EIGVEC TO IDENTITY MATRIX FOR IMTQL2, THEN
C-----      COMPUTE EIGENVALUES AND EIGENVALUES.
C-----        NOTE: ITH EIGENVALUE IS RETURNED BY EISPACK IN AR(I, 1)
C
 600  CALL PRESET( EIGVEC(1, 1), N * IORDER, 0.0 )
      DO 610 I = 1, IORDER
         EIGVEC(I, I) = 1.0
 610  CONTINUE
      CALL IMTQL2( N, IORDER, AR(1, 1), WORKSTG(1), EIGVEC, IERR )
      DO 625 I = 1, IORDER
         EIGVAL(I) = AR(I, 1)
 625  CONTINUE
      GO TO 205
C
C
 910  FORMAT( 35H *** WARNING FROM EIGRSYM *** ONLY , I5,
     +        24H EIGENVALUES WERE FOUND. )
C
 920  FORMAT( 35H *** WARNING FROM EIGRSYM *** ONLY , I5,
     +        36H EIGENVALUES AND VECTORS WERE FOUND. )
C
      END
      LOGICAL FUNCTION TRIDIAG( N, IORDER, ARRAY, SYMMET )
C
C
C-----------------------------------------------------------------------
C
C                            *** TRIDIAG ***
C        AUTHOR C.H. KERR                                  SEPT. 1974
C
C        THIS FUNCTION DETERMINES WHETHER OR NOT AN ARRAY IS
C        TRIDIAGONAL I.E. ALL ELEMENTS OFF THE 3 MAIN DIAGONALS
C        ( SUPERDIAGONAL, DIAGONAL, AND SUBDIAGONAL ) ARE ZERO.
C        TO BE CLASSIFIED AS TRIDIAGONAL, NON-SYMMETRIC ARRAYS MUST
C        SATISFY THE ADDITIONAL PROPERTY THAT ALL PRODUCTS OF
C        CORRESPONDING PAIRS OF SUPER AND SUBDIAGONAL ELEMENTS ARE
C        NON-NEGATIVE, AND NEITHER OR BOTH IN EACH PAIR ARE ZERO.
C
C        ON INPUT -
C
C             N         INTEGER;ROW DIMENSION OF ARRAY, AS SPECIFIED IN
C                       CALLING PROGRAM DIMENSION STATEMENT.
C
C             IORDER    INTEGER; ORDER OF SUBMATRIX OF ARRAY TO BE CHECK
C                       FOR TRIDIAGONALITY
C
C             ARRAY     2-DIMENSIONAL N X N ARRAY BEING CHECKED FOR
C                       TRIDIAGONALITY.
C
C             SYMMET    LOGICAL FLAG. .TRUE. IF ARRAY IS SYMMETRIC
C                                     .FALSE. OTHERWISE
C
C        ON OUTPUT -
C
C             TRIDIAG   IS .TRUE. IF ARRAY IS TRIDIAGONAL
C                          .FALSE. OTHERWISE
C
C-----------------------------------------------------------------------
C
C
      DIMENSION ARRAY(N, IORDER)
      LOGICAL SYMMET
C
C
      TRIDIAG = .FALSE.
      IF( IORDER - 2 ) 225, 20, 25
*IF -DEF,AELIB5
  20  IF( SYMMET ) 225, 175
*ENDIF
*IF DEF,AELIB5
  20  IF( SYMMET ) GOTO 225
      GOTO 175
*ENDIF
C
C-----   CHECK THAT ALL ENTRIES UNDER SUBDIAGONAL ARE ZERO
C
  25  DO 50 I = 3, IORDER
         IQUIT = I - 2
         DO 30 J = 1, IQUIT
            IF( ARRAY(I, J) .NE. 0.0 ) RETURN
  30     CONTINUE
  50  CONTINUE
      IF( SYMMET ) GO TO 225
C
C-----   CHECK THAT ALL ENTRIES ABOVE SUPERDIAGONAL ARE ZERO
C-----     NOTE: IF MATRIX IS SYMMETRIC AND LOWER TRIANGLE IS ALL
C-----           ZERO, NO NEED TO PERFORM THIS LOOP
C
      IQUIT = IORDER - 2
      DO 150 I = 1, IQUIT
         JBEG = I + 2
         DO 130 J = JBEG, IORDER
            IF( ARRAY(I, J) .NE. 0.0 ) RETURN
 130     CONTINUE
 150  CONTINUE
C
C-----   IF MATRIX IS NON-SYMMETRIC, CHECK THAT ALL PRODUCTS OF
C-----   CORRESPONDING PAIRS OF SUPER AND SUB-DIAGONAL ELEMENTS
C-----   ARE NON-NEGATIVE. IF ANY PRODUCT IS ZERO, BOTH MULTIPLICANDS
C-----   MUST BE ZERO.
C
  175 IQUIT = IORDER - 1
      DO 200 I = 1, IQUIT
         J = I + 1
         IF( ARRAY(I, J) * ARRAY(J, I) ) 230, 190, 200
 190     IF( .NOT. ( ( ARRAY(I, J) .EQ. 0.0 ) .AND.
     +               ( ARRAY(J, I) .EQ. 0.0 ) ) ) RETURN
 200  CONTINUE
 225  TRIDIAG = .TRUE.
 230  RETURN
      END
      SUBROUTINE SORT( N, IORDER, NUM, REALEVL, NOVECS, SORTIT,
     +                 EIGVALR, EIGVALI, EIGVECR, EIGVECI )
C
C
C-----------------------------------------------------------------------
C
C                            *** SORT ***
C        AUTHOR C.H. KERR                                  NOV. 1974
C
C        THIS SUBROUTINE PUTS THE ARRAY SORTIT INTO ASCENDING ORDER. IT
C        USED TO ORDER REAL OR COMPLEX EIGENVALUES (AND, OPTIONALLY,
C        PERMUTE THEIR CORRESPONDING EIGENVECTORS SIMILARLY.)
C
C        METHOD USED TO SORT A LIST OF LENGTH N IS AS FOLLOWS:
C             SEARCH ENTIRE LIST FOR SMALLEST NUMBER, AND PLACE IT IN 1S
C             POSITION OF SORTIT.
C                            . . .
C             TO FIND KTH SMALLEST ELEMENT, SEARCH REMAINDER OF LIST
C             ( FROM K + 1, ..., N ); THEN PLACE MINIMUM OBTAINED IN
C             KTH POSITION.
C
C        ON INPUT-
C
C             N         INTEGER; MAXIMUM ROW DIMENSION OF EIGVECR AND EI
C                       AS SPECIFIED IN CALLING PROGRAM DIMENSION STATEM
C
C             IORDER    INTEGER; ORDER OF SUBARRAY OF INPUT MATRIX FOR W
C                       EIGENSYSTEM HAS BEEN COMPUTED.
C
C             NUM       INTEGER; LENGTH OF LIST TO BE SORTED.
C
C             REALEVL   LOGICAL FLAG;  .TRUE.    TO SORT REAL EIGENVALUE
C                                      .FALSE.   TO SORT COMPLEX EIGENVA
C
C             NOVECS    LOGICAL FLAG;  .TRUE.    TO SORT ONLY EIGENVALUE
C                                      .FALSE.   TO REORDER EIGENVECTORS
C
C             SORTIT    ON INPUT - REAL ARRAY OF VALUES TO BE SORTED.
C                       IF EIGENVALUES TO BE SORTED ARE:
C
C                            REAL      SORTIT SHOULD BE SPECIFIED AS EIG
C                                      IN CALL TO SORT.
C
C                            COMPLEX   SORTIT SHOULD CONTAIN THEIR MODUL
C                                      IN THIS CASE, EIGVALR AND EIGVALI
C                                      AUTOMATICALLY PERMUTED IN SAME WA
C
C        ON OUTPUT-
C
C             EIGVALR   REAL ARRAYS CONTAINING REAL AND IMAGINARY PARTS,
C             EIGVALI   RESPECTIVELY, OF EIGENVALUES, SORTED INTO ASCEND
C                       ORDER BY: ACTUAL VALUE, IF REALEVL = .TRUE.
C                                 MODULI, IF REALEVL = .FALSE.
C
C             EIGVECR   REAL ARRAYS CONTAINING REAL AND IMAGINARY PARTS,
C             EIGVECI   RESPECTIVELY, OF EIGENVECTORS CORRESPONDING TO
C                       SORTED EIGENVALUES.
C
C-----------------------------------------------------------------------
C
C
      LOGICAL NOVECS, REALEVL
      DIMENSION EIGVALR(NUM), EIGVALI(NUM), EIGVECR(N, NUM),
     +          EIGVECI(N, NUM), SORTIT(NUM)
      DATA EPS / 1.0E-14 /
C
C
      NUMMIN1 = NUM - 1
      DO 200 K = 1, NUMMIN1
C
C--- FIND MINIMUM OF AS YET UNSORTED SUBLIST
C
         LOCMIN = K
         KPLUS1 = K + 1
         DO 150 J = KPLUS1, NUM
            IF( ( SORTIT(LOCMIN) - SORTIT(J) ) .GT. EPS ) LOCMIN = J
 150     CONTINUE
C
C--- PUT MINIMUM FROM THIS PASS INTO KTH POSITION OF EIGENVALUE ARRAYS
C--- ( AND INTERCHANGE EIGENVECTORS AS WELL, IF NECESSARY )
C
         TEMP = SORTIT(K)
         SORTIT(K) = SORTIT(LOCMIN)
         SORTIT(LOCMIN) = TEMP
         IF( REALEVL ) GO TO 160
C
         TEMP = EIGVALR(K)
         EIGVALR(K) = EIGVALR(LOCMIN)
         EIGVALR(LOCMIN) = TEMP
C
         TEMP = EIGVALI(K)
         EIGVALI(K) = EIGVALI(LOCMIN)
         EIGVALI(LOCMIN) = TEMP
 160     IF( NOVECS ) GO TO 200
C
         DO 175 J = 1, IORDER
            TEMP = EIGVECR(J, K)
            EIGVECR(J, K) = EIGVECR(J, LOCMIN)
            EIGVECR(J, LOCMIN) = TEMP
            IF( REALEVL ) GO TO 175
C
            TEMP = EIGVECI(J, K)
            EIGVECI(J, K) = EIGVECI(J, LOCMIN)
            EIGVECI(J, LOCMIN) = TEMP
 175     CONTINUE
 200  CONTINUE
      RETURN
      END
      SUBROUTINE TRED1(NM,N,A,D,E,E2)
      DIMENSION A(NM,1),D(1),E(1),E2(1)
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRED1,
C     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A REAL SYMMETRIC MATRIX
C     TO A SYMMETRIC TRIDIAGONAL MATRIX USING
C     ORTHOGONAL SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        A CONTAINS THE REAL SYMMETRIC INPUT MATRIX.  ONLY THE
C          LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.
C
C     ON OUTPUT-
C
C        A CONTAINS INFORMATION ABOUT THE ORTHOGONAL TRANS-
C          FORMATIONS USED IN THE REDUCTION IN ITS STRICT LOWER
C          TRIANGLE.  THE FULL UPPER TRIANGLE OF A IS UNALTERED,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO,
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
      DO 100 I = 1, N
  100 D(I) = A(I,I)
C     ********** FOR I=N STEP -1 UNTIL 1 DO -- **********
      DO  300 II = 1, N
         I = N + 1 - II
         L = I - 1
         H = 0.0
         SCALE = 0.0
         IF (L .LT. 1) GO TO 130
C     ********** SCALE ROW (ALGOL TOL THEN NOT NEEDED) **********
         DO 120 K = 1, L
  120    SCALE = SCALE + ABS(A(I,K))
C
         IF (SCALE .NE. 0.0) GO TO 140
  130    E(I) = 0.0
         E2(I) = 0.0
         GO TO 290
C
  140    DO 150 K = 1, L
            A(I,K) = A(I,K) / SCALE
            H = H + A(I,K) * A(I,K)
  150    CONTINUE
C
         E2(I) = SCALE * SCALE * H
         F = A(I,L)
         G = -SIGN(SQRT(H),F)
         E(I) = SCALE * G
         H = H - F * G
         A(I,L) = F - G
         IF (L .EQ. 1) GO TO 270
         F = 0.0
         DO 240 J = 1, L
            G = 0.0
C     ********** FORM ELEMENT OF A*U **********
            DO 180 K = 1, J
  180       G = G + A(J,K) * A(I,K)
            JP1 = J + 1
            IF (L .LT. JP1) GO TO 220
            DO 200 K = JP1, L
  200       G = G + A(K,J) * A(I,K)
C     ********** FORM ELEMENT OF P **********
  220       E(J) = G / H
            F = F + E(J) * A(I,J)
  240    CONTINUE
         H = F / (H + H)
C     ********** FORM REDUCED A **********
         DO 260 J = 1, L
            F = A(I,J)
            G = E(J) - H * F
            E(J) = G
            DO 260 K = 1, J
               A(J,K) = A(J,K) - F * E(K) - G * A(I,K)
  260    CONTINUE
  270    DO 280 K = 1, L
  280    A(I,K) = SCALE * A(I,K)
  290    H = D(I)
         D(I) = A(I,I)
         A(I,I) = H
  300 CONTINUE
      RETURN
      END
      SUBROUTINE TRED2(NM,N,A,D,E,Z)
      DIMENSION A(NM,1),Z(NM,1),D(1),E(1)
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRED2,
C     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A REAL SYMMETRIC MATRIX TO A
C     SYMMETRIC TRIDIAGONAL MATRIX USING AND ACCUMULATING
C     ORTHOGONAL SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        A CONTAINS THE REAL SYMMETRIC INPUT MATRIX.  ONLY THE
C          LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.
C
C     ON OUTPUT-
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO,
C
C        Z CONTAINS THE ORTHOGONAL TRANSFORMATION MATRIX
C          PRODUCED IN THE REDUCTION,
C
C        A AND Z MAY COINCIDE.  IF DISTINCT, A IS UNALTERED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
      DO 100 I = 1, N
         DO 100 J = 1, I
            Z(I,J) = A(I,J)
  100 CONTINUE
C
      IF (N .EQ. 1) GO TO 320
C     ********** FOR I=N STEP -1 UNTIL 2 DO -- **********
      DO 300 II = 2, N
         I = N + 2 - II
         L = I - 1
         H = 0.0
         SCALE = 0.0
         IF (L .LT. 2) GO TO 130
C     ********** SCALE ROW (ALGOL TOL THEN NOT NEEDED) **********
         DO 120 K = 1, L
  120    SCALE = SCALE + ABS(Z(I,K))
C
         IF (SCALE .NE. 0.0) GO TO 140
  130    E(I) = Z(I,L)
         GO TO 290
C
  140    DO 150 K = 1, L
            Z(I,K) = Z(I,K) / SCALE
            H = H + Z(I,K) * Z(I,K)
  150    CONTINUE
C
         F = Z(I,L)
         G = -SIGN(SQRT(H),F)
         E(I) = SCALE * G
         H = H - F * G
         Z(I,L) = F - G
         F = 0.0
C
         DO 240 J = 1, L
            Z(J,I) = Z(I,J) / (SCALE * H)
            G = 0.0
C     ********** FORM ELEMENT OF A*U **********
            DO 180 K = 1, J
  180       G = G + Z(J,K) * Z(I,K)
C
            JP1 = J + 1
            IF (L .LT. JP1) GO TO 220
C
            DO 200 K = JP1, L
  200       G = G + Z(K,J) * Z(I,K)
C     ********** FORM ELEMENT OF P **********
  220       E(J) = G / H
            F = F + E(J) * Z(I,J)
  240    CONTINUE
C
         HH = F / (H + H)
C     ********** FORM REDUCED A **********
         DO 260 J = 1, L
            F = Z(I,J)
            G = E(J) - HH * F
            E(J) = G
C
            DO 260 K = 1, J
               Z(J,K) = Z(J,K) - F * E(K) - G * Z(I,K)
  260    CONTINUE
C
         DO 280 K = 1, L
  280    Z(I,K) = SCALE * Z(I,K)
C
  290    D(I) = H
  300 CONTINUE
C
  320 D(1) = 0.0
      E(1) = 0.0
C     ********** ACCUMULATION OF TRANSFORMATION MATRICES **********
      DO 500 I = 1, N
         L = I - 1
         IF (D(I) .EQ. 0.0) GO TO 380
C
         DO 360 J = 1, L
            G = 0.0
C
            DO 340 K = 1, L
  340       G = G + Z(I,K) * Z(K,J)
C
            DO 360 K = 1, L
               Z(K,J) = Z(K,J) - G * Z(K,I)
  360    CONTINUE
  380    D(I) = Z(I,I)
         Z(I,I) = 1.0
         IF (L .LT. 1) GO TO 500
         DO 400 J = 1, L
            Z(I,J) = 0.0
            Z(J,I) = 0.0
  400    CONTINUE
  500 CONTINUE
      RETURN
      END
      SUBROUTINE IMTQL1(N,D,E,IERR)
      REAL D(1),E(1),MACHEP
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE IMTQL1,
C     NUM. MATH. 12, 377-383(1968) BY MARTIN AND WILKINSON,
C     AS MODIFIED IN NUM. MATH. 15, 450(1970) BY DUBRULLE.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 241-248(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES OF A SYMMETRIC
C     TRIDIAGONAL MATRIX BY THE IMPLICIT QL METHOD.
C
C     ON INPUT-
C
C        N IS THE ORDER OF THE MATRIX,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
C          IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
C
C      ON OUTPUT-
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT AND
C          ORDERED FOR INDICES 1,2,...IERR-1, BUT MAY NOT BE
C          THE SMALLEST EIGENVALUES,
C
C        E HAS BEEN DESTROYED,
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C
C     ********** MACHEP IS A MACHINE DEPENDENT PARAMETER SPECIFYING
C                THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
C
      MACHEP = 2.**(-47)
      IERR = 0
      IF (N .EQ. 1) GO TO 1001
      DO 100 I = 2, N
  100 E(I-1) = E(I)
      E(N) = 0.0
      DO 290 L = 1, N
         J = 0
C     ********** LOOK FOR SMALL SUB-DIAGONAL ELEMENT **********
  105    DO 110 M = L, N
            IF (M .EQ. N) GO TO 120
            IF (ABS(E(M)) .LE. MACHEP * (ABS(D(M)) + ABS(D(M+1))))
     X         GO TO 120
  110    CONTINUE
  120    P = D(L)
         IF (M .EQ. L) GO TO 215
         IF (J .EQ. 30) GO TO 1000
         J = J + 1
C     ********** FORM SHIFT **********
         G = (D(L+1) - P) / (2.0 * E(L))
         R = SQRT(G*G+1.0)
         G = D(M) - P + E(L) / (G + SIGN(R,G))
*IF -DEF,AELIB5
      S=C=1.   $   P=0
*ENDIF
*IF DEF,AELIB5
      S=C=1.
      P=0
*ENDIF
         MML = M - L
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
         DO 200 II = 1, MML
            I = M - II
            F = S * E(I)
            B = C * E(I)
            IF (ABS(F) .LT. ABS(G)) GO TO 150
            C = G / F
            R = SQRT(C*C+1.0)
            E(I+1) = F * R
            S = 1.0 / R
            C = C * S
            GO TO 160
  150       S = F / G
            R = SQRT(S*S+1.0)
            E(I+1) = G * R
            C = 1.0 / R
            S = S * C
  160       G = D(I+1) - P
            R = (D(I) - G) * S + 2.0 * C * B
            P = S * R
            D(I+1) = G + P
            G = C * R - B
  200    CONTINUE
         D(L) = D(L) - P
         E(L) = G
         E(M) = 0.0
         GO TO 105
C     ********** ORDER EIGENVALUES **********
  215    IF (L .EQ. 1) GO TO 250
C     ********** FOR I=L STEP -1 UNTIL 2 DO -- **********
         DO 230 II = 2, L
            I = L + 2 - II
            IF (P .GE. D(I-1)) GO TO 270
            D(I) = D(I-1)
  230    CONTINUE
  250    I = 1
  270    D(I) = P
  290 CONTINUE
      GO TO 1001
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
 1000 IERR = L
 1001 RETURN
      END
      SUBROUTINE IMTQL2(NM,N,D,E,Z,IERR)
      REAL D(1),E(1),Z(NM,1),MACHEP
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE IMTQL2,
C     NUM. MATH. 12, 377-383(1968) BY MARTIN AND WILKINSON,
C     AS MODIFIED IN NUM. MATH. 15, 450(1970) BY DUBRULLE.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 241-248(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A SYMMETRIC TRIDIAGONAL MATRIX BY THE IMPLICIT QL METHOD.
C     THE EIGENVECTORS OF A FULL SYMMETRIC MATRIX CAN ALSO
C     BE FOUND IF  TRED2  HAS BEEN USED TO REDUCE THIS
C     FULL MATRIX TO TRIDIAGONAL FORM.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
C          IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY,
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  TRED2, IF PERFORMED.  IF THE EIGENVECTORS
C          OF THE TRIDIAGONAL MATRIX ARE DESIRED, Z MUST CONTAIN
C          THE IDENTITY MATRIX.
C
C      ON OUTPUT-
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT
C          UNORDERED FOR INDICES 1,2,...,IERR-1,
C
C        E HAS BEEN DESTROYED,
C
C        Z CONTAINS ORTHONORMAL EIGENVECTORS OF THE SYMMETRIC
C          TRIDIAGONAL (OR FULL) MATRIX.  IF AN ERROR EXIT IS MADE,
C          Z CONTAINS THE EIGENVECTORS ASSOCIATED WITH THE STORED
C          EIGENVALUES,
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C
C     ********** MACHEP IS A MACHINE DEPENDENT PARAMETER SPECIFYING
C                THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
C
      MACHEP = 2.**(-47)
      IERR = 0
      IF (N .EQ. 1) GO TO 1001
      DO 100 I = 2, N
  100 E(I-1) = E(I)
      E(N) = 0.0
      DO 240 L = 1, N
         J = 0
C     ********** LOOK FOR SMALL SUB-DIAGONAL ELEMENT **********
  105    DO 110 M = L, N
            IF (M .EQ. N) GO TO 120
            IF (ABS(E(M)) .LE. MACHEP * (ABS(D(M)) + ABS(D(M+1))))
     X         GO TO 120
  110    CONTINUE
  120    P = D(L)
         IF (M .EQ. L) GO TO 240
         IF (J .EQ. 30) GO TO 1000
         J = J + 1
C     ********** FORM SHIFT **********
         G = (D(L+1) - P) / (2.0 * E(L))
         R = SQRT(G*G+1.0)
         G = D(M) - P + E(L) / (G + SIGN(R,G))
         S = 1.0
         C = 1.0
         P = 0.0
         MML = M - L
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
         DO 200 II = 1, MML
            I = M - II
            F = S * E(I)
            B = C * E(I)
            IF (ABS(F) .LT. ABS(G)) GO TO 150
            C = G / F
            R = SQRT(C*C+1.0)
            E(I+1) = F * R
            S = 1.0 / R
            C = C * S
            GO TO 160
  150       S = F / G
            R = SQRT(S*S+1.0)
            E(I+1) = G * R
            C = 1.0 / R
            S = S * C
  160       G = D(I+1) - P
            R = (D(I) - G) * S + 2.0 * C * B
            P = S * R
            D(I+1) = G + P
            G = C * R - B
C     ********** FORM VECTOR **********
            DO 180 K = 1, N
               F = Z(K,I+1)
               Z(K,I+1) = S * Z(K,I) + C * F
               Z(K,I) = C * Z(K,I) - S * F
  180       CONTINUE
  200    CONTINUE
         D(L) = D(L) - P
         E(L) = G
         E(M) = 0.0
         GO TO 105
  240 CONTINUE
C     ********** ORDER EIGENVALUES AND EIGENVECTORS **********
      DO 300 II = 2, N
         I = II - 1
         K = I
         P = D(I)
         DO 260 J = II, N
            IF (D(J) .GE. P) GO TO 260
            K = J
            P = D(J)
  260    CONTINUE
         IF (K .EQ. I) GO TO 300
         D(K) = D(I)
         D(I) = P
         DO 280 J = 1, N
            P = Z(J,I)
            Z(J,I) = Z(J,K)
            Z(J,K) = P
  280    CONTINUE
  300 CONTINUE
      GO TO 1001
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
 1000 IERR = L
 1001 RETURN
      END
