*DECK EINCO
      SUBROUTINE EINCO(XM1,PHI,CPHI,IFLAG,F,E,EPS)
C                                            L.YAMAZAKI  FEBRUARY,1974
C A SUBPROGRAM TO COMPUTE THE INCOMPLETE ELLIPTIC INTEGRALS OF THE
C FIRST AND SECOND KINDS.
C REFERENCE: LEE-WHITING,G.E.  FORMULAS FOR COMPUTING INCOMPLETE
C            ELLIPTIC INTEGRALS OF THE FIRST AND SECOND KINDS, JOURNAL
C            OF THE ASSOCIATION FOR COMPUTING MACHINERY, VOL.10,NO.2,
C            APRIL,1963
C
C FORMAL PARAMETERS:
C         XM1=1-M     USED AS THE ARGUMENT INSTEAD OF M.  OTHERWISE ,
C                     FOR M VALUES CLOSE TO 1 TOO MUCH ACCURACY IS LOST.
C         PHI         THE UPPER LIMIT OF INTEGRATION.  USE PHI AS THE
C                     ARGUMENT WHEN PHI @ PI/4.
C         CPHI        COS(PHI) TO BE USED AS THE ARGUMENT WHEN PHI > PI/
C         IFLAG       CONTROL ON  THE PARAMETERS BEING SUPPLIED
C            IFLAG = 1    PHI ONLY
C            IFLAG = 2   CPHI ONLY
C            IFLAG = 3   BOTH PHI AND CPHI SUPPLIED
C         F           INCOMPLETE ELLIPTIC INTEGRAL OF THE FIRST KIND.
C         E           INCOMPLETE ELLIPTIC INTEGRAL OF THE SECOND KIND.
C         EPS         RELATIVE ERROR CRITERION
C
*CALL  STATF
      PIBY2=XPI(.5)
C
C CHECK INPUT PARAMETERS.
C
      IF (XM1 .GT. 1.) GO TO 9
      IF (XM1 .LT. 0.) GO TO 11
      IF (IFLAG .EQ. 2) GO TO 1
      IF (PHI .LT. 0.) GO TO 10
      IF (PHI .GT. PIBY2) GO TO 17
      S=SIN(PHI)
      S2=S*S
      IF (IFLAG .EQ. 1) GO TO 2
    1 IF (CPHI .LT. 0.) GO TO 17
      IF (CPHI .GT. 1.) GO TO 10
      C2=CPHI*CPHI
      PHI=ACOS(CPHI)
      IF (IFLAG .EQ. 3) GO TO 3
      S2=1.-C2
      S=SQRT(S2)
      GO TO 3
    2 C2=1.-S2
      CPHI=SQRT(C2)
    3 IF (XM1 .EQ. 1.) GO TO 12
      IF (XM1 .EQ. 0.) GO TO 14
      IF (C2 .EQ. 1.) GO TO 16
      IF (C2 .EQ. 0.) GO TO 17
C
C AT THIS STAGE, ALL PARAMETERS HAVE BEEN CHECKED AND IF ILLEGAL,
C CONTROL HAS BEEN TRANSFERRED TO THE APPROPRIATE EXITS.  (IE. XM1 AND P
C AND/OR COS(PHI) ARE WITHIN LEGAL LIMITS AND CALCULATION WILL PROCEED B
C EITHER METHOD A OR METHOD B)
C
      XM=1.-XM1
      IF (XM .LE. .45) GO TO 31
      Y2=1.05-.5*XM
      IF (S2 .GT. Y2) GO TO 6
C
C METHOD  A
C ---------
C
   31 F=E=XJ=PHI
      A=B=1.
      H=CPHI*S
C
          DO 4 N=1,50
          XN=N
          Z=XM/XN
          A=A*Z*(XN-.5)
          B=B*Z*(XN-1.5)
          XN2=2.*XN
          XJ=(XJ*(XN2-1.)-H)/XN2
          AF=A*XJ
          AE=B*XJ
          F=F+AF
          E=E+AE
          IF (ABS(AF/F) .LT. EPS) GO TO 5
          IF (ABS(AE/E) .LT. EPS) GO TO 5
          H=H*S2
    4     CONTINUE
C
    5 RETURN
C
C METHOD  B
C----------
C
    6 A=SQRT(XM*C2+XM1)
      XM1S=SQRT(XM1)
      XMS=SQRT(XM)
      RATIO=XM1/XM
      XL=ALOG(XM1)
      XKS1=1./XMS
      FF=ELKCOMP(XM1)+.5*XL
      F=-ALOG(XMS*CPHI+A)*XKS1+FF+XM1*XL/(2.*(XMS+XM))
      E=ELECOMP(XM1)
      RHO=XMS*CPHI/XM1S
      TN=ALOG(RHO+SQRT(RHO*RHO+1.))
      QN=-XKS1
      TRM=A*XKS1*CPHI
C
          DO 7 N=1,50
          XN=N
          XN2=XN+XN
          XN21=XN2-1.
          QN1=QN
          TN1=TN
          QN=XN21/XN2*QN1
          TNKR=TN1*RATIO
          TN=(TRM-XN21*TNKR)/XN2
          AF=QN*TN
          AE=XM*QN1*(TN+TNKR)
          F=F+AF
          E=E+AE
          IF (ABS(AF/F).LT. EPS) GO TO 8
          IF (ABS(AE/E) .LT. EPS) GO TO 8
          TRM=TRM*C2
    7     CONTINUE
C
    8 RETURN
C
C ERROR EXITS
C
    9 F=E=PIBY2
      PRINT 100,XM1
      RETURN
   10 F=E=0.
      PRINT 102,PHI,F,E
      RETURN
   11 E=1.
*IF -DEF,AELIB5
      F=37770000000000000000B
*ENDIF
*IF DEF,AELIB5
      F=O"37770000000000000000"
*ENDIF
      PRINT 101,XM1
      RETURN
   12 IF (IFLAG .NE. 2) GO TO 13
      S=SQRT(S2)
      PHI=ASIN(S)
   13 F=E=PHI
      RETURN
   14 IF (CPHI .NE. 0.) GO TO 15
*IF -DEF,AELIB5
      F=3777000000000000000B
*ENDIF
*IF DEF,AELIB5
      F=O"3777000000000000000"
*ENDIF
      E=1.
      RETURN
   15 IF (IFLAG .EQ. 2) S=SQRT(S2)
      F=ALOG((1.+S)/CPHI)
      E=S
      RETURN
   16 F=E=0.
      RETURN
   17 F=ELKCOMP(XM1)
      E=ELECOMP(XM1)
      IF (PHI .GT. (PIBY2+1.E-8)) PRINT 102,PHI,F,E
      IF (CPHI .LT. 0.) PRINT 103,CPHI,F,E
      RETURN
C
*IF -DEF,AELIB5
  100 FORMAT (*0*,*     EINCO ENTERED WITH ILLEGAL XM1=*,E10.3,5X,
     1   "F=E=PI/2")
  101 FORMAT (*0*,*     EINCO ENTERED WITH ILLEGAL XM1=*,E10.3,5X,
     ,     "F SET TO INFINITY,   E=1.")
  102 FORMAT (*0*,*     EINCO ENTERED WITH ILLEGAL PHI=*,E10.3,5X,
     1    *F=*,E12.5 ,5X,*E=*,E12.5 )
  103 FORMAT (*0*,*     EINCO ENTERED WITH ILLEGAL CPHI=*,E10.3,4X,
     1    *F=*,E12.5 ,5X,*E=*,E12.5 )
*ENDIF
*IF DEF,AELIB5
  100 FORMAT ('0','     EINCO ENTERED WITH ILLEGAL XM1=',E10.3,5X,
     1   'F=E=PI/2')
  101 FORMAT ('0','     EINCO ENTERED WITH ILLEGAL XM1=',E10.3,5X,
     ,     'F SET TO INFINITY,   E=1.')
  102 FORMAT ('0','     EINCO ENTERED WITH ILLEGAL PHI=',E10.3,5X,
     1    'F=',E12.5 ,5X,'E=',E12.5 )
  103 FORMAT ('0','     EINCO ENTERED WITH ILLEGAL CPHI=',E10.3,4X,
     1    'F=',E12.5 ,5X,'E=',E12.5 )
*ENDIF
C
      END
