*DECK FILON
      SUBROUTINE FILON(F,XK,A,B,E,M,C,NC,S,NS,IFLAG)
C
C    ROUTINE FOR FILON QUADRATURE GIVING
C              C=INT(A TO B(F(X).COS(XK.X)DX))  IF NC =1
C       AND    S=INT(A TO B(F(X).SIN(XK.X)DX))  IF NS =1
C       WITHIN ESTIMATED RELATIVE ERROR OF E, USING UP TO 2**M STEPS.
C
C      PARAMETERS
C        F      EXTERNAL REAL FUNCTION TO RETURN F(X)
C        XK,A,B REAL VARIABLES AS DEFINED ABOVE
C        C,S,E  REAL VARIABLES AS DEFINED ABOVE
C        M,NC,NS INTEGER VARIABLES AS DEFINED ABOVE
C        NC     IF 1 ON ENTRY C IS EVALUATED.
C                    ON EXIT 2**NC IS NUMBER OF STEPS NEEDED FOR C.
C        NS     AS NC BUT FOR S.
C        IFLAG  INTEGER  TO REPORT ERORS AND CONTROL DIAGNOSTIC PRINTING
C               IF IFLAG RETURNS 0 , NO PROBLEMS WERE ENCOUNTERED
C               IF IFLAG RETURNS -1, C DID NOT CONVERGE
C               IF IFLAG RETURNS -2, S DID NOT CONVERGE
C               IF IFLAG RETURNS -3, C,S DID NOT CONVERGE
C               IF IFLAG WAS ENTERED NON ZERO AN ERROR MESSAGE IS GIVEN
C                          WHEN CONVERGENCE FAILS
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C   SOURCE - CHASE AND FOSDICK , ALGORITHM FOR FILON QUADRATURE,
C                                COMMUNICATIONS ACM V 12   AUG 1969
C        PROGRAMMED BY MIKE CARVER                         JUL 1970
C        REVISED BY    MIKE CARVER                         AUG 1976
C
      EXTERNAL F
*CALL ERRORF
      DATA MES/2303/
C
C   INITIALIZE. ESTIMATE STARTING NO OF STEPS 2**N. SUM END POINTS.
C
*CALL STATF
      B1=2./3.
      C=S=0
      CO=SO=0
      N=4
      IF (XK .NE. 0.) N=MAX1(4.0,ALOG(ABS(XK+XK))/.693)
      NSTOP=2**N
      H=(B-A)/NSTOP
      IF(H) 5,60,10
    5 H=-H
      SIGN=-1.
      A0=A
      A=B
      B=A0
   10 NST=MSW=1
      SIGN=1.
      ALF=F(A)
      BET=F(B)
      C1=XK*A
      S1=XK*B
      FAKS=ALF*SIN(C1)
      FAKC=ALF*COS(C1)
      FBKS=BET*SIN(S1)
      FBKC=BET*COS(S1)
      SUMCOS = 0.5*(FAKC+FBKC)
      SUMSIN = 0.5*(FAKS+FBKS)
      NSTOP=NSTOP-1
      TH=H*XK
      RH=1./TH
C
C FIX CONTROLS TO SELECT SIN,COS OR BOTH
C
      LC=LS=2
      IF (NC .EQ. 1) LC=NC
      IF (NS .EQ. 1) LS=NS
C
C THE ITERATIVE PART BEGINS BELOW
C
   15 ODCOS = ODSIN = 0.0
      DO 20 I=1,NSTOP,NST
         B2=A+H*I
         C1=F(B2)
         IF (LS .EQ. 1) ODSIN=ODSIN+C1*SIN(XK*B2)
   20 IF (LC .EQ. 1) ODCOS=ODCOS+C1*COS(XK*B2)
      IF (MSW .EQ. 2) GOTO 30
      NST=MSW=2
      NSTOP=NSTOP+1
   25 N = N+1
      IF (N .GT. M) GOTO 55
      H = 0.5*H
      TH=.5*TH
      NSTOP=NSTOP+NSTOP
      SUMSIN=SUMSIN+ODSIN
      SUMCOS=SUMCOS+ODCOS
      GOTO 15
C
C RETURN FOR NEXT  ITERATION  HAVING SPLIT ALL PARAMETERS
C     SECTION  BELOW  DOES THE ITERATION USING A POWER SERIES FOR
C  PARAMETERS  ALF,BET,GAM BELOW TH = .166
C
   30 TSQ     = TH*TH
      IF(TH .GT. .1667) GOTO 35
         ALF  = TH*TSQ *(1.-TSQ*(1.-TSQ/15.)/7.)/22.50
         B2  = B1*TSQ*0.2
         B3  = B2*TSQ/3.50
         B4  = B3*TSQ/10.8
         BET=(B1+B2+B4)-(B3+B4*TSQ/3850.)
         GAM=(B1+B1+.125*B3)-(B2+.025*B4)
         GOTO 40
C
C     FOR TH GT .166 USE THE CLOSED FORM OF ALF,BET,GAM.
C
   35 S1=SIN(TH)
      C1=COS(TH)
      PR=S1*C1
      SS=S1*S1
      RH=RH+RH
      ALF = (((-2.*SS*RH)+PR)*RH +1.)*RH
      BET = 2.*((-2.*PR*RH)+2. -SS)/TSQ
      GAM = 4.*(S1*RH - C1)/TSQ
C
C   HAVING THE COEFFICIENTS, NOW COMPUTE THE FILON FORMULAE.
C
   40 IF (LS .EQ. 2) GOTO 45
         SO=S
         S=H*(ALF*(FAKC-FBKC)+BET*SUMSIN+GAM*ODSIN)
         IF(ABS(SO-S).LE.E*ABS(S))LS=2
         NS=N
   45 IF (LC .EQ. 2) GOTO 50
         CO=C
         C=H*(ALF*(FBKS-FAKS)+BET*SUMCOS+GAM*ODCOS)
         IF(ABS(CO-C).LE.E*ABS(C))LC=2
         NC=N
   50 IF(LC+LS.LE.3) GOTO 25
      GOTO 60
C
C     ERROR EXIT FOR NON CONVERGENCE
C
   55 ENCODE(2,65,IERWD(3))M
*IF -DEF,AELIB5
      IERWD(3)=IERWD(3).AND.MASK(12)
      IF(IFLAG.NE.0) CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(3)=IERWD(3).AND.MASK(12)
      IF(IFLAG.NE.0) CALL AELERR(MES,'NF')
*ENDIF
      IF(NS.NE.0.AND.LS.NE.2)IFLAG=-2
      IF(NC.NE.0.AND.LC.NE.2)IFLAG=IFLAG-1
C
C     NORMAL EXIT
C
   60 IF(SIGN.EQ.1.)RETURN
      B=A
      A=A0
      C=C*SIGN
      S=S*SIGN
   65 FORMAT(I2)
      RETURN
      END
