*DECK GAJAC
      FUNCTION GAJAC(AUX, AA, BB, ALF, BTA, NO, EPS)
************************************************************************
**                                                                    **
**    NOTE THAT GAJAC  IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL**
**    ROUTINE:                                                        **
**       JACOBI                                                       **
**    JACOBI ACCOMPANIES GAJAC ON AELIB SOURCE TAPE UNDER DECK NAME   **
**    GAJAC                                                           **
**                                                                    **
************************************************************************
C
C  GAJAC, A ROUTINE TO EVALUATE DEFINITE INTEGRALS WITH ENDPOINT
C  SINGULARITIES, I.E.
C
C   INTEGRAL FROM AA TO BB OF
C          ( (X-AA)**ALF ) * ( (BB-X)**BTA ) * AUX(X)
C
C  AUX - REAL EXTERNAL FUNCTION DEFINED BY THE USER TO EVALUATE THE
C        FUNCTION.
C  AA,BB - REAL, LIMITS OF INTEGRATION (ALSO THE SINGULARITIES OF THE
C          INTEGRAND)
C  ALF,BTA - REAL, MULTIPLICITY OF THE SINGULARITIES
C  NO  - INTEGER PARAMETER TO SPECIFY NUMBER OF POINTS IF FIXED
C           AND TO SPECIFY WHETHER TO CALCULATE JACOBI NODES AND WEIGHTS
C         OTHERWISE TO CONTROL ERROR MESSAGE PRINTOUT, FLAG ERROR,
C               AND RETURN THE NUMBER OF POINTS ACTUALLY USED.
C  EPS  REAL REQUIRED RELATIVE ERROR TOLERANCE WHEN NO NOT FIXED
C
C     IF NO GE 4 THE INTEGRAL IS EVALUATED ONLY ONCE USING A NO POINT
C   FORMULA MAXIMUM 128. IF NO GT 0 NODES AND WEIGHTS ARE CALCULATED BY
C     SUBROUTINE JACOBI. IF LT 0 PREVIOUSLY CALCULATED VALUES ARE USED.
C    NO IS LIMITED 4@NO@128
C  IF NO = 0 THE INTEGRAL IS EVALUATED USING A 4-POINT AND A 8-POINT
C  FORMULA (EST8). IF THE DIFFERENCE @ EPS*EST8,EST8 IS RETURNED, IF NOT
C  INTEGRAL IS ESTIMATED USING A 16- THEN A 32- , 64-, 128-POINT
C  FORMULA UNTIL THE DIFFERENCE BETWEEN TWO CONSECUTIVE ESTIMATES IS
C  LESS THAN EPS. IF A 128-POINT FORMULA DOES NOT GIVE SUFFICIENT
C  ACCURACY, THE 128 POINT ESTIMATE IS RETURNED, NN IS SET TO -128
C     AND UNLESS NO WAS INPUT AS ZERO TO SUPPRESS PRINTING,
C      THE FOLLOWING MESSAGE IS PRINTED -
C
C   REQUESTED ERROR NOT ATTAINED. ERROR IN GAJAC PROBABLY LESS THAN ***
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C       SOURCE     STROUD AND SECREST - GAUSSIAN QUADRATURE FORMULAE
C                                       PRENTICE HALL          1966
C       PROGRAMMED BY    SUZANNE BERUBE                    DEC 1972
C       REVISED    BY VALERIE JONES                        AUG 1976
C
C
      DIMENSION X(128), A(128), B(128), C(128)
C
*CALL ERRORF
      DATA MES/2302/
C
*CALL STATF
      ANS=ANS2=0.
      XL=AA
      XU=BB
      SIGN=1.
      IF(BB-AA) 5,55,10
    5 XL=BB
      XU=AA
      SIGN=-1.
C
C     IF NO IS SPECIFIED THE INTEGRAL IS ESTIMATED ONLY ONCE
C
   10 N0=IABS(NO)
      NN=MAX0(4,MIN0(128,N0))
C
C     TWO INTERMEDIATE ARRAYS, B, AND C ARE CALCULATED.  THESE ARRAYS
C     ARE USED TO DETERMINE THE NODES (X(I)) AND THE WEIGHTS (A(I)).
C
      APB=ALF+BTA
      BMA=BTA-ALF
      AP=ALF+1.
      BP=BTA+1.
      AB=APB*BMA
      AB2=APB+2.
      AK2=APB-3.
      B(2)=AB/((AB2+2.)*AB2)
      C(2)=4.*AP*BP/((AB2+1.)*AB2*AB2)
   15 DO 20 JJ=3,NN
         JJM=JJ-1
         AK=AK2+JJ+JJ
         AK3 = AK+1.
         B(JJ) = AB/((AK+3.)*AK3)
         C(JJ) =4.*JJM*(ALF+JJM)*(BTA+JJM)*(APB+JJM)/((AK+2.)*AK3*AK3*AK
     ,   )
   20 CONTINUE
C
C     THE SUBROUTINE JACOBI EVALUATES THE NODES AND WEIGHTS NORMALLIZED
C     TO THE INTERVAL (-1, 1).
C
      IF(NO.GE.0)CALL JACOBI(NN,X,A,ALF,BTA,B,C)
C
C     BUR1 AND BUR2 SCALING FACTORS WHICH CHANGE THE INTERVAL FROM
C     (-1,U) TO (AA,BB).
C
   25 BUR1=(XU-XL)*.5
      BUR2=(XU+XL)*.5
      IF(N0.LT.4) GOTO 35
C
C         IF /NO/ IS LESS THAN 4 , THE INTEGRAL
C     ESTIMATED ONLY ONCE USING A NO-POINT FORMULA.
C
         DO 30 II=1,NN
            XX = BUR1*X(II) + BUR2
            YY = AUX(XX)
   30    ANS = ANS + A(II)*YY
         ANS= BUR1**(1.+APB)*ANS
         GOTO 55
C
C     CALCULATES TWO DIFFERENT VALUES OF NN AND CHECK IF
C     THE DIFFERENCE IS LESS THAN EPS.
C
   35 ANS=0.
      DO 40 II=1,NN
         XX = BUR1*X(II) + BUR2
         YY = AUX(XX)
   40 ANS = ANS + A(II)*YY
      ANS= BUR1**(1.+APB)*ANS
C
C     THE INTEGRAL IS ESTIMATED A MINIMUM OF TWICE.
C
      IF (NN.EQ.4) GOTO 45
C
C     IF TOLERANCE IS MET THE LAST ESTIMATE IS RETURNED.
C
         ERR=ABS(ANS-ANS2)
         IF(ERR.LE.EPS*ABS(ANS2)) GOTO 55
C
C     IF A HIGHER ORDER FORMULA IS AVAILABLE THE INTEGRAL IS ESTIMATED
C     AGAIN.
C
   45 IF(NN.GT.64) GOTO 50
         ANS2=ANS
         NN=NN+NN
         GOTO 15
C
C     IF 128-POINT FORMULA DOES NOT GIVE SUFFICIENT ACCURACY AN ERROR
C     MESSAGE IS PRINTED.
C
   50 CONTINUE
      ENCODE(10,60,IERWD(3))ERR
*IF -DEF,AELIB5
      IERWD(4)=IERWD(4).AND.MASK(0)
      IF(NO.NE.0) CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(4)=IERWD(4).AND.MASK(0)
      IF(NO.NE.0) CALL AELERR(MES,'NF')
*ENDIF
      NN=-NN
C
C    EXIT
C
   55 GAJAC=ANS*SIGN
      NO=NN
   60 FORMAT(E10.2)
      RETURN
      END
      SUBROUTINE JACOBI(NN, X, A, ALF, BTA, B, C)
*
*  CALCULATES THE ZEROS X(I) OF THE NO-TH ORDER JACOBI POLYNOMIAL
*  PN(ALF,BTA) FOR (-1,1). LARGEST ZERO IS STORED IN X(1). ALSO
*  CALCULATES CORRESPONDING COEFFICIENTS A(I) OF THE NO-TH ORDER
*  GAUSS - JACOBI QUADRATURE FORMULA OF DEGREE 2*NO-1.
*
*  TO BE CALLED BY GAJAC ONLY. NOT FOR GENERAL USE. SEE GAJAC.
*
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C       PROGRAMMED BY    JUDY BARTON                       APR 1971
C       REVISED    BY    VALERIE JONES                     AUG 1976
C
      DIMENSION X(1), A(1), B(1), C(1)
      EPA=.1E-11
      APB = ALF+BTA
      BMA=BTA-ALF
      AP=ALF+1.
      BP=BTA+1.
      FN = NN
      RN=1./FN
      AB2=APB+2.
      CC=2.**(APB+1.)*(GAMMA(AP)*GAMMA(BP)/GAMMA(AB2))
      DO 5 J=2,NN
    5 CC = CC*C(J)
      DO 75 I=1,NN
         IF(I-1) 75,10,15
C
C             LARGEST ZERO
C
   10    AN = ALF*RN
         BN = BTA*RN
         R1=AP*(2.78/(4.+FN*FN)+.768*AN*RN)
         R2=1.+AN*(1.48+.452*AN+.83*BN)+.96*BN
         XT = 1. - R1/R2
         GOTO 55
   15    IF (I-2) 75,20,25
C
C             SECOND ZERO
C
   20    R1 = (4.1+ALF)/(AP*(1.+.156*ALF))
         R2 = 1. + .06*(FN-8.)*(1.+.12*ALF)*RN
         R3=1.+(.012*BTA+.003*ABS(ALF))*RN
         XT=XT-R1*R2*R3*(1.-XT)
         GOTO 55
   25    IF (I-3) 75,30,35
C
C             THIRD ZERO
C
   30    R1 = (1.67+.28*ALF)/(1.+.37*ALF)
         R2 = 1. + .22*(FN-8.)*RN
         R3 = 1. + 8.*BTA/((6.28+BTA)*FN*FN)
         XT=XT-R1*R2*R3*(X(1)-XT)
         GOTO 55
   35    IF (NN-I-1) 50,45,40
C
C             MIDDLE ZEROS
C
   40    XT = 3.*(X(I-1)-X(I-2))+X(I-3)
         GOTO 55
C
C             SECOND LAST ZERO
C
   45    R1 = (1.+.235*BTA)/(.766+.119*BTA)
         R2 = 1./(1. + .639*(FN-4.)/(1. + .71*(FN-4.)))
         R3 = 1./(1. + 20.*ALF/((7.5+ALF)*FN*FN))
         XT=XT+R1*R2*R3*(XT-X(I-2))
         GOTO 55
C
C             LAST ZERO
C
   50    R1 = (1. + .37*BTA)/(1.67+.28*BTA)
         R2 = 1./(1. + .22*(FN-8.)*RN)
         R3 = 1./(1. + 8.*ALF/((6.28+ALF)*FN*FN))
         XT =XT + R1*R2*R3*(XT-X(I-2))
C
*  GET A BETTER APPROXI. ROOT X AND ALSO GET DPN = DERIV OF P(NN) AT X
C
   55    ITER = 0
   60    ITER = ITER+1
         P1 = DP20=1.
         DP1=0.
         P20 = XT - BMA/(APB+2.)
         DO 65 J=2,NN
            Q = (XT-B(J))* P20 - C(J)*P1
            DQ = (XT-B(J)) * DP20 + P20 - C(J) * DP1
            P1 = P20
            P20 = Q
            DP1=DP20
   65    DP20=DQ
         P=P20
         DP=DP20
         PN1=P1
         D = P/DP
         XT = XT-D
         IF (ABS(D).LE.EPA) GOTO 70
            IF (ITER - 10) 60,70,70
   70    DPN = DP
         X(I) = XT
         A(I) = CC/(DPN*PN1)
   75 CONTINUE
C
C     EXIT
C
      RETURN
      END
