*DECK JACOB
      SUBROUTINE JACOB(M,N,IRN,IP,FUNC,X,F,Y,H,HMAX,W,A,IA,IG,Z,MBD)
C
C       ROUTINE TO EVALUATE AND STORE A SPARSE JACOBIAN MATRIX PACKED
C
C     GIVEN A ROUTINE FUNC WHICH CALCULATES F(I)[X1,X2--XN] , I=1,M
C       JACOB EVALUATES PF(I)/PX(J) I=1,M , J=1,N AND PACKS NON ZERO
C       IN A FORM COMPATIBLE WITH OTHER HARWELL SPARSE MATRIX ROUTINES
C       RESULTS IN ARRAY A USING ARRAYS IRN AND IP TO INDEX THE ELEMENTS
C
C     M       THE NUMBER OF FUNCTIONS F
C     N       THE NUMBER OF VARIABLES X
C     IA      A DIMENSION  LARGER THAN THE NUMBER OF NON ZEROS EXPECTED
C     IRN     ARRAY DIMENSION IA TO HOLD POINTERS IN PACKED MATRIX
C     IP      ARRAY DIMENSION N+1 TO HOLD POINTERS
C     FUNC    ROUTINE THAT WILL BE CALLED TO EVALUATE THE FUNCTIONS F
C     X       ARRAY DIMENSION N FOR INDEPENDENT VARIABLES
C     F       ARRAY DIMENSION M FOR FUNCTIONAL RESPONSES
C     Y       INDEPENDENT VARIABLE IN EQUATIONS [ TIME MAYBE ]
C     H       ARRAY DIMENSION  N TO HOLD PERTURBATION SIZE FOR EACH X
C     HMAX    ARRAY DIMENSION N TO HOLD MAX PERTURBATION FOR EACH X
C     W       ARRAY DIMENSION M+N FOR WORKSPACE
C     A       ARRAY DIMENSION IA TO STORE THE PACKED JACOBIAN PF/PX
C     IG      ARRAY DIMENSION 2N+1 FOR WORSPACE
C     Z       ARRAY DIMENSION N    FOR WORSPACE
C     MBD     CONTROL
C               LT 0 - ESTABLISH SPARSITY STRUCTURE OF FULL MATRIX
C               EQ 0 - ESTABLISH SPARSITY STRUCTURE OF FULL MATRIX AND
C                            EVALUATE INITIAL JACOBIAN
C               GT 0 - EVALUATE JACOBIAN USING STRUCTURE PREVIOUSLY FND.
C
C      MATRIX A(I,J) IS STORED BY COLUMNS J IE A11 A31 A22 A42 A33
C      IRN  CONTAINS THE ROW NUMBERS ; ABOVE _ 1 3 2 4 3
C      IP   CONTAINS THE POSITION IN A OF START OF EACH COLUMN J;
C                       ABOVE _ 1 3 5 6
C      IP(N+1)-1  HAS THE NUMBER OF NONZEROS IN A AND IRN ON RETURN
C
C
C    IN THE COMMON BLOCK /SPARS/
C    LP      THE OUTPUT UNIT NO
C    EPS     THE ROUND OFF ERROR
C    ISCALE  OP CONTROL  <0 MODIFY ELEMENTS FOR GEARS ROUTINE
C                        =0 NORMAL
C                        >0 ADJUST H
C
C      ADAPTED FROM THE HARWELL ROUTINE TDO2A/D AERE-R-7293 [ REID ]
C
C           M CARVER  DEC 1974
C
C           REVISED BY  D. E. SMITH    JULY  1975
C
C
      COMMON/SPARS/LP,EPS,ISCALE,IDUMMY
      INTEGER IRN(1),IP(1),IG(1)
      REAL HMAX(N),H(N),Z(1),TRUNC,ROUND,HM,
     ,      X(N),F(M),A(1),W(1),Y(1)
      LOGICAL REPEAT,ADJUST,DIAG
      DATA  LP/6/, EPS/1.0E-14/, ISCALE/0/
      DATA  UMIN/10.0/, UAIM/100.0/, UMAX/1000.0/, EPS1/1.0E-14/
      DATA  ZERO/0.0/, ONE/1.0/
      DATA MES/550/
*CALL STATF
C
C        SET UP PROGRAM CONTROL
C
      DIAG = .FALSE.
      ADJUST = .FALSE.
      IF(ISCALE) 5,15,10
*IF -DEF,AELIB5
    5 DIAG=.T.
      ADJUST=.F.
*ENDIF
*IF DEF,AELIB5
    5 DIAG=.TRUE.
      ADJUST=.FALSE.
*ENDIF
      GOTO 15
*IF -DEF,AELIB5
   10 DIAG=.F.
      ADJUST=.T.
*ENDIF
*IF DEF,AELIB5
   10 DIAG=.FALSE.
      ADJUST=.TRUE.
*ENDIF
   15 IF(MBD.LE.0) GOTO 185
C
C     IG(J) POINTS TO THE FIRST ENTRY OF GROUP J
C
   20 NFUN=0
      N1=N+1
C
C     IG(N1+J),J=1,2,... ARE COLUMN NUMBERS FOR GROUP 1 THEN GROUP 2 ETC
C
      IF(IG(1).NE.0) GOTO 55
      ICC=1
      DO 25 J=1,N
         IG(J+1)=0
   25 W(M+J)=ZERO
C
C     W(M+J) IS ZERO IF COLUMN J HAS NOT BEEN INCLUDED IN A GROUP YET
C
      DO 50 NC=1,N1
         IG(NC)=ICC
         DO 30 I=1,M
   30    W(I)=ZERO
C
C     W(I),I=1,M HOLDS THE BOOLEAN PATTERN OF THE UNION OF THE COLUMNS
C     SO FAR INCLUDED IN GROUP NC
C
         DO 45 J=1,N
            IF(W(M+J).NE.ZERO) GOTO 45
               K1=IP(J)
               K2=IP(J+1)-1
               IF(K2.LT.K1) GOTO 45
                  DO 35 K=K1,K2
                     IF(W(IRN(K)).NE.ZERO) GOTO 45
   35             CONTINUE
C
C     ACCEPT COLUMN
C
                  DO 40 K=K1,K2
   40             W(IRN(K))=ONE
                  IG(N1+ICC)=J
                  ICC=ICC+1
                  W(M+J)=ONE
   45    CONTINUE
         IF(ICC.EQ.IG(NC)) GOTO 55
   50 CONTINUE
C
C     PRESERVE X IN W(M+1),...,W(M+N).   CHECK ALL H(J).
C
   55 DO 60 J=1,N
         HM=-HMAX(1)
         IF(HM.LT.0.)HM=HMAX(J)
         XJ=ABS(X(J))
         IF(ADJUST)H(J)=(XJ+AMAX1(EPS*XJ,AMIN1(H(J),HM),EPS1*HM))-XJ
   60 W(J+M)=X(J)
C
C     Z(J) HOLDS MAXIMUM OF ESTIMATED RATIO OF TRUNCATION ERROR TO
C      ROUNDOFF ERROR IN COLUMN J,J=1,2,...,N
C
   65 IF(DIAG) GOTO 75
         DO 70 J=1,N
   70    Z(J)=1.
C
C     FIND INITIAL APPROXIMATE DERIVATIVES
C
   75 DO 110 NG=1,N
         L1=IG(NG)
         L2=IG(NG+1)-1
         IF(L2.LT.L1) GOTO 115
         DO 80 L=L1,L2
            IF(H(IG(L+N1))) 80,80,85
   80    CONTINUE
         GOTO 110
   85    DO 90 L=L1,L2
            J=IG(L+N1)
            H(J)=ABS(H(J))
   90    X(J)=X(J)+H(J)
         CALL FUNC(M,N,X,W,Y)
         NFUN=NFUN+1
         DO 105 L=L1,L2
            J=IG(L+N1)
            X(J)=W(J+M)
            K1=IP(J)
            K2=IP(J+1)-1
            IF(K2.LT.K1) GOTO 105
               DO 95 K=K1,K2
                  I=IRN(K)
   95          A(K)=(W(I)-F(I))/H(J)
               IF(.NOT.DIAG) GOTO 105
                  DO 100 K=K1,K2
                     A(K)=A(K)*Z(1)
                     IF(IRN(K).EQ.J)A(K)=A(K)+1.
  100             CONTINUE
*IF -DEF,AELIB5
                  ADJUST=.F.
*ENDIF
*IF DEF,AELIB5
                  ADJUST=.FALSE.
*ENDIF
  105    CONTINUE
  110 CONTINUE
C
C     ESTIMATE ERRORS AND IMPROVED STEP-LENGTHS
C         THIS SECTION ENTERED ONLY IF ADJUST TRUE
C
  115 IF(.NOT.ADJUST) GOTO 180
      DO 145 NG=1,N
         L1=IG(NG)
         L2=IG(NG+1)-1
         IF(L2.LT.L1) GOTO 150
         DO 120 L=L1,L2
            IF(H(IG(N1+L))) 120,120,125
  120    CONTINUE
         GOTO 145
  125    DO 130 L=L1,L2
            J=IG(N1+L)
  130    X(J)=X(J)-H(J)
         CALL FUNC(M,N,X,W,Y)
         NFUN=NFUN+1
         DO 140 L=L1,L2
            J=IG(L+N1)
            X(J)=W(J+M)
            K1=IP(J)
            K2=IP(J+1)-1
            IF(K2.LT.K1) GOTO 140
               DO 135 K=K1,K2
                  I=IRN(K)
                  DER=(F(I)-W(I))/H(J)
                  ROUND=EPS*(0.5*( ABS(W(I))+ ABS(A(K)*H(J)+F(I)))+
     ,                  AMAX1(ABS(A(K)),ABS(DER))*(ABS(X(J))+H(J)))/H(J)
                  A(K)=(A(K)+DER)/2
                  TRUNC=DER-A(K)
                  IF(ROUND.EQ.0.)ROUND=1.
                  Z(J)=AMAX1(Z(J),ABS(TRUNC/ROUND))
  135          CONTINUE
  140    CONTINUE
  145 CONTINUE
C
C     FIND IMPROVED STEPS AND DECIDE WHETHER FURTHER SWEEP IS NEEDED
C
  150 REPEAT=.FALSE.
      DO 170 J=1,N
         IF(H(J).LT.0.) GOTO 170
            HJL=H(J)
            XJ=ABS(X(J))
            HM=-HMAX(1)
            IF(HM.LT.0.)HM=HMAX(J)
            IF(Z(J).LT.UMIN) GOTO 160
               H(J)=(AMAX1(H(J)*SQRT(UAIM/Z(J)),EPS*XJ,EPS1*HM)+XJ)-XJ
               IF(Z(J).GT.UMAX) GOTO 165
  155          H(J)=-H(J)
               GOTO 170
  160       H(J)=(AMIN1( SQRT(UAIM/Z(J))*H(J),HM)+XJ)-XJ
  165       IF(ABS(HJL/H(J)-1).LE.0.001) GOTO 155
            REPEAT=.TRUE.
  170 CONTINUE
      IF(REPEAT) GOTO 65
      DO 175 J=1,N
  175 H(J)= ABS(H(J))
  180 W(1)=NFUN
      W(2)=NC-1
      RETURN
C
C      ENTER HERE MERELY TO ESTABLISH SPARSITY STRUCTURE IN IRN AND IP
C        IF DIAG IS TRUE
C    INCLUDE DIAGONAL TO ENSURE MATRIX CAN BE MADE DIAGONALLY DOMINANT
C               TO PRECLUDE SINGULARITY
C
C
  185 K=1
      DO 190 J=1,N
         XJ=X(J)
         X(J)=XJ+H(J)
         CALL FUNC(M,N,X,W,Y)
         X(J)=XJ
         IP(J)=K
         DO 190 I=1,M
            IF((ABS(W(I)-F(I)).LT.EPS).AND.(I.NE.J.OR..NOT.DIAG))
     ,             GOTO 190
               IF(K.GT.IA) GOTO 200
               IRN(K)=I
               K=K+1
  190 CONTINUE
  195 IP(N+1)=K
      IF (MBD .LT. 0)  RETURN
      IG(1)=0.
      GOTO 20
C
C        ERROR EXIT
C
*IF -DEF,AELIB5
 200  CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
 200  CALL AELERR(MES,'NF')
*ENDIF
      IP(1)=-1
      RETURN
      END
