*DECK KIN
      SUBROUTINE KIN(X,AKIN)
C
C THE SUBROUTINE KIN EVALUATES THE BICKLEY FUNCTIONS KIN(X),
C N = 1,2,...,10, X \ 0, TO A MINIMUM PRECISION OF 12.75 SIGNIFICANT
C FIGURES AND RETURNS THE VALUES KIN(X),N = 1,2,...,10 IN THE ARRAY AKIN
C
C
C ARGUMENT LIST:
C
C     X - REAL, X \ 0.
C     AKIN - REAL, ONE DIMENSIONAL ARRAY TO STORE KIN(X),N = 1,2,...,10
C
C
C METHOD:
C
C THE BICKLEY FUNCTIONS SATISFY A RECURSIVE RELATION ENABLING A FUNCTION
C OF ANY ORDER TO BE FOUND IF THREE BICKLEY FUNCTIONS OF CONSECUTIVE
C ORDER ARE KNOWN.  THE FORMULA FOR FORWARD RECURSION IS:
C     (KI(N))(X) = (X * ((KI(N-3))(X) - (KI(N-1))(X)) / (N-1))
C                  + ((N-2) * (KI(N-2))(X)) / (N-1)
C THE FORMULA FOR BACKWARD RECURSION IS:
C     (KI(N))(X) = ((N+2) * (KI(N+3))(X)) / X + (KI(N+2))(X)
C                  - ((N+1) * (KI(N+1))(X)) / X
C FORWARD AND BACKWARD RECURSION WERE TESTED [ SEE "APPROXIMATING THE
C BICKLEY FUNCTIONS KIN(X),N=1,2,...,10",C.A.EDWARDS].  LEAST LOSS OF
C PRECISION WAS OBTAINED BY USING FORWARD RECURSION FOR 0@X@6, AND
C BACKWARD RECUSION FOR X\6.
C
C X \ 6.     KI,(X),KI9(X),KI10(X) ARE CALCULATED AND USED IN BACKWARD
C RECURSION TO DERIVE KIN(X),N=7,6,...,1
C
C 0 < X < 6.  KI1(X),KI2(X),KI3(X) ARE CALCULATED AND USED IN FORWARD
C RECURSION TO DERIVE KIN(X),N=4,5,...,10
C
C RATIONAL POLYNOMIAL APPROXIMATIONS TO THE BICKLEY FUNCTIONS WITH A
C MINIMUM ACCURACY OF 15D WERE OBTAINED FROM "RATIONAL CHEBYSHEV
C APPROXIMATIONS TO THE BICKLEY FUNCTIONS KIN(X)",AECL-     AND
C "RATIONAL CHEBYSHEV  APPROXIMATIONS TO THE BICKLEY FUNCTION KI3(X),
C X \ 6",AECL-5820.
C
C     X = 0.  KIN(X) ,N=1,2,...,10 IS APPROXIMATED BY CONSTANTS.
C
C     0 < X < 1.  KIN(X),N = 1,2,3 IS APPROXIMATED BY
C          R(X) + X**N * ALOG(X) * S(X**2)
C     WHERE R AND S ARE RATIONAL POLYNOMIALS IN X AND X**2 RESPECTIVELY.
C
C     1 @ X < 6.  KIN(X),N=1,2,3 IS APPROXIMATED BY
C          EXP(-X) * R(1. / X) ) / SQRT(X)
C     WHERE R IS A RATIONAL POLYNOMIAL IN (1. / X)
C
C     6 @ X < 673.5  KIN(X),N = 8,9,10 IS APPROXIMATED BY
C          EXP(-X) * R(1. / X) ) / SQRT(X)
C     WHERE R IS A RATIONAL POLYNOMIAL IN (1. / X)
C
C     X \ 673.5  KIN(X),1=1,10 IS APPROXIMATED BY ZERO
C
C
C ERROR CONDITIONS:
C
C THE SUBROUTINE KIN CAN DETECT  THE FOLLOWING ERROR:
C     ***ILLEGAL ARGUMENT FOR THE BICKLEY FUNCTION
C IF THE ARGUMENT X IS INFINITE,INDEFINITE OR NEGATIVE.  IF THIS ERROR
C OCCURS, THE ELEMENTS OF THE ARRAY AKIN ARE SET TO INDEFINITE, AN ERROR
C MESSAGE IS PRINTED AND CONTROL IS RETURNED TO THE CALLING PROGRAM.
C THE PRINTING OF ERROR MESSAGES IS HANDLED BY THE STANDARD AELIB
C ERROR PROCESSING ROUTINE,AELERR.
C
C WRITTEN BY P.CHRISTIE, MARCH,1977.
C
C
      REAL AKIN(10),J
      REAL LNX
      INTEGER VALID
C
*IF -DEF,AELIB5
      DATA R1P01,R1P11/60523065110053214577B,17274542523501426607B/
      DATA R1P21,R1P31/17237520011262777117B,17236774501546320122B/
      DATA R1P41,R1P51/17174272233033041772B,17174446735624667124B/
      DATA R1P61,R1P71/17114372013706333023B,17115100507253357666B/
      DATA R1P81,R1P91/17016613646030747007B,17025152035731444550B/
      DATA R1Q01,R1Q11/60510425716266001156B,17204701567264566265B/
      DATA S1P01,S1P11/17364216357065216430B,17324645236436474325B/
      DATA S1P21,S1P31/17254475175722551444B,17167336634440573436B/
      DATA S1P41      /17074623355443772474B/
      DATA S1Q01,S1Q11/17364216357065216430B,60500745376021762661B/
      DATA R2P01,R2P11/17244452346554363107B,17277454101756332427B/
      DATA R2P21,R2P31/17324106020637134353B,17327231357640403020B/
      DATA R2P41,R2P51/17325135014030616203B,17305071463647317355B/
      DATA R2P61      /17244435262665267750B/
      DATA R2Q01,R2Q11/17237240311460375417B,17276255263422334104B/
      DATA R2Q21,R2Q31/17317364013513722121B,17327405177126073343B/
      DATA R2Q41,R2Q51/17326457412572055541B,17314515022721570543B/
      DATA R2Q61      /17264074755304445432B/
      DATA R1P02,R1P12/17307411443407472662B,16537555205073567055B/
      DATA R1P22,R1P32/17326473472445532535B,17264546747655435371B/
      DATA R1P42,R1P52/17256103437171244666B,17177211024256223107B/
      DATA R1P62,R1P72/17204605242744557773B,61413376120302142517B/
      DATA R1P82      /17115440706751474223B/
      DATA R1Q02,R1Q12/17327045002034632425B,60513056235044021021B/
      DATA R1Q22      /60533210160406213361B/
      DATA S1P02      /60243357367041201462B/
      DATA S1Q02,S1Q12/17544420410736576315B,60301751764054255304B/
      DATA S1Q22,S1Q32/17416344225172332700B,60473055400435530705B/
      DATA S1Q42      /60520753553745161760B/
      DATA R2P02,R2P12/17174472055164707417B,17234610613366163447B/
      DATA R2P22,R2P32/17256515316203517017B,17267614735002121056B/
      DATA R2P42,R2P52/17267577127166444643B,17256001070100163255B/
      DATA R2P62,R2P72/17225461320406325370B,17145302753221376712B/
      DATA R2Q02,R2Q12/17167271262222105353B,17234037760323277015B/
      DATA R2Q22,R2Q32/17256226577165637722B,17274244636534250225B/
      DATA R2Q42,R2Q52/17275366426064762723B,17266057223221110032B/
      DATA R2Q62      /17245022225254467551B/
      DATA R1P03,R1P13/60342500463553027123B,17444657373215027600B/
      DATA R1P23,R1P33/61133734112222344516B,17444133763006627571B/
      DATA R1P43      /17355565246714755045B/
      DATA R1Q03,R1Q13/60322263774135040367B,17416211074621351210B/
      DATA R1Q23,R1Q33/17377326406720417000B,60440231323550406202B/
      DATA R1Q43,R1Q53/60503647573270174041B,60533471277754255771B/
      DATA R1Q63      /60552140076170404617B/
      DATA S1P03,S1P13/17335723756361534146B,17254515530176542264B/
      DATA S1P23,S1P33/17174346464064663121B,17075306737216050142B/
      DATA S1Q03,S1Q13/17364336762665205114B,60500621310023465661B/
      DATA R2P03,R2P13/17164710710744635066B,17225367673476423217B/
      DATA R2P23,R2P33/17254075167026357360B,17265107006507106453B/
      DATA R2P43,R2P53/17265324272413727230B,17254316615654576751B/
      DATA R2P63,R2P73/17224227772664460743B,17144244673345346554B/
      DATA R2Q03,R2Q13/17157635166540264364B,17224613252751304074B/
      DATA R2Q23,R2Q33/17254063062324426437B,17266171605756073771B/
      DATA R2Q43,R2Q53/17274273356060267017B,17265240560720254441B/
      DATA R2Q63      /17244576175337076403B/
      DATA R1P08,R1P18/17006664070616276122B,17065637337143412056B/
      DATA R1P28,R1P38/17127152355541333710B,17164047545421640501B/
      DATA R1P48,R1P58/17204224574461106273B,17207512602314664335B/
      DATA R1P68,R1P78/17204346434314213270B,17145600527426372712B/
      DATA R1Q08,R1Q18/17005357042444047670B,17064772355046122651B/
      DATA R1Q28,R1Q38/17126705024776102524B,17164433020203423612B/
      DATA R1Q48,R1Q58/17206020105215256621B,17217462460355160171B/
      DATA R1Q68      /17217542520114475422B/
      DATA R1P09,R1P19/17015434230521134777B,17075363645071261567B/
      DATA R1P29,R1P39/17144022544216374132B,17175466660501632474B/
      DATA R1P49,R1P59/17217473775107025666B,17234613732131326612B/
      DATA R1P69,R1P79/17234517724216734364B,17214426202225171520B/
      DATA R1Q09,R1Q19/17014335327665045502B,17074541101350607720B/
      DATA R1Q29,R1Q39/17137511264127413277B,17176076611152223307B/
      DATA R1Q49,R1Q59/17225062410075655053B,17244223603744306431B/
      DATA R1Q69,R1Q79/17246376772055167111B,17236570115261137343B/
      DATA R1P010,R1P110/17007646152302745137B,17074016136623321221B/
      DATA R1P210,R1P310/17136145447055000014B,17174372752467245774B/
      DATA R1P410,R1P510/17216264754274733002B,17234107507766447232B/
      DATA R1P610,R1P710/17234141315104350406B,17214166164753006741B/
      DATA R1Q010,R1Q110/17006174373437357017B,17066732573050636501B/
      DATA R1Q210,R1Q310/17135730605717617712B,17174765343057370744B/
      DATA R1Q410,R1Q510/17224262443146561110B,17237521115346176503B/
      DATA R1Q610,R1Q710/17246033373421067622B,17236410405377402046B/
*ENDIF
*IF DEF,AELIB5
      DATA R1P01,R1P11/O"60523065110053214577",O"17274542523501426607"/
      DATA R1P21,R1P31/O"17237520011262777117",O"17236774501546320122"/
      DATA R1P41,R1P51/O"17174272233033041772",O"17174446735624667124"/
      DATA R1P61,R1P71/O"17114372013706333023",O"17115100507253357666"/
      DATA R1P81,R1P91/O"17016613646030747007",O"17025152035731444550"/
      DATA R1Q01,R1Q11/O"60510425716266001156",O"17204701567264566265"/
      DATA S1P01,S1P11/O"17364216357065216430",O"17324645236436474325"/
      DATA S1P21,S1P31/O"17254475175722551444",O"17167336634440573436"/
      DATA S1P41      /O"17074623355443772474"/
      DATA S1Q01,S1Q11/O"17364216357065216430",O"60500745376021762661"/
      DATA R2P01,R2P11/O"17244452346554363107",O"17277454101756332427"/
      DATA R2P21,R2P31/O"17324106020637134353",O"17327231357640403020"/
      DATA R2P41,R2P51/O"17325135014030616203",O"17305071463647317355"/
      DATA R2P61      /O"17244435262665267750"/
      DATA R2Q01,R2Q11/O"17237240311460375417",O"17276255263422334104"/
      DATA R2Q21,R2Q31/O"17317364013513722121",O"17327405177126073343"/
      DATA R2Q41,R2Q51/O"17326457412572055541",O"17314515022721570543"/
      DATA R2Q61      /O"17264074755304445432"/
      DATA R1P02,R1P12/O"17307411443407472662",O"16537555205073567055"/
      DATA R1P22,R1P32/O"17326473472445532535",O"17264546747655435371"/
      DATA R1P42,R1P52/O"17256103437171244666",O"17177211024256223107"/
      DATA R1P62,R1P72/O"17204605242744557773",O"61413376120302142517"/
      DATA R1P82      /O"17115440706751474223"/
      DATA R1Q02,R1Q12/O"17327045002034632425",O"60513056235044021021"/
      DATA R1Q22      /O"60533210160406213361"/
      DATA S1P02      /O"60243357367041201462"/
      DATA S1Q02,S1Q12/O"17544420410736576315",O"60301751764054255304"/
      DATA S1Q22,S1Q32/O"17416344225172332700",O"60473055400435530705"/
      DATA S1Q42      /O"60520753553745161760"/
      DATA R2P02,R2P12/O"17174472055164707417",O"17234610613366163447"/
      DATA R2P22,R2P32/O"17256515316203517017",O"17267614735002121056"/
      DATA R2P42,R2P52/O"17267577127166444643",O"17256001070100163255"/
      DATA R2P62,R2P72/O"17225461320406325370",O"17145302753221376712"/
      DATA R2Q02,R2Q12/O"17167271262222105353",O"17234037760323277015"/
      DATA R2Q22,R2Q32/O"17256226577165637722",O"17274244636534250225"/
      DATA R2Q42,R2Q52/O"17275366426064762723",O"17266057223221110032"/
      DATA R2Q62      /O"17245022225254467551"/
      DATA R1P03,R1P13/O"60342500463553027123",O"17444657373215027600"/
      DATA R1P23,R1P33/O"61133734112222344516",O"17444133763006627571"/
      DATA R1P43      /O"17355565246714755045"/
      DATA R1Q03,R1Q13/O"60322263774135040367",O"17416211074621351210"/
      DATA R1Q23,R1Q33/O"17377326406720417000",O"60440231323550406202"/
      DATA R1Q43,R1Q53/O"60503647573270174041",O"60533471277754255771"/
      DATA R1Q63      /O"60552140076170404617"/
      DATA S1P03,S1P13/O"17335723756361534146",O"17254515530176542264"/
      DATA S1P23,S1P33/O"17174346464064663121",O"17075306737216050142"/
      DATA S1Q03,S1Q13/O"17364336762665205114",O"60500621310023465661"/
      DATA R2P03,R2P13/O"17164710710744635066",O"17225367673476423217"/
      DATA R2P23,R2P33/O"17254075167026357360",O"17265107006507106453"/
      DATA R2P43,R2P53/O"17265324272413727230",O"17254316615654576751"/
      DATA R2P63,R2P73/O"17224227772664460743",O"17144244673345346554"/
      DATA R2Q03,R2Q13/O"17157635166540264364",O"17224613252751304074"/
      DATA R2Q23,R2Q33/O"17254063062324426437",O"17266171605756073771"/
      DATA R2Q43,R2Q53/O"17274273356060267017",O"17265240560720254441"/
      DATA R2Q63      /O"17244576175337076403"/
      DATA R1P08,R1P18/O"17006664070616276122",O"17065637337143412056"/
      DATA R1P28,R1P38/O"17127152355541333710",O"17164047545421640501"/
      DATA R1P48,R1P58/O"17204224574461106273",O"17207512602314664335"/
      DATA R1P68,R1P78/O"17204346434314213270",O"17145600527426372712"/
      DATA R1Q08,R1Q18/O"17005357042444047670",O"17064772355046122651"/
      DATA R1Q28,R1Q38/O"17126705024776102524",O"17164433020203423612"/
      DATA R1Q48,R1Q58/O"17206020105215256621",O"17217462460355160171"/
      DATA R1Q68      /O"17217542520114475422"/
      DATA R1P09,R1P19/O"17015434230521134777",O"17075363645071261567"/
      DATA R1P29,R1P39/O"17144022544216374132",O"17175466660501632474"/
      DATA R1P49,R1P59/O"17217473775107025666",O"17234613732131326612"/
      DATA R1P69,R1P79/O"17234517724216734364",O"17214426202225171520"/
      DATA R1Q09,R1Q19/O"17014335327665045502",O"17074541101350607720"/
      DATA R1Q29,R1Q39/O"17137511264127413277",O"17176076611152223307"/
      DATA R1Q49,R1Q59/O"17225062410075655053",O"17244223603744306431"/
      DATA R1Q69,R1Q79/O"17246376772055167111",O"17236570115261137343"/
      DATA R1P010,R1P110/O"17007646152302745137",O"17074016136623321221"
     +/
      DATA R1P210,R1P310/O"17136145447055000014",O"17174372752467245774"
     +/
      DATA R1P410,R1P510/O"17216264754274733002",O"17234107507766447232"
     +/
      DATA R1P610,R1P710/O"17234141315104350406",O"17214166164753006741"
     +/
      DATA R1Q010,R1Q110/O"17006174373437357017",O"17066732573050636501"
     +/
      DATA R1Q210,R1Q310/O"17135730605717617712",O"17174765343057370744"
     +/
      DATA R1Q410,R1Q510/O"17224262443146561110",O"17237521115346176503"
     +/
      DATA R1Q610,R1Q710/O"17246033373421067622",O"17236410405377402046"
     +/
*ENDIF
C
C     INITIALIZE ERROR NUMBER FOR AELERR
C          ***ILLEGAL ARGUMENT FOR BICKLEY FUNCTION
C     ERROR NUMBER 250
C
*CALL  STATF
      NERR = 250
C
C     CHECK FOR ILLEGAL ARGUMENT
C
      VALID = LEGVAR(X)
      IF (VALID.NE.0) GO TO 400
      IF (X.LT.0.) GO TO 400
C
      IF (X.EQ.0.) GO TO 300
      IF (X.LT.1.) GO TO 200
      IF (X.GE.673.5) GO TO 110
      SQRTX = SQRT(X)
      EXPX = EXP(-X)
      XREC = 1. / X
      IF (X.LT.6.) GO TO 210
C
C
C     6 @ X < 673.5  CALCULATE KI(X),KI9(X),KI10(X)
C
      AKIN(10) =(((((((((R1P710 * XREC + R1P610) * XREC + R1P510) * XREC
     ,+ R1P410) * XREC + R1P310) * XREC + R1P210) * XREC + R1P110) *
     ,XREC + R1P010)         /  ((((((((XREC + R1Q710) * XREC + R1Q610)
     ,* XREC + R1Q510) * XREC + R1Q410) * XREC + R1Q310) * XREC + R1Q210
     ,) * XREC + R1Q110) * XREC + R1Q010) ) / SQRTX) * EXPX
C
      AKIN(9) =(((((((((R1P79 * XREC + R1P69) * XREC + R1P59) * XREC
     ,+ R1P49) * XREC + R1P39) * XREC + R1P29) * XREC + R1P19) * XREC
     ,+ R1P09)         /  ((((((((XREC + R1Q79) * XREC + R1Q69) * XREC
     ,+ R1Q59) * XREC + R1Q49) * XREC + R1Q39) * XREC + R1Q29) * XREC
     ,+ R1Q19) * XREC + R1Q09) ) / SQRTX) * EXPX
C
      AKIN(8) =(((((((((R1P78 * XREC + R1P68) * XREC + R1P58) * XREC
     ,+ R1P48) * XREC + R1P38) * XREC + R1P28) * XREC + R1P18) * XREC
     ,+ R1P08)         /  (((((((XREC + R1Q68) * XREC + R1Q58) * XREC
     ,+ R1Q48) * XREC + R1Q38) * XREC + R1Q28) * XREC + R1Q18) * XREC
     ,+ R1Q08) ) / SQRTX) * EXPX
C
C     BACKWARD RECURSION
C
      DO 100 I = 1,7
           J = 8. - FLOAT(I)
           AKIN(J) = (((J + 2.) * AKIN(J + 3.)) - ((J + 1.)
     ,               * AKIN(J + 1.))) / X + AKIN(J + 2.)
  100 CONTINUE
      RETURN
C
C
C     X \ 673.5  AUTOMATICALLY SET AKIN(I),I=1,10 EQUAL TO ZERO
C
  110 DO 120 I = 1,10
           AKIN(I)=0.
  120 CONTINUE
      RETURN
C
C
C     0 < X < 1.  CALCULATE KI1(X),KI2(X),KI3(X)
C
  200 XSQ = X**2
      LNX = ALOG(X)
      XI1 = X - .91169076342161
      XI2 = X - .2451000192866
      XI3 = X - 1.
      XI4 = X - .68448452306295
C
      AKIN(1) = (((((((((R1P91 * X + R1P81) * X + R1P71) * X + R1P61)
     ,* X + R1P51) * X + R1P41) * X + R1P31) * X + R1P21) * X + R1P11)
     ,*XI3 + R1P01) / ((X + R1Q11) * XI3 + R1Q01) + (X * LNX * ((((S1P41
     ,* XSQ + S1P31) * XSQ + S1P21) * XSQ + S1P11) * XSQ + S1P01) /
     ,((XSQ + S1Q11) * XSQ + S1Q01))
C
      AKIN(2) = ((((R1P12 * XI1 + R1P02) + ((((R1P52 * X + R1P42) * X
     ,+ R1P32) * X + R1P22) + (((R1P82 * XI2 + R1P72) * XI2 + R1P62)
     ,* (X**4))) * (XI1 * XI1)) / (R1Q02 + (((        XI3 + R1Q22) * X
     ,+ R1Q12) * XI3))) + (XSQ * LNX * ((S1P02) / (((((        XSQ
     ,+ S1Q42) * XSQ + S1Q32) * XSQ + S1Q22) * XSQ + S1Q12) * XSQ
     ,+ S1Q02))))
C
      AKIN(3) = (((((R1P43 * X + R1P33) * XI4 + R1P23) * XI4 + R1P13)
     ,* XI3 + R1P03) /  (((((((XI3 + R1Q63) * X + R1Q53) * X + R1Q43)
     ,* X + R1Q33) * XI3 + R1Q23) * X + R1Q13) * XI3 + R1Q03)) + (X**3
     ,* LNX * ((((S1P33 * XSQ + S1P23) * XSQ + S1P13) * XSQ + S1P03) /
     ,((XSQ + S1Q13) * XSQ + S1Q03)))
C
      GO TO 220
C
C
C     1 @ X < 6.  CALCULATE KI1(X),KI2(X),KI3(X)
C
  210 AKIN(1) = (((((((R2P61 * XREC + R2P51) * XREC + R2P41) * XREC
     ,+ R2P31) * XREC + R2P21) * XREC + R2P11) * XREC + R2P01) * EXPX) /
     ,((((((((XREC + R2Q61) *XREC + R2Q51) * XREC + R2Q41) * XREC
     ,+ R2Q31) * XREC + R2Q21) * XREC + R2Q11) * XREC + R2Q01) * SQRTX)
C
      AKIN(2) = ((((((((R2P72 * XREC + R2P62) * XREC + R2P52) * XREC
     ,+ R2P42) * XREC + R2P32) * XREC + R2P22) * XREC + R2P12) * XREC
     ,+ R2P02) * EXPX) / ((((((((XREC + R2Q62) * XREC + R2Q52) * XREC
     ,+ R2Q42) * XREC + R2Q32) * XREC + R2Q22) * XREC + R2Q12) * XREC
     ,+ R2Q02) * SQRTX)
C
      AKIN(3) = ((((((((R2P73 * XREC + R2P63) * XREC + R2P53) * XREC
     ,+ R2P43) * XREC + R2P33) * XREC + R2P23) * XREC + R2P13) * XREC
     ,+ R2P03) * EXPX) / ((((((((XREC + R2Q63) * XREC + R2Q53) * XREC
     ,+ R2Q43) * XREC + R2Q33) * XREC + R2Q23) * XREC + R2Q13) * XREC
     ,+ R2Q03) * SQRTX)
C
C     FORWARD RECURSION
C
  220 DO 230 I = 1,7
           J = FLOAT(I) + 3.
           AKIN(J) = (AKIN(J - 3.) - AKIN(J - 1.)) * X / (J - 1.)
     ,               + (J - 2.) * AKIN(J - 2.) / (J - 1.)
  230 CONTINUE
      RETURN
C
C
C     X = 0.
C
  300 AKIN(1) = 1.57079632679489
      AKIN(2) = 1.
      AKIN(3) = .78539816339745
      AKIN(4) = .66666666666667
      AKIN(5) = .58904862254808
      AKIN(6) = .53333333333333
      AKIN(7) = .49087385212340
      AKIN(8) = .45714285714286
      AKIN(9) = .42951462060798
      AKIN(10) = .40634920634920
      RETURN
C
C
C     ILLEGAL ARGUMENT.  PRINT ERROR MESSAFE, SET ELEMENTS OF ARRAY
C     AKIN TO INDEFINITE AND RETURN.
C
  400 DO 410 I = 1,10
*IF -DEF,AELIB5
           AKIN(I) = 17770000000000000000B
*ENDIF
*IF DEF,AELIB5
           AKIN(I) = O"17770000000000000000"
*ENDIF
  410 CONTINUE
*IF -DEF,AELIB5
      CALL AELERR(NERR,2HNF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(NERR,'NF')
*ENDIF
      RETURN
      END
