*DECK MLSQQ
      SUBROUTINE MLSQQ(IOPT,X,NRX,Y,W,YF,PARAM,NC,N,M,AUX,E,NE,S,RERR,WS
     ,                 ,IER)
C
C THIS SUBROUTINE PERFORMS LEAST SQUARES FITTING TO A NON-LINEAR
C FUNCTION SUPPLIED BY THE USER. IT USES THE DERIVATIVE-FREE ALGORITHM
C OF BROWN AND DENNIS IN NUMERISCHE MATHEMATIK,V18,PP 289-297. IF,
C HOWEVER, DERIVATIVES ARE AVAILABLE, THEY MAY BE SUPPLIED BY THE USER
C IN THE FUNCTION EVALUATION ROUTINE.
C THIS ROUTINE USES A ONE DIMENSIONAL WORKING STORAGE ARRAY WHICH IS
C DIMENSIONED IN THE MAIN PROGRAM AND PASSED AS A FORMAL PARAMETER
C
C ARGUMENT LIST
C    IOPT - INTEGER VALUE PASSED TO LSQQ SPECIFIES WHETHER OR NOT
C           DERIVATIVES ARE SUPPLIED BY THE USER AND WHETHER MARQUARDT
C           PARAMETER, U, IS REQUIRED.
C
C          =-2, LSQQ CALCULATES DERIVATIVES AND DOESN"T USE U.
C          =-1, LSQQ ASSUMES DERIVATIVES ARE PROVIDED AND DOESN"T USE U.
C               (THIS CASE STREAMLINES THE FITTING IF THE FUNCTION IS
C                LINEAR IN ALL PARAMETERS AND IN FACT SHOULD BE USED FOR
C                THE LINEAR CASE)
C          = 0, LSQQ WILL CALCULATE DERIVATIVES AND USES U.
C          = 1, LSQQ ASSUMES DERIVATIVES ARE PROVIDED BY AUX AND USES U.
C
C        ANY OTHER VALUE OF IOPT PRODUCES A WARNING MESSAGE AND IOPT=0
C        IS ASSUMED.
C
C    X - REAL ARRAY OF VALUES OF INDEPENDENT VARIABLES
C    NRX - INTEGER, NO. OF ROWS DIMENSIONED FOR X IN CALLING PROGRAM
C    Y - REAL ARRAY OF VALUES OF DEPENDENT VARIABLE
C    W - REAL ARRAY OF WEIGHTS FOR   DEPENDENT VARIABLE
C    YF - REAL ARRAY OF FITTED VALUES OF DEPENDENT VARIABLE
C    PARAM - REAL ARRAY OF PARAMETERS FOR USER SUPPLIED FUNCTION.
C            INITIAL VALUES ARE PASSED TO LSQQ AND FINAL VALUES RETURNED
C    NC - INTEGER ARRAY SPECIFYING WHETHER A PARAMETER IS TO BE FIXED OR
C         VARIED(0 MEANS FIXED,1 MEANS VARIED)
C    N  - NUMBER OF DATA POINTS,I.E. NUMBER OF VALUES OF DEPENDENT AND
C         INDEPENDENT VARIABLES.
C    M  - NUMBER OF PARAMETERS IN USER-SUPPLIED FUNCTION.
C    AUX - USER SUPPLIED FUNCTION ROUTINE - MUST BE DECLARED EXTERNAL IN
C          USER"S PROGRAM CALLING LSQQ
C    E - REAL ARRAY USED BY LSQQ TO
C        1. STORE THE JACOBIAN OF FUNCTION AT EACH ITERATION
C        2. STORE THE ERROR MATRIX (MXM ELEMENTS) AT END OF LSQQ
C      -E MUST BE DIMENSIONED E(I,J) WHERE I GE N AND J GE M IN
C       CALLING PROGRAM
C    NE - NUMBER OF ROWS OF E AS DIMENSIONED IN CALLING PROGRAM
C    S - REAL VALUE RETURNED FROM LSQQ IS THE RESIDUAL SUM OF SQUARES,
C        I.E.  SUM(W(I)*(Y(I)-YF(I))**2) FOR I=1,N
C    RERR - RELATIVE ERROR NECESSARY FOR CONVERGENCE, I.E. CONVERGENCE
C           ACHIEVED IF (NEWPARAM-OLDPARAM)/NEWPARAM LE RERR FOR ALL
C           PARAMETERS
C    WS - REAL WORKING STORAGE ARRAY. MUST HAVE DIMENSION AT LEAST
C         N+M+2MU+MU*(MU+1)/2 WHERE MU IS THE NUMBER OF VARYING
C         PARAMETERS
C
C WORKING STORAGE ALLOCATION
C  1.WS(1)-WS(N) - TEMPORARY STORAGE FOR FUNCTION VALUES AND RESIDUALS
C                  USED IN CALCULATION OF JACOBIAN
C  2.WS(N+1)-WS(N+M) - STORAGE FOR PARAMETER VALUES FROM PREVIOUS
C                      ITERATION (ALSO USED FOR TEXT AT START)
C  3.WS(N+M+1)-WS(N+M+MU) - STORAGE FOR INDICES OF VARYING PARAMETERS
C  4.WS(N+MU+M+1)-WS(N+M+2MU) - STORAGE FOR RHS OF NORMAL EQUATIONS
C  5.WS(N+M+2MU+1)-WS(N+M+2MU+MU*(MU+1)/2)
C                   -STORAGE FOR NORMAL MATRIX. SINCE NORMAL MATRIX IS
C                    SYMMETRIC, USE ONLY TRIANG MATRIX AND CONVERT TO 1D
C                    MATRIX WITH MU*(MU+1)/2 ELEMENTS
C
C  WORKING STORAGE IS INDEXED BY I2,I3,I4,I5,  WHERE EACH INDEX IS
C  THE BASE ADDRESS OF THE SUBARRAY
C
C     IER - ERROR FLAG SET IN LSQQ.  F = FATAL, NF = NON-FATAL
C         =0, NO ERRORS
C         =1, (NF) IOPT NOT -2, -1, 0 OR 1
C         =2, (NF) NON-CONVERGENCE
C         =3, (F) ILLEGAL PARAM. -WEIGHTS
C         =4, (F) TROUBLE IN SOLN. - SINGULAR NORMAL MATRIX
C        .GE. 10, (F) USER TERMINATION FROM AUX
C              (SET TO USER VALUE OR 10 WHICH EVER IS GREATER)
C
C
C ERROR EXITS
C     THERE ARE THREE ERROR CONDITIONS WHICH ARE FATAL TO LSQQ,
C     IER = 3, 4, OR .GE.10 (SEE EXPLANATIONS ABOVE FOR IER)
C     YF, S AND E ARE SET TO INDEFINITES (NOT E FOR IER = 4),
C     A MESSAGE IS PRINTED AND CONTROL IS RETURNED TO THE CALLING
C     PROGRAM
C
C ERROR CONDITIONS
C    THERE ARE TWO NON-FATAL ERRORS IN LSQQ.
C
C    1. IER = 1
C       IF IOPT IS NOT -2, -1, 0 NOR 1, IT IS NOT CLEAR WHETHER THE USER
C       HAS SUPPLIED DERIVATIVES AND WHETHER U IS TO BE USED. THE USER
C       IS INFORMED OF THIS AND IOPT=0 IS ASSUMED TO ALLOW THE PROGRAM
C       TO CONTINUE.(IF, HOWEVER, 1 OR -1 WAS INTENDED FOR IOPT,
C       UNPREDICTABLE THINGS WILL HAPPEN WHEN AUX ATTEMPTS TO SET UP
C       THE JACOBIAN)
C    2. IER = 2
C       IF CONVERGENCE IS NOT ACHIEVED IN "NITER" ITERATIONS THEN THE
C       LARGEST RELATIVE CHANGE IN PARAMETERS AT THE LAST ITERATION IS
C       CALCULATED AND PRINTED IN AN ERROR MESSAGE. LSQQ THEN
C       TERMINATES NORMALLY, CALCULATING S AND E.
C
C REVISION HISTORY OF MLSQQ
C
C WRITTEN BY J.SCHMIDT SUMMER,1973
C REVISED AND RECODED BY L.EVANS, MARCH,1974.
C    TO - 1. ALLOW USER CALCULATION OF DERIVATIVES
C       - 2. USE WORKING STORAGE PASSED FROM MAIN PROGRAM
C       - 3. STANDARDIZE ERRORS AND COMMENTS
C REVISED BY L EVANS, JUNE,1974
C    TO - 1. GIVE USER CONTROL OVER PRINTING OF PARAMETER ITERATES
C       - 2. SET UP COVARIANCES IN UPPER TRIANGLE OF ERROR MATRIX
C REVISED BY L. EVANS, E. LONG  JAN. 1977
C     TO - CORRECT FINITE DIFFERENCE APPROXIMATION TO JACOBIAN
C        - PROVIDE INFORMATIVE DIAGNOSTICS WHEN SOLUTION OF NORMAL
C          SYSTEM FAILS
C        - ADD AN ERROR FLAG
C        - FIX UP ERROR MATRIX CALCULATIONS, CORRECTLY DISTINGUISHING
C          BETWEEN WEIGHTED AND UNWEIGHTED FITS
C        - ALLOW USER CONTROL OVER NUMBER OF ITERATIONS AND PRINT
C          OPTIONS VIA COMMON BLOCK PARAMETERS
C        - ADD SUM OF SQUARES AND MARQUARDT PARAM. TO PRINT OUT
C REVISED BY A. WILLS, M. BOULANGER  MARCH 82
C     TO - HANDLE SPECIAL CASE WHEN ONE OF THE PARAMETERS
C          IS EQUAL TO ZERO.
C CORRECTION BY C. A. WILLS  MARCH 1982
C      TO TERMINATE IF AN EXACT SOLUTION IS FOUND WITHOUT ATTEMPTING
C         ANOTHER ITERATION.  IN THIS CASE THE VARIANCE-COVARANCE MATRIX
C         IS SET TO 0.
C
C
C
      COMMON  /MLSQOPT/  LIST,  NITER,  IWT
C
C.....LIST = 0, NO PRINTING
C          = 1, PARAM. DUMPED AFTER EACH ITERATION (PRINTER WIDTH)
C          = 2, PARAM. DUMPED AFTER EACH ITERATION (TERMINAL WIDTH)
C          = 3, DUMPS NORMAL MATRIX (LOWER TRIANGLE ONLY), RHS, LOWER
C               TRIANGULAR MATRIX AND ITS INVERSE.
C          = 4, DUMPS AS FOR LIST = 3 AND DERIVATIVE APPROXIMATIONS
C
C.....NITER,  NO. OF ITERATIONS
C
C......IWT, SET BY LSQQ TO INDICATE A "WEIGHTED" OR "UNWEIGHTED" FIT
C           = 0, ALL WEIGHTS EQUAL, UNWEIGHTED FIT
C           = 1, ALL WEIGHTS NOT EQUAL, WEIGHTED FIT
C
C
      REAL  X(NRX,1), Y(1), W(1), YF(1), PARAM(1), NC(1), E(NE,1), WS(1)
C
      LOGICAL   CONVRG
C                  ....CONVERGENCE FLAG, SET .TRUE. WHEN PROCESS HAS
C                  ....CONVERGED
      LOGICAL   ADERIV
C                  ....FLAG TO INDICATE SOURCE OF PARTIAL DERIVATIVES
C                  .... = .TRUE., LSQQ CALC. APPROXIMATE DERIVATIVES
C                  .... = .FALSE., USER SUPPLIES DERIVATIVES VIA AUX
      LOGICAL   MARQ
C                  ....FLAG TO INDICATE WHETHER OR NOT MARQUARDT PARAM.
C                  ....IS TO BE USED
C
      LOGICAL ALERPR
*IF -DEF,AELIB5
      DATA    RINDEF /17770000000000000000B/
*ENDIF
*IF DEF,AELIB5
      DATA    RINDEF /O"17770000000000000000"/
*ENDIF
      DATA    LIST/1/,  NITER /100/
C
C     IROUT IS AN INTEGER VARIABLE DEFINING THE ROUTINE NUMBER
C     FOR MLSQQ TO BE USED IN ERROR MESSAGE CONTROL
C
      IROUT=11120
C
C     FORMAT STATEMENTS
C
 6001 FORMAT((1H0,6X,R4,8(11X,R4)))
 6002 FORMAT((1H0,6X,R4,4(11X,R4)))
 6003 FORMAT(1X,9G15.7)
 6004 FORMAT(1X,5G15.7)
*IF -DEF,AELIB5
 6005 FORMAT(T8 "S = "G15.7, T29 "MARQ. PAR. = "G15.7)
*ENDIF
*IF DEF,AELIB5
 6005 FORMAT(T8 ,'S = ',G15.7, T29 ,'MARQ. PAR. = ',G15.7)
*ENDIF
*IF -DEF,AELIB5
 6011 FORMAT(" FINAL SUM OF SQUARES = "G15.7)
 6012 FORMAT("0 JACOBIAN"/)
 6013 FORMAT("0 NORMAL MATRIX"/)
 6014 FORMAT("0 RHS OF NORMAL SYSTEM"/)
 6015 FORMAT("0 LOWER TRIANGULAR MATRIX, L"/)
 6016 FORMAT("0 INVERSE OF L")
*ENDIF
*IF DEF,AELIB5
 6011 FORMAT(' FINAL SUM OF SQUARES = ',G15.7)
 6012 FORMAT('0 JACOBIAN'/)
 6013 FORMAT('0 NORMAL MATRIX'/)
 6014 FORMAT('0 RHS OF NORMAL SYSTEM'/)
 6015 FORMAT('0 LOWER TRIANGULAR MATRIX, L'/)
 6016 FORMAT('0 INVERSE OF L')
*ENDIF
*IF -DEF,AELIB5
 7100 FORMAT(" ***INVALID OPTION SELECTED - IOPT=0 ASSUMED")
 7200 FORMAT(" ***NO CONVERGENCE AFTER ",I4," ITERATIONS.  MAXIMUM RELAT
     $IVE CHANGE IN PARAMETERS IS ",G12.4,".")
 7300 FORMAT(" ***WEIGHTS ARE ZERO, NEGATIVE OR UNDEFINED")
 7400 FORMAT(" ***TRIANGULAR DECOMPOSITION REQUIRES SQUARE ROOT OF ",
     $G15.7," IN ROW ",I4,".")
*ENDIF
*IF DEF,AELIB5
 7100 FORMAT(' ***INVALID OPTION SELECTED - IOPT=0 ASSUMED')
 7200 FORMAT(' ***NO CONVERGENCE AFTER ',I4,' ITERATIONS.  MAXIMUM RELAT
     $IVE CHANGE IN PARAMETERS IS ',G12.4,'.')
 7300 FORMAT(' ***WEIGHTS ARE ZERO, NEGATIVE OR UNDEFINED')
 7400 FORMAT(' ***TRIANGULAR DECOMPOSITION REQUIRES SQUARE ROOT OF ',
     $G15.7,' IN ROW ',I4,'.')
*ENDIF
C
*CALL STATF
      IER = IERAUX = 0
      MU = 0
      CONVRG = .FALSE.
C
C     INITIALIZE TWO OF WS INDICES
C
      I2 = N
      I3 = N + M
C
C.....INITIALIZE "ADERIV", "MARQ" AS IF IOPT = 0
C
      ADERIV = MARQ = .TRUE.
C
C......CHECK IF IOPT IS NOT -2,-1, 0 OR 1
C
      IF ((IOPT .LT. -2) .OR. (IOPT .GT. 1)) GOTO 2000
C
C.....IOPT IS PROPERLY DEFINED, SET "ADERIV" AND "MARQ"
C
      IF (IOPT .EQ. -1 .OR. IOPT .EQ. 1) ADERIV = .FALSE.
      IF (IOPT .EQ. -2 .OR. IOPT .EQ. -1) MARQ = .FALSE.
C
 10   CONTINUE
C
C     SET UP TEXT FOR FIXED AND VARYING PARAMETERS AND POINTERS FOR
C     VARYING PARAMETERS.
C
C
      DO 20 I=1,M
        IND1=I2+I
*IF -DEF,AELIB5
        WS(IND1)=4R FIX
*ENDIF
*IF DEF,AELIB5
        WS(IND1)=R" FIX"
*ENDIF
        IF(NC(I).EQ.0)GOTO 20
*IF -DEF,AELIB5
          WS(IND1)=4RVARY
*ENDIF
*IF DEF,AELIB5
          WS(IND1)=R"VARY"
*ENDIF
          MU=MU+1
          IND1=I3+MU
          WS(IND1)=I
 20   CONTINUE
C
      IND1=I2+1
      IND2=I2+M
      IF (LIST .NE. 2 .AND. LIST .NE. 0) PRINT 6001, (WS(I),I=IND1,IND2)
      IF (LIST .EQ. 2) PRINT 6002, (WS(I), I=IND1,IND2)
C
C     INITALIZE REMAINDER OF WS INDICES NOW THAT WE HAVE MU
C
      I4=I3+MU
      I5=I4+MU
C
      L=MU*(MU+1)/2
C
C......STORE STARTING VALUES OF PARAMETERS
C
      DO 30 I=1,M
        IND1=I2+I
        WS(IND1)=PARAM(I)
 30   CONTINUE
C
C.....CHECK THAT WEIGHTS HAVE BEEN SET BY USER, AND DETERMINE IF THIS IS
C.....IS WEIGHTED OR UNWEIGHTED FIT
C
      IWT = 0
      WTOT = 0.
      WT = W(1)
      DO 40 I = 1, N
          IF (LEGVAR(W(I)) .NE. 0) GOTO 2100
          IF (W(I) .LE. 0.) GOTO 2100
          IF (W(I) .NE. WT) IWT = 1
          WTOT = WTOT + W(I)
   40 CONTINUE
      IF (IWT .NE. 0) WTOT = 0.
C
C     START ITERATION HERE
C
      DO 500 ITER = 1, NITER
C
C.....ZERO NORMAL MATRIX
C
      IND1=I5+1
      IND2=I5+L
      DO 110 I=IND1,IND2
 110  WS(I)=0.
C
C     ZERO RIGHT HAND SIDE
C
      IND1=I4+1
      IND2=I4+MU
      DO 120 I=IND1,IND2
 120  WS(I)=0.
C
C     CALCULATE FUNCTION VALUES WITH LATEST PARAMETERS AND FIND INFINITY
C     NORM OF RESIDUAL, FF.
C     (YF USED AS TEMPORARY STORAGE HERE)
C
      IF (ADERIV) CALL AUX(X,NRX,YF,PARAM,N,IERAUX)
      IF (.NOT. ADERIV) CALL AUX(X,NRX,YF,PARAM,N,E,NE,IERAUX)
C
C     ....CHECK ERROR FLAG AND EXIT IF SET
C
          IF (IERAUX .NE. 0) GOTO 2120
C
      FF = 0.
      DO 130 I=1,N
        YF(I)=Y(I)-YF(I)
        FF=AMAX1(FF,ABS(YF(I)*W(I)))
 130  CONTINUE
C
C.....TERMINATE IF AN EXACT SOLUTION HAS BEEN FOUND
C
      IF (FF .EQ. 0.0) GOTO 1000
C
C.....IF THE MARQUARDT PARAMETER, U, IS NOT TO BE USED, SKIP PAST ITS
C.....CALCULATION.
C
      U = 0.
      IF (.NOT. MARQ) GOTO 140
C
C     ....CALCULATE THE MARQUARDT PARAMETER
C
          Z=1.
          IF(FF .GE. 10.) Z=10.
          IF(FF .LE. 1.) Z=.01
          U = Z * FF
C
 140  CONTINUE
C
C     ....OUTPUT SUM OF SQ. OF RESIDUALS AND MARQUARDT PARAM. IF REQD.
C
          IF (LIST .EQ. 0) GOTO 160
              SUMSQ = 0.
              DO 150 I = 1, N
                  SUMSQ = SUMSQ + YF(I) * YF(I) * W(I)
  150         CONTINUE
              IF (LIST .NE. 2) PRINT 6003, (PARAM(I), I=1,M)
              IF (LIST .EQ. 2) PRINT 6004, (PARAM(I), I=1,M)
              PRINT 6005, SUMSQ, U
  160 CONTINUE
C
C.....IF DERIVATIVES ARE PROVIDED IN AUX SKIP PAST DERIVATIVE CALC.
C
      IF (.NOT. ADERIV) GOTO 190
C
C     JACOBIAN IS TO BE CALCULATED BY LSQQ.
C     FOR EACH VARYING PARAMETER, CALCULATE STEP SIZE, NEW PARAMETER
C     VALUE, NEW FUNCTION VALUE WITH THAT PARAMETER CHANGED, AND PARTIAL
C     DERIVATIVE. PARAMETER VALUE IS THEN RESET TO OLD VALUE.
C
      DO 180 K=1,MU
        IND1=I3+K
        J=WS(IND1)
           DELTA = 1.0E-9
           IF (ABS(PARAM(J)) .GE. 1.0E-6) DELTA = 0.001 * ABS(PARAM(J))
        SS = AMIN1(FF, DELTA)
        PARAM(J)=PARAM(J)+SS
        CALL AUX(X,NRX,WS,PARAM,N,IERAUX)
C
C     ....CHECK ERROR FLAG AND EXIT IF SET
C
          IF (IERAUX .NE. 0) GOTO 2120
        DO 170 I=1,N
          WS(I)=Y(I)-WS(I)
          E(I,J)=(WS(I)-YF(I))/SS
 170    CONTINUE
        IND2=I2+J
        PARAM(J)=WS(IND2)
 180  CONTINUE
C
 190  CONTINUE
      IF (LIST .LT. 4) GOTO 195
C
C     ....DUMP THE DERIVATIVE APPROXIMATIONS
C
          PRINT 6012
          DO 192 K = 1, N
          DO 191 I = 1, MU
              IND1 = I3 + I
              J = WS(IND1)
              IND2 = I4 + I
              WS(IND2) = E(K,J)
 191      CONTINUE
          IND1 = I4 + 1
          IND2 = IND1 + MU - 1
          PRINT 6003, (WS(J), J = IND1, IND2)
 192      CONTINUE
 195  CONTINUE
C
C     SET UP NORMAL MATRIX AND RIGHT HAND SIDE
C
      DO 230 I=1,MU
        IND1=I3+I
        J=WS(IND1)
        II=I*(I-1)/2
        DO 210 K=1,I
          IND1=I3+K
          IND1=WS(IND1)
          IND2=I5+II+K
          DO 200 KK=1,N
 200        WS(IND2)=WS(IND2)+E(KK,J)*E(KK,IND1)*W(KK)
 210    CONTINUE
C
        IND1=I4+I
        WS(IND1)=0.
        DO 220 K=1,N
 220    WS(IND1)=WS(IND1)-YF(K)*E(K,J)*W(K)
C
        IND2=I5+I+II
        WS(IND2)=WS(IND2)+U
 230  CONTINUE
C
      IF (LIST .LT. 3) GOTO 235
C
C     ....DUMP NORMAL MATRIX AND RHS
C
          PRINT 6013
          DO 231 I = 1, MU
              II = I * (I-1) / 2
              IND2 = I5 + II + 1
              IND3 = IND2 + I - 1
              PRINT 6003, (WS(J), J = IND2, IND3)
 231      CONTINUE
          PRINT 6014
          IND2 = I4 + 1
          IND3 = I4 + MU
          PRINT 6003, (WS(J), J = IND2, IND3)
 235  CONTINUE
C
C     NOW SOLVE NORMAL EQUATIONS BY CHOLESKY"S METHOD
C     SEE PAGES 9-12 OF "LINEAR ALGEBRA" BY WILKINSON AND REINSCH
C
C     FIND LOWER TRIANGULAR MATRIX,L, SUCH THAT NORMAL MATRIX=L*TRANS(L)
C     AND INVERSE=INV(TRANS(L))*INV(L)
C
      DO 270 I=1,MU
        II=I*(I-1)/2
        DO 260 J=1,I
          KK=J-1
          JJ=J*KK/2
          IND2=I5+II+J
          BB=WS(IND2)
          IF(KK.EQ.0.) GOTO 250
            DO 240 K=1,KK
              IND1=I5+II+K
              IND2=I5+JJ+K
              BB=BB-WS(IND1)*WS(IND2)
 240        CONTINUE
 250      IND1=I5+II+J
          IND2=I5+JJ+J
          IF (I .EQ. J) GOTO 255
            WS(IND1) = BB / WS(IND2)
            GOTO 260
C
 255      CONTINUE
          IF (BB .LE. 0.) GOTO 2110
            WS(IND1) = SQRT(BB)
C
 260    CONTINUE
 270  CONTINUE
C
      IF (LIST .LT. 3) GOTO 275
C
C     ....DUMP LOWER TRIANGULAR MATRIX, L
C
          PRINT 6015
          DO 271 I = 1, MU
              II = I * (I-1) / 2
              IND2 = I5 + II + 1
              IND3 = IND2 + I - 1
              PRINT 6003, (WS(J), J= IND2, IND3)
 271      CONTINUE
 275  CONTINUE
C
C     FIND INV(L)
C
      WS(I5+1)=1./WS(I5+1)
      IF(MU.EQ.1) GOTO 310
      DO 300 I=2,MU
        KK=I-1
        II=I*KK/2
        IND3=I5+II+I
        DO 290 J=1,KK
          BB=0.
          DO 280 K=J,KK
            IND1=I5+II+K
            IND2=I5+J+K*(K-1)/2
            BB=BB-WS(IND1)*WS(IND2)
 280      CONTINUE
          IND1=I5+II+J
          WS(IND1)=BB/WS(IND3)
 290    CONTINUE
        WS(IND3)=1./WS(IND3)
 300  CONTINUE
C
 310  CONTINUE
      IF (LIST .LT. 3) GOTO 315
C
C     ....DUMP INVERSE OF L
C
          PRINT 6016
          DO 311 I = 1, MU
              II = I * (I-1) / 2
              IND2 = I5 + II + 1
              IND3 = IND2 + I - 1
              PRINT 6003, (WS(J), J = IND2, IND3)
 311      CONTINUE
 315  CONTINUE
C
C     NOW CALCULATE CORRECTIONS TO PARAMETERS
C     FIRST CALCULATE INV(L)*RHS
C
      DO 330 I=1,MU
        KK=MU+1-I
        II=KK*(KK-1)/2
        BB=0.
        DO 320 K=1,KK
          IND1=I5+II+K
          IND2=I4+K
          BB=BB+WS(IND1)*WS(IND2)
 320    CONTINUE
        IND1=I4+KK
        WS(IND1)=BB
 330  CONTINUE
C
C     THEN CORRECTION VECTOR=INV(TRANS(L))*INV(L)*RHS
C
      DO 350 I=1,MU
        II=I*(I-1)/2
        IND1=I4+I
        DO 340 K=1,I
          IND2=I3+K
          IND3=I5+II+K
          IND2=WS(IND2)
          PARAM(IND2)=PARAM(IND2)+WS(IND3)*WS(IND1)
 340    CONTINUE
 350  CONTINUE
C
C.....CHECK FOR CONVERGENCE.
C..... CALCULATE MAX. RELATIVE CHANGE IN PARAMETERS, EMAX, AND IF
C..... GREATER THAN SPECIFIED RELATIVE ERROR, ITERATE AGAIN.
C
      EMAX = 0.
      DO 360 K = 1, MU
          IND1 = I3 + K
          IND1 = WS(IND1)
          IND2 = I2 + IND1
          IF(PARAM(IND1) .EQ. 0.0) GO TO 352
          EMAX = AMAX1(EMAX,ABS(1. - WS(IND2) / PARAM(IND1)))
          GO TO 354
  352     EMAX = AMAX1(EMAX,ABS(1. - WS(IND2)))
  354     CONTINUE
          WS(IND2) = PARAM(IND1)
 360  CONTINUE
      CONVRG = .FALSE.
      IF (EMAX .LE. RERR) CONVRG = .TRUE.
C
C.....IF CONVERGENCE ACHIEVED AND MARQ. PARAM. WAS NOT BEING USED, EXIT
C
*IF -DEF,AELIB5
      IF (CONVRG .AND. .NOT. MARQ) GOTO 1000
*ENDIF
*IF DEF,AELIB5
      IF (CONVRG .AND. .NOT. MARQ) GOTO 1000
*ENDIF
C
C.....IF CONVERGENCE ACHIEVED AND MARQ. PARAM. WAS BEING USED, CONTINUE
C.....ITERATING WITHOUT USING PARAM. (SET U=0) UNTIL CONVERGENCE IS
C.....AGAIN ACHIEVED.
C
*IF -DEF,AELIB5
      IF (CONVRG .AND. MARQ) MARQ = .FALSE.
*ENDIF
*IF DEF,AELIB5
      IF (CONVRG .AND. MARQ) MARQ = .FALSE.
*ENDIF
C
 500  CONTINUE
C
C     ....PROCESS HAS NOT CONVERGED IN "NITER" ITERATIONS.
C     ....PRINT ERROR MESSAGE AND TERMINATE AS IF CONVERGENCE HAS BEEN
C     ....ACHIEVED.
C
          GOTO 2010
C
 1000 CONTINUE
C
C     TERMINATE LSQQ BY CALCULATING FITTED FUNCTION VALUES, SUM OF
C     SQUARES OF RESIDUALS, AND THE ERROR MATRIX
C     WE STILL DETERMINE, BY TESTING "ADERIV", WHICH TYPE OF CALL TO
C     MAKE TO AUX, EVEN THOUGH WE WANT ONLY FUNCTION VALUES HERE.
C     THIS IS DONE TO MAKE LIFE EASIER FOR THE USER WRITING AUX.
C
      IF (ADERIV) CALL AUX(X,NRX,YF,PARAM,N,IERAUX)
      IF (.NOT. ADERIV) CALL AUX(X,NRX,YF,PARAM,N,E,NE,IERAUX)
C
C     ....CHECK ERROR FLAG AND EXIT IF SET
C
          IF (IERAUX .NE. 0) GOTO 2120
      S = 0.
      DO 1010 I=1,N
        BB=Y(I)-YF(I)
        S=S+BB*BB*W(I)
 1010 CONTINUE
          IF (LIST .NE. 2 .AND. LIST .NE. 0) PRINT 6003,(PARAM(I),I=1,M)
          IF (LIST .EQ. 2) PRINT 6004, (PARAM(I), I=1,M)
          IF (LIST .NE. 0) PRINT 6011, S
C
C.....CALCULATE FACTOR BY WHICH TO MULTIPLY INVERSE NORMAL MATRIX TO
C.....GET ERROR MATRIX (ERROR MATRIX IS MEANINGLESS IF N .LE. MU)
C
      IF (IWT .EQ. 1) ST = 1.
      IF ((IWT .EQ. 0) .AND. (N .LE. MU)) ST = 0.
      IF ((IWT .EQ. 0) .AND. (N .GT. MU)) ST = S * N / (WTOT * (N - MU))
C
C.....CALCULATE ERROR (VARIANCE-COVARIANCE) MATRIX WHICH IS,
C..... A, FOR WEIGHTED FIT - THE INVERSE OF THE NORMAL MATRIX,
C..... B, FOR UNWEIGHTED FIT - THE INVERSE OF THE NORMAL MATRIX TIMES AN
C.....    ESTIMATE OF THE DATA VARIANCE. THE RESIDUAL VARIANCE OF THE
C.....    FIT IS USED TO ESTIMATE THE DATA VARIANCE.
C
      DO 1020 I=1,M
      DO 1020 J=1,M
 1020 E(I,J)=0.
C
C.....IF EXACT SOLUTION FOUND RETURN WITH ZEROED ERROR MATRIX
C
      IF (S .EQ. 0.0) GOTO 3000
C
      DO 1050 I=1,MU
        IND1=I3+I
        L=WS(IND1)
        DO 1040 J=1,I
          BB=0.
            DO 1030 K=I,MU
            KK=K*(K-1)/2
            IND2=I5+KK+I
            IND3=I5+KK+J
            BB=BB+WS(IND2)*WS(IND3)
 1030     CONTINUE
          IND2=I3+J
          LL=WS(IND2)
          E(L,LL)=E(LL,L)=BB*ST
 1040   CONTINUE
 1050 CONTINUE
C
C     CALCULATE MODIFIED CORRELATION MATRIX,THAT IS,STANDARD DEVIATION
C     ON DIAGONAL AND CORRELATION COEFFICIENTS BELOW DIAGONAL
C     LEAVE COVARIANCES ABOVE DIAGONAL
C
      DO 1060 J=1,MU
        IND1=I3+J
        K=WS(IND1)
        E(K,K)=SQRT(E(K,K))
 1060 CONTINUE
C
      IF(MU.EQ.1) GOTO 3000
      DO 1080 J=2,MU
        IND1=I3+J
        I=WS(IND1)
         KK=J-1
        DO 1070 K=1,KK
          IND2=I3+K
          L=WS(IND2)
          E(I,L)=       E(I,L)/(E(I,I)*E(L,L))
 1070   CONTINUE
 1080 CONTINUE
C
      GOTO 3000
C
C.....NON-FATAL ERRORS
C
 2000 CONTINUE
C
C.....IOPT IS NOT -2, -1, 0 OR 1
C
      IER=1
      IF(.NOT. ALERPR(IROUT,IER)) GOTO 2005
         PRINT 7100
         CALL TRACEB(0)
 2005 CONTINUE
      IOPT = 0
      GOTO 10
C
 2010 CONTINUE
C
C.....NON-CONVERGENCE
C
      IER=2
      IF(.NOT.ALERPR(IROUT,IER)) GOTO 2015
         PRINT 7200, NITER,EMAX
         CALL TRACEB(0)
 2015 CONTINUE
C
C.....TERMINATE LSQQ AS IF CONVERGENCE ACHIEVED
C
      GOTO 1000
C
C
C.....FATAL ERRORS
C
 2100 CONTINUE
C
C.....ILLEGAL PARAMETERS - WEIGHTS
C
      IER=3
      IF(.NOT.ALERPR(IROUT,IER)) GOTO 2105
         PRINT 7300
         CALL TRACEB(0)
 2105 CONTINUE
      GOTO 2200
C
 2110 CONTINUE
C
C.....TROUBLE IN SOLUTION OF NORMAL SYSTEM
C.....SET YF INDEFINITE, RETAIN  E
C
      IER=4
      IF(.NOT.ALERPR(IROUT,IER)) GOTO 2115
         PRINT 7400, BB,I
         CALL TRACEB(0)
 2115 CONTINUE
      CALL PRESET(YF,N,RINDEF)
      S = RINDEF
      GOTO 3000
C
 2120 CONTINUE
C
C.....USER TERMINATION IN AUX
C
      IER = 10
      IF (IERAUX .GT. 10) IER = IERAUX
C
 2200 CONTINUE
C
C.....SET UP INDEFINITES FOR ERROR EXIT
C
      CALL PRESET(YF,N,RINDEF)
      S = RINDEF
      DO 2210 I = 1, MU
          CALL PRESET(E(1,I),MU,RINDEF)
 2210 CONTINUE
C
 3000 CONTINUE
      RETURN
      END
