*DECK NL2INT
      SUBROUTINE NL2INT(IOPT, X, NRX, Y, W, YF, P, NC, N, NP, AUX, E,
     ,                  NE, S, RERR, WS, IER)
C
C***********************************************************************
C WEIGHTED LEAST SQUARES FITTING OF DATA TO A FUNCTION OF 1 OR MORE
C INDEPENDENT VARIABLES WITH OR WITHOUT USER SUPPLIED PARTIAL
C DERIVATIVES.  WRITTEN TO TAKE AS INPUT AND RETURN AS OUTPUT THE
C PARAMETERS OF THE CALLING SEQUENCE NORMALLY USED FOR MLSQQ BUT TO
C CALL NL2SOL OR NL2SNO TO DO THE FITTING OF THE FUNCTION.
C
C IOPT--INTEGER INPUT VARIABLE INDICATING WHETHER OR NOT DERIVATIVES ARE
C       SUPPLIED.  IF THEY ARE NOT THEY WILL BE APPROXIMATED.
C       0,2  NO DERIVATIVES SUPPLIED
C       -1,1 DERIVATIVES SUPPLIED
C
C X--REAL ARRAY OF VALUES OF INDEPENDENT VARIABLE(S)
C
C NRX--INTEGER NUMBER OF ROWS DIMENSIONED FOR X IN THE CALLING PROGRAM
C
C Y--REAL ARRAY OF VALUES OF DEPENDENT VARIABLE
C
C W--REAL ARRAY OF WEIGHTS FOR DEPENDENT VARIABLE
C
C YF--REAL OUTPUT ARRAY OF FITTED VALUES OF DEPENDENT VARIABLE
C
C PARAM--REAL ARRAY OF PARAMETERS FOR USER SUPPLIED FUNCTION.  INITIAL
C        VALUES ARE PASSED TO NL2INT AND FINAL VALUES ARE RETURNED
C
C NC--INTEGER ARRAY SPECIFYING WHETHER A PARAMETTER IS TO BE FIXED OR
C     VARIED
C     NC(I) = 0  MEANS P(I) IS FIXED
C     NC(I) = 1  MEANS P(I) IS VARIED
C
C N--INTEGER NUMBER OF DATA POINTS
C
C M--INTEGER NUMBER OF PARAMETERS IN THE USER-SUPPLIED FUNCTION
C
C AUX--USER SUPPLIED FUNCTION WHICH CALCULATES THE FITTING FUNCTION
C
C E--REAL ARRAY RETURNED FROM NL2INT CONTAINING THE VARIANCE-COVARIANCE
C    MATRIX
C
C NE--INTEGER NUMBER OF ROWS OF E AS DIMENSIONED IN THE CALLING PROGRAM
C
C S--REAL VALUE RETURNED FROM NL2INT CONTAINING THE RESIDUAL SUM OF
C    SQUARES
C
C RERR--RELATIVE ERROR NECESSARY FOR CONVERGENCE
C
C WS--REAL WORKING STORAGE ARRAY WHICH MUST BE DIMENSIONED AT LEAST
C     93 + MU(3MU + 35) / 2 + N(MU + NIV + 6) + M(N + 2)
C     WHERE MU IS THE NUMBER OF VARING PARAMETERS
C     START                               LENGTH             INFORMATION
C     POSITION
C     1                                   MU                 VARYING
C                                                            PARAMETERS
C     MU + 1                              M                  P
C     MU + M + 1                          M                  NC
C     2M + MU + 1                         93 + N(MU + 3) +   V (FOR
C                                         MU(3MU + 33) / 2   NL2SOL)
C     94 + N(MU + 3) + MU(3MU + 35)       NIV * N            X
C     / 2 + 2M
C     94 + N(MU + 3 + NIV) + 2M +         N                  Y
C     MU(3MU + 35) / 2
C     94 + N(MU + 4 + NIV) + 2M +         N                  SQRT(W)
C     MU(3MU + 35) / 2
C     94 + N(MU + 5 + NIV) + 2M +         N                  YF
C     MU(3MU + 35) / 2
C     94 + N(MU + 6 + NIV) + 2M +         N * M              E
C     MU(3MU + 35) / 2
C
C IER--INTEGER VALUE RETURNED FROM NL2INT CONTAINING THE ERROR FLAG
C      VALUE
C
C REVISION HISTORY
C ****************
C
C WRITTEN BY MARK KENT                        SUMMER 1981
C
C REVISED TO ALLOW USE BY NEW VERSION OF NL2SOL
C         BY JANE FISHER AND C. A. WILLS      MAY 1982
C
C***********************************************************************
C
      DIMENSION E(NE,1), INDEX(8), IV(100), NC(1), P(1), W(1), WS(1),
     ,          X(NRX,1), Y(1), YF(1)
      LOGICAL ALERPR
C
C ... COMMON BLOCK VARIABLES
C     NIV--NUMBER OF INDEPENDENT VARIABLES
C     IV--INTEGER WORKING STORAGE (SHOULD BE DIMENSIONED M + 60)
C     LIST-- = 0 NO PRINTING
C            = 1 PRINT SUMMARY LINE AFTER EACH ITERATION (PRINTER WIDTH)
C            = 2 PRINT SUMMARY LINE AFTER EACH ITERATION (TERMINAL
C                WIDTH)
C            = 3 SAME AS LIST = 1
C            = 4 SAME AS LIST = 1 PLUS INITIAL AND FINAL PARAMETER
C                VALUES, FINAL SUMMARY, AND VARIANCE-COVARIANCE MATRIX
C     NITER--MAXIMUM NUMBER OF ITERATIONS
C     IWT--SET BY NL2INT TO INDICATE WEIGHTED FIT (= 1) OR UNWEIGHTED
C          FIT (= 0)
C
      COMMON / NL2COM / NIV, IV
      COMMON / MLSQOPT / LIST , NITER , IWT
      EXTERNAL CALCR , CALCJ , AUX
      DATA LIST /1/, NITER/100/, NIV/1/
C
C ... CHECK FOR VALID INPUT PARAMETERS
C
*CALL STATF
      IROUT = 11124
      IF( IOPT .LT. -2  .OR.  IOPT .GT. 1 ) GOTO 2000
      IF( N .LE. 0 .OR. N .LT. NP ) GOTO 2110
C
C ... LOAD WS WITH NEEDED INFO .
C
C ... LOAD THE FIRST PLACES OF WS WITH THE
C ... VARYING PARAMETERS .
C
 1    IWT = 0
      IER = 0
      K=0
      DO 2 I = 1 , NP
         IF( NC(I) .NE. 1 ) GOTO 2
         K = K + 1
         WS(K) = P(I)
 2    CONTINUE
      IF(K .NE. 0 ) GOTO 5
C
C ... THERE ARE NO VARYING PARAMETERS
C
      IF( IOPT * (IOPT + 2) .NE. 0 ) CALL AUX(X, NRX, YF, P, N, E, NE,
     ,                                        IER)
      IF( IOPT * (IOPT + 2) .EQ. 0 ) CALL AUX(X, NRX, YF, P, N, IER)
      IF( IER .GE. 10 ) GOTO 2170
      GOTO 2030
C
C ... LOAD THE NEXT NP PLACES OF WS WITH ALL
C ... THE PARAMETERS . THESE ARE PASSED TO AUX
C ... BUT NL2SOL ONLY KNOWS ABOUT THE VARYING
C ... PARAMETERS .
C
C ... NP1 IS THE NUMBER OF VARYING PARAMETERS
C
 5    NP1 = K
      DO 6 I = 1 , NP
         K = K + 1
         WS(K) = P(I)
 6    CONTINUE
C
C ... LOAD WS WITH THE FLAG ARRAY FOR VARYING PARAMETERS .
C ... THIS IS PASSED TO CALCR TO BE USED TO IDENTIFY THE
C ... VARYING PARAMETERS SO THE LIST OF PARAMETERS THAT
C ... AUX USES GETS UPDATED WITH THE NEW VALUES IN THE
C ... FIRST NP1 PLACES OF WS .
C
      DO 7 I = 1 , NP
         K = K + 1
         WS(K) = NC(I)
 7    CONTINUE
C
C ... JV IS THE LOCATION IN WS WHERE THE NL2SOL ARRAY
C ...  V IS LOCATED ( ACTUALLY WS(JV+1) ) .
C ... JX IS WHERE V ENDS , X STARTS AFTER V .
C
C ... HERE WE LOAD X , Y , AND W IN WS .
C
      IV(1) = 0
      JV = K
      JX = 93 + JV + N*(NP+3) + NP*(3*NP + 33)/2
      K = JX
      DO 10 J = 1 , NIV
         DO 10 I = 1 , N
            K = K + 1
            WS(K) = X( I , J )
 10   CONTINUE
      WT = W(1)
      DO 12 I = 1 , N
         WS( K + I ) = Y(I)
         IF(LEGVAR(W(I)) .NE. 0) GOTO 2100
         IF( W(I) .LE. 0.0 ) GOTO 2100
         WS( K + N + I ) = SQRT ( W(I) )
         IF( W(I) .NE. WT ) IWT = 1
 12   CONTINUE
C
C ...                           V   ENDS AT WS(INDEX(1))
C ...                           X ENDS AT WS(INDEX(2))
C ...                           Y   ENDS AT WS(INDEX(3))
C ...                           W   ENDS AT WS(INDEX(4))
C ...                           F   ENDS AT WS(INDEX(5))
C ...                           E STARTS AT WS(INDEX(5)+1)
C
      INDEX(1) = JX
      INDEX(2) = K
      INDEX(3) = INDEX(2) + N
      INDEX(4) = INDEX(3) + N
      INDEX(5) = INDEX(4) + N
      INDEX(6) = NP
      INDEX(8)= 0
C
C ... IOPT SPECIFIES DERIVATIVES OR NOT .
C ... IOPT = 0 OR -2 MEANS APPX JACOBIAN
C
      IDER = IOPT*(IOPT + 2)
      INDEX(7) = IDER
C
C ... SET IV AND V ARRAYS TO DEFAULT VALUES EXCEPT FOR THE MAXIMUM
C ... NUMBER OF ITERATIONS, MAXIMUM NUMBER OF FUNCTION EVALUATIONS,
C ... RELATIVE ERROR CRITERION FOR THE PARAMETERS, AND LISTING FLAGS
C
      CALL DFAULT( IV , WS( JV + 1 ) )
      IV(18) = NITER
      IV(17) = NITER*1.33333333333333
      IV(20) = 0
      WS(JV + 33) = RERR/2.0
        IF (LIST * (LIST - 4) .EQ. 0) GOTO 15
          IV(14) = 0
          IV(22) = 0
          IV(23) = 0
          IV(24) = 0
          IF(LIST .EQ. 2) IV(19) = -1
          GOTO 16
 15     IF (LIST .EQ. 0) IV(21) = 0
C
C ... SOLVE THE WEIGHTED LEAST SQUARES PROBLEM.  IF A SOLUTION WAS NOT
C ... FOUND GO TO APPROPRIATE ERROR EXIT.
C
 16   IF (IDER .EQ. 0)
     ,   CALL NL2SNO(N, NP1, WS, CALCR, IV, WS(JV+1), INDEX, WS, AUX)
      IF(IDER .NE. 0)
     ,   CALL NL2SOL(N, NP1, WS, CALCR, CALCJ, IV, WS(JV+1), INDEX, WS,
     ,               AUX)
      IF( INDEX(8) .LT. 0 ) GOTO 2170
      IF( IV(1) .LT. 3 .OR. IV(1) .GT. 10 ) GOTO 2160
C
C ... SET THE VARYING ELEMENTS OF THE P ARRAY TO THE VALUES FOUND BY
C ... NL2SOL
C
 20   L = 0
      DO 24 I = 1 , NP
         IF( NC(I) .NE. 1 ) GOTO 22
            L = L + 1
            P(I) = WS(L)
            GOTO 24
 22      P(I) = WS( NP1 + I )
 24   CONTINUE
C
C ... CHECK FOR NONCONVERGENCE
C
        IF( IV(1) .EQ. 9 .OR. IV(1) .EQ. 10) GOTO 2010
              IF( IV(1) .EQ. 7) GOTO 2125
                 IF( IV(1) .EQ. 8) GOTO 2130
C
C ... S IS THE RESIDUAL SUM OF SQUARES WHICH IS IN
C ... V(10) .
C ... WE STORE THE FITTED VALUES ( RESIDING IN WS ) IN
C ... ARRAY YF .
C
 25   JF = INDEX(4)
      S = WS(JV + 10) * 2.0
      DO 26 I = 1 , N
 26      YF(I) = WS( JF + I )
C
C ... IF IV(26)<0 THEN TROUBLE EXPERIENCED IN NL2
C ... IN COMPUTING THE JACOBIAN . OTHERWISE IV(26)
C ... IS THE POSITION IN V() WHERE THE VARIANCE
C ... TRIANGULAR MATRIX IS STORED . THIS GETS STORED
C ... IN E( , ) ACCORDING TO WHAT THE MLSQQ USER EXPECTS .
C
      K = IV(26)
      IF( K .LT. 0 ) GOTO 2020
         K = K + JV
         L = 0
        DO 36 I = 1 , NP
           IF( NC(I) .EQ. 0 ) GOTO 30
              DO 32 J = 1 , I
                 IF( NC(J) .EQ. 0 ) GOTO 32
                  E(J,I) = WS( K + L )
                  L = L + 1
 32           CONTINUE
               E(I,I) = SQRT( E(I,I) )
              GOTO 36
 30        DO 34 J = 1 , NP
               E(J,I) = 0.0
               E(I,J) = 0.0
 34        CONTINUE
 36      CONTINUE
         IF( NP .LT. 2 ) RETURN
C
C ...    THE LOWER TRI PART OF E( , ) IS COMPUTED
C ...    USING THE UPPER TRI PART .
C
         DO 40 I = 2 , NP
            IF( NC(I) .EQ. 0 ) GOTO 40
            IM1 = I - 1
            DO 42 J = 1 , IM1
               IF( NC(J) .EQ. 0 ) GOTO 42
               E(I,J) = E(J,I)/( E(I,I)*E(J,J) )
               IF( E(I,I) * E(J,J) .EQ. 0.0 ) E(I,J) = 0.0
 42         CONTINUE
 40      CONTINUE
      RETURN
C ... NON-FATAL ERRORS
C
 2000 CONTINUE
C
C ... IOPT IS NOT -2, -1, 0 OR 1
C
      IER=1
      IF(.NOT. ALERPR(IROUT,IER)) GOTO 2005
         PRINT 7100
         CALL TRACEB(0)
 2005 CONTINUE
      IOPT = 0
      GOTO 1
C
 2010 CONTINUE
C
C ... NON-CONVERGENCE.  TERMINATE NL2INT AS IF CONVERGENCE ACHIEVED.
C
      IER=2
      IF(.NOT.ALERPR(IROUT,IER)) GOTO 2015
         PRINT 7200, IV(31)
         CALL TRACEB(0)
 2015 CONTINUE
      GOTO 25
C
 2020 CONTINUE
C
C ... THE COVARIANCE MATRIX COULD NOT BE COMPUTED BY NL2SOL
C
      IF( IER .EQ. 0 ) IER = 3
      IF(.NOT. ALERPR(IROUT, IER)) GOTO 2025
        PRINT 7300
        CALL TRACEB(0)
 2025 CONTINUE
      CALL SETBAD(E, NE, NP, NP)
      GOTO 3000
C
 2030 CONTINUE
C
C ... NO VARYING PARAMETERS SPECIFIED
C
      IER = 4
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 2020
        PRINT 7400
        CALL TRACEB(0)
        GOTO 2020
C
C ... FATAL ERRORS
C
 2100 CONTINUE
C
C ... ILLEGAL PARAMETERS - WEIGHTS
C
      IER = 5
      IF(.NOT.ALERPR(IROUT,IER)) GOTO 2105
         PRINT 7500
         CALL TRACEB(0)
 2105 CONTINUE
      GOTO 2200
C
 2110 CONTINUE
C
C ... ILLEGAL PARAMETERS - N OR M
C
      IER = 6
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 2200
        PRINT 7600, N, NP
        CALL TRACEB(0)
        GOTO 2200
C
 2125 CONTINUE
C
C . . . TOO MANY PARAMETERS NEAR P
C
      IER = 7
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 2200
          PRINT 7700
          CALL TRACEB(0)
          GOTO 2200
C
 2130 CONTINUE
C
C . . . CONVERGED ON A SADDLE POINT
C
      IER = 8
      IF (.NOT. ALERPR(IROUT,IER)) GOTO 2140
         PRINT 7800
         CALL TRACEB(0)
 2140 CONTINUE
      GOTO 2200
C
 2160 CONTINUE
C
C . . . SYSTEM ERROR
C
      IER = 9
      IF (.NOT. ALERPR(IROUT,IER)) GOTO 2200
         PRINT 7900
         CALL TRACEB(0)
         GOTO  2200
C
 2170 CONTINUE
C
C ... USER TERMINATION IN AUX.  THIS SHOULD TAKE CARE OF THE RETURN
C ... CODES IV(1) = 13 AND IV(1) = 15 IN NL2SOL.
C
      IER = 10
      IF (INDEX(8) .LT. 10) IER = -INDEX(8)
C
 2200 CONTINUE
C
C ... SET UP INDEFINITES FOR ERROR EXIT
C
      CALL SETBAD(YF, N, 1, 1)
      CALL SETBAD(S, 1, 1, 1)
      CALL SETBAD(E, NE, NP, NP)
C
 3000 CONTINUE
      RETURN
*IF -DEF,AELIB5
 7100 FORMAT(" ***INVALID OPTION SELECTED - IOPT=0 ASSUMED")
 7200 FORMAT(" ***NO CONVERGENCE AFTER ",I4," ITERATIONS.  ")
 7300 FORMAT(" ***VARIANCE-COVARIANCE MATRIX NOT CALCULATED ")
 7400 FORMAT(" ***NO VARYING PARAMETERS SPECIFIED ")
 7500 FORMAT(" ***WEIGHTS ARE ZERO, NEGATIVE OR UNDEFINED ")
 7600 FORMAT(" ***VALUE OF N OR M IS ILLEGAL  N = ", I4, " M = ", I4)
 7700 FORMAT(" ***TOO MANY PARAMETERS AT P ")
 7800 FORMAT(" ***CONVERGED ON A NONCRITCAL POINT ")
 7900 FORMAT(" ***INTERNAL ERROR IN NL2INT ")
*ENDIF
*IF DEF,AELIB5
 7100 FORMAT(' ***INVALID OPTION SELECTED - IOPT=0 ASSUMED')
 7200 FORMAT(' ***NO CONVERGENCE AFTER ',I4,' ITERATIONS.  ')
 7300 FORMAT(' ***VARIANCE-COVARIANCE MATRIX NOT CALCULATED ')
 7400 FORMAT(' ***NO VARYING PARAMETERS SPECIFIED ')
 7500 FORMAT(' ***WEIGHTS ARE ZERO, NEGATIVE OR UNDEFINED ')
 7600 FORMAT(' ***VALUE OF N OR M IS ILLEGAL  N = ', I4, ' M = ', I4)
 7700 FORMAT(' ***TOO MANY PARAMETERS AT P ')
 7800 FORMAT(' ***CONVERGED ON A NONCRITCAL POINT ')
 7900 FORMAT(' ***INTERNAL ERROR IN NL2INT ')
*ENDIF
      END
      SUBROUTINE CALCR( N , NP , P , NF , R , INDEX , WS , AUX )
C.......................................................................
C THIS SUBROUTINE CALLS AUX TO CALCULATE YF AT THE CURRENT PARAMETER
C VALUES IN ORDER TO PROVIDE NL2SOL WITH THE FUNCTION VALUES
C R(I) = SQRT(W(I)) * (Y(I) - YF(I))
C
C INPUT:  N--THE NUMBER OF DATA POINTS
C         NP--THE NUMBER OF VARYING PARAMETERS
C         P--THE CURRENT VARYING PARAMETER VALUES
C         NF--INVOCATION COUNT FOR CALCR
C         INDEX--(1) ELEMENT OF WS AFTER WHICH X STARTS
C                (2) ELEMENT OF WS AFTER WHICH Y STARTS
C                (3) ELEMENT OF WS AFTER WHICH SQRT(W) STARTS
C                (4) ELEMENT OF WS AFTER WHICH YF IS TO BE STORED
C                (5) ELEMENT OF WS AFTER WHICH E IS TO BE STORED
C                (6) NUMBER OF PARAMETERS BOTH VARYING AND FIXED
C                (7) = 0 IF USER HAS NOT PROVIDED PARTIAL DERIVATIVES
C                        IN AUX
C                    = 1 IF USER HAS PROVIDED PARTIAL DERIVATIVES IN AUX
C         WS--ARRAY CONTAINING INFORMATION NEEDED BY AUX BUT NOT
C             BY NL2SOL.  INDEXED BY ARRAY INDEX.  ELEMENTS
C             WS(INDEX(6) + NP + 1) TO WS(2INDEX(6) + NP) ARE THE ARRAY
C             NC ORIGINALLY PROVIDED BY THE USER.
C         AUX--NAME OF SUBROUTINE PROVIDED BY USER TO CALCULATE YF
C
C OUTPUT:  INDEX--(8) SAVE CURRENT INVOCATION COUNT FOR CALCR OR
C                     NEGATIVE OF ERROR NUMBER RETURNED FROM AUX
C          NF-- = 0 IF ERROR ENCOUNTERED IN AUX; UNCHANGED OTHERWISE
C          R--THE FUNCTION VALUES
C          WS--ELEMENTS WS(NP + 1) TO WS(NP + INDEX(6)) ARE CURRENT
C              PARAMETER VALUES INCLUDING THOSE FIXED BY THE USER.
C              ELEMENTS WS(INDEX(4) + 1) TO WS(INDEX(4) + N) ARE
C              FUNCTION VALUES AS CALCULATED BY AUX AT CURRENT PARAMETER
C              VALUES
C.......................................................................
C
      DIMENSION P(1) , R(1) , INDEX(8) , WS(1)
      INDEX(8) = NF
      JX1 = INDEX(1) + 1
      JY  = INDEX(2)
      JW  = INDEX(3)
      JF  = INDEX(4)
      JE1 = INDEX(5) + 1
      NPS = INDEX(6)
C
C ... HERE WE UPDATE THE PARAMETERS THAT AUX USES WITH
C ... THE PARAMETERS THAT NL2SOL USES.
C
      IER = 0
      L=0
      K= NPS + NP + 1
      DO 2 I = 1 , NPS
         IF(WS(K) .NE. 1.0)GOTO 1
            L = L + 1
            WS( NP + I ) = P(L)
 1       K = K + 1
 2    CONTINUE
      JF1 = JF + 1
      IF( INDEX(7) .EQ. 0 )
     ! CALL AUX( WS(JX1) , N , WS(JF1) , WS(NP+1) , N , IER )
      IF( INDEX(7) .NE. 0 )
     ! CALL AUX(WS(JX1),N,WS(JF1),WS(NP+1),N,WS(JE1),N,IER)
      IF( IER .NE. 0 ) GOTO 4
C
C ... COMPUTE THE RESIDUAL THAT NL2SOL REQUIRES FOR
C ... MINIMIZATION.
C
      DO 3 I = 1 , N
         R(I) = ( WS( JF+I ) - WS( JY+I ) )*WS( JW+I )
 3    CONTINUE
      RETURN
C
C ... ERROR DURING CALCULATIONS IN AUX.  SETTING NF TO 0 MAY ALLOW
C ... NL2SOL TO CONTINUE WITH REVISED PARAMETER VALUES
C
 4    NF = 0
      INDEX(8) = -IER
      RETURN
      END
      SUBROUTINE CALCJ(N , NP , P , NF , XJACOB , INDEX , WS , AUX)
C.......................................................................
C THIS SUBROUTINE PROVIDES NL2SOL WITH THE JACOBIAN MATRIX OF PARTIAL
C DERIVATIVES AT THE REQUESTED PARAMETER VALUES.  THESE MAY HAVE TO BE
C CALCULATED BY A CALL TO AUX OR THEY MAY BE AVAILABLE FROM A PREVIOUS
C CALL.
C
C INPUT:  N--THE NUMBER OF DATA POINTS
C         NP--THE NUMBER OF VARYING PARAMETERS
C         P--THE CURRENT VARYING PARAMETER VALUES
C         NF--INVOCATION COUNT FOR CALCR WHEN IT WAS CALLED WITH THESE
C             PARAMETER VALUES
C         INDEX--(1) ELEMENT OF WS AFTER WHICH X STARTS
C                (3) ELEMENT OF WS AFTER WHICH W STARTS
C                (4) ELEMENT OF WS AFTER WHICH YF IS TO BE STORED
C                (5) ELEMENT OF WS AFTER WHICH E IS TO BE STORED
C                (6) NUMBER OF PARAMETERS BOTH VARYING AND FIXED
C         WS--ARRAY CONTAINING INFORMATION NEEDED BY AUX BY NOT BY
C             NL2SOL.  INDEXED BY ARRAY INDEX.
C
C OUTPUT:  XJACOB--THE JACOBIAN MATRIX FOR VARYING PARAMETERS ONLY
C          INDEX--(8) NEGATIVE OF ERROR NUMBER  RETURNED FROM AUX;
C                     UNCHANGED IF NO ERROR IN AUX
C          WS--ELEMENTS WS(NP + 1) TO WS(NP + INDEX(6)) ARE CURRENT
C              PARAMETER VALUES INCLUDING THOSE FIXED BY THE USER
C.......................................................................
C
      DIMENSION P(1) , INDEX(8) , WS(1) , XJACOB(1)
      IER = 0
      JX1 = INDEX(1) + 1
      JW1 = INDEX(3) + 1
      JF1 = INDEX(4) + 1
      JE1 = INDEX(5) + 1
      NPS = INDEX(6)
C
C ... UPDATE THE PARAMETERS THAT AUX USES WITH THE PARAMETERS REQUIRED
C ... BY NL2SOL
C
      IF (INDEX(8) .EQ. NF) GOTO 10
C
      L = 0
      K = NPS + NP + 1
      DO 2 I=1,NPS
        IF (WS(K) .NE. 1.0) GOTO 1
        L = L + 1
        WS(NP + I) = P(L)
 1      K = K + 1
 2    CONTINUE
C
C ... IF THE INVOCATION COUNT FOR CALCR WHEN IT WAS CALLED WITH THESE
C ... PARAMETER VALUES (NF) IS THE SAME AS THE INVOCATION COUNT FOR THE
C ... LAST CALL TO CALCR (INDEX(8)) THEN THE JACOBIAN ALREADY EXISTS
C ... STARTING AT WS(JE1).  OTHERWISE IT SHOULD BE CALCULATED BY CALLING
C ... AUX.
C
      CALL AUX( WS(JX1), N, WS(JF1), WS(NP+1), N, WS(JE1), N, IER)
      IF (IER .NE. 0) GOTO 20
C
C ... THIS LOOKS MORE COMPLICATED THAN WOULD BE
C ... EXPECTED BUT JN AND JE1 ARE USED TO SKIP OVER
C ... ANY UNDEFINED PARTS OF E( , ) STORED IN WS
C ... WHICH WILL OCCUR IF ANY PARAMETERS ARE FIXED.
C
 10   NC = NPS + NP + 1
      K = 0
      DO 13 L = 1 , NPS
         JN = JE1 + N - 1
         IF (WS(NC) .NE. 1) GOTO 12
         DO 11 I = JE1 , JN
            K = K + 1
            XJACOB(K) = -WS(I) * WS(JW1 + I - JE1)
 11      CONTINUE
 12      JE1 = JE1 + N
         NC = NC + 1
 13   CONTINUE
      RETURN
C
C ... ERROR IN CALCULATIONS IN AUX.  SETTING NF TO 0 FORCES TERMINATION
C ... OF NL2SOL.
C
 20   NF = 0
      INDEX(8) = -IER
      RETURN
      END
