*DECK ORDERF
      SUBROUTINE ORDERF(A,II,JJ,TAG,IER)
C
C THIS SUBROUTINE SORTS ARRAY A INTO INCREASING ORDER, FROM A(II)
C TO A(JJ).  ARRAY TAG IS PERMUTED THE SAME AS ARRAY A.  ALSO,
C SINCE THE ORDERING IS BY INTEGER SUBTRACTION, FLOATING POINT
C NUMBERS MUST BE IN NORMALIZED FORM.  THE SORTING METHOD USED
C BY THIS SUBROUTINE IS HOARE'S QUICKSORT, WITH THE PIVOT ELEMENT
C BEING CHOSEN AS THE MEDIAN OF THE FIRST, LAST AND MIDDLE ELEMENTS
C OF THE PARTITION UNDER CONSIDERATION.
C
C ARGUMENT LIST
C     A      - THE ARRAY TO BE SORTED; DEFINED AS INTEGER IN THE
C              ROUTINE BUT A REAL ARRAY MAY BE PASSED
C     II,JJ  - INDICES WHICH POINT TO THE PART OF A THAT IS TO BE
C              SORTED (I.E. A(II) TO A(JJ) IS SORTED)
C     TAG    - THE TAG ARRAY WHICH IS TO BE PERMUTED THE SAME WAY
C              AS A; AS WITH A IT IS DEFINED AS INTEGER IN THE
C              ROUTINE BUT A REAL ARRAY MAY BE PASSED
C     IER    - ERROR FLAG SET IN ORDERF.
C            = 0, NO ERRORS
C            = 1, ARRAY INDICES ARE INVALID
C
      INTEGER A(1),II,JJ,TAG(1),IER
C
C ERROR EXITS
C     THERE IS ONE FATAL ERROR CONDITION THAT IS DETECTED BY ORDERF:
C     1.  IER = 1
C         IF "II" IS GREATER THAN "JJ", OR ONE OR BOTH OF THEM ARE
C         NEGATIVE OR ZERO, WE CANNOT TELL WHAT THE USER WANTS SORTED.
C         HENCE, AN ERROR FLAG IS SET (A MESSAGE WILL BE PRINTED IF
C         THE USER HASN'T DISABLED IT) AND CONTROL IS RETURNED TO THE
C         USERS PROGRAM.
C
C REVISION HISTORY OF ORDERF
C
C ORDERF/ORDERB IS A COPY OF SORTAG/DESORT WITH AN ERROR FLAG.
C REVISED BY P. BUMBULIS MAY, 1979.
C     TO - 1. ADD AN ERROR FLAG TO THE ARGUEMENT LIST
C          2. COMMENT THE ACTUAL PROGRAM
C
C
C OTHER VARIABLES USED/FUNCTIONS CALLED:
C IU,IL  - STACK CONTAINING THE UPPER AND LOWER BOUNDARIES OF A
C          PARTITION.  IF IU & IL CONTAIN ROOM FOR K ENTRIES,
C          THEN THE ALGORITHM WILL SORT UP TO 2**(K+1)-1 ELEMENTS.
C M      - POINTER TO THE NEXT STACK ENTRY TO BE USED; IF IT GOES
C          TO 0 WE ARE DONE
C I,IJ,J - POINTERS TO THE FIRST, PIVOT AND LAST ELEMENTS IN THE
C          "CURRENT" PARTITION, RESPECTIVELY.
C T      - COPY OF THE PIVOT ELEMENT
C TT,TG  - TEMPORARY VARIABLES TO HOLD ELEMENTS OF A & TAG DURING
C          EXCHANGES
C K,L    - POINTERS USED TO HELP PARTION THE CURRENT PARTITION
C J1     - TEMPORARY VARIABLE
C IFLAG  - ENABLES THE ROUTINE TO TEST WHETHER ORDERF OR ORDERB
C          WAS CALLED
C        = 1, ORDERF WAS CALLED
C        = -1, ORDERB WAS CALLED
C ALERPR - TESTS WHETHER OR NOT THE USER WANTS THE ERROR
C          MESSAGE TO BE PRINTED OR NOT
C IROUT  - AELIB ROUTINE NUMBER FOR ORDERF/ORDERB
C
      INTEGER IU(16),IL(16),M,I,IJ,J,T,TT,TG,K,L,J1,IROUT
      LOGICAL ALERPR
      DATA IROUT/020311/
*CALL STATF
      IFLAG=1
      GO TO 2
C
*IF -DEF,AELIB5
      ENTRY ORDERB
*ENDIF
*IF DEF,AELIB5
      ENTRY ORDERB(A,II,JJ,TAG,IER)
*ENDIF
C
C     ENTRY INTO ORDERF TO SORT THE ARRAYS IN DESCENDING ORDER
C
      IFLAG=-1
C
C     THE SORT PROGRAM PROPER (SORTS ASCENDING)
C
    2 CONTINUE
C
C     TEST IF THE ARRAY INDICES ARE INVALID
C
      IF (II .GT. 0 .AND. JJ .GT. 0 .AND. II .LE. JJ) GO TO 1020
C        THE ARRAY INDICES ARE INVALID
         IER = 1
         IF (.NOT. ALERPR(IROUT,IER)) GO TO 1010
            PRINT 1000, II,JJ
*IF -DEF,AELIB5
 1000       FORMAT(" ***ARRAY INDICES I =",I7,", J =",I7,
     $             " ARE NEGATIVE, ZERO, OR I .GT. J")
*ENDIF
*IF DEF,AELIB5
 1000       FORMAT(' ***ARRAY INDICES I =',I7,', J =',I7,
     $             ' ARE NEGATIVE, ZERO, OR I .GT. J')
*ENDIF
            CALL TRACEB(0)
 1010    CONTINUE
      RETURN
 1020    IER = 0
      M = 1
      I=II
      J=JJ
    5 IF (I .GE. J) GO TO 70
   10 K=I
C
C     FIND THE MEDIAN OF A(I), A(IJ), AND A(J), AND ARRANGE
C     THEM IN ASCENDING ORDER
C
      IJ=(J+I)*.5
      T=A(IJ)
      IF (A(I) .LE. T) GO TO 20
      A(IJ)=A(I)
      A(I)=T
      T=A(IJ)
      TG=TAG(IJ)
      TAG(IJ)=TAG(I)
      TAG(I)=TG
   20 L=J
      IF (A(J) .GE. T) GO TO 40
      A(IJ)=A(J)
      A(J)=T
      T=A(IJ)
      TG=TAG(IJ)
      TAG(IJ)=TAG(J)
      TAG(J)=TG
      IF (A(I) .LE. T) GO TO 40
      A(IJ)=A(I)
      A(I)=T
      T=A(IJ)
      TG=TAG(IJ)
      TAG(IJ)=TAG(I)
      TAG(I)=TG
      GO TO 40
C
C     PARTITION A(I) THRU A(J) SUCH THAT ALL ELEMENTS WITH VALUES
C     LESS THAN A(IJ) PRECEED A(IJ), AND ALL ELEMENTS WITH VALUES
C     GREATER THAN A(IJ) FOLLOW A(IJ).
C
   30 A(L)=A(K)
      A(K)=TT
      TG=TAG(L)
      TAG(L)=TAG(K)
      TAG(K)=TG
   40 L=L-1
      IF(A(L) .GT. T) GO TO 40
      TT=A(L)
   50 K=K+1
      IF(A(K) .LT. T) GO TO 50
      IF (K .LE. L) GO TO 30
C
C     STACK THE LARGEST PARTITION; TREAT SMALLER AS CURRENT ONE
C
      IF (L-I .LE. J-K) GO TO 60
      IL(M)=I
      IU(M)=L
      I=K
      M=M+1
      GO TO 80
   60 IL(M)=K
      IU(M)=J
      J=L
      M=M+1
      GO TO 80
   70 M=M-1
      IF(M .EQ. 0) GO TO 110
      I=IL(M)
      J=IU(M)
C
C     IF A PARTION HAS LESS THAN 11 ELEMENTS IN IT, USE A
C     SIMPLE INSERTION SORT TO ORDER IT
C
   80 IF (J-I .GE. 11) GO TO 10
      IF(I .EQ. II) GO TO 5
      I=I-1
   90 I=I+1
      IF(I .EQ. J) GO TO 70
      T=A(I+1)
      IF (A(I) .LE. T) GO TO 90
      TG=TAG(I+1)
      K=I
  100 A(K+1)=A(K)
      TAG(K+1)=TAG(K)
      K=K-1
      IF(T .LT. A(K)) GO TO 100
      A(K+1)=T
      TAG(K+1)=TG
      GO TO 90
C
C     IF ORDERB WAS CALLED, REARRANGE "A" AND "TAG" SO THAT THEY
C     ARE IN DESCENDING ORDER.
C
 110  IF (IFLAG .GT. 0) GO TO 130
 120  J=II+(JJ-II-1)*.5
      J1=JJ
      DO 140 I=II,J
      T=A(J1)
      A(J1)=A(I)
      A(I)=T
      TG=TAG(J1)
      TAG(J1)=TAG(I)
      TAG(I)=TG
 140  J1=J1-1
  130 RETURN
      END
