*DECK PAKUNP
      SUBROUTINE PAKUNP(SOURCE,LSS,IBS,N,DEST,LDF,IBD)
C
C THIS SUBROUTINE COPIES CONSECUTIVE BIT STRINGS FROM A SOURCE ARRAY
C INTO CONSECUTIVE FIELDS IN A DESTINATION ARRAY,  IGNORING WORD
C BOUNDARIES.  THE FIRST BIT STRING CAN START AT ANY POSITION IN THE
C FIRST WORD OF THE SOURCE ARRAY, AND THE FIRST DESTINATION FIELD CAN
C START AT ANY BIT POSITION IN THE FIRST WORD OF THE DESTINATION ARRAY.
C    IF THE LENGTH OF THE SOURCE BIT STRING IS NOT EQUAL TO THE LENGTH
C OF THE DESTINATION FIELD, THEN THE FOLLOWING CONVENTIONS APPLY;
C    A) IF THE DESTINATION FIELD IS LONGER THAN THE SOURCE STRING, THEN
C       THE SOURCE STRING IS STORED IN THE DESTINATION FIELD RIGHT
C       JUSTIFIED, ZERO-FILLED.
C    B) IF THE DESTINATION FIELD IS SHORTER THAN THE SOURCE STRING,
C       THEN EXCESS BITS ARE DISCARDED FROM THE LEFT HAND END OF THE
C       SOURCE STRING UNTIL IT IS THE SAME LENGTH AS THE DESTINATION
C       FIELD. THE RESULTING TRUNCATED SOURCE STRING IS THEN STORED IN
C       THE DESTINATION FIELD.
C
C
C
C    FORMAL PARAMETERS.
C    *--------*------*
C
C    SOURCE   -    REAL OR INTEGER VARIABLE OR ARRAY CONTAINING THE
C                  SOURCE STRINGS.
C    LSS      -    INTEGER INPUT VARIABLE, THE LENGTH IN BITS OF A
C                  SOURCE STRING. ( 1.LE.LSS.LE.60 )
C    IBS      -    INTEGER INPUT VARIABLE, THE BIT POSITION IN THE
C                  FIRST SOURCE WORD AT WHICH THE FIRST BIT STRING START
C                   (0.LE.IBS.LE.59)
C    N        -    INTEGER INPUT VARIABLE, THE NUMBER OF BIT STRINGS
C                  TO BE COPIED.
C    DEST     -    REAL OR INTEGER OUTPUT VARIABLE OR ARRAY IN WHICH
C                  IS RETURNED THE DESTINATION STRING
C    LDF      -    INTEGER INPUT VARIABLE, THE LENGTH IN BITS OF THE
C                  DESTINATION FIELD. ( 1.LE.LDF.LE.60 )
C    IBD      -    INTEGER INPUT VARIABLE, THE BIT POSITION IN THE
C                  FIRST DESTINATION WORD AT WHICH THE FIRST DESTINATION
C                  FIELD STARTS. ( 0.LE.IBD.LE.59 )
C
C
C     EXITS.
C
C     THE ROUTINE WILL RETURN TO THE CALLING PROGRAM WHEN THE REQUESTED
C     OPERATION IS COMPLETE.
C     ***  NOTE THAT THE ROUTINE DOES NO PARAMETER CHECKING.
C
C    AUTHOR ---  G.N. WILLIAMS  ( DEC 77 )
C
C
C    HARDWARE CONVENTIONS.
C
C    A CM WORD ON THE CDC CYBER COMPUTER SYSTEMS CONSISTS OF 60 BITS.
C    THE LOWEST BIT (RIGHTMOST) IS BIT NR. 0 , THE HIGHEST (LEFTMOST)
C    IS BIT NR. 59 .
C
C    STATEMENT FUNCTIONS USED
C
C    1.  GTBITS(WORD,BITPOS,NUMBER)
C
C    "GTBITS" IS AN INTEGER STATEMENT FUNCTION THAT RETURNS RIGHT
C    JUSTIFIED ZERO FILLED ALL BITS OF THE VARIABLE "WORD" FROM
C    BIT POSITION "BITPOS" TO BIT POSITION "BITPOS"+"NUMBER"-1.
C    "WORD" MAY BE OF TYPE INTEGER OR REAL. "BITPOS" AND "NUMBER"
C    ARE INTEGER VARIABLES SATISFYING THE FOLLOWING CONDITIONS...
C
C       0 .GE. "BITPOS" .LE. 59
C       0 .GE. "NUMBER" .LE. 60-"BITPOS"
C
C
C    2.  CLBITS(WORD,BITPOS,NUMBER)
C
C    "CLBITS" IS AN INTEGER STATEMENT FUNCTION THAT RETURNS AS ITS
C    VALUE THE CONTENTS OF THE VARIABLE "WORD" MODIFIED BY CLEARING
C    THE BITS FROM BIT POSITION "BITPOS" TO BIT POSITION "BITPOS" +
C    "NUMBER"-1. ALL INPUT PARAMETERS ARE DEFINED AS IN "GTBITS".
C
C
C    3.  STBITS(WORD,BITPOS,NUMBER,BITS)
C
C    "STBITS" IS AN INTEGER STATEMENT FUNCTION THAT RETURNS AS ITS
C    VALUE THE CONTENTS OF THE VARIABLE "WORD" MODIFIED BY REPLACING
C    THE BITS FROM BIT POSITION "BITPOS" TO BIT POSITION "BITPOS" +
C    "NUMBER"-1 BY THE "NUMBER" LOWEST BITS OF THE VARIABLE "BITS".
C    "STBITS" IS AN EXTENSION OF "CLBITS". "BITS" MAY BE OF TYPE
C    INTEGER OR REAL. ALL OTHER INPUT PARAMETERS ARE DEFINED AS IN
C    "GTBITS".
C
      INTEGER SOURCE(1),DEST(1)
C
      INTEGER SBIT,SBYTCT,SWDCT,SWORD
      INTEGER DBIT,DWDCT
      INTEGER OVFLOW
*IF DEF,AELIB5
      INTEGER WORD, BITPOS, NUMBER, BITS
*ENDIF
C
      INTEGER GTBITS,STBITS,CLBITS
C
*IF -DEF,AELIB5
      GTBITS(WORD,BITPOS,NUMBER) =
     *        SHIFT(WORD,-BITPOS).AND..NOT.MASK(60-NUMBER)
*ENDIF
*IF DEF,AELIB5
      GTBITS(WORD,BITPOS,NUMBER) =
     *        SHIFT(WORD,-BITPOS).AND..NOT.MASK(60-NUMBER)
*ENDIF
C
*IF -DEF,AELIB5
      CLBITS(WORD,BITPOS,NUMBER) =
     *        SHIFT(MASK(60-NUMBER),BITPOS).AND.WORD
*ENDIF
*IF DEF,AELIB5
      CLBITS(WORD,BITPOS,NUMBER) =
     *        SHIFT(MASK(60-NUMBER),BITPOS).AND.WORD
*ENDIF
C
*IF -DEF,AELIB5
      STBITS(WORD,BITPOS,NUMBER,BITS) =
     *                    CLBITS(WORD,BITPOS,NUMBER)
     *                               .OR.
     *             SHIFT(.NOT.MASK(60-NUMBER).AND.BITS,BITPOS)
*ENDIF
*IF DEF,AELIB5
      STBITS(WORD,BITPOS,NUMBER,BITS) =
     *                    CLBITS(WORD,BITPOS,NUMBER)
     *                               .OR.
     *             SHIFT(.NOT.MASK(60-NUMBER).AND.BITS,BITPOS)
*ENDIF
C
*CALL STATF
      SBIT       =IBS-LSS+1
      SBYTCT     =0
      SWDCT      =1
      DBIT       =IBD-LDF+1
      DWDCT      =1
      SWORD      =SOURCE(1)
C
   5  IF(SBYTCT.GE.N)RETURN
      SBYTCT     =SBYTCT+1
C
C  *************************************************************
C
C     EXTRACT THE NEXT SOURCE STRING FROM THE SOURCE ARRAY AND
C     STORE IT RIGHT JUSTIFIED WITH ZERO FILL IN THE VARIABLE IBYTE.
C
C  *************************************************************
C
      IF(SBIT)10,20,30
C
C     SBIT NEGATIVE - SOME BITS IN THE NEXT SOURCE WORD
C
  10  OVFLOW     =IABS(SBIT)
      ITOP       =SHIFT(GTBITS(SWORD,0,LSS-OVFLOW),
     *                     OVFLOW)
      SWDCT      =SWDCT+1
      SWORD      =SOURCE(SWDCT)
      SBIT       =60-OVFLOW
      IBOT       =GTBITS(SWORD,SBIT,OVFLOW)
      IBYTE      =ITOP.OR.IBOT
      GO TO 100
C
C     SBIT IS ZERO - BIT FIELD ENDS ON A WORD BOUNDARY
C
  20  IBYTE      =GTBITS(SWORD,SBIT,LSS)
      SWDCT      =SWDCT+1
      SWORD      =SOURCE(SWDCT)
      SBIT       =60
      GO TO 100
C
C     SBIT GREATER THAN ZERO - MORE THAN ENOUGH BITS IN SOURCE
C
  30  IBYTE      =GTBITS(SWORD,SBIT,LSS)
C
C     RESET THE SOURCE WORD BIT COUNTER
C
 100  CONTINUE
      SBIT       =SBIT-LSS
C
C  *************************************************************
C
C     STORE THE SOURCE STRING, NOW RIGHT JUSTIFIED IN IBYTE, IN
C     THE NEXT DESTINATION FIELD.
C
C  *************************************************************
C
      IF(DBIT)110,120,130
C
C     DBIT NEGATIVE - SOME BITS TO GO IN NEXT DESTINATION WORD
C
 110  OVFLOW     =IABS(DBIT)
      ITOP       =GTBITS(IBYTE,OVFLOW,LDF-OVFLOW)
      DEST(DWDCT)=STBITS(DEST(DWDCT),0,LDF-OVFLOW,ITOP)
      DWDCT      =DWDCT+1
      DBIT       =60-OVFLOW
      DEST(DWDCT)=STBITS(DEST(DWDCT),DBIT,OVFLOW,IBYTE)
      GO TO 200
C
C     DBIT IS ZERO - EXACTLY THE RIGHT NUMBER OF BITS IN DESTINATION
C
 120  DEST(DWDCT)=STBITS(DEST(DWDCT),DBIT,LDF,IBYTE)
      DWDCT      =DWDCT+1
      DBIT       =60
      GO TO 200
C
C     DBIT GREATER THAN ZERO - PLENTY OF SPACE IN DESTINATION WORD
C
 130  DEST(DWDCT)=STBITS(DEST(DWDCT),DBIT,LDF,IBYTE)
C
C     RESET THE DESTINATION WORD BIT COUNTER
C
 200  CONTINUE
      DBIT       =DBIT-LDF
      GO TO 5
C
      END
