*DECK PCALC
      SUBROUTINE PCALC(YFIT,NOPTS,B,IH,T,WS)
C
C
C  PURPOSE:
C
C     COMPUTE THE CONSTANT TERM TO COMPLETE THE FIT PROVIDED BY RLSTEP
C     AND CALCULATE FROM THIS SET OF COEFFICIENTS THE FITTED VALUES OF
C     THE DEPENDENT VARIABLE.
C
C  ENTRY:
C     CALL PCALC(YFIT,NOPTS,B,IH,T,WS)
C
C     NOTE:  THIS ROUTINE IS TO BE USED ONLY AFTER SUCCESSFUL CALLS TO
C            SSCP AND RLSTEP.  THE PARAMETERS, NOPTS, B IH, T, AND WS,
C            MUST REMAIN UNCHANGED BETWEEN THE CALLS OF SSCP AND RLSTEP,
C            AND THE SUBSEQUENT CALL TO PCALC.
C
C     YFIT....REAL ARRAY OF FITTED VALUES OF THE DEPENDENT VARIABLE.
C     NOPTS...INTEGER, NUMBER OF DATA POINTS.
C     B.......REAL ARRAY OF COEFFICIENTS OF THE FITTED POLYNOMIAL
C                  RETURNED BY RLSTEP AND REQUIRED AS INPUT FOR PCALC.
C                  ON RETURN FROM PCALC, B(I), I = 2, 3, ..., T, IS THE
C                  COEFFICIENT OF THE (I-1)-ST NON-CONSTANT TERM AND
C                  B(1) IS THE CONSTANT TERM.  NRB \ T.
C     IH......INTEGER ARRAY RETURNED FROM RLSTEP AS A VECTOR OF PLUS AND
C                  MINUS ONES AND IS REQUIRED AS INPUT TO PCALC.  ON
C                  RETURN FROM PCALC, IH(I) = -1  IMPLIES THAT THE I-TH
C                  TERM IS NOT IN THE FITTED POLYNOMIAL.  IH(I) = 1
C                  IMPLIES THAT THE I-TH TERM IS IN THE POLYNOMIAL.
C                  NRI \ T.
C     T.......INTEGER, NUMBER OF TERMS IN THE POLYNOMIAL TO BE FITTED.
C     WS......REAL WORKING STORAGE ARRAY.  THE FIRST (T*(T+1))/2 WORDS
C                  IN THIS ARRAY CONTAIN THE ERROR INFORMATION SET UP
C                  BY RLSTEP, AND ARE NOT USED BY PCALC.
C             WS(((T*(T+1))/2)+1) TO WS(((T*(T+3))/2)-1)  IS EXPECTED
C                  TO CONTAIN ON ENTRY THE MEANS OF THE NON-CONSTANT
C                  TERMS IN THE POLYNOMIAL.
C             WS((T*(T+3))/2)  IS EXPECTED TO CONTAIN ON ENTRY THE
C                  MEAN OF THE DEPENDENT VARIABLE.
C             WS(((T*(T+3))/2)+1) TO WS(((T*(T+3))/2)+N*TM1)  IS
C                  EXPECTED TO CONTAIN ON ENTRY THE VALUES OF THE NON-
C                  CONSTANT TERMS OF THE POLYNOMIAL. THE VALUE TERM
C                  MATRIX IS STORED BY COLUMNS.
C             WS(((T*(T+3))/2)+N*TM1+1) TO WS((T*(2*N+T+3))/2)  IS
C                  EXPECTED TO CONTAIN ON ENTRY THE DEPENDENT VALUES OF
C                  Y.
C             THE EXPECTED VALUES FOR THE ARRAY WS INDEXED FROM
C                  (T*(T+1))/2+1 TO (T*(2*N+T+3))/2 ARE THOSE VALUES
C                  THAT ARE SET UP BY A CALL TO SSCP.
C             WS((T*(2*N+T+3))/2+1) CONTAINS THE SUM OF WEIGHTS AS
C                  CALCULATED BY SSCP
C
C  EXIT:
C
C     CONTROL IS RETURNED TO THE CALLING PROGRAM WITH THE FITTED VALUES
C     IN YFIT, THE ABSENCE OR PRESENCE OF TERMS OF THE POLYNOMIAL IN IH,
C     WITH THE COEFFICIENTS IN B.
C
C  AUTHORS:     D. E. SMITH AND L. E. EVANS         JUNE 1975
C
C
      INTEGER  IH(1)
      REAL  YFIT(1), B(1), WS(1)
      INTEGER  T, TM1
*CALL  STATF
C
C        INITIALIZATION
C
      TM1 = T - 1
      I2 = (T*(T+1)) / 2
      I3 = I2 + T
C
C        SHIFT THE ELEMENTS OF IH AND B DOWN ONE TO ADD IN
C                    THE CONSTANT TERM
C
      DO 65  I = 1, TM1
        B(T-I+1) = B(T-I)
        IH(T-I+1) = IH(T-I)
65    CONTINUE
      IH(1) = 1
C     PRINT 68, (IH(I), I = 1, T)
C68   FORMAT(1H0, 10X, 30I4)
C
C        CALCULATE THE CONSTANT TERM FIT
C
      SUM = 0.0
      IND2 = I2 + 1
      DO 70  I = 2, T
        SUM = SUM + B(I) * WS(IND2)
        IND2 = IND2 + 1
70    CONTINUE
      B(1) = WS(IND2) - SUM
C     PRINT 75, (B(I), I = 1, T)
C75   FORMAT(1H1, 6(6X, E14.7))
C
C        DETERMINE THE FIT FROM THE POLYNOMIAL
C
      DO 90  I = 1, NOPTS
        YFIT(I) = B(1)
        DO 80  J = 1, TM1
          IND3 = (J-1) * NOPTS + I + I3
          YFIT(I) = YFIT(I) + B(J+1) * WS(IND3)
80      CONTINUE
90    CONTINUE
      RETURN
      END
