*DECK PLODAM
      SUBROUTINE PLODAM(MODE,X,Y,NX,FRAC,DDLENG,WS,NWS)
C
C PLODAM, A ROUTINE FOR DRAWING A SMOOTH DASH LINE THROUGH A GIVEN SET
C OF DATA POINTS. FORMAT OF THE DASH LINE IS SPECIFIED BY USER.
C
C USAGE  -  CALL PLODAM(MODE,X,Y,NX,FRAC,DDLENG,WS,NWS)
C
C PARAMETERS --
C    MODE  - .EQ. 0 ON INPUT IMPLIES NO CUBIC SPLINE IS TO BE FITTED
C            ONTO INPUT DATA BEFORE PLOTTING THE DASH LINE(IN WHICH
C            CASE DATA POINTS ARE JOINED BY STRAIGHT LINES). MODE .EQ.
C            1 IMPLIES CUBIC SPLINE IS USED.
C    X     - VECTOR OF LENGTH NX CONTAINING THE ABSCISSAE OF THE NX DATA
C              POINTS (X(I),Y(I)) I=1,...,NX (INPUT). X MUST BE ORDERED
C              SO THAT X(I).LT.X(I + 1) FOR MODE 1 CALLS
C    Y     - VECTOR OF LENGTH NX CONTAINING THE ORDINATES (OR FUNCTION
C              VALUES) OF THE NX DATA POINTS (INPUT).
C    NX    - NUMBER OF ELEMENTS IN X AND Y (INPUT). ABS(NX) MUST BE.GE.2
C              NX.LT.0 IMPLIES THE DASH LINE TO BE DRAWN BACKWARDS
C              THROUGH DATA POINTS IN ORDER OF (X(NX),Y(NX)),........,
C              (X(1),Y(1))
C    FRAC  - REAL ARRAY OF DIMENSION 4. FRAC(I), I=1,..,4 HOLD D1,B1,D2,
C              B2 (RESPECTIVELY) AS SPECIFIED IN DASH CYCLE BELOW.
C    DDLENG- DDLENG= 0 ON INPUT IMPLIES LENGTH OF DASH CYCLE TO BE
C              DETERMINED BY PLODAM, NORMALLY SPECIFIED BY USER
C              IN INCHES OR MILLIMETRES.
C    WS    - REAL ARRAY OF WORKING STORAGE SUPPLIED TO PLODAM. IT
C              SHOULD NOT BE INITIALIZED
C    NWS   - DIMENSION OF WS, SHOULD BE .GE.(4 * NX - 4)
C
C REMARKS  -  A PLOT MODE 1 CALL SHOULD PRECEED CALLING PLODAM
C             WS,NWS WILL ONLY HAVE THEIR STATED MEANINGS IN MODE 1 CALL
C             BPAR IS PUT INTO COMMON BLOCK DPARS. IT IS INITIALIZED
C             TO ZEROS. THEY CAN BE MODIFIED AS OUTLINED IN THE WRITEUP
C             OF ICSICU OF IMSLIB TO RESULT IN A BETTER FIT OF CUBIC
C             SPLINE.(BPAR(I)=0 IMPLIES A NATURAL SPLINE IS USED.)
C
C OTHER SUBROUTINES CALLED -- ICSICU FROM IMSLIB
C                             COSIMP AND PLOT FROM AELIB
C                             AELERR FOR ERROR PROCESSING
C                             FCNINT,COSAUX,UTILITY ROUTINES PROVIDED
C                             BY PLODAM
C
C REPRESENTATION OF THE DASH LINE --
C    A DASH LINE DENOTES A LINE CONSISTING OF FOUR SEGMENTS (A DASH
C CYCLE) REPEATED IN SEQUENCE.THESE SEGMENTS ARE A DASH OF D1,A BLANK
C OF B1,A DASH OF D2 AND A BLANK OF B2. (D1,B1,D2,B2 ARE FRACTIONS
C W.R.T. THE LENGTH OF A DASH CYCLE).
C    THIS REPRESENTATION ALLOWS MANY POSSIBILITIES ENABLING DIFFERENT
C GRAPHS TO BE DIFFERENTIATED ON THE SAME PLOT.
C
C METHOD USED --
C  (A) MODE=1
C    A CUBIC SPLINE IS FIT ONTO THE GIVEN DATA POINTS. ACCORDING TO THE
C DEVIATION OF THE SPLINE FROM THE LINE JOINING THE TWO ADJACENT DATA
C POINTS, THE INTERVAL BETWEEN THE TWO POINTS IS DISECTED INTO
C SUBINTERVALS IN EACH OF WHICH THE SPLINE IS APPROXIMATED BY A
C STRAIGHT LINE. THEN THE DASH CYCLES ARE FIT ONTO THE RESULTING
C STRAIGHT LINE SEGMENTS.
C  (B) MODE=0
C    INPUT DATA POINTS ARE JOINED BY STRAIGHT LINES AND THEN THE DASH
C CYCLES ARE FIT ONTO THE RESULTING STRAIGHT LINE SEGMENTS.
C
C ERROR CHECKING --
C    INPUT NUMBER OF DATA POINTS MUST BE GE 2 FOR EITHER MODE.
C PARTICULARLY FOR MODE 1 CALLS, INPUT DATA MUST BE IN ASCENDING ORDER
C OF X,WORKING STORAGE MUST BE .GE. 4 * NX - 4 .OTHERWISE ERROR WILL
C RESULT AND JOB WILL BE ABORTED.
C
C    PROGRAMMED BY  -  JAMES LIU     MAY, 1977
C
      EXTERNAL FCNINT,COSAUX
C
C     MESSAGE IS AN ARRAY CONTINNING THE MESSAGE NUMBERS OF THE
C     MESSAGE IS AN ARRAY CONTAINING THE MESSAGE NUMBERS OF THE
C     ERROR MESSAGES POSSIBLY PRODUCED BY THE ROUTINE
C
      REAL MESSAGE(3)
      REAL X(NX),Y(NX),WS(1),FRAC(4)
      REAL XSEG(2),YSEG(2)
      REAL XSEGO(2),YSEGO(2),LENG(4)
      LOGICAL METRIC
      COMMON/INTVL/A3,A2,A1,XI
      COMMON/DPARS/BPAR(4),ESP
      DATA MESSAGE/3525,3526,3527/
      DATA BPAR/4*0./,ESP/1.0E-5/
*CALL  STATF
C
      METRIC=.TRUE.
      DLENG=DDLENG
      CALL PLOTM(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      GO TO 1
C
C     ENTRY POINT FOR PLODA.
C     HERE PARAMETERS WITH ENGLISH UNITS WILL BE CONVERTED TO METRIC
C     UNITS.
C
*IF -DEF,AELIB5
      ENTRY PLODA
*ENDIF
*IF DEF,AELIB5
      ENTRY PLODA(MODE,X,Y,NX,FRAC,DDLENG,WS,NWS)
*ENDIF
      METRIC=.FALSE.
      DLENG=DDLENG * 25.4
      CALL PLOT(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      XLR=XLR * 25.4
      YLR=YLR * 25.4
C
  1   INCRE=1
      IF (NX.GT.0) GO TO 5
         INCRE=-1
         NX=-NX
C
C     TEST FOR SIZE OF ARRAY WS AND NO. OF DATA POINTS SUPPLIED
C
  5   IF (NX.LT.2) GO TO 120
      IF (MODE.EQ.0) GO TO 7
         IF (NWS.LT.(4 * NX - 4)) GO TO 110
C
C     TO TRANSFORM THE DATA POINTS TO POINTS IN THE PLOT AREA.
C
  7   DO 8 I=1,NX
         X(I)=(X(I) - XMINR) * XLR / (XMAXR - XMINR)
  8      Y(I)=(Y(I) - YMINR) * YLR / (YMAXR - YMINR)
      IF (MODE.EQ.0) GO TO 13
C
C     FIT A CUBIC SPLINE ONTO INPUT DATA
C     AND CHECKING FOR STRICTLY INCREASING ORDER OF X
C
      DO 9 I=2,NX
  9      IF (X(I).LE.X(I - 1)) GO TO 130
      CALL ICSICU(X,Y,NX,BPAR,WS,(NX - 1),IER)
      NUMB=(NX - 1) * 3
C
C     TO CALCULATE THE DISTANCE BETWEEN ADJACENT DATA POINTS ALONG
C     THE CUBIC SPLINE FOR MODE 1 CALLS OR STRAIGHTLY EUCLIDEAN
C     DISTANCE FOR MODE 0 CALLS.
C
  13  DTTL=0.
      DMIN=1.0E300
      INTTL=NX - 1
      DO 20 I=1,INTTL
         IF (MODE.EQ.0) GO TO 15
C
C           TO SET UP PARAMETERS AND EVALUATE THE LINE INTEGRAL
C           FOR MODE=1
C           STRAIGHT EUCLIDEAN DISTANCE IS USED FOR MODE=0
C
            A1=WS(I)
            A2=WS(I + NX - 1)
            A3=WS(I + 2 * NX - 2)
            XI=X(I)
            ISUB=0
            DIST=COSIMP(X(I),X(I + 1),COSAUX,ESP,ISUB)
            WS(NUMB + I) = DIST
            GO TO 16
  15     DIST=SQRT((X(I)-X(I + 1))**2 + (Y(I) - Y(I + 1))**2)
  16     DTTL=DTTL + DIST
  20     IF (DMIN.GT.DIST) DMIN=DIST
C
C     SET UP THE LENGTHS OF THE SEGMENTS OF THE DASH CYCLE
C
      IF (DLENG.EQ.0.) DLENG=0.25 * (DMIN + DTTL / (NX - 1))
      DO 50 I=1,4
  50     LENG(I)=FRAC(I) * DLENG
C
C     INITIALIZATION OF PARAMETERS TO ENTER LOOP ACCORDING TO WHETHER
C     PLOTTING DASH LINE FORWARDS OR BACKWARDS AS INDICATED
C
      J=1
      DASH=LENG(J)
      IF (INCRE.EQ.1) GO TO 60
         I=NX + 1
         K=-1
         GO TO 70
  60  I=0
      K=0
C
C     MEANING OF SOME VARIABLES USED --
C     DASH - A DASH SEGMENT TO BE DRAWN EITHER AS A DASH OR A BLANK LINE
C     EDIST - EUCLIDEAN DISTANCE
C     NSUB - NUMBER OF SUBINTERVALS BETWEEN ADJACENT DATA POINTS
C     XINCRE - DELTA X OF EACH SUBINTERVAL
C     (XSEG(1),YSEG(1)) - STARTING POINT OF A DASH TO BE DRAWN
C     (XSEG(2),YSEG(2)) - END POINT OF A DASH TO BE DRAWN
C     (XEND,YEND) - END POINT OF A SUBINTERVAL
C     J - COUNTER FOR DRAWING SEGMENTS OF A DASH CYCLE IN SEQUENCE
C
C     FIT IN SEGMENTS OF DASH CYCLE UNTIL ALL DATA INTERVALS ARE FILLED.
C     CALCULATE NUMBER AND LENGTH OF SUBINTERVALS TO USE BETWEEN DATA
C     POINTS AND INITIALIZE XEND, YEND AT START OF DATA INTERVAL
C
  70  I=I + INCRE
      IF (((I.EQ.NX).AND.(INCRE.EQ.1)).OR.((I.EQ.1).AND.(INCRE.EQ.-1)))
     ,   GO TO 140
C
C        TO FORCE NSUB=1 FOR MODE=0
C
         IF (MODE.EQ.0) GO TO 73
            EDIST=SQRT((X(I + INCRE) - X(I))**2 + (Y(I + INCRE) - Y(I))
     ,                 **2)
            DIST=WS(NUMB + I + K)
            NSUB=15 * (DIST - EDIST) / DIST + 1 + DIST / DLENG
            GO TO 75
  73     NSUB=1
  75     XINCRE=(X(I + INCRE) - X(I)) / NSUB
         XEND=X(I)
         YEND=Y(I)
C
C     FIT IN SEGMENTS OF DASH UNTIL A DATA INTERVAL IS FILLED
C     AND THEN GO TO THE NEXT DATA INTERVAL
C
  80  IF (NSUB.EQ.0) GO TO 70
         NSUB=NSUB - 1
         XSEG(1)=XEND
         YSEG(1)=YEND
         XEND=XSEG(1) + XINCRE
         IF (MODE.EQ.0) YEND=Y(I + INCRE)
*IF -DEF,AELIB5
         IF (MODE.EQ.1) YEND=FCNINT(XEND,X,Y,WS,(NX - 1),(I + K))
*ENDIF
*IF DEF,AELIB5
         IF (MODE.EQ.1) YEND=FCNINT(XEND,X(1),Y(1),WS(1),(NX - 1),(I + K
     +))
*ENDIF
C
C     FIT IN SEGMENTS OF DASH UNTIL A SUBINTERVAL IS FILLED
C     WITHIN EACH SUBINTERVAL, FUNCTION IS APPROX. BY A STR. LINE.
C
  90  DIST=SQRT((XEND - XSEG(1))**2 + (YEND - YSEG(1))**2)
      IF (DASH.GE.DIST) GO TO 100
C
C     TO FIND THE END POINTS OF THE DASH BY LINEAR INTERPOLATION
C
      DX=(XEND - XSEG(1)) * DASH / DIST
      XSEG(2)=XSEG(1) + DX
      SIGN=1.
      IF (YEND.LT.YSEG(1)) SIGN=-1.
      IF (YEND.EQ.YSEG(1)) SIGN=0.0
C     YSEG(2)=YSEG(1) + SQRT(DASH * DASH - DX * DX) * SIGN
      TEMP=DASH*DASH-DX*DX
      IF(ABS(TEMP).LT.1.E-12)TEMP=0
      YSEG(2)=YSEG(1) + SQRT(TEMP)*SIGN
      IF (MOD(J,2).EQ.0) GO TO 96
      DO 95 M=1,2
         XSEGO(M)=XSEG(M) * (XMAXR - XMINR) / XLR + XMINR
  95     YSEGO(M)=YSEG(M) * (YMAXR - YMINR) / YLR + YMINR
         IF (METRIC) CALL PLOTM(2,1,0,XSEGO,YSEGO,2)
         IF (.NOT.METRIC) CALL PLOT(2,1,0,XSEGO,YSEGO,2)
  96  J=J + 1
      IF (J.GT.4) J=1
      DASH=LENG(J)
      XSEG(1)=XSEG(2)
      YSEG(1)=YSEG(2)
      GO TO 90
C
C     WHEN DISTANCE FROM STARTING POINT OF A DASH TO THE END POINT OF
C     THE SUBINTERVAL IS LESS THAN DASH ITSELF, DASH WILL HAVE TO BEND
C     OVER THE SUBINTERVAL END POINT. SO STARTING POINT OF DASH IS
C     JOINED TO SUBINTERVAL END POINT WHILE REST OF THE DASH WILL HAVE
C     TO BE DRAWN IN THE NEXT INTERVAL.
C
  100 XSEG(2)=XEND
      YSEG(2)=YEND
      IF (MOD(J,2).EQ.0) GO TO 106
      DO 105 M=1,2
         XSEGO(M)=XSEG(M) * (XMAXR - XMINR) / XLR + XMINR
  105    YSEGO(M)=YSEG(M) * (YMAXR - YMINR) / YLR + YMINR
         IF (METRIC) CALL PLOTM(2,1,0,XSEGO,YSEGO,2)
         IF (.NOT.METRIC) CALL PLOT(2,1,0,XSEGO,YSEGO,2)
  106 DASH=DASH - DIST
      GO TO 80
C
C     ERROR MESSAGES
C
*IF -DEF,AELIB5
  110 CALL AELERR(MESSAGE(1),1HF)
*ENDIF
*IF DEF,AELIB5
  110 CALL AELERR(MESSAGE(1),'F')
*ENDIF
      GO TO 160
*IF -DEF,AELIB5
  120 CALL AELERR(MESSAGE(2),1HF)
*ENDIF
*IF DEF,AELIB5
  120 CALL AELERR(MESSAGE(2),'F')
*ENDIF
      GO TO 160
*IF -DEF,AELIB5
  130 CALL AELERR(MESSAGE(3),1HF)
*ENDIF
*IF DEF,AELIB5
  130 CALL AELERR(MESSAGE(3),'F')
*ENDIF
  140 DO 150 I=1,NX
         X(I)=X(I) * (XMAXR - XMINR) / XLR + XMINR
  150    Y(I)=Y(I) * (YMAXR - YMINR) / YLR + YMINR
  160 RETURN
      END
      FUNCTION FCNINT(X0,X,Y,C,N,INT)
C
C     FCNINT EVALUATES THE FUNCTION VALUE OF AN INTERPOLATING FUNCTION
C     RESULTING FROM A CUBIC SPLINE FITTING.
C
      REAL X(N),Y(N),C(N,3)
      DIFF=X0 - X(INT)
      FCNINT=((C(INT,3) * DIFF + C(INT,2)) * DIFF + C(INT,1)) * DIFF
     ,     + Y(INT)
      RETURN
      END
      FUNCTION COSAUX(X)
      COMMON/INTVL/A3,A2,A1,XI
C
C     COSAUX EVALUATES THE LINE INTEGRAND AT X GIVEN A3,A2,A1,A0, WHERE
C     THESE ARE THE COEFFICIENTS OF THE INTERPOLATING CUBIC SPLINE
C        A3 * D**3 + A2 * D**2 + A1 * D + A0
C     IN ANY PARTICULAR INTERVAL I, WHERE D=X - X(I)
C
      D=X - XI
      COSAUX=(9. * A3 * D + 12. * A2) * A3 * D
      COSAUX=(COSAUX + 6. * A3 * A1 + 4 * A2 * A2) * D
      COSAUX=SQRT((COSAUX + 4. * A2 * A1) * D + A1 * A1 + 1.)
      RETURN
      END
