*DECK PLOGX
      SUBROUTINE PLOGX(MODE,PP1,PP2,P3,P4,P5,P6,P7,P8)
************************************************************************
**                                                                    **
**    NOTE THAT PLOGX IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL **
**    ROUTINES:                                                       **
**       LOGPR1                                                       **
**       LOGPR2                                                       **
**       LOGPR3                                                       **
**       LOGPR6                                                       **
**       LOGPR7                                                       **
**       LOGPR8                                                       **
**       LOGPR12                                                      **
**    LOGPR1 THROUGH LOGPR12 ACCOMPANY PLOGX ON THE AELIB SOURCE TAPE **
**    UNDER THE DECK NAME PLOGX.                                      **
**                                                                    **
************************************************************************
C
C
C PLOGX IS A SUBROUTINE FOR PRODUCING SEMI-LOGARITHMIC GRAPHS (X-AXIS
C LOGARITHMIC, Y-AXIS LINEAR).  ITS CALLING SEQUENCES AND RESULTS ARE
C DESIGNED TO PARALLEL, IN MANY RESPECTS, THOSE OF THE STANDARD PLOTTING
C SUBROUTINE, PLOT.  THE FOLLOWING ARE ALL LEGAL CALLING SEQUENCES:
C     CALL PLOGX(1,XL,YL,XMIN,XMAX,YMIN,YMAX,LGTICX,TICY)
C     CALL PLOGX(2,PEN,SYMB,XX,YY,N)
C     CALL PLOGX(3,FORM,SIZE,DIR,XLOC,YLOC,FWA,N)
C     CALL PLOGX(3,FORM,SIZE,DIR,XLOC,YLOC)
C     CALL PLOGX(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,LGTICXR,TICYR)
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A) PLOGX  - ALL DIMENSIONAL QUANTITIES PASSED IN ARGUMENT LIST
C ARE IN INCHES
C    B) PLOGXM - ALL DIMENSIONAL QUANTITIES ARE IN MILLIMETERS
C DIMENSIONAL QUANTITIES ARE PASSED IN THE ARGUMENT LIST AND, ON THE
C METRIC ENTRY, PLOGXM, DIMENSIONS ARE CONVERTED TO ENGLISH UNITS ON A
C MODE 4 CALL, OR ON A MODE 1 CALL AFTER A PLOTM MODE 1 CALL HAS BEEN
C MADE TO PLOT THE FRAME.  A SWITCH INDICATING WHETHER METRIC DIMENSIONS
C HAVE BEEN USED IS PASSED IN A COMMON BLOCK TO THE MODULES WHICH CALL
C PLOT THAT ARE CALLED BY THIS ROUTINE.  IN THESE MODULES WHEN PLOGX
C HAS BEEN CALLED, PLOT IS CALLED, AND WHEN PLOGXM HAS BEEN CALLED,
C CORRESPONDING CALLS TO PLOTM ARE MADE.
C
C
C ARGUMENT LIST:
C  MODE....EITHER INTEGER OR REAL, SPECIFIES THE JOB TO BE DONE AND
C          DETERMINES THE NUMBER, TYPES AND MEANINGS OF THE OTHER
C          PARAMETERS.  LEGAL VALUES ARE 1,2,3 AND 4, CORRESPONDING TO
C          THE FIRST FOUR MODES OF PLOT.
C  ALL OF THE OTHER PARAMETERS ARE ESSENTIALLY THE SAME AS THOSE USED
C  BY PLOT, EXCEPT LGTICX AND LGTICXR WHICH DEFINE AND RETURN THE LOG
C  AXIS TIC VALUE.  ALSO, THE LOGARITHMS, RATHER THAN THE ACTUAL MIN AND
C  MAX VALUES ARE RETURNED BY MODE 4.
C
C ERROR CONDITIONS:
C  PLOGX ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT FOUR DIFFERENT ERROR
C  CONDITIONS:
C   1) ***MODE NOT 1,2,3 OR 4
C   2) ***INVALID LOG SCALE TIC VALUE SPECIFIED - DEFAULT IS 3
C   3) ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL
C   4) ***ILLEGAL VALUES SPECIFIED FOR LOGARITHMIC SCALE
C  THE PRINTING OF ALL ERROR MESSAGES IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE LOGARITHMIC PLOTTING ROUTINES (PLOGX, PLOGY, PLOGXY) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE LOGARITHMIC PLOTTING PACK-
C  AGE.  MODULES IN THE PACKAGE INCLUDE LOGPR1, LOGPR2, LOGPR3, LOGPR6,
C  LOGPR7, LOGPR8, AND LOGPR12.  PLOGX FIRST CALLS LOGPR1 WHICH FORCES
C  THE MODE INTO TYPE INTEGER AND CHECKS IT FOR LEGALITY.  THEN A BRANCH
C  IS MADE, DEPENDING UPON THE VALUE OF MODE, TO THE APPROPRIATE SECTION
C  OF PLOGX.  (THE METHOD USED WITHIN THE MODE SECTIONS IS EXPLAINED AT
C  THE BEGINNING OF EACH.)
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
*CALL STATF
C
      METRIC=.FALSE.
      P1=PP1
      P2=PP2
      GO TO 50
C
C
C     ENTRY PLOGXM ALLOWS USE OF METRIC DIMENSIONS
C
*IF -DEF,AELIB5
      ENTRY PLOGXM
*ENDIF
*IF DEF,AELIB5
      ENTRY PLOGXM(MODE,PP1,PP2,P3,P4,P5,P6,P7,P8)
*ENDIF
*CALL STATF
C
      METRIC=.TRUE.
      ETOM=25.4
      MTOE=1/ETOM
C
C     FORCE MODE INTO TYPE INTEGER,CHECK IT FOR LEGALITY, AND BRANCH,
C     DEPENDING UPON ITS VALUE, TO THE APPROPRIATE ROUTINE.
C
*IF -DEF,AELIB5
   50 GO TO (100,200,300,400), LOGPR1(MODE)
*ENDIF
*IF DEF,AELIB5
   50 GO TO (100,200,300,400), LOGPR1(MODE)
      CALL GOTOER
*ENDIF
C
C
C
C
C
C MODE 1: CALCULATE SCALE FACTORS, DRAW PLOT FRAME AND TICS.
C
C
C     CALL LOGPR2 WHICH CHECKS THAT XMIN AND XMAX ARE GREATER THAN ZERO
C     AND FINDS THEIR LOGS.  IT ALSO CHECKS LGTICX FOR LEGALITY, FORCES
C     IT INTO TYPE INTEGER, AND DETERMINES THE TICX VALUE TO USED IN
C     THE MODE 1 CALL OF PLOT.  THAT PLOT MODE 1 CALL IS THEN MADE.  IT
C     DRAWS THE LINEAR Y-AXIS AND THE X-AXIS FRAME LINE (IF LGTICX\0).
C     ENTRY POINT LOGPR3 IN MODULE LOGPR3 IS THEN CALLED TO STORE THE
C     MODIFIED PLOGX MODE 1 PARAMETERS.  FINALLY, IF LGTICX>0, LOGPR6 IS
C     CALLED TO PERFORM THE LOGARITHMIC TICKING OF THE X-AXIS.
C
  100 CALL LOGPR2(P3,P4,P7,XMIN,XMAX,ITICX,LOGTIC)
      IF (METRIC) GO TO 110
      CALL PLOT(1,P1,P2,XMIN,XMAX,P5,P6,LOGTIC,P8)
      GO TO 120
C
C     USE METRIC DIMENSIONS TO PLOT THE FRAME
C
  110 CALL PLOTM(1,PP1,PP2,XMIN,XMAX,P5,P6,LOGTIC,P8)
C
C     CONVERT METRIC DIMENSIONS TO ENGLISH
C
      P1=PP1*MTOE
      P2=PP2*MTOE
  120 CALL LOGPR3(1,P1,P2,XMIN,XMAX,P5,P6,ITICX,P8)
      IF (ITICX.GT.0) CALL LOGPR6(P2,P5,P6,XMIN,XMAX,ITICX,1,2)
      RETURN
C
C
C
C
C
C MODE 2: PLOT DATA POINTS.
C
C
C     CALL LOGPR7 TO CHECK THAT A PREVIOUS MODE 1 CALL OF PLOGX HAS BEEN
C     EXECUTED AND TO PLOT THE DATA.
C
  200 CALL LOGPR7(1,PP1,PP2,P3,P4,P5)
      RETURN
C
C
C
C
C
C MODE 3: PLOT CHARACTER STRINGS.
C
C
C     CALL LOGPR8 TO CHECK THAT A PREVIOUS MODE 1 CALL OF PLOGX HAS BEEN
C     EXECUTED AND TO PLOT THE CHARACTER STRING.
C
  300 CALL LOGPR8(1,PP1,PP2,P3,P4,P5,P6,P7)
      RETURN
C
C
C
C
C
C MODE 4: RETURN THE MODIFIED PARAMETERS OF THE PREVIOUS PLOGX MODE 1
C         CALL.
C
C
C     CALL ENTRY POINT LOGPR4 IN MODULE LOGPR3 TO CHECK THAT A PREVIOUS
C     MODE 1 CALL OF PLOGX HAS BEEN EXECUTED AND TO RETRIEVE THE MODE 1
C     PARAMETERS.
C
  400 IF (METRIC) GO TO 410
      CALL LOGPR4(1,P1,P2,P3,P4,P5,P6,P7,P8)
      RETURN
C
C     CONVERT METRIC DIMENSIONS TO ENGLISH
C
  410 P1=PP1*MTOE
      P2=PP2*MTOE
      CALL LOGPR4(1,P1,P2,P3,P4,P5,P6,P7,P8)
      RETURN
      END
      FUNCTION LOGPR1(MODE)
C
C
C LOGPR1 IS ONE OF THE MODULES OF THE LOGARITHMIC PLOTTING SYSTEM.  IT
C FORCES THE MODE INTO TYPE INTEGER AND CHECKS IT FOR LEGALITY.
C
C ARGUMENT LIST:
C  MODE....EITHER REAL OR INTEGER, FIRST PARAMETER PASSED TO THE PLOGZZ
C          SUBROUTINE BY THE CALLING PROGRAM.
C
C ERROR CONDITIONS:
C  LOGPR1 CHECKS FOR ONLY ONE ERROR CONDITION:
C   ***MODE NOT 1,2,3 OR 4
C  THE PRINTING OF THIS ERROR MESSAGE IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  IF THE HIGH ORDER 12 BITS OF MODE EQUAL ZERO, MODE IS ASSUMED TO BE
C  INTEGER.  OTHERWISE, IT IS ASSUMED TO BE REAL AND IS CONVERTED TO
C  INTEGER.  ITS VALUE IS THEN CHECKED FOR LEGALITY BEFORE RETURNING.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      EQUIVALENCE (IMODE, RMODE)
C
C
C     INITIALIZE ERROR NUMBER FOR AELIB ERROR ROUTINE.
C        3520 - ***MODE NOT 1,2,3 OR 4.
C
      NERR=3520
C
C
C     FORCE MODE INTO TYPE INTEGER.
C
      INTMOD=IMODE=MODE
      IF(SHIFT(IMODE,-48).NE.0) INTMOD=RMODE
      LOGPR1=INTMOD
C
C
C     CHECK IF MODE IS LEGAL.
C
*IF -DEF,AELIB5
      IF((INTMOD.GE.1).A.(INTMOD.LE.4)) RETURN
*ENDIF
*IF DEF,AELIB5
      IF((INTMOD.GE.1).AND.(INTMOD.LE.4)) RETURN
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(NERR,1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(NERR,'F')
*ENDIF
      END
      SUBROUTINE LOGPR2(RMIN,RMAX,TIC,RMINLG,RMAXLG,ITIC,LOGTIC)
C
C
C LOGPR2 IS ONE OF THE MODULES OF THE LOGARITHMIC PLOTTING SYSTEM.  IT
C CHECKS RMIN AND RMAX FOR LEGALITY AND TAKES THEIR LOGS.  IT FORCES THE
C LOGARITHMIC TICKING VALUE INTO TYPE INTEGER AND CHECKS IT FOR LEGAL-
C ITY.  IT ALSO DETERMINES THE TIC VALUE TO BE USED FOR THE LOGARITHMIC
C AXIS IN THE PLOT MODE 1 CALL.
C
C ARGUMENT LIST:
C  RMIN....REAL, MINIMUM VALUE ON THE AXIS.
C  RMAX....REAL, MAXIMUM VALUE ON THE AXIS.
C  TIC.....EITHER REAL OR INTEGER, LOGARITHMIC TIC VALUE.
C  RMINLG..REAL, RETURNS THE LOGARITHM OF RMIN.
C  RMAXLG..REAL, RETURNS THE LOGARITHM OF RMAX.
C  ITIC....INTEGER, RETURNS THE INTEGER VALUE OF TIC.
C  LOGTIC..INTEGER, RETURNS THE TIC VALUE TO BE USED IN THE PLOT MODE 1
C          CALL.
C
C ERROR CONDITIONS:
C  LOGPR2 CHECKS FOR TWO DIFFERENT ERROR CONDITIONS:
C   1) ***ILLEGAL VALUES SPECIFIED FOR LOGARITHMIC SCALE
C        IN A MODE 1 CALL OF PLOGZZ, THE MIN AND MAX VALUES FOR A LOG
C        SCALE MUST BOTH BE GREATER THAN ZERO SINCE THE LOGARITHMS OF
C        NEGATIVE NUMBERS ARE NOT DEFINED.  FATAL ERROR.
C   2) ***INVALID LOG SCALE TIC VALUE SPECIFIED - DEFAULT IS 3
C        IN A MODE 1 CALL OF PLOGZZ, THE LGTICZ VALUE WAS OTHER THAN
C        -1,0,1,2,3,4,5,11,12,13,14 OR 15.  INFORMATIVE MESSAGE.
C        EXECUTION CONTINUES WITH THE VALUE ASSUMED TO BE 3.
C  THE PRINTING OF BOTH OF THESE MESSAGES IS HANDLED BY THE STANDARD
C  AELIB ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  RMIN AND RMAX ARE CHECKED FOR BEING GREATER THAN ZERO AND THEIR LOGS
C  ARE TAKEN.  THE HIGH ORDER 12 BITS OF TIC ARE COMPARED TO ZERO.  IF
C  EQUAL, TIC IS ASSUMED INTEGER.  OTHERWISE IT IS ASSUMED REAL AND IS
C  CONVERTED TO INTEGER.  THEN TIC IS CHECKED FOR LEGALITY AND, IF
C  ILLEGAL, ASSIGNED THE DEFAULT VALUE OF 3.  THEN THE TIC VALUE FOR THE
C  PLOT MODE 1 CALL IS DETERMINED: ZERO IF A FRAME IS TO BE PLOTTED,
C  I.E. IF LGTICZ\0, OTHERWISE -1.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      EQUIVALENCE (ITICK, RTICK)
C
C
C     INITIALIZE ERROR NUMBERS FOR AELIB ERROR ROUTINE.
C        3523 - ***ILLEGAL VALUES SPECIFIED FOR LOGARITHMIC SCALE.
C        3521 - ***INVALID LOG SCALE TIC VALUE SPECIFIED - DEFAULT IS 3.
C
      NERR1=3523
      NERR2=3521
C
C
C     CHECK RMIN AND RMAX FOR LEGALITY.
C
*IF -DEF,AELIB5
      IF((RMIN.LT.0.0).O.(RMAX.LT.0.0)) CALL AELERR(NERR1,1HF)
*ENDIF
*IF DEF,AELIB5
      IF((RMIN.LT.0.0).OR.(RMAX.LT.0.0)) CALL AELERR(NERR1,'F')
*ENDIF
C
C
C     TAKE LOGS OF RMIN AND RMAX.
C
      RMINLG=ALOG10(RMIN)
      RMAXLG=ALOG10(RMAX)
C
C
C     FORCE TIC INTO TYPE INTEGER.
C
      ITIC=RTICK=TIC
      IF(SHIFT(ITICK,-48).EQ.0) ITIC=ITICK
C
C
C     CHECK TIC FOR LEGALITY.  IF ILLEGAL, ASSIGN DEFAULT VALUE OF 3.
C
*IF -DEF,AELIB5
      IF(((ITIC.GE.-1).A.(ITIC.LE.5)).OR.((ITIC.GE.11).A.(ITIC.LE.15)))
     $GO TO 20
*ENDIF
*IF DEF,AELIB5
      IF(((ITIC.GE.-1).AND.(ITIC.LE.5)).OR.((ITIC.GE.11).AND.(ITIC.LE.15
     $))) GO TO 20
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(NERR2,2HNF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(NERR2,'NF')
*ENDIF
      ITIC=3
C
C
C     DETERMINE LOG SCALE TICKING VALUE TO BE USED IN PLOT MODE 1 CALL.
C
   20 LOGTIC=ITIC
      IF(LOGTIC.GT.0) LOGTIC=0
      RETURN
      END
      SUBROUTINE LOGPR3(MAINSW,XL,YL,XMIN,XMAX,YMIN,YMAX,TICX,TICY)
C
C
C LOGPR3 IS ONE OF THE MODULES OF WHICH THE LOGARITHMIC PLOTTING PACKAGE
C IS COMPRISED.  IT ALSO FORMS PART OF THE AXIS LABELLING SYSTEM.  ENTRY
C POINT LOGPR3 IS USED ONLY BY THE LOGARITHMIC ROUTINES TO STORE THE
C PLOGZZ MODE 1 PARAMETERS.  IT ALSO SAVES THE INDICATOR, MAINSW, OF
C WHICH PLOGZZ SUBROUTINE HAS BEEN ACTIVATED BY THE USER.  ENTRY POINT
C LOGPR4 RETURNS THE MODE 1 PARAMETERS THAT WERE SAVED BY LOGPR3.  IT
C ALSO CHECKS FOR THE PLOGZZ ERROR CONDITION WHERE A MODE 4 CALL IS NOT
C PRECEEDED BY A CORRESPONDING MODE 1 CALL.  LOGPR5 RETURNS MAINSW, THE
C INDICATOR OF WHICH PLOGZZ SUBROUTINE WAS LAST ACTIVATED BY THE USER.
C THE CALLING SEQUENCES FOR LOGPR3 AND LOGPR4 ARE IDENTICAL, BUT THE
C PARAMETERS XL THROUGH TICY ARE TREATED AS INPUT IN LOGPR3 AND AS OUT-
C PUT IN LOGPR4.  LOGPR5 ONLY REQUIRES ONE PARAMETER AND IT IS TREATED
C AS OUTPUT.  IT SHOULD BE NOTED THAT LOGPR3 STORES THE MODE 1 PARAMET-
C ERS AFTER THEY HAVE BEEN MODIFIED, I.E. AFTER THE LOGARITHMS HAVE BEEN
C TAKEN, WHERE APPROPRIATE, AND THE LOGARITHMIC TIC VALUES FORCED INTO
C VALID TYPE INTEGER FORM.
C
C ARGUMENT LIST:
C  MAINSW..INTEGER, INDICATES WHICH PLOGZZ MODE 1 CALL WAS LAST MADE:
C          1 - PLOGX,  2 - PLOGY,  3 - PLOGXY.
C          IT IS INITIALIZED AS ZERO AT THE BEGINNING OF EXECUTION,
C          INDICATING THAT NO PLOGZZ MODE 1 CALL HAS YET BEEN MADE.
C  XL......LENGTH OF THE X-AXIS IN INCHES.
C  YL......LENGTH OF THE Y-AXIS.
C  XMIN....MINIMUM VALUE ON THE X-AXIS.
C  XMAX....MAXIMUM VALUE ON THE X-AXIS.
C  YMIN....MINIMUM VALUE ON THE Y-AXIS.
C  YMAX....MAXIMUM VALUE ON THE Y-AXIS.
C  TICX....TIC VALUE FOR THE X-AXIS.
C  TICY....TIC VALUE FOR THE Y-AXIS.
C
C ERROR CONDITIONS:
C  ENTRY POINT LOGPR4 DETECTS ONE ERROR CONDITION:
C   ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL
C     THE MODE 1 PARAMETERS CANNOT BE RETURNED UNLESS THE APPROPRIATE
C     MODE 1 CALL HAS BEEN MADE PREVIOUSLY.  THIS ERROR CAN ONLY OCCUR
C     WHEN LOGPR4 IS CALLED FROM A PLOGZZ MODE 4 ROUTINE.  FATAL ERROR.
C  THE PRINTING OF THIS ERROR MESSAGE IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      DATA MANSWS/0/
C
C
C     STORE ALL OF THE PARAMETERS WHICH HAVE BEEN PASSED.
C
      MANSWS=MAINSW
      XLS=XL
      YLS=YL
      XMINS=XMIN
      XMAXS=XMAX
      YMINS=YMIN
      YMAXS=YMAX
      TICXS=TICX
      TICYS=TICY
      RETURN
C
C
C
C
C
*IF -DEF,AELIB5
      ENTRY LOGPR4
*ENDIF
*IF DEF,AELIB5
      ENTRY LOGPR4(MAINSW,XL,YL,XMIN,XMAX,YMIN,YMAX,TICX,TICY)
*ENDIF
C
C
C     INITIALIZE ERROR NUMBER FOR AELIB ERROR ROUTINE.
C        3522 - ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL.
C
      NERR=3522
C
C
C     RETURN THE PARAMETERS WHICH WERE SAVED BY LOGPR3.  ALSO CHECK FOR
C     PLOGZZ MODE 4 ERROR CONDITION.
C
*IF -DEF,AELIB5
      IF(MAINSW.NE.MANSWS) CALL AELERR(NERR,1HF)
*ENDIF
*IF DEF,AELIB5
      IF(MAINSW.NE.MANSWS) CALL AELERR(NERR,'F')
*ENDIF
C
      XL=XLS
      YL=YLS
      XMIN=XMINS
      XMAX=XMAXS
      YMIN=YMINS
      YMAX=YMAXS
      TICX=TICXS
      TICY=TICYS
      RETURN
C
C
C
C
C
*IF -DEF,AELIB5
      ENTRY LOGPR5
*ENDIF
*IF DEF,AELIB5
      ENTRY LOGPR5(MAINSW,XL,YL,XMIN,XMAX,YMIN,YMAX,TICX,TICY)
*ENDIF
C
C
C     RETURN INDICATOR OF WHICH PLOGZZ MODE 1 WAS LAST CALLED.
C
      MAINSW=MANSWS
      RETURN
      END
      SUBROUTINE LOGPR6(OPAXL,OPAXMN,OPAXMX,RMIN,RMAX,ITIC,I,J)
C
C
C LOGPR6 IS ONE OF THE MODULES OF THE LOGARITHMIC PLOTTING SYSTEM.  IT
C DRAWS THE TICS OR LINES FOR A LOGARITHMIC AXIS.
C
C IF THE MAIN ENTRY POINT HAS BEEN USED IN THE CALLING PROGRAM (ENGLISH
C DIMENSIONS HAVE BEEN USED ) PLOT IS CALLED AND IF THE ALTERNATE ENTRY
C HAS BEEN USED IN THE CALLING PROGRAM(METRIC DIMENSIONS HAVE BEEN
C USED ) PLOTM IS CALLED.
C
C
C ARGUMENT LIST:
C  OPAXL...REAL, LENGTH OF THE OPPOSITE AXIS.
C  OPAXMN..REAL, MINIMUM VALUE ON THE OPPOSITE AXIS.
C  OPAXMX..REAL, MAXIMUM VALUE ON THE OPPOSITE AXIS.
C  RMIN....REAL, ALOG10(MINIMUM VALUE ON AXIS TO BE TICKED).
C  RMAX....REAL, ALOG10(MAXIMUM VALUE ON AXIS TO BE TICKED).
C  ITIC....INTEGER, LOGARITHMIC TIC VALUE FOR AXIS TO BE TICKED.
C  I,J.....INTEGERS, TOGETHER SPECIFY WHICH AXIS IS TO BE TICKED:
C          1,2 - X-AXIS,  2,1 - Y-AXIS.
C
C ERROR CONDITIONS:
C  LOGPR6 DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C
C METHOD:
C  A CALL IS MADE TO LOGPR12 TO DETERMINE THE LOOP VALUES TO USE WHEN
C  PLOTTING THE TICS.  THEN THE CONSTANT OPPOSITE AXIS COORDINATES FOR
C  TICS ARE DETERMINED.  NEXT, THE APPROPRIATE TICS OR LINES ARE DRAWN
C  ALONG THE OPAXMN LINE BY USING THE LOOP VALUES RETURNED BY LOGPR12.
C  IF LINES WERE DRAWN, I.E. IF ITIC<10, RETURN.  OTHERWISE, DRAW THE
C  TICS ALONG THE OPAXMX LINE BEFORE RETURNING.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL X(2,2),XT(2,2),TABLOG(11),MTOE
      LOGICAL EXIT,METRIC
      DATA TABLOG/0.0,0.0,0.0,0.30103,0.47712,0.60206,0.69897,
     $            0.77815,0.845098,0.90309,0.954243/
C
C
C     ASSIGN RMIN2 THE ACTUAL MINIMUM OF RMIN AND RMAX.
C     ASSIGN RMAX2 THE ACTUAL MAXIMUM OF RMIN AND RMAX.
C
      RMIN2=RMIN
      RMAX2=RMAX
      IF(RMIN2.LE.RMAX2) GO TO 10
      RMIN2=RMAX2
      RMAX2=RMIN
C
C
C     DETERMINE LOOP VALUES TO BE USED WHEN PLOTTING TICS.
C
   10 CALL LOGPR12(RMIN2,RMAX2,ITIC,MIN,MAX,I1,I2,ISTEP)
C
C
C     INITIALIZE CONSTANT OPPOSITE AXIS COORDINATES IN ARRAYS OF POINTS
C     TO BE USED WHEN PLOTTING THE TICS.
C
      NTWO=2
      ITIC2=ITIC
      OPSC=(OPAXMX-OPAXMN)/OPAXL
      TENTIC=0.08*OPSC
      II=I
      JJ=J
      X(1,JJ)=OPAXMN
      XT(1,JJ)=X(1,JJ)-TENTIC
      IF(ITIC2.GT.10) GO TO 70
      X(2,JJ)=OPAXMX
      XT(2,JJ)=X(2,JJ)+TENTIC
      GO TO 80
C
   70 TICL=0.13*OPSC
      X(2,JJ)=XT(2,JJ)=X(1,JJ)+TICL
      MINS=MIN
      EXIT=.FALSE.
C
C
C     PLOT THE TICS OR LINES ALONG THE LOGARITHMIC AXIS.
C
   80 DO 100 K=I1,I2,ISTEP
         X(1,II)=X(2,II)=XT(1,II)=XT(2,II)=MIN+TABLOG(K)
         IF(X(1,II).LE.RMIN2) GO TO 100
         IF(X(1,II).GE.RMAX2) GO TO 150
C
         IF (METRIC) GO TO 90
         IF (TABLOG(K).EQ.0.0) CALL PLOT(2,1,0,XT(1,1),XT(1,2),NTWO)
         IF(TABLOG(K).NE.0.0) CALL PLOT(2,1,0,X(1,1),X(1,2),NTWO)
         GO TO 95
   90    IF (TABLOG(K).EQ.0.0) CALL PLOTM(2,1,0,XT(1,1),XT(1,2),NTWO)
         IF (TABLOG(K).NE.0.0) CALL PLOTM(2,1,0,X(1,1),X(1,2),NTWO)
   95    NTWO=-NTWO
C
  100 CONTINUE
C
      MIN=MIN+1
      IF(MIN.LE.MAX) GO TO 80
C
  150 IF(ITIC2.LT.10) RETURN
      IF(EXIT) RETURN
C
      EXIT=.TRUE.
      X(2,JJ)=OPAXMX
      XT(2,JJ)=X(2,JJ)+TENTIC
      X(1,JJ)=XT(1,JJ)=X(2,JJ)-TICL
      MIN=MINS
      GO TO 80
      END
      SUBROUTINE LOGPR7(MODESW,PEN,SYMB,X,Y,N)
C
C
C LOGPR7 IS ONE OF THE MODULES OF THE LOGARITHMIC PLOTTING SYSTEM.  IT
C HANDLES THE PLOGZZ MODE 2 FUNCTIONS OF PLOTTING DATA POINTS, WITH LOGS
C TAKEN WHERE APPROPRIATE.
C
C IF THE MAIN ENTRY POINT HAS BEEN USED IN THE CALLING PROGRAM (ENGLISH
C DIMENSIONS HAVE BEEN USED ) PLOT IS CALLED AND IF THE ALTERNATE ENTRY
C HAS BEEN USED IN THE CALLING PROGRAM(METRIC DIMENSIONS HAVE BEEN
C USED ) PLOTM IS CALLED.
C
C
C ARGUMENT LIST:
C  MODESW..INTEGER, INDICATES WHICH ROUTINE IS CALLING:
C          1 - PLOGX,  2 - PLOGY,  3 - PLOGXY.
C  ALL OF THE OTHER PARAMETERS CORRESPOND TO THOSE OF A NORMAL PLOT MODE
C  2 CALL.
C
C ERROR CONDITIONS:
C  LOGPR7 CHECKS FOR ONLY ONE ERROR CONDITION:
C   ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL
C     THIS IS A FATAL ERROR.
C  THE PRINTING OF THIS ERROR MESSAGE IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  AN INDICATOR OF WHICH PLOGZZ ROUTINE WAS LAST CALLED UNDER MODE1, IF
C  ANY, IS OBTAINED BY CALLING ENTRY POINT LOGPR5 IN MODULE LOGPR3.
C  MODESW IS THEN CHECKED AGAINST THIS INDICATOR TO ENSURE THAT THE
C  PRESENT PLOGZZ MODE 2 CALL CORRESPONDS TO THE LAST MODE 1 CALL.  IF
C  THEY DO CORRESPOND, A CALL IS MADE TO PLTFLG.  THIS CALL INDICATES TO
C  THE STANDARD PLOTTING SUBROUTINE, PLOT, WHICH COORDINATES TO TAKE
C  THE LOGARITHMS OF.  FINALLY, A MODE TWO CALL OF PLOT IS MADE TO PLOT
C  THE DATA POINTS.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
C
C     INITIALIZE ERROR NUMBER FOR AELIB ERROR ROUTINE.
C        3522 - ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL.
C
      NERR=3522
C
C
C     CHECK THAT CORRESPONDING MODE 1 CALL HAS BEEN MADE.
C
      CALL LOGPR5(MAINSW)
*IF -DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR,1HF)
*ENDIF
*IF DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR,'F')
*ENDIF
C
C
C     TELL PLOT WHICH LOGARITHMS TO TAKE AND PLOT THE DATA.
C
      CALL PLTFLG(MODESW)
      IF (METRIC) GO TO 100
      CALL PLOT(2,PEN,SYMB,X,Y,N)
      RETURN
  100 CALL PLOTM(2,PEN,SYMB,X,Y,N)
      RETURN
      END
      SUBROUTINE LOGPR8(MODESW,P1,P2,P3,P4,P5,P6,P7)
C
C
C LOGPR8 IS ONE OF THE MODULES OF THE LOGARITHMIC PLOTTING SYSTEM.  IT
C HANDLES THE PLOGZZ MODE 3 FUNCTIONS OF PLOTTING CHARACTER STRINGS,
C WITH LOGS OF COORDINATES TAKEN WHERE APPROPRIATE.
C
C IF THE MAIN ENTRY POINT HAS BEEN USED IN THE CALLING PROGRAM (ENGLISH
C DIMENSIONS HAVE BEEN USED ) PLOT IS CALLED AND IF THE ALTERNATE ENTRY
C HAS BEEN USED IN THE CALLING PROGRAM(METRIC DIMENSIONS HAVE BEEN
C USED ) PLOTM IS CALLED.
C
C
C ARGUMENT LIST:
C  MODESW..INTEGER, INDICATES WHICH ROUTINE IS CALLING:
C          1 - PLOGX,  2 - PLOGY,  3 - PLOGXY.
C  ALL OF THE OTHER PARAMETERS CORRESPOND TO THOSE OF A NORMAL PLOT MODE
C  3 CALL.
C
C ERROR CONDITIONS:
C  LOGPR8 CHECKS FOR ONLY ONE ERROR CONDITION:
C   ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL
C     THIS IS A FATAL ERROR.
C  THE PRINTING OF THIS ERROR MESSAGE IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  AN INDICATOR OF WHICH PLOGZZ ROUTINE WAS LAST CALLED UNDER MODE 1, IF
C  ANY, IS OBTAINED BY CALLING ENTRY POINT LOGPR5 IN MODULE LOGPR3.
C  MODESW IS THEN CHECKED AGAINST THIS INDICATOR TO ENSURE THAT THE
C  PRESENT PLOGZZ MODE 3 CALL CORRESPONDS TO THE LAST MODE 1 CALL.  IF
C  THEY DO CORRESPOND, A CALL IS MADE TO PLTFLG.  THIS CALL INDICATES TO
C  THE STANDARD PLOTTING SUBROUTINE, PLOT, WHICH COORDINATES TO TAKE THE
C  LOGARITHMS OF.  FINALLY, A MODE THREE CALL OF PLOT IS MADE TO PLOT
C  THE CHARACTER STRING.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
C
C     INITIALIZE ERROR NUMBER FOR AELIB ERROR ROUTINE.
C        3522 - ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL.
C
      NERR=3522
C
C
C     CHECK THAT CORRESPONDING MODE 1 CALL HAS BEEN MADE.
C
      CALL LOGPR5(MAINSW)
*IF -DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR,1HF)
*ENDIF
*IF DEF,AELIB5
      IF(MAINSW.NE.MODESW) CALL AELERR(NERR,'F')
*ENDIF
C
C
C     TELL PLOT WHICH LOGARITHMS TO TAKE AND PLOT THE DATA.
C
      CALL PLTFLG(MODESW)
      IF (METRIC) GO TO 100
      CALL PLOT(3,P1,P2,P3,P4,P5,P6,P7)
      RETURN
  100 CALL PLOTM(3,P1,P2,P3,P4,P5,P6,P7)
      RETURN
      END
      SUBROUTINE LOGPR12(RMIN2,RMAX2,ITIC,MIN,MAX,I1,I2,ISTEP)
C
C
C LOGPR12 IS ONE OF THE MODULES OF WHICH THE LOGARITHMIC PLOTTING
C PACKAGE IS COMPRISED.  IT ALSO FORMS PART OF THE AXIS LABELLING AND
C AUTOMATIC PLOTTING SYSTEMS.  LOGPR12 SETS UP THE PARAMETERS FOR THE
C LOOP WHICH EITHER TICS, LABELS OR COMPUTES THE AXIS VALUES FOR A
C LOGARITHMIC SCALE.
C
C ARGUMENT LIST:
C  RMIN2...REAL, ALOG10(MINIMUM VALUE ON THE AXIS TO BE PROCESSED).
C  RMAX2...REAL, ALOG10(MAXIMUM VALUE ON THE AXIS TO BE PROCESSED).
C  ITIC....INTEGER, LOGARITHMIC TIC VALUE FOR THE AXIS TO BE PROCESSED.
C  MIN.....INTEGER, RETURNS LARGEST INTEGER EXPONENT OF TEN < RMIN2.
C  MAX.....INTEGER, RETURNS LARGEST INTEGER EXPONENT OF TEN < RMAX2.
C  I1......INTEGER, RETURNS STARTING VALUE FOR DO LOOP.
C  I2......INTEGER, RETURNS FINAL VALUE FOR DO LOOP.
C  ISTEP...INTEGER, RETURNS THE STEP VALUE FOR THE DO LOOP.
C
C ERROR CONDITIONS:
C  LOGPR12 DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C
C METHOD:
C  MIN AND MAX ARE DETERMINED BY CONVERTING RMIN2 AND RMAX2 TO TYPE
C  INTEGER, HANDLING NEGATIVE VALUES ACCORDINGLY.  THEN A BRANCH IS
C  MADE, DEPENDING UPON THE VALUE OF ITIC, TO THE SECTION WHICH WILL
C  ASSIGN I1, I2 AND ISTEP THEIR APPROPRIATE VALUES.
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
C
C
C     FIND LARGEST INTEGER EXPONENT OF TEN WHICH IS LESS THAN RMIN2.
C     FIND LARGEST INTEGER EXPONENT OF TEN WHICH IS LESS THAN RMAX2.
C
      MIN=RMIN2
      IF(RMIN2.LT.0.0) MIN=MIN-1
      MAX=RMAX2
      IF(RMAX2.LT.0.0) MAX=MAX-1
C
C
C     ASSIGN LIMITS FOR DO LOOP IN ACCORDANCE WITH THE ITIC VALUE.
C
      ITIC2=ITIC
      IF(ITIC2.GT.10) ITIC2=ITIC2-10
*IF -DEF,AELIB5
      GO TO (11,22,33,44,55), ITIC2
*ENDIF
*IF DEF,AELIB5
      GO TO (11,22,33,44,55), ITIC2
      CALL GOTOER
*ENDIF
C
C
   11 I1=I2=ISTEP=3
      GO TO 60
C
C
   22 I1=2
      I2=7
      ISTEP=5
      GO TO 60
C
C
   33 I1=1
      I2=7
      ISTEP=3
      GO TO 60
C
C
   44 I1=2
      I2=10
      ISTEP=2
      GO TO 60
C
C
   55 I1=3
      I2=11
      ISTEP=1
C
C
   60 RETURN
      END
