*DECK PLOGXY
      SUBROUTINE PLOGXY(MODE,PP1,PP2,P3,P4,P5,P6,P7,P8)
************************************************************************
**                                                                    **
**    NOTE THAT PLOGXY IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL *
**    ROUTINES:                                                       **
**       LOGPR1                                                       **
**       LOGPR2                                                       **
**       LOGPR3                                                       **
**       LOGPR6                                                       **
**       LOGPR7                                                       **
**       LOGPR8                                                       **
**       LOGPR12                                                      **
**    LOGPR1 THROUGH LOGPR12 ARE STORED WITH THE ROUTINE PLOGX UNDER  **
**    THE DECK NAME PLOGX.                                            **
**                                                                    **
************************************************************************
C
C
C PLOGXY IS A SUBROUTINE FOR PRODUCING LOG - LOG GRAPHS.  ITS CALLING
C SEQUENCES AND RESULTS ARE DESIGNED TO PARALLEL, IN MANY RESPECTS,
C THOSE OF THE STANDARD PLOTTING SUBROUTINE, PLOT.  THE FOLLOWING ARE
C ALL LEGAL CALLING SEQUENCES:
C     CALL PLOGXY(1,XL,YL,XMIN,XMAX,YMIN,YMAX,LGTICX,LGTICY)
C     CALL PLOGXY(2,PEN,SYMB,XX,YY,N)
C     CALL PLOGXY(3,FORM,SIZE,DIR,XLOC,YLOC,FWA,N)
C     CALL PLOGXY(3,FORM,SIZE,DIR,XLOC,YLOC)
C     CALL PLOGXY(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,LGTICXR,LGTICYR)
C
C THE SUBROUTINE HAS TWO ENTRY POINTS:
C    A) PLOGXY - ALL DIMENSIONAL QUANTITIES PASSED IN ARGUMENT LIST
C ARE IN INCHES
C    B) PLOGXYM- ALL DIMENSIONAL QUANTITIES ARE IN MILLIMETERS
C DIMENSIONAL QUANTITIES ARE PASSED IN THE ARGUMENT LIST AND, ON THE
C METRIC ENTRY, PLOGXYM, DIMENSIONS ARE CONVERTED TO ENGLISH UNITS ON A
C MODE 4 CALL, OR ON A MODE 1 CALL AFTER A PLOTM MODE 1 CALL HAS BEEN
C MADE TO PLOT THE FRAME.  A SWITCH INDICATING WHETHER METRIC DIMENSIONS
C HAVE BEEN USED IS PASSED IN A COMMON BLOCK TO THE MODULES WHICH CALL
C PLOT THAT ARE CALLED BY THIS ROUTINE.  IN THESE MODULES WHEN PLOGXY
C HAS BEEN CALLED, PLOT IS CALLED, AND WHEN PLOGXYM HAS BEEN CALLED,
C CORRESPONDING CALLS TO PLOTM ARE MADE.
C
C
C ARGUMENT LIST:
C  MODE....EITHER INTEGER OR REAL, SPECIFIES THE JOB TO BE DONE AND
C          DETERMINES THE NUMBER, TYPES AND MEANINGS OF THE OTHER
C          PARAMETERS.  LEGAL VALUES ARE 1,2,3 AND 4, CORRESPONDING TO
C          THE FIRST FOUR MODES OF PLOT.
C  ALL OF THE OTHER PARAMETERS ARE ESSENTIALLY THE SAME AS THOSE USED
C  BY PLOT, EXCEPT LGTICX, LGTICY, LGTICXR, AND LGTICYR WHICH DEFINE AND
C  RETURN THE LOG AXIS TIC VALUES.  ALSO, THE LOGARITHMS, RATHER THAN
C  THE ACTUAL MIN AND MAX VALUES ARE RETURNED BY MODE 4.
C
C ERROR CONDITIONS:
C  PLOGXY ITSELF DOES NOT CHECK FOR ANY ERRORS IN THE ARGUMENTS PASSED.
C  HOWEVER, THE SUBROUTINES IT CALLS CAN DETECT FOUR DIFFERENT ERROR
C  CONDITIONS:
C   1) ***MODE NOT 1,2,3 OR 4
C   2) ***INVALID LOG SCALE TIC VALUE SPECIFIED - DEFAULT IS 3
C   3) ***NO MODE 1 CALL BEFORE MODE 2,3 OR 4 CALL
C   4) ***ILLEGAL VALUES SPECIFIED FOR LOGARITHMIC SCALE
C  THE PRINTING OF ALL ERROR MESSAGES IS HANDLED BY THE STANDARD AELIB
C  ERROR PROCESSING ROUTINE, AELERR.
C
C METHOD:
C  THE LOGARITHMIC PLOTTING ROUTINES (PLOGX, PLOGY, PLOGXY) ARE WRITTEN
C  IN A MODULAR STYLE.  THESE ROUTINES MERELY CONTROL THE COMBINATION OF
C  THE VARIOUS MODULES WHICH CONSTITUTE THE LOGARITHMIC PLOTTING PACK-
C  AGE.  MODULES IN THE PACKAGE INCLUDE LOGPR1, LOGPR2, LOGPR3, LOGPR6,
C  LOGPR7, LOGPR8, AND LOGPR12.  PLOGXY FIRST CALLS LOGPR1 WHICH FORCES
C  THE MODE INTO TYPE INTEGER AND CHECKS IT FOR LEGALITY.  THEN A BRANCH
C  IS MADE, DEPENDING UPON THE VALUE OF MODE, TO THE APPROPRIATE SECTION
C  OF PLOGXY.  (THE METHOD USED WITHIN THE MODE SECTIONS IS EXPLAINED AT
C  THE BEGINNING OF EACH.)
C
C
C
C WRITTEN BY J. WENDORF, JUNE,1975.
C
      COMMON/MET/METRIC,ETOM,MTOE
      REAL MTOE
      LOGICAL METRIC
C
*CALL STATF
C
C
      METRIC=.FALSE.
      P1=PP1
      P2=PP2
      GO TO 50
C
C
C     ENTRY PLOGXYM ALLOWS USE OF METRIC DIMENSIONS
C
*IF -DEF,AELIB5
      ENTRY PLOGXYM
*ENDIF
*IF DEF,AELIB5
      ENTRY PLOGXYM(MODE,PP1,PP2,P3,P4,P5,P6,P7,P8)
*ENDIF
*CALL STATF
C
      METRIC=.TRUE.
      ETOM=25.4
      MTOE=1./ETOM
C
C     FORCE MODE INTO TYPE INTEGER,CHECK IT FOR LEGALITY, AND BRANCH,
C     DEPENDING UPON ITS VALUE, TO THE APPROPRIATE ROUTINE.
C
*IF -DEF,AELIB5
   50 GO TO (100,200,300,400), LOGPR1(MODE)
*ENDIF
*IF DEF,AELIB5
   50 GO TO (100,200,300,400), LOGPR1(MODE)
      CALL GOTOER
*ENDIF
C
C
C
C
C
C MODE 1: CALCULATE SCALE FACTORS, DRAW PLOT FRAME AND TICS.
C
C
C     CALL LOGPR2 WHICH CHECKS THAT XMIN AND XMAX ARE GREATER THAN ZERO
C     AND FINDS THEIR LOGS.  IT ALSO CHECKS LGTICX FOR LEGALITY, FORCES
C     IT INTO TYPE INTEGER, AND DETERMINES THE TICX VALUE TO USED IN
C     THE MODE 1 CALL OF PLOT.  LOGPR2 IS CALLED ONCE AGAIN, THIS TIME
C     TO HANDLE THE Y-AXIS PARAMETERS.  THEN THE PLOT MODE 1 CALL IS
C     MADE.  IT DRAWS THE FRAME LINES IF THE CORRESPONDING LGTIC VALUES
C     ARE \0.  ENTRY POINT LOGPR3 IN MODULE LOGPR3 IS THEN CALLED TO
C     STORE THE MODIFIED PLOGXY MODE 1 PARAMETERS.  IF LGTICX>0, LOGPR6
C     IS CALLED TO PERFORM THE LOGARITHMIC TICKING OF THE X-AXIS.
C     FINALLY, IF LGTICY>0, LOGPR6 IS CALLED FOR A SECOND TIME.  THIS
C     CALL PERFORMS THE LOGARITHMIC TICKING OF THE Y-AXIS.
C
  100 CALL LOGPR2(P3,P4,P7,XMIN,XMAX,ITICX,LGTICX)
      CALL LOGPR2(P5,P6,P8,YMIN,YMAX,ITICY,LGTICY)
      IF (METRIC) GO TO 110
      CALL PLOT(1,P1,P2,XMIN,XMAX,YMIN,YMAX,LGTICX,LGTICY)
      GO TO 120
C
C     USE METRIC DIMENSIONS TO PLOT THE FRAME
C
  110 CALL PLOTM(1,PP1,PP2,XMIN,XMAX,YMIN,YMAX,LGTICX,LGTICY)
C
C     CONVERT METRIC DIMENSIONS TO ENGLISH
C
      P1=PP1*MTOE
      P2=PP2*MTOE
  120 CALL LOGPR3(3,P1,P2,XMIN,XMAX,YMIN,YMAX,ITICX,ITICY)
      IF(ITICX.GT.0) CALL LOGPR6(P2,YMIN,YMAX,XMIN,XMAX,ITICX,1,2)
      IF(ITICY.GT.0) CALL LOGPR6(P1,XMIN,XMAX,YMIN,YMAX,ITICY,2,1)
      RETURN
C
C
C
C
C
C MODE 2: PLOT DATA POINTS.
C
C
C     CALL LOGPR7 TO CHECK THAT A PREVIOUS MODE 1 CALL OF PLOGXY HAS
C     BEEN EXECUTED AND TO PLOT THE DATA.
C
  200 CALL LOGPR7(3,PP1,PP2,P3,P4,P5)
      RETURN
C
C
C
C
C
C MODE 3: PLOT CHARACTER STRINGS.
C
C
C     CALL LOGPR8 TO CHECK THAT A PREVIOUS MODE 1 CALL OF PLOGXY HAS
C     BEEN EXCUTED AND TO PLOT THE CHARACTER STRING.
C
  300 CALL LOGPR8(3,PP1,PP2,P3,P4,P5,P6,P7)
      RETURN
C
C
C
C
C
C MODE 4: RETURN THE MODIFIED PARAMETERS OF THE PREVIOUS PLOGXY MODE 1
C         CALL.
C
C
C     CALL ENTRY POINT LOGPR4 IN MODULE LOGPR3 TO CHECK THAT A PREVIOUS
C     MODE 1 CALL OF PLOGXY HAS BEEN EXECUTED AND TO RETRIEVE THE MODE 1
C     PARAMETERS.
C
  400 IF (METRIC) GO TO 410
      CALL LOGPR4(3,P1,P2,P3,P4,P5,P6,P7,P8)
      RETURN
C
C     CONVERT METRIC DIMENSIONS TO ENGLISH
C
  410 P1=PP1*MTOE
      P2=PP2*MTOE
      CALL LOGPR4(3,P1,P2,P3,P4,P5,P6,P7,P8)
      RETURN
      END
