*DECK PLOT3D
      SUBROUTINE PLOT3D(XLP,YLP,ZLP,XP,YP,ZP,NX,NY,NRZP,PHI)
*
*     XLP- PROJECTED LENGTH OF XP AXIS ON PLOTTING SURFACE (INCHES)
*     YLP- LENGTH OF YP AXIS(INCHES)
*     ZLP- LENGTH OF ZP AXIS(INCHES)
*
*          *                                *
*          *           * XP               Y *
*      ZP  *         *              *       *
*          *       *           *******      *
*          *     *                  *       *
*          *   *  PHI                       *
*          * *                              *
*          *****************                *****************
*                    YP                                  X
*
*                            X=YP+XP*COS(PHI)
*                            Y=ZP+XP*SIN(PHI)
*
*     XP - ARRAY OF NX VALUES OF XP AS MEASURED ALONG THE PROJECTION
*          ON THE PLOTTING SURFACE. XP(1) LIES IN THE PLANE OF THE
*          DRAWING, AND XP INCREASES AS ONE MOVES ALONG THE XP AXIS
*          AWAY FROM THE VIEWER
*     YP - ARRAY OF NY VALUES OF YP. YP INCREASES TO THE RIGHT.
*     ZP - ARRAY OF NX*NY VALUES OF ZP(XP,YP). ZP(I,J) IS THE HEIGHT
*          OF THE SURFACE AT THE POINT  XP(I),YP(J). ZP INCREASES
*          UPWARDS. NOTE THAT ZMIN IS ASSUMED TO BE ZERO
*
*     PHI IS THE ANGLE BETWEEN THE PROJECTED XP AXIS AND THE YP AXIS
*
*     MAXIMUM SIZES=  YLP+XLP*COS(PHI) .LE. 20
*                     ZLP+XLP*SIN(PHI) .LE. 10
*                        NY .LE.64
*
*
*
*     METHOD -
*             FOR I=1 BY 1 TO NX DO
*                  FOR J=NY BY -1 TO 1 DO BEGIN
*                       IF(J.LT.NY) DRAW (I,J+1) TO (I,J)
*                       IF(I.GT.1) DRAW (I,J) TO (I-1,J) AND BACK
*
*
*     THE SUBROUTINE HAS TWO ENTRY POINTS:
*        A) PLOT3D -  ALL DIMENSIONAL QUANTITIES PASSED IN ARGUMENT LIST
*     ARE IN INCHES
*        B) PLOT3DM - ALL DIMENSIONAL QUANTITIES ARE IN MILLIMETERS
*     DIMENSIONAL QUANTITIES ARE PASSED IN THE ARGUMENT LIST AND ON THE
*     METRIC ENTRY, PLOT3DM, ARE CONVERTED TO ENGLISH UNITS.  CALLS TO
*     PLOT ARE MADE BY PLOT3D AND PLOT3DM MAKES CORRESPONDING CALLS TO
*     PLOTM.  DIMENSIONAL QUANTITIES ARE CONVERTED BACK TO MILLIMETERS
*     BEFORE A PLOTM MODE 1 CALL TO CALCULATE THE SCALING FACTORS AND
*     DEFINE THE PLOTTING AREA IS MADE BY PLOT3DM.
*
*
*
      DIMENSION XP(NX),YP(NY),ZP(NRZP,NY)
      DIMENSION X(64),Y(64),SX(224),SY(224),FIN(100),R(2002)
      DIMENSION TX(224),TY(224)
      LOGICAL METRIC
      REAL MTOE
      INTEGER X,Y,FIN,R,BB,CC,SX,SY,STX,STY,XOLD,YOLD
*CALL  STATF
*
      METRIC=.FALSE.
      XLE=XLP
      YLE=YLP
      ZLE=ZLP
      GO TO 5
*
*
*     ALTERNATE ENTRY ALLOWS METRIC UNITS TO BE USED AS DIMENSIONS
*
*IF -DEF,AELIB5
      ENTRY PLOT3DM
*ENDIF
*IF DEF,AELIB5
      ENTRY PLOT3DM(XLP,YLP,ZLP,XP,YP,ZP,NX,NY,NRZP,PHI)
*ENDIF
*
*CALL STATF
*
      METRIC=.TRUE.
      ETOM=25.4
      MTOE=1./ETOM
*
*     CONVERT METRIC DIMENSIONS TO ENGLISH
*
      XLE=XLP*MTOE
      YLE=YLP*MTOE
      ZLE=ZLP*MTOE
*
*     INITIALIZE
    5 IPEN=1
      ISYMB=0
      FIN(1)=ZMAX=0
      RAD=PHI*3.14159265/180.
      XMIN=XP(1)
      XMAX=XP(NX)
      YMIN=YP(1)
      YMAX=YP(NY)
      DO 10 I=1,NX
      DO 10 J=1,NY
      IF(ZP(I,J).GT.ZMAX)ZMAX=ZP(I,J)
10    CONTINUE
      DO 11 I=1,2002
11    R(I)=0
*
*     CALCULATE SCALING FACTORS
*
      SCX1=XLE*100./(XMAX-XMIN)
      SCX2=SCX1*SIN(RAD)
      SCX1=SCX1*COS(RAD)
      SCY=YLE*100./(YMAX-YMIN)
      SCZ=ZLE*100./ZMAX
      XL=(YLE+XLE*COS(RAD))*100.
      LX=XL+.5
      XXL=XL/100.
      IF(XXL-20.)14,14,12
12    PRINT 13,XXL
      CALL EXIT
*IF -DEF,AELIB5
13    FORMAT(14H0PLOT LENGTH =,F7.2,X,14HEXCEEDS 20 IN.  )
*ENDIF
*IF DEF,AELIB5
13    FORMAT(14H0PLOT LENGTH =,F7.2,1X,14HEXCEEDS 20 IN.  )
*ENDIF
14    YYL=10.
      I=-1
      XXMIN=0
      XXMAX=LX
      YYMIN=0
      YYMAX=1000.
      IF (METRIC) GO TO 15
      CALL PLOT(1,XXL,YYL,XXMIN,XXMAX,YYMIN,YYMAX,I,I)
      GO TO 17
*
*     CONVERT ENGLISH DIMENSIONS TO METRIC BEFORE MAKING A PLOTM MODE 1
*     CALL TO CALCULATE THE SCALING FACTORS AND DEFINE THE PLOTTING
*     AREA.
*
   15 XXL=XXL*ETOM
      YYL=YYL*ETOM
      CALL PLOTM(1,XXL,YYL,XXMIN,XXMAX,YYMIN,YYMAX,I,I)
   17 DO 100 I=1,NX
*
*     SCAN LINES OF CONSTANT XP, BEGINNING AT XP(1). THE SCAN OF EACH
*     LINE IS FROM RIGHT TO LEFT, IE FROM YP(NY) TO YP(1).
*     THE COORDINATES OF THE PROJECTIONS OF THE POINTS Z(I,J),
*     J=1, NY ARE SAVED IN  X, Y - THEY ARE NEEDED DURING THE NEXT SCAN
*     TO DRAW THE CONNECTIONS YP=CONSTANT. THE CC SEGMENTS OF THE XP=
*     CONSTANT CURVE ARE STORED IN SX,SY. THE END POINT OF THE SEGMENT
*     CC IS HELD IN FIN(CC+1).
*
      K=BB=1
      CC=0
      XX=(XP(I)-XMIN)*SCX1
      XY=(XP(I)-XMIN)*SCX2
      J=NY
20    XOLD=X(J)
      YOLD=Y(J)
      X(J)=(YP(J)-YMIN)*SCY+XX+.5
      Y(J)=ZP(I,J)*SCZ+XY+.5
*
*     IF J=NY THEN PUT COORDINATES IN SX(1),SY(1) ELSE GO TO DRAW LINE
*          OF CONSTANT XP WHICH CONNECTS ZP(I,J+1) TO ZP(I,J)
      IF(J.NE.NY)GO TO 3000
      SX(K)=X(J)
      SY(K)=Y(J)
*
*     UNLESS THIS IS FIRST LINE IE XP(1), DRAW CONNECTION YP=CONST
30    IF(I.NE.1)GO TO 3001
*
*     DECREMENT J AND DO NEXT POINT
31    J=J-1
      IF(J.GT.0)GO TO 20
*
      CC=CC+1
      FIN(CC+1)=K
*
*     NOW DRAW THE SEGMENTS. TO MINIMIZE PLOTTING TIME, DRAW ALTERNATE-
*     LY LEFT TO RIGHT AND RIGHT TO LEFT
*
      IF((I.AND.1).EQ.0)GO TO 50
*     LEFT TO RIGHT
      DO 40 J=1,CC
      K1=FIN(J)
      K2=FIN(J+1)
      NK=K2-K1
      IF(NK.LT.2)GO TO 40
      DO 35 K=1,NK
      TX(K)=SX(K+K1)
   35 TY(K)=SY(K+K1)
      IF (METRIC) GO TO 36
      CALL PLOT(2,1,0,TX,TY,NK)
      GO TO 40
   36 CALL PLOTM(2,1,0,TX,TY,NK)
40    CONTINUE
      GO TO 100
*
*     RIGHT TO LEFT
*
50    DO 60 JJ=1,CC
      J=CC-JJ+1
      K1=FIN(J)
      K2=FIN(J+1)
      NK=K2-K1
      IF(NK.LT.2)GO TO 60
      DO 51 K=1,NK
      TX(K)=SX(K2-K+1)
51    TY(K)=SY(K2-K+1)
      IF (METRIC) GO TO 55
      CALL PLOT(2,1,0,TX,TY,NK)
      GO TO 60
   55 CALL PLOTM(2,1,0,TX,TY,NK)
60    CONTINUE
*
100   CONTINUE
*
      RETURN
*
*
*     ENTRY TO SET UP LINE OF CONSTANT XP FROM J+1 TO J  (ISW=1)
*
3000  ISW=1
      IC=X(J+1)
      ID=Y(J+1)
      IE=X(J)
      IF=Y(J)
      GO TO 3010
*
*     SET UP FOR LINE OF CONSTANT YP FROM I TO I-1 (ISW=2)
3001  IC=X(J)
      ID=Y(J)
      IE=XOLD
      IF=YOLD
      ISW=2
*
3010  YY=IC-IE
      IF(YY)3011,3012,3011
3011  AM=(ID-IF)/YY
      B=(IC*IF-IE*ID)/YY
      GO TO 3013
3012  AM=1
      B=0
3013  STX=IC
      STY=ID
3020  II=STX+1
*
*     SCAN LINE FROM X=STX TO X=IE. CALCULATE VALUE OF Y AT EACH POINT -
*     IF GREATER THAN THE R ENTRY FOR THAT VALUE OF X, UPDATE R AND
*     CONTINUE THE SCAN. IF LESS THAN THE R ENTRY THE LINE BECOMES (IS)
*     HIDDEN - GO TO 3025 TO FIND OUT WHAT TO DO.
*
3021  II=II-1
      IF(II.LT.IE)GO TO 3022
      IY=AM*II+B+.5
      IF(IY.LT.R(II+1))GO TO 3025
      R(II+1)=IY
      GO TO 3021
*
*     WE HAVE REACHED E,F.  NOW CHECK BB - IF BB.EQ.1 THE POINT STX,STY
*     HAS ALREADY BEEN PLOTTED.  IF BB.EQ.0 THE POINT STX,STY MUST BE
*     ENTERED INTO SX,SY AS THE START OF A NEW VISIBLE SEGMENT.
*
3022  IF(BB.NE.0)GO TO 3023
      K=K+1
      SX(K)=STX
      SY(K)=STY
      BB=1
3023  K=K+1
      SX(K)=IE
      SY(K)=IF
      IF(ISW.EQ.1)GO TO 3099
*
*     ELSE A LINE OF CONSTANT YP IS BEING DRAWN AND WE WISH TO RETRACE
*     LEAVING PEN POSITIONED AT C,D SO AS TO AVOID UNECCESSARY RAISING
*     AND LOWERING OF THE PEN
*
      K=K+1
      SX(K)=STX
      SY(K)=STY
      GO TO 3099
*
*
3025   KP=II+1
*
*     WE ARE HERE BECAUSE PART OR ALL OF THE LINE BEING DRAWN IS
*     INVISIBLE.   TEST FOR THE VARIOUS POSSIBILITIES -
*        1. THE STARTING POINT STX,STY IS NOT VISIBLE, WE ARE ALREADY ON
*           A HIDDEN SEGMENT, AND DRAWING A LINE OF YP=CONSTANT.  THE
*           WHOLE LINE MUST BE INVISIBLE SO EXIT.
*
      IF((II.EQ.STX).AND.(BB.EQ.0).AND.(ISW.EQ.2))GO TO 3099
*
*        2. AS ABOVE, BUT FOR CONSTANT XP. THE SCAN MUST CONTINUE
*           SINCE THE LINE MAY REAPPEAR BEFORE REACHING E,F.
*
      IF((II.EQ.STX).AND.(BB.EQ.0))GO TO 3060
*
*        3. IF PREVIOUSLY THE LINE WAS HIDDEN (BB=0) THEN IT HAS
*           REAPPEARED AT STX,STY WHICH MUST BE ENTERED INTO SX,SY TO
*           START THE NEW VISIBLE SEGMENT.  IF THE LINE IS ALREADY
*           VISIBLE, STX,STY HAS ALREADY BEEN PLOTTED.
*
      IF(BB.NE.0) GO TO 3035
      K=K+1
      SX(K)=STX
      SY(K)=STY
*
*        4. IN EITHER CASE THE LINE HAS VANISHED AT X=II. NOTE THIS.
*
3035  K=K+1
      SX(K)=KP
      SY(K)=R(KP+1)
*
*     NOTE - WE DO NOT YET END THE VISIBLE SEGMENT SINCE WE MAY WISH TO
*     RETRACE.
*        5. IF STX,STY IS THE POINT AT WHICH THE LINE VANISHES (THIS
*           IMPLIES BB=1 ELSE WE WOULD HAVE BRANCHED AT 2) AND A LINE OF
*           YP=CONSTANT IS UNDER CONSIDERATION, WE EXIT. DO NOT
*           TERMINATE THE VISIBLE SEGMENT HERE - THAT WILL BE LOOKED
*           AFTER WHEN DRAWING THE NEXT XP=CONSTANT SEGMENT.
*
      IF((II.EQ.STX).AND.(ISW.EQ.2))GO TO 3099
*
*        6. ON THE OTHER HAND, IF THE POINT OF DISSAPPEARANCE IS
*           STX,STY BUT ON A LINE OF XP=CONSTANT WE MUST END THE
*           VISIBLE SEGMENT AND RESUME THE SCAN.
*
      IF(II.EQ.STX)GO TO 3050
*
*        7. IF WE REACH THIS POINT, A LINE SEGMENT HAS VANISHED BUT NOT
*           AT STX,STY. IF ON A LINE OF YP=CONSTANT, RETRACE AND EXIT.
*           IF ON A LINE OF XP=CONSTANT, TERMINATE THE SEGMENT AND THEN
*           RESUME SCAN.
*
      IF(ISW.EQ.1) GO TO 3050
      K=K+1
      SX(K)=STX
      SY(K)=STY
      GO TO 3099
*
*     TERMINATE VISIBLE SEGMENT
3050  CC=CC+1
      FIN(CC+1)=K
      BB=0
*
*     SCAN REMAINDER OF LINE.
3060  II=KP-1
3061  II=II-1
      IF(II.LT.IE)GO TO 3099
      IY=AM*II+B+.5
      IF(IY.LT.R(II+1))GO TO 3061
*
*     REAPPEARANCE.  UPDATE R,SET STX,STY TO POINT OF REAPPEARANCE AND
*     GO BACK TO SQUARE ONE.
*
      R(II+1)=IY
      STX=II
      STY=IY
      GO TO 3020
*
*IF -DEF,AELIB5
3099  GO TO(30,31)ISW
*ENDIF
*IF DEF,AELIB5
3099  GO TO(30,31)ISW
      CALL GOTOER
*ENDIF
      END
