*DECK POLFIT
      SUBROUTINE POLFIT(X,Y,W,YF,PARAM,N,MA,E,NE,S,WS)
C THIS SUBROUTINE PERFORMS LINEAR LEAST SQUARES FITTING TO A
C POLYNOMIAL OF A SPECIFIED ORDER IN ONE INDEPENDENT VARIABLE OR TO
C A SERIES OF SUCH POLYNOMIALS OF VARYING ORDERS. THE METHOD USED
C IS FITTING TO A LINEAR COMBINATION OF LEGENDRE POLYNOMIALS ON THE
C INTERVAL (-1,1) WITH SUBSEQUENT TRANSFORMATION OF THE FITTED
C COEFFICIENTS TO COEFFICIENTS OF A POLYNOMIAL IN THE
C INDEPENDENT VARIABLE.
C THIS ROUTINE USES A ONE DIMENSIONAL WORKING STORAGE ARRAY
C WHICH IS DIMENSIONED IN THE MAIN PROGRAM AND PASSED AS A
C FORMAL PARAMETER.
C
C ARGUMENT LIST
C    X - REAL ARRAY OF VALUES OF INDEPENDENT VARIABLE
C    Y - REAL ARRAY OF VALUES OF DEPENDENT VARIABLE
C    W - REAL ARRAY OF WEIGHTS FOR DEPENDENT VARIABLE
C    YF - REAL ARRAY OF FITTED VALUES OF DEPENDENT VARIABLE RETURNED
C         BY POLFIT
C    PARAM - REAL ARRAY OF COEFFICIENTS OF THE FITTED POLYNOMIAL
C            RETURNED BY POLFIT. PARAM(I)=COEFF. OF X**(I-1)
C    N - INTEGER, NUMBER OF DATA POINTS I.E., NUMBER OF X,Y,W VALUES
C    MA   - FOR FITTING TO A SINGLE POLYNOMIAL, MA IS AN INTEGER
C           SPECIFYING THE ORDER OF THE POLYNOMIAL. PARAM MUST
C           BE DIMENSIONED AT LEAST M=MA+1 IN THE CALLING PROGRAM
C         - FOR FITTING TO A SERIES OF POLYNOMIALS, MA IS AN INTEGER
C           ARRAY DEFINED AS FOLLOWS -
C                  MA(1)=0
C                  MA(2)=MINIMUM ORDER OF FIT
C                  MA(3)=MAXIMUM ORDER OF FIT
C                  MA(4)=INCREMENT IN ORDER OF FIT
C           SO PARAM MUST BE DIMENSIONED AT LEAST M=MA(3)+1 IN
C           CALLING PROGRAM
C    E - REAL ARRAY FOR RETURN OF ERROR MATRIX FROM POLFIT.
C        MUST BE DIMENSIONED AT LEAST M*M WHERE M IS DEFINED AS ABOVE
C        FOR PARAM
C    NE - INTEGER, NUMBER OF ROWS OF E AS DIMENSIONED IN CALLING PROGRAM
C    S - REAL, SUM OF SQUARES OF RESIDUALS RETURNED FROM POLFIT.
C        SUM(W(I)*(Y(I)-YF(I))**2) FOR I=1,N
C    WS - REAL WORKING STORAGE ARRAY, MUST HAVE DIMENSION AT LEAST
C         M(4+N+2M) WHERE M=MA+1 FOR SINGLE FIT AND M=MA(3)+1 FOR
C         MULTIPLE FIT.
C
C WORKING STORAGE ALLOCATION
C    1. WS(1)-WS(M) - USED FOR RIGHT HAND SIDE OF NORMAL EQUATIONS
C                     AND REALLOCATED LATER FOR STORAGE OF POWERS OF
C                     ALPHA
C
C    2. WS(I2+1)-WS(I2+M*(M+1)/2),I2=M
C                  USED FOR LOWER TRIANGULAR PORTION OF NORMAL MATRIX
C
C    3. WS(I3+1)-WS(I3+N*M) ,I3=I2+M*(M+1)/2
C                  USED FOR LEGENDRE POLYNOMIAL VALUES FOR ALL
C                  DATA POINTS
C
C    4. WS(I4+1)-WS(I4+M*(M+1)/2), I4=I3+N*M
C                  USED FOR LOWER TRIANGULAR PORTION OF BINOMIAL ARRAY
C
C    5. WS(I5+1)WS(I5+M*M), I5=I4+M*(M+1)/2
C                  USED FOR LEGENDRE MATRIX FOR CONVERSION TO POLYNOMIAL
C
C    6. WS(I6+1)-WS(I6+M),I6=I5+M*M
C                  TEMPORARY STORAGE FOR POLYNOMIAL EXPANSION
C                  EQUIVALENT TO LEGENDRE EXPANSION
C
C    7. WS(I7+1)-WS(I7+M),I7=I6+M
C                  TEMPORARY STORAGE FOR FITTED LEGENDRE COEFFICIENTS
C
C
C AUTHOR AND REVISION HISTORY -
C    (REPLACES PROGRAM FITS AND SEVERAL VERSIONS OF LINFIT)
C    L EVANS     JULY,1974.
C
C
C
      REAL X(1),Y(1),W(1),YF(1),PARAM(1),E(NE,1),WS(1)
      INTEGER MA(1)
C
C     THE ARRAY WSI8 WAS ADDED TO GIVE ADDITIONAL WORKING STORAGE
C     SPACE OF LENGTH M WITHOUT NECESSITATING CHANGING THE USER
C     CALCULATION AND DIMENSIONING OF WS. WHEN THIS PROGRAM IS NEXT
C     MODIFIED, THIS ARRAY SHOULD BE ADDED TO THE END OF WS AND
C     THE WRITEUP REVISED
C
      REAL WSI8(50)
*CALL  STATF
C
C     SET HIGHEST AND LOWEST ORDER AND INCREMENT FOR SINGLE FIT
C
      NDM=1
      MAX1=MIN1=MA(1)+1
C
C     IF MULTIPLE FIT REQUIRED, I.E. MA(1)=0, THEN RETRIEVE
C     HIGHEST AND LOWEST ORDER AND INCREMENT
C
      IF(MA(1).NE.0) GOTO 2
      MIN1=MA(2)+1
      MAX1=MA(3)+1
      NDM=MA(4)
  2   M=MAX1
C
C     SET UP POINTERS TO WORKING STORAGE AND PRESET WORKING STORAGE TO 0
C
      MM=M*(M+1)/2
      I2=M
      I3=I2+MM
      I4=I3+N*M
      I5=I4+MM
      I6=I5+M*M
      I7=I6+M
      I8=I7+M
      CALL PRESET(WS,I8,0.)
C
C     SET UP ARRAY,A, OF BINOMIAL COEFFICIENTS IN WORKING STORAGE,
C     USING METHOD OF PASCALS TRIANGLE.(SEE APOSTOL,CALCULUS,VOL1,P 30)
C     SINCE A IS LOWER TRIANGULAR, THEN ONLY A(I,J),I GE J ARE STORED
C     A(I,J)=WS(I4+II+J) WHERE II=I*(I-1)/2
C
      WS(I4+1)=1.
      DO 10 I=2,M
         II=I*(I-1)/2
         WS(I4+II+1)=WS(I4+II+I)=1.
         IF(I.EQ.2) GOTO 10
         L=I-1
         LL=L*(L-1)/2
         DO 5 J=2,L
            WS(I4+II+J)=WS(I4+LL+J)+WS(I4+LL+J-1)
  5      CONTINUE
 10   CONTINUE
C
C     SET UP LEGENDRE MATRIX,RL,IN WORKING STORAGE. RL(I,J) IS COEFF.
C     OF X**(J-1) IN LEGENDRE POLYNOMIAL P(I-1).
C     RL(I,J)=WS(I5+II+J) WHERE II=(I-1)*M
C
      WS(I5+1)=WS(I5+M+2)=1.
      IF(M.EQ.2) GOTO 30
      DO 20 I=3,M
         J=I-1
         II=(I-1)*M
         JJ=(J-1)*M
         WS(I5+II+I)=WS(I5+JJ+J)*(I+I-3)/J
         L=I-1
         DO 15 K=1,L,2
            K1=L-K
            WS(I5+II+K1)=WS(I5+II+K1+2)*(K1+1)*K1/((K1-I)*(K1+I-1))
  15     CONTINUE
  20  CONTINUE
  30  CONTINUE
C
C     CHECK W AND X VALUES.  SET ALL WEIGHTS TO 1.0 IF ALL ARE 0.0.
C     IF NOT ALL 0.0, LEAVE WEIGHTS AS IS.  FIND MAX AND MIN X
C     VALUES TO CALCULATE SCALING FACTORS.
C
      IMIN=IMAX=1
      MM=0
      DO 40 I=1,N
           IF(W(I).NE.0.)GO TO 45
           MM=MM+1
   45      IF(X(I).LT.X(IMIN))IMIN=I
           IF(X(I).GT.X(IMAX))IMAX=I
   40 CONTINUE
      IF(MM.NE.N)GO TO 50
      DO 55 I=1,N
           W(I)=1.0
   55 CONTINUE
C
C     CALCULATE SCALING FACTORS TO TRANSFORM (X(IMIN),X(IMAX))
C     INTO(-1,1)
C
   50 BETA=1./(X(IMAX)-X(IMIN))
      ALPHA=-BETA*(X(IMAX)+X(IMIN))
      BETA=2*BETA
      BA=BETA/ALPHA
C
C     SET UP NORMAL EQUATIONS
C
C     FOR EACH OF I=1,N DATA POINTS, DO THE FOLLOWING -
C     CALCULATE THE LEGENDRE POLYNOMIAL VALUES FOR ALL M POLYNOMIALS
C        WS(I3+II+J)=P(J-1,X(I))  WHERE II=(I-1)*M
C
C     ACCUMULATE RHS OF NORMAL EQUATIONS (J=1,M)
C        WS(J)=WS(J)+W(I)*Y(I)*P(J-1,X(I))
C
C     ACCUMULATE NORMAL EQUATION MATRIX (J=1,M,K=1,J)
C     ONLY LOWER TRIANGULAR MATRIX IS STORED BECAUSE OF SYMMETRY
C        WS(I2+JJ+K)=WS(I2+JJ+K)+W(I)*P(J-1,X(I))*P(K-1,X(I))
C                  WHERE JJ=J*(J-1)/2
C
      DO 80 I=1,N
C
C     LEGENDRE POLYNOMIAL CALCULATION
C
         II=(I-1)*M
         WS(I3+II+1)=1.
         WS(I3+II+2)=ALPHA+BETA*X(I)
         IF(M.EQ.2) GOTO 61
         DO 60 J=3,M
            WS(I3+II+J)=((J+J-3)*WS(I3+II+2)*WS(I3+II+J-1)-
     1                  (J-2)*WS(I3+II+J-2))/(J-1)
  60     CONTINUE
  61     CONTINUE
         DO 70 J=1,M
            JJ=J*(J-1)/2
C
C           RHS CALCULATION
C
            WS(J)=WS(J)+W(I)*Y(I)*WS(I3+II+J)
            DO 65 K=1,J
C
C           NORMAL MATRIX CALCULATION
C
               WS(I2+JJ+K)=WS(I2+JJ+K)+W(I)*WS(I3+II+J)*WS(I3+II+K)
  65        CONTINUE
  70     CONTINUE
  80  CONTINUE
C
C     NOW SOLVE NORMAL EQUATIONS USING CHOLESKYS METHOD
C
C     FIRST THE NORMAL MATRIX IS TRIANGULARIZED. THE LOWER TRIANGULAR
C     REPRESENTATION OF THE NORMAL MATRIX IS REPLACED BY A LOWER
C     TRIANGULAR MATRIX,L , SUCH THAT L*LT=NORMAL MATRIX
C     SO WS(I2+II+J)=L(I,J) WHERE I GE J AND II=I*(I-1)/2
C
      DO 150 I=1,M
         II=I*(I-1)/2
         DO 140 J=1,I
            KK=J-1
            JJ=J*KK/2
            SUM=WS(I2+II+J)
            IF(KK.EQ.0) GOTO 130
            DO 120 K=1,KK
               SUM=SUM-WS(I2+II+K)*WS(I2+JJ+K)
  120       CONTINUE
  130       IF(I.EQ.J) GOTO 135
            WS(I2+II+J)=SUM/WS(I2+JJ+J)
            GOTO 140
  135 WS(I2+II+J)=SQRT(SUM)
  140    CONTINUE
  150 CONTINUE
C
C     THEN INVERT THE LOWER TRIANGULAR MATRIX USING SUBSTITUTION
C     FROM FIRST ROW TO LAST. INVERSE OF L REPLACES L IN WORKING
C     STORAGE.
C
      WS(I2+1)=1./WS(I2+1)
      DO 200 I=2,M
         L=I-1
         II=L*I/2
         DO 190 J=1,L
            SUM=0.
            DO 180 K=J,L
               KK=K*(K-1)/2
               SUM=SUM-WS(I2+II+K)*WS(I2+KK+J)
  180       CONTINUE
            WS(I2+II+J)=SUM/WS(I2+II+I)
  190    CONTINUE
         WS(I2+II+I)=1./WS(I2+II+I)
  200 CONTINUE
C
C     TO COMPLETE THE INVERSION OF THE NORMAL MATRIX, CALCULATE
C     INV(L)T*INV(L) AND REPLACE INV(L) WITH THIS IN WORKING STORAGE.
C
C     THEN THE WORKING STORAGE SPACE ALLOCATED TO THE NORMAL MATRIX
C     WILL CONTAIN ITS INVERSE
C
      DO 250 I=1,M
         II=I*(I-1)/2
         DO 240 J=1,I
            SUM=0.
            DO 230 K=I,M
               KK=K*(K-1)/2
               SUM=SUM+WS(I2+KK+J)*WS(I2+KK+I)
  230       CONTINUE
            WS(I2+II+J)=SUM
  240    CONTINUE
  250 CONTINUE
C
C
C     NOW CALCULATE THE PARAMETER VALUES FOR THE FIT TO LEGENDRE
C     POLYNOMIALS. MULTIPLY RHS BY INVERTED NORMAL MATRIX
C
      DO 320 I=1,M
         K=I+1
         II=I*(I-1)/2
         WSI8(I)=0.
C
C     GO ACROSS ROW TO DIAGONAL
C
         DO 300 J=1,I
         WSI8(I)=WSI8(I)+WS(I2+II+J)*WS(J)
  300    CONTINUE
C
C     AND THEN DOWN COLUMN (EXCEPT FOR LAST ROW)
C
         IF(I.EQ.M) GOTO 320
         DO 310 J=K,M
            JJ=J*(J-1)/2
         WSI8(I)=WSI8(I)+WS(I2+JJ+I)*WS(J)
  310       CONTINUE
  320 CONTINUE
C
C
C     REALLOCATE SOME OF WORKING STORAGE
C        WS(I),I=1,M IS NOW USED FOR VALUES OF ALPHA**(I-1)
C
C
C     SET UP POWERS OF ALPHA
C
      IF(ALPHA.EQ.0.) GOTO 330
      WS(1)=1.
      DO 325 I=2,MAX1
         WS(I)=ALPHA*WS(I-1)
  325 CONTINUE
  330 CONTINUE
C
C     THE REST OF THE CALCULATIONS ARE PERFORMED FOR EACH OF REQUIRED
C     FITS.
C
      DO 800 M=MIN1,MAX1,NDM
C
C
C     CALCULATE FITTED FUNCTION VALUES, USING STORED LEGENDRE
C     POLYNOMIAL VALUES.
C     ALSO ACCUMULATE SUM OF SQUARES OF RESIDUALS
C
      S=0.
      DO 350 I=1,N
         YF(I)=0.
         II=(I-1)*MAX1
         DO 340 J=1,M
         YF(I)=YF(I)+WS(I3+II+J)*WSI8(J)
  340    CONTINUE
         S=S+(Y(I)-YF(I))**2*W(I)
  350 CONTINUE
C
C     THE INVERSE OF THE NORMAL EQUATION MATRIX IS THE ERROR (VARIANCE-
C     COVARIANCE) MATRIX OF THE FIT. CALCULATE THE MODIFIED CORRELATION
C     MATRIX BY FIRST MULTIPLYING EACH ELEMENT BY S/(N-M) AND THEN
C     CALCULATING STANDARD DEVIATIONS AS DIAGONAL ELEMENTS AND
C     CORRELATION COEFFICIENTS BELOW THE DIAGONAL
C     LEAVE COVARIANCES ABOVE THE DIAGONAL
C
C     IF N EQUALS M THEN SET ST=0 AND ERROR MATRIX HAS NO MEANING
C
      ST=0.
      IF(N.NE.M) ST=S/(N-M)
      DO 410 I=1,M
         II=I*(I-1)/2
         K=I-1
         DO 400 J=1,K
            E(I,J)=WS(I2+II+J)*ST
  400    CONTINUE
         E(I,I)=SQRT(WS(I2+II+I)*ST)
  410 CONTINUE
      DO 430 I=2,M
         K=I-1
         DO 420 J=1,K
            E(J,I)=E(I,J)
            E(I,J)=       E(I,J)/(E(I,I)*E(J,J))
  420    CONTINUE
  430 CONTINUE
C
C     TRANSFORM COEFFICIENTS OF LEGENDRE EXPANSION TO COEFFICIENTS
C     OF POLYNOMIAL IN X
C
C     CALCULATE COEFFICIENTS OF POLYNOMIAL EXPANSION EQUIVALENT TO
C     LEGENDRE EXPANSION
C
      DO 500 I=1,M
         WS(I6+I)=0.
         DO 490 J=I,M
            JJ=(J-1)*MAX1
            WS(I6+I)=WS(I6+I)+WS(I5+JJ+I)*WSI8(J)
  490    CONTINUE
  500 CONTINUE
C
C     IF ALPHA IS ZERO THEN TRANSFORMATION IS SIMPLIFIED
C
      IF(ALPHA.EQ.0.) GOTO 700
C
C     CALCULATE Q*A*S*C  WHERE C IS ARRAY OF COEFFICIENTS OF
C     LEGENDRE EXPANSION IN POLYNOMIAL FORM, A IS BINOMIAL MATRIX,
C     S IS DIAGONAL MATRIX (S(I,I)=(BETA/ALPHA)**(I-1)),
C     AND Q IS DIAGONAL MATRIX (Q(I,I)=ALPHA**(I-1))
C
      BB=1./BA
      DO 530 J=1,M
         BB=BB*BA
         PARAM(J)=0.
         DO 520 I=J,M
         II=I*(I-1)/2
            PARAM(J)=PARAM(J)+WS(I)*WS(I4+II+J)*BB*WS(I6+I)
  520    CONTINUE
  530 CONTINUE
      GOTO 750
C
C     CALCULATE B*C WHERE C IS ARRAY OF COEFFICIENTS OF LEGENDRE
C     EXPANSION AND B IS DIAGONAL MATRIX(B(I,I)=BETA**I)
C
  700 BB=1.
      PARAM(1)=WS(I6+1)
      DO 710 J=2,M
         BB=BB*BETA
         PARAM(J)=BB*WS(I6+J)
  710 CONTINUE
 750  IF(MA(1).NE.0) GOTO 800
C
C    PRINT PARAMETERS VALUES ONLY IN THE CASE OF MULTIPLE FITS
C
      PRINT 1001,M-1,(PARAM(I),I=1,M)
*IF -DEF,AELIB5
 1001 FORMAT(1H0,*PARAMETERS FOR FIT OF ORDER *,I3,* ARE*,/,
     16(3X,E15.6))
*ENDIF
*IF DEF,AELIB5
 1001 FORMAT(1H0,'PARAMETERS FOR FIT OF ORDER ',I3,' ARE',/,
     16(3X,E15.6))
*ENDIF
  800 CONTINUE
C
C     SET ERROR MATRIX TO INDEFINITE BEFORE EXITING BECAUSE IT IS NOT BE
C     CALCULATED PROPERLY
C     USAGE OF LSQQ IS RECOMMENDED UNTIL POLFIT IS FIXED
C
C
 900  DO 910 I=1,MAX1
         DO 910 J=1,MAX1
*IF -DEF,AELIB5
             E(I,J)=17770000000000000000B
*ENDIF
*IF DEF,AELIB5
             E(I,J)=O"17770000000000000000"
*ENDIF
 910  CONTINUE
      IF(MM.NE.N)GO TO 960
      DO 950 I=1,N
           W(I)=0.0
  950 CONTINUE
  960 RETURN
      END
