*DECK POLREG
      SUBROUTINE POLREG(X,NRX,Y,W,YFIT,M,NOPTS,B,KTRM,NRK,IH,T,IER,WS)
C
C
C  PURPOSE:
C
C     MULTIPLE LINEAR REGRESSION TO AN ARBITRARY POLYNOMIAL IN ONE OR
C     MORE INDEPENDENT VARIABLES.
C
C  ENTRY:
C     CALL POLREG(X,NRX,Y,W,YFIT,M,NOPTS,B,KTRM,NRK,IH,T,IER,WS)
C
C     X.......REAL MATRIX OF DATA POINTS SUCH THAT X(I,J) IS THE I-TH
C                  DATA VALUE OF THE J-TH INDEPENDENT VARIABLE.
C     NRX.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR X IN THE CALLING
C                  PROGRAM.  NRX \ NOPTS.
C     Y.......REAL ARRAY OF DATA VALUES OF THE DEPENDENT VARIABLE.
C     W.......REAL ARRAY OF WEIGHTS FOR THE DEPENDENT VARIABLE (USE
C                  W(I) = 1 / VARIANCE(Y(I)), IF POSSIBLE).  W(I) = 1
C                  FOR ALL I RESULTS IN AN UNWEIGHTED FIT.
C     YFIT....REAL ARRAY OF FITTED VALUES OF THE DEPENDENT VARIABLE.
C     M.......INTEGER, NUMBER OF INDEPENDENT VARIABLES.
C     NOPTS...INTEGER, NUMBER OF DATA POINTS.
C     B.......REAL ARRAY OF COEFFICIENTS OF THE FITTED POLYNOMIAL.
C     KTRM....INTEGER MATRIX DEFINING THE NON-CONSTANT TERMS IN ORDER
C                  OF THE POLYNOMIAL TO BE FITTED.  KTRM(I,J) IS THE
C                  POWER OF THE I-TH INDEPENDENT VARIABLE IN THE J-TH
C                  NON-CONSTANT TERM OF THE REGRESSION POLYNOMIAL.
C     NRK.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR KTRM IN THE
C                  CALLING PROGRAM.  NRK \ M.
C     IH......INTEGER ARRAY RETURNED FROM POLREG AS A VECTOR OF PLUS AND
C                  MINUS ONES.  IH(I)=-1  IMPLIES THAT THE I-TH TERM IS
C                  NOT IN THE FITTED POLYNOMIAL.  IH(I)=1  IMPLIES THAT
C                  THE I-TH TERM IS IN THE POLYNOMIAL.
C     T.......INTEGER, NUMBER OF TERMS IN THE POLYNOMIAL TO BE FITTED.
C     IER.....INTEGER, ERROR PARAMETER RETURNED FROM RLSTEP THROUGH
C                  POLREG. ON A NORMAL EXIT FROM POLREG, IER=0, 39 OR
C                  132. OTHERWISE IER=129,130,133 OR 134
C     WS......REAL WORKING STORAGE ARRAY.
C             WS(1) TO WS((T*(T+1))/2)  CONTAINS AFTER THE CALL TO SSCP
C                  THE WEIGHTED SUMS OF SQUARES AND CROSS PRODUCTS
C                  MATRIX STORED IN SYMMETRIC STORAGE MODE.  AFTER THE
C                  CALL TO RLSTEP THIS SECTION OF THE WORKING STORAGE
C                  WILL CONTAIN ERROR INFORMATION SET UP BY RLSTEP.
C             WS(((T*(T+1))/2)+1) TO WS(((T*(T+3))/2)-1)  CONTAINS AFTER
C                  THE CALL TO SSCP THE MEANS OF THE NON-CONSTANT TERMS
C                  IN THE POLYNOMIAL.
C             WS((T*(T+3))/2)  CONTAINS THE MEAN OF THE DEPENDENT
C                  VARIABLE AFTER THE CALL TO SSCP.
C             WS(((T*(T+3))/2)+1) TO WS(((T*(T+3))/2)+N*TM1)  CONTAINS
C                  AFTER THE CALL TO SSCP THE VALUES OF THE NON-
C                  CONSTANT TERMS OF THE POLYNOMIAL. THE VALUE TERM
C                  MATRIX IS STORED BY COLUMNS.
C             WS(((T*(T+3))/2)+N*TM1+1) TO WS((T*(2*N+T+3))/2)  CONTAINS
C                  THE DEPENDENT VALUES OF Y AFTER THE CALL TO SSCP.
C             WS((T*(2*N+T+3))/2+1) CONTAINS THE SUM OF WEIGHTS AS
C                  CALCULATED BY SSCP
C             WS((T*(2*N+T+3))/2+2) TO  WS((T*(2*N+T+3))/2+(2+M))
C                  CONTAINS ZEROES SINCE FORCING OF VARIABLES INTO
C                  THE MODEL IS NOT DESIRED.
C
C
C NORMAL EXIT
C     CONTROL IS RETURNED TO THE CALLING PROGRAM WITH THE FITTED
C     VALUES IN YFIT, THE FITTED POLYNOMIAL DEFINED IN KTRM,IH,AND B
C     AND THE ERROR INFORMATION FROM RLSTEP IN WS. IER WILL BE ONE OF
C     0(NO ERRORS) OR EITHER 39 OR 132(NON-FATAL ERRORS IN RLSTEP)
C
C ERROR EXIT
C     IF ERRORS FATAL TO THE EXECUTION OF POLREG ARE DETECTED IN POLREG,
C     THEN CONTROL IS RETURNED TO THE CALLING ROUTINE WITH THE ELEMENTS
C     OF ARRAYS YFIT,B,AND WS USED BY POLREG SET TO INDEFINITE AND THE
C     ELEMENTS OF THE IH ARRAY USED BY POLREG SET TO 0. IER WILL BE
C     129,130,133, OR 134(FATAL ERRORS FROM RLSTEP)
C
C
C  AUTHORS:     D. E. SMITH AND L. E. EVANS         JUNE 1975
C
C
C  REVISION HISTORY:   -AUGUST 21, 1980.
C                      -IMSL ROUTINE RLSTEP HAD IT"S NAME CHANGED TO
C                       RLSTP IN IMSL EDITION 7.  A NUMBER OF CHANGES
C                       WERE MADE BECAUSE THE CALL TO RLSTEP IN POLREG
C                       WAS CHANGED TO CALL RLSTP.
C                      -REVISION BY M.T. BOULANGER.
C
C  REVISION HISTORY    -FEBRUARY 25,1982.
C                      -WARNING MESSAGE SPECIFYING SIZE OF WORKING
C                       STORAGE ARRAY IS TURNED OFF.
C                      -REVISION BY M.T. BOULANGER.
C
C
      LOGICAL ALERPR
      INTEGER  KTRM(NRK, 1), IH(1)
      REAL  X(NRX, 1), Y(1), B(1), YFIT(1), WS(1)
      REAL  W(1)
      INTEGER  T, TM1
*IF -DEF,AELIB5
      DATA IND/1777000000000000000B/
*ENDIF
*IF DEF,AELIB5
      DATA IND/O"1777000000000000000"/
*ENDIF
C
C   IROUT IS AN INTEGER VARIABLE DEFINING THE ROUTINE NUMBER
C   FOR POLREG TO BE USED IN ERROR MESSAGE CONTROL
C
      IROUT = 11111
*CALL  STATF
C
C        INITIALIZATION
C
      TM1 = T - 1
C
      CALL  SSCP(X, NRX, Y, W, M, NOPTS, KTRM, NRK, T, WS)
C
C     CALCULATE THE DIMENSION OF THE WORKING STORAGE ARRAY.
C
C     NRWS = (T*(2*NOPTS+T+3))/2 + (M+1)
C     IER = 0
C     IF(.NOT.ALERPR(IROUT,IER)) GO TO 90
C          PRINT 1000
C1000      FORMAT(" ***WARNING MESSAGE")
C          PRINT 2000,NRWS
C2000      FORMAT(" ***WORKING STORAGE ARRAY SHOULD BE GREATER THAN OR E
C    1QUAL TO",I10)
C          CALL TRACEB(0)
C  90 CONTINUE
C
C        CALL LEAST SQUARES FIT
C
      IND1 = 1
      IND2=((T*(2*NOPTS+T+3))/2+1)
      DO 10 I=1,M
           WS(IND2+I)=0.0
   10 CONTINUE
      IOPT = 0
      IH(1) = 0
      ALFAI = ALFAO = 0.05
  55  CALL RLSTP(WS(IND1),TM1,NOPTS,ALFAI,ALFAO,WS(IND2+1),IH,B,
     -            IOPT, IER)
*IF -DEF,AELIB5
      IF((IER.GT.128).A.(IER.NE.130)) GO TO 900
*ENDIF
*IF DEF,AELIB5
      IF((IER.GT.128).AND.(IER.NE.130)) GO TO 900
*ENDIF
C
      CALL PCALC(YFIT,NOPTS,B,IH,T,WS)
      GOTO 999
C
C     SET OUTPUT PARAMETERS FOR FATAL ERROR EXIT
C
 900  IWS=T*(NOPTS+(T+3)/2)
      CALL PRESET(WS,IWS,IND)
      CALL PRESET(YFIT,NOPTS,IND)
      CALL PRESET(B,T,IND)
      CALL PRESET(IH,T,0)
C
C
C
 999  RETURN
      END
