*DECK PRFIT
      SUBROUTINE  PRFIT(IDEG,X,NRX,Y,W,YFIT,M,NOPTS,B,KTRM,NRK,IH,T,
     -                  IER,WS)
C
C
C  PURPOSE:
C
C     MULTIPLE LINEAR REGRESSION TO A POLYNOMIAL OF SPECIFIED DEGREE
C     IN ONE OR MORE INDEPENDENT VARIABLES.
C
C  ENTRY:
C     CALL PRFIT(IDEG,X,NRX,Y,W,YFIT,M,NOPTS,B,KTRM,NRK,IH,T,IER,WS)
C
C     IDEG....INTEGER, HIGHEST DEGREE OF THE POLYNOMIAL TO BE FITTED.
C     X.......REAL MATRIX OF DATA POINTS SUCH THAT X(I,J) IS THE I-TH
C                  DATA VALUE OF THE J-TH INDEPENDENT VARIABLE.
C     NRX.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR X IN THE CALLING
C                  PROGRAM.  NRX \ NOPTS.
C     Y.......REAL ARRAY OF DATA VALUES OF THE DEPENDENT VARIABLE.
C     W.......REAL ARRAY OF WEIGHTS FOR THE DEPENDENT VARIABLE (USE
C                  W(I) = 1 / VARIANCE(Y(I)), IF POSSIBLE).  W(I) = 1
C                  FOR ALL I RESULTS IN AN UNWEIGHTED FIT.
C     YFIT....REAL ARRAY OF FITTED VALUES OF THE DEPENDENT VARIABLE.
C     M.......INTEGER, NUMBER OF INDEPENDENT VARIABLES.
C     NOPTS...INTEGER, NUMBER OF DATA POINTS.
C     B.......REAL ARRAY OF COEFFICIENTS OF THE FITTED POLYNOMIAL.
C     KTRM....INTEGER MATRIX RETURNED BY PRFIT DEFINING THE NON-CONSTANT
C                  TERMS IN ORDER OF THE POLYNOMIAL BEING FITTED.
C                  KTRM(I,J) IS THE POWER OF THE I-TH INDEPENDENT
C                  VARIABLE IN THE J-TH NON-CONSTANT TERM OF THE
C                  REGRESSION POLYNOMIAL.
C     NRK.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR KTRM IN THE
C                  CALLING PROGRAM.  NRK \ M.
C     IH......INTEGER ARRAY RETURNED FROM PRFIT AS A VECTOR OF PLUS AND
C                  MINUS ONES.  IH(I)=-1  IMPLIES THAT THE I-TH TERM IS
C                  NOT IN THE FITTED POLYNOMIAL.  IH(I)=1  IMPLIES THAT
C                  THE I-TH TERM IS IN THE POLYNOMIAL.
C     T.......INTEGER, NUMBER OF TERMS IN THE POLYNOMIAL TO BE FITTED.
C     IER.....INTEGER, ERROR PARAMETER RETURNED FROM RLSTEP THROUGH
C                  PRFIT. ON A NORMAL EXIT FROM PRFIT, IER=0,39 OR 132.
C                  OTHERWISE IER=129,130,133,134.
C     WS......REAL WORKING STORAGE ARRAY.
C             WS(1) TO WS((T*(T+1))/2)  CONTAINS AFTER THE CALL TO
C                  POLREG ERROR INFORMATION SET UP BY RLSTEP.
C             WS(((T*(T+1))/2)+1) TO WS(((T*(T+3))/2)-1)  CONTAINS AFTER
C                  THE CALL TO POLREG THE MEANS OF THE NON-CONSTANT
C                  TERMS IN THE POLYNOMIAL.
C             WS((T*(T+3))/2)  CONTAINS THE MEAN OF THE DEPENDENT
C                  VARIABLE AFTER THE CALL TO POLREG.
C             WS(((T*(T+3))/2)+1) TO WS(((T*(T+3))/2)+N*(T-1))  CONTAINS
C                  AFTER THE CALL TO POLREG THE VALUES OF THE NON-
C                  CONSTANT TERMS OF THE POLYNOMIAL. THE VALUE TERM
C                  MATRIX IS STORED BY COLUMNS.
C             WS(((T*(T+3))/2)+N*TM1+1) TO WS((T*(2*N+T+3))/2)  CONTAINS
C                  THE DEPENDENT VALUES OF Y AFTER THE CALL TO POLREG.
C             WS((T*(2*N+T+3))/2+1) CONTAINS THE SUM OF WEIGHTS AS
C                  CALCULATED BY SSCP
C             WS((T*(2*N+T+3))/2+2) TO  WS((T*(2*N+T+3))/2+(2+M))
C                  CONTAINS ZEROES SINCE FORCING OF VARIABLES INTO
C                  THE MODEL IS NOT DESIRED.
C
C
C NORMAL EXIT
C     CONTROL IS RETURNED TO THE CALLING PROGRAM WITH THE FITTED
C     VALUES IN YFIT, THE FITTED POLYNOMIAL DEFINED IN KTRM,IH,AND B
C     AND THE ERROR INFORMATION FROM RLSTEP IN WS. IER WILL BE ONE OF
C     0(NO ERRORS) OR EITHER 39 OR 132(NON-FATAL ERRORS IN RLSTEP)
C
C ERROR EXIT
C     ERROR CONDITIONS FOR PRFIT ARE DETECTED IN POLREG. INDEFINITE
C     VALUES ARE SET BY POLREG AND PASSED BACK THROUGH PRFIT. SEE
C     POLREG FOR DETAILS OF ERROR PROCESSING
C
C  AUTHORS:     D. E. SMITH AND L. E. EVANS         JUNE 1975
C
C
      INTEGER  KTRM(NRK, 1), IH(1), T
      REAL  X(NRX, 1), Y(1), B(1), YFIT(1), WS(1)
      REAL  W(1)
C
*CALL  STATF
      CALL COMKTM(IDEG,KTRM,NRK,M)
      CALL POLREG(X,NRX,Y,W,YFIT,M,NOPTS,B,KTRM,NRK,IH,T,IER,WS)
      RETURN
      END
