*DECK PSTATF
      SUBROUTINE PSTATF(ICON,IH,T,NOPTS,E,NRE,SDY,SUMSQ,WS)
C
C
C  PURPOSE:
C
C     TO COMPUTE ERROR STATISTICS FOR THE FITTED POLYNOMIAL SUCH AS
C     THE COVARIANCE AND CORRELATION OF PAIRS OF COEFFICIENTS, AND THE
C     STANDARD DEVIATION OF THE COEFFICIENTS AND FITTED FUNCTION VALUES.
C
C  ENTRY:
C     CALL PSTAT(ICON,IH,T,NOPTS,E,NRE,SUMSQ,WS)
C
C     THIS ROUTINE CAN BE USED AFTER A CALL TO ONE OF THE FOLLOWING
C     ROUTINES:    PRFIT, POLREG, RLSTEP(PROVIDED A PRIOR CALL TO SSCP
C     WAS MADE TO SET UP THE WORKING STORAGE ARRAY), AND PCALC.  IN
C     ADDITION IT IS POSSIBLE TO CALL THE PCALC ROUTINE AFTER A CALL TO
C     PSTAT IF THE USER DOES NOT REQUIRE STATISTICS FOR THE CONSTANT
C     TERM.
C     STATISTICS FOR THE FITTED FUNCTION VALUES ARE DEFINED ONLY IF
C     ICON IS 1, THAT IS THE CONSTANT TERM IS INCLUDED.
C
C     ICON....INTEGER SET TO 0 IF THE CONSTANT TERM IS NOT INCLUDED IN
C                  THE FIT OR TO 1 IF THE CONSTANT IS INCLUDED.  ICON
C                  WOULD BE SET TO 0 IF PSTAT WAS CALLED DIRECTLY AFTER
C                  A CALL TO THE RLSTEP ROUTINE.
C     IH(NRI).INTEGER ARRAY SET UP BY A PRIOR CALL TO ONE OF THE FOUR
C                  ROUTINES MENTIONED ABOVE AS A VECTOR OF PLUS AND
C                  MINUS ONES.  IH(I) = -1  IMPLIES THE I-TH TERM IS
C                  NOT IN THE FITTED POLYNOMIAL.  IH(I) = 1  IMPLIES
C                  THAT THE I-TH TERM IS IN THE POLYNOMIAL.  NRI \ T.
C     T.......INTEGER, SET TO THE NUMBER OF TERMS IN THE FITTED
C                  POLYNOMIAL (INCLUDING THE CONSTANT TERM).
C     N.......INTEGER, NUMBER OF DATA POINTS.
C     E(NRE,NCE)
C             REAL ARRAY RETURNED BY PSTAT CONTAINING THE ERROR MATRIX.
C                  NRE, NCE \ T.
C     NRE.....INTEGER NUMBER OF ROWS DIMENSIONED FOR E IN THE CALLING
C                  PROGRAM.
C     SDY(NRS) REAL ARRAY IN WHICH PSTATF WILL PUT STANDARD DEVIATION
C                  ESTIMATES FOR THE FITTED FUNCTION VALUES. IF ICON IS
C                  0 SDY IS SET TO INDEFINITES.
C                  NRS\N.
C     SUMSQ...REAL RETURNED BY PSTAT AS THE (WEIGHTED) SUM OF SQUARES
C                  OF RESIDUALS.
C     WS(NRW).REAL WORKING STORAGE ARRAY SET UP BY A PRIOR CALL TO
C                  PRFIT, POLREG, SSCP OR RLSTEP.
C                  THE FIRST (T*(T+1))/2 WORDS CONTAIN THE ERROR
C                  INFORMATION SET UP BY RLSTEP AND WILL BE ALTERED BY
C                  THE PSTAT ROUTINE.  ALSO REQUIRED AS SET UP BY THE
C                  PRFIT, POLREG OR SSCP ROUTINES ARE THE MEANS OF THE
C                  NON-CONSTANT TERMS OF THE POLYNOMIAL STORED FROM
C                  INDEX (T*(T+1))/2 + 1 TO (T*(T+1))/2 + T - 1 AND THE
C                  MEAN OF THE DEPENDENT VARIABLE STORED AT INDEX
C                  (T*(T+1))/2 + T.
C             WS((T*(2*N+T+3))/2+1) CONTAINS THE SUM OF WEIGHTS AS
C                  CALCULATED BY SSCP AND THIS IS ALSO REQUIRED BY PSTAT
C
C  EXIT:
C
C     CONTROL IS RETURNED TO THE CALLING PROGRAM WITH THE ERROR
C     STATISTICS STORED IN THE ERROR MATRIX, E, AND THE (WEIGHTED) SUM
C     OF SQUARES OF RESIDUALS IN SUMSQ AND THE STANDARD DEVIATION
C     ESTIMATES FOR FITTED VALUES IN SDY.
C
C  AUTHORS:     D. E. SMITH AND L. E. EVANS      AUGUST  1975
C  REVISED BY L E EVANS   TO ADD FITTED VALUE STATISTISC     MARCH 1977.
C
C   REVISED OCTOBER 26, 1981  BY MARIETTE BOULANGER.
C   REVISION TO FIX THE CASE WHEN A STRAIGHT LINE IS BEING FITTED SO
C   THE WORD IN OR AFTER THE E ARRAY DOES NOT GET OVERWRITTEN.
C
C
      REAL  E(NRE,1), WS(1)
      REAL SDY(1)
      INTEGER  IH(1), T, TM1
      LOGICAL  CONST
*CALL  STATF
C
C        INITIALIZATION
C
      TM1 = T - 1
      IF(ICON .EQ. 0)  CONST = .FALSE.
      IF(ICON .EQ. 1)  CONST = .TRUE.
      IF(.NOT. CONST)  ISTART = 1
      IF(CONST)  ISTART = 2
C
C     RETRIEVE SUM OF SQUARES OF RESIDUALS
C
      SUMSQ = WS((T*(T+1)) / 2)
C
C     RETRIEVE SUBMATRIX APPROPRIATE FOR FIT
C
      CALL RLSUBM(WS,TM1,IH(ISTART),WS,NSUBM)
C
C     CALL USWTSM(-1,0,0,T,WS)
C
C     CALCULATE ESTIMATE OF RESIDUAL VARIANCE OF FIT
C
      RVAR=SUMSQ/(NOPTS-NSUBM-1)
C
C
C     PREVIOUS RESIDUAL VARIANCE EXTIMATE
C     IND1=(T*(2*NOPTS+T+3))/2+1
C     RVAR=SUMSQ*NOPTS/(WS(IND1)*(NOPTS-NSUBM-1))
C
C     CALL USWTSM(-1,0,0,T,WS)
C
C        COMPUTE THE VARIANCE-COVARIANCE MATRIX OF THE FIT
C                  (STORED IN SYMMETRIC STORAGE MODE)
C
      INDEX = (NSUBM*(NSUBM+1))/2
      DO 10  I = 1, INDEX
        WS(I) = WS(I) * RVAR
10    CONTINUE
C
C     CALL USWTSM( 0,0,0,NSUBM,WS)
C
C     ***********  COMPUTE  THE  ERROR  MATRIX  ************************
C
C        CALCULATE STANDARD DEVIATION AND STORE ALONG THE DIAGONAL.
C        IN THE UPPER TRIANGULAR PART OF THE MATRIX STORE THE COVARIANCE
C        OF PARAMETERS, A(I) AND A(J).  IF EITHER A(I) AND/OR A(J) HAVE
C        BEEN DELETED FROM THE FIT, SET E(I,J) = 0.0.
C
      K = 1
      IF(.NOT. CONST)  NT = TM1
      IF(CONST)  NT = T
      DO 25  J = ISTART, NT
        DO 20  I = ISTART, J
          IF((IH(I) .EQ. -1) .OR. (IH(J) .EQ. -1)) GO TO 15
            IF(I .EQ. J)  E(I,J) = SQRT(WS(K))
            IF(I .NE. J)  E(I,J) = WS(K)
            K = K + 1
            GO TO 20
15        E(I,J) = 0.0
          E(J,I) = 0.0
20      CONTINUE
25    CONTINUE
C
C     CALL USWTFM(-1,0,0,NT,NT,NRE,E)
C
C        COMPUTE THE CORRELATION COEFFICIENTS OF PAIRS OF PARAMETERS
C        AND STORE IN THE LOWER TRIANGULAR PART OF THE MATRIX.
C
      IF(NT .EQ. 2) GO TO 45
      ISTP1 = ISTART + 1
      DO 40  I = ISTP1, NT
        IM1 = I - 1
        DO 35  J = ISTART, IM1
          IF((IH(I) .EQ. 1) .AND. (IH(J) .EQ. 1))
     -                       E(I,J) = E(J,I) / (E(I,I) * E(J,J))
35      CONTINUE
40    CONTINUE
C
C     CALL USWTFM(-1,0,0,NT,NT,NRE,E)
C
   45 CONTINUE
      IF(.NOT. CONST)  RETURN
C
C     ***********  COMPUTE STATISTICS FOR THE CONSTANT TERM  ***********
C
      I2 = (T*(T+1)) / 2
C
C     PRINT 42
C42   FORMAT(1H1)
C     IND2 = I2 + 1
C     I3 = I2 + T
C     PRINT *, (WS(I), I = IND2,I3)
C
C        CALCULATION OF THE STANDARD DEVIATION FOR THE CONSTANT TERM.
C        - FIRST PART OF THE CALCULATION _  COMPUTE THE VARIANCE OF THE
C                      MEAN OF Y (DEPENDENT VARIABLE).
C
      E(1,1)=RVAR/NOPTS
C
C     PRINT *, E(1,1)
C
C        - NEXT _  COMPUTE THE SUM(VAR(A(I)) * (X(I)BAR ** 2))
C                                  _ I = 1, ..., TM1 .
C
      IND2 = I2 + 1
      DO 50  I = 2, T
        E(1,1) = E(1,1) + (E(I,I) * WS(IND2)) ** 2
        IND2 = IND2 + 1
50    CONTINUE
C
C     PRINT *, E(1,1)
C
C        - AND FINALLY _  COMPUTE   2 * SUM(X(I)BAR * X(J)BAR
C                * COV(A(I),A(J))) _ I, J = 1, ..., TM1 AND  I < J.
C
      TEMP = 0.0
      IF(T.EQ.2) GOTO 65
      IND2J = I2 + 1
      DO 60  J = 3, T
        IND2I = I2 + 1
        IND2J = IND2J + 1
        JM1 = J - 1
        DO 55  I = 2, JM1
          TEMP = TEMP + WS(IND2I) * WS(IND2J) * E(I,J)
          IND2I = IND2I + 1
55      CONTINUE
60    CONTINUE
  65  CONTINUE
      E(1,1) = SQRT(E(1,1) + 2.0 * TEMP)
C
C     PRINT *, 2.0*TEMP, E(1,1)
C
C        COMPUTE THE COVARIANCE FOR THE CONSTANT TERM (THAT IS,
C        COV(A(0),A(1)) = - X(I)BAR * VAR(A(I)) - SUM(X(J)BAR *
C             COV(A(I),A(J)) _ J = 1, ..., T-1 AND  I " J.
C
      IND2I = I2
      DO 80  I = 2, T
        IND2I = IND2I + 1
        IF(IH(I) .EQ. -1)  GO TO 75
          E(1,I) = -WS(IND2I) * E(I,I) ** 2
C
          IND2J = I2
          TEMP = 0.0
          DO 70  J = 2, T
            IND2J = IND2J + 1
            IF((I .EQ. J) .OR. (IH(J) .EQ. -1))  GO TO 70
            IF(I .LT. J)  TEMP = TEMP + WS(IND2J) * E(I,J)
            IF(I .GT. J)  TEMP = TEMP + WS(IND2J) * E(J,I)
70        CONTINUE
          E(1,I) = E(1,I) - TEMP
          GO TO 80
C
75      E(1,I) = 0.0
        E(I,1) = 0.0
80    CONTINUE
C
C     PRINT *, (E(1,I), I = 1, T)
C     PRINT *, (E(I,1), I = 1, T)
C
C        COMPUTE THE CORRELATION COEFFICIENTS FOR THE CONSTANT TERM.
C
      DO 90  I = 2, T
        IF(IH(I) .EQ. 1)  E(I,1) = E(1,I) / (E(1,1) * E(I,I))
90    CONTINUE
C
C     CALL USWTFM( 0,0,0,NT,NT,NRE,E)
C
C
C     COMPUTE STANDARD DEVIATION ESTIMATES FOR FITTED FUNCTION VALUES,
C      RECALLING THAT VALUE TERM MATRIX CONTAINS VALUES OF T-1
C     NON-CONSTANT TERMS.
C     INDICES J AND K COUNT THE NUMBER OF TERMS INCLUDING THE CONSTANT
C     TERM, I COUNTS THE NUMBER OF DATA POINTS.
C
      IBASE=(T*(T+3))/2
      DO 200 I=1,NOPTS
      IF(ICON.EQ.1) GOTO 95
*IF -DEF,AELIB5
         SDY(I)=60000000000000000000B
*ENDIF
*IF DEF,AELIB5
         SDY(I)=O"60000000000000000000"
*ENDIF
         GOTO 200
 95   CONTINUE
         SDY(I)=E(1,1)**2
C
C     RETRIEVE COVARIANCE CONTRIBUTIONS WITH THE CONSTANT TERM AND
C     VARIANCE CONTRIBUTIONS FROM THE NON-CONSTANT TERMS.
C
         DO 150 J=2,T
            IND1=IBASE + (J-2)*NOPTS + I
            SDY(I)=SDY(I) +
     ,             2*WS(IND1)*E(1,J) +
     ,             (WS(IND1)*E(J,J))**2
            IF(J.EQ.2) GOTO 150
C
C     RETRIEVE VARIANCE CONTRIBUTIONS AMOUNG NON-CONSTANT TERMS.
C
            JM1=J-1
            DO 100 K=2,JM1
               IND2=IBASE + (K-2)*NOPTS + I
               SDY(I)=SDY(I) +
     ,                2*WS(IND1)*WS(IND2)*E(K,J)
 100        CONTINUE
 150     CONTINUE
         SDY(I)=SQRT(SDY(I))
 200  CONTINUE
      RETURN
      END
