*DECK PTREAD
      SUBROUTINE PTREAD (MODE,P1,P2,P3,P4,P5,P6,P7)
C
C    PTREAD READS THE PAPER TAPE IN A FORM WHICH IS PREPARED FOR IT BY
C      THE 3300 COMPUTER, CRACKS THE FORMAT, AND OUTPUTS THE RESULT IN
C      THE USER'S BUFFER.    ERRORS IN THE TAPE ARE EXAMINED AND OUTPUT
C      BY SEPARATE CALLS OF PTREAD.   PROVISION IS MADE FOR DEALING WITH
C      BINARY TAPES AND FOR COPYING THE TAPE FILE DIRECTLY INTO THE
C      USER'S ARRAY.
C
C    SEE AELIB 2-2-30 AND MCTD-6 FOR DETAILS.
C
C
C REVISION HISTORY : REVISED NOV 82 BY J.F. STELJES
C                    REVISION HALTS READ OF PAPER TAPE ON FIFTH ATTEMPT
C                    TO READ PAST END OF FILE AND REPORTS  SITUATION
C                    TO USER.
C
      IMPLICIT INTEGER (A-Z)
      INTEGER KODARR(8),DMPARR(10)
C
*IF -DEF,AELIB5
      DATA KODARR/5501020304050607B,1011121314151617B,
     1            2021222324252627B,3031326175627665B,
     2            5566646053636770B,5152474556465750B,
     3            3334353637404142B,4344637772547371B/
*ENDIF
*IF DEF,AELIB5
      DATA KODARR/O"5501020304050607",O"1011121314151617",
     1              O"2021222324252627",O"3031326175627665",
     2                O"5566646053636770",O"5152474556465750",
     3                  O"3334353637404142",O"4344637772547371"/
*ENDIF
      DATA OCTN/0/,INP/0/,LASTP/0/
      DATA ISC/0/,EC/0/,FWFORM/0/,FWA/0/
C
C    MODE 0 TOGGLES OCTN
C
      IF (MODE.NE.0) GO TO 100
      OCTN=XOR(OCTN,1)
      RETURN
C
C    MODE 1 DEFINES PARAMETER VALUES
C      IT MAY BE USED AT ANY TIME TO REDEFINE PARAMETERS WITHOUT
C      ALTERING ANY OTHER FACTORS
C      NBITS IS THE NUMBER OF LEAST SIGNIFICANT BITS OF THE TAPE TO BE
C      DECODED BY KODARR
C      NBLANKS DEFINES THE NUMBER OF BLANKS FRAMES WHICH IS CONSIDERED
C      MARK THE BOUNDARY OF A WORD
C      NBLANKS IS DENOTED DURING THE PROGRAM BY FRAME=-1
C
  100 IF (MODE.NE.1) GO TO 200
      CALL PARAMS (P1,P2,P3,P4,P5,P6,P7,D,D,D,D,D,D,KK,NPAR)
      IF (NPAR.LT.4) GO TO 1001
      ISC=SC=P1
      IF (ISC.EQ.0) ISC=SC=-1
      EC=P2
      IF (EC.EQ.0) EC=-1
      ITYPE=P3
      IPAR=P4
      NBITS=6
      NBLANKS=10
C
      IF (NPAR.EQ.4) GO TO 150
      NBITS=P5
      IF (NBITS.EQ.0) NBITS=6
      IF (NPAR.EQ.5) GO TO 150
      NBLANKS=P6
      IF (NBLANKS.EQ.0) NBLANKS=10
      IF (NPAR.EQ.6) GO TO 150
      RETURN
C
C    IF THE USER DOES NOT PROVIDE AN EXTERNAL DECODING ARRAY (KODARR),
C      THE PROGRAM WILL USE THE INTERNAL ARRAY KODARR TRANSLATING ASCII
C      CODES TO CDC DISPLAY CODE (NBITS=6)
C
  150 KK=LOCF (KODARR(1))
      RETURN
C
C    MODE 2 DEFINES THE FORMAT TO USED IN ENCODING THE TAPE
C      IT MAY BE CALLED AS OFTEN AS DESIRED TO CHANGE THE FORMAT
C      THERE IS NO DEFAULT FORMAT
C
  200 IF (MODE.NE.2) GO TO 300
      CALL PARAMS (P1,P2,P3,P4,P5,P6,P7,FWFORM,D,D,D,D,D,D,NPAR)
      IF (NPAR.NE.1) GO TO 1001
      RETURN
C
C    MODE 3 PERFORMS THE ACTUAL READING AND DECODING OF THE TAPE.
C      AFTER THE FIRST CALL, IF SC=-1, SC IS SET TO BE EQUAL TO EC
C
C    ON FIRST READ OR IF THE TAPE IS REWOUND, SC IS SET EQUAL TO ISC
C      (THE ORIGINAL VALUE)
C
C    IF NR=1, READING OF THE PRESENT TAPE WILL STOP AND THE PROGRAM WILL
C      SEARCH FOR THE BEGINNING OF A NEW TAPE
C      IARR IS THE USER'S ARRAY
C      NUM1 IS THE MAXIMUM NUMBER OF WORDS THAT CAN BE READ IN ONE CALL
C      NUM2 IS THE ACTUAL NUMBER OF WORDS READ BEFORE THE PROGRAM
C      ENCOUNTERS AN ENDING CODE (EC) OR NUM2=NUM1.
C      IERR IS A WORD CONTAINING BITS DENOTING VARIOUS ERROR CONDITIONS
C      OR INFORMATION REGARDING THE POSITION ON THE TAPE(S)
C
  300 IF (MODE.NE.3) GO TO 400
      IF (ISC.EQ.0.OR.FWFORM.EQ.0) GO TO 1002
      CALL PARAMS (P1,P2,P3,P4,P5,P6,P7,D,IARR,D,D,D,D,D,NPAR)
      IF (NPAR.NE.5) GO TO 1001
      NR=P1
      NUM1=P3
      NUM2=IERR=0
      CALL READ (FWA,INP,LASTP,FP,IERR,OCTN)
*IF -DEF,AELIB5
      IF (IERR.GE.20B) GO TO 350
*ENDIF
*IF DEF,AELIB5
      IF (IERR.GE.O"20") GO TO 350
*ENDIF
      IF (FP.NE.1) GO TO 310
      SC=ISC
      FRAME=NR=0
  310 CALL COMP (SC,EC,ITYPE,IPAR,NBITS,NBLANKS,KK,FWFORM,FWA,INP,
     + LASTP,FRAME,NR,IARR,NUM1,NUM2,IERR)
      IF (SC.EQ.-1) SC=EC
      P4=NUM2
  350 P5=IERR
      RETURN
C
C
C    MODE 4,1 DUMPS THE ERROR ARRAYS
C
C    MODE 4,2 PRODUCES A DECODED DUMP FROM 100 FRAMES BEFORE THE PRESENT
C      POSITION IN THE TAPE TO 100 FRAMES PAST THE PRESENT POSITION
C      FRAMES OUTSIDE THE ENDS OF THE TAPE ARE BLANK
C      NON-DATA FRAMES ARE DENOTED BY #
C      LASTP IS SET LARGE TO PREVENT A CALL TO READ.
C
C    MODE 4 CALLS WITH PARAMETERS OF 3 AND 4 SIMULATE MODE 0 CALLS BUT
C      DO NOT TOGGLE OCTN.
C
  400 IF (MODE.NE.4) GO TO 500
      IF (P1.EQ.1) GO TO 410
      IF (P1.EQ.2) GO TO 420
      IF (P1.EQ.3) GO TO 430
      IF (P1.EQ.4) GO TO 440
      GO TO 1003
C
  410 CALL ERROR (-ITYPE,0,0,0,0,0,0,0)
      RETURN
C
  420 IF (FWA.EQ.0) RETURN
      DINP=INP - 64*20
      IF (DINP.LT.0) DINP=DINP + 64*128
*IF -DEF,AELIB5
      LASTP1=LASTP.AND.77777B
*ENDIF
*IF DEF,AELIB5
      LASTP1=LASTP.AND.O"77777"
*ENDIF
      PRINT 401
      DO 480 I=1,2
      DO 470 J=1,10
      DMPARR(J)=0
      DO 460 K=1,10
*IF -DEF,AELIB5
      A=1R
*ENDIF
*IF DEF,AELIB5
      A=R" "
*ENDIF
*IF -DEF,AELIB5
      IF (LASTP.GT.100000B.AND.DINP.EQ.LASTP1) GO TO 450
*ENDIF
*IF DEF,AELIB5
      IF (LASTP.GT.O"100000".AND.DINP.EQ.LASTP1) GO TO 450
*ENDIF
*IF -DEF,AELIB5
      CALL DECODE (FWA,DINP,64*200,FR,A,IPAR,NBITS,NBLANKS,KK,EC,D),
     + RETURNS (490)
*ENDIF
*IF DEF,AELIB5
      CALL DECODE (FWA,DINP,64*200,FR,A,IPAR,NBITS,NBLANKS,KK,EC,D,
     +*490)
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.0) A=1R#
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.0) A=R"#"
*ENDIF
*IF -DEF,AELIB5
      IF (FR.EQ.0) A=1R
*ENDIF
*IF DEF,AELIB5
      IF (FR.EQ.0) A=R" "
*ENDIF
  450 DMPARR(J)=SHIFT (DMPARR(J),6).OR.A
  460 CONTINUE
  470 CONTINUE
      PRINT 402, DMPARR
  480 CONTINUE
  490 RETURN
C
  430 OCTN=1
      RETURN
C
  440 OCTN=0
      RETURN
C
  401 FORMAT (6X,12HDECODED DUMP)
  402 FORMAT (6X,10A10)
C
C    MODE 5 READS FROM THE PAPER TAPE FILE INTO THE USER'S ARRAY.   THIS
C      CALL IS COMPATIBLE ONLY WITH MODE 0 AND MODE 6
C
  500 IF (MODE.NE.5) GO TO 600
      CALL PARAMS (P1,P2,P3,P4,P5,P6,P7,IARR,D,D,D,D,D,D,NPAR)
      IF (NPAR.NE.3) GO TO 1001
      IERR=0
      CALL READ (FWA,INP,LASTP,D,IERR,OCTN+2)
      PTR=K=0
      DO 510 I=1,64
      CALL EXTRACT (FWA,PTR,60,A,D)
      IF (A.EQ.0) GO TO 520
      K=K + 1
      CALL STORE (IARR,K,A,0.0,-1)
  510 CONTINUE
  520 P2=K
      P3=-1
      IF (K.LT.64) P3=0
*IF -DEF,AELIB5
      IF (IERR.GE.20B) P3=1
*ENDIF
*IF DEF,AELIB5
      IF (IERR.GE.O"20") P3=1
*ENDIF
      RETURN
C
C   MODE 6 CHANGES THE NAME OF THE FILE CONTAINING THE PAPER TAPE DATA
C      FROM "PTREAD" TO A NAME SUPPLIED BY THE USER
C
  600 IF (MODE.NE.6) GO TO 1000
      IF (FWA.NE.0) GO TO 1004
*IF -DEF,AELIB5
      CALL READ (P1,D,D,D,D,20B)
*ENDIF
*IF DEF,AELIB5
      CALL READ (P1,D,D,D,D,O"20")
*ENDIF
      RETURN
C
C    ERROR MESSAGE
C
 1000 PRINT 1, MODE
      GO TO 1100
 1001 PRINT 2, NPAR + 1
      GO TO 1100
 1002 PRINT 3
      GO TO 1100
 1003 PRINT 4, MODE
      GO TO 1100
 1004 PRINT 5
 1100 CALL TRACEB(1)
      RETURN
C
*IF -DEF,AELIB5
    1 FORMAT (" MODE NOT 0 TO 6, MODE=",I2)
*ENDIF
*IF DEF,AELIB5
    1 FORMAT (' MODE NOT 0 TO 6, MODE=',I2)
*ENDIF
*IF -DEF,AELIB5
    2 FORMAT (" ILLEGAL NUMBER OF PARAMETERS, NPAR = ",I2)
*ENDIF
*IF DEF,AELIB5
    2 FORMAT (' ILLEGAL NUMBER OF PARAMETERS, NPAR = ',I2)
*ENDIF
*IF -DEF,AELIB5
    3 FORMAT (" CALL OF MODE 3 WITHOUT PREVIOUS CALLS OF MODE 1 AND 2")
*ENDIF
*IF DEF,AELIB5
    3 FORMAT (' CALL OF MODE 3 WITHOUT PREVIOUS CALLS OF MODE 1 AND 2')
*ENDIF
*IF -DEF,AELIB5
    4 FORMAT (" ILLEGAL PARAMETER, MODE =",I2)
*ENDIF
*IF DEF,AELIB5
    4 FORMAT (' ILLEGAL PARAMETER, MODE =',I2)
*ENDIF
*IF -DEF,AELIB5
    5 FORMAT (" MODE 6 MUST BE CALLED BEFORE MODE 3")
*ENDIF
*IF DEF,AELIB5
    5 FORMAT (' MODE 6 MUST BE CALLED BEFORE MODE 3')
*ENDIF
C
      END
      SUBROUTINE READ (FWA,INP,LASTP,FP,IERR,N)
C
C    THE SUBROUTINE READS 64 WORDS FROM FILE "PTREAD" ALTERNATELY INTO
C      BUF(1) AND BUF(65)
C
C    ON THE FIRST READ, WHEN "PTREAD" HAS BEEN REWOUND OR WHEN A NEW
C      PRU IS READ, THE BUFFER IS CLEARED, THE DATA IS READ INTO BUF(1),
C      AND THE INPUT POINTER IS SET TO ZERO
C
C    LASTP IS SET TO THE MIDDLE OF THE BUFFER BEING INPUT (WORD 32)
C      UNLESS THE RECORD IS SHORT, WHEN LASTP IS SET TO THE END OF THE
C      LAST WORD READ.   THE REST OF THE BUFFER IS CLEARED AND THE SHORT
C      PRU FLAG IS SET
C
C    THE POINTER IS DEFINED AS
C                  100B*(WORD COUNT) + BIT COUNT
C        EACH COUNT STARTING AT ZERO.    THE SUBROUTINE EXTRACT DEALS
C      THE EXTRA 4 BIT COUNT AT THE END OF EACH 60-BIT WORD
C
C    WHEN A SHORT PRU IS READ, A FLAG IS SET IN LASTP (100000B)
C      ON THE NEXT READ, BIT 4 IS SET IN THE ERROR WORD
C
C    INPUT PARAMETERS ARE INP, LASTP, IERR, N.
C    OUTPUT PARAMETERS ARE FWA, INP, LASTP, FP, IERR.
C
C    READ IS CALLED WITH DIFFERENT VALUES OF N:
C      N=0 OR 1    INITIAL CALL FROM EACH MODE 3  -  CHECKS FOR INITIAL
C                    READ OR REWIND FROM EXTERNAL PROGRAM
C                    IT DEFINES OCTN FOR LATER USE
C      N=2 OR 3    CALL FROM MODE 5
C                    IT DEFINES OCTN FOR USE LATER IN THE CALL
C      N=10B       CALL FROM DECODE DOES NOT CHECK FOR REWIND
C                    IT USES THE VALUE OF OCTN DEFINED ABOVE
C      N=20B       CALL FROM MODE 6 TO CHANGE FILE NAME
C
C
      IMPLICIT INTEGER (A-Z)
      COMMON/AEERCM/IRTN,IFLAG
      INTEGER FIT(1),BUF(128),DMPARR(30)
      DATA IRTN,IFLAG/20230,1/
      DATA FIRST/0/,EOFCT/0/
*IF -DEF,AELIB5
      DATA NAME/6LPTREAD/
*ENDIF
*IF DEF,AELIB5
      DATA NAME/L"PTREAD"/
*ENDIF
C
*IF -DEF,AELIB5
      IF (N.EQ.10B) GO TO 200
*ENDIF
*IF DEF,AELIB5
      IF (N.EQ.O"10") GO TO 200
*ENDIF
*IF -DEF,AELIB5
      IF (N.EQ.20B) GO TO 500
*ENDIF
*IF DEF,AELIB5
      IF (N.EQ.O"20") GO TO 500
*ENDIF
      IF (FIRST.NE.0) GO TO 100
      IPTR=IGETFIT (NAME,IER,0) - LOCF (FIT(1)) + 1
      IF (IER.NE.0) GO TO 1000
*IF -DEF,AELIB5
      I=IFETCH (FIT(IPTR),2LOC)
*ENDIF
*IF DEF,AELIB5
      I=IFETCH (FIT(IPTR),L"OC")
*ENDIF
      IF (I.NE.0) GO TO 1001
*IF -DEF,AELIB5
      CALL STOREF (FIT(IPTR),2LRT,1LU)
*ENDIF
*IF DEF,AELIB5
      CALL STOREF (FIT(IPTR),L"RT",L"U")
*ENDIF
*IF -DEF,AELIB5
      CALL STOREF (FIT(IPTR),2LBT,1LC)
*ENDIF
*IF DEF,AELIB5
      CALL STOREF (FIT(IPTR),L"BT",L"C")
*ENDIF
*IF -DEF,AELIB5
      CALL STOREF (FIT(IPTR),3LMRL,1280)
*ENDIF
*IF DEF,AELIB5
      CALL STOREF (FIT(IPTR),L"MRL",1280)
*ENDIF
*IF -DEF,AELIB5
      CALL STOREF (FIT(IPTR),3LDFC,0)
*ENDIF
*IF DEF,AELIB5
      CALL STOREF (FIT(IPTR),L"DFC",0)
*ENDIF
*IF -DEF,AELIB5
      CALL OPENM (FIT(IPTR),5LINPUT)
*ENDIF
*IF DEF,AELIB5
      CALL OPENM (FIT(IPTR),L"INPUT")
*ENDIF
      FWA=LOCF (BUF(1))
  100 OCTN=N.AND.1
*IF -DEF,AELIB5
      FP=IFETCH (FIT(IPTR),2LFP)
*ENDIF
*IF DEF,AELIB5
      FP=IFETCH (FIT(IPTR),L"FP")
*ENDIF
      IF (FP.NE.1.AND.EOFCT.NE.0) GO TO 400
      IF (FP.NE.1.AND.FIRST.NE.0.AND.N.LT.2) RETURN
      EOFCT=0
  110 DO 120 I=1,128
      BUF(I)=0
  120 CONTINUE
      INP=LASTP=0
  200 I=1
*IF -DEF,AELIB5
      IF (LASTP.LT.100000B) GO TO 210
*ENDIF
*IF DEF,AELIB5
      IF (LASTP.LT.O"100000") GO TO 210
*ENDIF
*IF -DEF,AELIB5
      IERR=IERR.OR.10B
*ENDIF
*IF DEF,AELIB5
      IERR=IERR.OR.O"10"
*ENDIF
      GO TO 110
*IF -DEF,AELIB5
  210 LASTP=LASTP.AND.77777B
*ENDIF
*IF DEF,AELIB5
  210 LASTP=LASTP.AND.O"77777"
*ENDIF
      IF (LASTP.NE.0.AND.LASTP.LE.64*64) I=65
      CALL GET (FIT(IPTR),BUF(I),0,0,0,640)
*IF -DEF,AELIB5
      J=IFETCH (FIT(IPTR),2LFP)
*ENDIF
*IF DEF,AELIB5
      J=IFETCH (FIT(IPTR),L"FP")
*ENDIF
*IF -DEF,AELIB5
      L=IFETCH (FIT(IPTR),2LRL)
*ENDIF
*IF DEF,AELIB5
      L=IFETCH (FIT(IPTR),L"RL")
*ENDIF
      IF (FIRST.EQ.0.AND.L.EQ.0) GO TO 1000
      FIRST=1
*IF -DEF,AELIB5
      IF (J.GE.40B) GO TO 400
*ENDIF
*IF DEF,AELIB5
      IF (J.GE.O"40") GO TO 400
*ENDIF
      LASTP=64*I + 64*31
      L=(L+9)/10
      IF (L.EQ.64) GO TO 230
*IF -DEF,AELIB5
      LASTP=64*(I+L) - 64 + 100000B
*ENDIF
*IF DEF,AELIB5
      LASTP=64*(I+L) - 64 + O"100000"
*ENDIF
      DO 220 K=L,64
      BUF(I+K)=0
  220 CONTINUE
  230 IF (OCTN.EQ.0) RETURN
C
C    OCTAL DUMP
C
C    IF OCTN=1, THE PAPER TAPE FILE IS DUMPED OUT IN O3 FORMAT, 30
C      WORDS PER LINE
C
      IF (L.EQ.0) RETURN
      K=I + L - 1
      M=0
      DO 310 J=I,K
      F=BUF (J)
      DO 300 L=1,5
      F=SHIFT (F,12)
      M=M + 1
*IF -DEF,AELIB5
      DMPARR(M)=F.AND.777B
*ENDIF
*IF DEF,AELIB5
      DMPARR(M)=F.AND.O"777"
*ENDIF
  300 CONTINUE
      IF (M.LT.30) GO TO 310
      PRINT 1, DMPARR
      M=0
  310 CONTINUE
      IF (M.NE.0) PRINT 1, (DMPARR(I),I=1,M)
      RETURN
C
C    END OF FILE ENCOUNTERED.    BIT 5 IS SET IN ERROR WORD WHEN END OF
C      FILE HAS BEEN REACHED MORE THAN ONCE WITHOUT A REWIND INTERVENING
C
C    AFTER FIVE ATTEMPTS TO READ PAST END OF FILE THE PROGRAM GOES TO
C      TRACEB
C
*IF -DEF,AELIB5
  400 IERR=IERR.OR.30B
*ENDIF
*IF DEF,AELIB5
  400 IERR=IERR.OR.O"30"
*ENDIF
      EOFCT=EOFCT + 1
      IF (EOFCT.EQ.1) RETURN
*IF -DEF,AELIB5
      IERR=IERR.OR.40B
*ENDIF
*IF DEF,AELIB5
      IERR=IERR.OR.O"40"
*ENDIF
      IF (EOFCT.LE.5) RETURN
      PRINT 4
      GO TO 1100
C
  500 NAME=FWA
      RETURN
C
 1000 PRINT 2
      GO TO 1100
 1001 PRINT 3
 1100 CALL TRACEB(1)
      RETURN
C
    1 FORMAT (30(1X,O3))
*IF -DEF,AELIB5
    2 FORMAT (" EMPTY PAPER TAPE FILE")
*ENDIF
*IF DEF,AELIB5
    2 FORMAT (' EMPTY PAPER TAPE FILE')
*ENDIF
*IF -DEF,AELIB5
    3 FORMAT (" FILE ILLEGALLY OPENED BY USER")
*ENDIF
*IF DEF,AELIB5
    3 FORMAT (' FILE ILLEGALLY OPENED BY USER')
*ENDIF
*IF -DEF,AELIB5
    4 FORMAT (" FIFTH ATTEMPT TO CALL PTREAD AFTER ENCOUNTERING EOF")
*ENDIF
*IF DEF,AELIB5
    4 FORMAT (' FIFTH ATTEMPT TO CALL PTREAD AFTER ENCOUNTERING EOF')
*ENDIF
C
      END
      SUBROUTINE COMP (SC,EC,ITYPE,IPAR,NBITS,NBLANKS,KODARR,FWFORM,FWA,
     + INP,LASTP,FRAME,NR,IARR,NUM1,NUM2,IERR)
C
C    ALL THE PARAMETERS ARE INPUT ONLY EXCEPT:
C        INP, LASTP, FRAME, IARR, NUM1, NUM2, IERR.
C
      IMPLICIT INTEGER (A-Z)
      INTEGER GROUP(9),ITER(9)
      INTEGER FTABLE(9)
      REAL DIV
C
*IF -DEF,AELIB5
      DATA FTABLE/1RC,1RX,1RA,1RL,1RR,1RO,1RI,1RF,1RE/
*ENDIF
*IF DEF,AELIB5
      DATA FTABLE/R"C",R"X",R"A",R"L",R"R",R"O",R"I",R"F",R"E"/
*ENDIF
*IF -DEF,AELIB5
      DATA INDEF/17770000000000000000B/
*ENDIF
*IF DEF,AELIB5
      DATA INDEF/O"17770000000000000000"/
*ENDIF
C
      ERRFLG=0
C
      IF (NR.LT.0) GO TO 200
      IF (NR.GT.0) GO TO 210
C
C    LOOK FOR SC
C
  100 IF (FRAME.EQ.SC) GO TO 105
*IF -DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,IERR),
     + RETURNS (500)
*ENDIF
*IF DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,IERR,
     +*500)
*ENDIF
      GO TO 100
  105 IF (ITYPE.EQ.1) GO TO 600
      FORMP=NBR=REPT=0
      CALL EXTRACT (FWFORM,FORMP,6,F,N)
*IF -DEF,AELIB5
      IF (F.NE.1R() GO TO 1000
*ENDIF
*IF DEF,AELIB5
      IF (F.NE.R"(") GO TO 1000
*ENDIF
C
C    THIS PART OF THE PROGRAM CRACKS THE FORMAT DEFINED IN MODE 2 CALL
C
C    BEGINNING OF FORMAT
C
C    FORMAT MUST START WITH LEFT BRACKET
C
  110 TYPE=COUNT=DEC=0
      MULT=10
  120 CALL EXTRACT (FWFORM,FORMP,6,F,N)
*IF -DEF,AELIB5
      IF (F.EQ.1R ) GO TO 120
*ENDIF
*IF DEF,AELIB5
      IF (F.EQ.R" ") GO TO 120
*ENDIF
*IF -DEF,AELIB5
      IF (F.LE.1RZ) GO TO 130
*ENDIF
*IF DEF,AELIB5
      IF (F.LE.R"Z") GO TO 130
*ENDIF
*IF -DEF,AELIB5
      IF (F.GE.1R+) GO TO 140
*ENDIF
*IF DEF,AELIB5
      IF (F.GE.R"+") GO TO 140
*ENDIF
*IF -DEF,AELIB5
      F=F - 33B
*ENDIF
*IF DEF,AELIB5
      F=F - O"33"
*ENDIF
      IF (TYPE.EQ.1) GO TO 121
      IF (DEC.EQ.0) COUNT=10*COUNT + F
      GO TO 120
C
C    THE NUMBER FOLLOWING C MUST BE OCTAL
C
  121 COUNT=8*COUNT + F
      GO TO 120
C
C    TYPE
C
  130 IF (TYPE.NE.0) GO TO 1001
      DO 131 TYPE=1,9
      IF (F.EQ.FTABLE(TYPE)) GO TO 132
  131 CONTINUE
      GO TO 1001
  132 WCOUNT=COUNT
      COUNT=0
      IF (TYPE.GE.3.AND.TYPE.LE.5) MULT=6
      IF (TYPE.EQ.6) MULT=3
      GO TO 120
C
C    SYMBOL
C
*IF -DEF,AELIB5
  140 IF (F.EQ.1R() GO TO 150
*ENDIF
*IF DEF,AELIB5
  140 IF (F.EQ.R"(") GO TO 150
*ENDIF
*IF -DEF,AELIB5
      IF (F.EQ.1R)) GO TO 160
*ENDIF
*IF DEF,AELIB5
      IF (F.EQ.R")") GO TO 160
*ENDIF
*IF -DEF,AELIB5
      IF (F.EQ.1R,) GO TO 170
*ENDIF
*IF DEF,AELIB5
      IF (F.EQ.R",") GO TO 170
*ENDIF
*IF -DEF,AELIB5
      IF (F.EQ.1R.) GO TO 180
*ENDIF
*IF DEF,AELIB5
      IF (F.EQ.R".") GO TO 180
*ENDIF
      GO TO 1001
C
C    LEFT BRACKET
C
  150 IF (NBR.GT.8) GO TO 1001
      NBR=NBR + 1
      GROUP(NBR)=FORMP
      IF (COUNT.EQ.0) COUNT=1
      ITER(NBR)=COUNT
      IF (NBR.EQ.1) REPT=COUNT
      COUNT=0
      GO TO 120
C
C    RIGHT BRACKET
C
C    LAST RIGHT BRACKET GOES BACK TO LEVEL 1 LEFT BRACKET AND ITS
C      ASSOCIATED MULTIPLIER (REPT)
C
  160 IF (NBR.EQ.0) GO TO 161
      ITER(NBR)=ITER(NBR) - 1
      IF (ITER(NBR).NE.0) GO TO 162
      NBR=NBR - 1
      GO TO 170
C
  161 FORMP=6
      IF (REPT.EQ.0) GO TO 170
      NBR=1
      ITER(1)=REPT
  162 FORMP=GROUP(NBR)
C
C    COMMA
C
  170 IF (TYPE.EQ.0) GO TO 120
      CCOUNT=COUNT
      IF (WCOUNT.EQ.0) WCOUNT=1
      IF (CCOUNT.EQ.0) CCOUNT=1
      GO TO 300
C
C    DECIMAL POINT
C
  180 IF (TYPE.LT.8) GO TO 1001
      IF (COUNT.EQ.0.OR.DEC.NE.0) GO TO 1001
      DEC=1
      GO TO 120
C
C    BACKSPACE 10*(M - 1) FRAMES AND CONTINUE READING WITHOUT SEARCHING
C      FOR SC
C      BACKSPACE CANNOT SPACE BEYOND THE BEGINNING OF THE PRESENT
C      PHYSICAL TAPE
C      IF NR=-1 NO BACKSPACING TAKES PLACE BUT READING CONTINUES
C      FROM THE PRESENT POSITION
C
  200 IF (NR.LT.-16) NR=-16
      INP=INP + 128*(NR + 1)
      IF (INP.LT.0) INP=INP + 64*128
      GO TO 105
C
C    FIND EOR
C    READ UNTIL BIT 4 IS SET IN ERROR WORD (J)
C      CONTINUE READING THE NEW PHYSICAL TAPE
C
  210 J=0
*IF -DEF,AELIB5
      CALL READ (FWA,INP,LASTP,N,J,10B)
*ENDIF
*IF DEF,AELIB5
      CALL READ (FWA,INP,LASTP,N,J,O"10")
*ENDIF
*IF -DEF,AELIB5
      IF (J.LT.10B) GO TO 210
*ENDIF
*IF DEF,AELIB5
      IF (J.LT.O"10") GO TO 210
*ENDIF
*IF -DEF,AELIB5
      IF (J.GE.20B) GO TO 500
*ENDIF
*IF DEF,AELIB5
      IF (J.GE.O"20") GO TO 500
*ENDIF
      FRAME=-1
      GO TO 100
C
C    THIS PART OF THE PROGRAM DOES THE ACTUAL DECODING OF THE TAPE
C
C    EC ALWAYS STOPS READING.
C      IF EC IS ENCOUNTERED BETWEEN WORDS THE PROGRAM RETURNS THE
C       PRESENT VALUE OF NUM2
C      IF EC IS ENCOUNTERED IN THE MIDDLE OF A WORD, THAT WORD IS MADE
C      INVALID AND THE PROGRAM EXITS AFTER PROCESSING ERROR ROUTINE
C
  300 DO 310 I=1,WCOUNT
      WORD=SIGN=DEC=ERRFLG=0
      DIV=1.0
      DO 304 J=1,CCOUNT
*IF -DEF,AELIB5
  302 CALL DECODE (FWA,INP,LASTP,FRAME,A,IPAR,NBITS,NBLANKS,KODARR,EC,
     + ERRFLG), RETURNS (500)
*ENDIF
*IF DEF,AELIB5
  302 CALL DECODE (FWA,INP,LASTP,FRAME,A,IPAR,NBITS,NBLANKS,KODARR,EC,
     + ERRFLG, *500)
*ENDIF
      IF (J.EQ.1.AND.FRAME.EQ.EC) RETURN
      IF (J.EQ.1) WORDP=INP
      IF (TYPE.EQ.1) GO TO 330
      IF (J.EQ.1.AND.A.EQ.0) GO TO 302
      IF (TYPE.EQ.2) GO TO 310
      IF (A.EQ.0) GO TO 400
      IF (TYPE.LE.5) GO TO 303
*IF -DEF,AELIB5
      IF (A.EQ.1R+) GO TO 320
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R"+") GO TO 320
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1R-) GO TO 321
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R"-") GO TO 321
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1R ) GO TO 322
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R" ") GO TO 322
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1R.) GO TO 323
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R".") GO TO 323
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1RE) GO TO 350
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R"E") GO TO 350
*ENDIF
*IF -DEF,AELIB5
      A=A - 33B
*ENDIF
*IF DEF,AELIB5
      A=A - O"33"
*ENDIF
      IF (A.LT.0.OR.A.GT.9) ERRFLG=ERRFLG.OR.1
      IF (TYPE.EQ.6.AND.A.GT.7) ERRFLG=ERRFLG.OR.1
      IF (DEC.NE.0) DIV=10.0*DIV
      IF (MULT.EQ.10) WORD=10*WORD + A
  303 IF (MULT.NE.10) WORD=SHIFT (WORD,MULT).OR.A
  304 CONTINUE
  305 IF (TYPE.GE.3.AND.TYPE.LE.4) GO TO 340
  306 IF (SIGN.NE.0) WORD=-WORD
  307 NUM2=NUM2 + 1
      IERR=IERR.OR.ERRFLG
      ERRFLG=ERRFLG.AND.7
      IF (ERRFLG.NE.0) GO TO 410
  308 CALL STORE (IARR,NUM2,WORD,DIV,TYPE-8)
  309 IF (NUM2.EQ.NUM1) RETURN
  310 CONTINUE
      GO TO 110
C
C    PLUS
C
  320 IF (WORD.NE.0.OR.DEC.NE.0) ERRFLG=ERRFLG.OR.1
      GO TO 304
C
C    MINUS
C
  321 IF (WORD.NE.0.OR.DEC.NE.0) ERRFLG=ERRFLG.OR.1
      SIGN=1
      GO TO 304
C
C    LEADING BLANK
C
  322 IF (WORD.NE.0.OR.DEC.NE.0) ERRFLG=ERRFLG.OR.1
      GO TO 304
C
C    DECIMAL POINT
C
  323 IF (TYPE.LT.8.OR.DEC.NE.0) ERRFLG=ERRFLG.OR.1
      DEC=1
      GO TO 304
C
C    C FORMAT
C
C    IF THE C FRAME IS NOT FOUND IMMEDATELY, SEARCH THROUGH ANY
C      FOLLOWING NON-DATA FRAMES UNTIL A DATA FRAME IS FOUND, THEN EXIT
C
  330 IF (FRAME.EQ.CCOUNT) GO TO 310
      IF (A.EQ.0) GO TO 302
      IERR=IERR.OR.2
      RETURN
C
C    A, L FORMATS
C    LEFT ADJUST AND FILL WITH BLANKS OR ZEROS.
C
  340 WM=WORD.AND.MASK(6)
      IF (WM.NE.0) GO TO 307
      WORD=SHIFT (WORD,6)
      IF (TYPE.EQ.4) GO TO 340
*IF -DEF,AELIB5
      WORD=WORD.OR.1R
*ENDIF
*IF DEF,AELIB5
      WORD=WORD.OR.R" "
*ENDIF
      GO TO 340
C
C    E FORMAT
C
  350 IF (TYPE.NE.9) ERRFLG=ERRFLG.OR.1
      EFAC=ESIGN=0
      J=J + 1
      DO 360 K=J,CCOUNT
*IF -DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,IPAR,NBITS,NBLANKS,KODARR,EC,
     + ERRFLG), RETURNS (500)
*ENDIF
*IF DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,IPAR,NBITS,NBLANKS,KODARR,EC,
     + ERRFLG, *500)
*ENDIF
      IF (A.EQ.0) GO TO 400
*IF -DEF,AELIB5
      IF (A.EQ.1R ) GO TO 360
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R" ") GO TO 360
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1R+) GO TO 360
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R"+") GO TO 360
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.1R-) GO TO 361
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.R"-") GO TO 361
*ENDIF
*IF -DEF,AELIB5
      A=A - 33B
*ENDIF
*IF DEF,AELIB5
      A=A - O"33"
*ENDIF
      IF (A.LT.0.OR.A.GT.9) ERRFLG=ERRFLG.OR.1
      EFAC=10*EFAC + A
  360 CONTINUE
      IF (ESIGN.EQ.0) EFAC=-EFAC
      DIV=DIV*10.0**EFAC
      GO TO 306
C
  361 IF (EFAC.NE.0) ERRFLG=ERRFLG.OR.1
      ESIGN=1
      GO TO 360
C
C    NON-DATA FRAME TERMINATES THE WORD AND SETS THE ERROR FLAG
C
  400 ERRFLG=ERRFLG.OR.1
      IF (FRAME.EQ.EC) NUM1=NUM2 + 1
      IF (TYPE.LE.4) GO TO 340
      GO TO 307
C
C    INVALID
C
C    SUBROUTINE ERROR ASSEMBLES THE WORD TO BE DUMPED FROM MODE 4 CALL
C
C      ALPHANUMERIC FORMATS STOP, BUT OTHER FORMATS STORE AN INDEFINITE
C      WORD IN THE USER'S ARRAY
C
  410 CALL ERROR (CCOUNT,FWA,WORDP,IPAR,NBITS,NBLANKS,KODARR,EC)
      IF (TYPE.LE.5) GO TO 308
      IF (ERRFLG.EQ.4) GO TO 308
      CALL STORE (IARR,NUM2,INDEF,0.0,-1)
      GO TO 309
C
C    END OF FILE ENCOUNTERED
C
*IF -DEF,AELIB5
  500 IERR=OR (IERR,ERRFLG,30B)
*ENDIF
*IF DEF,AELIB5
  500 IERR=OR (IERR,ERRFLG,O"30")
*ENDIF
      RETURN
C
C    BINARY TAPE
C
  600 M=SHIFT(MASK(NBITS),NBITS)
      N=-M
      ERRFLG=0
*IF -DEF,AELIB5
  610 CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,ERRFLG),
     + RETURNS (500)
*ENDIF
*IF DEF,AELIB5
  610 CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,ERRFLG,
     +*500)
*ENDIF
      IF (FRAME.EQ.EC) RETURN
      F=FRAME.AND.N
  620 FORMP=0
      CALL EXTRACT (FWFORM,FORMP,60,FORM,J)
      IF (F.NE.FORM) GO TO 610
      F=FRAME.AND.M
      CALL EXTRACT (FWFORM,FORMP,60,FORM,J)
      L=FORM*NBITS
      WORD=SHIFT(F,L)
      COUNT=1
  630 CALL EXTRACT (FWFORM,FORMP,60,FORM,J)
*IF -DEF,AELIB5
      IF (FORM.EQ.777B) GO TO 650
*ENDIF
*IF DEF,AELIB5
      IF (FORM.EQ.O"777") GO TO 650
*ENDIF
*IF -DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,ERRFLG),
     + RETURNS (500)
*ENDIF
*IF DEF,AELIB5
      CALL DECODE (FWA,INP,LASTP,FRAME,A,0,0,NBLANKS,0,0,ERRFLG,
     +*500)
*ENDIF
      COUNT=COUNT + 1
      IF (FRAME.EQ.EC) GO TO 660
      IF (FORM.LT.0) GO TO 640
      F=FRAME.AND.N
      IF (F.NE.FORM) GO TO 670
      F=FRAME.AND.M
      CALL EXTRACT (FWFORM,FORMP,60,FORM,J)
      L=FORM*NBITS
      WORD=WORD.OR.SHIFT (F,L)
      GO TO 630
C
C    CHECK FRAME IS NETIVE IN FORMAT
C
  640 IF (FRAME.EQ.-FORM) GO TO 630
      ERRFLG=ERRFLG.OR.2
      GO TO 680
C
  650 NUM2=NUM2 + 1
      CALL STORE (IARR,NUM2,WORD,0.0,-1)
      GO TO 690
C
  660 NUM1=NUM2 + 1
  670 ERRFLG=ERRFLG.OR.1
  680 NUM2=NUM2 + 1
      CALL STORE (IARR,NUM2,INDEF,0.0,-1)
  690 IERR=IERR.OR.ERRFLG
      ERRFLG=ERRFLG.AND.7
      IF (ERRFLG.NE.0) CALL ERROR (-1,1,WORD,COUNT,FRAME,0,0,0)
      IF (NUM2.EQ.NUM1) RETURN
      IF (ERRFLG.EQ.0) GO TO 610
      ERRFLG=0
      GO TO 620
C
 1000 PRINT 1
      GO TO 1100
 1001 PRINT 2, F
 1100 CALL TRACEB(1)
      RETURN
C
*IF -DEF,AELIB5
    1 FORMAT (" FORMAT MUST START WITH LEFT BRACKET")
*ENDIF
*IF DEF,AELIB5
    1 FORMAT (' FORMAT MUST START WITH LEFT BRACKET')
*ENDIF
*IF -DEF,AELIB5
    2 FORMAT (" INVALID CHARACTER IN FORMAT STATEMENT, CHARACTER = ",R1)
*ENDIF
*IF DEF,AELIB5
    2 FORMAT (' INVALID CHARACTER IN FORMAT STATEMENT, CHARACTER = ',R1)
*ENDIF
C
      END
*IF -DEF,AELIB5
      SUBROUTINE DECODE (FWA,INP,LASTP,FRAME,BYTE,IPAR,NBITS,NBLANKS,
     + KODARR,EC,ERRFLG), RETURNS (N)
*ENDIF
*IF DEF,AELIB5
      SUBROUTINE DECODE (FWA,INP,LASTP,FRAME,BYTE,IPAR,NBITS,NBLANKS,
     +                             KODARR, EC, ERRFLG, * )
*ENDIF
C
C    ZERO FRAMES ARE ALWAYS INVALID, AND BYTE IS ALSO ZERO
C    IF FRAME=EC, BYTE IS SET TO ZERO BUT FRAME IS UNCHANGED
C
      IMPLICIT INTEGER (A-Z)
C
*IF -DEF,AELIB5
  100 LASTP1=LASTP.AND.77777B
*ENDIF
*IF DEF,AELIB5
  100 LASTP1=LASTP.AND.O"77777"
*ENDIF
      IF (INP.NE.LASTP1) GO TO 110
*IF -DEF,AELIB5
      CALL READ (FWA,INP,LASTP,J,ERRFLG,10B)
*ENDIF
*IF DEF,AELIB5
      CALL READ (FWA,INP,LASTP,J,ERRFLG,O"10")
*ENDIF
*IF -DEF,AELIB5
      IF (ERRFLG.GE.20B) RETURN N
*ENDIF
*IF DEF,AELIB5
      IF (ERRFLG.GE.O"20") RETURN 1
*ENDIF
      GO TO 100
  110 IF (INP.GE.64*128) INP=0
      CALL EXTRACT (FWA,INP,12,FRAME,P)
*IF -DEF,AELIB5
      IF (FRAME.GE.400B) GO TO 200
*ENDIF
*IF DEF,AELIB5
      IF (FRAME.GE.O"400") GO TO 200
*ENDIF
      IF (FRAME.EQ.0) GO TO 300
      IF (FRAME.EQ.EC) GO TO 300
      IF (NBITS.EQ.0) RETURN
      IF (IPAR.GT.0.AND.P.EQ.1) GO TO 400
      IF (IPAR.LT.0.AND.P.EQ.0) GO TO 400
      FRP=FRAME
      P=0
C
C    READ DECODING ARRAY WITH POINTERS CALCULATED FROM CORRECTED FRAME
C      (FRP)
C
  150 M=SHIFT (MASK(NBITS),NBITS)
      J=FRP.AND.M
      K=SHIFT(J,-3)
      L=64*K + 6*(J.AND.7) + 12
      CALL EXTRACT (KODARR,L,6,BYTE,J)
      IF (BYTE.NE.0.AND.P.NE.0) ERRFLG=ERRFLG.OR.4
      RETURN
C
C    BLANK TAPE
C
C    IGNORE BLANK TAPE UNLESS THE NUMBER OF BLANKS IS GREAT OR EQUAL TO
C      NBLANKS
C
*IF -DEF,AELIB5
  200 FNB=FRAME.AND.377B
*ENDIF
*IF DEF,AELIB5
  200 FNB=FRAME.AND.O"377"
*ENDIF
      IF (FNB.LT.NBLANKS) GO TO 100
      FRAME=-1
  300 BYTE=0
      RETURN
C
C    THE PARITY ERROR ROUTINE CHANGES THE PARITY BIT AND THEN DECODES
C      CORRECTED FRAME
C      THE WORD "FRAME" IS RETURNED BY THE SUBROUTINE UNCHANGED
C
  400 K=IPAR - 1
      IF (IPAR.LT.0) K=-IPAR - 1
      M=1
      M=SHIFT (M,K)
      FRP=XOR (FRAME,M)
      P=1
      GO TO 150
C
      END
      SUBROUTINE ERROR (CCOUNT,FWA,WORDP,IPAR,NBITS,NBLANKS,KODARR,EC)
C
C    THIS SUBROUTINE IS CALLED IN VARIOUS WAYS ACCORDING TO THE VALUES
C      OF THE FIRST TWO PARAMETERS.
C
C    ALL PARAMETERS ARE INPUT ONLY.
C
      IMPLICIT INTEGER (A-Z)
      INTEGER ERRARR(10),PARARR(10)
      DATA PTR/0/
C
      IF (CCOUNT.GT.0.AND.FWA.NE.0) GO TO 100
      IF (CCOUNT.LT.0.AND.FWA.NE.0) GO TO 200
      IF (CCOUNT.EQ.0.AND.FWA.EQ.0) GO TO 300
      IF (CCOUNT.LT.0.AND.FWA.EQ.0) GO TO 400
      RETURN
C
C    BCD TAPE.
C
C      REREAD WORD AND ASSEMBLE IN A10 FORMAT.   IF THE NUMBER OF
C      CHARACTERS PER WORD IS GREAT THAN 10, ASSEMBLE ONLY THE RIGHTMOST
C      10 CHARACTERS.    IF THE PARITY OF ANY FRAME IS WRONG, PLACE A
C      ? IN THE APPROPRIATE PLACE IN PARARR ARRAY.
C
  100 IF (PTR.EQ.10) RETURN
      PTR=PTR + 1
*IF -DEF,AELIB5
      BITP=WORDP.AND.77B
*ENDIF
*IF DEF,AELIB5
      BITP=WORDP.AND.O"77"
*ENDIF
      INP=WORDP - 12
      IF (BITP.EQ.0) INP=INP - 4
      ERRARR(PTR)=PARARR(PTR)=FLAG=0
      DO 110 I=1,CCOUNT
*IF -DEF,AELIB5
      A=B=1R
*ENDIF
*IF DEF,AELIB5
      A=B=R" "
*ENDIF
      P=0
*IF -DEF,AELIB5
      CALL DECODE (FWA,INP,64*200,FR,A,IPAR,NBITS,NBLANKS,KODARR,EC,P),
     + RETURNS (120)
*ENDIF
*IF DEF,AELIB5
      CALL DECODE (FWA,INP,64*200,FR,A,IPAR,NBITS,NBLANKS,KODARR,EC,P,
     +*120)
*ENDIF
*IF -DEF,AELIB5
      IF (A.EQ.0) A=FLAG=1R#
*ENDIF
*IF DEF,AELIB5
      IF (A.EQ.0) A=FLAG=R"#"
*ENDIF
      P=P.AND.4
*IF -DEF,AELIB5
      IF (P.NE.0) B=1R?
*ENDIF
*IF DEF,AELIB5
      IF (P.NE.0) B=R"?"
*ENDIF
      ERRARR(PTR)=SHIFT (ERRARR(PTR),6).AND.MASK(54).OR.A
      PARARR(PTR)=SHIFT (PARARR(PTR),6).AND.MASK(54).OR.B
      IF (FLAG.NE.0) GO TO 120
  110 CONTINUE
  120 WM=ERRARR(PTR).AND.MASK(6)
      IF (WM.NE.0) RETURN
*IF -DEF,AELIB5
      ERRARR(PTR)=SHIFT (ERRARR(PTR),6).OR.1R
*ENDIF
*IF DEF,AELIB5
      ERRARR(PTR)=SHIFT (ERRARR(PTR),6).OR.R" "
*ENDIF
*IF -DEF,AELIB5
      PARARR(PTR)=SHIFT (PARARR(PTR),6).OR.1R
*ENDIF
*IF DEF,AELIB5
      PARARR(PTR)=SHIFT (PARARR(PTR),6).OR.R" "
*ENDIF
      GO TO 120
C
C    BINARY TAPE
C
C    FOR THIS CALL, WORDP = OUTPUT WORD
C                   IPAR  = FRAME COUNT
C                   NBITS = ERRONEOUS FRAME
C      DIGITS IN ERROR WORD:
C        0 - 2     ERRONEOUS FRAME
C            4     NUMBER OF FRAME IN FORMAT
C        6 -19     BINARY WORD AS ASSEMBLED TO DATE
C
  200 IF (PTR.EQ.20) RETURN
      PTR=PTR + 1
      IF (NBITS.LT.0) NBITS=0
      ERRARR(PTR)=OR (WORDP,SHIFT(IPAR,45),SHIFT(NBITS,51))
      RETURN
C
C    DUMP BCD ERROR ARRAYS.
C
  300 IF (PTR.EQ.0) GO TO 310
      PRINT 1, (PARARR(I),I=1,PTR)
      PRINT 1, (ERRARR(I),I=1,PTR)
      PTR=0
  310 PRINT 1
      RETURN
C
C    DUMP BINARY ERROR ARRAY.
C      THIS ARRAY SPREADS INTO PARARR ARRAY.
C
  400 IF (PTR.EQ.0) GO TO 410
      PRINT 2, (ERRARR(I),I=1,PTR)
      PTR=0
  410 PRINT 2
      RETURN
C
    1 FORMAT (4X,10(2X,A10))
    2 FORMAT (6(2X,O20))
C
      END
          IDENT  PARAMS
*
*    THIS SUBROUTINE CALCULATES THE NUMBER OF PARAMETERS IN CALL (NPAR)
*      IT PUTS THE ADDRESS OF THE FIRST SEVEN PARAMETERS INTO THE
*       SECOND SEVEN PARAMETERS.
*
*    P1 - P7 ARE INPUT PARAMETERS
*    P8 - P15 ARE OUPUT
*
          LDSET  LIB=BAMLIB
          ENTRY  PARAMS
*
 PARAMS   DATA   0
          SB1    1
          SB2    A1
          SB3    B0
          SB4    7
          SB5    B2+B4
 FTXIN    SA3    B2+B3
          ZR     X3,ALLIN
          BX6    X3
          SA4    B3+B5
          SA6    X4
          SB3    B1+B3
          NE     B3,B4,FTXIN
 ALLIN    SX7    B3
          SA4    B4+B5
          SA7    X4
          EQ     PARAMS
*
          END
          IDENT  EXTRACT
*
*    THIS SUBROUTINE EXTRACTS NBITS FROM THE TAPE BUFFER OR FORMAT
*      ACCORDING TO THE VALUE OF PTR.    PTR IS MADE UP OF
*             100B*(WORD POINTER) + (BIT POINTER)
*                                                  WHERE BOTH START
*      AT ZERO.    THE SUBROUTINE DEALS WITH THE EXTRA 4 BITS AT THE END
*      OF EACH WORD.   PTR IS UPDATED FOR THE NEXT CALL
*
*    THE SUBROUTINE WORKS FOR NBITS=60 OR ANY SUBMULTIPLE OF 60
*
*    FWA CONTAINS THE ADDRESS OF THE FIRST WORD OF THE INPUT BUFFER
*
          ENTRY  EXTRACT
*
 EXTRACT  DATA   0
          SB1    1
          SB5    60
          SA2    A1+B1
          SA3    A2+B1
          SB6    A3+B1
          SA1    X1          FWA
          SA2    X2          PTR
          SA3    X3          NBITS
          MX0    6
          LX0    6
          BX4    X0*X2       BIT POINTER  -  LOWER 6 BITS
          IX4    X3+X4
          SB4    X4          UPDATED BIT POINTER
          AX2    6
          SB2    X2          WORD POINTER
          SB3    B2
          NE     B4,B5,LESS
          SB3    B1+B2       WORD BOUNDARY
          SB4    B0            INCREMENT WORD POINTER AND
*                              RESET BIT POINTER
 LESS     SX2    B3
          LX2    6
          SX6    X2+B4
          SA6    A2          UPDATED PTR
          MX0    1           MAKE A MASK OF NBITS
          SB5    X3-59
          NE     B1,B5,SHIFT
          MX0    0           SPECIAL CASE - N=60
          SB5    -B0           NO MASKING
 SHIFT    SB5    -B5
          AX0    B5,X0
          SA4    X1+B2       READ APPROPRIATE WORD
          LX4    B4,X4       LEFT SHIFT AND
          BX6    -X0*X4        MASK
          SA3    B6          RETRIEVE ADDRESS OF BYTE
          SA6    X3          BYTE
          CX4    X6          NUMBER OF 1 BITS
          SX5    B1
          BX7    X4*X5       MASK WITH 1
          SA3    A3+B1
          SA7    X3          PARITY 0 EVEN, 1 ODD
          EQ     EXTRACT
*
          END
          IDENT  STORE
*
*    THIS SUBROUTINE STORES AN INTEGER OR A REAL WORD IN THE USER'S
*      ARRAY WHOSE STARTING ADDRESS IS FWA.    TYPE DETERMINES THE TYPE
*      OF WORD TO BE STORED.   THE RESULT IS INDEPENDENT OF THE DECLARED
*      TYPE OF ARRAY.
*
*   ALL PARAMETERS ARE INPUT ONLY
*
*    WORD IS INTEGER ON ENTRY TO THE PROGRAM
*    DIV IS REAL
*
          ENTRY  STORE
*
 STORE    DATA   0
          SB1    1
          SB7    -B1
          SA2    A1+B1
          SA3    A2+B1
          SA4    A3+B1
          SA5    A4+B1
          SA1    X1          FWA
          SA2    X2          INDEX
          SA3    X3          WORD
          SA4    X4          DIVISOR
          SA5    X5          TYPE - POS REAL, NEG INTEGER
          SX2    X2+B7       SUBTRACT 1 FROM INDEX
          PL     X5,REAL
          BX6    X3
          IX4    X1+X2       CALCULATE ADDRESS
          SA6    X4
          EQ     STORE
*
 REAL     PX6    B0,X3       PACK AND NORMALIZE DIVIDEND
          NX3    B0,X6
          FX3    X3/X4       DIVISOR IS ALREADY NORMALIZED
          NX6    B0,X3
          IX4    X1+X2
          SA6    X4
          EQ     STORE
*
          END
