*DECK QUAD
      FUNCTION QUAD(AA,BB,AUX,EPS,IFLAGG)
      COMMON/QSTATS /ERR,KOUNT,INTS
*CALL ERRORF
C
C  QUAD, A ROUTINE FOR EVALUATING DEFINITE INTEGRALS USING AN ADAPTIVE
C  10 POINT NEWTON-COTES ALGORITHM.
C
C     QUAD= INT( AA TO BB ( AUX(X) DX))     TO ERROR OF EPS
C
C     AA , BB  LOWER AND UPPER LIMITS
C     AUX        EXTERNAL FUNCTION SUPPLIED BY USER TO EVALUATE AUX(X)
C     EPS      REQUIRED RELATIVE ERROR TOLERANCE
C  IFLAGG ERROR WARNING FLAG.  IF RETURNED ZERO NO PROBLEMS ENCOUNTERED
C         OTHERWISE RETURNED AS MINUS NUMBER OF     PROBLEMS ENCOUNTERED
C        IF ENTERED  NON ZERO A MESSAGE IS GIVEN FOR EACH PROBLEM.
C
C  THE INTEGRAL IS EVALUATED OVER THE INTERVALS
C
C     EST = ESTIMATE OVER (A, B)
C     ESTL = ESTIMATE OVER (A, A+B/2)
C     ESTR = ESTIMATE OVER (A+B/2, B)
C
C  IF EST-(ESTR+ESTL) @ RELATIVE ERROR REQUIREMENT THE INTEGRAL IS
C  ACCEPTED.  OTHERWISE THE SAME PROCEEDURE IS CARRIED OUT ON THE
C  SMALLER INTERVALS (A, A+B/2) AND (A+B/2, B).
C
C
C  ERR - ESTIMATE OF THE MAXIMUM RELATIVE ERROR IN THE INTEGRAL.
C  KOUNT - NUMBER OF FUNCTION EVALUATIONS
C  INTS - NUMBER OF INTERVALS USED
C
C  W     - WEIGHTS USED IN NEWTON-COTES FORMULA
C  F     - REAL ARRAY, STORES VALUES OF F(X) USED TO ESTIMATE INTEGRAL
C          OVER THE CURRENT INTERVAL
C  FIT, I=1,9 - REAL ARRAYS, STORE VALUES OF F(X) FROM PREVIOUS
C          PREVIOUS INTERVALS TO AVOID RECALCULATION
C  LEG   - INTEGER ARRAY, =1 OR 2.  TELLS WHETHER RIGHT OR LEFT
C          SUBINTERVAL IS UNDER CONSIDERATION.
C  DXT   - REAL ARRAY, STORES LENGTH OF CORRESPONDING SUBINTERVAL
C  XMT   - REAL ARRAY, STORES ENDPOINT OF CORRESPONDING SUBINTERVAL
C  ART   - REAL ARRAY, STORES ESTIMATE OF AREA OF CORRESPONDING
C          SUBINTERVAL
C  EPST  - REAL ARRAY, STORES ESTIMATE OF RELATIVE ERROR OF
C          CORRESPONDING SUBINTERVAL
C  ESTT  - REAL ARRAY, STORES ESTIMATE OF INTEGRAL OVER
C          CORRESPONDING SUBINTERVAL
C  SUM   - REAL ARRAY, CUMULATIVE ESTIMATE OF INTEGRAL OVER ACCEPTED
C          REGIONS.
C
C
C  THE FUNCTION QUAD RETURNS THE BEST ESTIMATE OF THE INTEGRAL.
C  THE FINEST SUBDIVISION ALLOWED IS (B-A)/2**20.  IF THE ERROR
C  REQUIREMENT IS STILL NOT SATISFIED, AN ERROR MESSAGE IS PRINTED
C  GIVING THE BOUNDS OF THE UNACCEPTABLE REGION, THE+ESTIMATED INTEGRAL
C  OVER THIS INTERVAL, AND AN ESTIMATE OF THE ERROR THERE. THE VALUE
C  OF QUAD RETURNED IS STILL THE BEST ESTIMATE OVER THE ENTIRE RANGE.
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C  SOURCE KAHANER DK COMPARISON OF NUMERICAL QUADRATURE FORMULAS,
C                    MATHEMATICAL SOFTWARE JR RICE ED, ACADEMIC 1971
C       PROGRAMMED BY    DK KAHANER, J MELENDEZ LOS ALAMOS     1971
C       REVISED    BY    VALERIE JONES                     AUG 1976
C
      DIMENSION F0T(20), F1T(20), F2T(20), F3T(20), F4T(20), F5T(20),
     ,   F6T(20), F7T(20), F8T(20), F9T(20), F(19), W(5), LEG(20),
     ,   DXT(20), XMT(20), ART(20), EPST(20), ESTT(20), SUM1(20)
      DIMENSION ERTEMP(20)
      DATA MES/2250/
*CALL  STATF
C
       EPA=EPS
      IPRINT=IFLAGG
      W(1)=2857./89600.
      W(2)=15741./89600.
      W(3)=1080./89600.
      W(4)=19344./89600.
      W(5)=5778./89600.
      QUAD = ERR = 0.
      KOUNT = INTS = 0
      SIGN=1.
      IF(AA-BB) 10,50,5
    5 SIGN=-1.
      A0=AA
      AA=BB
      BB=A0
C
   10 INTS = 0
      ERR = 0.
      IFLAG = 0
      A = AA
      B = BB
      DA=B-A
      AREA=1.
      EST=1.
      L=1
      DO 15 I=1,19,2
   15 F(I)=AUX(A+DA*(I-1)/18.)
      KOUNT=10
   20 DX=DA/2.
      XM=A+DX
      DO 25 I=2,18,2
   25 F(I)=AUX(A+DA*(I-1)/18.)
      KOUNT=KOUNT+9
C
C     THE INTEGRAL IS ESTIMATED OVER EACH HALF OF THE CURRENT INTERVAL.
C
      ESTL=W(1)*F(1)+W(2)*F(2)+W(3)*F(3)+W(4)*F(4)+W(5)*F(5)+W(5)*F(6)+
     ,   W(4)*F(7)+W(3)*F(8)+W(2)*F(9)+W(1)*F(10)
      ESTL=ESTL*DX
      ESTR=W(1)*F(10)+W(2)*F(11)+W(3)*F(12)+W(4)*F(13)+W(5)*F(14)+W(5)*
     ,   F(15)+W(4)*F(16)+W(3)*F(17)+W(2)*F(18)+W(1)*F(19)
      ESTR=ESTR*DX
      AREA=AREA-ABS(EST)+ABS(ESTL)+ABS(ESTR)
      SUM=ESTL+ESTR
      LL = L
C
C     THE DIFFERENCE BETWEEN THE SUM OF THESE TWO ESTIMATES AND THE
C     LAST ESTIMATE OVER THE ENTIRE INTERVAL IS COMPARED WITH THE
C     THE REQUIRED TOLERANCE.
C
      IF(ABS(EST-SUM)-EPA*AREA.GT.0.) GO TO 30
C
C     THE INTERVAL MUST BE SUBDIVIDED AT LEAST ONCE FOR COMPARISON.
C
         IF(EST.NE.1.) GOTO 35
C
C     IF THE ERROR REQUIREMENT IS  NOT SATISFIED AND THE MAXIMUM NUMBER
C     OF SUBDIVISIONS HAS NOT BEEN REACHED, THE INTERVAL IS DIVIDED
C     IN HALF. FUNCTION EVALUATIONS ARE SAVED FOR LATER USE.
C
   30 IF (L.GE.20) GOTO 55
      L=L+1
      LEG(L)=2
      F0T(L)=F(10)
      F1T(L)=F(11)
      F2T(L)=F(12)
      F3T(L)=F(13)
      F4T(L)=F(14)
      F5T(L)=F(15)
      F6T(L)=F(16)
      F7T(L)=F(17)
      F8T(L)=F(18)
      F9T(L)=F(19)
      DXT(L)=DX
      XMT(L)=XM
      ART(L)=AREA
      EPST(L)=EPA/1.4
      ESTT(L)=ESTR
      DA=DX
      F(19)=F(10)
      F(17)=F(9)
      F(15)=F(8)
      F(13)=F(7)
      F(11)=F(6)
      F(9)=F(5)
      F(7)=F(4)
      F(5)=F(3)
      F(3)=F(2)
      EST=ESTL
      EPA=EPST(L)
      GOTO 20
C
C     IF THE ERROR REQUIREMENT IS MET ON THE LEFT HALF OF THE PREVIOUS
C     INTERVAL THIS HALF IS ACCEPTED.  THE ESTIMATES ARE STORED IN
C     ARRAYS AND THE RIGHT HALF OF THE INTERVAL IS CONSIDERED.
C
   35 IF(IFLAG.EQ.1) GOTO 65
   40 IF(LEG(L).LT.2) GOTO 45
      ERTEMP(L) = EST
      SUM1(L)=SUM
      LEG(L)=1
      A=XMT(L)
      DA=DXT(L)
      F(1)=F0T(L)
      F(3)=F1T(L)
      F(5)=F2T(L)
      F(7)=F3T(L)
      F(9)=F4T(L)
      F(11)=F5T(L)
      F(13)=F6T(L)
      F(15)=F7T(L)
      F(17)=F8T(L)
      F(19)=F9T(L)
      AREA=ART(L)
      EST=ESTT(L)
      EPA=EPST(L)
      GOTO 20
C
C     WHEN THE RIGHT HALF OF THE PREVIOUS INTERVAL HAS BEEN ESTIMATED TO
C     WITHIN THE REQUIRED ERROR TOLERANCE, THE ESTIMATE OF THE ENTIRE
C      INTERVAL IS ADDED ON TO THE CUMULATIVE ESTIMATE.
C     THE NUMBER OF INTERVALS IS INCREASED BY 1 AND THE NEXT SUBINTERVAL
C     IS CONSIDERED.
C
   45 SUM = SUM1(L) + SUM
      ERR = ERR + ERTEMP(L)
      INTS = INTS + 2
      L=L-1
      IF (L.GT.1) GOTO 35
C
C     WHEN THE ENTIRE INTERVAL HAS BEEN CONSIDERED, THE BEST ESTIMATE IS
C     RETURNED.
C
      QUAD=SUM
      ERR = ERR + ESTT(LL)
      ERR = ABS(ERR - SUM)
   50 IF(SIGN.EQ.1.)RETURN
      PRINT *,SIGN,AA,BB,A0
      BB=AA
      AA=A0
      QUAD=-SUM
      RETURN
C
C     IF THE MAXIMUM NUMBER OF SUBINTERVALS HAS BEEN REACHED WITHOUT
C     THE ERROR REQUIREMENT BEING MET, THE BOUNDS OF THE OFFENDING
C     ARE STORED ALONG WITH AN ESTIMATE OF THE INTEGRAL AND THE ERROR
C     OVER THIS REGION.
C
   55 IF (IFLAG.EQ.1) GOTO 60
         A1 = A
         B1 = A + DA
         SUBSUM = SUM
         SUBER = EST
         IFLAG = 1
         GOTO 40
C
C     ADJACENT INTERVALS WHERE THE ERROR REQUIREMENT IS NOT MET ARE
C     COMBINED FOR READABILITY.
C
   60 B1 = A + DA
      SUBSUM = SUBSUM + SUM
      SUBER = SUBER + EST
      GOTO 40
C
C     ONCE THE ENTIRE REGION OF DIFFICULTY HAS BEEN DETERMINED A
C     MESSAGE IS PRINTED GIVING THE BOUNDS OF THE INTERVAL, AN
C     ESTIMATE OF THE ERROR AND AN ESTIMATE OF THE INTEGRAL OVER THAT
C     REGION.
C
C
   65 IFLAG=0
      IFLAGG=IFLAGG-1
      IF(IPRINT.EQ.0) GOTO 40
      SSUBSUM=ABS(SUBER-SUBSUM)
      ENCODE(11,70,IERWD(3))A1
*IF -DEF,AELIB5
      IERWD(4)=IERWD(4).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(4)=IERWD(4).AND.MASK(6)
*ENDIF
      ENCODE(11,70,IERWD(5))B1
*IF -DEF,AELIB5
      IERWD(6)=IERWD(6).A.MASK(6)
*ENDIF
*IF DEF,AELIB5
      IERWD(6)=IERWD(6).AND.MASK(6)
*ENDIF
      ENCODE(20,75,IERWD(7))SUBSUM
*IF -DEF,AELIB5
      IERWD(9)=IERWD(9).A.MASK(0)
*ENDIF
*IF DEF,AELIB5
      IERWD(9)=IERWD(9).AND.MASK(0)
*ENDIF
      ENCODE(11,70,IERWD(10))SSUBSUM
*IF -DEF,AELIB5
      IERWD(11)=IERWD(11).A.MASK(6)
      CALL AELERR(MES,2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(11)=IERWD(11).AND.MASK(6)
      CALL AELERR(MES,'NF')
*ENDIF
      GOTO 40
C
   70 FORMAT(E11.5)
   75 FORMAT(E20.14)
C
      END
