*DECK RADCURV
      SUBROUTINE RADCURV(X,Y,N,RERR,Q,P,CINT,WK,WS,IER)
C
C*********************************************************************
C**                                                                 **
C**                          RADCURV                                **
C**                          1-11-30                                **
C**                                                                 **
C*********************************************************************
C*
C*   PURPOSE:TO FIND THE CENTRE AND RADIUS OF CURVATURE GIVEN A SET
C*           OF COORDINATES THAT DEFINE THE CURVE.
C*
C*   ENTRY  :CALL RADCURV(X,Y,W,N,RERR,Q,P,CINT,WK,WS,IER)
C*
C*   X    -INPUT ARRAY OF LENGTH N CONTAINING THE INDEPENDENT VARIABLE
C*   Y    -INPUT ARRAY OF LENGTH N CONTAINING THE DEPENDENT VARIABLE
C*   N    -INPUT NUMBER OF DATA POINTS
C*   RERR -REAL INPUT VARIABLE SPECIFYING THE RELATIVE ACCURACY
C*         TO WHICH THE RADIUS AND CENTRE OF CURVATURE ARE TO BE
C*         CALCULATED
C*   Q    -INPUT VARIABLE DEFINING THE CONFIDENCE INTERVAL TO BE
C*         CALCULATED FOR THE PARAMETERS (USUALLY SET TO 0.05 TO GET
C*         A 95 PERCENT CONFIDENCE INTERVAL)
C*   P    -REAL OUTPUT ARRAY OF LENGTH 3 CONTAINING THE FINAL VALUES
C*         OF THE CENTRE OF CURVATURE (H,K) AND THE RADIUS R, WHERE
C*         P(1)=H,P(2)=K,AND P(3)=R
C*   CINT -OUTPUT ARRAY(3,2) WITH EACH ROW CONTAINING THE LOWER AND
C*         UPPER VALUES OF THE CONFIDENCE INTERVAL FOR PARAMETERS IN P
C*   WK   -DOUBLE PRECISION WORK VECTOR OF LENGTH 4 * N
C*   WS   -REAL WORKING STORAGE ARRAY OF LENGTH 156 + 6*N
C*        -FIRST 5*N USED FOR XYW, NEXT 13 USED FOR ANOVA, AND LAST
C*         60 USED FOR B IN RLFOR
C*        -FIRST 156 + 6*N USED FOR V ARRAY IN NL2SOL
C*        -FIRST 3*N USED FOR ERROR MATRIX IF THE COVARIANCE
C*         MATRIX COULD NOT BE COMPUTED IN NL2SOL
C*        -NEXT 6 ELEMENTS USED TO HOLD THE ERROR MATRIX TIMES ITS
C*         TRANSPOSE J(TRANS)J IN SYMMETRIC STORAGE MODE
C*        -NEXT 6 ELEMENTS USED TO HOLD THE INVERSE OF J(TRANS)J IN
C*         SYMMETRIC STORAGE MODE
C*        -FINALLY, THE FIRST 9 ELEMENTS ARE USED TO HOLD
C*         CALCULATED VARIANCE-COVARIANCE MATRIX IN FULL STORAGE MODE
C*        -NEXT N+15 USED FOR WORKING STORAGE IN RADCURV
C*   IER  -ERROR PARAMETER,FLAG INDICATES WHICH,IF ANY,ERRORS OCCURRED
C*         DURING EXECUTION
C*
C*
C*   AUTHORS:   J. M. BLAIR                              1982 APRIL
C*              JANE FISHER
C*              C. A. WILLS
C*
C*
C***********************************************************************
C
      INTEGER I,IB,IP,IER,M,MDP(3)
      INTEGER NP,CP,RP,NCI,RCI,CCI,IV(63)
      REAL CINT(3,2),P(3),Q,ALBP(2)
      REAL WS(1),PRED,X(N),Y(N),XMIN,XMAX,STND(3)
      REAL A0,A1,A2,ANG,DYM,D2YM,H,K,R,RN1,RN2,XMID,YMID
      DOUBLE PRECISION WK(1)
      LOGICAL ALERPR
C
      COMMON/NL2PRNT/LIST
C
      EXTERNAL DERRES, RESID
C
      DATA LIST /0/
C
*CALL STATF
      IROUT = 11130
C
C     CHECK FOR VALID N VALUE
C
      IF (N .LE. 3) GOTO 500
C
C      TAKE MEANS OFF DATA
C
  10  SUMX = 0.0
      SUMY = 0.0
      DO 20 I = 1,N
         SUMX = SUMX + X(I)
         SUMY = SUMY + Y(I)
  20  CONTINUE
C
      XMEAN = SUMX/N
      YMEAN = SUMY/N
C
      DO 25 I = 1,N
        X(I) = X(I) - XMEAN
        Y(I) = Y(I) - YMEAN
  25  CONTINUE
C
C     TRANSFORM DATA TO NEW COORDINATES
C
      ALPHA = ATAN((Y(1)-Y(N))/(X(N)-X(1)))
      DO 30 I = 1,N
        WS(I) = (X(I)-X(1))*COS(ALPHA)-(Y(I)-Y(1))*SIN(ALPHA)
        WS(I+N) = (X(I)-X(1))*SIN(ALPHA)+(Y(I)-Y(1))*COS(ALPHA)
        WS(I+2*N) = 1.0
  30  CONTINUE
C
C     CHECK ENDPOINTS OF DATA FOR MAX AND MIN VALUES TO ENSURE
C     THE DATA IS ENTERED IN ORDER AROUND THE CURVE
C
      XMIN = WS(1)
      XMAX = WS(1)
      DO 40 I = 2,N
        XMIN = AMIN1(XMIN,WS(I))
        XMAX = AMAX1(XMAX,WS(I))
  40  CONTINUE
C
      IF (((WS(1) .NE. XMIN) .OR. (WS(N) .NE. XMAX)) .AND. ((WS(1) .NE.
     ,   XMAX) .OR. (WS(N) .NE. XMIN))) GOTO 520
C
C     CALCULATE MIDPOINT OF DATA
C
      XMID = (WS(1) + WS(N)) / 2.0
      IF (MOD(N,2) .EQ. 1) YMID = WS((3*N+1)/2)
      IF (MOD(N,2) .EQ. 0) YMID = (WS((3*N+1)/2)+WS((3*N+2)/2))/2.0
C
C    SET UP ARGUMENTS FOR IMSL ROUTINE RLFOR AND CALL THE ROUTINE
C
      MDP(1) = 2
      MDP(3) = 0
      RSQ = 100.0
      IB = 5
      IP = 1
      ALBP(1) = ALBP(2) = 0.05
C
      CALL RLFOR(WS,N,N,RSQ,MDP,ALBP,WS(5*N+1),WS(5*N+14),IB,PRED,IP,
     ,           WK,IER)
C
C     CHECK ERROR FLAG FOR ABNORMAL TERMINATION OF RLFOR
C     IF IER = 33 AN EXACT QUADRATIC FIT HAS BEEN FOUND
C
      IF (IER .EQ. 33) IER = 0
      IF (IER .NE. 0) GOTO 550
C
C      CHECK FOR STRAIGHT LINE(IE. COEFFICIENT FOR X**2 NOT
C        SIGNIFICANT)
C
      IF (WS(5*N+50) .GT. 0.05) GOTO 530
C
C      FIND INITIAL ESTIMATES OF VALUES FOR CENTRE AND RADIUS OF
C      CURVATURE:P(1)=H,P(2)=K,P(3)=R
C
C     GET RESULTS FROM B MATRIX RETURNED FROM RLFOR AND ASSIGN THE
C     CORRESPONDING VALUES TO THE COEFFICIENTS OF POLYNOMIAL
C     A0 + A1*X + A2*X**2
C
      A1 = WS(5*N+19)
      A2 = WS(5*N+20)
      A0 = WS(5*N+21)
C
C     CALCULATE THE FIRST AND SECOND DERIVATIVES WHERE DYM=A1+2*A2*X
C     IS THE FIRST DERIVATIVE AND D2YM=2*A2 IS THE SECOND DERIVATIVE
C
      DYM = A1 + 2*(A2*XMID)
      D2YM = 2*A2
C
C     CALCULATE THE APPROXIMATION FOR THE RADIUS USING THE EQUATION
C      R = SQRT(1 + (DYM**2)**3)/ABS(D2YM)
C
      RN1 = (1+(DYM**2))**3
      RN2 = SQRT(RN1)
      R = RN2/(ABS(D2YM))
C
C     CALCULATE APPROXIMATIONS FOR H AND K USING THE CALCULATED VALUE
C     FOR R AND THE SLOPE OF THE TANGENT AT THE MIDPOINT
C
      ANG = ATAN(DYM)
      H = XMID - SIGN(R,A2)*SIN(ANG)
      K = YMID + SIGN(R,A2)*COS(ANG)
C
C     TRANSLATE H AND K BACK TO ORIGINAL COORDINATES
C
      P(1) = X(1) + H * COS(ALPHA) + K * SIN(ALPHA)
      P(2) = Y(1) - H * SIN(ALPHA) + K * COS(ALPHA)
      P(3) = R
C
C     SET DEFAULT VALUES FOR NL2SOL
C
      CALL DFAULT(IV,WS)
C
C     SET MAXIMUM NUMBER OF ITERATIONS AND SET RELATIVE ACCURACY FOR X
C     CONVERGENCE
C
      IV(18) = 100
      IV(17) = 100*4/3
      IV(20) = 0
      WS(33) = RERR/2.0
C
C     CHECK TO SEE IF LIST IS ON (4) OR OFF (0). IF LIST
C     IS ON USE DEFAULT VALUE OF 4 ,OTHERWISE TURN OFF
C     PARAMETERS TO SUPPRESS PRINTING
C
      IF (LIST .EQ. 0) IV(21) = 0
      M = 3
C
      CALL NL2SOL(N,M,P,RESID,DERRES,IV,WS,X,Y,UFPARM)
C
C     CHECK FOR NORMAL TERMINATION OF NL2SOL
C
      IF ((IV(1) .GE. 7 .AND. IV(1) .LE. 10) .OR. IV(1) .EQ. 15) GOTO
     ,                                                            540
      IF (IV(1) .LT. 3 .OR. IV(1) .GT. 6) GOTO 550
C
C     CHECK TO SEE IF COVARIANCE MATRIX WAS COMPUTED,IF NOT
C     CALCULATE IT IN RADCURV.
C
      IF (IV(26) .GT. 0) GOTO 123
C
C     CALCULATE ERROR MATRIX IF NL2SOL COULD NOT USING THE FORMULA
C     E = SIGMA**2*(J(TRANS)*J)**-1, WHERE J IS THE JACOBIAN MATRIX
C
      ST = 2.0 * WS(10)
      CALL DERRES(N,M,P,NF,WS,X,Y,UFPARM)
      CALL VTPROF(WS,N,M,N,WS(3*N+1))
      CALL LINV1P(WS(3*N+1),M,WS(3*N+7),M,D1,D2,IER)
      CALL VCVTSF(WS(3*N+1),M,WS,N)
C
C     PUT THE STANDARD DEVIATION ON THE DIAGONAL OF THE ERROR MATRIX
C     AND THE CORRELATIONS BELOW THE DIAGONAL
C
      DO 113 I = 1,M
         WS(I+(I-1)*N) = SQRT(WS(I+(I-1)*N)*ST)
         L = I + 1
         IF( I .EQ. M) GO TO 115
            DO 114 J = L,M
               WS(I + (J-1)*N) = WS(I + (J-1)*N)*ST
 114        CONTINUE
 115     L = I-1
         IF(I .EQ. 1) GO TO 113
           DO 116 J = 1,L
             WS(I+(J-1)*N) = WS(J+(I-1)*N)/(WS(J+(J-1)*N)*
     , WS(I+(I-1)*N))
 116       CONTINUE
 113  CONTINUE
C
      DO 117 I = 1,3
        STND(I) = WS((I-1)*N + I)
 117  CONTINUE
      GO TO 124
C
C     GET STANDARD DEVIATIONS AND COMPUTE THE CONFIDENCE INTERVALS
C
 123  STND(1) = SQRT(WS(IV(26)))
      STND(2) = SQRT(WS(IV(26)+2))
      STND(3) = SQRT(WS(IV(26)+5))
C
 124  DEG = N - M
      CALL MDSTI(Q,DEG,TVAL)
C
      DO 125 I=1,3
         CINT(I,1) = P(I) - STND(I)*TVAL
 125  CONTINUE
C
      DO 126 I = 1,3
         CINT(I,2) = P(I) + STND(I)*TVAL
 126  CONTINUE
C
C     PUT MEANS BACK ON CINT
C
      DO 130 J = 1,2
         CINT(1,J) = CINT(1,J) + XMEAN
         CINT(2,J) = CINT(2,J) + YMEAN
 130  CONTINUE
C
C     PUT MEANS BACK ON ESTIMATES FOR H AND K
C
      P(1) = P(1) + XMEAN
      P(2) = P(2) + YMEAN
C
      GOTO 970
C
C *** ERROR CONDITIONS ***
C
C     INVALID INPUT VALUE FOR N
C
 500  IER = 1
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 950
      PRINT 1000, N
      CALL TRACEB(0)
      GOTO 950
C
C     INPUT DATA NOT IN ORDER AROUND CURVE
C
 520  IER = 3
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 950
      PRINT 1200
      CALL TRACEB(0)
      GOTO 950
C
C     DATA DEFINES A STRAIGHT LINE
C
 530  IER = 4
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 950
      PRINT 1300
      CALL TRACEB(0)
      GOTO 950
C
C     NO CONVERGENCE IN NL2SOL
C
 540  IER = 5
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 960
      PRINT 1400, IV(1)
      CALL TRACEB(0)
      GOTO 960
C
C     AN ERROR CONDITION HAS OCCURED WHICH SHOULD NOT BE POSSIBLE
C
 550  IER = 6
      IF (.NOT. ALERPR(IROUT, IER)) GOTO 950
      PRINT 1500
      CALL TRACEB(0)
      GOTO 950
C
C      SET OUTPUT PARAMETERS TO CONTAIN INDEFINITES
C
 950  NP = CP = 1
      RP = 3
      CALL SETBAD(P,NP,RP,CP)
C
 960  NCI = RCI = 3
      CCI = 2
      CALL SETBAD(CINT,NCI,RCI,CCI)
C
C     PUT MEANS BACK ON DATA
C
 970  DO 971 I=1,N
        X(I) = X(I) + XMEAN
        Y(I) = Y(I) + YMEAN
 971  CONTINUE
C
      RETURN
C
C     FORMAT STATEMENTS
C
*IF -DEF,AELIB5
 1000 FORMAT(" ***INVALID VALUE FOR N, N = ", I3)
 1100 FORMAT(" ***WEIGHTS ARE ZERO, NEGATIVE, OR UNDEFINED")
 1200 FORMAT(" ***DATA NOT IN ORDER AROUND THE CURVE")
 1300 FORMAT(" ***DATA DEFINES A STRAIGHT LINE")
 1400 FORMAT(" ***PREMATURE TERMINATION FROM NL2SOL, IV(1) = ", I3)
 1500 FORMAT(" ***INTERNAL ERROR IN RADCURV")
*ENDIF
*IF DEF,AELIB5
 1000 FORMAT(' ***INVALID VALUE FOR N, N = ', I3)
 1100 FORMAT(' ***WEIGHTS ARE ZERO, NEGATIVE, OR UNDEFINED')
 1200 FORMAT(' ***DATA NOT IN ORDER AROUND THE CURVE')
 1300 FORMAT(' ***DATA DEFINES A STRAIGHT LINE')
 1400 FORMAT(' ***PREMATURE TERMINATION FROM NL2SOL, IV(1) = ', I3)
 1500 FORMAT(' ***INTERNAL ERROR IN RADCURV')
*ENDIF
C
      END
      SUBROUTINE RESID(N,M,P,NF,F,X,Y,UFPARM)
C.......................................................................
C CALCULATE THE FUNCTION TO BE MINIMIZED
C F(I) = SQRT((X(I) - H) ** 2 + (Y(I) - K) ** 2) - R
C
C INPUT:  N--THE NUMBER OF DATA POINTS
C         M--THE NUMBER OF PARAMETERS (= 3)
C         P--THE CURRENT VALUES OF THE PARAMETERS
C            P(1) = H
C            P(2) = K
C            P(3) = R
C         X--THE INDEPENDENT VARIABLE VALUES
C         Y THE DEPENDENT VARIABLE VALUES
C
C OUTPUT:  F--THE FUNCTION VALUES
C.......................................................................
C
      INTEGER N,M,NF
      REAL P(M),F(N),X(1),Y(1),ROOT
C
      DO 10 I=1,N
         ROOT = (X(I)-P(1))**2 + (Y(I)-P(2))**2
         F(I) = SQRT(ROOT) - P(3)
  10  CONTINUE
      RETURN
      END
      SUBROUTINE DERRES(N,M,P,NF,J,X,Y,UFPARM)
C.......................................................................
C CALCULATE THE PARTIAL DERIVATIVES OF THE FUNCTION TO BE MINIMIZED
C DF(I) / DH = (H - X(I)) / SQRT((X(I) - H) ** 2 + Y(I) - K) ** 2)
C DF(I) / DK = (K - Y(I)) / SQRT((X(I) - H) ** 2 + (Y(I) - K) ** 2)
C DF(I) / DR = -1
C
C INPUT:  N--THE NUMBER OF DATA POINTS
C         M--THE NUMBER OF PARAMETERS
C         P--THE CURRENT PARAMETER VALUES
C            P(1) = H
C            P(2) = K
C            P(3) = R
C         X--THE INDEPENDENT VARIABLE VALUES
C         Y--THE DEPENDENT VARIABLE VALUES
C
C OUTPUT:  NF-- = 0 IF THE PARTIAL DERIVATIVES CANNOT BE CALCULATED
C              UNCHANGED OTHERWISE
C          J--JACOBIAN MATRIX OF PARTIAL DERIVATIVES
C.......................................................................
C
      INTEGER N,M,NF
      REAL P(M),J(N,M),X(1),Y(1),ROOT
C
      DO 10 I = 1,N
         ROOT = (X(I)-P(1))**2 + (Y(I)-P(2))**2
         IF(ROOT .LE. 0) GO TO 20
         J(I,1) = (P(1)-X(I))/SQRT(ROOT)
         J(I,2) = (P(2)-Y(I))/SQRT(ROOT)
         J(I,3) = -1
  10  CONTINUE
      GO TO 950
C
C     ERROR,ROOT EQUALS ZERO-CANNOT COMPUTE JACOBIAN
C
  20  NF = 0
 950  RETURN
      END
