*DECK REX
      SUBROUTINE REX(X,N)
C
C **********************************************************************
C ******                                                          ******
C ******                         1-05-01                          ******
C ******                        CALCULATE                         ******
C ******   SEQUENCE OF EXPONENTIALLY DISTRIBUTED RANDOM NUMBERS   ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE:  THIS SUBROUTINE GENERATES A SEQUENCE OF EXPONENTIALLY    *
C *           DISTRIBUTED RANDOM NUMBERS WITH MEAN 1.  THE AREA UNDER  *
C *           THE EXPONENTIAL DISTRIBUTION IS DIVIDED INTO AN          *
C *           ISOSCELES TRIANGLE, AN INFINITE SERIES OF TRIANGLES, AND *
C *           2 INFINITE SERIES OF REGIONS BETWEEN THE TRIANGLES AND   *
C *           THE FUNCTION.  EACH OF THESE REGIONS IS SAMPLED IN       *
C *           PROPORTION TO ITS AREA.  AN ACCEPTANCE REJECTION         *
C *           TECHNIQUE IS USED WITHIN EACH REGION TO PRODUCE THE      *
C *           REQUIRED RANDOM NUMBER.  THE ALGORITHM IS DESCRIBED MORE *
C *           FULLY IN AECL-7428 BY K. R. CHAPLIN AND C. A. WILLS      *
C *                                                                    *
C * ENTRY:    CALL REX(X, N)                                           *
C *                                                                    *
C *           X(N)......REAL OUTPUT ARRAY OF EXPONENTIALLY DESTRIBUTED *
C *                     RANDOM NUMBERS                                 *
C *           N.........INTEGER INPUT VARIABLE, NUMBER OF RANDOM       *
C *                     NUMBERS TO GENERATE                            *
C *                                                                    *
C * AUTHOR:   K. R. CHAPLIN                                            *
C *           WRITTEN AUGUST 1979                                      *
C *                                                                    *
C **********************************************************************
C
      REAL X(1)
C
C TATT0--TOTAL AREA OF ALL TRIANGLES
C TARU--TOTAL AREA OF ALL REGIONS RU(I)
C XRU--UPPER END OF INTERVAL FOR RU(T) = 1 - LN(2)
C ARU--MAXIMUM VALUE OF (RU(T) + RU(1 - LN(2) + T)) * (4 / E - 2 *
C      LN(2) ** 2 - .5)
C BRU--MINIMUM VALUE OF (RU(T) + RU(1 - LN(2) - T)) * (4 / E - 2 *
C      LN(2) ** 2 - .5)
C XRL--UPPER END OF INTERVAL FOR RL(T) = 2 * LN(2) - 1
C ARL--MAXIMUM VALUE OF (RL(T) + RL(2 * LN(2) - 1 - T)) * ( .5 *
C      LN(2) ** 2 + .5 - 2 / E)
C BRL--MINIMUM VALUE OF (RL(T) + RL(2 * LN(2) - 1 - T)) * (.5 *
C      LN(2) ** 2 + .5 - 2 / E)
C CL--CONSTANT VALUE LN(2)
C E4--CONSTANT VALUE 4 / E
C
      DATA TATT0,TARU,XRU,ARU,BRU,XRL,ARL,BRL,CL,E4/.9804530139182,
     ,.00893524923243,.30685281944005,.04261170178294,.02238058936147,
     ,.38629436111989,.08522340356588,.03982827773064,.69314718055994,
     ,1.4715177646858/
C
*CALL STATF
C
C     U IS USED THROUGHOUT THE PROGRAM TO PICK THE REQUIRED REGION
C
C     CHECK NUMBER OF RANDOM VALUES TO BE GENERATED
C
      IF (N .GT. 0) GOTO 1
C
      CALL SETBAD(X, 1, 1, 1)
      PRINT 100
*IF -DEF,AELIB5
  100 FORMAT(" *** VALUE OF N NOT GREATER THAN ZERO")
*ENDIF
*IF DEF,AELIB5
  100 FORMAT(' *** VALUE OF N NOT GREATER THAN ZERO')
*ENDIF
      CALL TRACEB(0)
      RETURN
C
1     DO 50  II=1,N
*IF -DEF,AELIB5
      U=RANF(A)
*ENDIF
*IF DEF,AELIB5
      U=RANF()
*ENDIF
      IF(U.GT..5) GOTO 5
C
C     THE MINIMUM OF TWO RANDOM NUMBERS IS DETERMINED HERE TO CHOOSE
C     FROM THE ISOSCELES TRIANGLE
C
*IF -DEF,AELIB5
         U0=RANF(A)
*ENDIF
*IF DEF,AELIB5
         U0=RANF()
*ENDIF
         X(II)=U
         IF(X(II) .LE. U0)GO TO 50
         X(II)=1.-X(II)
         GO TO 50
C
C     I REFERS TO THE ITH REGION  IN THE INFINITE SERIES OF
C     TRIANGLES , OR REGIONS CALLED RU(I),RL(I)
C     B PLACES REX IN THE CORRECT AREA OF THE LINE
C
*IF -DEF,AELIB5
    5 I=974-SHIFT(RANF(A),-48)
*ENDIF
*IF DEF,AELIB5
    5 I=974-SHIFT(RANF(),-48)
*ENDIF
      B=I*CL
C
      U=U-TATT0
      IF(U.GT.0.) GOTO 10
C
C     CHOOSE FROM AN ISOCELES TRIANGLE
C
*IF -DEF,AELIB5
         X(II)=(RANF(A)+RANF(A))*CL+B+1.
*ENDIF
*IF DEF,AELIB5
         X(II)=(RANF()+RANF())*CL+B+1.
*ENDIF
         GO TO 50
C
   10 U=U-TARU
      IF(U.GT.0.) GOTO 25
C
C     CHOOSE FROM A REGION RU
C
*IF -DEF,AELIB5
   15    U1=.15342640972002*(1.+RANF(A))
*ENDIF
*IF DEF,AELIB5
   15    U1=.15342640972002*(1.+RANF())
*ENDIF
*IF -DEF,AELIB5
         U2=RANF(A)*ARU
*ENDIF
*IF DEF,AELIB5
         U2=RANF()*ARU
*ENDIF
         A=EXP(-U1)
         IF(A+U1-1..GE.U2) GOTO 20
            U1=XRU-U1
            IF(BRU.GE.U2) GOTO 20
               IF(A+.73575888234288/A-1.6931471805599.GE.U2) GOTO 20
                  GOTO 15
   20    X(II)=B+CL+U1
         GO TO 50
C
C     CHOOSE FROM A REGION RL
C
*IF -DEF,AELIB5
   25    U1=.19314718055995*RANF(A)
*ENDIF
*IF DEF,AELIB5
   25    U1=.19314718055995*RANF()
*ENDIF
*IF -DEF,AELIB5
         U2=RANF(A)*ARL
*ENDIF
*IF DEF,AELIB5
         U2=RANF()*ARL
*ENDIF
         A=EXP(-U1)
         IF(E4*A+U1-1.3862943611199.GE.U2) GOTO 30
            U1=XRL-U1
            IF(BRL.GE.U2) GOTO 30
               IF(E4*A+1./A-2.3862943611199.LT.U2) GOTO 25
   30 X(II)=1.+B+U1
C
   50 CONTINUE
      RETURN
      END
