*DECK RNORM
      SUBROUTINE RNORM(Z,N)
C
C **********************************************************************
C ******                                                          ******
C ******                         1-05-11                          ******
C ******                        CALCULATE                         ******
C ******     SEQUENCE OF NORMALLY DISTRIBUTED RANDOM NUMBERS      ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE:  THIS SUBROUTINE GENERATES A SEQUENCE OF NORMALLY         *
C *           DISTRIBUTED RANDOM NUMBERS WITH MEAN 0 AND VARIANCE 1.   *
C *           THE POSITIVE REGION OF THE STANDARD NORMAL DISTRIBUTION  *
C *           IS DIVIDED INTO 4 REGIONS, EACH OF WHICH IS SAMPLED IN   *
C *           PROPORTION TO ITS AREA.  AN ACCEPTANCE REJECTION         *
C *           TECHNIQUE IS USED WITHIN EACH REGION TO PRODUCE THE      *
C *           REQUIRED RANDOM NUMBER.  HALF OF THE TIME A NEGATIVE     *
C *           VALUE IS RETURNED INSTEAD OF A POSITIVE.  THE ALGORITHM  *
C *           IS DESCRIBED MORE FULLY IN AECL-7428 BY K. R. CHAPLIN    *
C *           AND C. A. WILLS                                          *
C *                                                                    *
C * ENTRY:    CALL RNORM(X, N)                                         *
C *           X(N)......REAL OUTPUT ARRAY OF NORMALLY DISTRIBUTED      *
C *                     RANDOM NUMBERS                                 *
C *           N.........INTEGER INPUT VARIABLE, NUMBER OF RANDOM       *
C *                     NUMBERS TO GENERATE                            *
C *                                                                    *
C * AUTHOR:   K. R. CHAPLIN                                            *
C *           WRITTEN AUGUST 1979                                      *
C *                                                                    *
C **********************************************************************
C
      REAL Z(1)
C
C A-- CONSTANT VALUE 1 / SQRT(2 * PI)
C B--POSITIVE VERTEX ON X AXIS OF THE TRAPEZOID
C B2--CONSTANT VALUE B ** 2
C C1--CONSTANT VALUE Y / (B - X) WHERE (X,Y) IS THE POSITIVE VERTEX OF
C     THE TRAPEZOID NOT ON THE X AXIS
C C2--CONSTANT VALUE Y * B / (B - X) WHERE (X,Y) IS THE POSITIVE VERTEX
C     OF THE TRAPEZOID NOT ON THE X AXIS
C
      DATA B/2.1140280833374 /,A/.39894228040143 /,C1/.209694057195486/
     ,,C2/.44329912582022/,B2/4.4691147371392/
C
*CALL STATF
C
C     U IS USED THROUGHOUT TO CHOOSE WHICH REGION  RNORM IS IN.
C     U IS THEN REUSED IN THE TRAPEZOID.
C     U0 IS USED IN THE TRAPEZOID BUT IT IS ALSO USED TO CHOOSE IF THE
C     VALUE RETURNED IS POSITIVE OR NEGATIVE
C
C     CHECK NUMBER OF RANDOM VALUES TO BE GENERATED
C
      IF (N .GT. 0) GOTO 5
C
      CALL SETBAD(X, 1, 1, 1)
      PRINT 1000
*IF -DEF,AELIB5
 1000 FORMAT(" *** VALUE OF N NOT GREATER THAN ZERO")
*ENDIF
*IF DEF,AELIB5
 1000 FORMAT(' *** VALUE OF N NOT GREATER THAN ZERO')
*ENDIF
      CALL TRACEB(0)
      RETURN
C
5     DO 100  II=1,N
*IF -DEF,AELIB5
      U=RANF(A)
*ENDIF
*IF DEF,AELIB5
      U=RANF()
*ENDIF
*IF -DEF,AELIB5
      U0=RANF(A)
*ENDIF
*IF DEF,AELIB5
      U0=RANF()
*ENDIF
C
C     CHOOSE FROM THE TRAPEZOID.
C
      IF(U .GE. .91954440570693) GO TO 10
         Z(II)=2.4037576569374 * (U0 + U* .82533928253692) -B
         GO TO 100
C
C     CHOOSE FROM THE TAIL.
C
10    IF(U .LT. .96548713121386 )GO TO 30
*IF -DEF,AELIB5
20       U1=RANF(A)
*ENDIF
*IF DEF,AELIB5
20       U1=RANF()
*ENDIF
         X=B2-2.*ALOG(U1)
*IF -DEF,AELIB5
         U2=RANF(A)
*ENDIF
*IF DEF,AELIB5
         U2=RANF()
*ENDIF
         IF(X*U2*U2 .GT. B2)GO TO 20
         X=SQRT(X)
         GO TO 80
C
C     CHOOSE FROM REGION 3.
C
30    IF(U .LT. .94856274812909)GO TO 50
*IF -DEF,AELIB5
40       U1=RANF(A)
*ENDIF
*IF DEF,AELIB5
40       U1=RANF()
*ENDIF
         X=B-.1620140416687*U1
         Y=A*EXP(-X*X*.5)-C2+X*C1
*IF -DEF,AELIB5
         U2=RANF(A)*.05513592720665
*ENDIF
*IF DEF,AELIB5
         U2=RANF()*.05513592720665
*ENDIF
         IF(Y .GT. U2)GO TO 80
         X=3.9040280833374-X
         IF(.05077522365025 .GT. U2)GO TO 80
         IF(A*EXP(-X*X*.5)-C2+X*C1 .GT. U2-Y)GO TO 80
         GO TO 40
50    IF(U .LT. .9258523337077)GO TO 70
*IF -DEF,AELIB5
60       U1=RANF(A)
*ENDIF
*IF DEF,AELIB5
60       U1=RANF()
*ENDIF
         X=.2897295736 + 1.5002704264*U1
         Y=A*EXP(-X*X*.5)-C2+X*C1
*IF -DEF,AELIB5
         U2=RANF(A)*.016270801
*ENDIF
*IF DEF,AELIB5
         U2=RANF()*.016270801
*ENDIF
         IF(Y .GT. U2)GO TO 80
         X=2.0797295736-X
         IF(.01243334561586 .GT. U2)GO TO 80
         IF(A*EXP(-X*X*.5)-C2+X*C1 .GT. U2-Y)GO TO 80
         GO TO 60
C
C     CHOOSE FROM REGION 2.
C
*IF -DEF,AELIB5
70    U1=RANF(A)
*ENDIF
*IF DEF,AELIB5
70    U1=RANF()
*ENDIF
         X=U1*.2897295736
*IF -DEF,AELIB5
         U2=RANF(A)
*ENDIF
*IF DEF,AELIB5
         U2=RANF()
*ENDIF
         IF(A*EXP(-X*X*.5)-.38254455604252  .LT. U2*.016397724358915)
     ,   GO TO 70
80    Z(II)=X
      IF(U0 .LT. .5)Z(II)=-Z(II)
100   CONTINUE
      RETURN
      END
