*DECK SCALE
      SUBROUTINE SCALE(X,AXX,NPTS,I)
C
C AELIB SUBROUTINE SCALE   2-1-66
C
C
C THIS SUBROUTINE EXAMINES THE DATA VALUES IN AN ARRAY TO DETERMINE
C A STARTING VALUE, EITHER MINIMUM OR MAXIMUM, AND A SCALING FACTOR,
C POSITIVE OR NEGATIVE, SUCH THAT -
C    1. THE SCALE ANNOTATION DRAWN BY THE AXIS SUBROUTINE AT EACH
C       DIVISION WILL PROPERLY REPRESENT THE RANGE OF REAL DATA VALUES
C       IN THE ARRAY     AND
C    2. THE DATA POINTS, WHEN PLOTTED BY THE LINE SUBROUTINE WILL FIT
C       IN THE GIVEN PLOTTING AREA
C DELTAV IS THE SCALING FACTOR GIVING THE NUMBER OF DATA UNITS PER INCH
C OF AXIS ADJUSTED TO BE AN INTERVAL OF 1,2,4,5, OR 8X10**N
C FIRSTV IS THE FIRST VALUE TO APPEAR ON THE AXIS AND IS SOME MULTIPLE
C OF DELTAV EQUAL TO OR OUTSIDE THE LIMITS OF THE DATA IN THE ARRAY
C DELTAV AND FIRSTV ARE STORED IN THE ARRAY AS DISCUSSED IN ARGUMENT
C LIST
C
C ARGUMENT LIST-
C    X - FIRST ELEMENT OF THE ARRAY OF DATA POINTS TO BE EXAMINED
C    AXX - LENGTH OF AXIS TO WHICH DATA IS TO BE SCALED . ONLY HAS
C          MEANING IF GREATER THAN 1.0 IN
C    NPTS - NUMBER OF DATA POINTS TO BE EXAMINED IN X
C    I - INTEGER WHOSE MAGNITUDE IS THE INCREMENT AT WHICH DATA VALUES
C        WILL BE EXAMINED FROM X. THE FIRST EXAMINED VALUE IS ALWAYS
C        THE FIRST ELEMENT OF X, THE SECOND IS X(1+IABS(I)), ETC.
C      - IF I IS POSITIVE, FIRSTV IS THE MINIMUM AND DELTAV IS POSITIVE
C      - IF I IS NEGATIVE, FIRSTV IS THE MAXIMUM AND DELTAV IS NEGATIVE
C      - FIRSTV IS STORED AT X(NPTS*IABS(I)+1)
C      - DELTAV IS STORED AT X((NPTS+1)*IABS(I)+1)
C
C ROUTINES CALLED - NONE
C
C AUTHOR - L EVANS       SEPTEMBER 1973
C REVISED - L EVANS         1979 MARCH
C         - TO CALCULATE BETTER SCALE FACTORS FOR CONSTANT DATA
C
C         - L EVANS   1981 MAY
C         - TO MODIFY SCALING ALGORITHM TO WORK ON DIFFICULT SCALING
C           CASES
C         - AND TO PRINT WARNING MESSAGE WHEN SCALING FAILS TO SPAN
C          WHOLE RANGE
C
C
C
      DIMENSION X(1)
      LOGICAL ALERPR
      REAL CINT(10)
      DATA CINT/1.0,2.0,4.0,5.0,8.0,10.0,20.0,30.0,40.0,50.0/
*CALL STATF
      INC=IABS(I)
      N=NPTS*INC
      XMIN=X(1)
      XMAX=X(1)
      NCINT=10
      IROUT=20166
C
C CALCULATE MAXIMUM AND MINIMUM
C
      DO 10 J=1,N,INC
      IF(X(J).LT.XMIN)XMIN=X(J)
 10   IF(X(J).GT.XMAX)XMAX=X(J)
C
C TO ENSURE PROPER TRUNCATION WHEN XMIN LT 0, SET RND=0 IF XMIN GE 0
C  AND RND=-1 IF XMIN LT 0
C
      RND=0
      IF(XMIN.LT.0) RND=-1.
C
C
C CALCULATE SCALE FACTOR AND CORRECT INTERVAL
      SCF=(XMAX-XMIN)/AXX
      IF(SCF.LE.0) GOTO 50
      EXP=ALOG10(SCF)
      IEXP=EXP
*IF -DEF,AELIB5
      IF((EXP.LT.0).A.(FLOAT(IEXP).NE.EXP)) IEXP=EXP-1
*ENDIF
*IF DEF,AELIB5
      IF((EXP.LT.0).AND.(FLOAT(IEXP).NE.EXP)) IEXP=EXP-1
*ENDIF
      TST=SCF/10.**IEXP
      DO 20 J=1,NCINT
      IF(CINT(J).GE.TST) GOTO 25
  20  CONTINUE
      J=NCINT
 25   SCF=CINT(J)*10.**IEXP
C
C DOUBLE CHECK TO MAKE SURE INTERVAL IS LONG ENOUGH
C
      XS=SCF*INT(XMIN/SCF+RND)
      XXS=XS+AXX*SCF
      IF(XXS.GE.XMAX) GOTO 30
      J=J+1
      IF(J.LE.NCINT) GOTO 25
C
C     SCALING UNABLE TO INCLUDE WHOLE RANGE
C     PRINT A MESSAGE AND CONTINUE EXECUTING.
C
      IF(.NOT.ALERPR(IROUT,0)) GOTO 30
         PRINT 1000,XMIN,XMAX,AXX
*IF -DEF,AELIB5
 1000    FORMAT(" ***SCALING UNABLE TO SPAN WHOLE RANGE ",/,
     ,          "    MIN = ",E12.5," MAX = ",E12.5," AXIS LEN = ",E12.5)
*ENDIF
*IF DEF,AELIB5
 1000    FORMAT(' ***SCALING UNABLE TO SPAN WHOLE RANGE ',/,
     ,          '    MIN = ',E12.5,' MAX = ',E12.5,' AXIS LEN = ',E12.5)
*ENDIF
         CALL TRACEB(0)
C
C CALCULATE THE STARTING VALUE
C CURRENT VALUE OF XS IS SCALED MINIMUM. NEW VALUE WILL  BE NEXT LOWEST
C INCREMENT POINT SUCH THAT ARRAY IS CENTERED
C SCLEN=SCALED RANGE OF ARRAY,J=(INTEGER) NUMBER OF EXCESS INCREMENTS
C AT EACH END OF ARRAY
C
   30 SCLEN=(XMAX-XS)/SCF
      J=(AXX-SCLEN)/2
      XS=XS-J*SCF
C
C KEEP XS SAME SIGN AS XMIN
      IF(XMIN*XS.LE.0)XS=0
C
C PUT FIRST VALUE AND SCALE FACTOR IN ARRAY AND THEN RETURN
C
 35   IF(I.GT.0) GOTO 40
      XS=XS+AXX*SCF
      SCF=-SCF
  40  X(N+1)=XS
      X(N+1+INC)=SCF
      RETURN
C
 50   IF(XMIN.GE.0) GOTO 60
C
C        DATA ARRAY VALUES ARE NEGATIVE WITH CONSTANT VALUE XMIN.
C        SCALE TO INTERVAL (2*XMIN,0)
         SCF=-2*XMIN/AXX
         XS=2*XMIN
         GOTO 35
 60   IF(XMIN.EQ.0) GOTO 70
C
C        DATA ARRAY VALUES ARE POSITIVE WITH CONSTANT VALUE XMIN.
C        SCALE TO INTERVAL (0,2*XMIN)
         SCF=2.*XMIN/AXX
         XS=0
         GOTO 35
 70   CONTINUE
C
C        DATA ARRAY VALUES ARE CONSTANT AT ZERO
C        SCALE TO(-1.,1.)
C
         SCF=2./AXX
         XS=-1.0
         GOTO 35
      END
