*DECK SETBAD
*IF -DEF,EXTERNAL
      SUBROUTINE SETBAD (A, NR, LR, LC)
C
C
C **********************************************************************
C ******                                                          ******
C ******   AELIB UTILITY SETBAD  -  INTERNAL VERSION    2-5-77    ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE : SETBAD SETS A VARIABLE OR AN ARRAY TO "BAD VALUES", I.E. *
C *           INDEFINITE.  IT IS INTENDED FOR AELIB ROUTINES AND IS TO *
C *           BE USED TO SET OUTPUT PARAMETERS OF THOSE ROUTINES TO    *
C *           "BAD VALUES" IN THE EVENT OF ERRORS FOR WHICH THESE      *
C *           PARAMETERS HAVE NO BEST VALUE.                           *
C *                                                                    *
C * ENTRY :   CALL SETBAD(A,NR,LR,LC)                                  *
C *                                                                    *
C *           A  - REAL INPUT VARIABLE OR ARRAY TO BE SET TO "BAD      *
C *                VALUES".                                            *
C *           NR - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF ROWS IN ARRAY A AS DIMENSIONED IN THE     *
C *                CALLING ROUTINE IF A IS AN ARRAY.  IF A IS A        *
C *                VARIABLE THEN NR = 1.                               *
C *           LR - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF ROWS IN A TO BE SET TO "BAD VALUES".      *
C *                IF A IS A VARIABLE LR = 1.                          *
C *           LC - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF COLUMNS IN A TO BE SET TO "BAD VALUES".   *
C *                IF A IS A VARIABLE LC = 1.                          *
C *                                                                    *
C * ADDITIONAL VARIABLE :                                              *
C *                                                                    *
C *           IROUT : INTEGER VARIABLE DEFINING THE ROUTINE NUMBER     *
C *                   FOR SETBAD TO BE USED IN ERROR MESSAGE CONTROL.  *
C *                                                                    *
C * ABNORMAL TERMINATION :                                             *
C *                                                                    *
C *           1. AN ATTEMPT TO ENTER SETBAD WITH NR, LR OR LC < 1      *
C *              RESULTS IN THE OUTPUT OF AN APPROPRIATE ERROR MESSAGE *
C *              CONTAINING THE CURRENT VALUES OF NR, LR AND LC.       *
C *              CONTROL IS THEN RETURNED TO THE CALLING ROUTINE.      *
C *              VARIABLE A OR THE ENTRIES IN ARRAY A ARE NOT SET      *
C *              "BAD".                                                *
C *                                                                    *
C * SPECIAL NOTE :                                                     *
C *                                                                    *
C *           THERE ARE TWO VERSIONS OF SETBAD IN THE AELIB PROGRAM    *
C *           LIBRARY UNDER THE DECK NAME SETBAD.  ONE IS THE INTERNAL *
C *           VERSION USED BY ROUTINES IN AELIB AND THE OTHER IS AN    *
C *           EXTERNAL VERSION THAT IS TO ACCOMPANY AELIB ROUTINES     *
C *           (WHICH REFERENCE SETBAD) BEING SENT TO ANOTHER SITE.     *
C *                                                                    *
C *           THE UPDATE DIRECTIVE "*COMPILE SETBAD" WILL CAUSE THE    *
C *           INTERNAL VERSION TO BE WRITTEN TO THE COMPILE FILE.      *
C *           WITH THE ADDITION OF THE DIRECTIVE "*DEFINE EXTERNAL"    *
C *           THE EXTERNAL VERSION OF SETBAD WILL BE WRITTEN TO THE    *
C *           COMPILE FILE.                                            *
C *                                                                    *
C * AUTHOR :  E.G. LONG                                                *
C *           WRITTEN NOVEMBER 1978.                                   *
C *                                                                    *
C * REVISION HISTORY :                                                 *
C *                                                                    *
C *           REVISED BY G.L. KLAWITTER, JANUARY, 1980 TO INTRODUCE    *
C *           NEW ERROR PROCESSING PROCEDURE.                          *
C *                                                                    *
C **********************************************************************
C
      REAL A(NR,1)
      INTEGER NR,LR,LC
      LOGICAL ALERPR
C
C     LIBRARY STATS CALL.
C
*CALL STATF
C
C     IROUT DEFINED. IROUT IS THE ROUTINE NUMBER OF SETBAD IN AECL-6076.
C
      IROUT = 20577
C
C     INPUT PARAMETER CHECK.  NR<1 OR LR<1 OR LC<1 IS INVALID.
C
      IF (NR.GE.1.AND.LR.GE.1.AND.LC.GE.1) GO TO 10
      IF (.NOT.ALERPR(IROUT,0)) RETURN
         PRINT 200,NR,LR,LC
         CALL TRACEB(0)
         RETURN
C
C     ASSIGN INDEFINITE VALUE TO ELEMENTS OF ARRAY A OR TO VARIABLE A.
C
   10 CONTINUE
      DO 30 I=1,LR
         DO 20 J=1,LC
*IF -DEF,AELIB5
            A(I,J) = 60000000000000000000B
*ENDIF
*IF DEF,AELIB5
            A(I,J) = O"60000000000000000000"
*ENDIF
   20    CONTINUE
   30 CONTINUE
      RETURN
C
C
*IF -DEF,AELIB5
  200 FORMAT(" ***INPUT PARAMETER ERROR.   NR<1, LR<1 OR LC<1 IS INVALID
     ,.",/4X,"NR=",I5,"  LR=",I5,"  LC=",I5)
*ENDIF
*IF DEF,AELIB5
  200 FORMAT(' ***INPUT PARAMETER ERROR.   NR<1, LR<1 OR LC<1 IS INVALID
     ,.',/4X,'NR=',I5,'  LR=',I5,'  LC=',I5)
*ENDIF
      END
*IF DEF,EXTERNAL
*ENDIF
*ENDIF
*IF DEF,EXTERNAL
      SUBROUTINE SETBAD(A,NR,LR,LC)
C
C **********************************************************************
C ******                                                          ******
C ******   AELIB UTILITY SETBAD  -  EXTERNAL VERSION    2-5-77    ******
C ******                                                          ******
C **********************************************************************
C *                                                                    *
C * PURPOSE :                                                          *
C *                                                                    *
C *           THIS IS A "DO-NOTHING" ROUTINE INTENDED TO ACCOMPANY     *
C *           AELIB ROUTINES SENT TO ANOTHER SITE.  IT REPLACES THE    *
C *           INTERNAL VERSION OF SETBAD WHICH IS USED TO SET          *
C *           VARIABLES OR ARRAYS, RETURNED BY AELIB ROUTINES, TO      *
C *           "BAD VALUES" IE. INDEFINITE IN THE EVENT OF AN ERROR     *
C *           WHICH LEAVES SAID VARIABLES OR ARRAYS WITH NO BEST VALUE.*
C *           INDEFINITES ARE SPECIAL BIT PATTERNS THAT ARE UNACCEPT-  *
C *           ABLE TO CDC ARITHMETIC UNITS AND CAUSE A JOB ABORT IF    *
C *           USED IN ARITHMETIC CALCULATIONS.  THE PARTICULAR         *
C *           INDEFINITE USED BY SETBAD IS 60000000000000000000B.      *
C *                                                                    *
C * ENTRY :   CALL SETBAD(A,NR,LR,LC)                                  *
C *                                                                    *
C *           A  - REAL INPUT VARIABLE OR ARRAY TO BE SET TO "BAD      *
C *                VALUES".                                            *
C *           NR - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF ROWS IN ARRAY A AS DIMENSIONED IN THE     *
C *                CALLING ROUTINE IF A IS AN ARRAY.  IF A IS A        *
C *                VARIABLE THEN NR = 1.                               *
C *           LR - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF ROWS IN A TO BE SET TO "BAD VALUES".      *
C *                IF A IS A VARIABLE LR = 1.                          *
C *           LC - INTEGER INPUT VARIABLE.                             *
C *              - NUMBER OF COLUMNS IN A TO BE SET TO "BAD VALUES".   *
C *                IF A IS A VARIABLE LC = 1.                          *
C *                                                                    *
C * AUTHOR :                                                           *
C *                                                                    *
C *           G.L. KLAWITTER, CRNL.                                    *
C *           MARCH 1980.                                              *
C *                                                                    *
C **********************************************************************
C
C
      RETURN
      END
*ENDIF
