*DECK SMOOTH
      SUBROUTINE SMOOTH(M,N,XD,YD,WD,RD,XN,FN,GN,IPRINT,W,IER)
************************************************************************
**    NOTE THAT SMOOTH IS WRITTEN IN MODULAR FORM USING THE           **
**    ADDITIONAL ROUTINE:                                             **
**       V60BA                                                        **
**    V60BA ACCOMPANIES SMOOTH ON THE AELIB SOURCE TAPE UNDER THE     **
**    DECK NAME SMOOTH.                                               **
************************************************************************
C       ***************************************************************
C       ***************************************************************
C
C   TITLE    SMOOTH CURVE FITTING
C
C   PURPOSE   TO CALCULATE A SMOOTH CURVE FIT TO GIVEN WEIGHTED
C            DATA THE OUTPUT ALLOWS EASY EVALUATION OF THE FITTED
C            FUNCTION AT ANY POINT IN THE DATA RANGE
C        SUBROUTINE SMOOTH IS THE HARWELL ROUTINE VCO3A
C        THE SUBROUTINE NAME HAS BEEN CHANGED FOR INSERTION IN AELIB
C        AND ARGUMENT CHECKING HAS BEEN ADDED
C        AELERR IS NOW USED FOR ISSUING OF ALL ERROR MESSAGES
C  ENTRY   CALL SMOOTH(M,N,XD,YD,WD,RD,XN,FN,GN,IPRINT,W,IER)
C
C             M    NUMBER OF DATA POINTS
C             XD   AN ARRAY OF MINIMUM LENGTH M, THE DATA POSITIONS
C             YD   AN ARRAY OF MINIMUM LENGTH M, THE DATA VALUES
C             WD   AN ARRAY OF MINIMUM LENGTH M, THE WEIGHTS
C           IPRINT  INTEGER SPECIFYING THE PRINTING REQUIRED
C                  IPRINT = 0   NO PRINTING,EXCEPT FOR ERROR MESSAGES
C                  IPRINT > 0  PRINTS DETAILS OF THE FINAL FIT ONLY
C                  IPRINT <   PRINT DETAILS OF THE FIT AT EVERY
C                             ITERATION USING NEW KNOTS
C             N    SHOULD BE SET TO AN OVERESTIMATE OF THE NUMBER OF
C                  KNOTS IN THE FINAL FIT
C                  THE DIMENSIONS OF THE ARRAYS XN,FN,GN,DN,THETA,W,
C                  DEPEND ON N(SEE BELOW FOR ESTIMATING N)
C                  ON A NORMAL RETURN N WILL CONTAIN THE NUMBER OF
C                  KNOTS USED IN THE FITTING FUNCTION S(X)
C             W    AN ARRAY OF LENGTH AT LEAST 7M+11N+6 WORDS,USED AS
C                  WORKING SPACE
C             IER  ERROR FLAG
C                IER = 0  NO ERRORS
C                IER = 1  NEXT ITERATION REQUIRES LARGER NUMBER
C                       OF KNOTS THEN N SPECIFIED
C                       ONLY ARRAY RD CONTAINS USEFULL INFORMATION
C                       THE RESIDUALS FOR THE FIT TO N KNOTS
C                IER = 2  SOME DATA POINTS WHICH WERE OUT OF ORDER
C                         HAVE BEEN REMOVED
C                IER = 3  INSUFFICIENT DATA
C                IER = 4   N MUST BE LARGER THEN 4
C                IER = 5  DATA POINT 1 MUST BE LESS THEN DATA POINT M
C                IER = 6 NUMBER OF DATA POINTS M IS LESS THAN 6
C    THE FOLLOWING ARGUMENTS ARE SET BY SMOOTH
C             XN   ARRAY OF LENGTH N, THE KNOT POSITIONS
C             FN   ARRAY OF LENGTH N, THE VALUES OF S(X) AT THE KNOTS
C             GN   ARRAY OF LENGTH N, THE VALUES OF S"(X) AT THE KNOTS
C             RD   AN ARRAY OF LENGTH M, SET BY SMOOTH TO CONTAIN
C                  THE RESIDUALS
C
C     ESTIMATING N
C      (1) LARGE NUMBER OF DATA POINTS FAIRLY UNIFORMLY DISTRIBUTED
C          OVER THEIR RANGE, THEN N=M/2 IS AN OVER-ESTIMATE
C      (2)  DATA CONCENTRATED IN A SMALL REGION OF THE RANGE MAY
C           REQUIRE N APPROXIMATELY EQUAL TO M
C      (3)  IT IS NOT IMPOSSIBLE FOR N TO BE GREATER THAN M
C           (BUT N MUST BE LESS THAN OR EQUAL TO 2*M)
      DIMENSION XD(1),YD(1),WD(1),RD(1),XN(1),FN(1),GN(1),W(N,3)
      LOGICAL ALERPR
C
C  IROUT IS AN INTEGER VARIABLE DEFINING THE ROUTINE NUMBER FOR SMOOTH
C  TO BE USED IN ERROR MESSAGE CONTROL.
C
      IROUT = 11200
*CALL  STATF
      IER=0
      IF(N.GE.5.AND.N.LE.2*M) GO TO 120
      IER = 4
      GO TO 150
  120 CONTINUE
      IF(M.GE.6) GO TO 125
      IER=6
      GO TO 150
  125 CONTINUE
      IF(XD(M).GT.XD(1)) GO TO 130
      IER = 5
      GO TO 150
 130  CONTINUE
      NDIMAX=N
C     OMIT DATA WITH ZERO WEIGHTS
      MM=1
      J=1
      DO 1 I=2,M
      IF (WD(I)) 3,2,3
    2 IF (I-M) 4,3,3
*IF -DEF,AELIB5
    4 W(J)=XD(I)
      W(J+1)=YD(I)
      W(J+2)=FLOAT(I)
*ENDIF
*IF DEF,AELIB5
    4 W(J,1)=XD(I)
      W(J+1,1)=YD(I)
      W(J+2,1)=FLOAT(I)
*ENDIF
      J=J+3
      GO TO 1
    3 MM=MM+1
      XD(MM)=XD(I)
      YD(MM)=YD(I)
      WD(MM)=WD(I)
    1 CONTINUE
      J=1
      K=MM
    7 IF (K-M) 5,6,6
    5 K=K+1
*IF -DEF,AELIB5
      XD(K)=W(J)
      YD(K)=W(J+1)
      WD(K)=W(J+2)
*ENDIF
*IF DEF,AELIB5
      XD(K)=W(J,1)
      YD(K)=W(J+1,1)
      WD(K)=W(J+2,1)
*ENDIF
      J=J+3
      GO TO 7
C     INITIALIZATION OF ITERATIONS
    6 JA=1
      IF (WD(1)) 9,8,9
    8 JA=2
    9 N=5
  100 XN(1)=XD(1)
      XN(5)=XD(MM)
      XN(3)=0.5*(XN(1)+XN(5))
      XN(2)=0.5*(XN(1)+XN(3))
      XN(4)=0.5*(XN(3)+XN(5))
      IW=7*MM+46
      DO 10 I=1,5
      J=IW+I
*IF -DEF,AELIB5
      W(J)=XN(I)
*ENDIF
*IF DEF,AELIB5
      W(J,1)=XN(I)
*ENDIF
   10 CONTINUE
      IP=-IPRINT
C     CALCULATE THE HISTOGRAMS FOR NEW SCALE FACTORS
   11 J=0
      K=1
      SA=0.
   12 SA=SA+WD(K)**2
      K=K+1
      IF (XD(K)-XD(1)) 12,12,13
   13 SA=(SA+0.5*WD(K)**2)/(XD(K)-XD(1))
   14 J=J+1
      IF (XD(K)-XN(J+1)) 15,15,16
   16 GN(J)=SA*(XN(J+1)-XN(J))
      GO TO 14
   15 GN(J)=SA*(XD(K)-XN(J))+0.5*WD(K)**2
   17 K=K+1
      IF (K-MM) 18,18,19
   18 IF (XD(K)-XN(J+1)) 20,20,21
   20 GN(J)=GN(J)+WD(K)**2
      GO TO 17
   21 SA=0.5*(WD(K-1)**2+WD(K)**2)/(XD(K)-XD(K-1))
      GN(J)=GN(J)-0.5*WD(K-1)**2+SA*(XN(J+1)-XD(K-1))
      GO TO 14
C     CALCULATE THE NEW SCALE FACTORS
   19 K=IW+2
*IF -DEF,AELIB5
      GN(1)=0.00025216*GN(1)*(W(K)-W(K-1))**8/(XN(2)-XN(1))
*ENDIF
*IF DEF,AELIB5
      GN(1)=0.00025216*GN(1)*(W(K,1)-W(K-1,1))**8/(XN(2)-XN(1))
*ENDIF
      DO 22 J=3,N
*IF -DEF,AELIB5
      IF (XN(J)-W(K)) 23,23,24
*ENDIF
*IF DEF,AELIB5
      IF (XN(J)-W(K,1)) 23,23,24
*ENDIF
   24 K=K+1
*IF -DEF,AELIB5
   23 GN(J-1)=0.00025216*GN(J-1)*(W(K)-W(K-1))**8/(XN(J)-XN(J-1))
*ENDIF
*IF DEF,AELIB5
   23 GN(J-1)=0.00025216*GN(J-1)*(W(K,1)-W(K-1,1))**8/(XN(J)-XN(J-1))
*ENDIF
      GN(J-2)=ALOG(GN(J-2)+GN(J-1))
   22 CONTINUE
      NN=N-2
      HS=1.386294
      DO 97 J=2,NN
      GN(J)=AMIN1(GN(J),GN(J-1)+HS)
   97 CONTINUE
      J=NN-1
   98 GN(J)=AMIN1(GN(J),GN(J+1)+HS)
      J=J-1
      IF (J) 99,99,98
   99 DO 25 J=3,N
      W(J-1,2)  =SQRT(EXP(GN(J-2))/(XN(J)-XN(J-2)))
   25 CONTINUE
C     CALCULATE THE SPLINE APPROXIMATION WITH CURRENT KNOTS
      CALL VB06A (MM,N,XD,YD,WD,RD,XN,FN,GN,W(1,1),W(1,2),IP,W(1,3),IER)
      IF(IER.EQ.3) RETURN
C     APPLY STATISTICAL TEST FOR EXTRA KNOTS
      J=IW+1
      JJ=0
      K=JA
      TMAX=0.
      IIS=1
   26 KC=-1
      SW=0.
      SR=0.
      RP=0.
      J=J+1
      JJ=JJ+1
*IF -DEF,AELIB5
      W(JJ)=0.
   27 IF (W(J)-XD(K)) 28,29,30
*ENDIF
*IF DEF,AELIB5
      W(JJ,1)=0.
   27 IF (W(J,1)-XD(K)) 28,29,30
*ENDIF
   30 KC=KC+1
      SW=SW+RD(K)**2
      SR=SR+RP*RD(K)
      RP=RD(K)
      K=K+1
      GO TO 27
   29 IF (WD(K)) 31,28,31
   31 KC=KC+1
      SW=SW+RD(K)**2
      SR=SR+RP*RD(K)
   28 IF (SR) 32,32,33
   33 SW=(SW/SR)**2
      RP=SQRT(SR/FLOAT(KC))
      IF (FLOAT(KC)-SW) 32,32,34
*IF -DEF,AELIB5
   34 GO TO (35,36,37),IIS
*ENDIF
*IF DEF,AELIB5
   34 GO TO (35,36,37),IIS
      CALL GOTOER
*ENDIF
   35 PRP=RP
      IIS=3
      IF (FLOAT(KC)-2.*SW) 38,38,39
*IF -DEF,AELIB5
   37 W(JJ-1)=PRP
*ENDIF
*IF DEF,AELIB5
   37 W(JJ-1,1)=PRP
*ENDIF
      TMAX=AMAX1(TMAX,PRP)
   39 IIS=2
*IF -DEF,AELIB5
   36 W(JJ)=RP
*ENDIF
*IF DEF,AELIB5
   36 W(JJ,1)=RP
*ENDIF
      TMAX=AMAX1(TMAX,RP)
      GO TO 38
   32 IIS=1
*IF -DEF,AELIB5
   38 IF (W(J)-XN(N)) 26,40,40
*ENDIF
*IF DEF,AELIB5
   38 IF (W(J,1)-XN(N)) 26,40,40
*ENDIF
C     TEST WHETHER ANOTHER ITERATION IS REQUIRED
   40 IF (TMAX) 41,41,42
C     CALCULATE NEW TREND ARRAY, INCLUDING LARGER TRENDS ONLY
   42 TMAX=0.5*TMAX
      I=0
      J=1
      JW=1
      K=IW+1
*IF -DEF,AELIB5
      W(JW,2)=W(K)
*ENDIF
*IF DEF,AELIB5
      W(JW,2)=W(K,1)
*ENDIF
   43 I=I+1
      K=K+1
*IF -DEF,AELIB5
      IF (W(I)-TMAX) 44,44,45
*ENDIF
*IF DEF,AELIB5
      IF (W(I,1)-TMAX) 44,44,45
*ENDIF
   44 JW=JW+1
*IF -DEF,AELIB5
      W(JW,2)=W(K)
*ENDIF
*IF DEF,AELIB5
      W(JW,2)=W(K,1)
*ENDIF
   46 FN(J)=0.
      J=J+1
*IF -DEF,AELIB5
      IF (W(K)-XN(J)) 47,47,46
*ENDIF
*IF DEF,AELIB5
      IF (W(K,1)-XN(J)) 47,47,46
*ENDIF
   45 JW=JW+2
*IF -DEF,AELIB5
      W(JW-1,2)=0.5*(W(K-1)+W(K))
      W(JW,2)=W(K)
*ENDIF
*IF DEF,AELIB5
      W(JW-1,2)=0.5*(W(K-1,1)+W(K,1))
      W(JW,2)=W(K,1)
*ENDIF
      IF(XN(J+1)-W(JW-1,2)) 46,46,48
   48 FN(J)=1.
      J=J+1
   47 IF (J-N) 43,49,49
C     MAKE KNOT SPACINGS BE USED FOUR TIMES
   49 IK=1
      KL=1
      FN(2)=AMAX1(FN(1),FN(2))
      GO TO 102
   50 K=KL+3
   51 IF (FN(K)) 52,52,53
   52 K=K-1
      IF (K-KL) 74,74,51
   53 K=K-1
      FN(K)=1.
      IF (K-KL) 74,74,53
 102  K=KL+3
   54 K=K+1
      IF (K-N) 55,56,56
   55 IF (XN(K+1)-XN(K)-1.5*(XN(K)-XN(K-1))) 54,54,56
   56 KU=K
      FN(K-2)=AMAX1(FN(K-2),FN(K-1))
   57 KKU=K
   58 K=K-1
      IF (K-KL) 59,59,60
   60 IF (XN(K)-XN(K-1)-1.5*(XN(K+1)-XN(K))) 58,58,61
   61 FN(K+1)=AMAX1(FN(K),FN(K+1))
   59 KKL=K
      KZ=4
      IF (FN(K)) 62,62,63
   63 K=K+1
      IF (K-KKU) 64,65,65
   64 IF (FN(K)) 66,66,63
   66 KZ=0
   62 KZ=KZ+1
      K=K+1
      IF (K-KKU) 67,65,65
   67 IF (FN(K)) 62,62,68
   68 IF (KZ-3) 69,69,70
   69 J=K-KZ
   71 FN(J)=1.
      J=J+1
      IF (J-K) 71,63,63
   70 IF (K+1-KKU) 72,65,65
   72 K=K+1
      FN(K)=1.
      GO TO 63
 65   IF (KL-KKL) 73,50,50
   73 FN(KKL-2)=AMAX1(FN(KKL-2),FN(KKL+1))
      FN(KKL-1)=AMAX1(FN(KKL-1),FN(KKL+3))
   75 K=KKL-4
   78 IF (FN(K)) 76,76,77
   76 K=K+1
      IF (K-KKL) 78,79,79
   77 FN(K)=1.
      K=K+1
      IF (K-KKL) 77,79,79
*IF -DEF,AELIB5
   79 GO TO (57,80),IK
*ENDIF
*IF DEF,AELIB5
   79 GO TO (57,80),IK
      CALL GOTOER
*ENDIF
   74 IF (KU-N) 81,82,82
   81 KL=KU
      FN(KL+1)=AMAX1(FN(KL+1),FN(KL-2))
      FN(KL)=AMAX1(FN(KL),FN(KL-4))
      GO TO 102
   82 IK=2
      KKL=N
      GO TO 75
C     INSERT EXTRA KNOTS FOR NEW APPROXIMATION
   80 DO 83 J=1,N
      GN(J)=XN(J)
   83 CONTINUE
      NN=1
      DO 84 J=2,N
      IF (FN(J-1)) 85,85,86
   86 NN=NN+1
      XN(NN)=0.5*(GN(J-1)+GN(J))
   85 NN=NN+1
      XN(NN)=GN(J)
   84 CONTINUE
      IF(NN-NDIMAX) 101,110,110
  110 IER=1
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 150
      PRINT 1000,NN
*IF -DEF,AELIB5
 1000 FORMAT(" ***ESTIMATE FOR N IS TOO SMALL-NEXT ITERATION REQUIRES",I
     $10,"KNOTS")
*ENDIF
*IF DEF,AELIB5
 1000 FORMAT(' ***ESTIMATE FOR N IS TOO SMALL-NEXT ITERATION REQUIRES',I
     $10,'KNOTS')
*ENDIF
      CALL TRACEB(0)
      GO TO 150
 101  CONTINUE
      N=NN
      IW=7*MM+8*N+6
      DO 87 J=1,JW
      I=IW+J
*IF -DEF,AELIB5
      W(I)=W(J,2)
*ENDIF
*IF DEF,AELIB5
      W(I,1)=W(J,2)
*ENDIF
   87 CONTINUE
      GO TO 11
C     RESTORE DATA WITH ZERO WEIGHTS
   41 IF (MM-M) 88,89,89
   89 IF (IPRINT) 90,90,91
   88 J=-2
      K=MM
      IF(IER.NE.2) GO TO 92
      M=MM
      GO TO 89
   92 J=J+3
      K=K+1
*IF -DEF,AELIB5
      W(J)=XD(K)
      W(J+1)=YD(K)
      W(J+2)=WD(K)
*ENDIF
*IF DEF,AELIB5
      W(J,1)=XD(K)
      W(J+1,1)=YD(K)
      W(J+2,1)=WD(K)
*ENDIF
      IF (K-M) 92,93,93
*IF -DEF,AELIB5
   93 I=INT(W(J+2)+0.5)
*ENDIF
*IF DEF,AELIB5
   93 I=INT(W(J+2,1)+0.5)
*ENDIF
   94 IF (K-I) 95,95,96
   96 XD(K)=XD(MM)
      YD(K)=YD(MM)
      WD(K)=WD(MM)
      K=K-1
      MM=MM-1
      GO TO 94
*IF -DEF,AELIB5
   95 XD(K)=W(J)
      YD(K)=W(J+1)
*ENDIF
*IF DEF,AELIB5
   95 XD(K)=W(J,1)
      YD(K)=W(J+1,1)
*ENDIF
      WD(K)=0.
      K=K-1
      J=J-3
      IF (J) 91,91,93
   91 CALL VB06A (M,N,XD,YD,WD,RD,XN,FN,GN,W(1,1),W(1,2),IABS(IPRINT),
     $W(1,3),IER)
   90 RETURN
C  SET  ALL   ARRAYS TO + INDEFINATE EXCEPT IN THE CASE OF IER=1
C        THEN THE RESIDUALS(RD ARRAY) IS OK
 150  DO 160 I=1,N
*IF -DEF,AELIB5
      XN(I)=17770000000000000000B
*ENDIF
*IF DEF,AELIB5
      XN(I)=O"17770000000000000000"
*ENDIF
      FN(I)=XN(I)
      GN(I)=XN(I)
 160  CONTINUE
      IF(IER.NE.4) GO TO 190
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 190
      PRINT 2000,2*M
*IF -DEF,AELIB5
 2000 FORMAT(" ***INITIAL ESTIMATE FOR N IS LESS THAN 5 OR IS GREATER TH
     ,AN",I10,/,4X,"(TWICE THE NUMBER OF DATA POINTS)")
*ENDIF
*IF DEF,AELIB5
 2000 FORMAT(' ***INITIAL ESTIMATE FOR N IS LESS THAN 5 OR IS GREATER TH
     ,AN',I10,/,4X,'(TWICE THE NUMBER OF DATA POINTS)')
*ENDIF
      CALL TRACEB(0)
  190 CONTINUE
      IF(IER.NE.5) GO TO 210
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 210
      PRINT 3000
*IF -DEF,AELIB5
 3000 FORMAT(" ***ABSCISSA OF FIRST DATA POINT MUST BE LESS THAN ABSCISS
     $A OF LAST DATA POINT")
*ENDIF
*IF DEF,AELIB5
 3000 FORMAT(' ***ABSCISSA OF FIRST DATA POINT MUST BE LESS THAN ABSCISS
     $A OF LAST DATA POINT')
*ENDIF
      CALL TRACEB(0)
  210 CONTINUE
      IF(IER.NE.6) GO TO 230
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 230
      PRINT 4000
*IF -DEF,AELIB5
 4000 FORMAT (" ***NUMBER OF DATA POINTS M IS LESS THAN 6")
*ENDIF
*IF DEF,AELIB5
 4000 FORMAT (' ***NUMBER OF DATA POINTS M IS LESS THAN 6')
*ENDIF
      CALL TRACEB(0)
  230 CONTINUE
      IF(IER.EQ.1) RETURN
      DO 170 I=1,M
*IF -DEF,AELIB5
      RD(I)=17770000000000000000B
*ENDIF
*IF DEF,AELIB5
      RD(I)=O"17770000000000000000"
*ENDIF
 170  CONTINUE
      RETURN
      END
      SUBROUTINE VB06A (M,N,XD,YD,WD,RD,XN,FN,GN,DN,THETA,IPRINT,W,IER)
      LOGICAL SWITCH
      DIMENSION XD(1),YD(1),WD(1),RD(1),XN(1),FN(1),GN(1),DN(1),
     1THETA(1),W(1)
      LOGICAL ALERPR
      IROUT = 11200
C     RESERVE THE FIRST FIFTEEN LOCATIONS OF W FOR
C     COMPUTED VALUES AND THIRD DERIVATIVES OF B-SPLINES
      W(1)=0.
      W(7)=0.
C     THEN SET THE KNOT POSITIONS INCLUDING SIX OUTSIDE THE RANGE
C     AND DELETE ANY KNOTS NOT IN STRICTLY ASCENDING ORDER
      I=1
      NN=1
   10 W(NN+18)=XN(I)
   20 IF (I.GE.N) GO TO 30
      I=I+1
      IF (XN(I).LE.W(NN+18)) GO TO 20
      NN=NN+1
      XN(NN)=XN(I)
      THETA(NN)=THETA(I)
      GO TO 10
   30 IF (NN.GE.N) GO TO 50
      N=NN
C   THE FOLLOWING PRINT STATEMENT REMOVED THESE ERRORS ARE CHECKED IN
C       THE ROUTINE SMOOTH
C     PRINT 40
C  40 FORMAT (//5X,"SOME KNOTS HAVE BEEN DELETED BY VB06A")
   50 IF (NN.GE.2) GO TO 70
C   THE FOLLOWING PRINT STATEMENT REMOVED THESE ERRORS ARE CHECKED IN
C       THE ROUTINE SMOOTH
C     PRINT 60
C  60 FORMAT (//5X,"THE EXECUTION OF VB06A HAS BEEN STOPPED ",
C    1"BECAUSE THERE ARE TOO FEW KNOTS")
      GO TO 770
   70 DO 80 I=1,3
      W(19-I)=2.*W(20-I)-W(21-I)
   80 W(NN+I+18)=2.*W(NN+I+17)-W(NN+I+16)
C     DELETE ANY DATA POINTS THAT ARE NOT IN ASCENDING ORDER
      I=1
      MM=1
      XD(MM)=AMIN1(AMAX1(XD(1),XN(1)),XN(NN))
   90 IF (I.GE.M) GO TO 100
      I=I+1
      IF (XD(I).LT.XD(MM)) GO TO 90
      MM=MM+1
      XD(MM)=AMIN1(XD(I),XN(NN))
      YD(MM)=YD(I)
      WD(MM)=WD(I)
      GO TO 90
  100 IF (MM.GE.M) GO TO 120
      M=MM
      IER=2
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 110
      PRINT 5000
*IF -DEF,AELIB5
 5000 FORMAT(" ***SOME DATA POINTS HAVE BEEN DELETED")
*ENDIF
*IF DEF,AELIB5
 5000 FORMAT(' ***SOME DATA POINTS HAVE BEEN DELETED')
*ENDIF
      CALL TRACEB(0)
  110 CONTINUE
C     COMPUTE THE THIRD DERIVATIVES OF THE B-SPLINES AT XN(1)
  120 SWITCH=.TRUE.
      IN=1
      GO TO 280
  130 DO 140 I=2,5
      W(I+6)=W(I)
  140 W(I+10)=W(I)
C     INITIALIZE THE FACTORIZATION OF THE LEAST SQUARES MATRIX
      KS=NN+22
      KU=KS+22
      DO 150 I=KS,KU
  150 W(I)=0.
      IM=1
      IN=2
      KK=KS
      SWITCH=.FALSE.
C     TEST WHETHER A DATA POINT OR A KNOT DEFINES THE NEXT EQUATION
  160 KR=KK
      NC=5
      IF (IM.GT.MM) GO TO 260
      IF(XD(IM).GT.XN(IN)) GO TO 260
C     CALCULATE THE VALUES OF THE B-SPLINES AT XD(IM)
  170 W(2)=1./(W(IN+18)-W(IN+17))
      DO 190 J=2,4
      I=J+1
      W(I)=0.
  180 II=IN+I+16
      W(I)=((XD(IM)-W(II-J))*W(I-1)+(W(II)-XD(IM))*W(I))/(W(II)-W(II-J))
      I=I-1
      IF (I.GE.2) GO TO 180
  190 CONTINUE
      IF (SWITCH) GO TO 540
C     COMPLETE THE EQUATION OF THE DATA POINT
      W(6)=YD(IM)
      C=WD(IM)**2
      IM=IM+1
C     REVISE NC PRIOR TO THE NEXT GIVENS TRANSFORMATION
  200 NC=NC-1
      IF (NC.LE.0) GO TO 160
C     MAKE THE GIVENS TRANSFORMATION
  210 ALPHA=C*W(2)
      IF (ALPHA.NE.0.) GO TO 230
      DO 220 I=1,NC
  220 W(I+1)=W(I+2)
      GO TO 250
  230 DSTAR=W(KR)+ALPHA*W(2)
      BETA=W(KR)/DSTAR
      C=BETA*C
      GAMMA=ALPHA/DSTAR
      W(KR)=DSTAR
      DSTAR=W(2)
      DO 240 I=1,NC
      W(I+1)=W(I+2)-DSTAR*W(KR+I)
  240 W(KR+I)=BETA*W(KR+I)+GAMMA*W(I+2)
  250 KR=KR+7
      GO TO 200
C     ADVANCE THE CURRENT ROW OF THE UPPER TRIANGULAR MATRIX
  260 IF(IN.GE.NN) GO TO 330
      W(KK+28)=0.
      DO 270 I=4,28,6
      W(KK+I+1)=W(KK+I)
  270 W(KK+I)=0.
      KK=KK+7
C     CALCULATE THE THIRD DERIVATIVES OF THE B-SPLINES AT XN(IN+)
  280 W(2)=6./(W(IN+19)-W(IN+18))
      DO 300 J=2,4
      I=J+1
      W(I)=0.
  290 II=IN+I+17
      W(I)=(W(I-1)-W(I))/(W(II)-W(II-J))
      I=I-1
      IF (I.GE.2) GO TO 290
  300 CONTINUE
      IF (SWITCH) GO TO 130
C     SET UP THE EQUATION FOR THE SMOOTHING TERM AT A KNOT
      ALPHA=0.
      I=6
  310 W(I)=W(I-1)-ALPHA
      I=I-1
      IF (I.LE.1) GO TO 320
      ALPHA=W(I+6)
      W(I+6)=W(I)
      GO TO 310
  320 C=THETA(IN)**2
      IN=IN+1
      GO TO 210
C     BACK-SUBSTITUTE TO OBTAIN THE MULTIPLIERS OF THE B-SPLINES
  330 KU=KK
      KUU=KU+21
      KB=KS+7
      IBS=5
      DO 335 I=3,6
      KK=KK+7
  335 W(KK-2)=W(KK-I)
  340 KK=KUU+IBS
      KR=KK
  350 KK=KK-7
      K=KK
      J=KK-IBS
  360 J=J+1
      K=K+7
      W(KK)=W(KK)-W(J)*W(K)
      IF (K.LT.KR) GO TO 360
      KR=MIN0(KR,KK+21)
      IF (KK.GT.KB) GO TO 350
      IF (IBS-6) 370,470,500
  370 SWITCH=.TRUE.
      ALPHA=1.
      DO 380 K=KS,KUU,7
  380 ALPHA=AMIN1(ALPHA,W(K))
      IF (ALPHA.LE.0.) GO TO 592
      IF (NN.LE.2) GO TO 600
C     NEXT A SPLINE WITH FIXED VALUES OF S"""(XN(1)) AND S"""(XN(N))
C     IS CALCULATED TO OBTAIN THE OPTIMAL EXTREME VALUES OF S"""(X).
C     SET THE COEFFICIENTS OF S"""(X(1)) AND S"""(X(N))
      KR=KS
      KK=KU
      DO 410 I=8,11
      W(KK+4)=W(I)
      KK=KK+7
      W(KR+6)=W(I+4)
  410 KR=KR+7
      DO 420 K=KR,KUU,7
  420 W(K+6)=0.
      IBS=6
      KRR=KS
C     BACK-SUBSTITUTE USING THE TRANSPOSE OF THE UPPER TRIANGULAR
C     MATRIX TO CALCULATE THE RESIDUALS OF EACH EXTRA SPLINE
  425 I=KRR+7
      KR=KRR
      DO 450 KK=I,KUU,7
      J=KK
      K=KK
  430 K=K-7
      J=J-6
      W(KK+IBS)=W(KK+IBS)-W(J)*W(K+IBS)
      IF (K.GT.KR) GO TO 430
  450 KR=MAX0(KR,KK-21)
      DO 460 KK=KRR,KUU,7
  460 W(KK+IBS)=W(KK+IBS)/W(KK)
      IF (IBS.EQ.6) GO TO 340
      DO 465 K=KS,KUU,7
      W(K+7)=0.
  465 IF (K.GE.KU) W(K+7)=W(K+4)
      IBS=7
      GO TO 340
C     BACK-SUBSTITUTE TO GIVE THE VECTOR FOR S"""(X(N))
  470 IBS=4
      KRR=KU
      GO TO 425
C     AT EACH DATA POINT CALCULATE THE RESIDUALS OF THREE SPLINES
  500 DO 520 I=6,10
  520 W(I)=0.
      IN=2
      KK=KS
      DO 580 IM=1,MM
  530 IF (XD(IM).LE.XN(IN)) GO TO 170
      IN=IN+1
      KK=KK+7
      GO TO 530
  540 KR=KK
      W(11)=YD(IM)
      W(12)=0.
      W(13)=0.
      DO 560 I=2,5
      DO 550 J=5,7
  550 W(J+6)=W(J+6)-W(I)*W(KR+J)
  560 KR=KR+7
      DO 570 I=11,13
  570 W (I)=WD(IM)*W(I)
C     FORM THE NORMAL EQUATIONS TO FIX S"""(X) AT THE ENDS OF THE RANGE
      DO 580 I=12,13
      K=I+I-29
      DO 580 J=11,I
  580 W(K+J)=W(K+J)+W(I)*W(J)
C     CALCULATE THE B-SPLINE MULTIPLIERS OF THE REQUIRED FIT
      ALPHA=(W(8)*W(9)-W(6)*W(10))/(W(7)*W(10)-W(9)**2)
      BETA=(-W(8)-ALPHA*W(9))/W(10)
      DO 590 K=KS,KUU,7
  590 W(K+5)=W(K+5)+ALPHA*W(K+6)+BETA*W(K+7)
      GO TO 600
C     PRINT A DIAGNOSTIC MESSAGE IF THERE IS INSUFFICIENT DATA
  592 IER=3
      IF(.NOT.ALERPR(IROUT,IER)) GO TO 595
      PRINT 6000
*IF -DEF,AELIB5
 6000 FORMAT(" ***INSUFFICIENT DATA TO DEFINE THE SPLINE UNIQUELY-RESULT
     $UNRELIABLE")
*ENDIF
*IF DEF,AELIB5
 6000 FORMAT(' ***INSUFFICIENT DATA TO DEFINE THE SPLINE UNIQUELY-RESULT
     $UNRELIABLE')
*ENDIF
      CALL TRACEB(0)
  595 CONTINUE
C     CALCULATE THE REQUIRED VALUES OF S(X) AND S"(X) AT THE KNOTS
  600 KK=KS
      DO 620 IN=1,NN
      FN(IN)=0.
      GN(IN)=0.
      W(2)=(W(IN+19)-W(IN+18))/((W(IN+19)-W(IN+17))*(W(IN+19)-W(IN+16)))
      W(3)=(W(IN+18)-W(IN+17))/((W(IN+19)-W(IN+17))*(W(IN+20)-W(IN+17)))
      W(4)=0.
      DO 610 I=1,3
      J=IN+I+14
      ALPHA=((XN(IN)-W(J))*W(I)+(W(J+4)-XN(IN))*W(I+1))/(W(J+4)-W(J))
      FN(IN)=FN(IN)+ALPHA*W(KK+5)
      BETA=3.*(W(I)-W(I+1))/(W(J+4)-W(J))
      GN(IN)=GN(IN)+BETA*W(KK+5)
  610 KK=KK+7
  620 KK=KK-14
C     CALCULATE THE THIRD DERIVATIVE DISCONTINUITIES OF THE SPLINE
      DN(1)=0.
      IN=1
      IM=1
  630 H=XN(IN+1)-XN(IN)
      ALPHA=FN(IN)-FN(IN+1)+H*GN(IN)
      BETA=ALPHA+FN(IN)-FN(IN+1)+H*GN(IN+1)
      DN(IN+1)=6.*BETA/H**3
      DN(IN)=DN(IN+1)-DN(IN)
      IN=IN+1
C     CALCULATE THE RESIDUALS AT THE DATA POINTS
  640 IF (IM.GT.MM) GO TO 650
      IF (XD(IM).GT.XN(IN)) GO TO 630
      C=(XD(IM)-XN(IN-1))/H
      RD(IM)=YD(IM)-C*FN(IN)+(C-1.)*(FN(IN-1)+C*(ALPHA-C*BETA))
      IM=IM+1
      GO TO 640
  650 IF (IN.LT.NN) GO TO 630
C     PROVIDE PRINTING IF REQUESTED
      IF (IPRINT.LE.0) GO TO 770
      PRINT 660
*IF -DEF,AELIB5
  660 FORMAT (1H1,35X,"SPLINE APPROXIMATION OBTAINED BY VB06A"//4X,"I",
     19X,"XN(I)",18X,"FN(I)",18X,"GN(I)",13X,"3RD DERIV CHANGE",
     211X,"THETA(I)"//)
*ENDIF
*IF DEF,AELIB5
  660 FORMAT (1H1,35X,'SPLINE APPROXIMATION OBTAINED BY VB06A'//4X,'I',
     19X,'XN(I)',18X,'FN(I)',18X,'GN(I)',13X,'3RD DERIV CHANGE',
     211X,'THETA(I)'//)
*ENDIF
      I=1
  670 PRINT 680,I,XN(I),FN(I),GN(I)
  680 FORMAT (I5,5E23.14)
  690 I=I+1
      IF (I-NN) 700,670,710
  700 PRINT 680,I,XN(I),FN(I),GN(I),DN(I),THETA(I)
      GO TO 690
  710 PRINT 720
*IF -DEF,AELIB5
  720 FORMAT (///4X,"I",9X,"XD(I)",18X,"YD(I)",18X,"WD(I)",19X,"FIT",
     118X,"RESIDUAL"//)
*ENDIF
*IF DEF,AELIB5
  720 FORMAT (///4X,'I',9X,'XD(I)',18X,'YD(I)',18X,'WD(I)',19X,'FIT',
     118X,'RESIDUAL'//)
*ENDIF
      IN=2
      DO 760 IM=1,MM
  730 IF (XD(IM).LE.XN(IN)) GO TO 750
      IN=IN+1
      IF (SWITCH) GO TO 730
      SWITCH=.TRUE.
      PRINT 740
  740 FORMAT (5X)
      GO TO 730
  750 C=YD(IM)-RD(IM)
      PRINT 680,IM,XD(IM),YD(IM),WD(IM),C,RD(IM)
  760 SWITCH=.FALSE.
  770 RETURN
      END
