*DECK SPARSE
      SUBROUTINE SPARSE(A,IND,IW,N,NP,G,U,IA)
************************************************************************
**                                                                    **
**    NOTE THAT SPARSE IS WRITTEN IN MODULAR FORM USING THE           **
**    ADDITIONAL ROUTINE:                                             **
**       MSCALE                                                       **
**    MSCALE ACCOMPANIES ROUTINE SPARSE IN AELIB SOURCE UNDER         **
**    DECK NAME SPARSE                                                **
**                                                                    **
************************************************************************
C
C
C     SPARSE MATRIX ANALYSIS ROUTINE.  THIS ROUTINE ANALYSES A PACKED
C       MATRIX A AND PREPARES IT FOR SUBSEQUENT OPERATIONS IN SPARSEB
C       CREATING EXTRA NON ZERO TERMS IF REQUIRED FOR EFFICIENT OPERATIO
C
C     A       ARRAY DIMENSION IA HOLDS THE PACKED SPARSE MATRIX
C     IA  THE NUMBER OF ELEMENTS IN A,THE ROUTINE MAY CREATE A FEW EXTRA
C     IW      ARRAY DIMENSION NP,13 HOLDS THE INDICES TO OPERATE ON A
C     IND     ARRAY DIMENSION IA,2 HOLDING INDICES TO REFERENCE  'A'
C     N       THE NUMBER OF ROWS IN THE FULL MATRIX
C     NP  FIRST DIMENSION OF IW AT LEAST N+1
C     G       OUTPUT PARAMETER INDICATING ERROR
C     U       PIVOT CHOICE CONTROL PARAMETER
C
C      MATRIX A(I,J) IS STORED BY COLUMNS J IE A11 A31 A22 A42 A33  ETC
C     IND CONTAINS ROW NUMBERS I ABOVE  1 3 2 4 3 ETC
C     IW  CONTAINS  THE POSITION OF THE STARTT OF EACH COLUMN J _ 1,3,5
C     IW(N+1) HAS THE NUMBER OF NON ZEROS IN A AND IRN
C
C     ADAPTED FROM THE HARWELL ROUTINE MA18A/D AERE-R-6844 [ REID ]
C
C      M CARVER  DEC 1974
C
C     EPS IS THE RELATIVE ACCURACY OF FLOATING-POINT COMPUTATION.
C     MATRIX ELEMENTS ARE HELD IN A(K),K=1,2,...,KA.
C      ON ENTRY IND(K,1) HOLDS THE ROW NUMBER OF THE ELEMENT HELD IN
C A(K).  IN THE MAIN BODY OF THE SUBROUTINE IND(K,1),IND(K,2) HOLD THE
C ADDRESS OF THE NEXT ELEMENT IN THE ROW/COLUMN IF THERE IS ONE AND
C FOR THE LAST ELEMENT IN THE ROW/COLUMN HOLD (IA+ THE ROW/COLUMN
C NUMBER).  THESE NUMBERS ARE NEGATED IF THEY POINT TO ELEMENTS THAT
C HAVE BEEN IN A PIVOTAL COLUMN/ROW.  FINALLY IND(K,1) IS RESET TO THE
C ROW NUMBER OF THE ELEMENT HELD IN A(K).
C     ON ENTRY AND ON EXIT IW(I,1) CONTAINS THE ADDRESS OF THE FIRST
C ELEMENT OF COLUMN I AND IW(N+1,1) CONTAINS THE ADDRESS OF THE FIRST
C UNUSED ELEMENT IN A.  ON EXIT IW(I,2) HOLDS THE COLUMN NUMBERS IN
C PIVOTAL ORDER. AFTER A SUCCESSFUL ENTRY IW(N+1,2)=N+1 AND IW(N+1,3)=0;
C AFTER AN UNSUCCESSFUL ENTRY IW(N+1,2)=0.  IN THE MAIN BODY OF THE
C SUBROUTINE IW(I,1),IW(I,2) HOLD THE ADDRESS OF THE FIRST ELEMENT OF
C THE I TH ROW/COLUMN AND ARE NEGATED IF THE FIRST ELEMENT HAS BEEN
C IN A PIVOTAL COLUMN/ROW.
C
C        IW(I,3) (I,4) HOLD THE LOGS OF THE ROW OR COLUMN
C SCALING FACTORS USED.
C     IW(I,5),IW(I,6) HOLD THE POSITION IN THE ORDERING BY NUMBER OF
C NON-ZEROS OF THE LAST ROW/COLUMN TO HAVE LESS THAN I NON-ZERO
C ELEMENTS OR ZERO IF NONE HAVE LESS THAN I NON-ZERO ELEMENTS.
C ON EXIT IW(I,5) HOLDS THE POSITION OF THE I TH ROW IN THE PIVOTAL
C ORDERING.
C     IW(I,7),IW(I,8) HOLD THE NUMBER OF NON-ZEROS IN THE I TH ROW/
C COLUMN.
C     IW(I,9),IW(I,10) HOLD THE POSITION OF THE I TH ROW/COLUMN
C IN THE ORDERING BY NUMBER OF NON-ZEROS.
C     IW(I,11),IW(I,12) HOLD ROW/COLUMN NUMBERS IN PIVOTAL ORDER FOR
C I@IP AND IN ORDER OF INCREASING NUMBERS OF NON-ZEROS OTHERWISE.
C     IW(I,13) HOLDS THE EXPONENT OF THE MAXIMAL ELEMENT IN THE I TH
C COLUMN OF THE SCALED VERSION OF THE ORIGINAL MATRIX.
C
C
      COMMON/SPARS/LP,EPS,ISCALE,JSCALE
      INTEGER IND(IA,2),IW(NP,13)
     ,,       IFAC(2),IK(2),IC(2),JC(2),JP(2)
      REAL A(IA),ROWCOL(2)
C
      DATA ROWCOL/8H     ROW,8H COLUMN  /
      DATA LP,EPS ,ISCALE,JSCALE,ZERO,ONE/6,1.0E-14,0,0,0.,1./
*CALL  STATF
C
C     CALL MSCALE TO FIND SCALING FACTORS IF JSCALE " 0 ELSE USE 1.0
C
      KA=IW(N+1,1)-1
      U=AMIN1(ONE,AMAX1(U,EPS*ONE))
      N1=N+1
      DO 1 I=1,N1
1     IW(I,2)=I
      J1=3
      IF(JSCALE)6,8,2
2      J=2*(KA/2)+3
      CALL MSCALE(A,IND,IW,N,NP,IW(1,3),IW(1,5),I)
3     L=1
      IR=IABS(I)
      IF(I.LT.0)L=2
      IF(I.NE.0)GO TO 560
6     J1=5
8     DO 10 I=1,N1
      IW(I,2)=IW(I,1)
      IW(I,11)=I
      IW(I,12)=I
      IW(I,13)=0
      DO 10 J=J1,9
10    IW(I,J)=0
C
C     SCALE THE MATRIX, SET ROW AND COLUMN LINKS, FIND FIRST ELEMENTS
C OF THE ROWS, COUNT THE NUMBER OF NON-ZEROS IN THE ROWS AND COLUMNS
C AND FIND EXPONENTS OF MAXIMAL COLUMN ELEMENTS.
C
C     USE IW(I,9) TEMP TO HOLD ADDR OF LAST NONZERO OF ROW I
C
      IG=0
      DO 30 J=1,N
      FAC=ONE
      K1=IW(J,2)
      K2=IW(J+1,2)-1
      IW(J,8)=K2-K1+1
      IL=0
      AMAX=ZERO
      DO 20 K=K1,K2
      I=IND(K,1)
      IF(I.LE.IL)GO TO 520
      IL=I
      IND(K,1)=I+IA
      IND(K,2)=K+1
      KL=IW(I,9)
      IF(KL.LE.0)IW(I,1)=K
      IF(KL.GT.0)IND(KL,1)=K
      IW(I,9)=K
20    IW(I,7)=IW(I,7)+1
      IND(K2,2)=IA+J
 30   CONTINUE
C
C     SET UP THOSE VECTORS IN IW ASSOCIATED WITH ORDERING BY NUMBERS
C OF NON-ZEROS.
C
      DO 130 L=1,2
C
C
C     SEND L+8 TO BE SORTED AS SORTAG REARRANGES ARRAY LEAVE L+6 AS IS
C
      DO 1300 I=1,N
 1300 IW(I,L+8)=IW(I,L+6)
      CALL SORTAG(IW(1,L+8),1,N,IW(1,L+10))
C
C     CHECK FOR NULL ROW OR COLUMN.
C
      IR=IW(1,L+10)
      IF(IW(IR,L+6).LE.0)GO TO 560
      DO 110 I=1,N
      J=IW(I,L+10)
      IW(J,L+8)=I
      NZ=IW(J,L+6)
110   IF(NZ.NE.N)IW(NZ+1,L+4)=I
      J=0
      DO 130 I=1,N
      IF(IW(I,4+L).EQ.0)IW(I,L+4)=J
130   J=IW(I,L+4)
C
C     NOW PERFORM THE MAIN ELIMINATION.
C     FIND THE PIVOT. WE DO THIS BY SEARCHING A ROW/COLUMN. THE
C NEXT ROW/COLUMN TO BE USED IS JC(L)=IW(IK(L),L+10).
C
C     JCOST IS COST OF CHEAPEST PIVOT SO FAR
C     ICOST IS MINUMUM POSS COST OF PIVOT NOT YET FOUND
C
      DO 298 IP=1,N
133   IK(1)=IP
      IK(2)=IP
       JCOST=N*N
135   DO 140 L=1,2
      JC(L)=IW(IK(L),L+10)
140   IC(L)=IW(JC(L),L+6)
      ICOST=(IC(1)-1)*(IC(2)-1)
      IF(JCOST.LE.ICOST)GO TO 160
      L=1
      IF(IC(1).GT.IC(2))L=2
      IR=JC(L)
C
C     FIND THE MAXIMAL ELEMENT IN ROW/COLUMN UNDER CONSIDERATION.
      AMAX=ZERO
C
      K=IW(IR,L)
      GO TO 143
142   K=IND(-K,L)
143   IF(K.LT.0)GO TO 142
      KK=K
      GO TO 145
144   AMAX=AMAX1(AMAX, ABS(A(KK)))
      KK=IND(KK,L)
145   IF(KK.LE.IA)GO TO 144
      IF(AMAX.EQ.ZERO)GO TO 560
      AU=AMAX*U
      L3=3-L
      IK(L)=IK(L)+1
C
C     NOW CONSIDER THE ELEMENTS IN THE ROW/COLUMN IN TURN.
C
146   IF( ABS(A(K)).LT.AU)GO TO 150
      KK=K
147   KK=IND(KK,L3)
      IF(KK.LE.IA)GO TO 147
      KCOST=(IC(L)-1)*(IW(KK-IA,9-L)-1)
      IF(KCOST.GE.JCOST)GO TO 150
      JCOST=KCOST
      KP=K
      JP(L)=IR
      JP(L3)=KK-IA
      IF(JCOST.LE.ICOST)GO TO 160
150   K=IND(K,L)
      IF(IA-K)135,146,146
C
C     REARRANGE THE LINKS SO THAT THE PIVOTAL ROW AND COLUMN ARE IN
C CORRECT PIVOTAL SEQUENCE.
C     "MOVE" THE PIVOTAL COLUMN FIRST AND THEN THE PIVOTAL ROW.
C     K POINTS TO AN ELEMENT IN THE PIVOTAL COLUMN/ROW.
C     KM POINTS TO ITS PREDESSOR IN ITS ROW/COLUMN.
C     KL POINTS TO THE LAST ELEMENT THAT HAS BEEN PIVOTAL IN ITS
C ROW/COLUMN.
C
 160  DO 188 L=1,2
      IK(3-L)=JP(3-L)
      K=IABS(IW(IK(3-L),3-L)+0)
165   KO=K
170   KO=IABS(IND(KO,L)+0)
      IF(KO.LE.IA)GO TO 170
      IK(L)=KO-IA
      KL=KO
      KO=IW(KO-IA,L)
      IF(IW(IK(L),L+6).LE.0)GO TO 174
C
C     ON THE REMOVAL OF THE ELEMENT A(IK(1),IK(2)) THE FOLLOWING
C INSTRUCTIONS ARE USED TO UPDATE THE NUMBERS OF ELEMENTS IN THE
C CORRESPONDING ROW AND COLUMN AND MAKE CONSEQUENT CHANGES TO THE
C ORDERING BY NUMBER OF NON-ZEROS.
C
      DO 171 LM=1,2
      IR=IK(LM)
      NZ=IW(IR,LM+6)-1
      IW(IR,LM+6)=NZ
      JPOS=IW(NZ+1,LM+4)+1
      IPOS=IW(IR,LM+8)
      IF(IPOS.EQ.JPOS)GO TO 171
      JR=IW(JPOS,LM+10)
      JJ=IW(IPOS,LM+10)
      IW(IPOS,LM+10)=IW(JPOS,LM+10)
      IW(JPOS,LM+10)=JJ
      JJ=IW(IR,LM+8)
      IW(IR,LM+8)=IW(JR,LM+8)
      IW(JR,LM+8)=JJ
171   IW(NZ+1,LM+4)=JPOS
      GO TO 174
172   KL=-KO
      KO=IND(KL,L)
174   IF(KO.LT.0)GO TO 172
      KM=KL
      GO TO 178
176   KM=KO
      KO=IND(KO,L)
178   IF(KO.NE.K)GO TO 176
      IF(KL.EQ.KM)GO TO 182
      IND(KM,L)=IND(K,L)
      IF(KL.LE.IA)GO TO 183
      IND(K,L)=IW(IK(L),L)
180   IW(IK(L),L)=-K
      GO TO 186
182   IF(KM-IA)184,184,180
183   IND(K,L)=IND(KL,L)
184   IND(KL,L)=-K
186   K=IABS(IND(K,3-L)+0)
      IF(K.LE.IA)GO TO 165
188   CONTINUE
C
C     OVERWRITE THE ELEMENTS OF THE PIVOTAL COLUMN BY MULTIPLIERS
C AND PERFORM THE ELIMINATION.
C
      K=IND(KP,2)
      GO TO 295
190   M=K
193   M=IND(M,1)
      IF(M.LE.IA)GO TO 193
      L=M-IA
      A(K)=A(K)/A(KP)
      KI=KP
      KL=K
      GO TO 280
195   M=KL
200   M=IND(M,2)
      IF(M.LE.IA)GO TO 200
      JL=M-IA
      IF(JI-JL)210,275,270
C
C     CREATE A NEW NON-ZERO IN POSITION (L,JI).
C
210   KA=KA+1
      IF(KA.GT.IA)GO TO 580
      A(KA)=ZERO
      IK(1)=L
      IK(2)=JI
      IND(KA,1)=IND(KLAST,1)
      IND(KLAST,1)=KA
C
C     THE ORDER OF THE COLUMN LINKS DOES NOT MATTER SO WE PUT THE
C NEW ELEMENT AS THE SECOND IN ITS COLUMN.
C
      IND(KA,2)=IND(KI,2)
      IND(KI,2)=KA
C
C     ON THE ADDITION OF THE ELEMENT A(IK(1),IK(2)) THE FOLLOWING
C
C INSTRUCTIONS ARE USED TO UPDATE IW.
C
      DO 250 LM=1,2
      IR=IK(LM)
      NZ=IW(IR,LM+6)
      IW(IR,LM+6)=NZ+1
      JPOS=IW(NZ+1,LM+4)
      JR=IW(JPOS,LM+10)
      IF(IR.EQ.JR)GO TO 250
      IPOS=IW(IR,LM+8)
      JJ=IW(IPOS,LM+10)
      IW(IPOS,LM+10)=IW(JPOS,LM+10)
      IW(JPOS,LM+10)=JJ
      JJ=IW(IR,LM+8)
      IW(IR,LM+8)=IW(JR,LM+8)
      IW(JR,LM+8)=JJ
250   IW(NZ+1,LM+4)=JPOS-1
      KL=KA
      GO TO 275
265   M=KI
267   M=IND(M,2)
      IF(M.LE.IA)GO TO 267
      JI=M-IA
270   KLAST=KL
      KL=IND(KL,1)
      IF(IA-KL)210,195,195
275   A(KL)=A(KL)-A(K)*A(KI)
280   KI=IND(KI,1)
      IF(KI.LE.IA)GO TO 265
290   K=IND(K,2)
295   IF(K.LE.IA)GO TO 190
298   CONTINUE
C
C     SCAN BY ROWS REPLACING ROW LINKS BY ROW NUMBERS.
C
      DO 310 I=1,N
      K=IABS(IW(I,1)+0)
300   KK=IND(K,1)
      IND(K,1)=I
      K=IABS(KK)
      IF(K.LE.IA)GO TO 300
310   CONTINUE
C
C     SCAN BY COLUMNS REPLACING COLUMN LINKS BY ORDERING NUMBERS.
C
C     SCAN BY COLUMNS REPLACING COLUMN LINKS BY ORDERING NUMBERS.
C
      J=1
      DO 330 I=1,N
      K=IABS(IW(IW(I,12),2)+0)
      IW(IW(I,12),1)=J
320   KK=IND(K,2)
      IND(K,2)=J
      J=J+1
      K=IABS(KK)
      IF(K.LE.IA)GO TO 320
330   CONTINUE
      IW(N+1,1)=J
C
C     REORDER.
C
      KA=J-1
      DO 360 I=1,KA
      IF(I.EQ.IND(I,2))GO TO 360
      A1=A(I)
      I1=IND(I,1)
      J=I
350   K=IND(J,2)
      IND(J,2)=J
      A2=A(K)
      I2=IND(K,1)
      A(K)=A1
      IND(K,1)=I1
      A1=A2
      I1=I2
      J=K
      IF(K.NE.I)GO TO 350
360   CONTINUE
C
C     SET REMAINING VECTORS IN PREPARATION FOR FACTOR AND OPERATE.
C
      DO 370 I=1,N1
      IW(I,2)=IW(I,12)
370   IW(I,5)=IW(I,9)
C
C     RESTORE THE MATRIX TO ITS UNEQUILIBRATED STATE.
C
      FAC=ONE
      DO 400 J=1,N
      JCOL=IW(J,2)
      K1=IW(JCOL,1)
      K2=IW(IW(J+1,2),1)-1
      I0=IW(JCOL,4)
      DO 400 K=K1,K2
      JRO=IND(K,1)
400   IF(IW(JRO,5).EQ.J)I0=-IW(JRO,3)
      IW(N+1,3)=0
       G=0.
      RETURN
C
C     THE FOLLOWING INSTRUCTIONS IMPLEMENT THE FAILURE EXITS.
C
500   WRITE(LP,510)
*IF -DEF,AELIB5
 510  FORMAT(*+ERROR RETURN FROM SPARSE _ *)
*ENDIF
*IF DEF,AELIB5
 510  FORMAT('+ERROR RETURN FROM SPARSE _ ')
*ENDIF
      IW(N1,2)=0
      G=-ONE
      RETURN
520   WRITE(LP,530)K
      GO TO 500
*IF -DEF,AELIB5
 530  FORMAT(//34X* THE ELEMENT IN A(*I5*) IS OUT OF ORDER*)
*ENDIF
*IF DEF,AELIB5
 530  FORMAT(//34X,' THE ELEMENT IN A(',I5,') IS OUT OF ORDER')
*ENDIF
560   WRITE(LP,570)ROWCOL(L),IR
*IF -DEF,AELIB5
570   FORMAT(//34X
     1,* THE MATRIX IS SINGULSR.*A8,I4* DEPENNDS ON THE REST*)
*ENDIF
*IF DEF,AELIB5
570   FORMAT(//34X
     1,' THE MATRIX IS SINGULSR.',A8,I4,' DEPENNDS ON THE REST')
*ENDIF
      GO TO 500
580   WRITE(LP,590)IP
*IF -DEF,AELIB5
 590  FORMAT(//34X* IA IS TOO SMALL. NO SPACE AT PIVOT *I5)
*ENDIF
*IF DEF,AELIB5
 590  FORMAT(//34X,' IA IS TOO SMALL. NO SPACE AT PIVOT ',I5)
*ENDIF
      GO TO 500
C
      END
      SUBROUTINE MSCALE(A,IND,IP,N,NP,DIAG,RES,IS)
C
C     SPARSE MATRIX SCALING ROUTINE TO FIND OPTIMUM EQUILIBRIATION
C          FACTORS FOR OPERATIONS ON SPARSE MATRIX A.  ARRAYS ARE SET UP
C          ARRAYS ARE SET UP AS IN SPARSE
C          MSCALE IS CALLED BY SPARSE AND SPARSEB IF JSCALE NE 0.
C
C     ADAPTED FROM THE HARWELL ROUTINE MC12A  AERE-R-6844 [ REID ]
C
C        M B CARVER   DEC 1974
C
C      DIAG IS USED TO RETURN INTEGER SCALING BOWERS, AND TO HOLD
C     COUNTS OF NON-ZEROS IN ROWS AND COLUMNS DURING EXECUTION.
C     IS IS SET TO 0 ON SUCCESSFUL COMPLETION, TO I IF ROW I HAS ONLY
C        ZERO ELEMENTS, TO -I IF COLUMN I HAS ONLY ZERO ELEMENTS
C     SCALE, RES, DEL ARE WORKSPACE ARRAYS
C      SCALE HOLDS CURRENT APPROX TO COLUMN SCALING POWERS
C      DEL HOLDS CHANGES IN THESE OVER TWO ITERATIONS
C      RES HOLDS NON-ZERO HALF OF RESIDUAL VECTOR FOR TWO ITERATIONS
C
      INTEGER IND(1),IP(NP,2),DIAG(NP,2),JU(2)
      REAL A(1),RES(N,4)
      LOGICAL IU,IW(3)
      EQUIVALENCE (UU,IW(1)),(U,IU,JU(1))
      DATA SMIN/0.01/
      UU=100.
      IS=0
C
C     INITIALISE FOR ACCUMULATION OF SUMS AND PRODUCTS
C
      DO 2 L=1,2
      DO 2 I=1,N
      RES(I,L)=0.
      RES(I,L+2)=0.
    2 DIAG(I,L)=0
      DO 4 J=1,N
      I2=IP(J,2)
      K1=IP(I2,1)
      K2=IP(IP(J+1,2),1)-1
      IF(K1.GT.K2)GOTO4
      DO 3 K=K1,K2
      I1=IND(K)
      U= ABS(A(K))
      IF(U.EQ.0.)GO TO 3
C
C     ON THE IBM 360 THE FOLLOWING TWO INSTRUCTIONS FIND THE SMALLEST
C     INTEGER GREATER THAN ALOG16(U).
C
      U=AINT(ALOG10(U))+1
C
C     COUNT NON-ZEROS IN ROW AND COLUMN
C
C
      DIAG(I1,1)=DIAG(I1,1)+1
      DIAG(I2,2)=DIAG(I2,2)+1
      RES(I1,1)=RES(I1,1)+U
52    RES(I2,3)=RES(I2,3)+U
    3 CONTINUE
  4   CONTINUE
C
C     COMPUTE RHS VECTORS TESTING FOR ZERO ROW OR COLUMN
C
      JU(1)=17920
      DO 8 I=1,N
      J=J+DIAG(I,1)
      DO 9 L=1,2
      IF(DIAG(I,L).GT.0 )GO TO 13
      DIAG(I,L)=1
      IS=I*(3-2*L)
   13 CONTINUE
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,L)
    9 RES(I,2*L-1)=RES(I,2*L-1)/U
    8 CONTINUE
      SM=SMIN*J
C
C     SWEEP TO COMPUTE INITIAL RESIDUAL VECTOR
C
      DO 12 J1=1,N
      J=IP(J1,2)
      K1=IP(J,1)
      K2=IP(IP(J1+1,2),1)-1
      IF(K1.GT.K2)GOTO12
      DO 10 K=K1,K2
       IF(A(K).EQ.0. )GO TO 10
      I=IND(K)
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,1)
      RES(I,1)=RES(I,1)-RES(J,3)/U
   10 CONTINUE
  12  CONTINUE
C
C     INITIALISE ITERATION
C
      E=0.
      E1=0.
      Q=1.
      S=0.
      DO 11 I=1,N
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,1)
   11 S=S+U*RES(I,1)**2
      L=2
      IF(S.LE.SM)GO TO 101
C
C     ITERATION STEP
C
   20 EM=E*E1
C
C    SWEEP THROUGH MATRIX TO UPDATE RESIDUAL VECTOR
C
      DO 922 J3=1,N
      J1=IP(J3,2)
      K1=IP(J1,1)
      K2=IP(IP(J3+1,2),1)-1
      IF(K1.GT.K2)GOTO922
      DO 22 K=K1,K2
      IF(A(K).EQ.0. ) GO TO 22
      J2=(2-L)*(J1-IND(K))
      I=J1-J2
      J=IND(K)+J2
      RES(I,L)=RES(I,L)+RES(J,3-L)
   22 CONTINUE
  922 CONTINUE
      S1=S
      S=0.
      DO 23 I=1,N
      V=-RES(I,L)/Q
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,L)
      RES(I,L)=V/U
   23 S=S+V*RES(I,L)
      E1=E
      E=Q*S/S1
      Q1=Q
      Q=1.-E
      M=3-L
      IF(S.GT.SM)GO TO 27
      E=M-1
      M=1
      Q=1.
   27 IF(L.EQ.2)GO TO 25
      QM=Q*Q1
      DO 26 I=1,N
      RES (I,4)=(EM*RES(I,4)+RES(I,2))/QM
   26 RES(I,3)=RES(I,3)+RES(I,4)
   25 L=M
      DO 24 I=1,N
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,L)
   24 RES(I,L)=RES(I,L)*U*E
      IF(S.GT.SM)GO TO 20
C
C      SWEEP THROUGH MATRIX TO GET ROW SCALING POWERS
C
 101  DO 903 J1=1,N
      J=IP(J1,2)
      K1=IP(J,1)
      K2=IP(IP(J1+1,2),1)-1
      IF(K1.GT.K2)GOTO903
      DO 103 K=K1,K2
      U= ABS(A(K))
      IF(U.EQ.0.)GO TO 103
C
C      ON IBM 360 NEXT TWO INSTRUCTIONS FIND THE SMALLEST INTEGER
C     LESS THAN ALOG16(U)
C
      U=AINT(ALOG10(U))+1
      I=IND(K)
      RES(I,1)=RES(I,1)+RES(J,3)-U
  103 CONTINUE
 903  CONTINUE
C
C      CONVERT POWERS TO INTEGERS
C
      DO 104 I=1,N
C
C      ON IBM 360 NEXT INSTRUCTION SETS U TO VALUE OF POSITIVE INTEGER
C
      U=DIAG(I,1)
      V=RES(I,1)/U
      DIAG(I,1)=V+SIGN(0.5,V)
  104 DIAG(I,2)=-(RES(I,3)+SIGN(0.5,RES(I,3)))
      RETURN
      END
