*DECK SPARSEB
      SUBROUTINE SPARSEB(A,IRN,IP,N,NP,AGRO,MTYPE,AVECT,AWS,NAME)
************************************************************************
**                                                                    **
**    NOTE THAT SPARSEB IS WRITTEN IN MODULAR FORM USING THE          **
**    ADDITIONAL ROUTINE:                                             **
**       MSCALE                                                       **
**    MSCALE ACCOMPANIES ROUTINE SPARSE UNDER THE DECK NAME SPARSE    **
**                                                                    **
************************************************************************
C
C     SPARSE MATRIX OPERATE , FACTOR OR REFILL ROUTINE
C
C     A       ARRAY FROM SPARSE  HOLDS THE PACKED SPARSE MATRIX
C     IRN     ARRAY DIMENSION IA,2 HOLDS INDICES TO REFERENCE A
C     IP      ARRAY DIMENSION NP,13 HOLDS INDICES TO REFERENCE A
C     N       THE NUMBER OF ROWS IN THE FULL MATRIX
C     NP  FIRST DIMENSION OF IW AT LEAST N+1
C     AGRO    OUTPUT PARAMETER INDICATING ERROR
C     AVECT   VECTOR DIMENSION N TO BE OPERATED ON AND TO RETURN RESULT
C     AWS     ARRAY DIMENSION N FOR WORKSPACE
C     NAME    ROUTINE CALLED TO READ IN A NEW MATRIX FOR MTYPE=6
C     MTYPE   OPERATION CONTROL ?
C
C      THIS ENTRY OPERATES ON THE VECTOR AVECT WITH THE FOLLOWING
C      MATRICES ACCORDING TO THE VALUE OF MTYPE
C
C         MTYPE = 1      A**(-1)
C         MTYPE = 2      A**(-T)
C         MTYPE = 3      A
C         MTYPE = 4      A**T
C         MTYPE = 5      FACTOR A MATRIX OF SAME SPARSITY STRUCTURE AS A
C         MTYPE = 6      ENTER  A MATRIX OF SAME SPARSITY STRUCTURE AS A
C
C      MATRIX A(I,J) IS STORED BY COLUMNS J IE A11 A31 A22 A42 A33  ETC
C      IRN  CONTAINS THE ROW NUMBERS I ABOVE  1 3 2 4 3   ETC
C      IP   CONTAINS THE POSITION IN A OF START OF EACH COLUMN J _ 1,3,5
C      IP(N+1) HAS THE NUMBER OF NONZEROS IN A AND IRN ON ENTRY
C
C      JP IS STREAM NUMBER FOR DIAGNOSTICS
C      JSCALE CONTROLS EQUILIBRATION
C
C      ARND IS ROUNDING ERROR ESTIMATE
C      ANAME CONTAINS NAMES FOR USE IN DIAGNOSTICS
      COMMON/SPARS/JP,ARND,ISCALE,JSCALE
      REAL A(1),AWS(1),AVECT(1),ANAME(5)
      INTEGER IRN(1),IP(NP,5)
      LOGICAL LOG
      EQUIVALENCE (AU,JU)
C
      DATA JP,ARND,ISCALE,JSCALE,ZERO,ONE/6,1.0E-14,0,0,0.,1./
      DATA ANAME/7HSPARSE ,7HSPARSEB,10HSPARSEB(5),10HSPARSEB(6),
     ,             6HMSCALE/
*CALL  STATF
C
C         NORMAL ENTRY 1@MTYPE@4
C
C        CHECK LEGALITY OF ENTRY THEN JUMP FOR REQD OPERATION
C
      KERR=1
      MT=MTYPE
      MTYPE=0
      KENTRY=2
      IF(IP(N+1,2).EQ.0)GO TO 2205
      IF(IP(N+1,2).NE.N+1)GO TO 2207
      KERR=3
      IF(IP(N+1,3).NE.0)GO TO 2205
      LOG=.FALSE.
      MTYPE=MT
      IF(MTYPE.LE.0.OR.MTYPE.GT.6)GOTO2204
*IF -DEF,AELIB5
        GOTO (101,102,103,104,800,900)MTYPE
*ENDIF
*IF DEF,AELIB5
        GOTO (101,102,103,104,800,900)MTYPE
      CALL GOTOER
*ENDIF
C
C      SET UP FOR MTYPE=1,4, PERMUTING AVECT TO AWS
C
  104 LOG=.TRUE.
  101 DO 110 J=1,N
  110 AWS(IP(J,5))=AVECT(J)
      AU=ZERO
C
C      FIRST DIVIDE BY L OR MULTIPLY BY L**T
C
      DO 140 J=1,N
      J1=IP(IP(J+1,2),1)
  141 J1=J1-1
      JSEQ=IP(IRN(J1),5)
      IF(JSEQ.EQ.J)GO TO 111
      IF(LOG)GO TO 1412
      AWS(JSEQ)=AWS(JSEQ)-AWS(J)*A(J1)
      GO TO 141
 1412 AWS(J)=AWS(J)+AWS(JSEQ)*A(J1)
      GO TO 141
  111 IF(.NOT.LOG)JU=J1
C
C      SAVE PIVOT POSITION FOR MTYPE=1
C      SET ZERO FOR MTYPE=4
C      NOW DIVIDE BY U OR MULTIPLY BY U**T
C
  140 AVECT(IP(J,2))=AU
      J=N
  143 JCOL=IP(J,2)
      J1=IP(JCOL,1)
      IF(LOG)GO TO 144
C
C      RECOVER PIVOT POSITION IN JU
C
      AU=AVECT(JCOL)
      AVECT(JCOL)=AWS(J)/A(JU)
  144 JSEQ=IP(IRN(J1),5)
      IF(LOG)GO TO 1442
      IF(JSEQ.EQ.J)GO TO 119
      AWS(JSEQ)=AWS(JSEQ)-AVECT(JCOL)*A(J1)
      GO TO 1443
 1442 AVECT(JCOL)=AVECT(JCOL)+AWS(JSEQ)*A(J1)
      IF(JSEQ.EQ.J)GO TO 119
 1443 J1=J1+1
      GO TO 144
  119 J=J-1
      IF(J.GT.0) GO TO 143
      RETURN
C
C
C      ENTRY FOR MTYPE=2,3
C      FIRST DIVIDE BY U**T OR MULTIPLY BY U
C
  103 LOG=.TRUE.
  102 DO 120 J=1,N
      JCOL=IP(J,2)
      J1=IP(JCOL,1)
  121 JSEQ=IP(IRN(J1),5)
      IF(JSEQ.EQ.J)GO TO 122
      IF(LOG)GO TO 1212
      AVECT(JCOL)=AVECT(JCOL)-AWS(JSEQ)*A(J1)
      GO TO 1213
 1212 AWS(JSEQ)=AWS(JSEQ)+A(J1)*AVECT(JCOL)
 1213 J1=J1+1
      GO TO 121
  122 IF(LOG)GO TO 132
      AWS(JSEQ)=AVECT(JCOL)/A(J1)
      GO TO 120
  132 AWS(JSEQ)=AVECT(JCOL)*A(J1)
  120 CONTINUE
C
C      NOW DIVIDE BY L**T OR MULTIPLY BY L
C
      J=N
  123 J1=IP(IP(J+1,2),1)
  124 J1=J1-1
      JRO=IRN(J1)
      IF(IP(JRO,5).EQ.J)GO TO 125
      IF(LOG)GO TO 1242
      AWS(J)=AWS(J)-AVECT(JRO)*A(J1)
      GO TO 124
 1242 AVECT(JRO)=AVECT(JRO)+AWS(J)*A(J1)
      GO TO 124
  125 AVECT(JRO)=AWS(J)
      J=J-1
      IF(J.GT.0)GO TO 123
      RETURN
C
C        ENTRY TO FACTOR   MTYPE=5  ORIGINALLY ENTRY C
C
C     ON ENTRY THE ARRAYS ARE AS SET UP BY SPARSE, EXEPT THAT 'A'
C     CONTAINS THE ELEMENTS OF A NEW MATRIX IN UNFACTORISED FORM.
C     ON EXIT, THE CONTENTS OF A HAVE BEEN REPLACED BY THE L/U DECOMP-
C     OSITION OF THIS NEW MATRIX, EXACTLY AS ON EXIT FROM MA18AD.
C      THE ARGUMENT AGRO IS SET TO THE GROWTH ESTIMATE AS FOR MA18AD.
C      NEW SCALING FACTORS ARE CALCULATED AND STORED IF JSCALE.GT.1, OR
C     IF JSCALE=0 WHEN THEY WILL ALL BE UNITY.      OTHERWISE, THE OLD
C     SCALING FACTORS ARE USED . THE PIVOTAL SEQUENCE SET BY SPARSE
C     IS ALWAYS USED.
C
 800  IP(N+1,3)=0
      KERR=1
      AGRO=1.
      KENTRY=3
      IF(JSCALE.LE.0)GOTO991
      IF(JSCALE.LE.1)GO TO 991
      DO 898 I=1,N
  898 IP(I,13)=IP(I,5)
      CALL MSCALE(A,IRN,IP,N,NP,IP(1,3),IP(1,5),IS)
C
C      FIRST TEST FOR INVALID ENTRY THEN JUMP TO REQD OPERATION
C
      DO 897 I=1,N
  897 IP(I,5)=IP(I,13)
      IF(IS.NE.0)GO TO 2208
  991 IF(JSCALE.NE.0)GO TO 892
C
C     IF JSCALE GE 0 CALL MSCALE TO GET SCALING FACTORS
C        SAVE COL 5 IN COL 13 AS MSCALE USES 5 TO 12
C        IF JSCALE=0 ALL CSALE FACTORS SET TO 0
C
      DO 895 L=1,2
      DO 895 K=1,N
  895 IP(K,L+2)=0
  892 JUU=0
      IAG=0
C
C      OPERATE ON COLUMNS IN SEQUENCE
C
      DO 56 J=1,N
      JST=IP(IP(J,2),1)
      JND=IP(IP(J+1,2),1) -1
C
C      FIND APPROX LOG OF MAX ELEMENT IN COLUMN
C
      IA0=0
      DO 57 J1=JST,JND
      AU= ABS(A(J1))
C
C      ON SYSTEM/360, NEXT STATEMENT SETS JUU SO THAT
C      16**(JUU-64).GT.AU.GE.16**(JUU-65)
C
      JUU=ALOG10(AU)+1
   57 IA0=MAX0(IA0,JUU+IP(IRN(J1),3))
C
C      SCAN THROUGH COLUMN, ELIMINATING WITH ELEMENTS OF U AND
C     BRANCHING WHEN PIVOT FOUND
C
      ASSIGN 581 TO JPIV
      DO 58 J1=JST,JND
      GO TO JPIV,(581,582)
  581 KSEQ=IP(IRN(J1),5)
C
C      BRANCH ON PIVOT
C
      IF(KSEQ.EQ.J)GOTO 59
C      ELEMENT OF U - GET OLD PIVOTAL COLUMN
      KST=IP(IP(KSEQ,2),1)
      KND=IP(IP(KSEQ+1,2),1)-1
      AMULT=A(J1)
C
C      SKIP IRRELEVANT ELEMENTS AND SCAN DOWN COLUMNS
C
      L1=J1+1
      DO 60 K1=KST,KND
      KRO=IRN(K1)
      IF(IP(KRO,5).LE.KSEQ)GO TO 60
   62 IF(KRO.EQ.IRN(L1))GO TO 63
      L1=L1+1
      GO TO 62
C
C      ELIMINATION STEP
C
   63 A(L1)=A(L1)-AMULT*A(K1)
      AU= ABS(A(L1))
C
C      ON S/360, NEXT STATEMENT SETS JUU AS ABOVE
C
      JUU=ALOG10(AU)+1
C
C      UPDATE LOG OF GROWTH ESTIMATE
C
      IAG=MAX0(IAG,JUU+IP(IRN(L1),3)-IA0)
      L1=L1+1
   60 CONTINUE
      GO TO 58
C
C      TEST FOR ZERO PIVOT
C
   59 IF(A(J1).EQ.ZERO)GO TO 1011
      AMULT=A(J1)
      ASSIGN 582 TO JPIV
      GO TO 58
C
C      DIVIDE ELEMENTS OF L BY PIVOT
C
  582 A(J1)=A(J1)/AMULT
   58 CONTINUE
   56 CONTINUE
C
C      SET GROWTH ESTIMATE
C
      AGRO=ARND*10.**IAG
      RETURN
C
C      ENRTY TO READ IN A NEW MATRIX  MTYPE=6 WAS ENTRY D
C
 900  KERR=1
      KENTRY=4
C
C      INITIALLY CLEAR COLUMN
C
 2402 DO 301 J=1,N
      AWS(J)=ZERO
  301 CONTINUE
C
C      COLUMNS IN PIVOTAL ORDER
C
      DO 303 J=1,N
      JCOL=IP(J,2)
C
C        LOAD COLUMN FRON ROUTINE NAME
C        COPY TO REQD LOCATION AND RESET 0
C
      CALL NAME(N,AWS,JCOL)
      J1=IP(JCOL,1)
      J2=IP(IP(J+1,2),1)-1
      DO 302 L1=J1,J2
      JRO=IRN(L1)
      A(L1)=AWS(JRO)
  302 AWS(JRO)=ZERO
  303 CONTINUE
      RETURN
C
C      DIAGNOSTIC PRINTING
C
 2204 WRITE(JP,2304)ANAME(2),MTYPE
*IF -DEF,AELIB5
 2304 FORMAT(*0ERROR RETURN FROM *A8* - MTYPE=*I5* (OUT OF RANGE)*)
*ENDIF
*IF DEF,AELIB5
 2304 FORMAT('0ERROR RETURN FROM ',A8,' - MTYPE=',I5,' (OUT OF RANGE)')
*ENDIF
      GO TO 3004
 2205 WRITE(JP,2305)ANAME(KENTRY),ANAME(KERR)
*IF -DEF,AELIB5
 2305 FORMAT(*0ERROR RETURN FROM *A8* - PREVIOUS ENTRY TO *A8* BAD*)
*ENDIF
*IF DEF,AELIB5
 2305 FORMAT('0ERROR RETURN FROM ',A8,' - PREVIOUS ENTRY TO ',A8,' BAD')
*ENDIF
      GO TO 3004
 2207 WRITE(JP,2307) ANAME(KENTRY),ANAME(1)
*IF -DEF,AELIB5
 2307 FORMAT(*0ERROR RETURN FROM *A8* - NO PREVIOUS ENTRY TO *A8 )
*ENDIF
*IF DEF,AELIB5
 2307 FORMAT('0ERROR RETURN FROM ',A8,' - NO PREVIOUS ENTRY TO ',A8 )
*ENDIF
      GO TO 3004
 1011 WRITE(JP,2309)ANAME(3),IRN(J1),IP(J,2)
*IF -DEF,AELIB5
 2309 FORMAT(*0ERROR RETURN FROM *A8* - ZERO PIVOT(*2I4*)*)
*ENDIF
*IF DEF,AELIB5
 2309 FORMAT('0ERROR RETURN FROM ',A8,' - ZERO PIVOT(',2I4,')')
*ENDIF
      GO TO 3004
 2208 WRITE(JP,2308) ANAME(3),ANAME(5),IS
*IF -DEF,AELIB5
 2308 FORMAT(*0ERROR RETURN FROM *A8* - * A8* IN ERROR WITH IS=*I4)
*ENDIF
*IF DEF,AELIB5
 2308 FORMAT('0ERROR RETURN FROM ',A8,' - ' ,A8,' IN ERROR WITH IS=',I4)
*ENDIF
C
 3004 RETURN
      END
