*DECK SPLINT
      FUNCTION SPLINT(X,Y,N,A,B,Y1,Y2,Y3,E,IFLAG)
C
C    ROUTINE TO INTEGRATE TABULATED DATA SPLINT=IN( A TO B(Y(X)DX))
C      WHERE X AND Y(X) ARE STORED IN PARALLEL ARRAYS DIMENSION N
C       A,B ARE NOT NECESSARILY DATA POINTS
C       Y1,Y2,Y3 ARE ARRAYS DIMENSION N FOR SUBSEQUENT USE TO INTERPOLAT
C             S(X) X(I)<X<X(I+1)  ACCORDING TO THE SPLINE FORMULA
C      S(X)=Y(I)+Y1(I)*(X-X(I))+Y2(I)*(X-X(I))**2+Y3(I)*(X-X(I))**3
C     E IS A WORKING STORAGE ARRAY DIMENSION 2N
C     IFLAG  ERROR INDICATOR AND PRINT CONTROL.
C            IF IFLAG IS RETURNED ZERO INTEGRATION HAS PROCEEDED.
C            IF IFLAG IS ENTERED NON ZERO MESSAGES ARE GIVEN FOR ERRORS.
C            N MUST BE GE 3.      IF NOT SPLINT RETURNS 0 AND IFLAG -1.
C            X MUST BE MONOTONIC. IF NOT SPLINT RETURNS 0 AND IFLAG -2.
C
C   ATOMIC ENERGY OF CANADA MATHEMATICS AND COMPUTATION FORTRAN LIBRARY.
C
C  SOURCE DAVIS AND RABINOWITZ  METHODS OF NUMERICAL INTEGRATION,
C                               ACADEMIC PRESS      1975
C   REVISED BY VALERIE JONES                    AUG 1976
C
      REAL X(N),Y(N),Y1(N),Y2(N),Y3(N),E(N,2)
      DIMENSION MES(2)
*CALL ERRORF
      DATA MES/2200,2201/
C
*CALL STATF
      SPLINT=0.
      IF(N.LT.3) GOTO 60
C
C      DETERMINE SPLINE FIT PARAMETERS IF X SEQUENCE IS MONOTONIC
C
      M2 = N-1
      S = 0.
      DO 5 I=1,M2
*IF -DEF,AELIB5
         E(I)=X(I+1)-X(I)
         IF(E(I).LE.0) GOTO 70
         R=(Y(I+1)-Y(I))/E(I)
*ENDIF
*IF DEF,AELIB5
         E(I,1)=X(I+1)-X(I)
         IF(E(I,1).LE.0) GOTO 70
         R=(Y(I+1)-Y(I))/E(I,1)
*ENDIF
         Y2(I) = R-S
    5 S = R
      S = 0.
      R = 0.
      Y2(1) = 0.
      Y2(N) = 0.
      DO 10 I=2,M2
         Y2(I) = Y2(I)+R*Y2(I-1)
         E(I,2)=2*(X(I-1)-X(I+1))-R*S
*IF -DEF,AELIB5
         S=E(I)
*ENDIF
*IF DEF,AELIB5
         S=E(I,1)
*ENDIF
   10 R=S/E(I,2)
C
      DO 15 J=2,M2
         I = M2+2-J
*IF -DEF,AELIB5
   15 Y2(I)=(E(I)*Y2(I+1)-Y2(I))/E(I,2)
*ENDIF
*IF DEF,AELIB5
   15 Y2(I)=(E(I,1)*Y2(I+1)-Y2(I))/E(I,2)
*ENDIF
      DO 20 I=1,M2
*IF -DEF,AELIB5
         S=E(I)
*ENDIF
*IF DEF,AELIB5
         S=E(I,1)
*ENDIF
         R = Y2(I+1) - Y2(I)
         Y3(I) = R / S
         Y2(I) = 3.*Y2(I)
   20 Y1(I) = (Y(I+1)-Y(I)) / S-(Y2(I)+R)*S
C
C        E(I,1) ARE INTERMEDIATE INTEGRALS
C
*IF -DEF,AELIB5
      E(1) = 0.
*ENDIF
*IF DEF,AELIB5
      E(1,1) = 0.
*ENDIF
      DO 25 I=1,M2
         S = X(I+1) - X(I)
*IF -DEF,AELIB5
   25 E(I+1) = E(I)+(((Y3(I)*.25*S+Y2(I)/3.)*S+Y1(I)*.5)*S+Y(I))*S
*ENDIF
*IF DEF,AELIB5
   25 E(I+1,1) = E(I,1)+(((Y3(I)*.25*S+Y2(I)/3.)*S+Y1(I)*.5)*S+Y(I))*S
*ENDIF
C
C       DETERMINE END POINT POSITION AND ADJUST INTEGRALS
C
      R = A
      U = 1.
      DO 55 J=1,2
         IF(R.GT.X(1)) GOTO 30
            SPLINT = SPLINT - U*((R-X(1))*Y1(1)*.5+Y(1))*(R-X(1))
            GOTO 50
   30    IF(R.LT.X(N)) GOTO 35
*IF -DEF,AELIB5
            SPLINT = SPLINT - U*(E(N)+(R-X(N))*(Y(N)+.5*(Y(N-1)+(X(N)-X(
     ,      N-1))  *Y1(N-1))*(R-X(N))))
*ENDIF
*IF DEF,AELIB5
            SPLINT = SPLINT - U*(E(N,1)+(R-X(N))*(Y(N)+.5*(Y(N-1)+(X(N)-
     ,X(      N-1))  *Y1(N-1))*(R-X(N))))
*ENDIF
            GOTO 50
   35    DO 45 I=1,M2
            IF(R-X(I+1)) 40,40,45
   40       R = R - X(I)
*IF -DEF,AELIB5
            SPLINT = SPLINT - U*(E(I)+(((Y3(I)*.25*R+Y2(I)/3.)*R+Y1(I)*.
     ,      5)*R   +Y(I))*R)
*ENDIF
*IF DEF,AELIB5
            SPLINT = SPLINT - U*(E(I,1)+(((Y3(I)*.25*R+Y2(I)/3.)*R+Y1(I)
     ,*.      5)*R   +Y(I))*R)
*ENDIF
            GOTO 50
   45    CONTINUE
   50    U = -1
   55 R = B
C
C       EXIT
C
      RETURN
C
C      ERROR EXITS
C
   60 ENCODE(5,65,IERWD(3))N
   65 FORMAT(I3)
*IF -DEF,AELIB5
      IERWD(3)=IERWD(3).AND.MASK(30)
      IF(IFLAG.NE.0) CALL AELERR(MES(1),2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(3)=IERWD(3).AND.MASK(30)
      IF(IFLAG.NE.0) CALL AELERR(MES(1),'NF')
*ENDIF
      IFLAG=-1
      RETURN
   70 ENCODE(3,65,IERWD(4))I
      IERWD(4)=IERWD(4).AND.MASK(18)
      ENCODE(3,65,IERWD(5))I+1
*IF -DEF,AELIB5
      IERWD(5)=IERWD(5).AND.MASK(18)
      IF(IFLAG.NE.0) CALL AELERR(MES(2),2HNF)
*ENDIF
*IF DEF,AELIB5
      IERWD(5)=IERWD(5).AND.MASK(18)
      IF(IFLAG.NE.0) CALL AELERR(MES(2),'NF')
*ENDIF
      IFLAG=-2
      RETURN
      END
