*DECK SSCP
      SUBROUTINE SSCP(X, NRX, Y, W, M, NOPTS, KTRM, NRK, T, WS)
C
C
C  PURPOSE:
C
C     SUM OF SQUARES AND CROSS PRODUCTS CALCULATION FOR USE WITH THE
C     IMSL ROUTINE RLSTEP TO DO MULTIPLE LINEAR REGRESSION TO A
C     POLYNOMIAL.
C
C  ENTRY:
C     CALL SSCP(X,NRX,Y,W,M,NOPTS,KTRM,NRK,T,WS)
C
C     X.......REAL MATRIX OF DATA POINTS SUCH THAT X(I,J) IS THE I-TH
C                  DATA VALUE OF THE J-TH INDEPENDENT VARIABLE.
C     NRX.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR X IN THE CALLING
C                  PROGRAM.  NRX \ NOPTS.
C     Y.......REAL ARRAY OF DATA VALUES OF THE DEPENDENT VARIABLE.
C     W.......REAL ARRAY OF WEIGHTS FOR THE DEPENDENT VARIABLE (USE
C                  W(I) = 1 / VARIANCE(Y(I)), IF POSSIBLE).  W(I) = 1
C                  FOR ALL I RESULTS IN AN UNWEIGHTED FIT.
C     M.......INTEGER, NUMBER OF INDEPENDENT VARIABLES.
C     NOPTS...INTEGER, NUMBER OF DATA POINTS.
C     KTRM....INTEGER MATRIX DEFINING THE NON-CONSTANT TERMS IN ORDER
C                  OF THE POLYNOMIAL TO BE FITTED.  KTRM(I,J) IS THE
C                  POWER OF THE I-TH INDEPENDENT VARIABLE IN THE J-TH
C                  NON-CONSTANT TERM OF THE REGRESSION POLYNOMIAL.
C     NRK.....INTEGER, NUMBER OF ROWS DIMENSIONED FOR KTRM IN THE
C                  CALLING PROGRAM.  NRK \ M.
C     T.......INTEGER, NUMBER OF TERMS IN THE POLYNOMIAL TO BE FITTED.
C     WS......REAL WORKING STORAGE ARRAY.
C             WS(1) TO WS((T*(T+1))/2)  IS SET TO CONTAIN THE WEIGHTED
C                  SUMS OF SQUARES AND CROSS PRODUCTS MATRIX STORED IN
C                  SYMMETRIC STORAGE MODE.
C             WS(((T*(T+1))/2)+1) TO WS(((T*(T+3))/2)-1)  IS SET TO
C                  CONTAIN THE MEANS OF THE NON-CONSTANT TERMS IN THE
C                  POLYNOMIAL.
C             WS((T*(T+3))/2)  IS SET TO CONTAIN THE MEAN OF THE
C                  DEPENDENT VARIABLE.
C             WS(((T*(T+3))/2)+1) TO WS(((T*(T+3))/2)+N*TM1)  IS SET TO
C                  CONTAIN THE VALUES OF THE NON-CONSTANT TERMS OF THE
C                  POLYNOMIAL.  THE VALUE TERM MATRIX IS STORED BY
C                  COLUMNS.
C             WS(((T*(T+3))/2)+N*TM1+1) TO WS((T*(2*N+T+3))/2)  IS SET
C                  TO CONTAIN THE DEPENDENT VALUES OF Y.
C             WS((T*(2*N+T+3))/2+1) CONTAINS THE SUM OF WEIGHTS AS
C                  CALCULATED BY SSCP
C
C NORMAL EXIT
C
C     CONTROL IS RETURNED TO THE CALLING PROGRAM WITH THE CORRECTED
C     SUMS OF SQUARES AND CROSS PRODUCTS MATRIX STORED IN WS IN
C     SYMMETRIC STORAGE MODE READY FOR A SUBSEQUENT CALL TO RLSTEP.
C
C
C  AUTHORS:     D. E. SMITH AND L. E. EVANS         JUNE 1975
C
C
      INTEGER  KTRM(NRK, 1), T
      REAL  X(NRX, 1), Y(1), WS(1)
      REAL  W(1)
      INTEGER  TM1, D
*CALL  STATF
C
C
C        INITIALIZATION
C
      TM1 = T - 1
      I2 = (T*(T+1)) / 2
      I3 = I2 + T
      I4 = I3 + NOPTS * T
C
C     CALCULATE THE SUM OF WEIGHTS FOR ALL DATA POINTS
C
      IND3=I4+1
      WS(IND3)=0.
      DO 3 L=1,NOPTS
         WS(IND3)=WS(IND3)+W(L)
 3    CONTINUE
C
C        SET UP VALUE TERM MATRIX
C
      IND3 = I3 + 1
      DO 15  D = 1, TM1
        DO 10  L = 1, NOPTS
          WS(IND3) = 1.0
          DO 5  I = 1, M
      IF ( KTRM(I, D) .EQ. 0) GO TO 5
            WS(IND3) = WS(IND3) * X(L, I) ** KTRM(I, D)
5         CONTINUE
          IND3 = IND3 + 1
10      CONTINUE
15    CONTINUE
C
C        ADD THE DEPENDENT ARRAY, Y, TO THE  T  COLUMN OF THE
C                  VALUE TERM MATRIX
C
      IND3 = TM1 * NOPTS + I3 + 1
      DO 20  I = 1, NOPTS
        WS(IND3) = Y(I)
        IND3 = IND3 + 1
20    CONTINUE
C     IND3 = I3 + 1
C     PRINT 22, (WS(K), K = IND3, I4)
C22   FORMAT (1H0, 6G20.7)
C
C        COMPUTE THE WEIGHTED MEAN OF VALUE TERM MATRIX COLUMNS
C
      IND2 = I2 + 1
      IND3 = I3 + 1
      IND4=I4+1
      DO 24  I = IND2, I3
        WS(I) = 0.0
        DO 23  J = 1, NOPTS
          WS(I) = WS(I) + WS(IND3)*W(J)
          IND3 = IND3 + 1
23      CONTINUE
        WS(I) = WS(I) / WS(IND4)
24    CONTINUE
C     PRINT 21, (WS(I), I = IND2, I3)
C21   FORMAT (1H0, 6G20.7)
C
C        CALCULATE THE WEIGHTED SUMS OF SQUARES AND CROSS PRODUCTS
C                  MATRIX STORED IN SYMMETRIC STORAGE MODE.
C
      IND1 = 1
      IND22 = I2
      IND32 = I3 + 1
      DO 50  I = 1, T
        IND21 = I2 + 1
        IND22 = IND22 + 1
        IND31 = I3 + 1
        DO 45  J = 1, I
          WS(IND1) = 0.0
          DO 40  L = 1, NOPTS
            WS(IND1) = WS(IND1) + W(L) * (WS(IND31) - WS(IND21))
     -                          * (WS(IND32) - WS(IND22))
            IND31 = IND31 + 1
            IND32 = IND32 + 1
40        CONTINUE
          IND1 = IND1 + 1
          IND21 = IND21 + 1
          IF (I .NE. J)  IND32 = IND32 - NOPTS
45      CONTINUE
50    CONTINUE
C     PRINT 60, (WS(K), K = 1, I2)
C60   FORMAT (1H0, 6G20.7)
  999 RETURN
      END
