*DECK STIFFZ
      SUBROUTINE STIFFZ(DIFFUN,Y,N,T,DTOUT,EPS,H,MFS,ICALL,WS)
************************************************************************
**    NOTE THAT STIFFZ IS WRITTEN IN MODULAR FORM USING THE           **
**    ADDITIONAL ROUTINES:                                            **
**       COSET                                                        **
**       DECB                                                         **
**       DECOMP                                                       **
**       GEARZ                                                        **
**       JAKOB                                                        **
**       SOLB                                                         **
**       SOLVE                                                        **
**    ALL OF THE ABOVE ROUTINES ACCOMPANY STIFFZ ON THE AELIB SOURCE  **
**    TAPE UNDER THE DECK NAME STIFFZ.                                **
************************************************************************
C*
C*     CONTROL GEARS ALGORITHM
C*
C* THE INPUT PARAMETERS ARE..
C*    DIFFUN = NAME OF EXTERNAL FUNCTION WHICH WOULD EVALUATE THE RIGHT
C*              HAND SIDE OF THE DIFFERENTIAL EQNS. ITS CALLING
C*              SEQUENCE SHOULD BE DIFFUN(N,T,Y,DY)
C*    Y      = ARRAY DIMENS N.IT SHOULD CONTAIN INITIAL VALUES OF THE
C*              DEPENDENT VARIABLES ON INPUT. IT WILL CONTAIN THE
C*              INTERPOLATED VALUES OF THE DEPENDENT VARIABLES AT
C*              PRINTOUT.
C*    N      = THE NUMBER OF DIFFERENTIAL EQUATIONS
C*    T      = STARTING POINT OF THE INTEGRATION. DEFAULT IS 0. IT
C*              CONTAINS THE CURRENT VALUE OF THE INDEPENDENT VARIABLE
C*    DTOUT  = THE PRINTOUT INTERVAL, -VE IMPLIES INTEGRATING BACKWARDS
C*    EPS    = RELATIVE ERROR PERMITTED TO CONTROL STEP SIZE
C*    H      = SHOULD CONTAIN THE SUGGESTED INITIAL STEP SIZE(IF KNOWN)
C*              IT SHOULD STILL BE +VE WHEN INTEGRATING BACKWARDS.
C*              IF UNINITIALIZED ON INPUT, IT WILL BE SET INTERNALLY.
C*              ON RETURN,IT CONTAINS THE SUGGESTED INITIAL STEP SIZE ON
C*              NEXT CALL TO STIFFS. IT HOLDS THE CURRENT STEP SIZE TO
C*              BE TAKEN.
C*    MF     = METHOD FLAG SEE GEARS
C*    ICALL  = 0 ON FIRST ENTRY THEN RETURNS NEGATIVE IF ERROR ELSE
C*              RETURN NQ
C*    WS     = WORKING STORAGE PROVIDED BY USER. WS(1) SHOULD CONTAIN
C*              THE DIMENSION OF WS FOR BOUNDS TO BE CHECKED
C*              IF NOT SPECIFIED, IT WILL BE ASSUMED TO BE 1 AND AN
C*              AN ERROR MESSAGE WILL RESULT.
C*             WS(1) TO WS(N*LMAX) CONTAINS THE CURRENT VALUES OF Y AND
C*              UP TO (LMAX-1) ORDERS OF SCALED DERIVATIVES. CURRENTLY
C*              LMAX IS 6 FOR BOTH  THE GEARS AND THE ADAMS METHODS. TO
C*              CHANGE LMAX SEE SUBROUTINE COSET , AND ASSIGN  LMAX TO
C*              MAXDER+1 IN STIFFS BELOW
C*             WS(N*LMAX+1) TO WS(N*LMAX+N*2) CONTAINS TWO ESTIMATES
C*              OF DY/DT
C*             WS(N*LMAX+N*2+1) TO WS(N*LMAX+N*3) CONTAINS ESTIMATES OF
C*              INDIVIDUAL TRUNCATION ERRORS IN Y
C*             WS(N*LMAX+N*3+1) TO WS(N*LMAX+N*4) CONTAINS THE MAXIMUM
C*              OF Y(I) AND YCUT. YCUT IS 1.E-14 BY DEFAULT.
C*             REST OF WS IS USED TO HOLD JACOBIAN MATRIX DF/DY
C*
C*   OUTPUT VARIABLE
C*    ICALL -1 A NUMBER OF STEPS (KOUNT(7)) WERE TAKEN AT DTMAX SO
C*             EFFICIENCY HAS BEEN DEGRADED.
C*          -2 INTEGRATION FAILED TO SATISFY THE ERROR TEST AT DTMIN
C*             A TOTAL NUMBER OF KOUNT(5) STEPS.
C*          -3 CORRECT CONVERGENCE WAS NOT ACHIEVED AT DTMIN A
C*             TOTAL NUMBER OF KOUNT(6) STEPS
C*          -4 SPARSITY DETECTED TO BE INCREASING RESTRUCTURING HALTED
C*          -5 ERROR CRITERION TOO STRICT - PROBLEM FAILED ONE OF THE
C              ABOVE CRITERIA 10 CONSECUTIVE STEPS
C*          -6 ILLEGAL VALUE OF N,EPS OR MF
C*          -7 TROUBLE WITH JACOBIAN SPARSE. INCREASE ML AND NWS.
C*  NOTE       THE LAST 3 PROBLEMS ARE FATAL.  IF STIFFZ IS ENTERED
C*             AGAIN THE PROGRAM STOPPS.
C*
C*           IN /CONTROL/
C*              CONVGD IS TRUE IF A STEP HAS COMPLETED SUCCESSFULLY
C*              KFLAG1 RETURNS NEGATIVE FOR ERRORS IN GEARS
C*             IOUT,METHOD   USED BY FORSIM ONLY
C*             NQ IS CURRENT ORDER
C*
C
C              IN      / STAT /
C*   KOUNT(1) - TOTAL NUMBER OF CALLS TO THE AUXILIARY SUBROUTINE
C*   KOUNT(2) - NUMBER OF SUCCESSFUL STEPS TAKEN
C*   KOUNT(3) - NUMBER OF CALLS TO AUXILIARY SUBROUTINE FOR
C*              JACOBIAN EVALUATION
C*   KOUNT(4) - THE NUMBER OF JACOBIAN EVALUATIONS
C*   KOUNT(5) - THE NUMBER OF STEPS AT DTMIN FAILING THE ERROR
C*   KOUNT(6) - NUMBER OF STEPS AT DTMIN FAILING THE CONVERGENCE TEST
C*   KOUNT(7) THE NUMBER OF STEPS AT DTMAX
C
C         IN / BASINT /
C   DTMAX - MAX. STEP SIZE ALLOWED RELATIVE TO ABS(HINTVL).
C             UNINITIALIZED DTMAX IMPLIES DEFAULT VALUE OF 1.0
C   DTMIN - MIN. STEP SIZE ALLOWED RELATIVE TO ABS(HINTVL).
C             UNINITIALIZED DTMIN IMPLIES DEFAULT VALUE OF 1.E-12
C   YCUT  - LOWER BOUND FOR Y IN CALCULATING THE RELATIVE L.E.
C     DYCUT - LOWER BOUND OF SIGNIFICANCE OF DERIVATIVE
C             ERROR EST MUST BE LE EMAX*(AMAX(Y,YCUT,DYCUT-DY))
C        DEFAULTS ARE YCUT=1.OE-14,DYCUT=1.OE-5
C
C*             ML,MU ARE THE WIDTHS OF THE LOWER AND UPPER PARTS
C*              RESPECTIVELY OF THE BAND IN THE JACOBIAN MATRIX.
C*              THE FULL BANDWIDTH IS ML+MU+1. IF MITER=5 THE NUMBER OF
C*              PERMITTED NONZEROS IS ML*N AND THE JACOBIAN STRUCTURE
C*              IS REASSESSED EVERY MU EVALUATIONS.
C*
C*          IN /SPARS/
C*    LOUT   =  THE LOGICAL UNIT NUMBER FOR THE OUTPUT OF MESSAGES
C*              DURING THE INTEGRATION.
C*
C*    ANOISE IS THE MACHINE ACCURACY
C*   JSCALE,ISCALE ARE USED BY SPARSE ROUTINES
C*
C*             IN /STIFS/
C*    RSTART IF 0 THEN PERMIT T TO EXCEED PRINT TIME
C*             IF - DO NOT PERMIT THIS, AS A RESTART IS EXPECTED
C*             IF + COMPUTE EXACT VALUES AT NEXT PRINTOUT TIME
C*             STIFFS THEN SETS RESTART BACK TO -1
C*             THEN RESTART NEXT TIME BY ENTERING WITH JSTART 0
C*    JSTART   NORMALLY 1  .  USER SHOULD SET -1 IF HE CHANGES H,MF, OR
C*             SET TO 0 TO RESTART INTEGRATOR FROM CURRENT VALUES
C*    MAXDER   MAXIMUM PERMITTED ORDER
C*
C*
C*    ADAPTED FROM HINDMARSH VERSION OF GEAR DRIVER UCID 30001 REV2
C*       REVISED BY MB CARVER AND DG STEWART JAN 1978
C*
      EXTERNAL DIFFUN
      COMMON/CNTROL/IOUT,ICONV,KFLAG1
      COMMON/INT1/NQ,ZERO,ONE,KJ
      COMMON/SPARS/LOUT,ANOISE,ISCALE,JSCALE
      COMMON/DISCO/DISCO,DISC2
     ,      /BASINT/YCUT,DYCUT,HMAX,HMIN,ML,MU
     ,      /STIFS/RSTART,JSTART,MAXDER
     ,      /STAT/KOUNT(7)
      REAL Y(N),WS(N),WSS(5)
      INTEGER TEMP,WSFS
      DIMENSION MES(8)
      LOGICAL DISCO,DISC2
      DATA MES/2450,2451,2452,2453,2454,2455,2456,2457/
      DATA ML,MU/1,1/
      DATA LOUT,ANOISE,ISCALE,JSCALE,ZERO,ONE
     1     /6,1.0E-14,-1,1,0.,1./
*CALL STATF
C*
      IF(JSTART.EQ.-2) GOTO 20
      IF(ICALL.EQ.0) GO TO 5
      IF(ICALL+4) 190,55,55
C
C       INITIALIZE VARIABLES
C
    5 JSTART=0
C
      DO 6 I=1,7
 6    KOUNT(I)=0
      MF=MFS
      N2=N+N
      LMAX=6
      IY=1
      IFSAVE=LMAX*N+1
      IERROR=IFSAVE+2*N
      IYMAX=IFSAVE+3*N
      IPW=IFSAVE+4*N
      ANOISE=1.0E-14
C
C        VALUUES BELOW WILL BE DEFINED IF CALLED FRON FORSIM
C              BUT NOT NECESSARILY IF CALLED FRON LIBRARY
C
      IF(LEGVAR(HMAX).NE.0) HMAX=1.E15
      IF(LEGVAR(HMIN).NE.0) HMIN=1.0E-15
      IF(LEGVAR(DTOUT).NE.0)DTOUT=0.1
      IF(LEGVAR(EPS).NE.0)EPS=1.0E-5
      IF(LEGVAR(T).NE.0)T=0
      IF(N.LT.1) GO TO 170
      IF((EPS.GT.1.E-2).OR.(EPS.LT.1.E-10)) GO TO 170
      IPRINT=0
      IF(MF.LT.0) IPRINT=1
      MF=IABS(MF)
      IF(MF.LT.11.OR.MF.GT.25) GO TO 170
      IF(MF.GT.15.AND.MF.LT.21) GO TO 170
      IF (LEGVAR(WS(1)).NE.0) WS(1)=1
      IF(LEGVAR(RSTART).NE.0)RSTART=0
      IF(LEGVAR(DYCUT).NE.0) DYCUT=1.0E-09
      IF(LEGVAR(YCUT).NE.0) YCUT=1.0E-14
      DISCO=.FALSE.
      DISC2=.FALSE.
C
   10 KHFLAG=0
C
      TSAVE=T
      TLAST=T
      IOUT=0
      ICONV=0
      ISCALE=0
      JSCALE=0
      LOUT=6
C
C      CHECK SIZE OF PW ARRAY HERE
C
      WSS(1)=IPW
      WSS(2)=IPW+N*(N+1)
      WSS(3)=IPW+N
      WSS(4)=IPW+N*(2*ML+MU+2)
      WSS(5)=IPW+N*(4*ML+16)+16
      MITER=MF/10
      MITER=MAX0(1,MF-MITER*10)
      IF(MITER.EQ.1) MF=MF-1
      IF (N.EQ.1) MF=(MF/10)*10
C
C     IF(WS(1).LT.WSS(MITER)) INSUFFICIENT STORAGE EXIT INOUT = -7
C
      IF(WS(1).LT.WSS(MITER)) GO TO 180
C
C            CONTINUE INITIALISATION
C
   20 DO 25 I=1,N
   25 WS(I)=Y(I)
      DO 30 I=1,N2
   30 WS(IFSAVE-1+I)=0.0
      DO 35 I=1,N
   35 WS((LMAX+3)*N+I)=YCUT
      IF (LEGVAR(H).EQ.0) GOTO 50
      KOUNT(1)=KOUNT(1)+1
         CALL DIFFUN(N,T,Y,WS(IFSAVE))
         DO 40 I=1,N
   40    WS(IFSAVE)=AMAX1(WS(IFSAVE),ABS(WS(IFSAVE-1+I)))
         IF(WS(IFSAVE).EQ.0.) GOTO 45
            H=(EPS/WS(IFSAVE))**0.5
            GOTO 50
   45    H=0.001
   50 H=AMIN1(H,HMAX,ABS(DTOUT*EPS))
      H=AMAX1(H,HMIN)
      H=SIGN(H,DTOUT)
C
C          NORMAL ENTRY
C
   55 T=TSAVE
      IF(ICALL.LT.0) ICALL=NQ
      TLAST=TLAST+DTOUT
      TLEFT=TLAST-T
      TEMP=LMAX*N+1
      IF(TLEFT.LE.0.) GO TO 70
C
C       TRANSFER TO GEARS ROUTINE TO INTEGRATE
C
   60 CALL GEARZ(DIFFUN,N,T,H,EPS,WS(IY),WS(IFSAVE),WS(IERROR),
     ,      WS(IYMAX),WS(IPW),MF)
C
C      CHECK ERROR FLAG
C
      TLEFT=TLAST-T
      IF(KOUNT(7).GT.0) KFLAG1=-1
      IF(KFLAG1.LT.0) GOTO 115
 61   CONTINUE
C
C     IFDISCO IS SET TRUE CALL USER ROUTINE AFTER A GOOD STEP
C
      IF(.NOT.DISCO) GOTO 65
         DISC2=.TRUE.
      KOUNT(1)=KOUNT(1)+1
         CALL DIFFUN(N,T,WS(IY),WS(IFSAVE))
   65 DISC2=.FALSE.
      IF(IOUT.NE.0) GOTO 85
         ICONV=1
      KOUNT(2)=KOUNT(2)+1
C
C*    CHECK PRINT TIME AND INTERPOLATE OR EXTRAPOLATE AS RESTART RQUIRES
C
      IF(TLEFT.GT.0.) GO TO 60
 70   ICONV=1
      IF(RSTART)85,85,80
 80   H=T-TLAST
         JSTART=-1
      RSTART=-1
         GOTO 60
   85 S=TLEFT/ABS(H)
      DO 90 I=1,N
   90 Y(I)=WS(I)
      R=1.0
      L=NQ+1
      DO 100 J=2,L
         R=R*S
         DO 95 I=1,N
   95    Y(I)=Y(I)+WS(N*(J-1)+I)*R
  100 CONTINUE
  105 TSAVE=T
      T=TLAST
      IF(ICALL.GE.0) ICALL=NQ
  110 RETURN
C
C        ERROR PROCESSING
C
C      ERROR REPORTING FROM GEARZ
C
C
C*
  115 KFLAG1=-KFLAG1
*IF -DEF,AELIB5
      GO TO (120,130,140,150,160,170,180),KFLAG1
*ENDIF
*IF DEF,AELIB5
      GO TO (120,130,140,150,160,170,180),KFLAG1
      CALL GOTOER
*ENDIF
C
C        A NUMBER OF STEPS WERE TAKEN AT DTMAX
C        SO EFFICIENCY HAS BEEN DEGRADED
*IF -DEF,AELIB5
 120  IF(IPRINT.EQ.0) CALL AELERR(MES(1),2HNF)
*ENDIF
*IF DEF,AELIB5
 120  IF(IPRINT.EQ.0) CALL AELERR(MES(1),'NF')
*ENDIF
      ICALL=-1
      GO TO 61
C
C        INTEGRATION FAILED TO SATISFY THE ERROR TEST AT DTMIN
*IF -DEF,AELIB5
 130  IF(IPRINT.EQ.0) CALL AELERR(MES(2),2HNF)
*ENDIF
*IF DEF,AELIB5
 130  IF(IPRINT.EQ.0) CALL AELERR(MES(2),'NF')
*ENDIF
      KOUNT(5)=KOUNT(5)+1
      ICALL=-2
 135  IF(KHFLAG.EQ.3) GO TO 160
      KHFLAG=KHFLAG+1
      HMIN=HMIN*.1
      H=H*EPS*10.
      JSTART=-1
      GO TO 60
C
C        CORRECT CONVERGENCE WAS NOT ACHIEVED AT DTMIN
C        A TOTAL OF KOUNT(6) STEPS
*IF -DEF,AELIB5
 140  IF(IPRINT.EQ.0) CALL AELERR(MES(3),2HNF)
*ENDIF
*IF DEF,AELIB5
 140  IF(IPRINT.EQ.0) CALL AELERR(MES(3),'NF')
*ENDIF
      KOUNT(6)=KOUNT(6)+1
      ICALL=-3
      GO TO 135
C
C        SPARSITY DETECTED TO BE INCREASING - RESTRUCTURING HALTED
*IF -DEF,AELIB5
 150  IF(IPRINT.EQ.0) CALL AELERR(MES(4),2HNF)
*ENDIF
*IF DEF,AELIB5
 150  IF(IPRINT.EQ.0) CALL AELERR(MES(4),'NF')
*ENDIF
      ICALL=-4
      GO TO 61
C
C        ERROR CRITERION TO STRICT
*IF -DEF,AELIB5
 160  CALL AELERR(MES(5),2HNF)
*ENDIF
*IF DEF,AELIB5
 160  CALL AELERR(MES(5),'NF')
*ENDIF
      ICALL=-5
      RETURN
C
C        ILLEGAL VALUE OF N,EPS, OR MF
*IF -DEF,AELIB5
 170  CALL AELERR(MES(6),2HNF)
*ENDIF
*IF DEF,AELIB5
 170  CALL AELERR(MES(6),'NF')
*ENDIF
      KFLAG1=-6
      ICALL=-6
      RETURN
C
C        INSUFFICIENT STORAGE IN JACOBIAN SPARSE.
C        INCREASE ML AND NWS
*IF -DEF,AELIB5
 180  CALL AELERR(MES(7),2HNF)
*ENDIF
*IF DEF,AELIB5
 180  CALL AELERR(MES(7),'NF')
*ENDIF
      ICALL=-7
      RETURN
C
C        ENTER HERE WITH PREVIOUS FATAL ERROR
*IF -DEF,AELIB5
 190  CALL AELERR(MES(8),1HF)
*ENDIF
*IF DEF,AELIB5
 190  CALL AELERR(MES(8),'F')
*ENDIF
      RETURN
      END
      SUBROUTINE GEARZ(DIFFUN,N,T,H,EPS,Y,FSAVE,ERROR,YMAX,PW,MF)
C***********************************************************************
C*
C*   ORIGINALLY PUBLISHED AS GEAR BY A C HINDMARSH LAWRENCE RAD LAB
C*     MODIFIED BY M B CARVER TO INCLUDE THE ACH BANDED OPTION GEARB
C*        AND A SPARSE MATRIX OPTION USING J K REIDS HARWELL ROUTINES
C*           FURTHER MODIFIED BY MBC AND J LIU TO RATIONALISE ERROR BASE
C*
C*
C* GEARS PERFORMS ONE STEP OF THE INTEGRATION OF AN INITIAL VALUE
C* PROBLEM FOR A SYSTEM OF ORDINARY DIFFERENTIAL EQUATIONS.
C* COMMUNICATION WITH STIFF IS DONE WITH THE FOLLOWING VARIABLES...
C*
C*   DIFFUN= NAME OF EXTERNAL FUNCTION WHICH WOULD EVALUATE THE RIGHT
C*           HAND SIDE OF THE DIFFERENTIAL EQUATIONS. THE CALLING
C*           SEQUENCE SHOULD BE DIFFUN(N,T,Y,DY)
C*   N     = THE NUMBER OF FIRST ORDER DIFFERENTIAL EQUATIONS.
C*   T     = THE INDEPENDENT VARIABLE. T IS UPDATED ON EACH STEP TAKEN
C*   H     = THE STEP SIZE TO BE ATTEMPTED ON THE NEXT STEP. H MAY BE
C*           ADJUSTED UP OR DOWN BY THE ROUTINE IN ORDER TO ACHIEVE AN
C*           ECONOMICAL INTEGRATION. HOWEVER IF THE H PROVIDED BY THE
C*           USER DOES NOT CAUSE A LARGER ERROR THAN REQUESTED, IT WILL
C*           BE USED. TO SAVE COMPUTER TIME, THE USER IS ADVISED TO USE
C*           A FAIRLY SMALL STEP FOR THE FIRST CALL. IT WILL BE
C*           AUTOMATICALLY INCREASED LATER. H CAN BE EITHER POSITIVE OR
C*           NEGATIVE, BUT ITS SIGN MUST REMAIN CONSTAND THROUGHOUT THE
C*           PROBLEM
C*   EPS   = THE RELATIVE ERROR TEST CONSTANT. SINGLE STEP ERROR
C*           ESTIMATES DIVIDED BY YMAX(I) MUST BE LESS THAN THIS IN THE
C*           EUCLIDEAN NORM. THE STEP AND/OR ORDER IS ADJUSTED TO
C*           ACHIEVE THIS.
C*   Y     = AN N BY LMAX ARRAY CONTAINING THE DEPENDENT VARIABLES
C*           AND THEIR SCALED DERIVATIVES. LMAX IS CURRENTLY 6 FOR BOTH
C*           THE ADAMS AND THE GEARS METHODS. LMAX-1
C*           IS MAXDER , THE MAXIMUM ORDER USED . SEE SUBROUTINE COSET .
C*           Y(I,J+1) CONTAINS THE J-TH DERIVATIVE OF Y(I) , SCALED BY
C*           H**J/FACTORIAL(J) . ONLY Y(I) , 1 .LE. I .LE. N , NEED BE
C*           SET BY THE CALLING PROGRAM ON THE FIRST ENTRY . IF IT
C*           IS DESIRED TO INTERPOLATE TO NON - MESH POINTS , THE Y
C*           ARRAY CAN BE USED . IF THE CURRENT STEP SIZE IS H AND
C*           THE VALUE AT T + E IS NEEDED , FORM S = E/H , AND THEN
C*           COMPUTE...
C*                           NQ
C*           Y(I) (T+E) = SUM Y(I,J+1)*S**J .
C*                           J=0
C*           THE Y ARRAY SHOULD NOT BE ALTERED BY THE CALLING PROGRAM .
C*           WHEN REFERENCING Y AS A 2-DIMENSIONAL ARRAY , USE A
C*           COLUMN LENGTH OF NO, AS THIS THE THE VALUE USED IN STIFF.
C*   ERROR = AN ARRAY OF N ELEMENTS PROPORTIONAL TO THE ESTIMATED ONE
C*           STEP ERROR IN EACH COMPONENT.
C*   YMAX  = AN ARRAY OF N LOCATIONS WHICH CONTAINS THE MAXIMUN OF
C*           (CURRENT Y(I), YCUT). IT IS USED AS THE BASE FOR
C*           CALCULATING RELATIVE ERROR.
C*   PW    = A BLOCK OF LOCATIONS USED FOR PARTIAL DERIVATIVES IF MITER
C*           IS NOT 0. IF MITER IS 1 OR 2 , ITS LENGTH MUST BE AT LEAST
C*           N*(N+1),FOR THE JACOBIAN AND PIVOT INFORMATION. IF MITER=3
C*           ITS LOCATION MUST BE ONLY BE AT LEAST N.
C*           MITER=4 REQUIRES N*(2*ML+MU+2)
C*           MITER=5 REQUIRES (3*ML+18)*N
C*
C*           ALLOW IA=NSPARS*N NONZEROS IN JACOBIAN AND PACK PW AS
C*           FOLLOWS -- A(IA),IRN(IA,2),IG(N1,2),IRN2(IA,2),IP2(N1,6),
C*                      IW(N1,2),IH(7N1)
C*   MF    = THE METHOD FLAG . MF IS A POSITIVE INTEGER WITH
C*           TWO DECIMAL DIGITS--METH AND MITER (MF=10*METH+MITER) .
C*           METH IS THE BASIC METHOD INDICATOR...
C*            METH  = 1 MEANS THE ADAMS METOHS .
C*            METH  = 2 MEANS THE STIFF SYSTEMS METHODS OF GEAR .
C*           MITER IS THE INTERGRATION METHOD INDICATOR ...
C*            MITER = 0 MEANS FUNCTIONAL INTERATION (NO PARTIAL
C*                      DERIVATIVES NEEDED) .
C*            MITER = 1 MEANS CHORD METHOD WITH ANALYTIC JACOBIAN .
C*                      FOR THIS USER SUPPLIES SUBROUTINE
C*                      PEDEPV(N,T,Y,PD,NO) . PD IS AN N BY N ARRAY ,
C*                      STORED AS AN NO BY NO ARRAY . PD(I,J) IS
C*                      TO BE SET TO THE PARTIAL DERIVATIVE OF
C*                      YDOT(I) WITH RESPECT TO Y(J) .
C*                       HOWEVER, THIS OPTION IS BEING DEACTIVATED AT
C*                       PRESENT.
C*            MITER = 2 MEANS CHORD METHOD WITH JACOBIAN
C*                      CALCULATED IN STIFF BY FINITE DIFFERENCES .
C*                      PEDERV IS NOT CALLED AND A DUMMY CAN BE USED .
C*            MITER = 3 MEANS CHORD METHOD WITH JACOBIAN REPLACED
C*                      BY A DIAGONAL APPROXIMATION BASED ON A
C*                      DIRECTIONAL DERIVATIVE .
C*          MITER = 4 IS GEARB  METHOD WITH JACOBIAN TREATED AS BANDED
C*          MITER = 5 IS CARVER METHOD WITH JACOBIAN TREATED AS SPARSE
C*    ML,MU =   THE WIDTHS OF THE LOWER AND UPPER PARTS, RESPECTIVELY,
C*               OF THE BAND IN THE JACOBIAN MATRIX, NOT COUNTING THE
C*               MAIN DIAGONAL.  THE FULL BANDWIDTH IS ML + NU + 1.
C*              IF MITER=5 THE NUMBER OF PERMITTED NONZEROS IS ML*N
C*           AND THE JACONIAN STRUCTURE IS REASSESSED EVERY MU EVALUATNS
C*            UNLESS MU IS NEG WHEN IT IS ASSESSED ONLY INITIALLY
C*   HMIN  = THE MINIMUM ABSOLUTE VALUE OF THE STEP SIZE THAT WILL BE
C*           USED FOR THE PROBLEM. OF STARTING THIS MUST BE MUCH SMALLER
C*           THAN THE AVERAGE ABS(H) EXPECTED, SINCE A FIRST ORDER
C*           METHOD IS USED INITIALLY
C*   HMAX  = THE MAXIMUM ABSOLUTE VALUE OF THE STEP SIZE THAT WILL BE
C*           USED FOR THE PROBLEM.
C*   KFLAG = A COMPLETION CODE WITH THE FOLLOWING MEANINGS...
C*                      0 THE STEP WAS SUCCESSFUL .
C*                     -1 THE REQUESTED ERROR COULD NOT BE ACHIEVED
C*                        WITH ABS(H) =HMIN .
C*                     -2 THE REQUESTED ERROR IS SMALLER THAN CAN
C*                        BE HANDLED FOR THIS PROMBLEM .
C*                     -3 CORRECTOR CONVERGENCE COULD NOT BE
C*                        ACHIEVED FOR ABS(H) .GT. HMIN .
C*                       -4 PROBLEMS WITH SPARSE MATRIX ROUTINES STORAGE
C*           ON A RETURN WITH KFLAG NEGATIVE , THE VALUES OF T AND
C*           THE Y ARRAY ARE AS OF THE BEGINNING OF THE LAST STEP .
C*           , AND H IS THE LAST STEP SIZE ATTEMPTED .
C*   JSTART= AN INTEGER USED ON INPUT AND OUTPUT .
C*           ON INPUT , IT HAS THE FOLLOWING VALUES AND MEANINGS...
C*                     0 PREFORM THE FIRST STEP . THIS VALUE ENABLES
C*                       THE SUBROUTINE TO INITIALIZE ITSELF .
C*                 .GT.0 TAKE A NEW STEP CONTINUING FROM THE LAST .
C*                       ASSUMES THE LAST STEP WAS SUCCESSFUL AND
C*                       USER HAS NOT CHANGED ANY PARAMETERS .
C*                 .LT.0 TAKE THE NEXT STEP WITH A NEW VALUE OF
C*                       H AND/OR EPS AND/OR MF . THIS MAY BE
C*                       EITHER IN REDOING A STEP THAT FAILED , OR
C*                       IN CONTINUING FROM A SUCCESSFUL STEP .
C*           ON EXIT , JSTART IS SET TO NQ , THE CURRENT ORDER OF THE
C*           METHOD . THIS IS ALSO THE ORDER OF THE MAXIMUM DERIVATIVE
C*           AVAILABLE IN THE Y ARRAY . AFTER A SUCCESSFUL STEP ,
C*           JSTART MAY NOT BE RESET FOR THE NEXT CALL .
C*
C* ADDITIONAL SUBROUTINES REQUIRED ARE...
C* COSET(METH,NQ,EL,TQ,MAXDER)    SETS METHOD COEFFICIENTS . IT IS
C*                                PROVIDED IN THE PACKAGE .
C*     DECOMP      LU DECOMPOSITION FULL MATRIX
C*     DECB        LU DECOMPOSITION BAND MATRIX
C*     SPARSE      LU DECOMPOSITION SPARSE ATRIX
C*     SOLVE       DO A.X=B SOLUTION OF DECOMPOSED FULL MATRIX
C*     SOLB        DO A.X=B SOLUTION OF DECOMPOSED BAND MATRIX
C*     SPARSEB     DO A.X=B SOLUTION OF DECOMPOSED SPARSE ATRIX
C*     JACOB       CALCULATE JACOBIAN AS SPARSE MATRIX
C*     DIFFUM      INTERFACE JACOB AND DIFFUM    CALLED BY JACOB
C*     SORTAG      SORT PIVOTAL SEQUENCE CALLED BY SPARSE
C*
C* THE CALLING PROGRAM MUST CONTAIN THE FOLLOWING COMMON
C* DEDECLARATIONS , IN WHICH THE LENGTHS ARE AS GIVEN BY THE DESCRIPTION
C* OF THE VARIABLES ABOVE . THE OPTIONAL ROUTINE DRIVES MAY BE
C* USED FOR THIS PURPOSE . THE CONSTANT ANOISE IN THE FOLLOWING
C* DATA STATEMENT SHOULD BE SET TO THE NOISE LEVEL OF THE MACHINE .
C***********************************************************************
      COMMON/CNTROL/IOUT,ICONV,KFLAG
      COMMON/INT1/NQ,ZERO,ONE,KJ
      COMMON/JACK/JACK
      COMMON/BASINT/YCUT,DYCUT,HMAX,HMIN,ML,MU
      REAL Y(N,6),FSAVE(2,N),ERROR(N),YMAX(N),PW(N)
      DIMENSION EL(13),TQ(4)
      COMMON/SPARS/LP,ANOISE,ISCALE,JSCALE
     ,      /STIFS/RSTART,JSTART,MAXDER
     ,      /STAT/KOUNT(7)
     ,      /YSET/AZERO,YSTART,YMOD
      LOGICAL FULL
      EQUIVALENCE(PNZ,NZZ)
      EXTERNAL DIFFUN
      EQUIVALENCE(SPW,ISPW)
      DATA TOOBIG/1.0E+300/
      KFLAG = 0
      TOLD  = T
      JACK=0
      IF (JSTART.GT.0) GOTO 55
      IF (JSTART.NE.0) GOTO 10
C***********************************************************************
C* ON THE FIRST CALL , THE ORDER IS SET TO 1 AND THE INITIAL
C* DERIVATIVES ARE CALCULATED. YMAX IS INITIALIZED USING THE MAXIMUM
C* OF YCUT, Y AND YDOT. DEFAULT VALUE FOR YCUT IS 1.E-14.
C* RMAX IS THE MAXIMUM RATIO BY WHICH H CAN BE INCREASED
C* IN A SINGLE STEP . IT IS INITIALLY 1.E4 TO COMPENSATE FOR THE SMALL
C* INITIAL H , BUT THEN IS NORMALLY EQUAL TO 10 . IF A FAILURE
C* OCCURS (IN CORRECTOR CONVERGENCE OR ERROR TEST) , RMAX IS SET AT 2
C* FOR THE NEXT INCREASE . EPSJ IS USED AS THE RELATIVE INCREMENT TO
C* Y WHEN GETTING PARTIALS BY FINE DIFFERENCING .
C***********************************************************************
      NO=N
      NSQ  = NO*NO
      NSQ1 = NSQ +1
      N1   = NO + 1
C
C     PARAMETERS FOR BANDED MATRIX ANALYSIS
C
      NM1= NO-1
      NOML=NO*ML
      NOW=NO*(ML+MU+1)
      NOW1=NOW+NOML+1
C
C     PARAMETERS FOR SPARSE MATRIX ANALYSIS
C
      IJAM=MU
      IPR=0
      HMAXXJ=-.01
      IAJ=N*ML
      IRNJ=IAJ+1
      IPJ=IRNJ+IAJ
      IGJ=IPJ+N1
      IRN2J=IGJ+2*N1
      IP2J=IRN2J+2*IAJ
      IWJ=IP2J+6*N1
      IHJ=IWJ+2*N1
      IZJ=IHJ+N
      IJAC=0
      U=.25
      ISCALE=-1
      JSCALE=1
C
C     CALL DIFFUN TO DEFINE Y ARRAY AND YMAX
C
      KOUNT(1)=KOUNT(1)+1
      CALL DIFFUN(N,T,Y,FSAVE)
      DO 5 I=1,N
*IF -DEF,AELIB5
         Y(I,2) = FSAVE(I)*H
         AYI   = ABS(Y(I))
*ENDIF
*IF DEF,AELIB5
         Y(I,2) = FSAVE(I,1)*H
         AYI   = ABS(Y(I,1))
*ENDIF
    5 YMAX(I)=AMAX1(AYI,YMAX(I))
      NQ = 1
      L = 2
      RMAX = 1.E4
      EPSJ = SQRT(ANOISE)
      CRATE = 1.00
      OLDLO = 1.00
      RC = 0.0
      MFOLD = 0
      MITER=0
      IWEVAL=0
      METH = 0
      HOLD = H
C***********************************************************************
C* IF THE CALLER HAS CHANGED METH , OR IF JSTART = 0 , COSET IS CALLED
C* TO SET THE COEFFIENTS OF THE METHOD . IF THE CALLER HAS CHANGED
C* EPS OR METH , THE CONSTANTS E , EDN , EUP , AND BND MUST BE RESET .
C* E IS A COMPARISON FOR ERRORS OF THE CURRENT ORDER NQ . EUP IS
C* TO TEST FOR INCREASING THE ORDER , EDN FOR DECREASING THE ORDER .
C* BND IS USED TO TEST FOR CONVERGENCE OF THE CORRECTOR ITERATES .
C* IF THE CALLER HAS CHANGED H , Y MUST BE RESCALED .
C* IF H OR METH HAS BEEN CHANGED , IDOUB IS RESET TO L+1 TO PREVENT
C* FURTHER CHANGES IN H FOR THAT MANY STEPS . ALSO , RC IS RESET .
C* RC IS THE RATIO OF NEW TO OLD VALUES OF THE COEFFICIENT L(0)*H.
C* WHEN RC DIFFERS FROM 1 BY MORE THAN 30 PERCENT , OR THE CALLER HAS
C* CHANGED MITER , IWEVAL IS SET TO MITER TO FORCE THE PARTIALS TO BE
C* UPDATED , IF PARTIALS ARE USED .
C***********************************************************************
   10 IF (MF.EQ.MFOLD) GOTO 35
      MEO   = METH
      MIO   = MITER
      METH  = MF/10
      MITER = MF-10*METH
      MFOLD = MF
      IF (MITER.NE.MIO) IWEVAL = MITER
   15 FORMAT(10E12.4)
   20 FORMAT(10I10)
C
      IF (METH.EQ.MEO) GOTO 35
         IDOUB = L+1
         IRET  = 1
   25    CALL COSET(METH,NQ,EL,TQ,MAXDER)
         RC    = RC*EL(1)/OLDLO
         OLDLO = EL(1)
   30    EDN   = (TQ(1)*EPS)**2
         E     = (TQ(2)*EPS)**2
         EUP   = (TQ(3)*EPS)**2
         BND   = (TQ(4)*EPS)**2
*IF -DEF,AELIB5
         GOTO(40,45,55),IRET
*ENDIF
*IF DEF,AELIB5
         GOTO(40,45,55),IRET
      CALL GOTOER
*ENDIF
   35 IF (EPS.EQ.EPSOLD) GOTO 40
         IRET  = 1
         GOTO 30
   40 LMAX  = MAXDER+1
      EPSOLD= EPS
      IF (H.EQ.HOLD) GOTO 55
      RH    = H/HOLD
      H     = HOLD
   45 RH    = AMAX1(RH,HMIN/ABS(H))
      RH    = AMIN1(RH,HMAX/ABS(H),RMAX)
      R1    = 1.00
      DO 50 J=2,L
         R1    = R1*RH
         DO 50 I=1,N
   50 Y(I,J)= Y(I,J)*R1
      H     = H*RH
      RC    = RC*RH
      IF(H.EQ.HMAX) KOUNT(7)=KOUNT(7)+1
      IDOUB = L+1
      IF (T.NE.TOLD) GOTO 380
C***********************************************************************
C* THIS SECTION COMPUTES THE PREDICTED VALUES BY EFFECTIVELY
C* MULTIPLYING THE Y ARRAY BY THE PASCAL TRIANGLE MATRIX .
C***********************************************************************
   55 IF (ABS(RC-1.00).GT.0.30) IWEVAL = MITER
      T     = T+H
      DO 60 J1=1,NQ
         DO 60 J2=J1,NQ
            J     = NQ-J2+J1
            DO 60 I=1,N
   60 Y(I,J)= Y(I,J)+Y(I,J+1)
C***********************************************************************
C* UP TO 3 CORRECTOR ITERATIONS ARE TAKEN . CONVERGENCE IS TESTED
C* BY REQIRING CHANGES TO BE LESS THAN BND , WHICH IS DEPENDENT ON
C* EPS , IN EUCLIDEAN NORM . THE SUM OF THE CORRECTIONS IS ACCUMULATED
C* IN THE VECTOR ERROR(I) . IT IS APPROXIMATELY EQUAL TO THE L-TH
C* DERIVATIVE OF Y MULTIPLIED BY H**L/(FACTORIAL(L-1)*EL(L)) , AND IS
C* THUS PROPORTIONAL TO THE ACTUAL ERRORS TO THE LOWEST POWER OF
C* H PRESENT (H**L) .
C* THE Y ARRAY IS NOT ALTERED IN THE CORRECTION LOOP . THE UPDATED
C* Y VECTOR IS STORED TEMPORARILY IN FSAVE . THE NORM OF THE
C* ITERATE DIFFERENCE IS STORED IN D .
C***********************************************************************
   65 DO 70 I=1,N
   70 ERROR(I) = 0.0
      M = 0
      KOUNT(1)=KOUNT(1)+1
*IF -DEF,AELIB5
      CALL DIFFUN(N,T,Y,FSAVE(N1))
*ENDIF
*IF DEF,AELIB5
      CALL DIFFUN(N,T,Y,FSAVE(N1,1))
*ENDIF
      IF (IWEVAL.LE.0) GOTO 180
C***********************************************************************
C* IF NECESSARY , THE PARTIALS ARE RE-EVALUATED PRIOR TO STARTING THE
C* CORRECTOR ITERATION . IWEVAL IS THEN SET TO 0 AS AN INDICATOR
C* THAT THIS HAS BEEN DONE .
C* IF MITER = 1 OR 2 , THE MATRIX P=I-L(0)*H*JACOBIAN IS STORED
C* IN PW AND SUBJECTED TO LU DECOMPOSITION , WITH RESULTS ALSO
C* STORED IN PW . IF MITER = 3 , THE MATRIX USED IS P = I-L(0)*H*D ,
C* WHERE D IS A DIAGONAL MATRIX .
C***********************************************************************
      JACK=1
      KOUNT(4)=KOUNT(4)+1
*IF -DEF,AELIB5
      GOTO(115,115,165,110,75)MITER
*ENDIF
*IF DEF,AELIB5
      GOTO(115,115,165,110,75)MITER
      CALL GOTOER
*ENDIF
C
C      MITER = 5   SPARSE  JACOBIAN ANALYSIS
C
   75 D=0.
      DO 80 I=1,N
*IF -DEF,AELIB5
   80 D=D+FSAVE(I+NO)**2
*ENDIF
*IF DEF,AELIB5
   80 D=D+FSAVE(I+NO,1)**2
*ENDIF
      RO=ABS(H)*SQRT(D)*1.0E03*ANOISE
      DO 85 I=1,N
   85 PW(IHJ+I-1)=AMAX1(RO,EPSJ*YMAX(I))
      PW(IZJ)=-EL(1)*H
*IF -DEF,AELIB5
      CALL JAKOB(NO,N,PW(IRNJ),PW(IPJ),DIFFUN,Y,FSAVE(N1),T,PW(IHJ),
     ,     HMAXXJ ,PW(IWJ),PW,IAJ,PW(IGJ),PW(IZJ),IJAC)
*ENDIF
*IF DEF,AELIB5
      CALL JAKOB(NO,N,PW(IRNJ),PW(IPJ),DIFFUN,Y,FSAVE(N1,1),T,PW(IHJ),
     ,       HMAXXJ ,PW(IWJ),PW,IAJ,PW(IGJ),PW(IZJ),IJAC)
*ENDIF
      SPW=PW(IPJ)
      KJS=KJ
      IF(ISPW.EQ.-1) GOTO 280
      IF(FLOAT(IAJ-KJ).GT..05*FLOAT(IAJ).OR.IPR.EQ.1) GOTO 90
         IJAM=-1
      KFLAG=-4
         IPR=1
   90 CONTINUE
      DO 95 I=1,N1
   95 PW(IP2J+I-1)=PW(IPJ+I-1)
      PNZ=PW(IGJ-1)
      DO 100 I=1,NZZ
  100 PW(IRN2J +I-1)=PW(IRNJ+I-1)
      CALL SPARSE(PW,PW(IRN2J),PW(IP2J),N,N1,AGRO,U,IAJ)
      ISPW=INT(AGRO)
      KJS=KJ
      IF(ISPW.EQ.-1) GO TO 280
C
C     PICK UP JACOBIAN STRUCTURE EVERY IJAM EVALUATIONS
C
      IJAC=IJAC+1
      IF(IJAC.EQ.IJAM)IJAC=0
      MTYPE=1
  105 MATRIX=1
      IWEVAL=0
      RC=1.
      GOTO 190
C
C      MITER = 4   BANDED  JACOBIAN ANALYSIS
C
*IF -DEF,AELIB5
  110 FULL=.F.
*ENDIF
*IF DEF,AELIB5
  110 FULL=.FALSE.
*ENDIF
      J1=NOML
      NMI=NM1
      MATRIX=0
      GOTO 120
C
C      MITER = 2   FULL    JACOBIAN ANALYSIS
C
*IF -DEF,AELIB5
  115 FULL=.T.
*ENDIF
*IF DEF,AELIB5
  115 FULL=.TRUE.
*ENDIF
      J1=-NO
      NMI=-1
      I1=1
      I2=N
      MATRIX=-1
  120 D=0.
      DO 125 I=1,N
*IF -DEF,AELIB5
  125 D     = D+FSAVE(I+NO)**2
*ENDIF
*IF DEF,AELIB5
  125 D     = D+FSAVE(I+NO,1)**2
*ENDIF
      RO    = ABS(H)*SQRT(D)*1.E03*ANOISE
      DO 140 J=1,N
         J1    = J1+NO
*IF -DEF,AELIB5
         YJ    = Y(J)
*ENDIF
*IF DEF,AELIB5
         YJ    = Y(J,1)
*ENDIF
         R     = EPSJ*YMAX(J)
         R     = AMAX1(R,RO)
*IF -DEF,AELIB5
         Y(J)  = Y(J)+R
*ENDIF
*IF DEF,AELIB5
         Y(J,1)  = Y(J,1)+R
*ENDIF
         D     = -EL(1)*H/R
      KOUNT(1)=KOUNT(1)+1
      KOUNT(3)=KOUNT(3)+1
         CALL DIFFUN(N,T,Y,FSAVE)
         IF(FULL) GOTO 130
            I1 = MAX0(J-MU,1)
            I2 = MIN0(J+ML,N)
  130    DO 135 I=I1,I2
*IF -DEF,AELIB5
  135    PW(J1-NMI*I)=(FSAVE(I)-FSAVE(I+NO))*D
  140 Y(J)  = YJ
*ENDIF
*IF DEF,AELIB5
  135    PW(J1-NMI*I)=(FSAVE(I,1)-FSAVE(I+NO,1))*D
  140 Y(J,1)  = YJ
*ENDIF
C
C     LINEAR EQUATION DECOMPOSITION FOR FULL AND BANDED OPTIONS
C
      IWEVAL=0
      RC=1.
      IF(FULL) GOTO 150
         DO 145 I=1,N
  145    PW(NOML+I)=PW(NOML+I)+1.
         CALL DECB(NO,N,ML,MU,PW,PW(NOW1),IER)
         GOTO 160
  150 DO 155 I=1,N
  155 PW(I*N1-NO) = PW(I*N1-NO)+1.00
      CALL DECOMP(NO,N,PW,PW(NSQ1),FSAVE,IER)
  160 IF(IER) 240,190,240
C
C       DIAGONAL APPROXIMATION MITER=3
C
  165 R     = EL(1)*.100
      DO 170 I=1,N
*IF -DEF,AELIB5
  170 PW(I) = Y(I)+R*(FSAVE(I+NO)*H-Y(I,2))
*ENDIF
*IF DEF,AELIB5
  170 PW(I) = Y(I,1)+R*(FSAVE(I+NO,1)*H-Y(I,2))
*ENDIF
      JACK=0
      KOUNT(1)=KOUNT(1)+1
      KOUNT(3)=KOUNT(3)+1
      CALL DIFFUN(N,T,PW,FSAVE)
      DO 175 I=1,N
*IF -DEF,AELIB5
         RO    = PW(I)-Y(I)
*ENDIF
*IF DEF,AELIB5
         RO    = PW(I)-Y(I,1)
*ENDIF
         PW(I) = 1.00
*IF -DEF,AELIB5
         D     = RO-EL(1)*H*(FSAVE(I)-FSAVE(NO+I))
         FSAVE(I) = RO/R
*ENDIF
*IF DEF,AELIB5
         D     = RO-EL(1)*H*(FSAVE(I,1)-FSAVE(NO+I,1))
         FSAVE(I,1) = RO/R
*ENDIF
         IF(ABS(RO).EQ.0.00) GOTO 175
            IF(D.EQ.0.00) GOTO 240
            PW(I) = RO/D
*IF -DEF,AELIB5
            FSAVE(I)=FSAVE(I)*PW(I)
*ENDIF
*IF DEF,AELIB5
            FSAVE(I,1)=FSAVE(I,1)*PW(I)
*ENDIF
  175 CONTINUE
      IWEVAL= 0
      RC    = 1.00
      GOTO 215
  180 IF(MITER.EQ.3) GOTO 225
      IF(MITER.NE.0) GOTO 190
C***********************************************************************
C* IN THE CASE OF FUNCTIONAL ITERATION , UPDATE Y DIRECTLY FROM
C* THE RESULT OF THE LAST DIFFUN CALL .
C***********************************************************************
      D     = 0
      DO 185 I=1,N
*IF -DEF,AELIB5
         R     = H*FSAVE(I+NO)-Y(I,2)
*ENDIF
*IF DEF,AELIB5
         R     = H*FSAVE(I+NO,1)-Y(I,2)
*ENDIF
         D     = D+((R-ERROR(I))/YMAX(I))**2
*IF -DEF,AELIB5
         FSAVE(I) = Y(I)+EL(1)*R
*ENDIF
*IF DEF,AELIB5
         FSAVE(I,1) = Y(I,1)+EL(1)*R
*ENDIF
  185 ERROR(I) = R
      GOTO 235
C***********************************************************************
C* IN THE CASE OF THE CHORD METHOD , COMPUTE THE CORRECTOR ERROR ,
C* F SUB(M) , AND SOLVE THE LINEAR SYSTEM WITH THAT AS RIGHT HAND
C* SIDE AND P AS COEFFICIENT MATRIX , USING THE LU DECOMPOSITION
C* IF MITER = 1 OR 2 .
C***********************************************************************
  190 DO 195 I=1,N
*IF -DEF,AELIB5
         FSAVE(I+NO)=FSAVE(I+NO)*H-Y(I,2)-ERROR(I)
  195 FSAVE(I)=FSAVE(I+NO)
*ENDIF
*IF DEF,AELIB5
         FSAVE(I+NO,1)=FSAVE(I+NO,1)*H-Y(I,2)-ERROR(I)
  195 FSAVE(I,1)=FSAVE(I+NO,1)
*ENDIF
      IF(MATRIX) 200,205,210
*IF -DEF,AELIB5
  200 CALL SOLVE(NO,N,PW,FSAVE(N1),FSAVE,PW(NSQ1))
*ENDIF
*IF DEF,AELIB5
  200 CALL SOLVE(NO,N,PW,FSAVE(N1,1),FSAVE,PW(NSQ1))
*ENDIF
      GOTO 215
  205 CALL SOLB(NO,N,ML,MU,PW,FSAVE,PW(NOW1))
      GOTO 215
C
C      DIFFUN NOT CALLED IN SPARSEB SINCE MTYPE = 1.
C      THEREFORE ONLY TO SATISFY PARAMETER LIST LENGTH.
C
 210  CALL SPARSEB(PW,PW(IRN2J),PW(IP2J),N,N1,AGRO,MTYPE,FSAVE,PW(IWJ)
     ,      ,DIFFUN)
  215 D     = 0.0
      DO 220 I=1,N
*IF -DEF,AELIB5
         ERROR(I) = ERROR(I)+FSAVE(I)
         D     = D+(FSAVE(I)/YMAX(I))**2
  220 FSAVE(I) = Y(I)+EL(1)*ERROR(I)
*ENDIF
*IF DEF,AELIB5
         ERROR(I) = ERROR(I)+FSAVE(I,1)
         D     = D+(FSAVE(I,1)/YMAX(I))**2
  220 FSAVE(I,1) = Y(I,1)+EL(1)*ERROR(I)
*ENDIF
      GOTO 235
C
  225 DO 230 I=1,N
*IF -DEF,AELIB5
  230 FSAVE(I) = PW(I)*(FSAVE(I+NO)*H-Y(I,2)-ERROR(I))
*ENDIF
*IF DEF,AELIB5
  230 FSAVE(I,1) = PW(I)*(FSAVE(I+NO,1)*H-Y(I,2)-ERROR(I))
*ENDIF
      GOTO 215
C***********************************************************************
C* TEST FOR CONVERGENCE . IF M.GT.0 , AN ESTIMATE OF THE CONVERGENCE
C* RATE CONSTANT IS STORED IN CRATE , AND THIS IS USED IN THE TEST .
C***********************************************************************
  235 IF (M.NE.0) CRATE = AMAX1(.90*CRATE,D/D1)
      JACK=0
      IF ((D*AMIN1(1.00,2.00*CRATE)).LE.BND) GOTO 255
      D1    = D
      M     = M+1
      IF (M.EQ.3) GOTO 240
      KOUNT(1)=KOUNT(1)+1
*IF -DEF,AELIB5
         CALL DIFFUN(N,T,FSAVE,FSAVE(N1))
*ENDIF
*IF DEF,AELIB5
         CALL DIFFUN(N,T,FSAVE,FSAVE(N1,1))
*ENDIF
         GOTO 180
C***********************************************************************
C* THE CORRECTOR ITERATION FAILED TO CONVERGE IN 3 TRIES . IF PARTIALS
C* ARE INVOLVED BUT ARE NOT UP TO DATE ,THEY ARE RE-EVALUATED FOR THE
C* NEXT TRY . OTHERWISE THE Y ARRAY IS RETRACTED TO ITS VALUES
C* BEFORE PREDICTION , AND H IS REDUCED , IF POSSIBLE . IF NOT , A
C* NO - CONVERGENCE EXIT IS USED .
C***********************************************************************
  240 IF (IWEVAL.EQ.-1) GOTO 250
         T     = TOLD
         RMAX  = 2.00
         DO 245 J1=1,NQ
            DO 245 J2=J1,NQ
               J     = NQ-J2+J1
               DO 245 I=1,N
  245    Y(I,J)= Y(I,J)-Y(I,J+1)
         IF (ABS(H).LE.(HMIN*1.00001)) GOTO 370
         RH    = .250
         GOTO 45
  250 IWEVAL = MITER
      GOTO 65
C***********************************************************************
C* THE CORRECTOR HAS CONVERGED . IWEVAL IS SET TO -1 IF PARTIAL
C* DERIVATIVES WERE USED , TO SIGNAL THAT THEY MAY NEED UPDATING ON
C* SUBSEQUENT STEPS . THE ERROR TEST IS MADE AND CONTROL PASSES TO
C* STATEMENT 500 IF IT FAILS .
C***********************************************************************
  255 D     = 0.00
      DO 260 I=1,N
  260 D     = D+(ERROR(I)/YMAX(I))**2
      IF (MITER.NE.0) IWEVAL = -1
      IF (D.GT.E) GOTO 280
C***********************************************************************
C* AFTER A SUCCESSFUL STEP , UPDATE THE Y ARRAY AND YMAX .
C* CONSIDER CHANGING H IF IDOUB = 1.0 . OTHERWISE DECREASE IDOUB BY 1 .
C* IF IDOUB IS THEN 1 AND NQ .LT. MAXDER , THEN ERROR IS SAVED FOR
C* USE IN A POSSIBLE ORDER INCREASE ON THE NEXT STEP . IF A CHANGE
C* IN H IS CONSIDERED , AN INCREASE OR DECREASE IN ORDER BY ONE IS
C* CONSIDERED ALSO . A CHANGE IN H IS MADE ONLY IF IT IS BY A
C* FACTOR OF AT LEAST 1.1 . IF NOT , IDOUB IS SET TO 10 TO PREVENT
C* TESTING FOR THAT MANY STEPS .
C***********************************************************************
      KFLAG = 0
      DO 265 J=1,L
         DO 265 I=1,N
  265 Y(I,J) = Y(I,J)+EL(J)*ERROR(I)
      DO 270 I=1,N
*IF -DEF,AELIB5
 270  YMAX(I)=AMAX1(ABS(Y(I)),YCUT,DYCUT-ABS(Y(I,2)))
*ENDIF
*IF DEF,AELIB5
 270  YMAX(I)=AMAX1(ABS(Y(I,1)),YCUT,DYCUT-ABS(Y(I,2)))
*ENDIF
      IF (IDOUB.EQ.1) GOTO 290
      IDOUB = IDOUB -1
      IF (IDOUB.GT.1) GOTO 385
      IF (NQ.EQ.MAXDER) GOTO 385
      DO 275 I=1,N
  275 Y(I,LMAX) = ERROR(I)
      GOTO 385
C***********************************************************************
C* THE ERROR TEST FAILED . KFLAG KEEPS TRACK OF MULTIPLE FAILURES .
C* RESTORE T AND THE Y ARRAY TO THEIR PREVIOUS VALUES , AND PREPARE
C* TO TRY THE STEP AGAIN . COMPUTE THE OPTIMUM STEP SIZE FOR THIS OR
C* ONE LOWER ORDER .
C***********************************************************************
  280 KFLAG = KFLAG-1
      T     = TOLD
      DO 285 J1=1,NQ
         DO 285 J2=J1,NQ
            J     = NQ-J2+J1
            DO 285 I=1,N
  285 Y(I,J) = Y(I,J)-Y(I,J+1)
      IF(ISPW.EQ.-1) GOTO 375
      RMAX  = 2.00
      IF (ABS(H).LE.(HMIN*1.00001)) GOTO 360
      IF (KFLAG.LE.-3) GOTO 350
      PR3   = 1.E+20
      GOTO 300
C***********************************************************************
C* REGARDLESS OF THE SUCCESS OR FAILURE OF THE STEP , FACTORS
C* PR1 , PR2 , AND PR3 ARE COMPUTED , BY WHICH H COULD BE DIVIDED
C* AT ORDER NQ - 1 , ORDER NQ , OR ORDER NQ + 1 , RESPECTIVELY
C* IN THE CASE OF FAILURE , PR3 = 1.E20 TO AVOID AN ORDER INCREASE .
C* THE SMALLEST OF THESE IS DETERMINED AND THE NEW ORDER CHOSEN .
C* ACCORDINGLY . IF THE ORDER IS TO BE INCREASED , WE COMPUTE ONE
C* ADDITIONAL SCALED DERIVATIVE .
C***********************************************************************
  290  PR3=TOOBIG
      IF (NQ.EQ.MAXDER) GOTO 300
         D1    = 0.00
         DO 295 I=1,N
  295    D1    = D1+((ERROR(I)-Y(I,LMAX))/YMAX(I))**2
         ENQ3  = 0.5/FLOAT(L+1)
         PR3   = ((D1/EUP)**ENQ3)*1.4+1.4E-6
  300 ENQ2  = 0.5/FLOAT(L)
      PR2   = ((D/E)**ENQ2)*1.2+1.2E-6
      PR1   = TOOBIG
      IF (NQ.EQ.1) GOTO 310
         D     = 0.00
         DO 305 I=1,N
  305    D     = D+(Y(I,L)/YMAX(I))**2
         ENQ1  = 0.5/FLOAT(NQ)
         PR1   = ((D/EDN)**ENQ1)*1.3+1.3E-6
  310 IF (PR2.LE.PR3) GOTO 315
         IF (PR3.LT.PR1) GOTO 325
  315 IF (PR2.GT.PR1) GOTO 320
         NEWQ  = NQ
         RH    = 1.00/PR2
         GOTO 340
  320 NEWQ = NQ-1
      RH = 1.0/PR1
      GOTO 340
  325 NEWQ  = L
      RH    = 1.00/PR3
      IF (RH.LT.1.1) GOTO 335
         DO 330 I=1,N
  330    Y(I,NEWQ+1) = ERROR(I)*EL(L)/FLOAT(L)
         GOTO 345
  335 IDOUB = 10
      GOTO 385
  340 IF ((KFLAG.EQ.0).AND.(RH.LT.1.1)) GOTO 335
C***********************************************************************
C* IF THERE IS A CHANGE OF ORDER , RESET NQ , L , AND THE COEFFICIENTS .
C* IN ANY CASE H IS RESET ACCORDING TO RH AND THE Y ARRAY IS RESCALED .
C* THEN EXIT FROM 690 IF THE STEP WAS OKAY , OR REDO THE STEP OTHERWISE
C***********************************************************************
      IF (NEWQ.EQ.NQ) GOTO 45
  345 NQ    = NEWQ
      L     = NQ+1
      IRET  = 2
      GOTO 25
C***********************************************************************
C* CONTROL REACHES THIS SECTION IF 3 OR MORE FAILURES HAVE OCCURED .
C* IT IS ASSUMED THAT THE DERIVATIVES THAT HAVE ACCUMULATED IN THE
C* Y ARRAY HAVE ERRORS OF THE WRONG ORDER . HENCE THE FIRST
C* DERIVATIVE IS RECOMPUTED , AND THE ORDER IS SET TO 1 . THEN
C* H IS REDUCED BY A FACTOR OF 10 , AND THE STEP IS RETRIED .
C        AFTER A TOTAL OF 10 FAILURES NAD EXIT IS TAKEN WITH KFLAG=-5
C***********************************************************************
 350  IF(KFLAG.EQ.-10) GO TO 365
      RH=0.1**(-KFLAG)
      RH    = AMAX1(HMIN/ABS(H),RH)
      H     = H*RH
      KOUNT(1)=KOUNT(1)+1
      CALL DIFFUN(N,T,Y,FSAVE)
      DO 355 I=1,N
*IF -DEF,AELIB5
  355 Y(I,2) = H*FSAVE(I)
*ENDIF
*IF DEF,AELIB5
  355 Y(I,2) = H*FSAVE(I,1)
*ENDIF
      IWEVAL = MITER
      IDOUB  = 10
      IF (NQ.EQ.1) GOTO 55
      NQ    = 1
      L     = 2
      IRET  = 3
      GOTO 25
C***********************************************************************
C* ALL RETURNS ARE MADE THROUGH THIS SECTION . H IS SAVED IN HOLD
C* TO ALLOW THE CALLER TO CHANGE H ON THE NEXT STEP .
C***********************************************************************
 360  KFLAG=-2
      GOTO 385
 365  KFLAG=-5
      GOTO 385
  370 KFLAG = -3
      GOTO 385
 375  KFLAG=-7
      ISPW=0
      GOTO 385
  380 RMAX  = 10.00
  385 HOLD  = H
      JSTART= NQ
C*******************************END OF STIFF****************************
      END
      SUBROUTINE JAKOB(M,N,IRN,IP,FUNC,X,F,Y,H,HMAX,W,A,IA,IG,Z,MBD)
C
C       ROUTINE TO EVALUATE AND STORE A SPARSE JACOBIAN MATRIX PACKED
C
C     GIVEN A ROUTINE FUNC WHICH CALCULATES F(I)[X1,X2--XN] , I=1,M
C       JACOB EVALUATES PF(I)/PX(J) I=1,M , J=1,N AND PACKS NON ZERO
C       IN A FORM COMPATIBLE WITH OTHER HARWELL SPARSE MATRIX ROUTINES
C       RESULTS IN ARRAY A USING ARRAYS IRN AND IP TO INDEX THE ELEMENTS
C
C     M       THE NUMBER OF FUNCTIONS F
C     N       THE NUMBER OF VARIABLES X
C     IA      A DIMENSION  LARGER THAN THE NUMBER OF NON ZEROS EXPECTED
C     IRN     ARRAY DIMENSION IA TO HOLD POINTERS IN PACKED MATRIX
C     IP      ARRAY DIMENSION N+1 TO HOLD POINTERS
C     FUNC     ROUTINE THAT WILL BE CALLED TO EVALUATE THE FUNCTIONS F
C     X       ARRAY DIMENSION N FOR INDEPENDENT VARIABLES
C     F       ARRAY DIMENSION M FOR FUNCTIONAL RESPONSES
C         Y       INDEPENDENT VARIABLE IN EQUATIONS [ TIME MAYBE ]
C     H       ARRAY DIMENSION  N TO HOLD PERTURBATION SIZE FOR EACH X
C     HMAX    ARRAY DIMENSION N TO HOLD MAX PERTURBATION FOR EACH X
C     W       ARRAY DIMENSION M+N FOR WORKSPACE
C     A       ARRAY DIMENSION IA TO STORE THE PACKED JACOBIAN PF/PX
C     IG      ARRAY DIMENSION 2N+1 FOR WORSPACE
C     Z       ARRAY DIMENSION N    FOR WORSPACE
C     MBD     CONTROL
C               LT 0 - ESTABLISH SPARSITY STRUCTURE OF BANDED MATRIX
C                                                      BAND WIDTH /MBD/
C               EQ 0 - ESTABLISH SPARSITY STRUCTURE OF FULL MATRIX
C               GT 0 EVALUATE JACOBIAN USING STRUCTURE PREVIOUSLY FOUND
C
C      MATRIX A(I,J) IS STORED BY COLUMNS J IE A11 A31 A22 A42 A33  ETC
C      IRN  CONTAINS THE ROW NUMBERS I ABOVE  1 3 2 4 3   ETC
C      IP   CONTAINS THE POSITION IN A OF START OF EACH COLUMN J _ 1,3,5
C      IP(N+1) HAS THE NUMBER OF NONZEROS IN A AND IRN ON RETURN
C
C
C    INTHE COMMON BLOCK /SPARS/
C    LP    THE OUTPUT UNIT NO
C    EPS   THE ROUND OFF ERROR
C    ISCALE OP CONTROL  <0 MODIFY ELEMENTS FOR GEARS ROUTINE
C                       =0 NORMAL
C                       >0 ADJUST H
C
C      ADAPTED FROM THE HARWELL ROUTINE TDO2A/D AERE-R-7293 [ REID ]
C
C           M CARVER  DEC 1974
C
      COMMON/SPARS/LP,EPS,ISCALE,JSCALE
      COMMON/INT1/NQ,ZERO,ONE,K
      COMMON/STAT/KOUNT(7)
      INTEGER IRN(1),IP(1),IG(1)
      REAL HMAX(N),H(N),Z(1),HM
     ,,     X(N),F(M),A(1),W(1),Y(1)
      LOGICAL ADJUST,DIAG
C
      DATA DIAG,ADJUST,EPS1/.FALSE.,.FALSE.,1.0E-14/
C
C     IG(J) POINTS TO THE FIRST ENTRY OF GROUP J
C
      IF(ISCALE) 5,15,10
    5 DIAG=.TRUE.
      ADJUST=.FALSE.
      GOTO 15
   10 DIAG=.FALSE.
      ADJUST=.TRUE.
   15 IF(MBD.EQ.0) GOTO 120
   20 NFUN=0
      N1=N+1
C
C     IG(N1+J),J=1,2,... ARE COLUMN NUMBERS FOR GROUP 1 THEN GROUP 2 ETC
C
      IF(IG(1).NE.0) GOTO 55
      ICC=1
      DO 25 J=1,N
         IG(J+1)=0
         MJ=M+J
   25 W(MJ)=ZERO
C
C     W(M+J) IS ZERO IF COLUMN J HAS NOT BEEN INCLUDED IN A GROUP YET
C
      DO 50 NC=1,N1
         IG(NC)=ICC
         DO 30 I=1,M
   30    W(I)=ZERO
C
C     W(I),I=1,M HOLDS THE BOOLEAN PATTERN OF THE UNION OF THE COLUMNS
C     SO FAR INCLUDED IN GROUP NC
C
         DO 45 J=1,N
            MJ=M+J
            IF(W(MJ).NE.ZERO) GOTO 45
            K1=IP(J)
            K2=IP(J+1)-1
            IF(K2.LT.K1) GOTO 45
            DO 35 K=K1,K2
               IW=IRN(K)
               IF(W(IW).NE.ZERO) GOTO 45
   35       CONTINUE
C
C     ACCEPT COLUMN
C
            DO 40 K=K1,K2
               IW=IRN(K)
   40       W(IW)=ONE
            N1IC=N1+ICC
            IG(N1IC)=J
            ICC=ICC+1
            MJ=M+J
            W(MJ)=ONE
   45    CONTINUE
         IF(ICC.EQ.IG(NC)) GOTO 55
   50 CONTINUE
C
C     PRESERVE X IN W(M+1),...,W(M+N).   CHECK ALL H(J).
C
   55 DO 60 J=1,N
         HM=-HMAX(1)
         IF(HM.LT.0.)HM=HMAX(J)
         XJ=ABS(X(J))
         IF(ADJUST)H(J)=(XJ+AMAX1(EPS*XJ,AMIN1(H(J),HM),EPS1*HM))-XJ
         JM=J+M
   60 W(JM)=X(J)
C
C     Z(J) HOLDS MAXIMUM OF ESTIMATED RATIO OF TRUNCATION ERROR TO
C      ROUNDOFF ERROR IN COLUMN J,J=1,2,...,N
C
   65 IF(DIAG) GOTO 75
         DO 70 J=1,N
   70    Z(J)=1.
C
C     FIND INITIAL APPROXIMATE DERIVATIVES
C
   75 DO 110 NG=1,N
         L1=IG(NG)
         L2=IG(NG+1)-1
         IF(L2.LT.L1) GOTO 115
         DO 80 L=L1,L2
            LN1=L+N1
            IH=IG(LN1)
            IF(H(IH)) 80,80,85
   80    CONTINUE
         GOTO 110
   85    DO 90 L=L1,L2
            LN1=L+N1
            J=IG(LN1)
            H(J)=ABS(H(J))
   90    X(J)=X(J)+H(J)
      KOUNT(1)=KOUNT(1)+1
      KOUNT(3)=KOUNT(3)+1
      CALL FUNC(N,Y,X,W)
         NFUN=NFUN+1
         DO 105 L=L1,L2
            LN1=L+N1
            J=IG(LN1)
            JM=J+M
            X(J)=W(JM)
            K1=IP(J)
            K2=IP(J+1)-1
            IF(K2.LT.K1) GOTO 105
               DO 95 K=K1,K2
                  I=IRN(K)
   95          A(K)=(W(I)-F(I))/H(J)
               IF(.NOT.DIAG) GOTO 105
                  DO 100 K=K1,K2
                     A(K)=A(K)*Z(1)
                     IF(IRN(K).EQ.J)A(K)=A(K)+1.
  100             CONTINUE
                  ADJUST=.FALSE.
  105    CONTINUE
  110 CONTINUE
C
C
C     ELIMINATE ADJUST FOR FORSIM PURPOSES
C
*IF -DEF,AELIB5
  115 W=NFUN
      W(2)=NC-1
*ENDIF
*IF DEF,AELIB5
  115 W(1)=NFUN
      W(2)=NC-1
*ENDIF
      RETURN
C
C      ENTER HERE MERELY TO ESTABLIDH SPARSITY STRUCTURE IN IRN AND IP
C        IF DIAG IS TRUE
C    INCLUDE DIAGONAL TO ENSURE MATRIX CAN BE MADE DIAGONALLY DOMINANT
C               TO PRECLUDE SINGULARITY
C
C
  120 K=1
      DO 125 J=1,N
         XJ=X(J)
         X(J)=XJ+H(J)
      KOUNT(1)=KOUNT(1)+1
      KOUNT(3)=KOUNT(3)+1
      CALL FUNC(N,Y,X,W)
         X(J)=XJ
         IP(J)=K
         DO 125 I=1,M
            IF((ABS(W(I)-F(I)).LE.EPS*ABS(W(I))).AND.(I.NE.J.OR..NOT.
     ,      DIAG)) GOTO 125
               IF(K.GT.IA) GOTO 135
               IRN(K)=I
               K=K+1
  125 CONTINUE
  130 IP(N+1)=K
      IG(1)=0.
      GOTO 20
C
C        ERROR EXIT
C
  135 CONTINUE
      IP(1)=-1
      RETURN
C
C       ELIMINATE BANDED MATRIX SET UP
C
      END
      SUBROUTINE COSET(METH,NQ,EL,TQ,MAXDER)
C***********************************************************************
C* COSET IS CALLED BY STIFF AND SETS COEFFICIENTS FOR USE THERE .
C* THE VECTOR EL , OF LENGTH NQ + 1 , DETERMINES THE BASIC METHOD .
C* THE VECTOR TQ , OF LENGTH 3 , IS INVOLVED IN ADJUSTING THE STEP SIZE
C* IN RELATION TO TRUNCATION ERROR . ITS VALUES ARE GIVEN BY THE
C* PERTST ARRAY .
C* THE VECTORS EL AND TQ DEPEND ON METH AND NQ .
C* COSET ALSO SETS MAXDER , THE MAXIMUM ORDER OF THE METHOD AVAILABLE .
C* CURRENTLY IT IS 12 FOR THE ADAMS METHODS AND 5 FOR GEAR METHODS .
C* LMAX = MAXDER + 1 IS THE NUMBER OF COLUMNS IN THE Y ARRAY .
C* THE MAXIMUM ORDER USED MAY BE REDUCED SIMPLY BY CHANGING THE
C* THE NUMBERS IN STATEMENTS 1 AND 2 BELOW .
C*
C* THE COEFFICIENTS IN PERTST NEED BE GIVEN TO ONLY ABOUT
C* ONE PERCENT ACCURACY . THE ORDER IN WHICH THE GROUPS APPEAR BELOW
C* IS...  COEFFICIENTS FOR ORDER NQ - 1 , COEFFICIENTS FOR ORDER NQ ,
C* COEFFICIENTS FOR ORDER NQ + 1 . WITHIN EACH GROUP ARE THE
C* COEFFICIENTS FOR THE ADAMS METHODS , FOLLOWED BY THOSE FOR THE
C* GEAR METHODS .
C***********************************************************************
      DIMENSION PERTST(12,2,3),EL(13),TQ(4)
*IF -DEF,AELIB5
      DATA (PERTST = 1.,1.,2.,1.,.3158,.07407,.01391,.002182,
     1               .0002945,.00003492,.000003692,.0000003524,
     1               1.,1.,.5,.1667,.04167,1.,1.,1.,1.,1.,1.,1.,
     1               2.,12.,24.,37.89,53.33,70.08,87.97,106.9,
     1               126.7,147.4,168.8,191.0,
     1               2.0,4.5,7.333,10.42,13.7,1.,1.,1.,1.,1.,1.,1.,
     1               12.0,24.0,37.89,53.33,70.08,87.97,106.9,
     1               126.7,147.4,168.8,191.0,1.,
     1               3.0,6.0,9.167,12.5,1.,1.,1.,1.,1.,1.,1.,1.)
*ENDIF
*IF DEF,AELIB5
      DATA PERTST /1.,1.,2.,1.,.3158,.07407,.01391,.002182,
     1             .0002945,.00003492,.000003692,.0000003524,
     1             1.,1.,.5,.1667,.04167,1.,1.,1.,1.,1.,1.,1.,
     1             2.,12.,24.,37.89,53.33,70.08,87.97,106.9,
     1             126.7,147.4,168.8,191.0,
     1             2.0,4.5,7.333,10.42,13.7,1.,1.,1.,1.,1.,1.,1.,
     1             12.0,24.0,37.89,53.33,70.08,87.97,106.9,
     1             126.7,147.4,168.8,191.0,1.,
     1             3.0,6.0,9.167,12.5,1.,1.,1.,1.,1.,1.,1.,1./
*ENDIF
      EL(2) = 1.0
*IF -DEF,AELIB5
      GO TO (1,2),METH
*ENDIF
*IF DEF,AELIB5
      GO TO (1,2),METH
      CALL GOTOER
*ENDIF
  1   MAXDER=5
*IF -DEF,AELIB5
      GO TO (101,102,103,104,105,106,107,108,109,110,111,112),NQ
*ENDIF
*IF DEF,AELIB5
      GO TO (101,102,103,104,105,106,107,108,109,110,111,112),NQ
      CALL GOTOER
*ENDIF
   2  MAXDER = 5
*IF -DEF,AELIB5
      GO TO (201,202,203,204,205),NQ
*ENDIF
*IF DEF,AELIB5
      GO TO (201,202,203,204,205),NQ
      CALL GOTOER
*ENDIF
C**********************************************************************
C* THE FOLLOWING COEFFICIENTS SHOULD BE DEFINED TO MACHINE ACCURACY .
C* FOR EACH ORDER NQ , THEY CAN BE CALCULATED FROM THE GENERATING
C* POLYNOMIAL ,
C*      L(T) = EL(1) +EL(2)*T + ... +EL(NQ+1)*T**NQ .
C* FOR THE IMPLICIT ADAMS METHODS , L(T) IS GIVEN BY ,
C*      DL/DT = (T+1)*(T+2)*... *(T+NQ-1)/K,   L(-1)=0 ,
C*      WHERE  K = FACTORIAL(NQ-1) .
C* FOR THE GEARS METHODS ,
C*      L(T) = (T+1)*(T+2)*...*(T+NQ)/K ,
C*      WHERE K = FACTORIAL (NQ)*(1+1/2+....+1/NQ)
C*
C* THE ORDER IN WHICH THE GROUPS APPEAR BELOW IS...
C* IMPLICIT ADAMS METHODS OF ORDERS 1 TO 12 ,
C* STIFFLY STABLE GEAR METHODS OF ORDERS 1 TO 5 .
C***********************************************************************
 101  EL(1) = 1.0
      GO TO 900
 102  EL(1) = 0.5
      EL(3) = 0.5
      GO TO 900
 103  EL(1) = 4.1666666666667E-01
      EL(3) = 0.75
      EL(4) = 1.6666666666667E-01
      GO TO 900
 104  EL(1) = 0.375
      EL(3) = 9.1666666666667E-01
      EL(4) = 3.3333333333333E-01
      EL(5) = 4.1666666666667E-02
      GO TO 900
 105  EL(1) = 3.4861111111111E-01
      EL(3) = 1.0416666666667
      EL(4) = 4.8611111111111E-01
      EL(5) = 1.0416666666667E-01
      EL(6) = 8.3333333333333E-03
      GO TO 900
 106  EL(1) = 3.2986111111111E-01
      EL(3) = 1.1416666666667
      EL(4) = 0.625
      EL(5) = 1.7708333333333E-01
      EL(6) = 0.025
      EL(7) = 1.3888888888889E-03
      GO TO 900
 107  EL(1) = 3.1559193121693E-01
      EL(3) = 1.225
      EL(4) = 7.5185185185185E-01
      EL(5) = 2.5520833333333E-01
      EL(6) = 4.8611111111111E-02
      EL(7) = 4.8611111111111E-03
      EL(8) = 1.9841269841270E-04
      GO TO 900
 108  EL(1) = 3.0422453703704E-01
      EL(3) = 1.2964285714286
      EL(4) = 8.6851851851852E-01
      EL(5) = 3.3576388888889E-01
      EL(6) = 7.7777777777778E-02
      EL(7) = 1.0648148148148E-02
      EL(8) = 7.9365079365079E-04
      EL(9) = 2.4801587301587E-05
      GO TO 900
 109  EL(1) = 2.9486800044092E-01
      EL(3) = 1.3589285714286
      EL(4) = 9.7655423280423E-01
      EL(5) = 0.4171875
      EL(6) = 1.1135416666667E-01
      EL(7) = 0.01875
      EL(8) = 1.9345238095238E-03
      EL(9) = 1.1160714295714E-04
      EL(10) = 2.7557319223986E-06
      GO TO 900
 110  EL(1) = 2.8697544642857E-01
      EL(3) = 1.4144841259841
      EL(5) = 4.9856701940035E-01
      EL(4) = 1.0772156084656
      EL(6) = 0.1484375
      EL(7) = 2.9060570987654E-02
      EL(8) = 3.7202380952381E-03
      EL(9) = 2.9968584656085E-04
      EL(10) = 1.3778659611993E-05
      EL(11) = 2.7557319223986E-07
      GO TO 900
 111  EL(1) = 2.8018953644394E-01
      EL(3) = 1.4644841269841
      EL(4) = 1.1715145502646
      EL(5) = 5.7935819003527E-01
      EL(6) = 1.8832286155203E-01
      EL(7) = 4.1430362654321E-02
      EL(8) = 6.2111441798942E-03
      EL(9) = 6.2520667989418E-04
      EL(10) = 4.0417401528513E-05
      EL(11) = 1.5156525573192E-06
      EL(12) = 2.5052109385442E-08
      GO TO 900
 112  EL(1) = 2.7426554003160E-01
      EL(3) = 1.5099386724387
      EL(4) = 1.2602711640212
      EL(5) = 6.5923418209877E-01
      EL(6) = 2.3045800264550E-01
      EL(7) = 5.5697246105232E-02
      EL(8) = 9.4394841269841E-03
      EL(9) = 1.1192749669312E-03
      EL(10) = 9.0939153439153E-05
      EL(11) = 4.8225308641975E-06
      EL(12) = 1.5031265031265E-07
      EL(13) = 2.0876756987868E-09
      GO TO 900
 201  EL(1) = 1.00
      GO TO 900
 202  EL(1) = 6.6666666666667E-01
      EL(3) = 3.3333333333333E-01
      GO TO 900
 203  EL(1) = 5.4545454545455E-01
      EL(3) = EL(1)
      EL(4) = 9.0909090909091E-02
      GO TO 900
 204  EL(1) = 0.48
      EL(3) = 0.7
      EL(4) = 0.2
      EL(5) = 0.02
      GO TO 900
 205  EL(1) = 4.3795620437956E-01
      EL(3) = 8.2116788321168E-01
      EL(4) = 3.10218977810219E-01
      EL(5) = 5.4744525547445E-02
      EL(6) = 3.6496350364964E-03
C*
 900  DO 910 K=1,3
 910  TQ(K) = PERTST(NQ,METH,K)
      TQ(4)=.5*TQ(2)/FLOAT(NQ+2)
      RETURN
C*******************************END OF COSET****************************
      END
      SUBROUTINE DECOMP(NDIM,N,LU,IPS,SCALES,IER)
C***********************************************************************
C*
C*       LINEAR SYSTEMS SUBROUTINE
C*
C*       DECOMPOSE THE N X N MATRIX INTO TRIANGULAR L AND U SO THAT
C*       L * U = P * A FOR SOME PERMUTATION MATRIX P .
C*
C*       NDIM    THE MATRIX OF ROWS IN THE DIMENSION STATEMENT FOR
C*               THE MATRIX A IN THE CALLING PROGRAM .
C*       N       THE NUMBER OF ROWS OR COLUMNS IN THE MATRIX .
C*       LU      ON INPUT , THE MATRIX A TO BE DECOMPOSED .
C*               ON OUTPUT , THE ARRAY WHERE L AND U ARE STORED .
C*       IPS     THE ROW PIVOT VECTOR , GIVING THE PERMUTATION MATRIX P
C*       IER     THE ERROR RETURN FLAG . IT IS 1 FOR ALL ZERO ELEMENTS
C*               IN A ROW , 2 FOR A ZERO PIVOT , AND 0 FOR NO ERROR .
C*
C***********************************************************************
      REAL LU(NDIM,N)
      DIMENSION IPS(N) , SCALES(N)
      INTEGER PIVROW
C***********************************************************************
C*           INITIALIZE IER , IPS , LU , SCALES
C***********************************************************************
      IER = 0
      DO 5 I=1,N
      IPS(I) = I
      ROWNRM = 0.0
      DO 2 J=1,N
      ROWNRM = AMAX1(ROWNRM , ABS(LU(I,J)))
   2  CONTINUE
C***********************************************************************
C*           TEST FOR MATRIX WITH ZERO ROW .
C***********************************************************************
      IF (ROWNRM.EQ.0.0) GO TO 95
      SCALES(I) = 1.00/ROWNRM
   5  CONTINUE
C***********************************************************************
C*           GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING .
C***********************************************************************
      NM1 = N-1
      DO 17 K=1,NM1
      BIG = 0.00
      DO 11 I=K,N
      IP = IPS(I)
      SIZE = ABS(LU(IP,K))*SCALES(IP)
      IF (SIZE.LE.BIG) GO TO 11
  10  BIG = SIZE
      PIVROW = I
  11  CONTINUE
C***********************************************************************
C*           TEST FOR ZERO PIVOT
C***********************************************************************
      IF (BIG.EQ.0.0)GO TO 96
C***********************************************************************
C*           INTERCHANGE ROW IF NECESSARY .
C***********************************************************************
      IF (PIVROW.EQ.K) GO TO 15
      J = IPS(K)
      IPS(K) = IPS(PIVROW)
      IPS(PIVROW) = J
  15  KP = IPS(K)
      PIVOT = LU(KP,K)
      PIVOT = 1.00/PIVOT
      LU(KP,K) = PIVOT
      KP1 = K+1
      DO 16 I=KP1,N
      IP = IPS(I)
      EM = LU(IP,K)*PIVOT
      LU(IP,K) =EM
      DO 16 J=KP1,N
      LU(IP,J) = LU(IP,J)-EM*LU(KP,J)
  16  CONTINUE
  17  CONTINUE
C***********************************************************************
C*           TEST FOR LAST PIVOT .
C***********************************************************************
      IP = IPS(N)
      PIVOT = LU(IP,N)
      IF (PIVOT.EQ.0.0) GO TO 96
      LU(IP,N) = 1.00/PIVOT
      RETURN
C***********************************************************************
C*           ALL ZERO ELEMENTS IN A ROW .
C***********************************************************************
  95  IER = 1
      RETURN
C***********************************************************************
C*           ZERO PIVOT
C***********************************************************************
  96  IER = 2
      RETURN
C*****************************END OF DECOMP*****************************
      END
      SUBROUTINE SOLVE(NDIM,N,LU,B,X,IPS)
C***********************************************************************
C*
C*       LINEAR SYSTEM PACKAGE .
C*       SOLVE A X = B USING LU FROM SUBROUTINE DECOMP .
C*
C*       SEE DECOMP FOR DESCRIPTION OF PARAMETERS .
C*
C***********************************************************************
      DIMENSION B(N) , X(N) , IPS(N)
      REAL LU(NDIM,N)
      NP1 = N+1
C***********************************************************************
C*           FORWARD SUBSTITUTION - SOLVE L*Z = B
C***********************************************************************
      IP = IPS(1)
      X(1) = B(IP)
      DO 2 I=2,N
      IP = IPS(I)
      IM1 = I-1
      SUM = 0.00
      DO 1 J = 1,IM1
   1  SUM = SUM +LU(IP,J)*X(J)
   2  X(I) = B(IP)-SUM
      IP = IPS(N)
      X(N) = X(N)*LU(IP,N)
C***********************************************************************
C*           BACKWARD SUBSTITUTION - SOLVE U*X = Z , WHERE L*Z = B .
C***********************************************************************
      DO 4 IBACK=2,N
      I = NP1-IBACK
      IP = IPS(I)
      IP1 = I+1
      SUM = 0.00
      DO 3 J=IP1,N
   3  SUM = SUM+LU(IP,J)*X(J)
   4  X(I) = (X(I)-SUM)*LU(IP,I)
      RETURN
C******************************END OF SOLVE*****************************
      END
      SUBROUTINE DECB(NDIM,N,ML,MU,B,IP,IER)
      DIMENSION B(NDIM,1),IP(N)
C
C     LU DECOMPOSITION OF BAND MATRIX A.. L*U = P*A , WHERE IS A
C        PERMUTATION MATRIX, L IS A UNIT LOWER TRIANGULAR MATRIX,
C        AND U IS AN UPPER TRIANGULAR MATRIX.
C     N      =  ORDER OF MATRIX.
C     B      =  N BY (2*ML+MU+1) ARRAY CONTAINING THE MATRIX A ON INPUT
C               AND ITS FACTORED FORM ON OUTPUT.
C               ON INPUT, B(I,K) (1.LE.I.LE.N) CONTAINS THE K-TH
C               DIAGONAL OF A, OR A(I,J) IS STORED IN B(I,J-I+ML+1).
C               ON OUTPUT, B CONTAINS THE L AND U FACTORS, WITH
C               U IN COLUMNS 1 TO ML+MU+1, AND L IN COLUMNS
C               ML+MU+2 TO 2*ML+MU+1.
C     ML,MU  =  WIDTHS OF THE LOWER AND UPPER PARTS OF THE BAND, NOT
C               COUNTING THE MAIN DIAGONAL. TOTAL BANDWIDTH IS ML+MU+1.
C     NDIM   =  THE FIRST DIMENSION (COLUMN LENGTH) OF THE ARRAY B.
C               NDIM MUST BE .GE. N.
C     IP     =  ARRAY OF LENGTH N CONTAINING PIVOT INFORMATION.
C     IER    =  ERROR INDICATOR..
C            =  0  IF NO ERROR,
C            =  K  IF THE K-TH PIVOT CHOSEN WAS ZERO (A IS SINGULAR).
C     CAUTION.. IF ML = 0, THIS ROUTINE CONTAINS EMPTH DO-LOOPS
C        WHICH MUST BE COMPILED CORRECTLY (I.E. NO ACTION TAKEN).
C     THE INPUT ARGUMENTS ARE  NDIM,N,ML,MU,B.
C     THE OUTPUT ARGUMENTS ARE  B,IP,IER.
C
      IER=0
      LL=ML+MU+1
      N1=N-1
      DO 3 I=1,ML
        II=MU+I
        K=ML+1-I
      DO 1 J=1,II
1         B(I,J)=B(I,J+K)
        K=II+1
        DO 2 J=K,LL
2         B(I,J)=0.
3       CONTINUE
      LR=ML
      DO 9 NR=1,N1
        NP=NR+1
        IF (LR.NE.N) LR=LR+1
        MX=NR
*IF -DEF,AELIB5
        XM=ABS(B(NR))
*ENDIF
*IF DEF,AELIB5
        XM=ABS(B(NR,1))
*ENDIF
        DO 4 I=NP,LR
*IF -DEF,AELIB5
          IF (ABS(B(I)).LE.XM) GO TO 4
*ENDIF
*IF DEF,AELIB5
          IF (ABS(B(I,1)).LE.XM) GO TO 4
*ENDIF
          MX=I
*IF -DEF,AELIB5
          XM=ABS(B(I))
*ENDIF
*IF DEF,AELIB5
          XM=ABS(B(I,1))
*ENDIF
4         CONTINUE
        IP(NR)=MX
        IF (MX.EQ.NR) GO TO 6
        DO 5 I=1,LL
          XX=B(NR,I)
          B(NR,I)=B(MX,I)
5         B(MX,I)=XX
*IF -DEF,AELIB5
6       XM=B(NR)
*ENDIF
*IF DEF,AELIB5
6       XM=B(NR,1)
*ENDIF
        IF (XM.EQ.0.) GO TO 10
*IF -DEF,AELIB5
        B(NR)=1./XM
        XM=-B(NR)
*ENDIF
*IF DEF,AELIB5
        B(NR,1)=1./XM
        XM=-B(NR,1)
*ENDIF
        KK=MIN0(N-NR,LL-1)
      DO 8 I=NP,LR
          J=LL+I-NR
*IF -DEF,AELIB5
          XX=B(I)*XM
*ENDIF
*IF DEF,AELIB5
          XX=B(I,1)*XM
*ENDIF
          B(NR,J)=XX
          DO 7 II=1,KK
7           B(I,II)=B(I,II+1)+XX*B(NR,II+1)
8         B(I,LL)=0.
9       CONTINUE
      NR=N
*IF -DEF,AELIB5
      IF (B(N).EQ.0.) GO TO 10
      B(N)=1./B(N)
*ENDIF
*IF DEF,AELIB5
      IF (B(N,1).EQ.0.) GO TO 10
      B(N,1)=1./B(N,1)
*ENDIF
      RETURN
10    IER=NR
      RETURN
      END
      SUBROUTINE SOLB(NDIM,N,ML,MU,B,Y,IP)
      DIMENSION B(NDIM,1),Y(N),IP(N)
C
C     SOLUTION CF  A*X = C  GIVEN LO DECOMPOSITION OF A FROM DECB.
C     Y  =  RIGHT-HAND VECTOR C, OF LENGTH N, CN INPUT,
C        = SOLUTION VECTOR X ON OUTPUT.
C     CAUTION.. IF ML = 0, THIS ROUTINE CONTAINS EMPTY OO-LOOPS
C        WHICH MUST BE COMPILED CORRECTLY (I.E. NO ACTION TAKEN).
C     ALL THE ARGUMENTS ARE INPUT ARGUMENTS.
C     THE OUTPUT ARGUMENT IS  Y.
C
      N1=N-1
      LL=ML+MU+1
      DO 3 NR=1,N1
        IF (IP(NR).EQ.NR) GO TO 1
        J=IP(NR)
        XX=Y(NR)
        Y(NR)=Y(J)
        Y(J)=XX
1       KK=MIN0(N-NR,ML)
        DO 2 I=1,KK
2        Y(NR+I)=Y(NR+I)+Y(NR)*B(NR,LL+I)
3       CONTINUE
      LL=LL-1
*IF -DEF,AELIB5
      Y(N)=Y(N)*B(N)
*ENDIF
*IF DEF,AELIB5
      Y(N)=Y(N)*B(N,1)
*ENDIF
      KK=0
      DO 5 NB=1,N1
        NR=N-NB
        IF (KK.NE.LL) KK=KK+1
        DP=0.
        DO 4 I=1,KK
4        DP=DP+B(NR,I+1)*Y(NR+I)
*IF -DEF,AELIB5
5     Y(NR)=(Y(NR)-DP)*B(NR)
*ENDIF
*IF DEF,AELIB5
5     Y(NR)=(Y(NR)-DP)*B(NR,1)
*ENDIF
      RETURN
      END
