*DECK TDISTN
      FUNCTION TDISTN ( ESTIM, DEGFRM, TAIL )
C***********************************************************************
C***
C***  THIS FUNCTION RETURNS A 1 OR 2 - SIDED CUMULATIVE PROBABILITY FOR
C***  STUDENT'S T DISTRIBUTION.
C***
C***
C***     ESTIM     T STATISTIC TO BE TESTED.
C***
C***     DEGFRM    NUMBER OF DEGREES OF FREEDOM ASSOCIATED WITH ESTIM.
C***
C***
C*** 1 OF 3 SERIES EXPANSIONS IS USED FOR THE INTEGRAL, CORRESPONDING TO
C***
C***     DEGFRM = 1
C***     DEGFRM EVEN
C***     DEGFRM ODD AND GREATER THAN 1
C***
C***
C*** SERIES EXPANSIONS ARE TAKEN FROM HARWELL LIBRARY ROUTINE SA02A
C***
C***
C***     SEE ALSO  STEGUN AND ABRAMOWITZ
C***               HANDBOOK OF MATHEMATICAL FUNCTIONS
C***               FORMULAE 26.7.3 AND 26.7.4
C***
C***
C***
C***********************************************************************
      INTEGER DEGFRM, TAIL
      LOGICAL ODD
      DATA PI / 3.14159265358979 /
*CALL  STATF
*IF -DEF,AELIB5
      TDISTN = 60000000000000000000B
*ENDIF
*IF DEF,AELIB5
      TDISTN = O"60000000000000000000"
*ENDIF
C*** IF 0 OR -VE NUMBER OF DEGREES OF FREEDOM, RESULTING PROBABILITY
C*** IS SET TO -VE INDEFINITE
C***
      IF ( DEGFRM .GE. 1 ) GO TO 10
      PRINT 7
*IF -DEF,AELIB5
   7  FORMAT ( * ERROR IN TDISTN - DEGREES OF FREEDOM LESS THAN 1 * )
*ENDIF
*IF DEF,AELIB5
   7  FORMAT ( ' ERROR IN TDISTN - DEGREES OF FREEDOM LESS THAN 1 ' )
*ENDIF
      GO TO 501
   10 ODD = .FALSE.
      ALPHA = ATAN2 ( ESTIM, SQRT( FLOAT ( DEGFRM ) ) )
      IF ( DEGFRM .NE. 1 ) GO TO 15
      TDISTN = ( 1.0 / PI ) * ( ALPHA + PI / 2.0 )
      GO TO 500
   15 IF ( MOD ( DEGFRM, 2 )  .EQ. 1 )  ODD = .TRUE.
C*** SERIES IS CUMULATIVE SUM OF THE SERIES EXPANSION
C*** TERM IS NEWEST TERM TO BE ADDED ON - IT IS A MULTIPLE OF PREVIOUS
C*** TERM
C*** IEND IS NO. OF TERMS REQUIRED IN THE SERIES
      IEND = DEGFRM / 2
      COSINE = ( COS ( ALPHA ) ) ** 2
      IF ( ODD ) GO TO 300
      SERIES = TERM = 1.0
      IF ( DEGFRM .EQ. 2 ) GO TO 350
C***NO. OF DEGREES OF FREEDOM IS EVEN
      IFUDGE = -1
   25 DO 50 N = 2, IEND
      KONST = N + IFUDGE
      TERM = TERM * COSINE * KONST / ( KONST + 1 )
      SERIES = SERIES + TERM
   50 IFUDGE = IFUDGE + 1
      GO TO 350
C*** INITIALIZATIONS TO CALCULATE SERIES FOR DEGFRM ODD AND GREATER
C*** THAN 1
  300 SERIES = TERM = COS( ALPHA )
      IF ( DEGFRM .EQ. 3 ) GO TO 400
      IFUDGE = 0
      GO TO 25
  350 IF ( ODD ) GO TO 400
      TDISTN = 0.5 * ( 1.0 + ( SIN ( ALPHA ) ) * SERIES )
      GO TO 500
  400 TDISTN = 0.5 + ALPHA / PI + ( SIN ( ALPHA ) ) / PI * SERIES
C*** IS 2 - TAILED PROBABILITY REQUIRED
  500 IF ( TAIL .EQ. 2 ) TDISTN = ABS ( 2.0 * TDISTN  - 1.0 )
  501 RETURN
      END
