*DECK VARSIZM
      SUBROUTINE VARSIZM(MODE,ISTRING,ASIZE,ADIR,XLOC,YLOC,ICODE,LENGTH)
C
C THIS SUBROUTINE PLOTS A STRING OF TEXT AND ALLOWS TWO CHARACTER SIZES
C TO BE USED, PARALLELING THE USE OF UPPER AND LOWER CASE LETTERS.
C A PLOT OR PLOTM MODE 1 CALL MUST BE MADE BEFORE CALLING VARSIZ OR
C VARSIZM RESPECTIVELY.
C
C ARGUMENT LIST:
C  MODE......EITHER INTEGER OR REAL; SPECIFIES WHETHER XLOC, YLOC
C            EXPRESSED IN USER"S UNITS (MODE=3), OR INCHES OR
C            MILLIMETRES (MODE=13).
C  STRING....AN INTEGER VARIABLE OR ARRAY CONTAINING A HOLLERITH STRING.
C            THIS HOLLERITH STRING CONTAINS THE STRING OF TEXT TO BE
C            PLOTTED.
C  SIZE......EITHER INTEGER OR REAL; SPECIFIES THE LARGER OF THE TWO
C            SIZES OF TEXT TO BE PLOTTED.  THE SMALLER SIZE IS EQUAL TO
C            SIZE/2, ROUNDED UP IF SIZE IS ODD.  1@SIZE@10.  VALUES
C            OUTSIDE THIS RANGE WILL BE REPLACED BY SIZE=3.  THE SIZE OF
C            THE CHARACTERS TO BE PLOTTED IS SIZE*2 MILLIMETRES (.08
C            INCHES) IF DIR IS EVEN, AND SIZE*SQRT(2)*2 MILLIMETRES
C            (.08 INCHES) IF DIR IS ODD.
C  DIR.......EITHER INTEGER OR REAL; SPECIFIES THE DIRECTION OF THE
C            CHARACTER LINE, AS WITH A PLOT MODE 3 OR 13 CALL.  0@DIR@7.
C            VALUES OUTSIDE THIS RANGE WILL BE REPLACED BY DIR=0.
C            THE ANGLE AT WHICH THE CHARACTER LINE IS PLOTTED IS
C            (360-(DIR*45)) DEGREES.
C  XLOC,YLOC.REAL VARIABLES SPECIFYING THE POSITION WHERE THE LINE OF
C            CHARACTERS IS TO BEGIN.  FOR MODE=3, XLOC AND YLOC MUST BE
C            EXPRESSED IN USER"S UNITS, AND FOR MODE=13 XLOC AND YLOC
C            MUST BE EXPRESSED IN INCHES (VARSIZ, MODE=13), OR
C            MILLIMETRES (VARSIZM, MODE=13).
C  CODE......AN INTEGER VARIABLE OR ARRAY CONTAINING A HOLLERITH STRING
C            SPECIFYING THE SIZE OF TEXT TO BE PLOTTED FROM STRING.
C            CODE IS PROCESSED CHARACTER BY CHARACTER IN PARALLEL WITH
C            STRING, AND FOR EACH CHARACTER IN STRING THERE MUST BE A
C            CORRESPONDING LETTER IN CODE TO SPECIFY THE SIZE OF TEXT
C            TO BE PLOTTED.  TO PLOT THE LARGER SIZE OF TEXT AN L IS
C            PLACED IN CODE, AND FOR THE SMALLER SIZE AN S IS USED.
C  LENGTH....INTEGER VARIABLE SPECIFYING THE NUMBER OF CHARACTERS TO
C            BE PLOTTED.  L\1.
C
C ERROR CONDITIONS:
C
C  1) ***VALUE OUT-OF-RANGE FOR SIZE - DEFAULT IS 3
C  2) ***ILLEGAL VALUE FOR LENGTH - MUST BE \1
C  3) ***VALUE OUT-OF-RANGE FOR DIRECTION - DEFAULT IS 0
C  4) ***ILLEGAL VALUE FOR MODE - MUST BE 3 OR 13
C  5) ***ILLEGAL CODE CHARACTER
C
C  REVISION HISTORY -
C
C        DATE    : 1982 FEB 23
C
C        AUTHOR  : D. CHEUNG
C
C        REMARKS : CORRECT AN INCREMENT AND SPACING PROBLEM (WHEN
C                  THE VALUE OF ASIZE IS BETWEEN TWO INTEGERS)
C                  WHICH LEADS TO EXTRA BLANKS INSERTED IN THE TEXT.
C
C
      REAL MTOE
      LOGICAL METRIC
      INTEGER CHARF,I2FORM(2),I1FORM(2),SUBTOT,TOTAL,SUBT,CODE,STRING
      INTEGER ISTRING(1),ICODE(1)
      INTEGER MESNO(5)
      DATA MESNO/3535,3536,3537,3538,3539/
*IF -DEF,AELIB5
      DATA I1FORM/14H("(R",I1," )")/
*ENDIF
*IF DEF,AELIB5
      DATA I1FORM/10H("(R",I1,",4H )")/
*ENDIF
*IF -DEF,AELIB5
      DATA I2FORM/13H("(R",I2,")")/
*ENDIF
*IF DEF,AELIB5
      DATA I2FORM/10H("(R",I2,",3H)")/
*ENDIF
C
C     ENTRY VARSIZM: XLOC AND YLOC ARE EXPRESSED IN MILLIMETRES
C     (MODE=13), OR IN USER"S UNITS (MODE=3) AND THE SCALING FACTORS
C     USED TO CONVERT USER"S UNITS HAVE BEEN CALCULATED FROM A PLOTM
C     MODE 1 CALL TO PLOT THE FRAME USING MILLIMETRES TO SPECIFY AXES
C     LENGTHS
C
*CALL STATF
      METRIC=.TRUE.
      GO TO 10
C
C     ENTRY VARSIZ: XLOC AND YLOC ARE EXPRESSED IN INCHES (MODE=13), OR
C     IN USER"S UNITS (MODE=3) AND THE SCALING FACTORS USED TO CONVERT
C     USER"S UNITS HAVE BEEN CALCULATED FROM A PLOT MODE 1 CALL TO PLOT
C     THE FRAME USING INCHES TO SPECIFY AXES LENGTHS.
C
*IF -DEF,AELIB5
      ENTRY VARSIZ
*ENDIF
*IF DEF,AELIB5
      ENTRY VARSIZ(MODE,ISTRING,ASIZE,ADIR,XLOC,YLOC,ICODE,LENGTH)
*ENDIF
C
*CALL STATF
      METRIC=.FALSE.
      ETOM=25.4
      MTOE=1./ETOM
C
C     CHECK FOR INVALID DIRECTION.  0@DIR@7.  VALUES OUTSIDE THIS RANGE
C     WILL BE REPLACED BY DIR=0.
C
   10 IF (ADIR.GT.7.) GO TO 20
      IF (ADIR.LT.0.) GO TO 20
      DIR=ADIR
      GO TO 30
*IF -DEF,AELIB5
  20  CALL AELERR(MESNO(3),2HNF)
*ENDIF
*IF DEF,AELIB5
  20  CALL AELERR(MESNO(3),'NF')
*ENDIF
      DIR=0.
C
C     CHECK FOR INVALID SIZE.  1@SIZE@10.  VALUES OUTSIDE THIS RANGE
C     WILL BE REPLACED BY SIZE=3.
C
   30 IF (ASIZE.LT.1.) GO TO 40
      IF (ASIZE.GT.10.) GO TO 40
      SIZE=ASIZE
      GO TO 50
*IF -DEF,AELIB5
  40  CALL AELERR(MESNO(1),2HNF)
*ENDIF
*IF DEF,AELIB5
  40  CALL AELERR(MESNO(1),'NF')
*ENDIF
      SIZE=3.
C
C     CHECK FOR INVALID LENGTH.  LENGTH\1.  IF INVALID, DIAGNOSTIC WILL
C     WILL BE PRINTED AND CONTROL WILL BE RETURNED TO CALLING PROGRAM.
C
   50 IF (LENGTH.GE.1) GO TO 60
*IF -DEF,AELIB5
      CALL AELERR(MESNO(2),1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MESNO(2),'F')
*ENDIF
      RETURN
C
C     CHECK FOR INVALID MODE.  IF INVALID, DIAGNOSTIC WILL BE PRINTED
C     AND JOB TERMINATED.
C
*IF -DEF,AELIB5
   60 IF ((MODE.EQ.3).O.(MODE.EQ.13)) GO TO 70
*ENDIF
*IF DEF,AELIB5
   60 IF ((MODE.EQ.3).OR.(MODE.EQ.13)) GO TO 70
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(MESNO(4),1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MESNO(4),'F')
*ENDIF
      RETURN
C
C     SET UP THE TWO CHARACTER SIZES TO PLOT THE STRING
C
   70 ISIZE=SIZE
      IF ((ISIZE/2*2).EQ.ISIZE) GO TO 100
      ISIZE=ISIZE+1
  100 SIZEL=SIZE
      SIZES=ISIZE/2
C
C     DETERMINE WHETHER XLOC, YLOC EXPRESSED IN USER"S UNITS (MODE=3),
C     OR INCHES OR MILLIMETRES (MODE=13)
C
      IF (MODE.EQ.3) GO TO 300
C
C     MODE=13.  XLOC, YLOC ARE EXPRESSED IN INCHES OR MILLIMETRES.
C     DETERMINE WHETHER XLOC, YLOC ARE IN MILLIMETRES (METRIC IS TRUE)
C     WITH NO CONVERSION OF XLOC OR YLOC REQUIRED BECAUSE THE ROUTINE
C     USES MILLIMETRES INTERNALLY, OR WHETHER THEY ARE IN INCHES (METRIC
C     IS FALSE).  IF IN INCHES, CONVERT XLOC, YLOC TO MILLIMETRES.
C
      IF (METRIC) GO TO 200
C
C     XLOC, YLOC EXPRESSED IN INCHES.  CONVERT TO MILLIMETRES.
C
      XLOCM=XLOC*ETOM
      YLOCM=YLOC*ETOM
      GO TO 400
C
C     XLOC, YLOC EXPRESSED IN MILLIMETRES.  NO CONVERSION REQUIRED.
C
  200 XLOCM=XLOC
      YLOCM=YLOC
      GO TO 400
C
C     XLOC, YLOC IN USER"S UNITS.  DETERMINE WHETHER INCHES OR
C     MILLIMETRES WERE USED WHEN CALCULATING THE SCALE FOR USER"S UNITS
C     AND CONVERT USER"S UNITS TO MILLIMETRES.
C
  300 IF (METRIC) GO TO 310
C
C     INCHES WERE USED WHEN CALCULATING SCALE FOR USER"S UNITS.
C     RETRIEVE PARAMETERS OF LAST PLOT MIDE 1 CALL TO CALCULATE SCALING
C     FACTORS TO CONVERT USER"S UNITS TO INCHES.
C
      CALL PLOT(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      SFX=XLR/(XMAXR-XMINR)
      SFY=YLR/(YMAXR-YMINR)
      XLOCE=(XLOC-XMINR)*SFX
      YLOCE=(YLOC-YMINR)*SFY
C
C     CONVERT XLOC, YLOC EXPRESSED IN INCHES TO MILLIMETRES
C
      XLOCM=XLOCE*ETOM
      YLOCM=YLOCE*ETOM
      GO TO 400
C
C     MILLIMETRES WERE USED WHEN CALCULATING SCALE FOR USER"S UNITS.
C     RETRIEVE PARAMETERS OF LAST MODE 1 CALL OF PLOTM TO CALCULATE
C     SCALING FACTORS TO CONVERT USER"S UNITS TO MILLIMETRES.
C
  310 CALL PLOTM(4,XLR,YLR,XMINR,XMAXR,YMINR,YMAXR,TICXR,TICYR)
      SFX=XLR/(XMAXR-XMINR)
      SFY=YLR/(YMAXR-YMINR)
      XLOCM=(XLOC-XMINR)*SFX
      YLOCM=(YLOC-YMINR)*SFY
C
C
C INITIALIZATION FOR STRING DECODING
C
C
  400 LASTSZ=0
      TOTAL=0
      SUBTOT=0
      NEWCODE=0
*IF -DEF,AELIB5
      IS=1RS
*ENDIF
*IF DEF,AELIB5
      IS=R"S"
*ENDIF
*IF -DEF,AELIB5
      IL=1RL
*ENDIF
*IF DEF,AELIB5
      IL=R"L"
*ENDIF
      ICOUNT=0
      J=0
C
C     REINTIALIZATION FOR DECODING OF NEW WORD IN STRING.
C
  410 SUBTOT=0
      J=J+1
      CODE=ICODE(J)
      STRING=ISTRING(J)
      ICOUNT=1
C
C     SHIFT AND MASK THE CODE TO DETERMINE THE SIZE OF THE FIRST
C     CHARACTER TO BE PLOTTED.  NEWCODE IS ASSIGNED THIS VALUE.
C
      CODE=SHIFT(CODE,6)
*IF -DEF,AELIB5
      NEWCODE=CODE.A.77B
*ENDIF
*IF DEF,AELIB5
      NEWCODE=CODE.AND.O"77"
*ENDIF
C
C     CHECK FOR ILLEGAL FIRST CHARACTER IN CODE.  IF ILLEGAL, PRINT
C     DIAGNOSTIC AND TERMINATE JOB.
C
*IF -DEF,AELIB5
      IF ((NEWCODE.EQ.IL).O.(NEWCODE.EQ.IS)) GO TO 500
*ENDIF
*IF DEF,AELIB5
      IF ((NEWCODE.EQ.IL).OR.(NEWCODE.EQ.IS)) GO TO 500
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(MESNO(5),1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MESNO(5),'F')
*ENDIF
      RETURN
C
C
C CHECK FOR CHANGE IN CHARACTER SIZE - MAIN LOOP IN STRING PROCESSING
C ALGORITHM
C
C
C
C     LASTSZ IS ASSIGNED THE VALUE OF NEWCODE.  AS CODE IS SHIFTED AND
C     MASKED, NEWCODE IS COMPARED WITH LASTSZ TO CHECK FOR A CHANGE IN
C     CHARACTER SIZE.
C
  500 LASTSZ=NEWCODE
C
C     SUBTOT COUNTS THE NUMBER OF CHARACTERS OF ONE SIZE THAT HAVE BEEN
C     READ IN FROM CODE
C
  510 SUBTOT=SUBTOT+1
C
C     IF THE NUMBER OF CHARACTERS IN CODE WHICH HAVE BEEN SHIFTED IS
C     GREATER THAN OR EQUAL TO THE NUMBER OF CHARACTERS TO BE PLOTTED,
C     SHIFT AND PLOT THE STRING, AND RETURN TO THE CALLING PROGRAM.
C
      IF ((SUBTOT+TOTAL).GE.LENGTH) GO TO 600
C
C     SHIFT AND MASK CODE TO DETERMINE SIZE OF NEXT CHARACTER TO BE
C     PLOTTED.  NEWCODE IS ASSIGNED THIS VALUE.
C
      CODE=SHIFT(CODE,6)
*IF -DEF,AELIB5
      NEWCODE=CODE.A.77B
*ENDIF
*IF DEF,AELIB5
      NEWCODE=CODE.AND.O"77"
*ENDIF
      ICOUNT=ICOUNT+1
C
C     CHECK FOR ILLEGAL CODE CHARACTER.  IF ILLEGAL, PRINT DIAGNOSTIC
C     AND TERMINATE JOB.
C
*IF -DEF,AELIB5
      IF ((NEWCODE.EQ.IL).O.(NEWCODE.EQ.IS)) GO TO 530
*ENDIF
*IF DEF,AELIB5
      IF ((NEWCODE.EQ.IL).OR.(NEWCODE.EQ.IS)) GO TO 530
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(MESNO(5),1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MESNO(5),'F')
*ENDIF
      RETURN
C
C     ICOUNT COUNTS THE SHIFTS OF CODE FOR ONE WORD.  WHEN THE CODE FOR
C     ONE WORD HAS BEEN TOTALLY SHIFTED, PLOT THE STRING CONTAINING THAT
C     WORD, AND GO ON TO THE NEXT WORD.
C
  530 IF (ICOUNT.GT.10) GO TO 600
C
C     NEWCODE IS COMPARED WITH LASTSZ TO DETERMINE IF THERE HAS BEEN A
C     CHANGE IN CHARACTER SIZE. IF THERE HAS NOT BEEN, SHIFT CODE AGAIN.
C     IF THERE HAS BEEN A CHANGE IN CHARACTER SIZE, SHIFT AND PLOT
C     STRING.
C
      IF (NEWCODE.EQ.LASTSZ) GO TO 510
C
C
C PLOT STRING SEGMENT
C
C
C
C     SHIFT STRING BY SUBTOT, THE NUMBER OF CHARACTERS DENOTED TO BE OF
C     ONE SIZE BY CODE
C
  600 SUBT=6*SUBTOT
      STRING=SHIFT(STRING,SUBT)
      IF (SUBTOT.LT.10) ENCODE(5,I1FORM,CHARF)SUBTOT
      IF (SUBTOT.GE.10) ENCODE(5,I2FORM,CHARF)SUBTOT
C
C     DETERMINE WHICH SIZE OF TEXT IS TO BE PLOTTED
C
      IF (LASTSZ.EQ.IS) GO TO 610
      SIZ=SIZEL
      GO TO 620
  610 SIZ=SIZES
  620 IF (METRIC) GO TO 630
C
C     CONVERT XLOC, YLOC TO INCHES BEFORE MAKING A PLOT CALL TO PLOT
C     THE TEXT
C
      XLOCE=XLOCM*MTOE
      YLOCE=YLOCM*MTOE
C
C     PLOT THE TEXT
C
      CALL PLOT(13,CHARF,SIZ,DIR,XLOCE,YLOCE,STRING,1)
      GO TO 640
C
C     PLOT THE TEXT
C
  630 CALL PLOTM(13,CHARF,SIZ,DIR,XLOCM,YLOCM,STRING,1)
C
C     CHECK FOR ILLEGAL CODE CHARACTER.  IF ILLEGAL, PRINT DIAGNOSTIC
C     AND TERMINATE JOB.
C
*IF -DEF,AELIB5
  640 IF ((NEWCODE.EQ.IL).O.(NEWCODE.EQ.IS)) GO TO 645
*ENDIF
*IF DEF,AELIB5
  640 IF ((NEWCODE.EQ.IL).OR.(NEWCODE.EQ.IS)) GO TO 645
*ENDIF
*IF -DEF,AELIB5
      CALL AELERR(MESNO(5),1HF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MESNO(5),'F')
*ENDIF
      RETURN
  645 TOTAL=TOTAL+SUBTOT
C
C     IF NUMBER OF CHARACTERS IN STRING WHICH HAVE BEEN PLOTTED IS
C     GREATER THAN OR EQUAL TO LENGTH, RETURN CONTROL TO  THE CALLING
C     PROGRAM.
C
      IF (TOTAL.GE.LENGTH) RETURN
      JSIZE = SIZ
      RSIZE = JSIZE
      IDIR=DIR
      IF ((IDIR/2*2).EQ.IDIR) GO TO 650
      RSIZE=RSIZE*1.4
  650 A=(8-DIR)*XPI(0.25)
C
C     INCREMENT XLOC, YLOC
C
      XLOCINC=COS(A)*RSIZE*2*SUBTOT
      YLOCINC=SIN(A)*RSIZE*2*SUBTOT
      XLOCM=XLOCM+XLOCINC
      YLOCM=YLOCM+YLOCINC
      SUBTOT=0
C
C     IF ONE WORD OF STRING HAS BEEN COMPLETELY PLOTTED, REINITALIZE FOR
C     DECODING OF NEW WORD IN STRING; ELSE GO BACK TO MAIN LOOP OF
C     STRING PROCESSING ALGORITHM.
C
      IF (ICOUNT.GT.10) GO TO 410
C
C     INCREASE SPACING BY HALF CHARACTER IF SMALL LETTER BEFORE LARGE ON
C
      IF(LASTSZ.NE.IS) GOTO 500
      XLOCM=XLOCM+COS(A)*RSIZE
      YLOCM=YLOCM+SIN(A)*RSIZE
      GO TO 500
      END
