*DECK WCOEF
      FUNCTION WCOEF(A,B,C,D,E,F)
*  THIS PROGRAM, WRITTEN BY R.Y.CUSSON AND J.H.SCHMIDT IN SEPT. 1971,
*  USES REGGES FORMULA (NUOVO CIMENTO,VOL11(1959),P116) TO EVALUATE THE
*  RACAH COEFFICIENT W(A,B,C,D,E,F)=(-1)**(A+B+C+D)*SIXJ(A,B,E/D,C,F) .
*
*  SINCE MANY PROGRAMS WHICH USE RACAH COEFFICENTS ALSO USE CLEBSCH -
*  GORDANS, COMMON BLOCK CGRACAH HAS BEEN SET UP SO TABLES OF LOGS AND
*  SQUARE ROOTS DO NOT HAVE TO BE CALCULATED EACH TIME THE FUNCTIONS
*  ARE CALLED. NOTE THAT ALOGF(0) IS SOMETIMES REQUIRED IN THE
*  CALCULATIONS AND SO ALOGF0=ALOGF(0)=0.
*
*  COMMON BLOCK RACASYS IS INCLUDED TO ENSURE THAT ALL ITS ELEMENTS ARE
*  STORED SEQUENTIALLY IN CORE. THIS IS IMPORTANT FOR EQUIVALENCING.
*
*  COMMON BLOCK XRACAH IS USED IN CALCULATING X-COEFFICIENTS.
      COMMON/CGRACAH/INITIAL,ALOGF0,ALOGF(80),SQRF(80)
      COMMON/RACASYS/X1,X2,X3,Y1,Y2,Y3,Y4,IX1,IX2,IX3,IY1,IY2,IY3,IY4
      COMMON/XRACAH/JXSWTCH
      COMMON/TRIANG/IPRINT
      DIMENSION RV(7),IV(7)
*IF -DEF,AELIB5
      EQUIVALENCE (RV,X1),(IV,IX1,VA),(IX2,VB),(IY1,VC),(IY2,VD),
     1 (IY3,VE),(IY4,VF)
*ENDIF
*IF DEF,AELIB5
      EQUIVALENCE (RV(1),X1),(IV(1),IX1,VA),(IX2,VB),(IY1,VC),(IY2,VD),
     1       (IY3,VE),(IY4,VF)
*ENDIF
      DATA INITIAL/0/,ALOGF0/0./,JXSWTCH/0/
      DATA IPRINT/0/
*IF -DEF,AELIB5
 400  FORMAT(*  ILLEGAL PARAMETERS IN WCOEF(*,6F5.1,*)*)
*ENDIF
*IF DEF,AELIB5
 400  FORMAT('  ILLEGAL PARAMETERS IN WCOEF(',6F5.1,')')
*ENDIF
*  ENSURES THAT ALOGF AND SQRF ARE SET UP ONLY ONCE IN EACH PROGRAM
*CALL  STATF
      IF (INITIAL .NE. 0) GO TO 20
      ALOGF(1)=0.
*IF -DEF,AELIB5
      SQRF(1)=1.   $   SM=1.
*ENDIF
*IF DEF,AELIB5
      SQRF(1)=1.
      SM=1.
*ENDIF
      DO 25 J=2,80
      SM=SM+1.
      SQRF(J)=SQRT(SM)
 25   ALOGF(J)=ALOGF(J-1)+ALOG(SM)
      INITIAL = 1
*IF -DEF,AELIB5
 20   SIXJ=1.    $    IFAS=0    $    WCOEF=0.
*ENDIF
*IF DEF,AELIB5
 20   SIXJ=1.
      IFAS=0
      WCOEF=0.
*ENDIF
*  SET UP THE REGGE SYMBOL
*IF -DEF,AELIB5
      Y4= A+B    $    Y3=C+D      $    Y2=E+F
      X1=Y4+Y3   $    X2=A+D+Y2   $    X3=B+C+Y2
      Y1=Y4+E    $    Y2=Y3+E     $    Y3=A+C+F    $    Y4=B+D+F
*ENDIF
*IF DEF,AELIB5
      Y4= A+B
      Y3=C+D
      Y2=E+F
      X1=Y4+Y3
      X2=A+D+Y2
      X3=B+C+Y2
      Y1=Y4+E
      Y2=Y3+E
      Y3=A+C+F
      Y4=B+D+F
*ENDIF
*   CHECK THAT THE REGGE SYMBOL CONTAINS ONLY NON-NEGATIVE INTEGERS
      DO 1 K=1,7
      J=ABS(RV(K))
      IF (FLOAT(J) .NE. RV(K)) GO TO 998
 1    IV(K)= J
*  CHECK FOR TRIANGLE AND IN RANGE CONDITION .
      GO TO 201
*IF -DEF,AELIB5
      ENTRY FWCOEF
*ENDIF
*IF DEF,AELIB5
      ENTRY FWCOEF(A,B,C,D,E,F)
*ENDIF
      WCOEF=0.
*  CALLING SEQUENCE IS F=FWCOEF(IX1,IX2,IY1,IY2,IY3,IY4)  WHERE
*          IX1 = A+B+C+D               IY2=C+D+E
*          IX2 = A+D+E+F               IY3=A+C+F
*          IY1 = A+B+E                 IY4=B+D+F
*  IF PARAMETERS ARE LEGAL THE VALUE OF WCOEF(A,B,C,D,E,F) IS RETURNED
*  WCOEF MUST BE CALLED FIRST TO INITIALISE /CGRACAH/
*  NOTE - IF FWCOEF IS CALLED, NO ARGUMENT CHECKING IS DONE
*IF -DEF,AELIB5
      VA=A$ VB=B$ VC=C$ VD=D$ VE=E$ VF=F
*ENDIF
*IF DEF,AELIB5
      VA=A
      VB=B
      VC=C
      VD=D
      VE=E
      VF=F
*ENDIF
      IX3= IY1+IY2+IY3+IY4-IX1-IX2
 201  IY0= MAX0(IY1,IY2,IY3,IY4)
      IX0= MIN0(IX1,IX2,IX3)
      IF (IY0 .GT. 78) GO TO 998
      X=IX0-IY0+1
      IF(X.LT.1.) GOTO 990
      IFAS=IX1
      IF (IX0 .LT. 0) GO TO 999
*   THE EXPRESSION (IY0 .AND. 1) SIMPLY CHECKS IF IY0 IS ODD OR EVEN
      K=IY0 .AND. 1
*  NOTE   DELC IS EVALUATED AS FOLLOWS
*          DELC=(1)-(2)+((3)-(4))
*  WHERE      (1)=SUM OF SMALLER LOGS
*             (2)=SUM OF SMALLER LOGS
*             (3)=MOST LIKELY TO BE THE LARGEST LOG
*             (4)=SUM OF LARGE LOGS
      DELC=((ALOGF(IX1-IY1)+ALOGF(IX1-IY2)+ALOGF(IX1-IY3)+ALOGF(IX1-IY4)
     1      +ALOGF(IX2-IY1)+ALOGF(IX2-IY2)+ALOGF(IX2-IY3)+ALOGF(IX2-IY4)
     2 +ALOGF(IX3-IY1)+ALOGF(IX3-IY2)+ALOGF(IX3-IY3)+ALOGF(IX3-IY4))*.5)
     3    - (ALOGF(IY0-IY1)+ALOGF(IY0-IY2)+ALOGF(IY0-IY3)+ALOGF(IY0-IY4)
     4      +ALOGF(IX1-IY0)+ALOGF(IX2-IY0)+ALOGF(IX3-IY0))+(ALOGF(IY0+1)
     5      -.5*(ALOGF(IY1+1)+ALOGF(IY2+1)+ALOGF(IY3+1)+ALOGF(IY4+1)))
      DELC=(1-K-K)*EXP(DELC)
*   INITIALISE THE SUM
      SM=1.
      IF (X .LE. 1.) GO TO 13
      Z0=IX0+3
*IF -DEF,AELIB5
      K=IX0+1  $  X1=IX1-K  $  X2=IX2-K  $  X3=IX3-K
      K=K+1    $  Y1=K-IY1  $  Y2=K-IY2  $  Y3=K-IY3  $  Y4=K-IY4
*ENDIF
*IF DEF,AELIB5
      K=IX0+1
      X1=IX1-K
      X2=IX2-K
      X3=IX3-K
      K=K+1
      Y1=K-IY1
      Y2=K-IY2
      Y3=K-IY3
      Y4=K-IY4
*ENDIF
*   THE SUM IS DONE RECURSIVELY WITHOUT ALOGF
*  NOTE BECAUSE OF THE DEFINITION OF X, THE SUM FROM 2 TO X IS THE
*  SAME AS THE SUM FROM IY0 TO IX0
      XK=2.
 15   SM= 1.-SM*(Z0-XK)*(X1+XK)*(X2+XK)*(X3+XK)/
     1 ((Y1-XK)*(Y2-XK)*(Y3-XK)*(Y4-XK))
      XK=XK+1.
      IF (XK .LE. X) GO TO 15
*   SIXJ IS THE USUAL 6-J SYMBOL OF WIGNER .
 13   SIXJ= DELC*SM
 999  IFAS=IFAS .AND. 1
      WCOEF=SIXJ*(1-IFAS-IFAS)
 997  RETURN
*  JXSWTCH DETERMINES WHETHER OR NOT WCOEF WAS CALLED BY XCOEF. IF IT
*  WAS, NO ERROR MESSAGE IS PRINTED BY WCOEF
  990 IF(IPRINT.EQ.1) GOTO 998
      IF(JXSWTCH.EQ.1) JXSWTCH=2
      RETURN
 998  IF (JXSWTCH .EQ. 0) GO TO 996
      JXSWTCH=3
      GO TO 997
 996  PRINT 400,A,B,C,D,E,F
      GO TO 997
      END
