*DECK YFST
      SUBROUTINE YFST(IOPT,X,NRX,Y,SDY,P,NC,N,M,AUX,E,NE,WS,IER)
C
C     THIS ROUTINE CALCULATES AN ESTIMATE OF THE STANDARD DEVIATION FOR
C     THE FITTED FUNCTION VALUES DETERMINED BY MLSQQ.
C
      INTEGER IOPT
C                 ....SPECIFIES WHETHER OR NOT DERIVATIVES ARE SUPPLIED
C                 ....BY THE USER VIA AUX.
C                 ....IABS(IOPT) = 1, IMPLIES DERIVATIVES ARE SUPPLIED
      REAL    X(NRX,1)
C                 ....ARRAY OF VALUES OF INDEPENDENT VARIABLE
      INTEGER NRX
C                 ....NO. OF ROWS IN X AS DIMENSIONED IN CALLING PROG.
      REAL    Y(1)
C                 ....ARRAY OF VALUES OF DEPENDENT VARIABLE
      REAL    SDY(1)
C                 ....ARRAY OF STANDARD DEVIATIONS OF FUNCTION VALUES
C                 ....CALCULATED BY THIS ROUTINE
      REAL    P(1)
C                 ....ARRAY OF FITTED PARAMETERS FROM MLSQQ
      INTEGER NC(1)
C                 ....ARRAY SPECIFYING WHETHER A PARAMETER HAS BEEN
C                 ....FIXED OR VARIED (0 MEANS FIXED, 1 MEANS VARIED)
      INTEGER N
C                 ....NO. OF DATA POINTS
      INTEGER M
C                 ....NO. OF PARAMETERS IN USER"S FUNCTION
      REAL    E(NE,1)
C                 ....ERROR MATRIX (M*M) PROVIDED BY MLSQQ
      INTEGER NE
C                 ....NO. OF ROWS OF E AS DIMENSIONED IN CALLING PROGRAM
      REAL    WS(1)
C                 ....WORKING STORAGE ARRAY. MUST BE DIMENSIONED AT
C                 ....LEAST (M*M)+(2*N)
C
C     WORKING STORAGE ALLOCATION
C      1. WS(1) - WS(M*M) -TEMPORARY STORAGE OF ERROR MATRIX FROM MLSQQ
C      2. LET I2 = M*M
C             I3 = I2 + N
C         WS(I2+1) - WS(I2+N)
C         WS(I3+1) - WS(I3+N)
C                         -USED IN CALCULATING THE DERIVATIVE APPROX.
C
      INTEGER IER
C                 ....ERROR FLAG
C                     =1, ONE OR MORE VARIANCE ESTIMATES ARE NEGATIVE.
C                         CORRESPONDING STD. DEVIATION ESTIMATES ARE
C                         SET TO INDEFINATES.
C                  .GE.10, USER TERMINATION FROM AUX
C                          (SET TO 10 OR USER VALUE WHICH EVER GREATER)
C                           ALL "SDY" SET TO INDEFINITES.
C
C     AUX - SUBPROGRAM CONTAINING USER"S FUNCTION
C
*IF -DEF,AELIB5
      DATA RINDEF/ 17770000000000000000B /
*ENDIF
*IF DEF,AELIB5
      DATA RINDEF/ O"17770000000000000000" /
*ENDIF
C
C     WRITTEN BY L. EVANS, E. LONG - FEB./77
C
C     REVISION HISTORY :   - JANUARY 7,1980.
C                          - RESTORATION OF ERROR MATRIX WRONG.
C                            E(I,J)=WS(INDX+J) BECOMES E(I,J)=WS(INDX+I)
C                          - REVISION BY L.E. EVANS AND A. WILLIS.
*CALL  STATF
C
C.....INITIALIZE BASE INDECIES FOR WORKING STORAGE ALLOCATION
C
      I2 = M * M
      I3 = I2 + N
      IER = IERAUX = 0
C
C.....STORE ERROR MATRIX FROM MLSQQ
C
      DO 20 J = 1, M
          INDX = (J - 1) * M
      DO 20 I = 1, M
          WS(INDX+I) = E(I,J)
   20 CONTINUE
C
C.....IF THE DERIVATIVES ARE TO BE PROVIDED BY AUX SKIP PAST THE
C.....DERIVATIVE CALCULATIONS
C
      IF (IABS(IOPT) .EQ. 1) GOTO 60
C
C.....CALCULATE THE FUNCTION VALUES WITH THE FITTED PARAMTERS FROM MLSQQ
C.....AND FIND THE INFINITY NORM, FF
C.....(UNWEIGHTED FIT ASSUMED)
C
      CALL AUX(X,NRX,WS(I2+1),P,N,IERAUX)
C
C     ....CHECK THE ERROR FLAG AND EXIT IF SET
C
          IF (IERAUX .NE. 0) GOTO 200
      FF = 0.
      DO 30 I = 1, N
          WS(I2+I) = Y(I) - WS(I2+I)
          FF = AMAX1(FF, ABS(WS(I2+I)))
   30 CONTINUE
C
C.....FOR EACH VARYING PARAMETER CALCULATE
C         -STEP SIZE, SS
C         -NEW PARAM. VALUE
C         -NEW FUNCTION VALUE WITH THAT PARAM. CHANGED
C         -PARTIAL DERIVATIVE
C.....AND RESTORE THE PARAM. VALUE
C
      DO 50 J = 1, M
C
C     ....ENSURE THAT NO. CALC. ARE PROVIDED FOR NON-VARYING PARAM.
C
          IF (NC(J) .EQ. 0) GOTO 50
C
          DELTA = 1.0E-9
          IF (ABS(P(J)) .GE. 1.0E-6) DELTA = 0.001 * ABS(P(J))
          SS = AMIN1(FF, DELTA)
          PSAVE = P(J)
          P(J) = P(J) + SS
C
          CALL AUX(X,NRX,WS(I3+1),P,N,IERAUX)
C
C         ....CHECK ERROR FLAG AND EXIT IF SET
C
C             IF (IERAUX .NE. 0) GOTO 200
C
          DO 40 I = 1, N
              WS(I3+I) = Y(I) - WS(I3+I)
              E(I,J) = (WS(I3+I) - WS(I2+I)) / SS
   40     CONTINUE
C
          P(J) = PSAVE
   50 CONTINUE
      GOTO 70
C
   60 CONTINUE
C
C.....DERIVATIVES ARE PROVIDED BY AUX.
C
      CALL AUX(X,NRX,WS(I2+1),P,N,E,NE,IERAUX)
C
C     ....CHECK ERROR FLAG AND EXIT IF SET
C
          IF (IERAUX .NE. 0) GOTO 200
C
   70 CONTINUE
C
C.....CALC. THE STANDARD DEVIATION OF FITTED FUNCTION VALUES
C
      DO 110 I = 1, N
          TERM1 = TERM2 = 0.
          DO 80 J = 1, M
              IF (NC(J) .EQ. 0) GOTO 80
              INDX = (J - 1) * M + J
              TERM1 = TERM1 + (WS(INDX) * WS(INDX) * E(I,J) * E(I,J))
   80     CONTINUE
C
          DO 100 J = 2, M
              IF (NC(J) .EQ. 0) GOTO 100
              TERM3 = 0.
              JJ = J - 1
              INDX = (J - 1) * M
C
              DO 90 K = 1, JJ
                  IF (NC(K) .EQ. 0) GOTO 90
                  TERM3 = TERM3 + (WS(INDX+K) * E(I,J) * E(I,K))
   90         CONTINUE
C
              TERM2 = TERM2 + TERM3
  100     CONTINUE
C
          TERM3 = TERM1 + 2. * TERM2
          SDY(I) = RINDEF
          IF (TERM3 .LT. 0.) IER = 1
          IF (TERM3 .GE. 0.) SDY(I) = SQRT(TERM3)
  110 CONTINUE
C
      GOTO 300
C
  200 CONTINUE
C
C.....USER TERMINATION FROM AUX
C
      IER = 10
      IF (IERAUX .GT. 10) IER = IERAUX
      CALL PRESET(SDY,N,RINDEF)
C
  300 CONTINUE
C
C.....RESTORE THE ERROR MATRIX FROM MLSQQ
C
      DO 310 J = 1, M
          INDX = (J - 1) * M
      DO 310 I = 1, M
          E(I,J) = WS(INDX+I)
  310 CONTINUE
C
      RETURN
      END
